/****************************************************************
 *
 *        Copyright 2013, Big Switch Networks, Inc.
 *
 * Licensed under the Eclipse Public License, Version 1.0 (the
 * "License"); you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *        http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the
 * License.
 *
 ***************************************************************/

#include <OS/os_config.h>

#if OS_CONFIG_INCLUDE_UCLI == 1

#include <uCli/ucli.h>
#include <uCli/ucli_argparse.h>
#include <uCli/ucli_handler_macros.h>
#include <OS/os_sleep.h>

static ucli_status_t
os_ucli_ucli__config__(ucli_context_t* uc)
{
    UCLI_HANDLER_MACRO_MODULE_CONFIG(os);
}

static ucli_status_t
os_ucli_ucli__sleep__(ucli_context_t* uc)
{
    uint32_t usecs;
    UCLI_COMMAND_INFO(uc,
                      "sleep", 1,
                      "$summary#Sleep the given number of seconds.");
    UCLI_ARGPARSE_OR_RETURN(uc, "i", &usecs);
    os_sleep_usecs(usecs*1000000);
    return UCLI_STATUS_OK;
}



/* <auto.ucli.handlers.start> */
/******************************************************************************
 *
 * These handler table(s) were autogenerated from the symbols in this
 * source file.
 *
 *****************************************************************************/
static ucli_command_handler_f os_ucli_ucli_handlers__[] =
{
    os_ucli_ucli__config__,
    os_ucli_ucli__sleep__,
    NULL
};
/******************************************************************************/
/* <auto.ucli.handlers.end> */


static ucli_module_t
os_ucli_module__ =
    {
        "os_ucli",
        NULL,
        os_ucli_ucli_handlers__,
        NULL,
        NULL,
    };


ucli_node_t*
os_ucli_node_create(void)
{
    ucli_node_t* n;
    ucli_module_init(&os_ucli_module__);
    n = ucli_node_create("os", NULL, &os_ucli_module__);
    ucli_node_subnode_add(n, ucli_module_log_node_create("os"));
    return n;
}

#else
void*
os_ucli_node_create(void)
{
    return NULL;
}
#endif



