/* Copyright (c) 2008 The Board of Trustees of The Leland Stanford Junior University */
/* Copyright (c) 2011, 2012 Open Networking Foundation */
/* Copyright (c) 2012, 2013 Big Switch Networks, Inc. */
/* See the file LICENSE.loci which should have been included in the source distribution */

/*
 * Common test code for LOCI
 *
 * AUTOMATICALLY GENERATED FILE.  Edits will be lost on regen.
 */

#define DISABLE_WARN_UNUSED_RESULT
#include "loci_log.h"
#include <loci/loci_obj_dump.h>
#include <locitest/unittest.h>
#include <locitest/test_common.h>

/* mcheck is a glibc extension */
#if defined(__linux__)
#include <mcheck.h>
#define MCHECK_INIT mcheck(NULL)
#else
#define MCHECK_INIT do { } while (0)
#endif

/**
 * Exit on error if set to 1
 */
int exit_on_error = 1;

/**
 * Global error state: 0 is okay, 1 is error
 */
int global_error = 0;

extern int run_unified_accessor_tests(void);
extern int run_match_tests(void);
extern int run_utility_tests(void);

extern int run_scalar_acc_tests(void);
extern int run_list_tests(void);
extern int run_message_tests(void);

/**
 * Macros for initializing and checking scalar types
 *
 * @param var The variable being initialized or checked
 * @param val The integer value to set/check against, see below
 *
 * Note that equality means something special for strings.  Each byte
 * is initialized to an incrementing value.  So check is done against that.
 *
 */


#define VAR_UINT8_T_INIT(var, val) var = (uint8_t)(val)
#define VAR_UINT8_T_CHECK(var, val) ((var) == (uint8_t)(val))

#define VAR_UINT16_T_INIT(var, val) var = (uint16_t)(val)
#define VAR_UINT16_T_CHECK(var, val) ((var) == (uint16_t)(val))

#define VAR_UINT32_T_INIT(var, val) var = (uint32_t)(val)
#define VAR_UINT32_T_CHECK(var, val) ((var) == (uint32_t)(val))

#define VAR_UINT64_T_INIT(var, val) var = (uint64_t)(val)
#define VAR_UINT64_T_CHECK(var, val) ((var) == (uint64_t)(val))

#define VAR_OF_PORT_NO_T_INIT(var, val) var = (of_port_no_t)(val)
#define VAR_OF_PORT_NO_T_CHECK(var, val) ((var) == (of_port_no_t)(val))

#define VAR_OF_FM_CMD_T_INIT(var, val) var = (of_fm_cmd_t)(val)
#define VAR_OF_FM_CMD_T_CHECK(var, val) ((var) == (of_fm_cmd_t)(val))

#define VAR_OF_WC_BMAP_T_INIT(var, val) var = (of_wc_bmap_t)(val)
#define VAR_OF_WC_BMAP_T_CHECK(var, val) ((var) == (of_wc_bmap_t)(val))

#define VAR_OF_MATCH_BMAP_T_INIT(var, val) var = (of_match_bmap_t)(val)
#define VAR_OF_MATCH_BMAP_T_CHECK(var, val) ((var) == (of_match_bmap_t)(val))

#define VAR_OF_IPV4_T_INIT(var, val) var = (of_ipv4_t)(val)
#define VAR_OF_IPV4_T_CHECK(var, val) ((var) == (of_ipv4_t)(val))

#define VAR_OF_PORT_NAME_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_PORT_NAME_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_TABLE_NAME_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_TABLE_NAME_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_DESC_STR_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_DESC_STR_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_SERIAL_NUM_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_SERIAL_NUM_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_MAC_ADDR_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_MAC_ADDR_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_IPV6_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_IPV6_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_BITMAP_128_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_BITMAP_128_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_CHECKSUM_128_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_CHECKSUM_128_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))

#define VAR_OF_STR64_T_INIT(var, val) \
    of_test_str_fill((uint8_t *)&(var), val, sizeof(var))
#define VAR_OF_STR64_T_CHECK(var, val) \
    of_test_str_check((uint8_t *)&(var), val, sizeof(var))


/**
 * The increment to use on values inside a string
 */
#define OF_TEST_STR_INCR 3

/**
 * Fill in a buffer with incrementing values starting
 * at the given offset with the given value
 * @param buf The buffer to fill
 * @param value The value to use for data
 * @param len The number of bytes to fill
 */

void
of_test_str_fill(uint8_t *buf, int value, int len)
{
    int i;

    for (i = 0; i < len; i++) {
        *buf = value;
        value += OF_TEST_STR_INCR;
        buf++;
    }
}

/**
 * Given a buffer, verify that it's filled as above
 * @param buf The buffer to check
 * @param value The value to use for data
 * @param len The number of bytes to fill
 * @return Boolean True on equality (success)
 */

int
of_test_str_check(uint8_t *buf, int value, int len)
{
    int i;
    uint8_t val8;

    val8 = value;

    for (i = 0; i < len; i++) {
        if (*buf != val8) {
            return 0;
        }
        val8 += OF_TEST_STR_INCR;
        buf++;
    }

    return 1;
}

/**
 * Global that determines how octets should be populated
 * -1 means use value % MAX (below) to determine length
 * 0, 1, ... means used that fixed length
 *
 * Note: Was 16K, but that made objects too big.  May add flexibility
 * to call populate with a max parameter for length
 */
int octets_pop_style = -1;
#define OCTETS_MAX_VALUE (128) /* 16K was too big */
#define OCTETS_MULTIPLIER 6367 /* A prime */

int
of_octets_populate(of_octets_t *octets, int value)
{
    if (octets_pop_style < 0) {
        octets->bytes = (value * OCTETS_MULTIPLIER) % OCTETS_MAX_VALUE;
    } else {
        octets->bytes = octets_pop_style;
    }

    if (octets->bytes != 0) {
        if ((octets->data = (uint8_t *)MALLOC(octets->bytes)) == NULL) {
            return 0;
        }
        of_test_str_fill(octets->data, value, octets->bytes);
        value += 1;
    }

    return value;
}

int
of_octets_check(of_octets_t *octets, int value)
{
    int len;

    if (octets_pop_style < 0) {
        len =  (value * OCTETS_MULTIPLIER) % OCTETS_MAX_VALUE;
        TEST_ASSERT(octets->bytes == len);
    } else {
        TEST_ASSERT(octets->bytes == octets_pop_style);
    }

    if (octets->bytes != 0) {
        TEST_ASSERT(of_test_str_check(octets->data, value, octets->bytes)
            == 1);
        value += 1;
    }

    return value;
}

int
of_match_populate(of_match_t *match, of_version_t version, int value)
{
    MEMSET(match, 0, sizeof(*match));
    match->version = version;

    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_MPLS_TTL))) {
        OF_MATCH_MASK_OFDPA_MPLS_TTL_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_mpls_ttl, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_MPLS_NEXT_LABEL_IS_GAL))) {
        OF_MATCH_MASK_OFDPA_MPLS_NEXT_LABEL_IS_GAL_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_mpls_next_label_is_gal, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_LAG_ID))) {
        OF_MATCH_MASK_BSN_LAG_ID_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_lag_id, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_COLOR_ACTIONS_INDEX))) {
        OF_MATCH_MASK_OFDPA_COLOR_ACTIONS_INDEX_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.ofdpa_color_actions_index, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_VLAN_PCP))) {
        OF_MATCH_MASK_VLAN_PCP_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.vlan_pcp, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV4_SRC))) {
        OF_MATCH_MASK_IPV4_SRC_EXACT_SET(match);
        VAR_OF_IPV4_T_INIT(match->fields.ipv4_src, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV6_DST))) {
        OF_MATCH_MASK_IPV6_DST_EXACT_SET(match);
        VAR_OF_IPV6_T_INIT(match->fields.ipv6_dst, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ARP_SHA))) {
        OF_MATCH_MASK_ARP_SHA_EXACT_SET(match);
        VAR_OF_MAC_ADDR_T_INIT(match->fields.arp_sha, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_QOS_INDEX))) {
        OF_MATCH_MASK_OFDPA_QOS_INDEX_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_qos_index, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_SCTP_SRC))) {
        OF_MATCH_MASK_SCTP_SRC_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.sctp_src, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ICMPV6_CODE))) {
        OF_MATCH_MASK_ICMPV6_CODE_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.icmpv6_code, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ETH_DST))) {
        OF_MATCH_MASK_ETH_DST_EXACT_SET(match);
        VAR_OF_MAC_ADDR_T_INIT(match->fields.eth_dst, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_RXFCL))) {
        OF_MATCH_MASK_OFDPA_RXFCL_EXACT_SET(match);
        VAR_UINT64_T_INIT(match->fields.ofdpa_rxfcl, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV4_DST))) {
        OF_MATCH_MASK_IPV4_DST_EXACT_SET(match);
        VAR_OF_IPV4_T_INIT(match->fields.ipv4_dst, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_UDP_SRC))) {
        OF_MATCH_MASK_UDP_SRC_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.udp_src, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_TUNNEL_ID))) {
        OF_MATCH_MASK_TUNNEL_ID_EXACT_SET(match);
        VAR_UINT64_T_INIT(match->fields.tunnel_id, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ARP_OP))) {
        OF_MATCH_MASK_ARP_OP_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.arp_op, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ETH_TYPE))) {
        OF_MATCH_MASK_ETH_TYPE_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.eth_type, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_RX_TIMESTAMP))) {
        OF_MATCH_MASK_OFDPA_RX_TIMESTAMP_EXACT_SET(match);
        VAR_UINT64_T_INIT(match->fields.ofdpa_rx_timestamp, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_ACTSET_OUTPUT))) {
        OF_MATCH_MASK_OFDPA_ACTSET_OUTPUT_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.ofdpa_actset_output, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_LMEP_ID))) {
        OF_MATCH_MASK_OFDPA_LMEP_ID_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.ofdpa_lmep_id, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_L3_INTERFACE_CLASS_ID))) {
        OF_MATCH_MASK_BSN_L3_INTERFACE_CLASS_ID_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_l3_interface_class_id, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_TRAFFIC_CLASS))) {
        OF_MATCH_MASK_OFDPA_TRAFFIC_CLASS_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_traffic_class, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_MPLS_BOS))) {
        OF_MATCH_MASK_MPLS_BOS_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.mpls_bos, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_OAM_Y1731_MDL))) {
        OF_MATCH_MASK_OFDPA_OAM_Y1731_MDL_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_oam_y1731_mdl, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_TCP_SRC))) {
        OF_MATCH_MASK_TCP_SRC_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.tcp_src, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ARP_SPA))) {
        OF_MATCH_MASK_ARP_SPA_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.arp_spa, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IN_PORT))) {
        OF_MATCH_MASK_IN_PORT_EXACT_SET(match);
        VAR_OF_PORT_NO_T_INIT(match->fields.in_port, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_DEI))) {
        OF_MATCH_MASK_OFDPA_DEI_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_dei, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_UDP_DST))) {
        OF_MATCH_MASK_UDP_DST_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.udp_dst, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV6_ND_TARGET))) {
        OF_MATCH_MASK_IPV6_ND_TARGET_EXACT_SET(match);
        VAR_OF_IPV6_T_INIT(match->fields.ipv6_nd_target, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ICMPV6_TYPE))) {
        OF_MATCH_MASK_ICMPV6_TYPE_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.icmpv6_type, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_TXFCL))) {
        OF_MATCH_MASK_OFDPA_TXFCL_EXACT_SET(match);
        VAR_UINT64_T_INIT(match->fields.ofdpa_txfcl, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_BFD_DISCRIMINATOR))) {
        OF_MATCH_MASK_OFDPA_BFD_DISCRIMINATOR_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.ofdpa_bfd_discriminator, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ICMPV4_TYPE))) {
        OF_MATCH_MASK_ICMPV4_TYPE_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.icmpv4_type, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_MPLS_LABEL))) {
        OF_MATCH_MASK_MPLS_LABEL_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.mpls_label, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_TCP_DST))) {
        OF_MATCH_MASK_TCP_DST_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.tcp_dst, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IP_PROTO))) {
        OF_MATCH_MASK_IP_PROTO_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ip_proto, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_OVID))) {
        OF_MATCH_MASK_OFDPA_OVID_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.ofdpa_ovid, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IP_DSCP))) {
        OF_MATCH_MASK_IP_DSCP_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ip_dscp, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_MPLS_L2_PORT))) {
        OF_MATCH_MASK_OFDPA_MPLS_L2_PORT_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.ofdpa_mpls_l2_port, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_MPLS_DATA_FIRST_NIBBLE))) {
        OF_MATCH_MASK_OFDPA_MPLS_DATA_FIRST_NIBBLE_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_mpls_data_first_nibble, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV6_FLABEL))) {
        OF_MATCH_MASK_IPV6_FLABEL_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.ipv6_flabel, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ARP_TPA))) {
        OF_MATCH_MASK_ARP_TPA_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.arp_tpa, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV6_SRC))) {
        OF_MATCH_MASK_IPV6_SRC_EXACT_SET(match);
        VAR_OF_IPV6_T_INIT(match->fields.ipv6_src, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_IN_PORTS_128))) {
        OF_MATCH_MASK_BSN_IN_PORTS_128_EXACT_SET(match);
        VAR_OF_BITMAP_128_T_INIT(match->fields.bsn_in_ports_128, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ETH_SRC))) {
        OF_MATCH_MASK_ETH_SRC_EXACT_SET(match);
        VAR_OF_MAC_ADDR_T_INIT(match->fields.eth_src, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_L3_IN_PORT))) {
        OF_MATCH_MASK_OFDPA_L3_IN_PORT_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.ofdpa_l3_in_port, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_OAM_Y1731_OPCODE))) {
        OF_MATCH_MASK_OFDPA_OAM_Y1731_OPCODE_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_oam_y1731_opcode, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_L3_DST_CLASS_ID))) {
        OF_MATCH_MASK_BSN_L3_DST_CLASS_ID_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_l3_dst_class_id, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV6_ND_SLL))) {
        OF_MATCH_MASK_IPV6_ND_SLL_EXACT_SET(match);
        VAR_OF_MAC_ADDR_T_INIT(match->fields.ipv6_nd_sll, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_MPLS_TC))) {
        OF_MATCH_MASK_MPLS_TC_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.mpls_tc, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_PROTECTION_INDEX))) {
        OF_MATCH_MASK_OFDPA_PROTECTION_INDEX_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_protection_index, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_EGR_PORT_GROUP_ID))) {
        OF_MATCH_MASK_BSN_EGR_PORT_GROUP_ID_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_egr_port_group_id, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_ETH_SUBTYPE))) {
        OF_MATCH_MASK_OFDPA_ETH_SUBTYPE_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_eth_subtype, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_VLAN_VID))) {
        OF_MATCH_MASK_VLAN_VID_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.vlan_vid, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ARP_THA))) {
        OF_MATCH_MASK_ARP_THA_EXACT_SET(match);
        VAR_OF_MAC_ADDR_T_INIT(match->fields.arp_tha, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_METADATA))) {
        OF_MATCH_MASK_METADATA_EXACT_SET(match);
        VAR_UINT64_T_INIT(match->fields.metadata, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_L3_SRC_CLASS_ID))) {
        OF_MATCH_MASK_BSN_L3_SRC_CLASS_ID_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_l3_src_class_id, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_SCTP_DST))) {
        OF_MATCH_MASK_SCTP_DST_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.sctp_dst, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_ICMPV4_CODE))) {
        OF_MATCH_MASK_ICMPV4_CODE_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.icmpv4_code, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_VRF))) {
        OF_MATCH_MASK_BSN_VRF_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_vrf, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IP_ECN))) {
        OF_MATCH_MASK_IP_ECN_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ip_ecn, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_GLOBAL_VRF_ALLOWED))) {
        OF_MATCH_MASK_BSN_GLOBAL_VRF_ALLOWED_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.bsn_global_vrf_allowed, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IN_PHY_PORT))) {
        OF_MATCH_MASK_IN_PHY_PORT_EXACT_SET(match);
        VAR_OF_PORT_NO_T_INIT(match->fields.in_phy_port, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_MPLS_ACH_CHANNEL))) {
        OF_MATCH_MASK_OFDPA_MPLS_ACH_CHANNEL_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.ofdpa_mpls_ach_channel, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_COLOR))) {
        OF_MATCH_MASK_OFDPA_COLOR_EXACT_SET(match);
        VAR_UINT8_T_INIT(match->fields.ofdpa_color, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_OFDPA_VRF))) {
        OF_MATCH_MASK_OFDPA_VRF_EXACT_SET(match);
        VAR_UINT16_T_INIT(match->fields.ofdpa_vrf, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF1))) {
        OF_MATCH_MASK_BSN_UDF1_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf1, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF0))) {
        OF_MATCH_MASK_BSN_UDF0_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf0, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_IPV6_ND_TLL))) {
        OF_MATCH_MASK_IPV6_ND_TLL_EXACT_SET(match);
        VAR_OF_MAC_ADDR_T_INIT(match->fields.ipv6_nd_tll, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF2))) {
        OF_MATCH_MASK_BSN_UDF2_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf2, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF5))) {
        OF_MATCH_MASK_BSN_UDF5_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf5, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF4))) {
        OF_MATCH_MASK_BSN_UDF4_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf4, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF7))) {
        OF_MATCH_MASK_BSN_UDF7_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf7, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF6))) {
        OF_MATCH_MASK_BSN_UDF6_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf6, value);
        value += 1;
    }


    if (!(of_match_incompat[version] &
            OF_OXM_BIT(OF_OXM_INDEX_BSN_UDF3))) {
        OF_MATCH_MASK_BSN_UDF3_EXACT_SET(match);
        VAR_UINT32_T_INIT(match->fields.bsn_udf3, value);
        value += 1;
    }


    if (value % 2) {
        /* Sometimes set ipv4 addr masks to non-exact */
        match->masks.ipv4_src = 0xffff0000;
        match->masks.ipv4_dst = 0xfffff800;
    }

    /* Restrict values according to masks */
    of_match_values_mask(match);
    return value;
}

int
of_match_check(of_match_t *match, of_version_t version, int value)
{
    of_match_t check;

    value = of_match_populate(&check, match->version, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(MEMCMP(match, &check, sizeof(check)) == 0);

    return value;
}

/**
 * Populate the scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_byte_count_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_flow_count_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_0_check_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_aggregate_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_flow_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_0_populate_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_aggregate_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_aggregate_stats_request_out_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_0_check_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;

    of_aggregate_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_aggregate_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_0_populate_scalars(
    of_async_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_0_check_scalars(
    of_async_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_0_populate_scalars(
    of_async_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_0_check_scalars(
    of_async_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_set,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_0_populate_scalars(
    of_async_set_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_set,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_0_check_scalars(
    of_async_set_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_0_populate_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_action_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_action_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_0_check_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_action_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_action_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_0_populate_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_0_check_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_0_populate_scalars(
    of_bad_match_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_0_check_scalars(
    of_bad_match_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_0_populate_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_request_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_request_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_0_check_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_request_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_request_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_0_populate_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_0_check_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    of_barrier_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_0_populate_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_0_check_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    of_barrier_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_0_populate_scalars(
    of_bsn_arp_idle_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_0_check_scalars(
    of_bsn_arp_idle_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_clear_data_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_0_check_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_enabled_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_0_check_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_enable_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_enable_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_0_check_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_request_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_0_check_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_idle_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_idle_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_0_check_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_ip_mask_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_ip_mask_reply_index_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_ip_mask_reply_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_ip_mask_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_ip_mask_reply_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_get_ip_mask_reply_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_ip_mask_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_ip_mask_request_index_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_0_check_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_ip_mask_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_ip_mask_request_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_l2_table_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_l2_table_reply_l2_table_enable_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_get_l2_table_reply_l2_table_priority_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    of_bsn_get_l2_table_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_l2_table_reply_l2_table_enable_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_get_l2_table_reply_l2_table_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_l2_table_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_0_check_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_l2_table_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_reply_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_request_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_0_check_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_0_check_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_0_populate_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_0_check_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    of_bsn_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_hybrid_get_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_hybrid_get_reply_hybrid_enable_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_hybrid_get_reply_hybrid_version_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    of_bsn_hybrid_get_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_hybrid_get_reply_hybrid_enable_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_hybrid_get_reply_hybrid_version_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_hybrid_get_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_0_check_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_hybrid_get_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_0_populate_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_0_check_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_timeout_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_0_check_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_0_populate_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_timeout_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_timeout_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_timeout_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_0_check_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_tx_interval_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_0_check_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_0_populate_scalars(
    of_bsn_role_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_0_check_scalars(
    of_bsn_role_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_0_check_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_ip_mask_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_ip_mask_index_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_ip_mask_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_0_check_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_set_ip_mask_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_ip_mask_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_ip_mask_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_l2_table_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_l2_table_reply_l2_table_enable_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_l2_table_reply_l2_table_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_l2_table_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    of_bsn_set_l2_table_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_l2_table_reply_l2_table_enable_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_l2_table_reply_l2_table_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_l2_table_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_l2_table_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_l2_table_request_l2_table_enable_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_l2_table_request_l2_table_priority_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_0_check_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    of_bsn_set_l2_table_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_l2_table_request_l2_table_enable_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_l2_table_request_l2_table_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_0_check_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_mirroring_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_mirroring_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_0_check_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_set_mirroring_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_pktin_suppression_request_enabled_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_priority_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_set_pktin_suppression_request_cookie_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_0_check_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_0_check_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_0_populate_scalars(
    of_bsn_shell_command_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_shell_command_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_shell_command_service_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_0_check_scalars(
    of_bsn_shell_command_t *obj, int value) {
    uint32_t val32;

    of_bsn_shell_command_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_shell_command_service_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_0_populate_scalars(
    of_bsn_shell_output_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_shell_output_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_0_check_scalars(
    of_bsn_shell_output_t *obj, int value) {
    uint32_t val32;

    of_bsn_shell_output_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_0_populate_scalars(
    of_bsn_shell_status_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_shell_status_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_shell_status_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_0_check_scalars(
    of_bsn_shell_status_t *obj, int value) {
    uint32_t val32;

    of_bsn_shell_status_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_shell_status_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_0_populate_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_0_check_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_time_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_time_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_time_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_0_check_scalars(
    of_bsn_time_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_status_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_0_check_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_0_check_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_0_check_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_0_populate_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_0_check_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_mfr_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_hw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_sw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_SERIAL_NUM_T_INIT(ser_num, value);
    of_desc_stats_reply_serial_num_set(obj, ser_num);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_dp_desc_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_0_check_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    of_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    TEST_ASSERT(VAR_OF_SERIAL_NUM_T_CHECK(ser_num, value));
    value += 1;

    of_desc_stats_reply_dp_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_0_populate_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_0_check_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_0_populate_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_0_check_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    of_echo_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_0_populate_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_0_check_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    of_echo_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_0_populate_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_error_msg_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_0_check_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    of_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_0_populate_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_0_check_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    of_experimenter_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_0_populate_scalars(
    of_experimenter_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_0_check_scalars(
    of_experimenter_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_0_populate_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_0_check_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_0_populate_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_0_check_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_0_populate_scalars(
    of_experimenter_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_0_check_scalars(
    of_experimenter_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_0_check_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_0_populate_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_0_check_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_0_populate_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_features_reply_datapath_id_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_n_buffers_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_features_reply_n_tables_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_capabilities_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_actions_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_0_check_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    of_features_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_datapath_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_features_reply_n_buffers_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_n_tables_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_features_reply_capabilities_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_actions_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_0_populate_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_features_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_0_check_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    of_features_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_0_populate_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_add_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_add_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_0_check_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_add_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_add_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_0_populate_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_0_check_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_0_populate_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_0_check_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_0_populate_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_mod_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_mod_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_0_check_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_mod_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_mod_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_0_populate_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_0_check_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_0_populate_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_0_check_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_0_populate_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_priority_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_removed_reason_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_0_check_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    of_flow_removed_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_removed_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_0_check_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_0_populate_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_stats_request_out_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_0_check_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;

    of_flow_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_0_populate_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_0_check_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_get_config_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_get_config_reply_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_0_populate_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_0_check_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    of_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_0_populate_scalars(
    of_group_add_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_0_check_scalars(
    of_group_add_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_0_populate_scalars(
    of_group_delete_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_0_check_scalars(
    of_group_delete_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_group_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_0_check_scalars(
    of_group_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_0_populate_scalars(
    of_group_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_0_check_scalars(
    of_group_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_group_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_0_check_scalars(
    of_group_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_0_populate_scalars(
    of_group_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_0_check_scalars(
    of_group_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_0_populate_scalars(
    of_group_mod_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_0_check_scalars(
    of_group_mod_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_0_populate_scalars(
    of_group_modify_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_0_check_scalars(
    of_group_modify_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_group_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_0_check_scalars(
    of_group_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_0_populate_scalars(
    of_group_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_0_check_scalars(
    of_group_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_0_populate_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_0_check_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    of_hello_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_hello_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_hello_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_hello_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_0_populate_scalars(
    of_meter_add_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_0_check_scalars(
    of_meter_add_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_0_populate_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_0_check_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_meter_config_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_0_check_scalars(
    of_meter_config_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_0_populate_scalars(
    of_meter_config_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_0_check_scalars(
    of_meter_config_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_0_populate_scalars(
    of_meter_delete_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_0_check_scalars(
    of_meter_delete_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_meter_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_0_check_scalars(
    of_meter_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_0_populate_scalars(
    of_meter_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_0_check_scalars(
    of_meter_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_0_populate_scalars(
    of_meter_mod_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_0_check_scalars(
    of_meter_mod_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_0_populate_scalars(
    of_meter_modify_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_0_check_scalars(
    of_meter_modify_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_meter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_0_check_scalars(
    of_meter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_0_populate_scalars(
    of_meter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_0_check_scalars(
    of_meter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_0_populate_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_controller_role_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_controller_role_reply_role_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_0_check_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {
    uint32_t val32;

    of_nicira_controller_role_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_nicira_controller_role_reply_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_0_populate_scalars(
    of_nicira_controller_role_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_controller_role_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_controller_role_request_role_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_0_check_scalars(
    of_nicira_controller_role_request_t *obj, int value) {
    uint32_t val32;

    of_nicira_controller_role_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_nicira_controller_role_request_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_0_populate_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_0_check_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    of_nicira_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_0_populate_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_buffer_id_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_packet_in_total_len_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_in_in_port_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_packet_in_reason_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_0_check_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;
    uint8_t val8;

    of_packet_in_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_total_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_packet_in_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_packet_in_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_0_populate_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_out_in_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_0_check_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_packet_out_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_port_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_0_check_scalars(
    of_port_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_0_populate_scalars(
    of_port_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_0_check_scalars(
    of_port_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_0_populate_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_mod_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_mod_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_mask_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_advertise_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_0_check_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    of_port_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_mod_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_advertise_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_0_check_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_0_populate_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_request_port_no_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_0_check_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_port_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_port_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_0_populate_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_port_status_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_port_status_reason_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_0_check_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_port_status_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_status_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_0_populate_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_reply_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_0_check_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_reply_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_0_populate_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_request_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_request_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_0_check_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_request_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_op_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_op_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_op_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_0_check_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_0_populate_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_0_check_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_queue_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_queue_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_request_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_0_populate_scalars(
    of_role_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_0_check_scalars(
    of_role_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_0_populate_scalars(
    of_role_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_role_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_0_check_scalars(
    of_role_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_set_config,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_0_populate_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_set_config_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_set_config,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_0_check_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_set_config_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_set_config_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_set_config_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_0_check_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_0_populate_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_0_check_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_table_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_0_check_scalars(
    of_table_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_0_populate_scalars(
    of_table_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_0_check_scalars(
    of_table_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_0_populate_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_table_mod_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_config_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_0_check_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_table_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_mod_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_0_populate_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_0_check_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_0_populate_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_0_check_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_0_populate_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_0_check_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_0_populate_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_0_check_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_0_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_0_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_0_populate_scalars(
    ofdpa_queue_mod_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_0_check_scalars(
    ofdpa_queue_mod_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_0_populate_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_0_check_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_0_populate_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_0_check_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_0_populate_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_0_check_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_0_populate_scalars(
    of_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_0_check_scalars(
    of_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_0_populate_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_0_check_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_0_populate_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_dest_port_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_vlan_tag_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_bsn_mirror_copy_stage_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_0_check_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_action_bsn_mirror_dest_port_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_copy_stage_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_populate_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_set_tunnel_dst_dst_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_check_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_0_populate_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_0_check_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_0_populate_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_0_check_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_0_check_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_0_check_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_0_populate_scalars(
    of_action_enqueue_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_action_enqueue_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_enqueue_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_0_check_scalars(
    of_action_enqueue_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;

    of_action_enqueue_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_action_enqueue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_0_populate_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_0_check_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_group,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_0_populate_scalars(
    of_action_group_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_group,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_0_check_scalars(
    of_action_group_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_0_populate_scalars(
    of_action_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_0_check_scalars(
    of_action_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_0_populate_scalars(
    of_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_0_check_scalars(
    of_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_0_populate_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_0_check_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_0_populate_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_0_check_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_0_populate_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_0_check_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_0_populate_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_0_check_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_0_populate_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_0_check_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_0_check_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_0_check_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_0_populate_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_0_check_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_0_populate_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_0_check_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_0_populate_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_0_check_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_0_populate_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_0_check_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_0_check_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_0_check_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_0_populate_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_0_check_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_0_populate_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_0_check_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_0_populate_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_0_check_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_0_populate_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_0_check_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_0_populate_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_0_check_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_0_populate_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_0_check_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_0_populate_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_0_check_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_0_populate_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_0_check_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_0_check_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_0_check_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_0_populate_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_0_check_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_0_populate_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_0_check_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_0_check_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_0_populate_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_0_check_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_0_populate_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_action_output_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_action_output_max_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_0_check_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    of_action_output_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_action_output_max_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_0_populate_scalars(
    of_action_pop_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_0_check_scalars(
    of_action_pop_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_0_populate_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_0_check_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_0_populate_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_0_check_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_0_populate_scalars(
    of_action_push_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_0_check_scalars(
    of_action_push_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_0_populate_scalars(
    of_action_push_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_0_check_scalars(
    of_action_push_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_0_populate_scalars(
    of_action_push_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_0_check_scalars(
    of_action_push_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_0_populate_scalars(
    of_action_set_dl_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_action_set_dl_dst_dl_addr_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_0_check_scalars(
    of_action_set_dl_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_action_set_dl_dst_dl_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_0_populate_scalars(
    of_action_set_dl_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_action_set_dl_src_dl_addr_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_0_check_scalars(
    of_action_set_dl_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_action_set_dl_src_dl_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_0_populate_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_0_check_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_0_populate_scalars(
    of_action_set_mpls_label_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_0_check_scalars(
    of_action_set_mpls_label_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_0_populate_scalars(
    of_action_set_mpls_tc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_0_check_scalars(
    of_action_set_mpls_tc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_0_check_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_0_populate_scalars(
    of_action_set_nw_dst_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_nw_dst_nw_addr_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_0_check_scalars(
    of_action_set_nw_dst_t *obj, int value) {
    uint32_t val32;

    of_action_set_nw_dst_nw_addr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_0_populate_scalars(
    of_action_set_nw_ecn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_0_check_scalars(
    of_action_set_nw_ecn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_0_populate_scalars(
    of_action_set_nw_src_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_nw_src_nw_addr_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_0_check_scalars(
    of_action_set_nw_src_t *obj, int value) {
    uint32_t val32;

    of_action_set_nw_src_nw_addr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_0_populate_scalars(
    of_action_set_nw_tos_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_nw_tos_nw_tos_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_0_check_scalars(
    of_action_set_nw_tos_t *obj, int value) {
    uint8_t val8;

    of_action_set_nw_tos_nw_tos_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_0_populate_scalars(
    of_action_set_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_0_check_scalars(
    of_action_set_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_0_populate_scalars(
    of_action_set_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_0_check_scalars(
    of_action_set_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_0_populate_scalars(
    of_action_set_tp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_set_tp_dst_tp_port_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_0_check_scalars(
    of_action_set_tp_dst_t *obj, int value) {
    uint16_t val16;

    of_action_set_tp_dst_tp_port_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_0_populate_scalars(
    of_action_set_tp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_set_tp_src_tp_port_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_0_check_scalars(
    of_action_set_tp_src_t *obj, int value) {
    uint16_t val16;

    of_action_set_tp_src_tp_port_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_0_populate_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_vlan_pcp_vlan_pcp_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_0_check_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    of_action_set_vlan_pcp_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_0_populate_scalars(
    of_action_set_vlan_vid_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_set_vlan_vid_vlan_vid_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_0_check_scalars(
    of_action_set_vlan_vid_t *obj, int value) {
    uint16_t val16;

    of_action_set_vlan_vid_vlan_vid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_0_populate_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_0_check_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_0_populate_scalars(
    of_bsn_controller_connection_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_0_check_scalars(
    of_bsn_controller_connection_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_0_populate_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_interface_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_interface_name_set(obj, port_name);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_addr_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_netmask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_0_check_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_interface_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_mac_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_mac_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_0_populate_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_0_check_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_0_check_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_0_populate_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_0_check_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_0_populate_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_0_check_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_0_populate_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_vport_q_in_q_port_no_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_vlan_id_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_vlan_id_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_vport_q_in_q_if_name_set(obj, port_name);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_0_check_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_0_populate_scalars(
    of_bucket_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bucket,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_0_check_scalars(
    of_bucket_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_0_populate_scalars(
    of_bucket_counter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_0_check_scalars(
    of_bucket_counter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_0_check_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_flow_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_entry_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_group_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_group_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_group_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_0_check_scalars(
    of_group_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_0_populate_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_0_check_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    of_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_0_populate_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_0_check_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_0_populate_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_0_check_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_0_populate_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_0_check_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_0_populate_scalars(
    of_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_0_check_scalars(
    of_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_0_populate_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_0_check_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_0_populate_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_0_check_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_0_populate_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_0_check_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_0_populate_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_0_check_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_0_populate_scalars(
    of_instruction_goto_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_0_check_scalars(
    of_instruction_goto_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_0_populate_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_0_check_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_0_check_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_0_check_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_0_check_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_0_check_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_0_check_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_0_check_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_0_check_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_0_check_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_0_check_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_0_populate_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_0_check_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_0_populate_scalars(
    of_instruction_meter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_0_check_scalars(
    of_instruction_meter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_0_populate_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_0_check_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_0_populate_scalars(
    of_instruction_write_metadata_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_0_check_scalars(
    of_instruction_write_metadata_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_0_populate_scalars(
    of_match_v1_t *obj, int value) {
    of_wc_bmap_t wc_bmap;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    uint16_t val16;
    uint8_t val8;
    of_ipv4_t ipv4;

    VAR_OF_WC_BMAP_T_INIT(wc_bmap, value);
    of_match_v1_wildcards_set(obj, wc_bmap);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_match_v1_in_port_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_match_v1_eth_src_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_match_v1_eth_dst_set(obj, mac_addr);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v1_vlan_vid_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_match_v1_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v1_eth_type_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_match_v1_ip_dscp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_match_v1_ip_proto_set(obj, val8);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_match_v1_ipv4_src_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_match_v1_ipv4_dst_set(obj, ipv4);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v1_tcp_src_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v1_tcp_dst_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_0_check_scalars(
    of_match_v1_t *obj, int value) {
    of_wc_bmap_t wc_bmap;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    uint16_t val16;
    uint8_t val8;
    of_ipv4_t ipv4;

    of_match_v1_wildcards_get(obj, &wc_bmap);
    TEST_ASSERT(VAR_OF_WC_BMAP_T_CHECK(wc_bmap, value));
    value += 1;

    of_match_v1_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_match_v1_eth_src_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_match_v1_eth_dst_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_match_v1_vlan_vid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_match_v1_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_match_v1_eth_type_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_match_v1_ip_dscp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_match_v1_ip_proto_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_match_v1_ipv4_src_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_match_v1_ipv4_dst_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_match_v1_tcp_src_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_match_v1_tcp_dst_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_0_populate_scalars(
    of_match_v2_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_0_check_scalars(
    of_match_v2_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_0_populate_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_0_check_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_0_populate_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_0_check_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_0_populate_scalars(
    of_meter_band_drop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_0_check_scalars(
    of_meter_band_drop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_0_populate_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_0_check_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_0_populate_scalars(
    of_meter_band_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_0_check_scalars(
    of_meter_band_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_0_populate_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_0_check_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_0_populate_scalars(
    of_meter_band_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_0_check_scalars(
    of_meter_band_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_0_populate_scalars(
    of_meter_config_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_0_check_scalars(
    of_meter_config_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_0_populate_scalars(
    of_meter_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_0_check_scalars(
    of_meter_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_0_populate_scalars(
    of_meter_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_0_check_scalars(
    of_meter_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_0_populate_scalars(
    of_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_0_check_scalars(
    of_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_op_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_op_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_sha_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_sha_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_spa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_spa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_tha_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_tha_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_tpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_tpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_0_populate_scalars(
    of_oxm_eth_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_0_check_scalars(
    of_oxm_eth_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_0_populate_scalars(
    of_oxm_eth_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_0_check_scalars(
    of_oxm_eth_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_0_populate_scalars(
    of_oxm_eth_type_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_0_check_scalars(
    of_oxm_eth_type_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_0_populate_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_0_check_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_0_populate_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_0_check_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_0_populate_scalars(
    of_oxm_in_phy_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_0_check_scalars(
    of_oxm_in_phy_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_0_populate_scalars(
    of_oxm_in_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_0_check_scalars(
    of_oxm_in_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_in_port_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_in_port_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_0_populate_scalars(
    of_oxm_ip_dscp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_0_check_scalars(
    of_oxm_ip_dscp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_0_populate_scalars(
    of_oxm_ip_ecn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_0_check_scalars(
    of_oxm_ip_ecn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_0_populate_scalars(
    of_oxm_ip_proto_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_0_check_scalars(
    of_oxm_ip_proto_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv4_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv4_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_0_populate_scalars(
    of_oxm_metadata_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_0_check_scalars(
    of_oxm_metadata_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_metadata_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_metadata_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_0_populate_scalars(
    of_oxm_mpls_bos_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_0_check_scalars(
    of_oxm_mpls_bos_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_0_populate_scalars(
    of_oxm_mpls_label_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_0_check_scalars(
    of_oxm_mpls_label_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_0_populate_scalars(
    of_oxm_mpls_tc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_0_check_scalars(
    of_oxm_mpls_tc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_0_populate_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_0_check_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_0_populate_scalars(
    of_oxm_sctp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_0_check_scalars(
    of_oxm_sctp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_0_populate_scalars(
    of_oxm_sctp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_0_check_scalars(
    of_oxm_sctp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_0_populate_scalars(
    of_oxm_tcp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_0_check_scalars(
    of_oxm_tcp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_0_populate_scalars(
    of_oxm_tcp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_0_check_scalars(
    of_oxm_tcp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_0_populate_scalars(
    of_oxm_tunnel_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_0_check_scalars(
    of_oxm_tunnel_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_0_populate_scalars(
    of_oxm_udp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_0_check_scalars(
    of_oxm_udp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_0_populate_scalars(
    of_oxm_udp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_0_check_scalars(
    of_oxm_udp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_0_populate_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_0_check_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_0_populate_scalars(
    of_oxm_vlan_vid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_0_check_scalars(
    of_oxm_vlan_vid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_0_populate_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_0_check_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_0_populate_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_queue_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_0_check_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;

    of_packet_queue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_0_populate_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_desc_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_desc_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_port_desc_name_set(obj, port_name);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_state_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_curr_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_advertised_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_supported_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_peer_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_0_check_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    of_port_desc_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_desc_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_desc_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_port_desc_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_state_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_curr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_advertised_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_supported_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_peer_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_frame_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_over_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_crc_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_collisions_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_0_check_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;

    of_port_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_stats_entry_rx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_over_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_collisions_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_0_populate_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_0_check_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_0_populate_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_0_check_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_0_populate_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_0_check_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_0_populate_scalars(
    of_queue_prop_max_rate_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_0_check_scalars(
    of_queue_prop_max_rate_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_0_populate_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_prop_min_rate_rate_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_0_check_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    of_queue_prop_min_rate_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_entry_queue_id_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_0_check_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    of_queue_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_entry_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_0_populate_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_0_check_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_0_populate_scalars(
    of_table_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_0_check_scalars(
    of_table_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    of_wc_bmap_t wc_bmap;
    uint32_t val32;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_table_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_OF_TABLE_NAME_T_INIT(table_name, value);
    of_table_stats_entry_name_set(obj, table_name);
    value += 1;

    VAR_OF_WC_BMAP_T_INIT(wc_bmap, value);
    of_table_stats_entry_wildcards_set(obj, wc_bmap);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_max_entries_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_active_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_lookup_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_matched_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_0_check_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    of_wc_bmap_t wc_bmap;
    uint32_t val32;
    uint64_t val64;

    of_table_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_stats_entry_name_get(obj, &table_name);
    TEST_ASSERT(VAR_OF_TABLE_NAME_T_CHECK(table_name, value));
    value += 1;

    of_table_stats_entry_wildcards_get(obj, &wc_bmap);
    TEST_ASSERT(VAR_OF_WC_BMAP_T_CHECK(wc_bmap, value));
    value += 1;

    of_table_stats_entry_max_entries_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_active_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_lookup_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_matched_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint32,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_0_populate_scalars(
    of_uint32_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint32,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_0_check_scalars(
    of_uint32_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint64,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_0_populate_scalars(
    of_uint64_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint64,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_0_check_scalars(
    of_uint64_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint8,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_0_populate_scalars(
    of_uint8_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint8,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_0_check_scalars(
    of_uint8_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_0_populate_scalars(
    ofp_queue_desc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_0_check_scalars(
    ofp_queue_desc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_0_populate_scalars(
    of_list_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_0_check_scalars(
    of_list_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_0_populate_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_0_check_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_0_check_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_0_check_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_0_check_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_0_populate_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_0_check_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_0_populate_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_0_check_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_0_populate_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_0_check_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_0_populate_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_0_check_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_0_populate_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_0_check_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_0_populate_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_0_check_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_0_populate_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_0_check_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_0_populate_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_0_check_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_0_populate_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_0_check_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_0_populate_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_0_check_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_0_populate_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_0_check_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_0_populate_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_0_check_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_0_populate_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_0_check_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_0_populate_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_0_check_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_0_populate_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_0_check_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_0_populate_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_0_check_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_0_populate_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_0_check_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_0_populate_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_0
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_0_check_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_byte_count_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_flow_count_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_1_check_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_aggregate_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_flow_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_1_populate_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_aggregate_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_aggregate_stats_request_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_request_out_group_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_request_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_request_cookie_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_1_check_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    of_aggregate_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_aggregate_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_aggregate_stats_request_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_request_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_1_populate_scalars(
    of_async_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_1_check_scalars(
    of_async_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_1_populate_scalars(
    of_async_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_1_check_scalars(
    of_async_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_set,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_1_populate_scalars(
    of_async_set_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_set,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_1_check_scalars(
    of_async_set_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_1_populate_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_action_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_action_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_1_check_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_action_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_action_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_1_populate_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_instruction_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_instruction_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_1_check_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_instruction_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_instruction_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_1_populate_scalars(
    of_bad_match_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_match_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_match_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_1_check_scalars(
    of_bad_match_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_match_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_match_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_1_populate_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_request_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_request_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_1_check_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_request_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_request_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_1_populate_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_1_check_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    of_barrier_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_1_populate_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_1_check_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    of_barrier_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_1_populate_scalars(
    of_bsn_arp_idle_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_1_check_scalars(
    of_bsn_arp_idle_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_clear_data_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_1_check_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_enabled_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_1_check_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_enable_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_enable_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_1_check_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_request_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_1_check_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_idle_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_idle_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_1_check_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_1_check_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_1_check_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_reply_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_request_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_1_check_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_1_check_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_1_populate_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_1_check_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    of_bsn_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_1_check_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_1_populate_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_1_check_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_timeout_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_1_check_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_1_populate_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_timeout_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_timeout_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_timeout_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_1_check_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_tx_interval_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_1_check_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_1_populate_scalars(
    of_bsn_role_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_1_check_scalars(
    of_bsn_role_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_1_check_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_1_check_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_1_check_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_1_check_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_mirroring_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_mirroring_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_1_check_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_set_mirroring_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_pktin_suppression_request_enabled_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_priority_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_set_pktin_suppression_request_cookie_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_1_check_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_1_check_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_1_populate_scalars(
    of_bsn_shell_command_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_1_check_scalars(
    of_bsn_shell_command_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_1_populate_scalars(
    of_bsn_shell_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_1_check_scalars(
    of_bsn_shell_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_1_populate_scalars(
    of_bsn_shell_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_1_check_scalars(
    of_bsn_shell_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_1_populate_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_1_check_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_time_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_time_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_time_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_1_check_scalars(
    of_bsn_time_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_status_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_1_check_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_1_check_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_1_check_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_1_populate_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_1_check_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_mfr_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_hw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_sw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_SERIAL_NUM_T_INIT(ser_num, value);
    of_desc_stats_reply_serial_num_set(obj, ser_num);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_dp_desc_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_1_check_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    of_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    TEST_ASSERT(VAR_OF_SERIAL_NUM_T_CHECK(ser_num, value));
    value += 1;

    of_desc_stats_reply_dp_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_1_populate_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_1_check_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_1_populate_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_1_check_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    of_echo_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_1_populate_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_1_check_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    of_echo_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_1_populate_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_error_msg_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_1_check_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    of_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_1_populate_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_1_check_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    of_experimenter_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_1_populate_scalars(
    of_experimenter_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_1_check_scalars(
    of_experimenter_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_1_populate_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_1_check_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_1_populate_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_1_check_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_1_populate_scalars(
    of_experimenter_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_1_check_scalars(
    of_experimenter_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_1_check_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_1_populate_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_1_check_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_1_populate_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_features_reply_datapath_id_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_n_buffers_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_features_reply_n_tables_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_capabilities_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_reserved_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_1_check_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    of_features_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_datapath_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_features_reply_n_buffers_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_n_tables_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_features_reply_capabilities_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_reserved_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_1_populate_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_features_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_1_check_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    of_features_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_1_populate_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_add_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_add_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_add_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_add_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_1_check_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_add_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_add_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_add_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_add_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_1_populate_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_delete_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_1_check_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_delete_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_1_populate_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_strict_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_delete_strict_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_1_check_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_strict_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_strict_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_strict_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_1_populate_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_mod_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_mod_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_mod_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_mod_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_1_check_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_mod_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_mod_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_mod_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_mod_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_1_populate_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_modify_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_1_check_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_modify_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_1_populate_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_strict_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_modify_strict_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_1_check_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_strict_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_strict_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_strict_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_1_populate_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_priority_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_removed_reason_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_removed_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_1_check_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    of_flow_removed_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_removed_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_removed_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_1_check_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_1_populate_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_stats_request_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_request_out_group_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_request_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_request_cookie_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_1_check_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    of_flow_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_stats_request_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_request_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_1_populate_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_1_check_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_get_config_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_get_config_reply_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_1_populate_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_1_check_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    of_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_1_populate_scalars(
    of_group_add_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_add_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_add_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_1_check_scalars(
    of_group_add_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_add_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_add_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_1_populate_scalars(
    of_group_delete_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_delete_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_delete_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_1_check_scalars(
    of_group_delete_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_delete_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_delete_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_group_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_1_check_scalars(
    of_group_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_1_populate_scalars(
    of_group_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_1_check_scalars(
    of_group_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_group_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_1_check_scalars(
    of_group_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_1_populate_scalars(
    of_group_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_1_check_scalars(
    of_group_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_1_populate_scalars(
    of_group_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_mod_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_1_check_scalars(
    of_group_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_mod_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_mod_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_1_populate_scalars(
    of_group_modify_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_modify_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_modify_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_1_check_scalars(
    of_group_modify_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_modify_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_modify_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_group_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_1_check_scalars(
    of_group_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_1_populate_scalars(
    of_group_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_request_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_1_check_scalars(
    of_group_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_group_stats_request_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_1_populate_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_1_check_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    of_hello_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_hello_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_hello_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_hello_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_1_populate_scalars(
    of_meter_add_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_1_check_scalars(
    of_meter_add_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_1_populate_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_1_check_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_meter_config_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_1_check_scalars(
    of_meter_config_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_1_populate_scalars(
    of_meter_config_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_1_check_scalars(
    of_meter_config_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_1_populate_scalars(
    of_meter_delete_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_1_check_scalars(
    of_meter_delete_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_meter_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_1_check_scalars(
    of_meter_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_1_populate_scalars(
    of_meter_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_1_check_scalars(
    of_meter_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_1_populate_scalars(
    of_meter_mod_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_1_check_scalars(
    of_meter_mod_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_1_populate_scalars(
    of_meter_modify_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_1_check_scalars(
    of_meter_modify_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_meter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_1_check_scalars(
    of_meter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_1_populate_scalars(
    of_meter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_1_check_scalars(
    of_meter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_1_populate_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_1_check_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_1_populate_scalars(
    of_nicira_controller_role_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_1_check_scalars(
    of_nicira_controller_role_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_1_populate_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_1_check_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    of_nicira_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_1_populate_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_in_in_port_set(obj, port_no);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_in_in_phy_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_packet_in_total_len_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_packet_in_reason_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_packet_in_table_id_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_1_check_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint16_t val16;
    uint8_t val8;

    of_packet_in_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_packet_in_in_phy_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_packet_in_total_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_packet_in_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_packet_in_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_1_populate_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_out_in_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_1_check_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_packet_out_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_port_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_1_check_scalars(
    of_port_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_1_populate_scalars(
    of_port_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_1_check_scalars(
    of_port_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_1_populate_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_mod_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_mod_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_mask_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_advertise_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_1_check_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    of_port_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_mod_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_advertise_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_1_check_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_1_populate_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_request_port_no_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_1_check_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_port_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_port_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_1_populate_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_port_status_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_port_status_reason_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_1_check_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_port_status_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_status_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_1_populate_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_reply_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_1_check_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_reply_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_1_populate_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_request_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_request_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_1_check_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_request_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_op_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_op_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_op_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_1_check_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_1_populate_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_1_check_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_queue_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_queue_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_request_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_1_populate_scalars(
    of_role_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_1_check_scalars(
    of_role_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_1_populate_scalars(
    of_role_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_role_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_1_check_scalars(
    of_role_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_set_config,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_1_populate_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_set_config_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_set_config,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_1_check_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_set_config_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_set_config_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_set_config_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_1_check_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_1_populate_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_1_check_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_switch_config_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_switch_config_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_switch_config_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_switch_config_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_table_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_1_check_scalars(
    of_table_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_1_populate_scalars(
    of_table_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_1_check_scalars(
    of_table_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_1_populate_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_table_mod_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_config_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_1_check_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_table_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_mod_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_1_populate_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_1_check_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_1_populate_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_1_check_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_1_populate_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_1_check_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_1_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_1_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_1_populate_scalars(
    ofdpa_queue_mod_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_1_check_scalars(
    ofdpa_queue_mod_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_1_populate_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_1_check_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_1_populate_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_1_check_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_1_populate_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_1_check_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_1_populate_scalars(
    of_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_1_check_scalars(
    of_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_1_populate_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_1_check_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_1_populate_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_dest_port_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_vlan_tag_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_bsn_mirror_copy_stage_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_1_check_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_action_bsn_mirror_dest_port_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_copy_stage_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_populate_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_set_tunnel_dst_dst_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_check_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_1_populate_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_1_check_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_1_populate_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_1_check_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_1_check_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_1_check_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_1_populate_scalars(
    of_action_enqueue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_1_check_scalars(
    of_action_enqueue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_1_populate_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_1_check_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_group,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_1_populate_scalars(
    of_action_group_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_group_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_group,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_1_check_scalars(
    of_action_group_t *obj, int value) {
    uint32_t val32;

    of_action_group_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_1_populate_scalars(
    of_action_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_1_check_scalars(
    of_action_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_1_populate_scalars(
    of_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_1_check_scalars(
    of_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_1_populate_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_1_check_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_1_populate_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_1_check_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_1_populate_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_1_check_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_1_populate_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_1_check_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_1_populate_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_1_check_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_1_check_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_1_check_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_1_populate_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_1_check_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_1_populate_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_1_check_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_1_populate_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_1_check_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_1_populate_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_1_check_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_1_check_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_1_check_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_1_populate_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_1_check_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_1_populate_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_1_check_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_1_populate_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_1_check_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_1_populate_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_1_check_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_1_populate_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_1_check_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_1_populate_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_1_check_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_1_populate_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_1_check_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_1_populate_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_1_check_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_1_check_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_1_check_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_1_populate_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_1_check_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_1_populate_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_1_check_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_1_check_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_1_populate_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_1_check_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_1_populate_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_action_output_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_action_output_max_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_1_check_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    of_action_output_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_action_output_max_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_1_populate_scalars(
    of_action_pop_mpls_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_pop_mpls_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_1_check_scalars(
    of_action_pop_mpls_t *obj, int value) {
    uint16_t val16;

    of_action_pop_mpls_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_1_populate_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_1_check_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_1_populate_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_1_check_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_1_populate_scalars(
    of_action_push_mpls_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_push_mpls_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_1_check_scalars(
    of_action_push_mpls_t *obj, int value) {
    uint16_t val16;

    of_action_push_mpls_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_1_populate_scalars(
    of_action_push_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_1_check_scalars(
    of_action_push_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_1_populate_scalars(
    of_action_push_vlan_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_push_vlan_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_1_check_scalars(
    of_action_push_vlan_t *obj, int value) {
    uint16_t val16;

    of_action_push_vlan_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_1_populate_scalars(
    of_action_set_dl_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_action_set_dl_dst_dl_addr_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_1_check_scalars(
    of_action_set_dl_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_action_set_dl_dst_dl_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_1_populate_scalars(
    of_action_set_dl_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_action_set_dl_src_dl_addr_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_1_check_scalars(
    of_action_set_dl_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_action_set_dl_src_dl_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_1_populate_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_1_check_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_1_populate_scalars(
    of_action_set_mpls_label_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_mpls_label_mpls_label_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_1_check_scalars(
    of_action_set_mpls_label_t *obj, int value) {
    uint32_t val32;

    of_action_set_mpls_label_mpls_label_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_1_populate_scalars(
    of_action_set_mpls_tc_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_mpls_tc_mpls_tc_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_1_check_scalars(
    of_action_set_mpls_tc_t *obj, int value) {
    uint8_t val8;

    of_action_set_mpls_tc_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_mpls_ttl_mpls_ttl_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_1_check_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    of_action_set_mpls_ttl_mpls_ttl_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_1_populate_scalars(
    of_action_set_nw_dst_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_nw_dst_nw_addr_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_1_check_scalars(
    of_action_set_nw_dst_t *obj, int value) {
    uint32_t val32;

    of_action_set_nw_dst_nw_addr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_1_populate_scalars(
    of_action_set_nw_ecn_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_nw_ecn_nw_ecn_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_1_check_scalars(
    of_action_set_nw_ecn_t *obj, int value) {
    uint8_t val8;

    of_action_set_nw_ecn_nw_ecn_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_1_populate_scalars(
    of_action_set_nw_src_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_nw_src_nw_addr_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_1_check_scalars(
    of_action_set_nw_src_t *obj, int value) {
    uint32_t val32;

    of_action_set_nw_src_nw_addr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_1_populate_scalars(
    of_action_set_nw_tos_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_nw_tos_nw_tos_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_1_check_scalars(
    of_action_set_nw_tos_t *obj, int value) {
    uint8_t val8;

    of_action_set_nw_tos_nw_tos_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_1_populate_scalars(
    of_action_set_nw_ttl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_nw_ttl_nw_ttl_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_1_check_scalars(
    of_action_set_nw_ttl_t *obj, int value) {
    uint8_t val8;

    of_action_set_nw_ttl_nw_ttl_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_1_populate_scalars(
    of_action_set_queue_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_queue_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_1_check_scalars(
    of_action_set_queue_t *obj, int value) {
    uint32_t val32;

    of_action_set_queue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_1_populate_scalars(
    of_action_set_tp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_set_tp_dst_tp_port_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_1_check_scalars(
    of_action_set_tp_dst_t *obj, int value) {
    uint16_t val16;

    of_action_set_tp_dst_tp_port_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_1_populate_scalars(
    of_action_set_tp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_set_tp_src_tp_port_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_1_check_scalars(
    of_action_set_tp_src_t *obj, int value) {
    uint16_t val16;

    of_action_set_tp_src_tp_port_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_1_populate_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_vlan_pcp_vlan_pcp_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_1_check_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    of_action_set_vlan_pcp_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_1_populate_scalars(
    of_action_set_vlan_vid_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_set_vlan_vid_vlan_vid_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_1_check_scalars(
    of_action_set_vlan_vid_t *obj, int value) {
    uint16_t val16;

    of_action_set_vlan_vid_vlan_vid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_1_populate_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_1_check_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_1_populate_scalars(
    of_bsn_controller_connection_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_1_check_scalars(
    of_bsn_controller_connection_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_1_populate_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_interface_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_interface_name_set(obj, port_name);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_addr_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_netmask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_1_check_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_interface_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_mac_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_mac_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_1_populate_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_1_check_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_1_check_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_1_populate_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_1_check_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_1_populate_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_1_check_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_1_populate_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_vport_q_in_q_port_no_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_vlan_id_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_vlan_id_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_vport_q_in_q_if_name_set(obj, port_name);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_1_check_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_1_populate_scalars(
    of_bucket_t *obj, int value) {
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_bucket_weight_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bucket_watch_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bucket_watch_group_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bucket,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_1_check_scalars(
    of_bucket_t *obj, int value) {
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;

    of_bucket_weight_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bucket_watch_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bucket_watch_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_1_populate_scalars(
    of_bucket_counter_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bucket_counter_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bucket_counter_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_1_check_scalars(
    of_bucket_counter_t *obj, int value) {
    uint64_t val64;

    of_bucket_counter_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_bucket_counter_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_1_check_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_flow_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_entry_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_group_desc_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;

    VAR_UINT8_T_INIT(val8, value);
    of_group_desc_stats_entry_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_entry_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_group_desc_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;

    of_group_desc_stats_entry_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_desc_stats_entry_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_group_stats_entry_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_group_id_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_ref_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_group_stats_entry_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_group_stats_entry_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_1_check_scalars(
    of_group_stats_entry_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_group_stats_entry_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_entry_ref_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_entry_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_group_stats_entry_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_1_populate_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_1_check_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    of_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_1_populate_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_1_check_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_1_populate_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_1_check_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_1_populate_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_1_check_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_1_populate_scalars(
    of_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_1_check_scalars(
    of_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_1_populate_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_1_check_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_1_populate_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_1_check_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_1_populate_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_1_check_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_1_populate_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_1_check_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_1_populate_scalars(
    of_instruction_goto_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_instruction_goto_table_table_id_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_1_check_scalars(
    of_instruction_goto_table_t *obj, int value) {
    uint8_t val8;

    of_instruction_goto_table_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_1_populate_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_1_check_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_1_check_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_1_check_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_1_check_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_1_check_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_1_check_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_1_check_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_1_check_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_1_check_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_1_check_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_1_populate_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_1_check_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_1_populate_scalars(
    of_instruction_meter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_1_check_scalars(
    of_instruction_meter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_1_populate_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_1_check_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_1_populate_scalars(
    of_instruction_write_metadata_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_instruction_write_metadata_metadata_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_instruction_write_metadata_metadata_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_1_check_scalars(
    of_instruction_write_metadata_t *obj, int value) {
    uint64_t val64;

    of_instruction_write_metadata_metadata_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_instruction_write_metadata_metadata_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_1_populate_scalars(
    of_match_v1_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_1_check_scalars(
    of_match_v1_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_1_populate_scalars(
    of_match_v2_t *obj, int value) {
    of_port_no_t port_no;
    of_wc_bmap_t wc_bmap;
    of_mac_addr_t mac_addr;
    uint16_t val16;
    uint8_t val8;
    of_ipv4_t ipv4;
    uint32_t val32;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_match_v2_in_port_set(obj, port_no);
    value += 1;

    VAR_OF_WC_BMAP_T_INIT(wc_bmap, value);
    of_match_v2_wildcards_set(obj, wc_bmap);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_match_v2_eth_src_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_match_v2_eth_src_mask_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_match_v2_eth_dst_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_match_v2_eth_dst_mask_set(obj, mac_addr);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v2_vlan_vid_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_match_v2_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v2_eth_type_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_match_v2_ip_dscp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_match_v2_ip_proto_set(obj, val8);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_match_v2_ipv4_src_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_match_v2_ipv4_src_mask_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_match_v2_ipv4_dst_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_match_v2_ipv4_dst_mask_set(obj, ipv4);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v2_tcp_src_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_match_v2_tcp_dst_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_match_v2_mpls_label_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_match_v2_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_match_v2_metadata_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_match_v2_metadata_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_1_check_scalars(
    of_match_v2_t *obj, int value) {
    of_port_no_t port_no;
    of_wc_bmap_t wc_bmap;
    of_mac_addr_t mac_addr;
    uint16_t val16;
    uint8_t val8;
    of_ipv4_t ipv4;
    uint32_t val32;
    uint64_t val64;

    of_match_v2_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_match_v2_wildcards_get(obj, &wc_bmap);
    TEST_ASSERT(VAR_OF_WC_BMAP_T_CHECK(wc_bmap, value));
    value += 1;

    of_match_v2_eth_src_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_match_v2_eth_src_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_match_v2_eth_dst_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_match_v2_eth_dst_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_match_v2_vlan_vid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_match_v2_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_match_v2_eth_type_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_match_v2_ip_dscp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_match_v2_ip_proto_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_match_v2_ipv4_src_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_match_v2_ipv4_src_mask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_match_v2_ipv4_dst_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_match_v2_ipv4_dst_mask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_match_v2_tcp_src_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_match_v2_tcp_dst_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_match_v2_mpls_label_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_match_v2_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_match_v2_metadata_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_match_v2_metadata_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_1_populate_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_1_check_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_1_populate_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_1_check_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_1_populate_scalars(
    of_meter_band_drop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_1_check_scalars(
    of_meter_band_drop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_1_populate_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_1_check_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_1_populate_scalars(
    of_meter_band_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_1_check_scalars(
    of_meter_band_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_1_populate_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_1_check_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_1_populate_scalars(
    of_meter_band_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_1_check_scalars(
    of_meter_band_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_1_populate_scalars(
    of_meter_config_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_1_check_scalars(
    of_meter_config_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_1_populate_scalars(
    of_meter_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_1_check_scalars(
    of_meter_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_1_populate_scalars(
    of_meter_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_1_check_scalars(
    of_meter_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_1_populate_scalars(
    of_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_1_check_scalars(
    of_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_op_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_op_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_sha_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_sha_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_spa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_spa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_tha_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_tha_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_tpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_tpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_1_populate_scalars(
    of_oxm_eth_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_1_check_scalars(
    of_oxm_eth_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_1_populate_scalars(
    of_oxm_eth_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_1_check_scalars(
    of_oxm_eth_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_1_populate_scalars(
    of_oxm_eth_type_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_1_check_scalars(
    of_oxm_eth_type_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_1_populate_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_1_check_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_1_populate_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_1_check_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_1_populate_scalars(
    of_oxm_in_phy_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_1_check_scalars(
    of_oxm_in_phy_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_1_populate_scalars(
    of_oxm_in_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_1_check_scalars(
    of_oxm_in_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_in_port_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_in_port_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_1_populate_scalars(
    of_oxm_ip_dscp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_1_check_scalars(
    of_oxm_ip_dscp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_1_populate_scalars(
    of_oxm_ip_ecn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_1_check_scalars(
    of_oxm_ip_ecn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_1_populate_scalars(
    of_oxm_ip_proto_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_1_check_scalars(
    of_oxm_ip_proto_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv4_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv4_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_1_populate_scalars(
    of_oxm_metadata_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_1_check_scalars(
    of_oxm_metadata_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_metadata_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_metadata_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_1_populate_scalars(
    of_oxm_mpls_bos_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_1_check_scalars(
    of_oxm_mpls_bos_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_1_populate_scalars(
    of_oxm_mpls_label_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_1_check_scalars(
    of_oxm_mpls_label_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_1_populate_scalars(
    of_oxm_mpls_tc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_1_check_scalars(
    of_oxm_mpls_tc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_1_populate_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_1_check_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_1_populate_scalars(
    of_oxm_sctp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_1_check_scalars(
    of_oxm_sctp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_1_populate_scalars(
    of_oxm_sctp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_1_check_scalars(
    of_oxm_sctp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_1_populate_scalars(
    of_oxm_tcp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_1_check_scalars(
    of_oxm_tcp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_1_populate_scalars(
    of_oxm_tcp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_1_check_scalars(
    of_oxm_tcp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_1_populate_scalars(
    of_oxm_tunnel_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_1_check_scalars(
    of_oxm_tunnel_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_1_populate_scalars(
    of_oxm_udp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_1_check_scalars(
    of_oxm_udp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_1_populate_scalars(
    of_oxm_udp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_1_check_scalars(
    of_oxm_udp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_1_populate_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_1_check_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_1_populate_scalars(
    of_oxm_vlan_vid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_1_check_scalars(
    of_oxm_vlan_vid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_1_populate_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_1_check_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_1_populate_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_queue_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_1_check_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;

    of_packet_queue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_1_populate_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_desc_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_desc_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_port_desc_name_set(obj, port_name);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_state_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_curr_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_advertised_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_supported_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_peer_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_curr_speed_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_max_speed_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_1_check_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    of_port_desc_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_desc_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_desc_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_port_desc_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_state_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_curr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_advertised_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_supported_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_peer_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_curr_speed_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_max_speed_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_frame_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_over_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_crc_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_collisions_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_1_check_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;

    of_port_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_stats_entry_rx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_over_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_collisions_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_1_populate_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_1_check_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_1_populate_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_1_check_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_1_populate_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_1_check_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_1_populate_scalars(
    of_queue_prop_max_rate_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_1_check_scalars(
    of_queue_prop_max_rate_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_1_populate_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_prop_min_rate_rate_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_1_check_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    of_queue_prop_min_rate_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_entry_queue_id_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_1_check_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    of_queue_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_entry_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_1_populate_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_1_check_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_1_populate_scalars(
    of_table_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_1_check_scalars(
    of_table_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    of_wc_bmap_t wc_bmap;
    of_match_bmap_t match_bmap;
    uint32_t val32;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_table_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_OF_TABLE_NAME_T_INIT(table_name, value);
    of_table_stats_entry_name_set(obj, table_name);
    value += 1;

    VAR_OF_WC_BMAP_T_INIT(wc_bmap, value);
    of_table_stats_entry_wildcards_set(obj, wc_bmap);
    value += 1;

    VAR_OF_MATCH_BMAP_T_INIT(match_bmap, value);
    of_table_stats_entry_match_set(obj, match_bmap);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_instructions_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_write_actions_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_apply_actions_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_max_entries_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_active_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_lookup_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_matched_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_1_check_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    of_wc_bmap_t wc_bmap;
    of_match_bmap_t match_bmap;
    uint32_t val32;
    uint64_t val64;

    of_table_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_stats_entry_name_get(obj, &table_name);
    TEST_ASSERT(VAR_OF_TABLE_NAME_T_CHECK(table_name, value));
    value += 1;

    of_table_stats_entry_wildcards_get(obj, &wc_bmap);
    TEST_ASSERT(VAR_OF_WC_BMAP_T_CHECK(wc_bmap, value));
    value += 1;

    of_table_stats_entry_match_get(obj, &match_bmap);
    TEST_ASSERT(VAR_OF_MATCH_BMAP_T_CHECK(match_bmap, value));
    value += 1;

    of_table_stats_entry_instructions_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_write_actions_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_apply_actions_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_max_entries_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_active_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_lookup_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_matched_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint32,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_1_populate_scalars(
    of_uint32_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint32,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_1_check_scalars(
    of_uint32_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint64,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_1_populate_scalars(
    of_uint64_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint64,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_1_check_scalars(
    of_uint64_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint8,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_1_populate_scalars(
    of_uint8_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint8,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_1_check_scalars(
    of_uint8_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_1_populate_scalars(
    ofp_queue_desc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_1_check_scalars(
    ofp_queue_desc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_1_populate_scalars(
    of_list_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_1_check_scalars(
    of_list_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_1_populate_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_1_check_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_1_check_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_1_check_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_1_check_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_1_populate_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_1_check_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_1_populate_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_1_check_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_1_populate_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_1_check_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_1_populate_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_1_check_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_1_populate_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_1_check_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_1_populate_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_1_check_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_1_populate_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_1_check_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_1_populate_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_1_check_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_1_populate_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_1_check_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_1_populate_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_1_check_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_1_populate_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_1_check_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_1_populate_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_1_check_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_1_populate_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_1_check_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_1_populate_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_1_check_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_1_populate_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_1_check_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_1_populate_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_1_check_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_1_populate_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_1_check_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_1_populate_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_1
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_1_check_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_byte_count_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_flow_count_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_2_check_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_aggregate_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_flow_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_2_populate_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_aggregate_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_aggregate_stats_request_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_request_out_group_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_request_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_request_cookie_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_2_check_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    of_aggregate_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_aggregate_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_aggregate_stats_request_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_request_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_2_populate_scalars(
    of_async_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_2_check_scalars(
    of_async_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_2_populate_scalars(
    of_async_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_2_check_scalars(
    of_async_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_set,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_2_populate_scalars(
    of_async_set_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_async_set,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_2_check_scalars(
    of_async_set_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_2_populate_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_action_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_action_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_2_check_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_action_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_action_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_2_populate_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_instruction_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_instruction_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_2_check_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_instruction_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_instruction_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_2_populate_scalars(
    of_bad_match_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_match_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_match_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_2_check_scalars(
    of_bad_match_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_match_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_match_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_2_populate_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_request_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_request_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_2_check_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_request_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_request_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_2_populate_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_2_check_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    of_barrier_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_2_populate_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_2_check_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    of_barrier_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_2_populate_scalars(
    of_bsn_arp_idle_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_2_check_scalars(
    of_bsn_arp_idle_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_clear_data_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_2_check_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_enabled_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_2_check_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_enable_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_enable_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_2_check_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_request_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_2_check_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_idle_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_idle_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_2_check_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_2_check_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_2_check_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_reply_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_request_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_2_check_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_2_check_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_2_populate_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_2_check_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    of_bsn_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_2_check_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_2_populate_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_2_check_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_timeout_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_2_check_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_2_populate_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_timeout_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_timeout_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_timeout_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_2_check_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_tx_interval_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_2_check_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_2_populate_scalars(
    of_bsn_role_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_2_check_scalars(
    of_bsn_role_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_2_check_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_2_check_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_2_check_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_2_check_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_mirroring_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_mirroring_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_2_check_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_set_mirroring_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_pktin_suppression_request_enabled_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_priority_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_set_pktin_suppression_request_cookie_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_2_check_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_2_check_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_2_populate_scalars(
    of_bsn_shell_command_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_2_check_scalars(
    of_bsn_shell_command_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_2_populate_scalars(
    of_bsn_shell_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_2_check_scalars(
    of_bsn_shell_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_2_populate_scalars(
    of_bsn_shell_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_2_check_scalars(
    of_bsn_shell_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_2_populate_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_2_check_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_time_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_time_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_time_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_2_check_scalars(
    of_bsn_time_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_status_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_2_check_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_2_check_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_2_check_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_2_populate_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_2_check_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_mfr_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_hw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_sw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_SERIAL_NUM_T_INIT(ser_num, value);
    of_desc_stats_reply_serial_num_set(obj, ser_num);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_dp_desc_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_2_check_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    of_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    TEST_ASSERT(VAR_OF_SERIAL_NUM_T_CHECK(ser_num, value));
    value += 1;

    of_desc_stats_reply_dp_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_2_populate_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_2_check_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_2_populate_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_2_check_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    of_echo_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_2_populate_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_2_check_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    of_echo_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_2_populate_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_error_msg_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_2_check_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    of_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_2_populate_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_2_check_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    of_experimenter_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_2_populate_scalars(
    of_experimenter_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_error_msg_subtype_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_error_msg_experimenter_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_2_check_scalars(
    of_experimenter_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_error_msg_subtype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_experimenter_error_msg_experimenter_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_2_populate_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_2_check_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_2_populate_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_2_check_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_2_populate_scalars(
    of_experimenter_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_2_check_scalars(
    of_experimenter_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_reply_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_2_check_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_experimenter_stats_reply_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_2_populate_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_request_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_2_check_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_experimenter_stats_request_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_2_populate_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_features_reply_datapath_id_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_n_buffers_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_features_reply_n_tables_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_capabilities_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_reserved_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_2_check_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    of_features_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_datapath_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_features_reply_n_buffers_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_n_tables_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_features_reply_capabilities_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_reserved_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_2_populate_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_features_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_2_check_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    of_features_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_2_populate_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_add_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_add_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_add_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_add_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_2_check_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_add_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_add_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_add_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_add_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_2_populate_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_delete_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_2_check_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_delete_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_2_populate_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_strict_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_delete_strict_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_2_check_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_strict_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_strict_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_strict_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_2_populate_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_mod_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_mod_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_mod_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_mod_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_2_check_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_mod_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_mod_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_mod_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_mod_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_2_populate_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_modify_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_2_check_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_modify_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_2_populate_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_strict_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_modify_strict_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_2_check_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_strict_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_strict_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_strict_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_2_populate_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_priority_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_removed_reason_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_removed_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_2_check_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    of_flow_removed_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_removed_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_removed_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_2_check_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_2_populate_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_stats_request_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_request_out_group_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_request_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_request_cookie_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_2_check_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    of_flow_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_stats_request_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_request_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_2_populate_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_2_check_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_get_config_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_get_config_reply_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_2_populate_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_2_check_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    of_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_2_populate_scalars(
    of_group_add_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_add_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_add_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_2_check_scalars(
    of_group_add_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_add_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_add_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_2_populate_scalars(
    of_group_delete_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_delete_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_delete_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_2_check_scalars(
    of_group_delete_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_delete_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_delete_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_group_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_2_check_scalars(
    of_group_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_2_populate_scalars(
    of_group_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_2_check_scalars(
    of_group_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_group_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_features_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_types_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_capabilities_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_all_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_select_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_indirect_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_ff_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_all_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_select_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_indirect_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_ff_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_2_check_scalars(
    of_group_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_features_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_group_features_stats_reply_types_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_capabilities_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_all_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_select_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_indirect_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_ff_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_all_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_select_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_indirect_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_ff_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_2_populate_scalars(
    of_group_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_features_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_2_check_scalars(
    of_group_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_features_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_2_populate_scalars(
    of_group_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_mod_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_2_check_scalars(
    of_group_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_mod_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_mod_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_2_populate_scalars(
    of_group_modify_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_modify_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_modify_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_2_check_scalars(
    of_group_modify_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_modify_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_modify_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_group_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_2_check_scalars(
    of_group_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_2_populate_scalars(
    of_group_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_request_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_2_check_scalars(
    of_group_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_group_stats_request_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_2_populate_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_2_check_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    of_hello_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_hello_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_hello_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_hello_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_2_populate_scalars(
    of_meter_add_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_2_check_scalars(
    of_meter_add_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_2_populate_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_2_check_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_meter_config_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_2_check_scalars(
    of_meter_config_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_2_populate_scalars(
    of_meter_config_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_2_check_scalars(
    of_meter_config_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_2_populate_scalars(
    of_meter_delete_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_2_check_scalars(
    of_meter_delete_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_meter_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_2_check_scalars(
    of_meter_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_2_populate_scalars(
    of_meter_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_2_check_scalars(
    of_meter_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_2_populate_scalars(
    of_meter_mod_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_2_check_scalars(
    of_meter_mod_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_2_populate_scalars(
    of_meter_modify_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_2_check_scalars(
    of_meter_modify_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_meter_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_2_check_scalars(
    of_meter_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_2_populate_scalars(
    of_meter_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_2_check_scalars(
    of_meter_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_2_populate_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_2_check_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_2_populate_scalars(
    of_nicira_controller_role_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_2_check_scalars(
    of_nicira_controller_role_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_2_populate_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_2_check_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    of_nicira_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_2_populate_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_buffer_id_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_packet_in_total_len_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_packet_in_reason_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_packet_in_table_id_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_2_check_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    of_packet_in_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_total_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_packet_in_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_packet_in_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_2_populate_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_out_in_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_2_check_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_packet_out_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_port_desc_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_2_check_scalars(
    of_port_desc_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_2_populate_scalars(
    of_port_desc_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_2_check_scalars(
    of_port_desc_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_2_populate_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_mod_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_mod_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_mask_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_advertise_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_2_check_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    of_port_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_mod_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_advertise_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_2_check_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_2_populate_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_request_port_no_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_2_check_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_port_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_port_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_2_populate_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_port_status_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_port_status_reason_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_2_check_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_port_status_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_status_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_2_populate_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_reply_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_2_check_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_reply_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_2_populate_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_request_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_request_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_2_check_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_request_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_op_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_op_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_op_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_2_check_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_2_populate_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_2_check_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_queue_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_queue_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_request_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_2_populate_scalars(
    of_role_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_role_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_role_reply_role_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_role_reply_generation_id_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_2_check_scalars(
    of_role_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_role_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_reply_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_reply_generation_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_2_populate_scalars(
    of_role_request_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_role_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_role_request_role_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_role_request_generation_id_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_role_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_2_check_scalars(
    of_role_request_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_role_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_request_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_request_generation_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_role_request_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_role_request_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_role_request_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_request_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_set_config,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_2_populate_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_set_config_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_set_config,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_2_check_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_set_config_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_set_config_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_set_config_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_2_check_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_2_populate_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_2_check_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_switch_config_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_switch_config_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_switch_config_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_switch_config_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_table_features_stats_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_2_check_scalars(
    of_table_features_stats_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_2_populate_scalars(
    of_table_features_stats_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_2_check_scalars(
    of_table_features_stats_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_2_populate_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_table_mod_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_config_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_2_check_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_table_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_mod_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_2_populate_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_2_check_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_2_populate_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_2_check_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_2_populate_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_2_check_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_2_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_2_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_2_populate_scalars(
    ofdpa_queue_mod_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_2_check_scalars(
    ofdpa_queue_mod_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_2_populate_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_2_check_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_2_populate_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_2_check_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_2_populate_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_2_check_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_2_populate_scalars(
    of_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_2_check_scalars(
    of_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_2_populate_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_2_check_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_2_populate_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_dest_port_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_vlan_tag_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_bsn_mirror_copy_stage_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_2_check_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_action_bsn_mirror_dest_port_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_copy_stage_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_populate_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_set_tunnel_dst_dst_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_check_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_2_populate_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_2_check_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_2_populate_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_2_check_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_2_check_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_2_check_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_2_populate_scalars(
    of_action_enqueue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_2_check_scalars(
    of_action_enqueue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_2_populate_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_2_check_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_group,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_2_populate_scalars(
    of_action_group_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_group_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_group,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_2_check_scalars(
    of_action_group_t *obj, int value) {
    uint32_t val32;

    of_action_group_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_2_populate_scalars(
    of_action_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_2_check_scalars(
    of_action_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_2_populate_scalars(
    of_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_2_check_scalars(
    of_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_2_populate_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_2_check_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_2_populate_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_2_check_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_2_populate_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_2_check_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_2_populate_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_2_check_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_2_populate_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_2_check_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_2_check_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_2_check_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_2_populate_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_2_check_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_2_populate_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_2_check_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_2_populate_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_2_check_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_2_populate_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_2_check_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_2_check_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_2_check_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_2_populate_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_2_check_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_2_populate_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_2_check_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_2_populate_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_2_check_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_2_populate_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_2_check_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_2_populate_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_2_check_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_2_populate_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_2_check_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_2_populate_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_2_check_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_2_populate_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_2_check_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_2_check_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_2_check_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_2_populate_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_2_check_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_2_populate_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_2_check_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_2_check_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_2_populate_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_2_check_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_2_populate_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_action_output_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_action_output_max_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_2_check_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    of_action_output_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_action_output_max_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_2_populate_scalars(
    of_action_pop_mpls_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_pop_mpls_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_2_check_scalars(
    of_action_pop_mpls_t *obj, int value) {
    uint16_t val16;

    of_action_pop_mpls_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_2_populate_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_2_check_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_2_populate_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_2_check_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_2_populate_scalars(
    of_action_push_mpls_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_push_mpls_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_2_check_scalars(
    of_action_push_mpls_t *obj, int value) {
    uint16_t val16;

    of_action_push_mpls_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_2_populate_scalars(
    of_action_push_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_2_check_scalars(
    of_action_push_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_2_populate_scalars(
    of_action_push_vlan_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_push_vlan_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_2_check_scalars(
    of_action_push_vlan_t *obj, int value) {
    uint16_t val16;

    of_action_push_vlan_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_2_populate_scalars(
    of_action_set_dl_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_2_check_scalars(
    of_action_set_dl_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_2_populate_scalars(
    of_action_set_dl_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_2_check_scalars(
    of_action_set_dl_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_2_populate_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_2_check_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_2_populate_scalars(
    of_action_set_mpls_label_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_2_check_scalars(
    of_action_set_mpls_label_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_2_populate_scalars(
    of_action_set_mpls_tc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_2_check_scalars(
    of_action_set_mpls_tc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_mpls_ttl_mpls_ttl_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_2_check_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    of_action_set_mpls_ttl_mpls_ttl_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_2_populate_scalars(
    of_action_set_nw_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_2_check_scalars(
    of_action_set_nw_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_2_populate_scalars(
    of_action_set_nw_ecn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_2_check_scalars(
    of_action_set_nw_ecn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_2_populate_scalars(
    of_action_set_nw_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_2_check_scalars(
    of_action_set_nw_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_2_populate_scalars(
    of_action_set_nw_tos_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_2_check_scalars(
    of_action_set_nw_tos_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_2_populate_scalars(
    of_action_set_nw_ttl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_nw_ttl_nw_ttl_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_2_check_scalars(
    of_action_set_nw_ttl_t *obj, int value) {
    uint8_t val8;

    of_action_set_nw_ttl_nw_ttl_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_2_populate_scalars(
    of_action_set_queue_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_queue_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_2_check_scalars(
    of_action_set_queue_t *obj, int value) {
    uint32_t val32;

    of_action_set_queue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_2_populate_scalars(
    of_action_set_tp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_2_check_scalars(
    of_action_set_tp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_2_populate_scalars(
    of_action_set_tp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_2_check_scalars(
    of_action_set_tp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_2_populate_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_2_check_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_2_populate_scalars(
    of_action_set_vlan_vid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_2_check_scalars(
    of_action_set_vlan_vid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_2_populate_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_2_check_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_2_populate_scalars(
    of_bsn_controller_connection_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_2_check_scalars(
    of_bsn_controller_connection_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_2_populate_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_interface_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_interface_name_set(obj, port_name);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_addr_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_netmask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_2_check_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_interface_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_mac_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_mac_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_2_populate_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_2_check_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_2_check_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_2_populate_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_2_check_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_2_populate_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_2_check_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_2_populate_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_vport_q_in_q_port_no_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_vlan_id_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_vlan_id_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_vport_q_in_q_if_name_set(obj, port_name);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_2_check_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_2_populate_scalars(
    of_bucket_t *obj, int value) {
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_bucket_weight_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bucket_watch_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bucket_watch_group_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bucket,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_2_check_scalars(
    of_bucket_t *obj, int value) {
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;

    of_bucket_weight_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bucket_watch_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bucket_watch_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_2_populate_scalars(
    of_bucket_counter_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bucket_counter_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bucket_counter_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_2_check_scalars(
    of_bucket_counter_t *obj, int value) {
    uint64_t val64;

    of_bucket_counter_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_bucket_counter_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_2_check_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_flow_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_entry_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_group_desc_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;

    VAR_UINT8_T_INIT(val8, value);
    of_group_desc_stats_entry_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_entry_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_group_desc_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;

    of_group_desc_stats_entry_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_desc_stats_entry_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_group_stats_entry_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_group_id_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_ref_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_group_stats_entry_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_group_stats_entry_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_2_check_scalars(
    of_group_stats_entry_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_group_stats_entry_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_entry_ref_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_entry_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_group_stats_entry_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_2_populate_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_2_check_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    of_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_2_populate_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_2_check_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_2_populate_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_2_check_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_2_populate_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_2_check_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_2_populate_scalars(
    of_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_2_check_scalars(
    of_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_2_populate_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_2_check_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_2_populate_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_2_check_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_2_populate_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_2_check_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_2_populate_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_2_check_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_2_populate_scalars(
    of_instruction_goto_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_instruction_goto_table_table_id_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_2_check_scalars(
    of_instruction_goto_table_t *obj, int value) {
    uint8_t val8;

    of_instruction_goto_table_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_2_populate_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_2_check_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_2_check_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_2_check_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_2_check_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_2_check_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_2_check_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_2_check_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_2_check_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_2_check_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_2_check_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_2_populate_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_2_check_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_2_populate_scalars(
    of_instruction_meter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_2_check_scalars(
    of_instruction_meter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_2_populate_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_2_check_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_2_populate_scalars(
    of_instruction_write_metadata_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_instruction_write_metadata_metadata_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_instruction_write_metadata_metadata_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_2_check_scalars(
    of_instruction_write_metadata_t *obj, int value) {
    uint64_t val64;

    of_instruction_write_metadata_metadata_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_instruction_write_metadata_metadata_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_2_populate_scalars(
    of_match_v1_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_2_check_scalars(
    of_match_v1_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_2_populate_scalars(
    of_match_v2_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_2_check_scalars(
    of_match_v2_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_2_populate_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_2_check_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_2_populate_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_2_check_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_2_populate_scalars(
    of_meter_band_drop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_2_check_scalars(
    of_meter_band_drop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_2_populate_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_2_check_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_2_populate_scalars(
    of_meter_band_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_2_check_scalars(
    of_meter_band_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_2_populate_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_2_check_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_2_populate_scalars(
    of_meter_band_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_2_check_scalars(
    of_meter_band_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_2_populate_scalars(
    of_meter_config_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_2_check_scalars(
    of_meter_config_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_2_populate_scalars(
    of_meter_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_2_check_scalars(
    of_meter_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_2_populate_scalars(
    of_meter_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_2_check_scalars(
    of_meter_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_2_populate_scalars(
    of_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_2_check_scalars(
    of_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_op_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_arp_op_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_op_t *obj, int value) {
    uint16_t val16;

    of_oxm_arp_op_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_arp_op_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_arp_op_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_arp_op_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_arp_op_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_sha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_sha_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_sha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_sha_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_sha_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_sha_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_sha_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_arp_sha_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_spa_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_spa_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_spa_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_spa_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_spa_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_spa_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_spa_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_arp_spa_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_tha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_tha_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_tha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_tha_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_tha_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_tha_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_tha_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_arp_tha_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_tpa_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_tpa_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_tpa_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_tpa_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_tpa_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_tpa_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_tpa_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_arp_tpa_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_egr_port_group_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_egr_port_group_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_egr_port_group_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_egr_port_group_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_egr_port_group_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_egr_port_group_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_bsn_global_vrf_allowed_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {
    uint8_t val8;

    of_oxm_bsn_global_vrf_allowed_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_bsn_global_vrf_allowed_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_bsn_global_vrf_allowed_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_bsn_global_vrf_allowed_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_bsn_global_vrf_allowed_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    VAR_OF_BITMAP_128_T_INIT(bitmap_128, value);
    of_oxm_bsn_in_ports_128_value_set(obj, bitmap_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    of_oxm_bsn_in_ports_128_value_get(obj, &bitmap_128);
    TEST_ASSERT(VAR_OF_BITMAP_128_T_CHECK(bitmap_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    VAR_OF_BITMAP_128_T_INIT(bitmap_128, value);
    of_oxm_bsn_in_ports_128_masked_value_set(obj, bitmap_128);
    value += 1;

    VAR_OF_BITMAP_128_T_INIT(bitmap_128, value);
    of_oxm_bsn_in_ports_128_masked_value_mask_set(obj, bitmap_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    of_oxm_bsn_in_ports_128_masked_value_get(obj, &bitmap_128);
    TEST_ASSERT(VAR_OF_BITMAP_128_T_CHECK(bitmap_128, value));
    value += 1;

    of_oxm_bsn_in_ports_128_masked_value_mask_get(obj, &bitmap_128);
    TEST_ASSERT(VAR_OF_BITMAP_128_T_CHECK(bitmap_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_dst_class_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_dst_class_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_dst_class_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_dst_class_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_dst_class_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_l3_dst_class_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_interface_class_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_interface_class_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_interface_class_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_interface_class_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_interface_class_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_l3_interface_class_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_src_class_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_src_class_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_src_class_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_src_class_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_src_class_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_l3_src_class_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_lag_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_lag_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_lag_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_lag_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_lag_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_lag_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf0_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf0_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf0_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf0_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf0_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf0_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf1_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf1_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf1_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf1_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf1_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf1_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf2_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf2_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf2_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf2_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf2_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf2_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf3_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf3_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf3_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf3_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf3_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf3_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf4_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf4_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf4_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf4_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf4_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf4_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf5_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf5_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf5_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf5_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf5_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf5_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf6_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf6_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf6_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf6_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf6_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf6_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf7_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf7_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf7_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf7_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf7_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf7_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_vrf_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_vrf_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_vrf_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_vrf_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_vrf_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_vrf_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_2_populate_scalars(
    of_oxm_eth_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_dst_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_2_check_scalars(
    of_oxm_eth_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_dst_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_dst_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_dst_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_dst_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_eth_dst_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_2_populate_scalars(
    of_oxm_eth_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_src_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_2_check_scalars(
    of_oxm_eth_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_src_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_src_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_src_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_src_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_eth_src_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_2_populate_scalars(
    of_oxm_eth_type_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_eth_type_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_2_check_scalars(
    of_oxm_eth_type_t *obj, int value) {
    uint16_t val16;

    of_oxm_eth_type_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_eth_type_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_eth_type_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_eth_type_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_eth_type_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_2_populate_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_2_check_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_2_populate_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_2_check_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_code_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_code_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_code_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_code_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_code_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv4_code_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_type_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_type_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_type_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_type_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_type_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv4_type_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_code_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_code_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_code_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_code_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_code_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv6_code_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_type_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_type_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_type_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_type_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_type_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv6_type_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_2_populate_scalars(
    of_oxm_in_phy_port_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_phy_port_value_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_2_check_scalars(
    of_oxm_in_phy_port_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_phy_port_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_phy_port_masked_value_set(obj, port_no);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_phy_port_masked_value_mask_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_phy_port_masked_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_oxm_in_phy_port_masked_value_mask_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_2_populate_scalars(
    of_oxm_in_port_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_port_value_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_2_check_scalars(
    of_oxm_in_port_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_port_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_in_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_port_masked_value_set(obj, port_no);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_port_masked_value_mask_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_in_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_port_masked_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_oxm_in_port_masked_value_mask_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_2_populate_scalars(
    of_oxm_ip_dscp_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_dscp_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_2_check_scalars(
    of_oxm_ip_dscp_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_dscp_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_dscp_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_dscp_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_dscp_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_ip_dscp_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_2_populate_scalars(
    of_oxm_ip_ecn_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_ecn_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_2_check_scalars(
    of_oxm_ip_ecn_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_ecn_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_ecn_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_ecn_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_ecn_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_ip_ecn_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_2_populate_scalars(
    of_oxm_ip_proto_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_proto_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_2_check_scalars(
    of_oxm_ip_proto_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_proto_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_proto_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_proto_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_proto_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_ip_proto_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_dst_value_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_dst_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_dst_masked_value_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_dst_masked_value_mask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_dst_masked_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_oxm_ipv4_dst_masked_value_mask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv4_src_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_src_value_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv4_src_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_src_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_src_masked_value_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_src_masked_value_mask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_src_masked_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_oxm_ipv4_src_masked_value_mask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_dst_value_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_dst_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_dst_masked_value_set(obj, ipv6);
    value += 1;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_dst_masked_value_mask_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_dst_masked_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    of_oxm_ipv6_dst_masked_value_mask_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ipv6_flabel_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {
    uint32_t val32;

    of_oxm_ipv6_flabel_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ipv6_flabel_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ipv6_flabel_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_ipv6_flabel_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_ipv6_flabel_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_sll_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_sll_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_sll_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_sll_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_sll_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_ipv6_nd_sll_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_nd_target_value_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_nd_target_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_nd_target_masked_value_set(obj, ipv6);
    value += 1;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_nd_target_masked_value_mask_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_nd_target_masked_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    of_oxm_ipv6_nd_target_masked_value_mask_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_tll_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_tll_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_tll_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_tll_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_tll_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_ipv6_nd_tll_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_src_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_src_value_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_src_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_src_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_src_masked_value_set(obj, ipv6);
    value += 1;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_src_masked_value_mask_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_src_masked_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    of_oxm_ipv6_src_masked_value_mask_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_2_populate_scalars(
    of_oxm_metadata_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_metadata_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_2_check_scalars(
    of_oxm_metadata_t *obj, int value) {
    uint64_t val64;

    of_oxm_metadata_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_metadata_masked_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_metadata_masked_value_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_metadata_masked_value_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_metadata_masked_t *obj, int value) {
    uint64_t val64;

    of_oxm_metadata_masked_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_oxm_metadata_masked_value_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_2_populate_scalars(
    of_oxm_mpls_bos_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_bos_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_2_check_scalars(
    of_oxm_mpls_bos_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_bos_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_bos_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_bos_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_bos_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_mpls_bos_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_2_populate_scalars(
    of_oxm_mpls_label_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_mpls_label_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_2_check_scalars(
    of_oxm_mpls_label_t *obj, int value) {
    uint32_t val32;

    of_oxm_mpls_label_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_mpls_label_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_mpls_label_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_mpls_label_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_mpls_label_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_2_populate_scalars(
    of_oxm_mpls_tc_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_tc_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_2_check_scalars(
    of_oxm_mpls_tc_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_tc_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_tc_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_tc_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_tc_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_mpls_tc_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_2_populate_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_2_check_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_2_populate_scalars(
    of_oxm_sctp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_dst_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_2_check_scalars(
    of_oxm_sctp_dst_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_dst_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_dst_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_dst_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_dst_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_sctp_dst_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_2_populate_scalars(
    of_oxm_sctp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_src_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_2_check_scalars(
    of_oxm_sctp_src_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_src_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_src_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_src_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_src_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_sctp_src_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_2_populate_scalars(
    of_oxm_tcp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_dst_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_2_check_scalars(
    of_oxm_tcp_dst_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_dst_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_dst_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_dst_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_dst_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_tcp_dst_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_2_populate_scalars(
    of_oxm_tcp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_src_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_2_check_scalars(
    of_oxm_tcp_src_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_src_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_src_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_src_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_src_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_tcp_src_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_2_populate_scalars(
    of_oxm_tunnel_id_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_tunnel_id_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_2_check_scalars(
    of_oxm_tunnel_id_t *obj, int value) {
    uint64_t val64;

    of_oxm_tunnel_id_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_tunnel_id_masked_value_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_tunnel_id_masked_value_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {
    uint64_t val64;

    of_oxm_tunnel_id_masked_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_oxm_tunnel_id_masked_value_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_2_populate_scalars(
    of_oxm_udp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_dst_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_2_check_scalars(
    of_oxm_udp_dst_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_dst_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_dst_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_dst_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_dst_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_udp_dst_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_2_populate_scalars(
    of_oxm_udp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_src_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_2_check_scalars(
    of_oxm_udp_src_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_src_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_src_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_src_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_src_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_udp_src_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_2_populate_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_vlan_pcp_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_2_check_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    of_oxm_vlan_pcp_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_vlan_pcp_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_vlan_pcp_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_vlan_pcp_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_vlan_pcp_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_2_populate_scalars(
    of_oxm_vlan_vid_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_vlan_vid_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_2_check_scalars(
    of_oxm_vlan_vid_t *obj, int value) {
    uint16_t val16;

    of_oxm_vlan_vid_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_2_populate_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_vlan_vid_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_vlan_vid_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_2_check_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_vlan_vid_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_vlan_vid_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_2_populate_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_queue_queue_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_queue_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_2_check_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_packet_queue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_queue_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_2_populate_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_desc_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_desc_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_port_desc_name_set(obj, port_name);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_state_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_curr_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_advertised_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_supported_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_peer_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_curr_speed_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_max_speed_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_2_check_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    of_port_desc_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_desc_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_desc_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_port_desc_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_state_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_curr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_advertised_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_supported_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_peer_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_curr_speed_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_max_speed_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_frame_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_over_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_crc_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_collisions_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_2_check_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;

    of_port_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_stats_entry_rx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_over_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_collisions_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_2_populate_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_2_check_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_2_populate_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_2_check_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_2_populate_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_2_check_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_2_populate_scalars(
    of_queue_prop_max_rate_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_prop_max_rate_rate_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_2_check_scalars(
    of_queue_prop_max_rate_t *obj, int value) {
    uint16_t val16;

    of_queue_prop_max_rate_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_2_populate_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_prop_min_rate_rate_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_2_check_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    of_queue_prop_min_rate_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_entry_queue_id_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_2_check_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    of_queue_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_entry_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_2_populate_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_2_check_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_2_populate_scalars(
    of_table_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_features,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_2_check_scalars(
    of_table_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    of_match_bmap_t match_bmap;
    of_wc_bmap_t wc_bmap;
    uint32_t val32;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_table_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_OF_TABLE_NAME_T_INIT(table_name, value);
    of_table_stats_entry_name_set(obj, table_name);
    value += 1;

    VAR_OF_MATCH_BMAP_T_INIT(match_bmap, value);
    of_table_stats_entry_match_set(obj, match_bmap);
    value += 1;

    VAR_OF_WC_BMAP_T_INIT(wc_bmap, value);
    of_table_stats_entry_wildcards_set(obj, wc_bmap);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_write_actions_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_apply_actions_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_write_setfields_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_apply_setfields_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_metadata_match_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_metadata_write_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_instructions_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_max_entries_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_active_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_lookup_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_matched_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_2_check_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    of_match_bmap_t match_bmap;
    of_wc_bmap_t wc_bmap;
    uint32_t val32;
    uint64_t val64;

    of_table_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_stats_entry_name_get(obj, &table_name);
    TEST_ASSERT(VAR_OF_TABLE_NAME_T_CHECK(table_name, value));
    value += 1;

    of_table_stats_entry_match_get(obj, &match_bmap);
    TEST_ASSERT(VAR_OF_MATCH_BMAP_T_CHECK(match_bmap, value));
    value += 1;

    of_table_stats_entry_wildcards_get(obj, &wc_bmap);
    TEST_ASSERT(VAR_OF_WC_BMAP_T_CHECK(wc_bmap, value));
    value += 1;

    of_table_stats_entry_write_actions_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_apply_actions_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_write_setfields_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_apply_setfields_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_metadata_match_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_metadata_write_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_instructions_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_max_entries_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_active_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_lookup_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_matched_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint32,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_2_populate_scalars(
    of_uint32_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint32,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_2_check_scalars(
    of_uint32_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint64,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_2_populate_scalars(
    of_uint64_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint64,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_2_check_scalars(
    of_uint64_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint8,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_2_populate_scalars(
    of_uint8_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_uint8,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_2_check_scalars(
    of_uint8_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_2_populate_scalars(
    ofp_queue_desc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_2_check_scalars(
    ofp_queue_desc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_2_populate_scalars(
    of_list_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_2_check_scalars(
    of_list_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_2_populate_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_2_check_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_2_check_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_2_check_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_2_check_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_2_populate_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_2_check_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_2_populate_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_2_check_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_2_populate_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_2_check_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_2_populate_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_2_check_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_2_populate_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_2_check_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_2_populate_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_2_check_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_2_populate_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_2_check_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_2_populate_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_2_check_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_2_populate_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_2_check_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_2_populate_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_2_check_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_2_populate_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_2_check_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_2_populate_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_2_check_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_2_populate_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_2_check_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_2_populate_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_2_check_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_2_populate_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_2_check_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_2_populate_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_2_check_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_2_populate_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_2_check_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_2_populate_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_2
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_2_check_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_reply_byte_count_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_reply_flow_count_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_reply_OF_VERSION_1_3_check_scalars(
    of_aggregate_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_aggregate_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_reply_flow_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_3_populate_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_aggregate_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_aggregate_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_aggregate_stats_request_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_aggregate_stats_request_out_group_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_request_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_aggregate_stats_request_cookie_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_aggregate_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_aggregate_stats_request_OF_VERSION_1_3_check_scalars(
    of_aggregate_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    of_aggregate_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_aggregate_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_aggregate_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_aggregate_stats_request_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_aggregate_stats_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_aggregate_stats_request_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_3_populate_scalars(
    of_async_get_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_reply_packet_in_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_reply_packet_in_mask_slave_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_reply_port_status_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_reply_port_status_mask_slave_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_reply_flow_removed_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_reply_flow_removed_mask_slave_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_reply_OF_VERSION_1_3_check_scalars(
    of_async_get_reply_t *obj, int value) {
    uint32_t val32;

    of_async_get_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_reply_packet_in_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_reply_packet_in_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_reply_port_status_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_reply_port_status_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_reply_flow_removed_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_reply_flow_removed_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_3_populate_scalars(
    of_async_get_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_request_packet_in_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_request_packet_in_mask_slave_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_request_port_status_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_request_port_status_mask_slave_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_request_flow_removed_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_get_request_flow_removed_mask_slave_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_async_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_get_request_OF_VERSION_1_3_check_scalars(
    of_async_get_request_t *obj, int value) {
    uint32_t val32;

    of_async_get_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_request_packet_in_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_request_packet_in_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_request_port_status_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_request_port_status_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_request_flow_removed_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_get_request_flow_removed_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_async_set,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_3_populate_scalars(
    of_async_set_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_async_set_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_set_packet_in_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_set_packet_in_mask_slave_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_set_port_status_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_set_port_status_mask_slave_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_set_flow_removed_mask_equal_master_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_async_set_flow_removed_mask_slave_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_async_set,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_async_set_OF_VERSION_1_3_check_scalars(
    of_async_set_t *obj, int value) {
    uint32_t val32;

    of_async_set_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_set_packet_in_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_set_packet_in_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_set_port_status_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_set_port_status_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_set_flow_removed_mask_equal_master_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_async_set_flow_removed_mask_slave_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_3_populate_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_action_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_action_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_action_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_action_error_msg_OF_VERSION_1_3_check_scalars(
    of_bad_action_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_action_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_action_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_3_populate_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_instruction_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_instruction_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_instruction_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_instruction_error_msg_OF_VERSION_1_3_check_scalars(
    of_bad_instruction_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_instruction_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_instruction_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_3_populate_scalars(
    of_bad_match_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_match_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_match_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_match_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_match_error_msg_OF_VERSION_1_3_check_scalars(
    of_bad_match_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_match_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_match_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_3_populate_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bad_request_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bad_request_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bad_request_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bad_request_error_msg_OF_VERSION_1_3_check_scalars(
    of_bad_request_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bad_request_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bad_request_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_3_populate_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_reply_OF_VERSION_1_3_check_scalars(
    of_barrier_reply_t *obj, int value) {
    uint32_t val32;

    of_barrier_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_3_populate_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_barrier_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_barrier_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_barrier_request_OF_VERSION_1_3_check_scalars(
    of_barrier_request_t *obj, int value) {
    uint32_t val32;

    of_barrier_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_3_populate_scalars(
    of_bsn_arp_idle_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_ipv4_t ipv4;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_arp_idle_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_arp_idle_vlan_vid_set(obj, val16);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_arp_idle_ipv4_addr_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_arp_idle,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_arp_idle_OF_VERSION_1_3_check_scalars(
    of_bsn_arp_idle_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_ipv4_t ipv4;

    of_bsn_arp_idle_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_arp_idle_vlan_vid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_arp_idle_ipv4_addr_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_bw_clear_data_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_clear_data_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_clear_data_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_clear_data_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_clear_data_request_OF_VERSION_1_3_check_scalars(
    of_bsn_bw_clear_data_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_reply_enabled_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_bw_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_get_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_get_request_OF_VERSION_1_3_check_scalars(
    of_bsn_bw_enable_get_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_enable_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_bw_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_bw_enable_set_request_enable_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_bw_enable_set_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_bw_enable_set_request_OF_VERSION_1_3_check_scalars(
    of_bsn_bw_enable_set_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_bw_enable_set_request_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_controller_connections_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_controller_connections_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_controller_connections_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_controller_connections_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connections_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connections_request_OF_VERSION_1_3_check_scalars(
    of_bsn_controller_connections_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_controller_connections_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_debug_counter_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_debug_counter_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_debug_counter_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_debug_counter_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_debug_counter_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_debug_counter_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_debug_counter_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_debug_counter_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_debug_counter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_debug_counter_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_debug_counter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_debug_counter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_debug_counter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_debug_counter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_debug_counter_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_debug_counter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_debug_counter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_debug_counter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_checksum_bucket_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_flow_checksum_bucket_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_flow_checksum_bucket_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_flow_checksum_bucket_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_checksum_bucket_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_flow_checksum_bucket_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_flow_checksum_bucket_stats_request_table_id_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    of_bsn_flow_checksum_bucket_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_flow_checksum_bucket_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_flow_checksum_bucket_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_idle_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_flow_idle_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_flow_idle_priority_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_flow_idle_table_id_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_idle_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    of_bsn_flow_idle_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_flow_idle_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_bsn_flow_idle_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_flow_idle_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_get_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_get_reply_enabled_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_flow_idle_enable_get_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_flow_idle_enable_get_reply_enabled_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_get_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_get_request_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_idle_enable_get_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_flow_idle_enable_get_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_set_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_set_reply_enable_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_set_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_flow_idle_enable_set_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_flow_idle_enable_set_reply_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_flow_idle_enable_set_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_set_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_flow_idle_enable_set_request_enable_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_idle_enable_set_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_idle_enable_set_request_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_idle_enable_set_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_flow_idle_enable_set_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_flow_idle_enable_set_request_enable_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_bucket_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_bucket_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_bucket_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_bucket_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_bucket_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_bucket_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_bucket_stats_request_table_id_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_bucket_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_bucket_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_bucket_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_bucket_stats_request_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_clear_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_clear_reply_table_id_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_clear_reply_deleted_count_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_clear_reply_error_count_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_clear_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_clear_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_clear_reply_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_clear_reply_deleted_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_clear_reply_error_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_clear_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_clear_request_table_id_set(obj, val16);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_clear_request_checksum_set(obj, checksum_128);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_clear_request_checksum_mask_set(obj, checksum_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_clear_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_clear_request_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_clear_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    of_bsn_gentable_clear_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_clear_request_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_clear_request_checksum_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    of_bsn_gentable_clear_request_checksum_mask_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_entry_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_add_table_id_set(obj, val16);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_entry_add_checksum_set(obj, checksum_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_add_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_add_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    of_bsn_gentable_entry_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_entry_add_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_entry_add_checksum_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_entry_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_delete_table_id_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_delete_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_delete_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_entry_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_entry_delete_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_entry_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_entry_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_entry_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_entry_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_desc_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_desc_stats_request_table_id_set(obj, val16);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_entry_desc_stats_request_checksum_set(obj, checksum_128);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_entry_desc_stats_request_checksum_mask_set(obj, checksum_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    of_bsn_gentable_entry_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_entry_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_entry_desc_stats_request_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_entry_desc_stats_request_checksum_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    of_bsn_gentable_entry_desc_stats_request_checksum_mask_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_entry_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_entry_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_entry_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_entry_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_entry_stats_request_table_id_set(obj, val16);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_entry_stats_request_checksum_set(obj, checksum_128);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_entry_stats_request_checksum_mask_set(obj, checksum_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    of_bsn_gentable_entry_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_entry_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_entry_stats_request_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_entry_stats_request_checksum_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    of_bsn_gentable_entry_stats_request_checksum_mask_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_set_buckets_size_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_set_buckets_size_table_id_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_set_buckets_size_buckets_size_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_set_buckets_size,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_set_buckets_size_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_set_buckets_size_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_set_buckets_size_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_set_buckets_size_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_set_buckets_size_buckets_size_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_gentable_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_get_interfaces_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_interfaces_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_interfaces_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_interfaces_request_OF_VERSION_1_3_check_scalars(
    of_bsn_get_interfaces_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_get_ip_mask_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_ip_mask_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_ip_mask_request_OF_VERSION_1_3_check_scalars(
    of_bsn_get_ip_mask_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_get_l2_table_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_l2_table_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_l2_table_request_OF_VERSION_1_3_check_scalars(
    of_bsn_get_l2_table_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_reply_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_get_mirroring_reply_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_mirroring_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_get_mirroring_request_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_mirroring_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_mirroring_request_OF_VERSION_1_3_check_scalars(
    of_bsn_get_mirroring_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {
    uint32_t val32;
    of_desc_str_t desc_str;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_switch_pipeline_reply_xid_set(obj, val32);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_bsn_get_switch_pipeline_reply_pipeline_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_get_switch_pipeline_reply_t *obj, int value) {
    uint32_t val32;
    of_desc_str_t desc_str;

    of_bsn_get_switch_pipeline_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_get_switch_pipeline_reply_pipeline_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_get_switch_pipeline_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_get_switch_pipeline_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_get_switch_pipeline_request_OF_VERSION_1_3_check_scalars(
    of_bsn_get_switch_pipeline_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_get_switch_pipeline_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_3_populate_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_header_OF_VERSION_1_3_check_scalars(
    of_bsn_header_t *obj, int value) {
    uint32_t val32;

    of_bsn_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_hybrid_get_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_hybrid_get_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_hybrid_get_request_OF_VERSION_1_3_check_scalars(
    of_bsn_hybrid_get_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_3_populate_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_lacp_convergence_notif_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_lacp_convergence_notif_convergence_status_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_lacp_convergence_notif_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_actor_sys_priority_set(obj, val16);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_lacp_convergence_notif_actor_sys_mac_set(obj, mac_addr);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_actor_port_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_actor_port_num_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_actor_key_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_partner_sys_priority_set(obj, val16);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_lacp_convergence_notif_partner_sys_mac_set(obj, mac_addr);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_partner_port_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_partner_port_num_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_convergence_notif_partner_key_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_convergence_notif,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_convergence_notif_OF_VERSION_1_3_check_scalars(
    of_bsn_lacp_convergence_notif_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;

    of_bsn_lacp_convergence_notif_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_lacp_convergence_notif_convergence_status_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_lacp_convergence_notif_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_lacp_convergence_notif_actor_sys_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_convergence_notif_actor_sys_mac_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_lacp_convergence_notif_actor_port_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_convergence_notif_actor_port_num_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_convergence_notif_actor_key_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_convergence_notif_partner_sys_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_convergence_notif_partner_sys_mac_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_lacp_convergence_notif_partner_port_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_convergence_notif_partner_port_num_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_convergence_notif_partner_key_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_lacp_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_lacp_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_lacp_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_lacp_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_lacp_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_lacp_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_lacp_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_lacp_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_pdu_rx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_request_timeout_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_request_OF_VERSION_1_3_check_scalars(
    of_bsn_pdu_rx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_3_populate_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_rx_timeout_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_rx_timeout_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_rx_timeout_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_rx_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_rx_timeout_OF_VERSION_1_3_check_scalars(
    of_bsn_pdu_rx_timeout_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_reply_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_reply_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_pdu_tx_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_pdu_tx_request_tx_interval_ms_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_pdu_tx_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_pdu_tx_request_slot_num_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_pdu_tx_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_pdu_tx_request_OF_VERSION_1_3_check_scalars(
    of_bsn_pdu_tx_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_port_counter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_port_counter_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_port_counter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_port_counter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_port_counter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_port_counter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_port_counter_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_port_counter_stats_request_port_no_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_port_counter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_bsn_port_counter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_port_counter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_port_counter_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_3_populate_scalars(
    of_bsn_role_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_role_status_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_role_status_role_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_role_status_reason_set(obj, val8);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_role_status_generation_id_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_role_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_role_status_OF_VERSION_1_3_check_scalars(
    of_bsn_role_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint64_t val64;

    of_bsn_role_status_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_role_status_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_role_status_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_role_status_generation_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_aux_cxns_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_aux_cxns_reply_num_aux_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_aux_cxns_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_set_aux_cxns_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_set_aux_cxns_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_aux_cxns_reply_num_aux_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_aux_cxns_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_aux_cxns_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_aux_cxns_request_num_aux_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_aux_cxns_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_aux_cxns_request_OF_VERSION_1_3_check_scalars(
    of_bsn_set_aux_cxns_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_set_aux_cxns_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_aux_cxns_request_num_aux_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_ip_mask,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_ip_mask_OF_VERSION_1_3_check_scalars(
    of_bsn_set_ip_mask_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_set_l2_table_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_l2_table_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_l2_table_request_OF_VERSION_1_3_check_scalars(
    of_bsn_set_l2_table_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_lacp_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_lacp_reply_status_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_set_lacp_reply_port_no_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_set_lacp_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_bsn_set_lacp_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_lacp_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_lacp_reply_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_lacp_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_lacp_request_enabled_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_set_lacp_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_lacp_request_actor_sys_priority_set(obj, val16);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_set_lacp_request_actor_sys_mac_set(obj, mac_addr);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_lacp_request_actor_port_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_lacp_request_actor_port_num_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_lacp_request_actor_key_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_lacp_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_lacp_request_OF_VERSION_1_3_check_scalars(
    of_bsn_set_lacp_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;

    of_bsn_set_lacp_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_lacp_request_enabled_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_lacp_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_set_lacp_request_actor_sys_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_lacp_request_actor_sys_mac_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_set_lacp_request_actor_port_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_lacp_request_actor_port_num_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_lacp_request_actor_key_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_mirroring_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_mirroring_report_mirror_ports_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_mirroring,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_mirroring_OF_VERSION_1_3_check_scalars(
    of_bsn_set_mirroring_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_bsn_set_mirroring_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_set_pktin_suppression_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_pktin_suppression_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_set_pktin_suppression_request_enabled_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_set_pktin_suppression_request_priority_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_set_pktin_suppression_request_cookie_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_pktin_suppression_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_pktin_suppression_request_OF_VERSION_1_3_check_scalars(
    of_bsn_set_pktin_suppression_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_switch_pipeline_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_switch_pipeline_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_set_switch_pipeline_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_set_switch_pipeline_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_switch_pipeline_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {
    uint32_t val32;
    of_desc_str_t desc_str;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_set_switch_pipeline_request_xid_set(obj, val32);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_bsn_set_switch_pipeline_request_pipeline_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_set_switch_pipeline_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_set_switch_pipeline_request_OF_VERSION_1_3_check_scalars(
    of_bsn_set_switch_pipeline_request_t *obj, int value) {
    uint32_t val32;
    of_desc_str_t desc_str;

    of_bsn_set_switch_pipeline_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_set_switch_pipeline_request_pipeline_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_3_populate_scalars(
    of_bsn_shell_command_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_command,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_command_OF_VERSION_1_3_check_scalars(
    of_bsn_shell_command_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_3_populate_scalars(
    of_bsn_shell_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_output_OF_VERSION_1_3_check_scalars(
    of_bsn_shell_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_3_populate_scalars(
    of_bsn_shell_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_shell_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_shell_status_OF_VERSION_1_3_check_scalars(
    of_bsn_shell_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_switch_pipeline_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_switch_pipeline_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_switch_pipeline_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_switch_pipeline_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_switch_pipeline_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_switch_pipeline_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_switch_pipeline_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_switch_pipeline_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_switch_pipeline_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_table_checksum_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_table_checksum_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_table_checksum_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_table_checksum_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_table_checksum_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_table_checksum_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_table_checksum_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_table_checksum_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_table_checksum_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_table_checksum_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_3_populate_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_table_set_buckets_size_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_table_set_buckets_size_table_id_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_table_set_buckets_size_buckets_size_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_set_buckets_size,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_set_buckets_size_OF_VERSION_1_3_check_scalars(
    of_bsn_table_set_buckets_size_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_table_set_buckets_size_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_table_set_buckets_size_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_table_set_buckets_size_buckets_size_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_time_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_time_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_time_reply_time_ms_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_time_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_bsn_time_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_time_reply_time_ms_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_time_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_time_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_time_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_time_request_OF_VERSION_1_3_check_scalars(
    of_bsn_time_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_time_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_status_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_reply_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_virtual_port_create_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_create_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_create_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_create_request_OF_VERSION_1_3_check_scalars(
    of_bsn_virtual_port_create_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_reply_status_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_virtual_port_remove_reply_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_virtual_port_remove_request_vport_no_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_virtual_port_remove_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_virtual_port_remove_request_OF_VERSION_1_3_check_scalars(
    of_bsn_virtual_port_remove_request_t *obj, int value) {
    uint32_t val32;

    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_vlan_counter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vlan_counter_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_reply_OF_VERSION_1_3_check_scalars(
    of_bsn_vlan_counter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_vlan_counter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_vlan_counter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_3_populate_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_vlan_counter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vlan_counter_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vlan_counter_stats_request_vlan_vid_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_request_OF_VERSION_1_3_check_scalars(
    of_bsn_vlan_counter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_bsn_vlan_counter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_vlan_counter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vlan_counter_stats_request_vlan_vid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_mfr_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_hw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_sw_desc_set(obj, desc_str);
    value += 1;

    VAR_OF_SERIAL_NUM_T_INIT(ser_num, value);
    of_desc_stats_reply_serial_num_set(obj, ser_num);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_desc_stats_reply_dp_desc_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_reply_OF_VERSION_1_3_check_scalars(
    of_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    of_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    TEST_ASSERT(VAR_OF_SERIAL_NUM_T_CHECK(ser_num, value));
    value += 1;

    of_desc_stats_reply_dp_desc_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_3_populate_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_desc_stats_request_OF_VERSION_1_3_check_scalars(
    of_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_3_populate_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_reply_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_reply_OF_VERSION_1_3_check_scalars(
    of_echo_reply_t *obj, int value) {
    uint32_t val32;

    of_echo_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_3_populate_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_echo_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_echo_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_echo_request_OF_VERSION_1_3_check_scalars(
    of_echo_request_t *obj, int value) {
    uint32_t val32;

    of_echo_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_3_populate_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_error_msg_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_error_msg_OF_VERSION_1_3_check_scalars(
    of_error_msg_t *obj, int value) {
    uint32_t val32;

    of_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_3_populate_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_OF_VERSION_1_3_check_scalars(
    of_experimenter_t *obj, int value) {
    uint32_t val32;

    of_experimenter_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_3_populate_scalars(
    of_experimenter_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_error_msg_subtype_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_error_msg_experimenter_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_error_msg_OF_VERSION_1_3_check_scalars(
    of_experimenter_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_error_msg_subtype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_experimenter_error_msg_experimenter_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_3_populate_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_multipart_reply_ofdpa_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_multipart_reply_ofdpa_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_reply_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_3_check_scalars(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_multipart_reply_ofdpa_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_multipart_reply_ofdpa_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_3_populate_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_multipart_request_ofdpa_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_multipart_request_ofdpa_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_multipart_request_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_multipart_request_ofdpa_OF_VERSION_1_3_check_scalars(
    of_experimenter_multipart_request_ofdpa_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_multipart_request_ofdpa_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_multipart_request_ofdpa_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_3_populate_scalars(
    of_experimenter_ofdpa_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_ofdpa_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_ofdpa_OF_VERSION_1_3_check_scalars(
    of_experimenter_ofdpa_t *obj, int value) {
    uint32_t val32;

    of_experimenter_ofdpa_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_reply_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_reply_OF_VERSION_1_3_check_scalars(
    of_experimenter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_experimenter_stats_reply_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_3_populate_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_experimenter_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_experimenter_stats_request_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_experimenter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_experimenter_stats_request_OF_VERSION_1_3_check_scalars(
    of_experimenter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_experimenter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_experimenter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_experimenter_stats_request_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_3_populate_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_features_reply_datapath_id_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_n_buffers_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_features_reply_n_tables_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_features_reply_auxiliary_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_capabilities_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_features_reply_reserved_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_reply_OF_VERSION_1_3_check_scalars(
    of_features_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    of_features_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_datapath_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_features_reply_n_buffers_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_n_tables_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_features_reply_auxiliary_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_features_reply_capabilities_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_features_reply_reserved_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_features_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_3_populate_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_features_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_features_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_features_request_OF_VERSION_1_3_check_scalars(
    of_features_request_t *obj, int value) {
    uint32_t val32;

    of_features_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_3_populate_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_add_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_add_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_add_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_add_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_add_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_add_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_add_OF_VERSION_1_3_check_scalars(
    of_flow_add_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_add_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_add_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_add_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_add_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_add_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_add_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_3_populate_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_delete_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_OF_VERSION_1_3_check_scalars(
    of_flow_delete_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_delete_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_3_populate_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_delete_strict_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_delete_strict_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_delete_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_delete_strict_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_delete_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_delete_strict,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_delete_strict_OF_VERSION_1_3_check_scalars(
    of_flow_delete_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_delete_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_strict_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_delete_strict_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_delete_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_delete_strict_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_delete_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_3_populate_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_mod_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_mod_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_mod_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_mod_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_OF_VERSION_1_3_check_scalars(
    of_flow_mod_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_mod_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_mod_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_mod_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_mod_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_mod_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_flow_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_3_populate_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_modify_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_OF_VERSION_1_3_check_scalars(
    of_flow_modify_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_modify_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_3_populate_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_strict_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_modify_strict_cookie_mask_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_modify_strict_table_id_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_priority_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_modify_strict_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_modify_strict_out_group_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_modify_strict_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_modify_strict,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_modify_strict_OF_VERSION_1_3_check_scalars(
    of_flow_modify_strict_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;

    of_flow_modify_strict_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_strict_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_modify_strict_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_modify_strict_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_modify_strict_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_modify_strict_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_3_populate_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_xid_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_cookie_set(obj, val64);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_priority_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_removed_reason_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_removed_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_removed_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_removed_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_removed_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_removed,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_removed_OF_VERSION_1_3_check_scalars(
    of_flow_removed_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    of_flow_removed_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_removed_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_removed_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_removed_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_removed_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_removed_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_reply_OF_VERSION_1_3_check_scalars(
    of_flow_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_flow_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_3_populate_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_request_table_id_set(obj, val8);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_flow_stats_request_out_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_request_out_group_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_request_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_request_cookie_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_request_OF_VERSION_1_3_check_scalars(
    of_flow_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;

    of_flow_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_request_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_request_out_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_flow_stats_request_out_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_request_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_request_cookie_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_3_populate_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_get_config_reply_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_reply_OF_VERSION_1_3_check_scalars(
    of_get_config_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_get_config_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_get_config_reply_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_3_populate_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_get_config_request_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_get_config_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_get_config_request_OF_VERSION_1_3_check_scalars(
    of_get_config_request_t *obj, int value) {
    uint32_t val32;

    of_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_3_populate_scalars(
    of_group_add_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_add_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_add_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_add_OF_VERSION_1_3_check_scalars(
    of_group_add_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_add_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_add_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_3_populate_scalars(
    of_group_delete_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_delete_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_delete_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_delete_OF_VERSION_1_3_check_scalars(
    of_group_delete_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_delete_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_delete_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_group_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_reply_OF_VERSION_1_3_check_scalars(
    of_group_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_3_populate_scalars(
    of_group_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_request_OF_VERSION_1_3_check_scalars(
    of_group_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_group_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_features_stats_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_types_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_capabilities_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_all_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_select_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_indirect_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_max_groups_ff_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_all_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_select_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_indirect_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_reply_actions_ff_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_reply_OF_VERSION_1_3_check_scalars(
    of_group_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_features_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_group_features_stats_reply_types_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_capabilities_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_all_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_select_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_indirect_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_max_groups_ff_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_all_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_select_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_indirect_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_reply_actions_ff_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_3_populate_scalars(
    of_group_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_features_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_features_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_features_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_features_stats_request_OF_VERSION_1_3_check_scalars(
    of_group_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_features_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_features_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_3_populate_scalars(
    of_group_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_mod_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_OF_VERSION_1_3_check_scalars(
    of_group_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_mod_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_mod_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_group_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_3_populate_scalars(
    of_group_modify_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_group_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_group_modify_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_modify_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_modify,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_modify_OF_VERSION_1_3_check_scalars(
    of_group_modify_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_group_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_modify_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_modify_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_group_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_reply_OF_VERSION_1_3_check_scalars(
    of_group_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_3_populate_scalars(
    of_group_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_group_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_request_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_request_OF_VERSION_1_3_check_scalars(
    of_group_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_group_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_group_stats_request_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_3_populate_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_OF_VERSION_1_3_check_scalars(
    of_hello_t *obj, int value) {
    uint32_t val32;

    of_hello_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_hello_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_hello_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_hello_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_hello_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_hello_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_hello_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_3_populate_scalars(
    of_meter_add_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_add_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_add_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_add_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_add,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_add_OF_VERSION_1_3_check_scalars(
    of_meter_add_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_add_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_add_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_add_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_3_populate_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_ofdpa_color_set_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_band_ofdpa_color_set_meter_command_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_band_ofdpa_color_set_meter_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_ofdpa_color_set_meter_id_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_ofdpa_color_set_rate_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_ofdpa_color_set_burst_size_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_meter_band_ofdpa_color_set_mode_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_meter_band_ofdpa_color_set_color_aware_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_meter_band_ofdpa_color_set_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_ofdpa_color_set,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_ofdpa_color_set_OF_VERSION_1_3_check_scalars(
    of_meter_band_ofdpa_color_set_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    of_meter_band_ofdpa_color_set_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_ofdpa_color_set_meter_command_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_band_ofdpa_color_set_meter_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_band_ofdpa_color_set_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_ofdpa_color_set_rate_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_ofdpa_color_set_burst_size_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_ofdpa_color_set_mode_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_meter_band_ofdpa_color_set_color_aware_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_meter_band_ofdpa_color_set_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_meter_config_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_config_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_config_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_reply_OF_VERSION_1_3_check_scalars(
    of_meter_config_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_config_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_config_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_3_populate_scalars(
    of_meter_config_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_config_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_config_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_config_stats_request_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_stats_request_OF_VERSION_1_3_check_scalars(
    of_meter_config_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_config_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_config_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_config_stats_request_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_3_populate_scalars(
    of_meter_delete_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_delete_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_delete_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_delete_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_delete,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_delete_OF_VERSION_1_3_check_scalars(
    of_meter_delete_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_delete_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_delete_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_delete_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_meter_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_features_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_features_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_reply_OF_VERSION_1_3_check_scalars(
    of_meter_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_features_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_features_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_3_populate_scalars(
    of_meter_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_features_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_features_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_stats_request_OF_VERSION_1_3_check_scalars(
    of_meter_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_features_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_features_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_3_populate_scalars(
    of_meter_mod_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_mod_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_mod_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_OF_VERSION_1_3_check_scalars(
    of_meter_mod_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_mod_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_mod_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_meter_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_3_populate_scalars(
    of_meter_modify_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_modify_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_modify_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_modify_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_modify,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_modify_OF_VERSION_1_3_check_scalars(
    of_meter_modify_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_modify_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_modify_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_modify_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_meter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_reply_OF_VERSION_1_3_check_scalars(
    of_meter_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_3_populate_scalars(
    of_meter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_stats_request_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_request_OF_VERSION_1_3_check_scalars(
    of_meter_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_meter_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_stats_request_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_3_populate_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_reply_OF_VERSION_1_3_check_scalars(
    of_nicira_controller_role_reply_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_3_populate_scalars(
    of_nicira_controller_role_request_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_controller_role_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_controller_role_request_OF_VERSION_1_3_check_scalars(
    of_nicira_controller_role_request_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_3_populate_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_nicira_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_nicira_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_nicira_header_OF_VERSION_1_3_check_scalars(
    of_nicira_header_t *obj, int value) {
    uint32_t val32;

    of_nicira_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_3_populate_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_in_buffer_id_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_packet_in_total_len_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_packet_in_reason_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_packet_in_table_id_set(obj, val8);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_packet_in_cookie_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_in_OF_VERSION_1_3_check_scalars(
    of_packet_in_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    uint64_t val64;

    of_packet_in_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_in_total_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_packet_in_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_packet_in_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_packet_in_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_3_populate_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_out_buffer_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_out_in_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_out_OF_VERSION_1_3_check_scalars(
    of_packet_out_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_packet_out_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_buffer_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_out_in_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_port_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_desc_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_reply_OF_VERSION_1_3_check_scalars(
    of_port_desc_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_desc_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_3_populate_scalars(
    of_port_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_desc_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_stats_request_OF_VERSION_1_3_check_scalars(
    of_port_desc_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_desc_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_3_populate_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_mod_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_mod_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_mask_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_advertise_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_OF_VERSION_1_3_check_scalars(
    of_port_mod_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    of_port_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_mod_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_advertise_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_port_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_reply_OF_VERSION_1_3_check_scalars(
    of_port_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_port_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_3_populate_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_port_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_request_port_no_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_request_OF_VERSION_1_3_check_scalars(
    of_port_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_port_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_port_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_3_populate_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_port_status_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_port_status_reason_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_status_OF_VERSION_1_3_check_scalars(
    of_port_status_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_port_status_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_status_reason_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_3_populate_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_reply_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_reply_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_reply_OF_VERSION_1_3_check_scalars(
    of_queue_get_config_reply_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_reply_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_3_populate_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_get_config_request_xid_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_get_config_request_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_get_config_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_get_config_request_OF_VERSION_1_3_check_scalars(
    of_queue_get_config_request_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_queue_get_config_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_get_config_request_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_op_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_op_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_op_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_op_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_queue_op_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_op_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_reply_OF_VERSION_1_3_check_scalars(
    of_queue_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_queue_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_3_populate_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_stats_request_flags_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_request_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_request_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_request_OF_VERSION_1_3_check_scalars(
    of_queue_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    of_queue_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_queue_stats_request_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_request_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_3_populate_scalars(
    of_role_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_role_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_role_reply_role_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_role_reply_generation_id_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_role_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_reply_OF_VERSION_1_3_check_scalars(
    of_role_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_role_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_reply_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_reply_generation_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_3_populate_scalars(
    of_role_request_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_role_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_role_request_role_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_role_request_generation_id_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_role_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_OF_VERSION_1_3_check_scalars(
    of_role_request_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_role_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_request_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_request_generation_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_role_request_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_role_request_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_role_request_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_role_request_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_role_request_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_role_request_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_role_request_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_set_config,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_3_populate_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_set_config_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_set_config_miss_send_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_set_config,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_set_config_OF_VERSION_1_3_check_scalars(
    of_set_config_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_set_config_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_set_config_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_set_config_miss_send_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_reply_OF_VERSION_1_3_check_scalars(
    of_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_3_populate_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_stats_request_OF_VERSION_1_3_check_scalars(
    of_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_switch_config_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_switch_config_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_switch_config_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_switch_config_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_switch_config_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_switch_config_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_switch_config_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_features_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_features_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_table_features_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_features_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_features_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_table_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_features_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_features_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_reply_OF_VERSION_1_3_check_scalars(
    of_table_features_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_features_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_features_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_3_populate_scalars(
    of_table_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_features_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_features_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_features_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_stats_request_OF_VERSION_1_3_check_scalars(
    of_table_features_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_features_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_features_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_3_populate_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_table_mod_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_config_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_OF_VERSION_1_3_check_scalars(
    of_table_mod_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_table_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_mod_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_mod_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_mod_failed_error_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_mod_failed_error_msg_code_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_mod_failed_error_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
    of_table_mod_failed_error_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_mod_failed_error_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_mod_failed_error_msg_code_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_3_populate_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_reply_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_reply_OF_VERSION_1_3_check_scalars(
    of_table_stats_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_3_populate_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_table_stats_request_flags_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_request_OF_VERSION_1_3_check_scalars(
    of_table_stats_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    of_table_stats_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_class_based_ctr_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_class_based_ctr_mod_msg_block_index_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_class_based_ctr_mod_msg_transmit_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_class_based_ctr_mod_msg_receive_packets_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_class_based_ctr_mod_msg_traffic_class_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    ofdpa_class_based_ctr_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_class_based_ctr_mod_msg_block_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_class_based_ctr_mod_msg_transmit_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_class_based_ctr_mod_msg_receive_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_class_based_ctr_mod_msg_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_class_based_ctr_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_class_based_ctr_multipart_reply_block_index_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_class_based_ctr_multipart_reply_transmit_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_class_based_ctr_multipart_reply_receive_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_class_based_ctr_multipart_reply_reference_count_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_class_based_ctr_multipart_reply_traffic_class_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    ofdpa_class_based_ctr_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_class_based_ctr_multipart_reply_block_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_class_based_ctr_multipart_reply_transmit_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_class_based_ctr_multipart_reply_receive_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_class_based_ctr_multipart_reply_reference_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_class_based_ctr_multipart_reply_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_class_based_ctr_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_class_based_ctr_multipart_request_block_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_class_based_ctr_multipart_request_traffic_class_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_class_based_ctr_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    ofdpa_class_based_ctr_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_class_based_ctr_multipart_request_block_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_class_based_ctr_multipart_request_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_color_based_ctr_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_color_based_ctr_mod_msg_block_index_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_color_based_ctr_mod_msg_transmit_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_color_based_ctr_mod_msg_receive_packets_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_color_based_ctr_mod_msg_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    ofdpa_color_based_ctr_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_color_based_ctr_mod_msg_block_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_color_based_ctr_mod_msg_transmit_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_color_based_ctr_mod_msg_receive_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_color_based_ctr_mod_msg_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_color_based_ctr_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_color_based_ctr_multipart_reply_block_index_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_color_based_ctr_multipart_reply_transmit_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_color_based_ctr_multipart_reply_receive_packets_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_color_based_ctr_multipart_reply_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    ofdpa_color_based_ctr_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_reply_block_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_reply_transmit_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_reply_receive_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_reply_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_color_based_ctr_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_color_based_ctr_multipart_request_block_index_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_color_based_ctr_multipart_request_transmit_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_color_based_ctr_multipart_request_receive_packets_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_color_based_ctr_multipart_request_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_color_based_ctr_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    ofdpa_color_based_ctr_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_request_block_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_request_transmit_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_request_receive_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_color_based_ctr_multipart_request_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_l2_interface_remark_action_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_l2_interface_remark_action_mod_msg_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_mod_msg_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_mod_msg_color_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_mod_msg_dscp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_mod_msg_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_mod_msg_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    ofdpa_l2_interface_remark_action_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_l2_interface_remark_action_mod_msg_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_l2_interface_remark_action_mod_msg_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_mod_msg_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_mod_msg_dscp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_mod_msg_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_mod_msg_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_l2_interface_remark_action_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_l2_interface_remark_action_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_l2_interface_remark_action_multipart_reply_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_reply_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_reply_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_reply_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_reply_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_l2_interface_remark_action_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_reply_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_reply_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_reply_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_reply_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_reply_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_l2_interface_remark_action_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_l2_interface_remark_action_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_l2_interface_remark_action_multipart_request_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_request_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_request_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_request_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_l2_interface_remark_action_multipart_request_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_l2_interface_remark_action_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_l2_interface_remark_action_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_request_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_request_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_request_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_request_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_l2_interface_remark_action_multipart_request_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_set_qos_action_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_mod_msg_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_mod_msg_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_mod_msg_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_mod_msg_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    ofdpa_mpls_set_qos_action_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_set_qos_action_mod_msg_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_mod_msg_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_mod_msg_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_mod_msg_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_set_qos_action_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_mpls_set_qos_action_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_reply_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_reply_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_reply_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_reply_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_mpls_set_qos_action_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_reply_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_reply_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_reply_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_reply_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_set_qos_action_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_mpls_set_qos_action_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_request_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_request_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_request_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_set_qos_action_multipart_request_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_set_qos_action_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_mpls_set_qos_action_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_request_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_request_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_request_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_set_qos_action_multipart_request_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_tunnel_label_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_mpls_tunnel_label_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_tunnel_label_multipart_reply_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_reply_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_reply_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_reply_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_reply_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_mpls_tunnel_label_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_reply_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_reply_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_reply_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_reply_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_reply_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_tunnel_label_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_mpls_tunnel_label_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_tunnel_label_multipart_request_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_request_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_request_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_request_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_multipart_request_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_mpls_tunnel_label_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_request_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_request_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_request_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_request_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_multipart_request_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_color_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_tunnel_label_remark_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    ofdpa_mpls_tunnel_label_remark_action_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_tunnel_label_remark_action_mod_msg_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_tunnel_label_remark_action_mod_msg_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_remark_action_mod_msg_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_remark_action_mod_msg_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_remark_action_mod_msg_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_tunnel_label_remark_action_mod_msg_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_color_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    ofdpa_mpls_vpn_label_remark_action_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_mod_msg_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_mod_msg_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_mod_msg_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_mod_msg_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_mod_msg_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_mod_msg_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_mpls_vpn_label_remark_action_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_reply_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_reply_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_reply_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_reply_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_reply_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_mpls_tc_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_vlan_pcp_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_vlan_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_mpls_vpn_label_remark_action_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_mpls_vpn_label_remark_action_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_request_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_request_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_request_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_request_vlan_pcp_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_mpls_vpn_label_remark_action_multipart_request_vlan_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_data_plane_ctr_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_data_plane_ctr_mod_msg_lmep_id_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_oam_data_plane_ctr_mod_msg_transmit_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_oam_data_plane_ctr_mod_msg_receive_packets_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_data_plane_ctr_mod_msg_traffic_class_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    ofdpa_oam_data_plane_ctr_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_mod_msg_lmep_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_mod_msg_transmit_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_mod_msg_receive_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_mod_msg_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_data_plane_ctr_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_oam_data_plane_ctr_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_data_plane_ctr_multipart_reply_lmep_id_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_data_plane_ctr_multipart_reply_traffic_class_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_oam_data_plane_ctr_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_reply_lmep_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_reply_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_data_plane_ctr_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_oam_data_plane_ctr_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_data_plane_ctr_multipart_request_lmep_id_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_oam_data_plane_ctr_multipart_request_transmit_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_oam_data_plane_ctr_multipart_request_receive_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    ofdpa_oam_data_plane_ctr_multipart_request_reference_count_set(obj, val64);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_data_plane_ctr_multipart_request_traffic_class_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_data_plane_ctr_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;
    uint8_t val8;

    ofdpa_oam_data_plane_ctr_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_request_lmep_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_request_transmit_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_request_receive_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_request_reference_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    ofdpa_oam_data_plane_ctr_multipart_request_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_drop_status_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_drop_status_mod_msg_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_drop_status_mod_msg_entry_type_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_drop_status_mod_msg_drop_status_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    ofdpa_oam_drop_status_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_drop_status_mod_msg_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_drop_status_mod_msg_entry_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_oam_drop_status_mod_msg_drop_status_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_drop_status_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_oam_drop_status_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_drop_status_multipart_reply_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_drop_status_multipart_reply_entry_type_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_drop_status_multipart_reply_drop_status_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_oam_drop_status_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_reply_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_reply_entry_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_reply_drop_status_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_drop_status_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_oam_drop_status_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_oam_drop_status_multipart_request_index_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_drop_status_multipart_request_entry_type_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    ofdpa_oam_drop_status_multipart_request_drop_status_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_oam_drop_status_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    ofdpa_oam_drop_status_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_request_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_request_entry_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    ofdpa_oam_drop_status_multipart_request_drop_status_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_3_populate_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {
    uint32_t val32;
    of_mac_addr_t mac_addr;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_port_experimenter_prop_scheduling_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_port_experimenter_prop_scheduling_port_no_set(obj, val32);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    ofdpa_port_experimenter_prop_scheduling_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_port_experimenter_prop_scheduling_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_port_experimenter_prop_scheduling_mask_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_port_experimenter_prop_scheduling_mode_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_port_experimenter_prop_scheduling_parameter_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_port_experimenter_prop_scheduling,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_3_check_scalars(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value) {
    uint32_t val32;
    of_mac_addr_t mac_addr;
    uint16_t val16;

    ofdpa_port_experimenter_prop_scheduling_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_port_experimenter_prop_scheduling_port_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_port_experimenter_prop_scheduling_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    ofdpa_port_experimenter_prop_scheduling_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_port_experimenter_prop_scheduling_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_port_experimenter_prop_scheduling_mode_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_port_experimenter_prop_scheduling_parameter_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_queue_desc_prop_congestion_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_congestion_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_congestion_multipart_reply_mode_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    ofdpa_queue_desc_prop_congestion_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_queue_desc_prop_congestion_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_queue_desc_prop_congestion_multipart_reply_mode_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_queue_desc_prop_congestion_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_congestion_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_congestion_multipart_request_mode_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_congestion_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    ofdpa_queue_desc_prop_congestion_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_queue_desc_prop_congestion_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_queue_desc_prop_congestion_multipart_request_mode_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_3_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_queue_desc_prop_parent_multipart_reply_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_parent_multipart_reply_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_parent_multipart_reply_scheduler_level_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_reply,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_3_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    ofdpa_queue_desc_prop_parent_multipart_reply_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_queue_desc_prop_parent_multipart_reply_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_queue_desc_prop_parent_multipart_reply_scheduler_level_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_queue_desc_prop_parent_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_parent_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_queue_desc_prop_parent_multipart_request_scheduler_level_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_desc_prop_parent_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    ofdpa_queue_desc_prop_parent_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_queue_desc_prop_parent_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_queue_desc_prop_parent_multipart_request_scheduler_level_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_3_populate_scalars(
    ofdpa_queue_mod_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_queue_mod_xid_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_queue_mod_port_no_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_queue_mod_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_queue_mod,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_queue_mod_OF_VERSION_1_3_check_scalars(
    ofdpa_queue_mod_t *obj, int value) {
    uint32_t val32;

    ofdpa_queue_mod_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_queue_mod_port_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_queue_mod_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_3_populate_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_scheduler_node_mod_msg_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_mod_msg_level_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_scheduler_node_mod_msg_port_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_mod_msg_max_rate_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_mod_msg_min_rate_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_mod_msg_mode_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_mod_msg_weight_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_mod_msg,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_mod_msg_OF_VERSION_1_3_check_scalars(
    ofdpa_scheduler_node_mod_msg_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    ofdpa_scheduler_node_mod_msg_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_scheduler_node_mod_msg_level_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_mod_msg_port_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_scheduler_node_mod_msg_max_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_mod_msg_min_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_mod_msg_mode_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_mod_msg_weight_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_3_populate_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_scheduler_node_multipart_request_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_request_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_scheduler_node_multipart_request_port_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_request_level_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_request_max_rate_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_request_min_rate_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_request_mode_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_request_weight_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_request,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_request_OF_VERSION_1_3_check_scalars(
    ofdpa_scheduler_node_multipart_request_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    ofdpa_scheduler_node_multipart_request_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_scheduler_node_multipart_request_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_request_port_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_scheduler_node_multipart_request_level_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_request_max_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_request_min_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_request_mode_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_request_weight_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_3_populate_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_scheduler_node_multipart_response_xid_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_response_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofdpa_scheduler_node_multipart_response_port_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_response_level_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_response_max_rate_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_response_min_rate_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_response_mode_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    ofdpa_scheduler_node_multipart_response_weight_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofdpa_scheduler_node_multipart_response,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofdpa_scheduler_node_multipart_response_OF_VERSION_1_3_check_scalars(
    ofdpa_scheduler_node_multipart_response_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;

    ofdpa_scheduler_node_multipart_response_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_scheduler_node_multipart_response_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_response_port_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofdpa_scheduler_node_multipart_response_level_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_response_max_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_response_min_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_response_mode_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    ofdpa_scheduler_node_multipart_response_weight_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_3_populate_scalars(
    of_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_OF_VERSION_1_3_check_scalars(
    of_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_3_populate_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_OF_VERSION_1_3_check_scalars(
    of_action_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_3_populate_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_dest_port_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_mirror_vlan_tag_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_bsn_mirror_copy_stage_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_mirror,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_mirror_OF_VERSION_1_3_check_scalars(
    of_action_bsn_mirror_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_action_bsn_mirror_dest_port_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_action_bsn_mirror_copy_stage_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_populate_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_bsn_set_tunnel_dst_dst_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_bsn_set_tunnel_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_check_scalars(
    of_action_bsn_set_tunnel_dst_t *obj, int value) {
    uint32_t val32;

    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_3_populate_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_in_OF_VERSION_1_3_check_scalars(
    of_action_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_3_populate_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_copy_ttl_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_copy_ttl_out_OF_VERSION_1_3_check_scalars(
    of_action_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_mpls_ttl_OF_VERSION_1_3_check_scalars(
    of_action_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_dec_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_dec_nw_ttl_OF_VERSION_1_3_check_scalars(
    of_action_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_3_populate_scalars(
    of_action_enqueue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_enqueue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_enqueue_OF_VERSION_1_3_check_scalars(
    of_action_enqueue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_3_populate_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_experimenter_OF_VERSION_1_3_check_scalars(
    of_action_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_group,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_3_populate_scalars(
    of_action_group_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_group_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_group,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_group_OF_VERSION_1_3_check_scalars(
    of_action_group_t *obj, int value) {
    uint32_t val32;

    of_action_group_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_3_populate_scalars(
    of_action_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_header_OF_VERSION_1_3_check_scalars(
    of_action_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_3_populate_scalars(
    of_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_OF_VERSION_1_3_check_scalars(
    of_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_3_populate_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_OF_VERSION_1_3_check_scalars(
    of_action_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_3_populate_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_mirror,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_mirror_OF_VERSION_1_3_check_scalars(
    of_action_id_bsn_mirror_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_populate_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_bsn_set_tunnel_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_check_scalars(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_3_populate_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_in_OF_VERSION_1_3_check_scalars(
    of_action_id_copy_ttl_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_3_populate_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_copy_ttl_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_copy_ttl_out_OF_VERSION_1_3_check_scalars(
    of_action_id_copy_ttl_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_mpls_ttl_OF_VERSION_1_3_check_scalars(
    of_action_id_dec_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_dec_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_dec_nw_ttl_OF_VERSION_1_3_check_scalars(
    of_action_id_dec_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_3_populate_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_experimenter_OF_VERSION_1_3_check_scalars(
    of_action_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_3_populate_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_group,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_group_OF_VERSION_1_3_check_scalars(
    of_action_id_group_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_3_populate_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_header_OF_VERSION_1_3_check_scalars(
    of_action_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_3_populate_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_OF_VERSION_1_3_check_scalars(
    of_action_id_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_nicira_dec_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_nicira_dec_ttl_OF_VERSION_1_3_check_scalars(
    of_action_id_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_check_drop_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_check_drop_status_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_copy_tc_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_cw_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_push_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_counter_fields,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_3_populate_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_output_OF_VERSION_1_3_check_scalars(
    of_action_id_output_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_3_populate_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_mpls_OF_VERSION_1_3_check_scalars(
    of_action_id_pop_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_3_populate_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_pbb_OF_VERSION_1_3_check_scalars(
    of_action_id_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_3_populate_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_pop_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_pop_vlan_OF_VERSION_1_3_check_scalars(
    of_action_id_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_3_populate_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_mpls_OF_VERSION_1_3_check_scalars(
    of_action_id_push_mpls_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_3_populate_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_pbb_OF_VERSION_1_3_check_scalars(
    of_action_id_push_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_3_populate_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_push_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_push_vlan_OF_VERSION_1_3_check_scalars(
    of_action_id_push_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_3_populate_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_field,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_field_OF_VERSION_1_3_check_scalars(
    of_action_id_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_mpls_ttl_OF_VERSION_1_3_check_scalars(
    of_action_id_set_mpls_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_nw_ttl_OF_VERSION_1_3_check_scalars(
    of_action_id_set_nw_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_3_populate_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_id_set_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_id_set_queue_OF_VERSION_1_3_check_scalars(
    of_action_id_set_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_3_populate_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_OF_VERSION_1_3_check_scalars(
    of_action_nicira_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_nicira_dec_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_nicira_dec_ttl_OF_VERSION_1_3_check_scalars(
    of_action_nicira_dec_ttl_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_check_drop_status_drop_type_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_check_drop_status_drop_index_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_check_drop_status,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_check_drop_status_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_check_drop_status_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_check_drop_status_drop_type_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_check_drop_status_drop_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_class_based_rx_count_traffic_class_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_class_based_rx_count_index_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_class_based_rx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_class_based_rx_count_traffic_class_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_class_based_rx_count_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_class_based_tx_count_traffic_class_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_class_based_tx_count_index_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_class_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_class_based_tx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_class_based_tx_count_traffic_class_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_class_based_tx_count_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_color_based_rx_count_color_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_color_based_rx_count_index_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_color_based_rx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_color_based_rx_count_color_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_color_based_rx_count_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_color_based_tx_count_color_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_color_based_tx_count_index_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_color_based_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_color_based_tx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_color_based_tx_count_color_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_color_based_tx_count_index_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_in,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_copy_tc_in_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_copy_tc_out,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_copy_tc_out_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_dec_ttl_mtu,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_oam_lm_rx_count_traffic_class_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_oam_lm_rx_count_lmep_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_rx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_oam_lm_rx_count_traffic_class_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_oam_lm_rx_count_lmep_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_oam_lm_tx_count_traffic_class_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_oam_lm_tx_count_lmep_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_oam_lm_tx_count,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_oam_lm_tx_count_traffic_class_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_oam_lm_tx_count_lmep_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_cw_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_pop_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_pop_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_pop_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_cw,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_cw_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_push_cw_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_push_l2hdr,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_push_l2hdr_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_action_ofdpa_set_counter_fields_traffic_class_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_action_ofdpa_set_counter_fields_lmep_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_counter_fields,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_counter_fields_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_action_ofdpa_set_counter_fields_traffic_class_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_action_ofdpa_set_counter_fields_lmep_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_dscp_from_table_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_dscp_from_table_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_dscp_from_table_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_dscp_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value) {
    uint8_t val8;

    of_action_ofdpa_set_dscp_from_table_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_dscp_from_table_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_dscp_from_table_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_table_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_table_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_table_color_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_table_pcp_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value) {
    uint8_t val8;

    of_action_ofdpa_set_mpls_pcpdei_from_table_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_pcpdei_from_table_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_pcpdei_from_table_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_pcpdei_from_table_pcp_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_color_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_pcp_dei_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value) {
    uint8_t val8;

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_pcp_dei_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_qos_from_table_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_qos_from_table_mpls_tc_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_qos_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value) {
    uint8_t val8;

    of_action_ofdpa_set_mpls_qos_from_table_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_qos_from_table_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_color_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_mpls_tc_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_tunnel_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value) {
    uint8_t val8;

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_color_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_mpls_tc_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_mpls_tc_from_vpn_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value) {
    uint8_t val8;

    of_action_ofdpa_set_mpls_tc_from_vpn_table_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_tc_from_vpn_table_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_tc_from_vpn_table_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_mpls_tc_from_vpn_table_mpls_tc_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_pcpdei_from_table_qos_index_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_pcpdei_from_table_traffic_class_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_action_ofdpa_set_pcpdei_from_table_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_ofdpa_set_pcpdei_from_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value) {
    uint8_t val8;

    of_action_ofdpa_set_pcpdei_from_table_qos_index_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_pcpdei_from_table_traffic_class_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_action_ofdpa_set_pcpdei_from_table_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_3_populate_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_action_output_port_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_action_output_max_len_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_output_OF_VERSION_1_3_check_scalars(
    of_action_output_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;

    of_action_output_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_action_output_max_len_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_3_populate_scalars(
    of_action_pop_mpls_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_pop_mpls_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_mpls_OF_VERSION_1_3_check_scalars(
    of_action_pop_mpls_t *obj, int value) {
    uint16_t val16;

    of_action_pop_mpls_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_3_populate_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_pbb_OF_VERSION_1_3_check_scalars(
    of_action_pop_pbb_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_3_populate_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_pop_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_pop_vlan_OF_VERSION_1_3_check_scalars(
    of_action_pop_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_3_populate_scalars(
    of_action_push_mpls_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_push_mpls_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_mpls,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_mpls_OF_VERSION_1_3_check_scalars(
    of_action_push_mpls_t *obj, int value) {
    uint16_t val16;

    of_action_push_mpls_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_3_populate_scalars(
    of_action_push_pbb_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_push_pbb_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_pbb,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_pbb_OF_VERSION_1_3_check_scalars(
    of_action_push_pbb_t *obj, int value) {
    uint16_t val16;

    of_action_push_pbb_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_3_populate_scalars(
    of_action_push_vlan_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_action_push_vlan_ethertype_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_push_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_push_vlan_OF_VERSION_1_3_check_scalars(
    of_action_push_vlan_t *obj, int value) {
    uint16_t val16;

    of_action_push_vlan_ethertype_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_3_populate_scalars(
    of_action_set_dl_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_dst_OF_VERSION_1_3_check_scalars(
    of_action_set_dl_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_3_populate_scalars(
    of_action_set_dl_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_dl_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_dl_src_OF_VERSION_1_3_check_scalars(
    of_action_set_dl_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_3_populate_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_field,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_field_OF_VERSION_1_3_check_scalars(
    of_action_set_field_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_3_populate_scalars(
    of_action_set_mpls_label_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_label,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_label_OF_VERSION_1_3_check_scalars(
    of_action_set_mpls_label_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_3_populate_scalars(
    of_action_set_mpls_tc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_tc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_tc_OF_VERSION_1_3_check_scalars(
    of_action_set_mpls_tc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_mpls_ttl_mpls_ttl_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_mpls_ttl_OF_VERSION_1_3_check_scalars(
    of_action_set_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    of_action_set_mpls_ttl_mpls_ttl_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_3_populate_scalars(
    of_action_set_nw_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_dst_OF_VERSION_1_3_check_scalars(
    of_action_set_nw_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_3_populate_scalars(
    of_action_set_nw_ecn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ecn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ecn_OF_VERSION_1_3_check_scalars(
    of_action_set_nw_ecn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_3_populate_scalars(
    of_action_set_nw_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_src_OF_VERSION_1_3_check_scalars(
    of_action_set_nw_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_3_populate_scalars(
    of_action_set_nw_tos_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_tos,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_tos_OF_VERSION_1_3_check_scalars(
    of_action_set_nw_tos_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_3_populate_scalars(
    of_action_set_nw_ttl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_action_set_nw_ttl_nw_ttl_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_nw_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_nw_ttl_OF_VERSION_1_3_check_scalars(
    of_action_set_nw_ttl_t *obj, int value) {
    uint8_t val8;

    of_action_set_nw_ttl_nw_ttl_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_3_populate_scalars(
    of_action_set_queue_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_action_set_queue_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_queue_OF_VERSION_1_3_check_scalars(
    of_action_set_queue_t *obj, int value) {
    uint32_t val32;

    of_action_set_queue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_3_populate_scalars(
    of_action_set_tp_dst_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_dst_OF_VERSION_1_3_check_scalars(
    of_action_set_tp_dst_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_3_populate_scalars(
    of_action_set_tp_src_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_tp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_tp_src_OF_VERSION_1_3_check_scalars(
    of_action_set_tp_src_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_3_populate_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_pcp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_pcp_OF_VERSION_1_3_check_scalars(
    of_action_set_vlan_pcp_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_3_populate_scalars(
    of_action_set_vlan_vid_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_set_vlan_vid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_set_vlan_vid_OF_VERSION_1_3_check_scalars(
    of_action_set_vlan_vid_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_3_populate_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_action_strip_vlan,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_action_strip_vlan_OF_VERSION_1_3_check_scalars(
    of_action_strip_vlan_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_3_populate_scalars(
    of_bsn_controller_connection_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    of_desc_str_t desc_str;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_controller_connection_state_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_controller_connection_auxiliary_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_controller_connection_role_set(obj, val32);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_bsn_controller_connection_uri_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_controller_connection,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_controller_connection_OF_VERSION_1_3_check_scalars(
    of_bsn_controller_connection_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    of_desc_str_t desc_str;

    of_bsn_controller_connection_state_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_controller_connection_auxiliary_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_controller_connection_role_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_controller_connection_uri_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {
    uint64_t val64;
    of_str64_t str64;
    of_desc_str_t desc_str;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_debug_counter_desc_stats_entry_counter_id_set(obj, val64);
    value += 1;

    VAR_OF_STR64_T_INIT(str64, value);
    of_bsn_debug_counter_desc_stats_entry_name_set(obj, str64);
    value += 1;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_bsn_debug_counter_desc_stats_entry_description_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value) {
    uint64_t val64;
    of_str64_t str64;
    of_desc_str_t desc_str;

    of_bsn_debug_counter_desc_stats_entry_counter_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_bsn_debug_counter_desc_stats_entry_name_get(obj, &str64);
    TEST_ASSERT(VAR_OF_STR64_T_CHECK(str64, value));
    value += 1;

    of_bsn_debug_counter_desc_stats_entry_description_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_debug_counter_stats_entry_counter_id_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_debug_counter_stats_entry_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_debug_counter_stats_entry_t *obj, int value) {
    uint64_t val64;

    of_bsn_debug_counter_stats_entry_counter_id_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_bsn_debug_counter_stats_entry_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_flow_checksum_bucket_stats_entry_checksum_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {
    uint64_t val64;

    of_bsn_flow_checksum_bucket_stats_entry_checksum_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {
    of_checksum_128_t checksum_128;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_bucket_stats_entry_checksum_set(obj, checksum_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value) {
    of_checksum_128_t checksum_128;

    of_bsn_gentable_bucket_stats_entry_checksum_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {
    uint16_t val16;
    of_table_name_t table_name;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_desc_stats_entry_table_id_set(obj, val16);
    value += 1;

    VAR_OF_TABLE_NAME_T_INIT(table_name, value);
    of_bsn_gentable_desc_stats_entry_name_set(obj, table_name);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_desc_stats_entry_buckets_size_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_desc_stats_entry_max_entries_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_desc_stats_entry_t *obj, int value) {
    uint16_t val16;
    of_table_name_t table_name;
    uint32_t val32;

    of_bsn_gentable_desc_stats_entry_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_desc_stats_entry_name_get(obj, &table_name);
    TEST_ASSERT(VAR_OF_TABLE_NAME_T_CHECK(table_name, value));
    value += 1;

    of_bsn_gentable_desc_stats_entry_buckets_size_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_desc_stats_entry_max_entries_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {
    of_checksum_128_t checksum_128;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_entry_desc_stats_entry_checksum_set(obj, checksum_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {
    of_checksum_128_t checksum_128;

    of_bsn_gentable_entry_desc_stats_entry_checksum_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;
    of_checksum_128_t checksum_128;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_gentable_stats_entry_table_id_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_gentable_stats_entry_entry_count_set(obj, val32);
    value += 1;

    VAR_OF_CHECKSUM_128_T_INIT(checksum_128, value);
    of_bsn_gentable_stats_entry_checksum_set(obj, checksum_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_gentable_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_gentable_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_gentable_stats_entry_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;
    of_checksum_128_t checksum_128;

    of_bsn_gentable_stats_entry_table_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_gentable_stats_entry_entry_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_gentable_stats_entry_checksum_get(obj, &checksum_128);
    TEST_ASSERT(VAR_OF_CHECKSUM_128_T_CHECK(checksum_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_3_populate_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_interface_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_interface_name_set(obj, port_name);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_addr_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_interface_ipv4_netmask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_interface,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_interface_OF_VERSION_1_3_check_scalars(
    of_bsn_interface_t *obj, int value) {
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_interface_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;
    uint8_t val8;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_lacp_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_actor_sys_priority_set(obj, val16);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_lacp_stats_entry_actor_sys_mac_set(obj, mac_addr);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_actor_port_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_actor_port_num_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_actor_key_set(obj, val16);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_lacp_stats_entry_convergence_status_set(obj, val8);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_partner_sys_priority_set(obj, val16);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_lacp_stats_entry_partner_sys_mac_set(obj, mac_addr);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_partner_port_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_partner_port_num_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_lacp_stats_entry_partner_key_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_lacp_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_lacp_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_lacp_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;
    uint8_t val8;

    of_bsn_lacp_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bsn_lacp_stats_entry_actor_sys_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_stats_entry_actor_sys_mac_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_lacp_stats_entry_actor_port_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_stats_entry_actor_port_num_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_stats_entry_actor_key_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_stats_entry_convergence_status_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_lacp_stats_entry_partner_sys_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_stats_entry_partner_sys_mac_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_bsn_lacp_stats_entry_partner_port_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_stats_entry_partner_port_num_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_lacp_stats_entry_partner_key_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_port_counter_stats_entry_port_no_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_port_counter_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_port_counter_stats_entry_t *obj, int value) {
    of_port_no_t port_no;

    of_bsn_port_counter_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {
    of_desc_str_t desc_str;

    VAR_OF_DESC_STR_T_INIT(desc_str, value);
    of_bsn_switch_pipeline_stats_entry_pipeline_set(obj, desc_str);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value) {
    of_desc_str_t desc_str;

    of_bsn_switch_pipeline_stats_entry_pipeline_get(obj, &desc_str);
    TEST_ASSERT(VAR_OF_DESC_STR_T_CHECK(desc_str, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_bsn_table_checksum_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_table_checksum_stats_entry_checksum_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_table_checksum_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint64_t val64;

    of_bsn_table_checksum_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_bsn_table_checksum_stats_entry_checksum_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_tlv_broadcast_query_timeout_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_broadcast_query_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value) {
    uint32_t val32;

    of_bsn_tlv_broadcast_query_timeout_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_circuit_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_circuit_id_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_circuit_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_header_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_notification,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_notification_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_idle_notification_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_tlv_idle_time_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_time,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_time_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_idle_time_t *obj, int value) {
    uint64_t val64;

    of_bsn_tlv_idle_time_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_tlv_idle_timeout_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_idle_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_idle_timeout_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_idle_timeout_t *obj, int value) {
    uint32_t val32;

    of_bsn_tlv_idle_timeout_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_bsn_tlv_ipv4_value_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_ipv4,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_ipv4_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_ipv4_t *obj, int value) {
    of_ipv4_t ipv4;

    of_bsn_tlv_ipv4_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_mac_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_bsn_tlv_mac_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_mac,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_mac_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_mac_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_bsn_tlv_mac_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_tlv_miss_packets_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_miss_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_miss_packets_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_miss_packets_t *obj, int value) {
    uint64_t val64;

    of_bsn_tlv_miss_packets_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_port_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bsn_tlv_port_value_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_port_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_port_t *obj, int value) {
    of_port_no_t port_no;

    of_bsn_tlv_port_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_tlv_reply_packets_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_reply_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_reply_packets_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_reply_packets_t *obj, int value) {
    uint64_t val64;

    of_bsn_tlv_reply_packets_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_tlv_request_packets_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_request_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_request_packets_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_request_packets_t *obj, int value) {
    uint64_t val64;

    of_bsn_tlv_request_packets_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_tlv_rx_packets_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_rx_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_rx_packets_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_rx_packets_t *obj, int value) {
    uint64_t val64;

    of_bsn_tlv_rx_packets_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bsn_tlv_tx_packets_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_tx_packets,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_tx_packets_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_tx_packets_t *obj, int value) {
    uint64_t val64;

    of_bsn_tlv_tx_packets_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_tlv_udf_anchor_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_anchor,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_anchor_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_udf_anchor_t *obj, int value) {
    uint16_t val16;

    of_bsn_tlv_udf_anchor_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_tlv_udf_id_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_id_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_udf_id_t *obj, int value) {
    uint16_t val16;

    of_bsn_tlv_udf_id_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_tlv_udf_length_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_length,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_length_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_udf_length_t *obj, int value) {
    uint16_t val16;

    of_bsn_tlv_udf_length_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_tlv_udf_offset_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_udf_offset,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_udf_offset_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_udf_offset_t *obj, int value) {
    uint16_t val16;

    of_bsn_tlv_udf_offset_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_tlv_unicast_query_timeout_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_unicast_query_timeout,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value) {
    uint32_t val32;

    of_bsn_tlv_unicast_query_timeout_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_3_populate_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_tlv_vlan_vid_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_tlv_vlan_vid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_tlv_vlan_vid_OF_VERSION_1_3_check_scalars(
    of_bsn_tlv_vlan_vid_t *obj, int value) {
    uint16_t val16;

    of_bsn_tlv_vlan_vid_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vlan_counter_stats_entry_vlan_vid_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check_scalars(
    of_bsn_vlan_counter_stats_entry_t *obj, int value) {
    uint16_t val16;

    of_bsn_vlan_counter_stats_entry_vlan_vid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_3_populate_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_OF_VERSION_1_3_check_scalars(
    of_bsn_vport_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_3_populate_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_header_OF_VERSION_1_3_check_scalars(
    of_bsn_vport_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_3_populate_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    VAR_UINT32_T_INIT(val32, value);
    of_bsn_vport_q_in_q_port_no_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_ingress_vlan_id_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_tpid_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_bsn_vport_q_in_q_egress_vlan_id_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_bsn_vport_q_in_q_if_name_set(obj, port_name);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bsn_vport_q_in_q,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bsn_vport_q_in_q_OF_VERSION_1_3_check_scalars(
    of_bsn_vport_q_in_q_t *obj, int value) {
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_3_populate_scalars(
    of_bucket_t *obj, int value) {
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_bucket_weight_set(obj, val16);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_bucket_watch_port_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_bucket_watch_group_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bucket,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_OF_VERSION_1_3_check_scalars(
    of_bucket_t *obj, int value) {
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;

    of_bucket_weight_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_bucket_watch_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_bucket_watch_group_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_3_populate_scalars(
    of_bucket_counter_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_bucket_counter_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_bucket_counter_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_bucket_counter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_bucket_counter_OF_VERSION_1_3_check_scalars(
    of_bucket_counter_t *obj, int value) {
    uint64_t val64;

    of_bucket_counter_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_bucket_counter_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_flow_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_flow_stats_entry_duration_nsec_set(obj, val32);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_priority_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_idle_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_hard_timeout_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_flow_stats_entry_flags_set(obj, val16);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_cookie_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_flow_stats_entry_byte_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_flow_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_flow_stats_entry_OF_VERSION_1_3_check_scalars(
    of_flow_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    of_flow_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_flow_stats_entry_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_flow_stats_entry_priority_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_flow_stats_entry_cookie_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_flow_stats_entry_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_group_desc_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;

    VAR_UINT8_T_INIT(val8, value);
    of_group_desc_stats_entry_group_type_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_desc_stats_entry_group_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_group_desc_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;

    of_group_desc_stats_entry_group_type_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_group_desc_stats_entry_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_group_stats_entry_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_group_id_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_ref_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_group_stats_entry_packet_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_group_stats_entry_byte_count_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_group_stats_entry_duration_nsec_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_group_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_group_stats_entry_OF_VERSION_1_3_check_scalars(
    of_group_stats_entry_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_group_stats_entry_group_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_entry_ref_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_entry_packet_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_group_stats_entry_byte_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_group_stats_entry_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_group_stats_entry_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_3_populate_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_header_xid_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_header_OF_VERSION_1_3_check_scalars(
    of_header_t *obj, int value) {
    uint32_t val32;

    of_header_xid_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_3_populate_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_OF_VERSION_1_3_check_scalars(
    of_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_3_populate_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_header_OF_VERSION_1_3_check_scalars(
    of_hello_elem_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_3_populate_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_hello_elem_versionbitmap,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_hello_elem_versionbitmap_OF_VERSION_1_3_check_scalars(
    of_hello_elem_versionbitmap_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_3_populate_scalars(
    of_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_OF_VERSION_1_3_check_scalars(
    of_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_3_populate_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_apply_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_apply_actions_OF_VERSION_1_3_check_scalars(
    of_instruction_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_arp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_arp_offload_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_deny,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_deny_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_dhcp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_disable_src_mac_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_packet_of_death,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_packet_of_death_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_3_populate_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_bsn_permit,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_bsn_permit_OF_VERSION_1_3_check_scalars(
    of_instruction_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_3_populate_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_clear_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_clear_actions_OF_VERSION_1_3_check_scalars(
    of_instruction_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_3_populate_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_experimenter_OF_VERSION_1_3_check_scalars(
    of_instruction_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_3_populate_scalars(
    of_instruction_goto_table_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_instruction_goto_table_table_id_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_goto_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_goto_table_OF_VERSION_1_3_check_scalars(
    of_instruction_goto_table_t *obj, int value) {
    uint8_t val8;

    of_instruction_goto_table_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_3_populate_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_header_OF_VERSION_1_3_check_scalars(
    of_instruction_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_OF_VERSION_1_3_check_scalars(
    of_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_apply_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_apply_actions_OF_VERSION_1_3_check_scalars(
    of_instruction_id_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_arp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_arp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_deny,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_deny_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_deny_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_dhcp_offload,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_split_horizon_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_disable_src_mac_check,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_packet_of_death,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_packet_of_death_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_bsn_permit,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_bsn_permit_OF_VERSION_1_3_check_scalars(
    of_instruction_id_bsn_permit_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_clear_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_clear_actions_OF_VERSION_1_3_check_scalars(
    of_instruction_id_clear_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_experimenter_OF_VERSION_1_3_check_scalars(
    of_instruction_id_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_goto_table,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_goto_table_OF_VERSION_1_3_check_scalars(
    of_instruction_id_goto_table_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_header_OF_VERSION_1_3_check_scalars(
    of_instruction_id_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_meter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_meter_OF_VERSION_1_3_check_scalars(
    of_instruction_id_meter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_actions_OF_VERSION_1_3_check_scalars(
    of_instruction_id_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_3_populate_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_id_write_metadata,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_id_write_metadata_OF_VERSION_1_3_check_scalars(
    of_instruction_id_write_metadata_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_3_populate_scalars(
    of_instruction_meter_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_instruction_meter_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_meter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_meter_OF_VERSION_1_3_check_scalars(
    of_instruction_meter_t *obj, int value) {
    uint32_t val32;

    of_instruction_meter_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_3_populate_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_actions_OF_VERSION_1_3_check_scalars(
    of_instruction_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_3_populate_scalars(
    of_instruction_write_metadata_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_instruction_write_metadata_metadata_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_instruction_write_metadata_metadata_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_instruction_write_metadata,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_instruction_write_metadata_OF_VERSION_1_3_check_scalars(
    of_instruction_write_metadata_t *obj, int value) {
    uint64_t val64;

    of_instruction_write_metadata_metadata_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_instruction_write_metadata_metadata_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_3_populate_scalars(
    of_match_v1_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v1,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v1_OF_VERSION_1_3_check_scalars(
    of_match_v1_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_3_populate_scalars(
    of_match_v2_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v2,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v2_OF_VERSION_1_3_check_scalars(
    of_match_v2_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_3_populate_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_match_v3,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_match_v3_OF_VERSION_1_3_check_scalars(
    of_match_v3_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_3_populate_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_OF_VERSION_1_3_check_scalars(
    of_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_3_populate_scalars(
    of_meter_band_drop_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_drop_rate_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_drop_burst_size_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_drop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_drop_OF_VERSION_1_3_check_scalars(
    of_meter_band_drop_t *obj, int value) {
    uint32_t val32;

    of_meter_band_drop_rate_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_drop_burst_size_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_3_populate_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_dscp_remark_rate_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_dscp_remark_burst_size_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_meter_band_dscp_remark_prec_level_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_dscp_remark,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_dscp_remark_OF_VERSION_1_3_check_scalars(
    of_meter_band_dscp_remark_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_meter_band_dscp_remark_rate_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_dscp_remark_burst_size_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_dscp_remark_prec_level_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_3_populate_scalars(
    of_meter_band_experimenter_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_experimenter_rate_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_band_experimenter_burst_size_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_experimenter_OF_VERSION_1_3_check_scalars(
    of_meter_band_experimenter_t *obj, int value) {
    uint32_t val32;

    of_meter_band_experimenter_rate_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_band_experimenter_burst_size_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_3_populate_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_header_OF_VERSION_1_3_check_scalars(
    of_meter_band_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_3_populate_scalars(
    of_meter_band_stats_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_meter_band_stats_packet_band_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_meter_band_stats_byte_band_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_band_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_band_stats_OF_VERSION_1_3_check_scalars(
    of_meter_band_stats_t *obj, int value) {
    uint64_t val64;

    of_meter_band_stats_packet_band_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_meter_band_stats_byte_band_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_3_populate_scalars(
    of_meter_config_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    VAR_UINT16_T_INIT(val16, value);
    of_meter_config_flags_set(obj, val16);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_config_meter_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_config,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_config_OF_VERSION_1_3_check_scalars(
    of_meter_config_t *obj, int value) {
    uint16_t val16;
    uint32_t val32;

    of_meter_config_flags_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_meter_config_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_3_populate_scalars(
    of_meter_features_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_features_max_meter_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_features_band_types_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_features_capabilities_set(obj, val32);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_meter_features_max_bands_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_meter_features_max_color_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_features,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_features_OF_VERSION_1_3_check_scalars(
    of_meter_features_t *obj, int value) {
    uint32_t val32;
    uint8_t val8;

    of_meter_features_max_meter_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_features_band_types_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_features_capabilities_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_features_max_bands_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_meter_features_max_color_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_3_populate_scalars(
    of_meter_stats_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_stats_meter_id_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_stats_flow_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_meter_stats_packet_in_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_meter_stats_byte_in_count_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_stats_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_meter_stats_duration_nsec_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_meter_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_meter_stats_OF_VERSION_1_3_check_scalars(
    of_meter_stats_t *obj, int value) {
    uint32_t val32;
    uint64_t val64;

    of_meter_stats_meter_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_stats_flow_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_stats_packet_in_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_meter_stats_byte_in_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_meter_stats_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_meter_stats_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_3_populate_scalars(
    of_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_OF_VERSION_1_3_check_scalars(
    of_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_op_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_arp_op_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_op_t *obj, int value) {
    uint16_t val16;

    of_oxm_arp_op_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_arp_op_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_arp_op_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_op_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_op_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_op_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_arp_op_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_arp_op_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_sha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_sha_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_sha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_sha_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_sha_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_sha_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_sha_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_sha_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_sha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_sha_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_arp_sha_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_spa_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_spa_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_spa_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_spa_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_spa_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_spa_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_spa_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_spa_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_spa_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_spa_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_arp_spa_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_tha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_tha_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_tha_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_tha_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_tha_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_arp_tha_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tha_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tha_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_tha_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_arp_tha_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_arp_tha_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_tpa_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_tpa_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_tpa_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_tpa_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_tpa_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_arp_tpa_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_arp_tpa_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_arp_tpa_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_arp_tpa_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_arp_tpa_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_arp_tpa_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_egr_port_group_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_egr_port_group_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_egr_port_group_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_egr_port_group_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_egr_port_group_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_egr_port_group_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_egr_port_group_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_egr_port_group_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_bsn_global_vrf_allowed_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value) {
    uint8_t val8;

    of_oxm_bsn_global_vrf_allowed_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_bsn_global_vrf_allowed_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_bsn_global_vrf_allowed_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_global_vrf_allowed_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_bsn_global_vrf_allowed_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_bsn_global_vrf_allowed_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    VAR_OF_BITMAP_128_T_INIT(bitmap_128, value);
    of_oxm_bsn_in_ports_128_value_set(obj, bitmap_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_in_ports_128_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    of_oxm_bsn_in_ports_128_value_get(obj, &bitmap_128);
    TEST_ASSERT(VAR_OF_BITMAP_128_T_CHECK(bitmap_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    VAR_OF_BITMAP_128_T_INIT(bitmap_128, value);
    of_oxm_bsn_in_ports_128_masked_value_set(obj, bitmap_128);
    value += 1;

    VAR_OF_BITMAP_128_T_INIT(bitmap_128, value);
    of_oxm_bsn_in_ports_128_masked_value_mask_set(obj, bitmap_128);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_in_ports_128_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value) {
    of_bitmap_128_t bitmap_128;

    of_oxm_bsn_in_ports_128_masked_value_get(obj, &bitmap_128);
    TEST_ASSERT(VAR_OF_BITMAP_128_T_CHECK(bitmap_128, value));
    value += 1;

    of_oxm_bsn_in_ports_128_masked_value_mask_get(obj, &bitmap_128);
    TEST_ASSERT(VAR_OF_BITMAP_128_T_CHECK(bitmap_128, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_dst_class_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_dst_class_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_dst_class_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_dst_class_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_dst_class_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_dst_class_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_l3_dst_class_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_interface_class_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_interface_class_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_interface_class_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_interface_class_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_interface_class_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_interface_class_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_l3_interface_class_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_src_class_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_l3_src_class_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_src_class_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_src_class_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_l3_src_class_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_l3_src_class_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_l3_src_class_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_l3_src_class_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_lag_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_lag_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_lag_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_lag_id_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_lag_id_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_lag_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_lag_id_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_lag_id_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_lag_id_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf0_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf0_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf0_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf0_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf0_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf0_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf0_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf0_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf0_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf0_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf1_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf1_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf1_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf1_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf1_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf1_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf1_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf1_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf1_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf1_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf2_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf2_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf2_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf2_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf2_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf2_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf2_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf2_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf2_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf2_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf3_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf3_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf3_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf3_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf3_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf3_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf3_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf3_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf3_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf3_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf4_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf4_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf4_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf4_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf4_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf4_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf4_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf4_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf4_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf4_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf5_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf5_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf5_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf5_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf5_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf5_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf5_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf5_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf5_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf5_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf6_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf6_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf6_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf6_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf6_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf6_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf6_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf6_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf6_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf6_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf7_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf7_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf7_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf7_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_udf7_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_udf7_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_udf7_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_udf7_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_udf7_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_udf7_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_vrf_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_vrf_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_vrf_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_vrf_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_bsn_vrf_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_bsn_vrf_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_bsn_vrf_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_bsn_vrf_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_bsn_vrf_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_bsn_vrf_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_3_populate_scalars(
    of_oxm_eth_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_dst_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_OF_VERSION_1_3_check_scalars(
    of_oxm_eth_dst_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_dst_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_dst_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_dst_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_dst_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_eth_dst_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_dst_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_eth_dst_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_3_populate_scalars(
    of_oxm_eth_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_src_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_OF_VERSION_1_3_check_scalars(
    of_oxm_eth_src_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_src_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_src_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_eth_src_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_src_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_eth_src_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_eth_src_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_eth_src_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_3_populate_scalars(
    of_oxm_eth_type_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_eth_type_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_OF_VERSION_1_3_check_scalars(
    of_oxm_eth_type_t *obj, int value) {
    uint16_t val16;

    of_oxm_eth_type_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_eth_type_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_eth_type_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_eth_type_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_eth_type_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_eth_type_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_eth_type_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_eth_type_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_3_populate_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_experimenter_OF_VERSION_1_3_check_scalars(
    of_oxm_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_3_populate_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_header_OF_VERSION_1_3_check_scalars(
    of_oxm_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_code_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv4_code_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_code_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_code_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_code_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_code_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_code_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv4_code_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_code_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv4_code_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_type_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv4_type_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_type_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_type_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv4_type_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv4_type_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv4_type_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv4_type_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv4_type_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv4_type_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_code_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv6_code_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_code_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_code_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_code_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_code_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_code_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv6_code_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_code_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv6_code_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_type_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv6_type_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_type_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_type_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_icmpv6_type_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_icmpv6_type_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_icmpv6_type_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_icmpv6_type_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_icmpv6_type_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_icmpv6_type_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_3_populate_scalars(
    of_oxm_in_phy_port_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_phy_port_value_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_OF_VERSION_1_3_check_scalars(
    of_oxm_in_phy_port_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_phy_port_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_phy_port_masked_value_set(obj, port_no);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_phy_port_masked_value_mask_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_phy_port_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_phy_port_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_in_phy_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_phy_port_masked_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_oxm_in_phy_port_masked_value_mask_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_3_populate_scalars(
    of_oxm_in_port_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_port_value_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_OF_VERSION_1_3_check_scalars(
    of_oxm_in_port_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_port_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_in_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_port_masked_value_set(obj, port_no);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_oxm_in_port_masked_value_mask_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_in_port_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_in_port_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_in_port_masked_t *obj, int value) {
    of_port_no_t port_no;

    of_oxm_in_port_masked_value_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_oxm_in_port_masked_value_mask_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_3_populate_scalars(
    of_oxm_ip_dscp_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_dscp_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_OF_VERSION_1_3_check_scalars(
    of_oxm_ip_dscp_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_dscp_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_dscp_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_dscp_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_dscp_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_dscp_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ip_dscp_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_dscp_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_ip_dscp_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_3_populate_scalars(
    of_oxm_ip_ecn_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_ecn_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_OF_VERSION_1_3_check_scalars(
    of_oxm_ip_ecn_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_ecn_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_ecn_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_ecn_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_ecn_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_ecn_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ip_ecn_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_ecn_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_ip_ecn_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_3_populate_scalars(
    of_oxm_ip_proto_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_proto_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_OF_VERSION_1_3_check_scalars(
    of_oxm_ip_proto_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_proto_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_proto_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ip_proto_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ip_proto_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ip_proto_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ip_proto_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_ip_proto_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_ip_proto_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_dst_value_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv4_dst_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_dst_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_dst_masked_value_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_dst_masked_value_mask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_dst_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv4_dst_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_dst_masked_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_oxm_ipv4_dst_masked_value_mask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv4_src_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_src_value_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv4_src_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_src_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_src_masked_value_set(obj, ipv4);
    value += 1;

    VAR_OF_IPV4_T_INIT(ipv4, value);
    of_oxm_ipv4_src_masked_value_mask_set(obj, ipv4);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv4_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv4_src_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv4_src_masked_t *obj, int value) {
    of_ipv4_t ipv4;

    of_oxm_ipv4_src_masked_value_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    of_oxm_ipv4_src_masked_value_mask_get(obj, &ipv4);
    TEST_ASSERT(VAR_OF_IPV4_T_CHECK(ipv4, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_dst_value_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_dst_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_dst_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_dst_masked_value_set(obj, ipv6);
    value += 1;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_dst_masked_value_mask_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_dst_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_dst_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_dst_masked_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    of_oxm_ipv6_dst_masked_value_mask_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ipv6_flabel_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_flabel_t *obj, int value) {
    uint32_t val32;

    of_oxm_ipv6_flabel_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ipv6_flabel_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ipv6_flabel_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_flabel_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_flabel_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_ipv6_flabel_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_ipv6_flabel_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_sll_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_nd_sll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_sll_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_sll_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_sll_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_sll_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_sll_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_ipv6_nd_sll_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_nd_target_value_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_nd_target_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_nd_target_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_nd_target_masked_value_set(obj, ipv6);
    value += 1;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_nd_target_masked_value_mask_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_target_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_nd_target_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_nd_target_masked_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    of_oxm_ipv6_nd_target_masked_value_mask_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_tll_value_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_nd_tll_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_tll_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_tll_masked_value_set(obj, mac_addr);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_oxm_ipv6_nd_tll_masked_value_mask_set(obj, mac_addr);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_nd_tll_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value) {
    of_mac_addr_t mac_addr;

    of_oxm_ipv6_nd_tll_masked_value_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_oxm_ipv6_nd_tll_masked_value_mask_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_src_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_src_value_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_src_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_src_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_src_masked_value_set(obj, ipv6);
    value += 1;

    VAR_OF_IPV6_T_INIT(ipv6, value);
    of_oxm_ipv6_src_masked_value_mask_set(obj, ipv6);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ipv6_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ipv6_src_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ipv6_src_masked_t *obj, int value) {
    of_ipv6_t ipv6;

    of_oxm_ipv6_src_masked_value_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    of_oxm_ipv6_src_masked_value_mask_get(obj, &ipv6);
    TEST_ASSERT(VAR_OF_IPV6_T_CHECK(ipv6, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_3_populate_scalars(
    of_oxm_metadata_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_metadata_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_OF_VERSION_1_3_check_scalars(
    of_oxm_metadata_t *obj, int value) {
    uint64_t val64;

    of_oxm_metadata_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_metadata_masked_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_metadata_masked_value_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_metadata_masked_value_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_metadata_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_metadata_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_metadata_masked_t *obj, int value) {
    uint64_t val64;

    of_oxm_metadata_masked_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_oxm_metadata_masked_value_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_3_populate_scalars(
    of_oxm_mpls_bos_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_bos_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_OF_VERSION_1_3_check_scalars(
    of_oxm_mpls_bos_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_bos_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_bos_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_bos_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_bos_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_bos_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_mpls_bos_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_bos_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_mpls_bos_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_3_populate_scalars(
    of_oxm_mpls_label_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_mpls_label_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_OF_VERSION_1_3_check_scalars(
    of_oxm_mpls_label_t *obj, int value) {
    uint32_t val32;

    of_oxm_mpls_label_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_mpls_label_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_mpls_label_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_label_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_label_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_mpls_label_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_mpls_label_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_mpls_label_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_3_populate_scalars(
    of_oxm_mpls_tc_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_tc_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_OF_VERSION_1_3_check_scalars(
    of_oxm_mpls_tc_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_tc_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_tc_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_mpls_tc_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_mpls_tc_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_mpls_tc_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_mpls_tc_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_mpls_tc_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_mpls_tc_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_actset_output_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_actset_output,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_actset_output_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_actset_output_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_actset_output_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_bfd_discriminator_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_bfd_discriminator_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_bfd_discriminator_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_bfd_discriminator_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_bfd_discriminator_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_bfd_discriminator_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_ofdpa_bfd_discriminator_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_color_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_color_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_color_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_color_actions_index_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_color_actions_index,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_color_actions_index_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_color_actions_index_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_dei_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_dei,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_dei_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_dei_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_dei_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_eth_subtype_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_eth_subtype,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_eth_subtype_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_eth_subtype_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_l3_in_port_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_l3_in_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_l3_in_port_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_l3_in_port_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_lmep_id_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_lmep_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_lmep_id_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_lmep_id_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_ofdpa_mpls_ach_channel_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ach_channel,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value) {
    uint16_t val16;

    of_oxm_ofdpa_mpls_ach_channel_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_mpls_data_first_nibble_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_data_first_nibble,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_mpls_data_first_nibble_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_mpls_l2_port_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_mpls_l2_port_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_mpls_l2_port_masked_value_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_oxm_ofdpa_mpls_l2_port_masked_value_mask_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_l2_port_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value) {
    uint32_t val32;

    of_oxm_ofdpa_mpls_l2_port_masked_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_oxm_ofdpa_mpls_l2_port_masked_value_mask_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_mpls_next_label_is_gal_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_next_label_is_gal,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_mpls_next_label_is_gal_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_mpls_ttl_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_mpls_ttl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_mpls_ttl_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_oam_y1731_mdl_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_mdl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_oam_y1731_mdl_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_oam_y1731_opcode_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_oam_y1731_opcode,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_oam_y1731_opcode_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_ofdpa_ovid_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_ovid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_ovid_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_ovid_t *obj, int value) {
    uint16_t val16;

    of_oxm_ofdpa_ovid_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_protection_index_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_protection_index,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_protection_index_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_protection_index_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_protection_index_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_qos_index_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_qos_index,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_qos_index_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_qos_index_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_qos_index_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_ofdpa_rx_timestamp_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rx_timestamp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value) {
    uint64_t val64;

    of_oxm_ofdpa_rx_timestamp_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_ofdpa_rxfcl_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_rxfcl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_rxfcl_t *obj, int value) {
    uint64_t val64;

    of_oxm_ofdpa_rxfcl_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_ofdpa_traffic_class_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_traffic_class,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_traffic_class_t *obj, int value) {
    uint8_t val8;

    of_oxm_ofdpa_traffic_class_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_ofdpa_txfcl_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_txfcl,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_txfcl_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_txfcl_t *obj, int value) {
    uint64_t val64;

    of_oxm_ofdpa_txfcl_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_3_populate_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_ofdpa_vrf_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_ofdpa_vrf,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_ofdpa_vrf_OF_VERSION_1_3_check_scalars(
    of_oxm_ofdpa_vrf_t *obj, int value) {
    uint16_t val16;

    of_oxm_ofdpa_vrf_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_3_populate_scalars(
    of_oxm_sctp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_dst_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_OF_VERSION_1_3_check_scalars(
    of_oxm_sctp_dst_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_dst_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_dst_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_dst_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_dst_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_sctp_dst_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_dst_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_sctp_dst_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_3_populate_scalars(
    of_oxm_sctp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_src_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_OF_VERSION_1_3_check_scalars(
    of_oxm_sctp_src_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_src_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_src_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_sctp_src_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_sctp_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_sctp_src_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_sctp_src_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_sctp_src_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_sctp_src_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_3_populate_scalars(
    of_oxm_tcp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_dst_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_OF_VERSION_1_3_check_scalars(
    of_oxm_tcp_dst_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_dst_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_dst_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_dst_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_dst_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_tcp_dst_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_dst_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_tcp_dst_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_3_populate_scalars(
    of_oxm_tcp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_src_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_OF_VERSION_1_3_check_scalars(
    of_oxm_tcp_src_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_src_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_src_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_tcp_src_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tcp_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tcp_src_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_tcp_src_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_tcp_src_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_tcp_src_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_3_populate_scalars(
    of_oxm_tunnel_id_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_tunnel_id_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_OF_VERSION_1_3_check_scalars(
    of_oxm_tunnel_id_t *obj, int value) {
    uint64_t val64;

    of_oxm_tunnel_id_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_tunnel_id_masked_value_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_oxm_tunnel_id_masked_value_mask_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_tunnel_id_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_tunnel_id_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_tunnel_id_masked_t *obj, int value) {
    uint64_t val64;

    of_oxm_tunnel_id_masked_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_oxm_tunnel_id_masked_value_mask_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_3_populate_scalars(
    of_oxm_udp_dst_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_dst_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_OF_VERSION_1_3_check_scalars(
    of_oxm_udp_dst_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_dst_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_dst_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_dst_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_dst_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_dst_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_udp_dst_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_dst_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_udp_dst_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_3_populate_scalars(
    of_oxm_udp_src_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_src_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_OF_VERSION_1_3_check_scalars(
    of_oxm_udp_src_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_src_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_src_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_udp_src_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_udp_src_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_udp_src_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_udp_src_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_udp_src_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_udp_src_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_3_populate_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_vlan_pcp_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_OF_VERSION_1_3_check_scalars(
    of_oxm_vlan_pcp_t *obj, int value) {
    uint8_t val8;

    of_oxm_vlan_pcp_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_vlan_pcp_masked_value_set(obj, val8);
    value += 1;

    VAR_UINT8_T_INIT(val8, value);
    of_oxm_vlan_pcp_masked_value_mask_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_pcp_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_pcp_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_vlan_pcp_masked_t *obj, int value) {
    uint8_t val8;

    of_oxm_vlan_pcp_masked_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_oxm_vlan_pcp_masked_value_mask_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_3_populate_scalars(
    of_oxm_vlan_vid_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_vlan_vid_value_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_OF_VERSION_1_3_check_scalars(
    of_oxm_vlan_vid_t *obj, int value) {
    uint16_t val16;

    of_oxm_vlan_vid_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_3_populate_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_vlan_vid_masked_value_set(obj, val16);
    value += 1;

    VAR_UINT16_T_INIT(val16, value);
    of_oxm_vlan_vid_masked_value_mask_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_oxm_vlan_vid_masked,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_oxm_vlan_vid_masked_OF_VERSION_1_3_check_scalars(
    of_oxm_vlan_vid_masked_t *obj, int value) {
    uint16_t val16;

    of_oxm_vlan_vid_masked_value_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    of_oxm_vlan_vid_masked_value_mask_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_3_populate_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    VAR_UINT32_T_INIT(val32, value);
    of_packet_queue_queue_id_set(obj, val32);
    value += 1;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_packet_queue_port_set(obj, port_no);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_packet_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_packet_queue_OF_VERSION_1_3_check_scalars(
    of_packet_queue_t *obj, int value) {
    uint32_t val32;
    of_port_no_t port_no;

    of_packet_queue_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_packet_queue_port_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_3_populate_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_desc_port_no_set(obj, port_no);
    value += 1;

    VAR_OF_MAC_ADDR_T_INIT(mac_addr, value);
    of_port_desc_hw_addr_set(obj, mac_addr);
    value += 1;

    VAR_OF_PORT_NAME_T_INIT(port_name, value);
    of_port_desc_name_set(obj, port_name);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_state_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_curr_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_advertised_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_supported_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_peer_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_curr_speed_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_desc_max_speed_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_desc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_desc_OF_VERSION_1_3_check_scalars(
    of_port_desc_t *obj, int value) {
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    of_port_desc_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_desc_hw_addr_get(obj, &mac_addr);
    TEST_ASSERT(VAR_OF_MAC_ADDR_T_CHECK(mac_addr, value));
    value += 1;

    of_port_desc_name_get(obj, &port_name);
    TEST_ASSERT(VAR_OF_PORT_NAME_T_CHECK(port_name, value));
    value += 1;

    of_port_desc_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_state_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_curr_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_advertised_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_supported_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_peer_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_curr_speed_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_desc_max_speed_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;
    uint32_t val32;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_port_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_dropped_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_frame_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_over_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_rx_crc_err_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_port_stats_entry_collisions_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_entry_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_port_stats_entry_duration_nsec_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_port_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_port_stats_entry_OF_VERSION_1_3_check_scalars(
    of_port_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint64_t val64;
    uint32_t val32;

    of_port_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_port_stats_entry_rx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_dropped_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_over_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_collisions_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_port_stats_entry_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_port_stats_entry_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_3_populate_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_OF_VERSION_1_3_check_scalars(
    of_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_3_populate_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_experimenter_OF_VERSION_1_3_check_scalars(
    of_queue_prop_experimenter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_3_populate_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_header_OF_VERSION_1_3_check_scalars(
    of_queue_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_3_populate_scalars(
    of_queue_prop_max_rate_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_prop_max_rate_rate_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_max_rate,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_max_rate_OF_VERSION_1_3_check_scalars(
    of_queue_prop_max_rate_t *obj, int value) {
    uint16_t val16;

    of_queue_prop_max_rate_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_3_populate_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    VAR_UINT16_T_INIT(val16, value);
    of_queue_prop_min_rate_rate_set(obj, val16);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_prop_min_rate,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_prop_min_rate_OF_VERSION_1_3_check_scalars(
    of_queue_prop_min_rate_t *obj, int value) {
    uint16_t val16;

    of_queue_prop_min_rate_rate_get(obj, &val16);
    TEST_ASSERT(VAR_UINT16_T_CHECK(val16, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    VAR_OF_PORT_NO_T_INIT(port_no, value);
    of_queue_stats_entry_port_no_set(obj, port_no);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_entry_queue_id_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_bytes_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_packets_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_queue_stats_entry_tx_errors_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_entry_duration_sec_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_queue_stats_entry_duration_nsec_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_queue_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_queue_stats_entry_OF_VERSION_1_3_check_scalars(
    of_queue_stats_entry_t *obj, int value) {
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    of_queue_stats_entry_port_no_get(obj, &port_no);
    TEST_ASSERT(VAR_OF_PORT_NO_T_CHECK(port_no, value));
    value += 1;

    of_queue_stats_entry_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_packets_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_tx_errors_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_queue_stats_entry_duration_sec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_queue_stats_entry_duration_nsec_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_apply_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_actions_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_apply_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_apply_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_apply_setfield_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_table_feature_prop_experimenter_experimenter_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_feature_prop_experimenter_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_experimenter_t *obj, int value) {
    uint32_t val32;

    of_table_feature_prop_experimenter_experimenter_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_feature_prop_experimenter_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_table_feature_prop_experimenter_miss_experimenter_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_feature_prop_experimenter_miss_subtype_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_experimenter_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_experimenter_miss_t *obj, int value) {
    uint32_t val32;

    of_table_feature_prop_experimenter_miss_experimenter_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_feature_prop_experimenter_miss_subtype_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_header,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_header_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_header_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_instructions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_instructions_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_instructions_miss_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_instructions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_match,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_match_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_match_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_next_tables_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_next_tables_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_next_tables_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_wildcards,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_wildcards_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_wildcards_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_write_actions_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_actions_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_write_actions_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_write_setfield_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_populate_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_table_feature_prop_write_setfield_miss,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_check_scalars(
    of_table_feature_prop_write_setfield_miss_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_features,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_3_populate_scalars(
    of_table_features_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    uint64_t val64;
    uint32_t val32;

    VAR_UINT8_T_INIT(val8, value);
    of_table_features_table_id_set(obj, val8);
    value += 1;

    VAR_OF_TABLE_NAME_T_INIT(table_name, value);
    of_table_features_name_set(obj, table_name);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_features_metadata_match_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_features_metadata_write_set(obj, val64);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_features_config_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_features_max_entries_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_features,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_features_OF_VERSION_1_3_check_scalars(
    of_table_features_t *obj, int value) {
    uint8_t val8;
    of_table_name_t table_name;
    uint64_t val64;
    uint32_t val32;

    of_table_features_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_features_name_get(obj, &table_name);
    TEST_ASSERT(VAR_OF_TABLE_NAME_T_CHECK(table_name, value));
    value += 1;

    of_table_features_metadata_match_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_features_metadata_write_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_features_config_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_features_max_entries_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint64_t val64;

    VAR_UINT8_T_INIT(val8, value);
    of_table_stats_entry_table_id_set(obj, val8);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    of_table_stats_entry_active_count_set(obj, val32);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_lookup_count_set(obj, val64);
    value += 1;

    VAR_UINT64_T_INIT(val64, value);
    of_table_stats_entry_matched_count_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_table_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_table_stats_entry_OF_VERSION_1_3_check_scalars(
    of_table_stats_entry_t *obj, int value) {
    uint8_t val8;
    uint32_t val32;
    uint64_t val64;

    of_table_stats_entry_table_id_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    of_table_stats_entry_active_count_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    of_table_stats_entry_lookup_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    of_table_stats_entry_matched_count_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint32,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_3_populate_scalars(
    of_uint32_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    of_uint32_value_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_uint32,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint32_OF_VERSION_1_3_check_scalars(
    of_uint32_t *obj, int value) {
    uint32_t val32;

    of_uint32_value_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint64,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_3_populate_scalars(
    of_uint64_t *obj, int value) {
    uint64_t val64;

    VAR_UINT64_T_INIT(val64, value);
    of_uint64_value_set(obj, val64);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_uint64,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint64_OF_VERSION_1_3_check_scalars(
    of_uint64_t *obj, int value) {
    uint64_t val64;

    of_uint64_value_get(obj, &val64);
    TEST_ASSERT(VAR_UINT64_T_CHECK(val64, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_uint8,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_3_populate_scalars(
    of_uint8_t *obj, int value) {
    uint8_t val8;

    VAR_UINT8_T_INIT(val8, value);
    of_uint8_value_set(obj, val8);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type of_uint8,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_uint8_OF_VERSION_1_3_check_scalars(
    of_uint8_t *obj, int value) {
    uint8_t val8;

    of_uint8_value_get(obj, &val8);
    TEST_ASSERT(VAR_UINT8_T_CHECK(val8, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_3_populate_scalars(
    ofp_queue_desc_t *obj, int value) {
    uint32_t val32;

    VAR_UINT32_T_INIT(val32, value);
    ofp_queue_desc_port_no_set(obj, val32);
    value += 1;

    VAR_UINT32_T_INIT(val32, value);
    ofp_queue_desc_queue_id_set(obj, val32);
    value += 1;

    return value;
}

/**
 * Check scalar values in obj of type ofp_queue_desc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int ofp_queue_desc_OF_VERSION_1_3_check_scalars(
    ofp_queue_desc_t *obj, int value) {
    uint32_t val32;

    ofp_queue_desc_port_no_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    ofp_queue_desc_queue_id_get(obj, &val32);
    TEST_ASSERT(VAR_UINT32_T_CHECK(val32, value));
    value += 1;

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_3_populate_scalars(
    of_list_action_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_OF_VERSION_1_3_check_scalars(
    of_list_action_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_3_populate_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_action_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_action_id_OF_VERSION_1_3_check_scalars(
    of_list_action_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_controller_connection,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_controller_connection_OF_VERSION_1_3_check_scalars(
    of_list_bsn_controller_connection_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_debug_counter_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_debug_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_debug_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_flow_checksum_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_bucket_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_gentable_bucket_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_gentable_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_gentable_entry_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_entry_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_gentable_entry_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_gentable_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_gentable_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_gentable_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_interface,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_interface_OF_VERSION_1_3_check_scalars(
    of_list_bsn_interface_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_lacp_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_lacp_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_lacp_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_port_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_port_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_switch_pipeline_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_switch_pipeline_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_table_checksum_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_table_checksum_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_tlv,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_tlv_OF_VERSION_1_3_check_scalars(
    of_list_bsn_tlv_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bsn_vlan_counter_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_bsn_vlan_counter_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_3_populate_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_OF_VERSION_1_3_check_scalars(
    of_list_bucket_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_3_populate_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_bucket_counter,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_bucket_counter_OF_VERSION_1_3_check_scalars(
    of_list_bucket_counter_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_flow_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_flow_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_flow_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_desc_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_desc_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_group_desc_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_group_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_group_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_group_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_3_populate_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_hello_elem,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_hello_elem_OF_VERSION_1_3_check_scalars(
    of_list_hello_elem_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_3_populate_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_OF_VERSION_1_3_check_scalars(
    of_list_instruction_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_3_populate_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_instruction_id,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_instruction_id_OF_VERSION_1_3_check_scalars(
    of_list_instruction_id_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_3_populate_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_OF_VERSION_1_3_check_scalars(
    of_list_meter_band_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_3_populate_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_band_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_band_stats_OF_VERSION_1_3_check_scalars(
    of_list_meter_band_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_3_populate_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_meter_stats,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_meter_stats_OF_VERSION_1_3_check_scalars(
    of_list_meter_stats_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_3_populate_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_oxm,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_oxm_OF_VERSION_1_3_check_scalars(
    of_list_oxm_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_3_populate_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_packet_queue,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_packet_queue_OF_VERSION_1_3_check_scalars(
    of_list_packet_queue_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_3_populate_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_desc,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_desc_OF_VERSION_1_3_check_scalars(
    of_list_port_desc_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_port_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_port_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_port_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_3_populate_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_prop_OF_VERSION_1_3_check_scalars(
    of_list_queue_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_queue_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_queue_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_queue_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_3_populate_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_feature_prop,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_feature_prop_OF_VERSION_1_3_check_scalars(
    of_list_table_feature_prop_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_3_populate_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_features,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_features_OF_VERSION_1_3_check_scalars(
    of_list_table_features_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_3_populate_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_table_stats_entry,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_table_stats_entry_OF_VERSION_1_3_check_scalars(
    of_list_table_stats_entry_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_3_populate_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint32,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint32_OF_VERSION_1_3_check_scalars(
    of_list_uint32_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_3_populate_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint64,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint64_OF_VERSION_1_3_check_scalars(
    of_list_uint64_t *obj, int value) {

    return value;
}


/**
 * Populate the scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_3_populate_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}

/**
 * Check scalar values in obj of type of_list_uint8,
 * version OF_VERSION_1_3
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int of_list_uint8_OF_VERSION_1_3_check_scalars(
    of_list_uint8_t *obj, int value) {

    return value;
}


/**
 * Set up a list of type of_list_action with two of each type of subclass
 */
int
list_setup_of_list_action_OF_VERSION_1_0(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_enqueue_t *enqueue;
    of_action_strip_vlan_t *strip_vlan;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_set_tp_src_t *set_tp_src;
    of_action_set_dl_src_t *set_dl_src;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    set_vlan_pcp = &elt.set_vlan_pcp;
    enqueue = &elt.enqueue;
    strip_vlan = &elt.strip_vlan;
    set_dl_dst = &elt.set_dl_dst;
    set_tp_src = &elt.set_tp_src;
    set_dl_src = &elt.set_dl_src;
    output = &elt.output;

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_0_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_0_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_tos */

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_0_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_0_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_dst */

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_0_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_0_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_src */

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_0_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_0_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_vid */

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_dst */

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_0_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_0_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_pcp */

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_enqueue */

    of_action_enqueue_init(enqueue, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)enqueue);
    value = of_action_enqueue_OF_VERSION_1_0_populate(
        enqueue, value);
    cur_len += enqueue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_enqueue_init(enqueue, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)enqueue);
    value = of_action_enqueue_OF_VERSION_1_0_populate(
        enqueue, value);
    cur_len += enqueue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_strip_vlan */

    of_action_strip_vlan_init(strip_vlan, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)strip_vlan);
    value = of_action_strip_vlan_OF_VERSION_1_0_populate(
        strip_vlan, value);
    cur_len += strip_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_strip_vlan_init(strip_vlan, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)strip_vlan);
    value = of_action_strip_vlan_OF_VERSION_1_0_populate(
        strip_vlan, value);
    cur_len += strip_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_dst */

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_0_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_0_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_src */

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_0_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_0_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_src */

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_0_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_0_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_0_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_0_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_action generated by
 * list_setup_of_list_action_OF_VERSION_1_0
 */
int
list_check_of_list_action_OF_VERSION_1_0(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    /* Declare pointers for each subclass */
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_enqueue_t *enqueue;
    of_action_strip_vlan_t *strip_vlan;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_set_tp_src_t *set_tp_src;
    of_action_set_dl_src_t *set_dl_src;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    set_vlan_pcp = &elt.set_vlan_pcp;
    enqueue = &elt.enqueue;
    strip_vlan = &elt.strip_vlan;
    set_dl_dst = &elt.set_dl_dst;
    set_tp_src = &elt.set_tp_src;
    set_dl_src = &elt.set_dl_src;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_0_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_0_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_tos */
    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_0_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_0_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_dst */
    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_0_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_0_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_src */
    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_0_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_0_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_vid */
    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_dst */
    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_0_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_0_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_pcp */
    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type enqueue */
    TEST_ASSERT(enqueue->length == 16);
    if (enqueue->wire_length_get != NULL) {
        int length;

        enqueue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(enqueue->object_id == OF_ACTION_ENQUEUE);
    value = of_action_enqueue_OF_VERSION_1_0_check(
        enqueue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(enqueue->length == 16);
    if (enqueue->wire_length_get != NULL) {
        int length;

        enqueue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(enqueue->object_id == OF_ACTION_ENQUEUE);
    value = of_action_enqueue_OF_VERSION_1_0_check(
        enqueue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type strip_vlan */
    TEST_ASSERT(strip_vlan->length == 8);
    if (strip_vlan->wire_length_get != NULL) {
        int length;

        strip_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(strip_vlan->object_id == OF_ACTION_STRIP_VLAN);
    value = of_action_strip_vlan_OF_VERSION_1_0_check(
        strip_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(strip_vlan->length == 8);
    if (strip_vlan->wire_length_get != NULL) {
        int length;

        strip_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(strip_vlan->object_id == OF_ACTION_STRIP_VLAN);
    value = of_action_strip_vlan_OF_VERSION_1_0_check(
        strip_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_dst */
    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_0_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_0_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_src */
    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_0_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_0_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_src */
    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_0_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_0_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 8);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_0_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 8);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_0_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_interface with two of each type of subclass
 */
int
list_setup_of_list_bsn_interface_OF_VERSION_1_0(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * list_setup_of_list_bsn_interface_OF_VERSION_1_0
 */
int
list_check_of_list_bsn_interface_OF_VERSION_1_0(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_flow_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_flow_stats_entry_OF_VERSION_1_0(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * list_setup_of_list_flow_stats_entry_OF_VERSION_1_0
 */
int
list_check_of_list_flow_stats_entry_OF_VERSION_1_0(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_packet_queue with two of each type of subclass
 */
int
list_setup_of_list_packet_queue_OF_VERSION_1_0(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * list_setup_of_list_packet_queue_OF_VERSION_1_0
 */
int
list_check_of_list_packet_queue_OF_VERSION_1_0(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_desc with two of each type of subclass
 */
int
list_setup_of_list_port_desc_OF_VERSION_1_0(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * list_setup_of_list_port_desc_OF_VERSION_1_0
 */
int
list_check_of_list_port_desc_OF_VERSION_1_0(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_port_stats_entry_OF_VERSION_1_0(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * list_setup_of_list_port_stats_entry_OF_VERSION_1_0
 */
int
list_check_of_list_port_stats_entry_OF_VERSION_1_0(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_prop with two of each type of subclass
 */
int
list_setup_of_list_queue_prop_OF_VERSION_1_0(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_0, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_0, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * list_setup_of_list_queue_prop_OF_VERSION_1_0
 */
int
list_check_of_list_queue_prop_OF_VERSION_1_0(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_queue_stats_entry_OF_VERSION_1_0(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * list_setup_of_list_queue_stats_entry_OF_VERSION_1_0
 */
int
list_check_of_list_queue_stats_entry_OF_VERSION_1_0(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_table_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_table_stats_entry_OF_VERSION_1_0(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * list_setup_of_list_table_stats_entry_OF_VERSION_1_0
 */
int
list_check_of_list_table_stats_entry_OF_VERSION_1_0(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_action with two of each type of subclass
 */
int
list_setup_of_list_action_OF_VERSION_1_1(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_set_mpls_tc_t *set_mpls_tc;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_mpls_label_t *set_mpls_label;
    of_action_group_t *group;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_set_nw_ecn_t *set_nw_ecn;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_push_mpls_t *push_mpls;
    of_action_set_tp_src_t *set_tp_src;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_dl_src_t *set_dl_src;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    set_mpls_tc = &elt.set_mpls_tc;
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_mpls_label = &elt.set_mpls_label;
    group = &elt.group;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_vlan_pcp = &elt.set_vlan_pcp;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    set_nw_ecn = &elt.set_nw_ecn;
    set_dl_dst = &elt.set_dl_dst;
    push_mpls = &elt.push_mpls;
    set_tp_src = &elt.set_tp_src;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_dl_src = &elt.set_dl_src;
    set_queue = &elt.set_queue;
    output = &elt.output;

    /* Append two instances of type of_action_copy_ttl_out */

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_tc */

    of_action_set_mpls_tc_init(set_mpls_tc, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_tc);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_populate(
        set_mpls_tc, value);
    cur_len += set_mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_tc_init(set_mpls_tc, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_tc);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_populate(
        set_mpls_tc, value);
    cur_len += set_mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_1_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_1_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_tos */

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_1_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_1_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_mpls_ttl */

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_dst */

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_1_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_1_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_label */

    of_action_set_mpls_label_init(set_mpls_label, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_label);
    value = of_action_set_mpls_label_OF_VERSION_1_1_populate(
        set_mpls_label, value);
    cur_len += set_mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_label_init(set_mpls_label, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_label);
    value = of_action_set_mpls_label_OF_VERSION_1_1_populate(
        set_mpls_label, value);
    cur_len += set_mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_group */

    of_action_group_init(group, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_1_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_group_init(group, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_1_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_src */

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_1_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_1_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_vid */

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_ttl */

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_vlan */

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_1_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_1_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_dst */

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_1_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_1_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_mpls */

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_1_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_1_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_vlan */

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_1_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_1_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_pcp */

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ttl */

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_in */

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ecn */

    of_action_set_nw_ecn_init(set_nw_ecn, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ecn);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_populate(
        set_nw_ecn, value);
    cur_len += set_nw_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ecn_init(set_nw_ecn, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ecn);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_populate(
        set_nw_ecn, value);
    cur_len += set_nw_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_dst */

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_1_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_1_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_mpls */

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_1_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_1_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_src */

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_1_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_1_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_nw_ttl */

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_src */

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_1_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_1_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_queue */

    of_action_set_queue_init(set_queue, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_1_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_queue_init(set_queue, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_1_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_1_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_1_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_action generated by
 * list_setup_of_list_action_OF_VERSION_1_1
 */
int
list_check_of_list_action_OF_VERSION_1_1(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_set_mpls_tc_t *set_mpls_tc;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_mpls_label_t *set_mpls_label;
    of_action_group_t *group;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_set_nw_ecn_t *set_nw_ecn;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_push_mpls_t *push_mpls;
    of_action_set_tp_src_t *set_tp_src;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_dl_src_t *set_dl_src;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    set_mpls_tc = &elt.set_mpls_tc;
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_mpls_label = &elt.set_mpls_label;
    group = &elt.group;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_vlan_pcp = &elt.set_vlan_pcp;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    set_nw_ecn = &elt.set_nw_ecn;
    set_dl_dst = &elt.set_dl_dst;
    push_mpls = &elt.push_mpls;
    set_tp_src = &elt.set_tp_src;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_dl_src = &elt.set_dl_src;
    set_queue = &elt.set_queue;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_tc */
    TEST_ASSERT(set_mpls_tc->length == 8);
    if (set_mpls_tc->wire_length_get != NULL) {
        int length;

        set_mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_tc->object_id == OF_ACTION_SET_MPLS_TC);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_check(
        set_mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_tc->length == 8);
    if (set_mpls_tc->wire_length_get != NULL) {
        int length;

        set_mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_tc->object_id == OF_ACTION_SET_MPLS_TC);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_check(
        set_mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_1_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_1_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_tos */
    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_1_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_1_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_dst */
    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_1_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_1_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_label */
    TEST_ASSERT(set_mpls_label->length == 8);
    if (set_mpls_label->wire_length_get != NULL) {
        int length;

        set_mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_label->object_id == OF_ACTION_SET_MPLS_LABEL);
    value = of_action_set_mpls_label_OF_VERSION_1_1_check(
        set_mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_label->length == 8);
    if (set_mpls_label->wire_length_get != NULL) {
        int length;

        set_mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_label->object_id == OF_ACTION_SET_MPLS_LABEL);
    value = of_action_set_mpls_label_OF_VERSION_1_1_check(
        set_mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_1_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_1_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_src */
    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_1_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_1_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_vid */
    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_1_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_1_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_dst */
    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_1_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_1_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_1_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_1_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_1_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_1_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_pcp */
    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ecn */
    TEST_ASSERT(set_nw_ecn->length == 8);
    if (set_nw_ecn->wire_length_get != NULL) {
        int length;

        set_nw_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ecn->object_id == OF_ACTION_SET_NW_ECN);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_check(
        set_nw_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ecn->length == 8);
    if (set_nw_ecn->wire_length_get != NULL) {
        int length;

        set_nw_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ecn->object_id == OF_ACTION_SET_NW_ECN);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_check(
        set_nw_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_dst */
    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_1_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_1_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_1_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_1_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_src */
    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_1_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_1_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_src */
    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_1_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_1_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_1_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_1_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_1_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_1_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_interface with two of each type of subclass
 */
int
list_setup_of_list_bsn_interface_OF_VERSION_1_1(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * list_setup_of_list_bsn_interface_OF_VERSION_1_1
 */
int
list_check_of_list_bsn_interface_OF_VERSION_1_1(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bucket with two of each type of subclass
 */
int
list_setup_of_list_bucket_OF_VERSION_1_1(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket */

    of_bucket_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bucket generated by
 * list_setup_of_list_bucket_OF_VERSION_1_1
 */
int
list_check_of_list_bucket_OF_VERSION_1_1(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bucket_counter with two of each type of subclass
 */
int
list_setup_of_list_bucket_counter_OF_VERSION_1_1(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket_counter */

    of_bucket_counter_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_counter_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bucket_counter generated by
 * list_setup_of_list_bucket_counter_OF_VERSION_1_1
 */
int
list_check_of_list_bucket_counter_OF_VERSION_1_1(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_counter_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_counter_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_counter_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_flow_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_flow_stats_entry_OF_VERSION_1_1(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * list_setup_of_list_flow_stats_entry_OF_VERSION_1_1
 */
int
list_check_of_list_flow_stats_entry_OF_VERSION_1_1(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_group_desc_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_group_desc_stats_entry_OF_VERSION_1_1(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_desc_stats_entry */

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_group_desc_stats_entry generated by
 * list_setup_of_list_group_desc_stats_entry_OF_VERSION_1_1
 */
int
list_check_of_list_group_desc_stats_entry_OF_VERSION_1_1(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_group_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_group_stats_entry_OF_VERSION_1_1(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_stats_entry */

    of_group_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_group_stats_entry generated by
 * list_setup_of_list_group_stats_entry_OF_VERSION_1_1
 */
int
list_check_of_list_group_stats_entry_OF_VERSION_1_1(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_instruction with two of each type of subclass
 */
int
list_setup_of_list_instruction_OF_VERSION_1_1(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;

    /* Append two instances of type of_instruction_clear_actions */

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_1_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_1_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_actions */

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_1_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_1_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_goto_table */

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_1_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_1_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_apply_actions */

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_1_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_1_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_metadata */

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_1_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_1_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_instruction generated by
 * list_setup_of_list_instruction_OF_VERSION_1_1
 */
int
list_check_of_list_instruction_OF_VERSION_1_1(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;
    TEST_OK(of_list_instruction_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_1_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_1_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_1_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_1_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_1_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_1_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_1_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_1_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_1_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_1_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_packet_queue with two of each type of subclass
 */
int
list_setup_of_list_packet_queue_OF_VERSION_1_1(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * list_setup_of_list_packet_queue_OF_VERSION_1_1
 */
int
list_check_of_list_packet_queue_OF_VERSION_1_1(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_desc with two of each type of subclass
 */
int
list_setup_of_list_port_desc_OF_VERSION_1_1(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * list_setup_of_list_port_desc_OF_VERSION_1_1
 */
int
list_check_of_list_port_desc_OF_VERSION_1_1(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_port_stats_entry_OF_VERSION_1_1(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * list_setup_of_list_port_stats_entry_OF_VERSION_1_1
 */
int
list_check_of_list_port_stats_entry_OF_VERSION_1_1(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_prop with two of each type of subclass
 */
int
list_setup_of_list_queue_prop_OF_VERSION_1_1(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_1, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_1, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * list_setup_of_list_queue_prop_OF_VERSION_1_1
 */
int
list_check_of_list_queue_prop_OF_VERSION_1_1(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_queue_stats_entry_OF_VERSION_1_1(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * list_setup_of_list_queue_stats_entry_OF_VERSION_1_1
 */
int
list_check_of_list_queue_stats_entry_OF_VERSION_1_1(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_table_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_table_stats_entry_OF_VERSION_1_1(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * list_setup_of_list_table_stats_entry_OF_VERSION_1_1
 */
int
list_check_of_list_table_stats_entry_OF_VERSION_1_1(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_action with two of each type of subclass
 */
int
list_setup_of_list_action_OF_VERSION_1_2(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_group_t *group;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_push_mpls_t *push_mpls;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    group = &elt.group;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    push_mpls = &elt.push_mpls;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_queue = &elt.set_queue;
    output = &elt.output;

    /* Append two instances of type of_action_copy_ttl_out */

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_2_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_2_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_field */

    of_action_set_field_init(set_field, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_2_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_field_init(set_field, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_2_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_mpls_ttl */

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_group */

    of_action_group_init(group, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_2_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_group_init(group, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_2_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_ttl */

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_vlan */

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_2_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_2_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_mpls */

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_2_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_2_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_vlan */

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_2_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_2_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ttl */

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_in */

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_mpls */

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_2_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_2_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_nw_ttl */

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_queue */

    of_action_set_queue_init(set_queue, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_2_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_queue_init(set_queue, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_2_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_2_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_2_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_action generated by
 * list_setup_of_list_action_OF_VERSION_1_2
 */
int
list_check_of_list_action_OF_VERSION_1_2(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_group_t *group;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_push_mpls_t *push_mpls;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    group = &elt.group;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    push_mpls = &elt.push_mpls;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_queue = &elt.set_queue;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_2_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_2_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_field */
    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_2_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_2_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_2_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_2_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_2_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_2_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_2_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_2_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_2_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_2_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_2_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_2_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_2_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_2_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_2_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_2_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_interface with two of each type of subclass
 */
int
list_setup_of_list_bsn_interface_OF_VERSION_1_2(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * list_setup_of_list_bsn_interface_OF_VERSION_1_2
 */
int
list_check_of_list_bsn_interface_OF_VERSION_1_2(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bucket with two of each type of subclass
 */
int
list_setup_of_list_bucket_OF_VERSION_1_2(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket */

    of_bucket_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bucket generated by
 * list_setup_of_list_bucket_OF_VERSION_1_2
 */
int
list_check_of_list_bucket_OF_VERSION_1_2(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bucket_counter with two of each type of subclass
 */
int
list_setup_of_list_bucket_counter_OF_VERSION_1_2(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket_counter */

    of_bucket_counter_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_counter_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bucket_counter generated by
 * list_setup_of_list_bucket_counter_OF_VERSION_1_2
 */
int
list_check_of_list_bucket_counter_OF_VERSION_1_2(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_counter_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_counter_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_counter_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_flow_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_flow_stats_entry_OF_VERSION_1_2(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * list_setup_of_list_flow_stats_entry_OF_VERSION_1_2
 */
int
list_check_of_list_flow_stats_entry_OF_VERSION_1_2(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_group_desc_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_group_desc_stats_entry_OF_VERSION_1_2(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_desc_stats_entry */

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_group_desc_stats_entry generated by
 * list_setup_of_list_group_desc_stats_entry_OF_VERSION_1_2
 */
int
list_check_of_list_group_desc_stats_entry_OF_VERSION_1_2(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_group_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_group_stats_entry_OF_VERSION_1_2(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_stats_entry */

    of_group_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_group_stats_entry generated by
 * list_setup_of_list_group_stats_entry_OF_VERSION_1_2
 */
int
list_check_of_list_group_stats_entry_OF_VERSION_1_2(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_instruction with two of each type of subclass
 */
int
list_setup_of_list_instruction_OF_VERSION_1_2(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;

    /* Append two instances of type of_instruction_clear_actions */

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_2_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_2_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_actions */

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_2_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_2_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_goto_table */

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_2_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_2_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_apply_actions */

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_2_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_2_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_metadata */

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_2_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_2_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_instruction generated by
 * list_setup_of_list_instruction_OF_VERSION_1_2
 */
int
list_check_of_list_instruction_OF_VERSION_1_2(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;
    TEST_OK(of_list_instruction_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_2_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_2_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_2_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_2_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_2_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_2_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_2_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_2_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_2_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_2_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_oxm with two of each type of subclass
 */
int
list_setup_of_list_oxm_OF_VERSION_1_2(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    mpls_tc_masked = &elt.mpls_tc_masked;
    arp_sha = &elt.arp_sha;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    in_phy_port_masked = &elt.in_phy_port_masked;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;

    /* Append two instances of type of_oxm_mpls_tc_masked */

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha */

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_2_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_2_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src_masked */

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst */

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_2_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_2_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id */

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id */

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op */

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_2_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_2_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type */

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_2_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_2_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128_masked */

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos */

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa */

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_2_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_2_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type_masked */

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha_masked */

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst */

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_2_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_2_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf_masked */

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target */

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type_masked */

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed_masked */

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6_masked */

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type */

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst_masked */

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst */

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto */

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_2_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_2_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata */

    of_oxm_metadata_init(metadata, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_2_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_init(metadata, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_2_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa_masked */

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel */

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id_masked */

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll_masked */

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa */

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src */

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_2_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_2_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src */

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata_masked */

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll */

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id_masked */

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id */

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid */

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha */

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_2_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_2_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp */

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7_masked */

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos_masked */

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst */

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id */

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst_masked */

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2_masked */

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port_masked */

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn */

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code_masked */

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst_masked */

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src_masked */

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst */

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst_masked */

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1 */

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0 */

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll */

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2 */

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5 */

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4 */

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7 */

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto_masked */

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port_masked */

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp_masked */

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst_masked */

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp */

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src */

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst */

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src_masked */

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op_masked */

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src */

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_2_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_2_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code */

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5_masked */

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code_masked */

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src_masked */

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn_masked */

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src */

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_2_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_2_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid_masked */

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha_masked */

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src_masked */

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port */

    of_oxm_in_port_init(in_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_2_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_init(in_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_2_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id */

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll_masked */

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src */

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_2_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_2_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target_masked */

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type_masked */

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0_masked */

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3_masked */

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label */

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_2_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_2_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128 */

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id_masked */

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel_masked */

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id_masked */

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp_masked */

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type */

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id_masked */

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_tc */

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3 */

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id_masked */

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa_masked */

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst_masked */

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6 */

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code */

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf */

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1_masked */

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4_masked */

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src_masked */

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed */

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port */

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label_masked */

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id */

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_oxm generated by
 * list_setup_of_list_oxm_OF_VERSION_1_2
 */
int
list_check_of_list_oxm_OF_VERSION_1_2(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    /* Declare pointers for each subclass */
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    mpls_tc_masked = &elt.mpls_tc_masked;
    arp_sha = &elt.arp_sha;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    in_phy_port_masked = &elt.in_phy_port_masked;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;
    TEST_OK(of_list_oxm_first(list, &elt));

    /* Check two instances of type mpls_tc_masked */
    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha */
    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_2_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_2_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src_masked */
    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst */
    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_2_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_2_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id */
    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id */
    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op */
    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_2_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_2_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type */
    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_2_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_2_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128_masked */
    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos */
    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa */
    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_2_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_2_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type_masked */
    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha_masked */
    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst */
    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_2_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_2_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf_masked */
    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target */
    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type_masked */
    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed_masked */
    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6_masked */
    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type */
    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst_masked */
    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst */
    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto */
    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_2_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_2_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata */
    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_2_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_2_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa_masked */
    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel */
    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id_masked */
    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll_masked */
    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa */
    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src */
    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_2_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_2_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src */
    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata_masked */
    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll */
    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id_masked */
    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id */
    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid */
    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha */
    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_2_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_2_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp */
    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7_masked */
    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos_masked */
    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst */
    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id */
    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst_masked */
    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2_masked */
    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port_masked */
    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn */
    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code_masked */
    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst_masked */
    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src_masked */
    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst */
    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst_masked */
    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1 */
    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0 */
    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll */
    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2 */
    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5 */
    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4 */
    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7 */
    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto_masked */
    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port_masked */
    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp_masked */
    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst_masked */
    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp */
    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src */
    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst */
    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src_masked */
    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op_masked */
    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src */
    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_2_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_2_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code */
    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5_masked */
    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code_masked */
    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src_masked */
    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn_masked */
    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src */
    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_2_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_2_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid_masked */
    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha_masked */
    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src_masked */
    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port */
    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_2_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_2_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id */
    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll_masked */
    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src */
    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_2_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_2_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target_masked */
    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type_masked */
    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0_masked */
    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3_masked */
    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label */
    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_2_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_2_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128 */
    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id_masked */
    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel_masked */
    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id_masked */
    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp_masked */
    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type */
    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id_masked */
    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_tc */
    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3 */
    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id_masked */
    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa_masked */
    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst_masked */
    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6 */
    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code */
    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf */
    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1_masked */
    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4_masked */
    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src_masked */
    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed */
    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port */
    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label_masked */
    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id */
    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_oxm_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_packet_queue with two of each type of subclass
 */
int
list_setup_of_list_packet_queue_OF_VERSION_1_2(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * list_setup_of_list_packet_queue_OF_VERSION_1_2
 */
int
list_check_of_list_packet_queue_OF_VERSION_1_2(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_desc with two of each type of subclass
 */
int
list_setup_of_list_port_desc_OF_VERSION_1_2(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * list_setup_of_list_port_desc_OF_VERSION_1_2
 */
int
list_check_of_list_port_desc_OF_VERSION_1_2(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_port_stats_entry_OF_VERSION_1_2(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * list_setup_of_list_port_stats_entry_OF_VERSION_1_2
 */
int
list_check_of_list_port_stats_entry_OF_VERSION_1_2(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_prop with two of each type of subclass
 */
int
list_setup_of_list_queue_prop_OF_VERSION_1_2(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_queue_prop_max_rate */

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * list_setup_of_list_queue_prop_OF_VERSION_1_2
 */
int
list_check_of_list_queue_prop_OF_VERSION_1_2(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    /* Check two instances of type max_rate */
    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_queue_stats_entry_OF_VERSION_1_2(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * list_setup_of_list_queue_stats_entry_OF_VERSION_1_2
 */
int
list_check_of_list_queue_stats_entry_OF_VERSION_1_2(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_table_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_table_stats_entry_OF_VERSION_1_2(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * list_setup_of_list_table_stats_entry_OF_VERSION_1_2
 */
int
list_check_of_list_table_stats_entry_OF_VERSION_1_2(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_action with two of each type of subclass
 */
int
list_setup_of_list_action_OF_VERSION_1_3(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_pop_pbb_t *pop_pbb;
    of_action_group_t *group;
    of_action_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_push_pbb_t *push_pbb;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_push_mpls_t *push_mpls;
    of_action_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_set_queue_t *set_queue;
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    pop_pbb = &elt.pop_pbb;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_queue = &elt.set_queue;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;

    /* Append two instances of type of_action_ofdpa_set_pcpdei_from_table */

    of_action_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_out */

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_field */

    of_action_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_push_cw */

    of_action_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_cw);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_cw);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_copy_tc_in */

    of_action_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_in);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_in);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_dec_ttl_mtu */

    of_action_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_dec_ttl_mtu);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_dec_ttl_mtu);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_mpls_ttl */

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_copy_tc_out */

    of_action_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_out);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_out);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_dscp_from_table */

    of_action_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_dscp_from_table);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_dscp_from_table);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_pbb */

    of_action_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_pbb);
    value = of_action_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_pbb);
    value = of_action_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_group */

    of_action_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_push_l2hdr */

    of_action_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_l2hdr);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_l2hdr);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_color_based_rx_count */

    of_action_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_rx_count);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_rx_count);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_oam_lm_tx_count */

    of_action_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_tx_count);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_tx_count);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_ttl */

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_vlan */

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_counter_fields */

    of_action_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_counter_fields);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_counter_fields);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_tc_from_vpn_table */

    of_action_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_pop_cw */

    of_action_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_cw);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_cw);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_qos_from_table */

    of_action_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_pbb */

    of_action_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_pbb);
    value = of_action_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_pbb);
    value = of_action_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_oam_lm_rx_count */

    of_action_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_rx_count);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_rx_count);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_color_based_tx_count */

    of_action_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_tx_count);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_tx_count);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_mpls */

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_vlan */

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_pcpdei_from_table */

    of_action_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_tc_from_tunnel_table */

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ttl */

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_in */

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_pop_l2hdr */

    of_action_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_l2hdr);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_l2hdr);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_mpls */

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_class_based_rx_count */

    of_action_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_rx_count);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_rx_count);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_nw_ttl */

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_class_based_tx_count */

    of_action_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_tx_count);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_tx_count);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_check_drop_status */

    of_action_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_check_drop_status);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_check_drop_status);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_queue */

    of_action_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table */

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_action generated by
 * list_setup_of_list_action_OF_VERSION_1_3
 */
int
list_check_of_list_action_OF_VERSION_1_3(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    /* Declare pointers for each subclass */
    of_action_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_pop_pbb_t *pop_pbb;
    of_action_group_t *group;
    of_action_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_push_pbb_t *push_pbb;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_push_mpls_t *push_mpls;
    of_action_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_set_queue_t *set_queue;
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    pop_pbb = &elt.pop_pbb;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_queue = &elt.set_queue;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type ofdpa_set_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 16);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 16);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_field */
    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_push_cw */
    TEST_ASSERT(ofdpa_push_cw->length == 16);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_OFDPA_PUSH_CW);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_push_cw->length == 16);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_OFDPA_PUSH_CW);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_in */
    TEST_ASSERT(ofdpa_copy_tc_in->length == 16);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_OFDPA_COPY_TC_IN);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_in->length == 16);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_OFDPA_COPY_TC_IN);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_dec_ttl_mtu */
    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 16);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_OFDPA_DEC_TTL_MTU);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 16);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_OFDPA_DEC_TTL_MTU);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_out */
    TEST_ASSERT(ofdpa_copy_tc_out->length == 16);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_OFDPA_COPY_TC_OUT);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_out->length == 16);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_OFDPA_COPY_TC_OUT);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_dscp_from_table */
    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 16);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 16);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_pbb */
    TEST_ASSERT(pop_pbb->length == 8);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_POP_PBB);
    value = of_action_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_pbb->length == 8);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_POP_PBB);
    value = of_action_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_push_l2hdr */
    TEST_ASSERT(ofdpa_push_l2hdr->length == 16);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_OFDPA_PUSH_L2HDR);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_push_l2hdr->length == 16);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_OFDPA_PUSH_L2HDR);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_rx_count */
    TEST_ASSERT(ofdpa_color_based_rx_count->length == 16);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_rx_count->length == 16);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_tx_count */
    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 16);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 16);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_counter_fields */
    TEST_ASSERT(ofdpa_set_counter_fields->length == 18);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_counter_fields->length == 18);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_vpn_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 16);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 16);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_pop_cw */
    TEST_ASSERT(ofdpa_pop_cw->length == 16);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_OFDPA_POP_CW);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_cw->length == 16);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_OFDPA_POP_CW);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_qos_from_table */
    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 16);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 16);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_pbb */
    TEST_ASSERT(push_pbb->length == 8);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_PUSH_PBB);
    value = of_action_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_pbb->length == 8);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_PUSH_PBB);
    value = of_action_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_rx_count */
    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 16);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 16);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_tx_count */
    TEST_ASSERT(ofdpa_color_based_tx_count->length == 16);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_tx_count->length == 16);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_pop_l2hdr */
    TEST_ASSERT(ofdpa_pop_l2hdr->length == 16);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_OFDPA_POP_L2HDR);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_l2hdr->length == 16);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_OFDPA_POP_L2HDR);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_rx_count */
    TEST_ASSERT(ofdpa_class_based_rx_count->length == 16);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_rx_count->length == 16);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_tx_count */
    TEST_ASSERT(ofdpa_class_based_tx_count->length == 16);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_tx_count->length == 16);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_check_drop_status */
    TEST_ASSERT(ofdpa_check_drop_status->length == 16);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_OFDPA_CHECK_DROP_STATUS);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_check_drop_status->length == 16);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_OFDPA_CHECK_DROP_STATUS);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_action_id with two of each type of subclass
 */
int
list_setup_of_list_action_id_OF_VERSION_1_3(
    of_list_action_id_t *list, int value)
{

    of_action_id_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_id_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_id_copy_ttl_out_t *copy_ttl_out;
    of_action_id_bsn_mirror_t *bsn_mirror;
    of_action_id_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_id_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_id_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_id_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_id_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_id_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_id_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_id_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_id_copy_ttl_in_t *copy_ttl_in;
    of_action_id_group_t *group;
    of_action_id_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_id_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_id_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_id_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_id_set_field_t *set_field;
    of_action_id_set_mpls_ttl_t *set_mpls_ttl;
    of_action_id_pop_vlan_t *pop_vlan;
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_id_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_id_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_id_push_pbb_t *push_pbb;
    of_action_id_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_id_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_id_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_id_pop_mpls_t *pop_mpls;
    of_action_id_push_vlan_t *push_vlan;
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_id_set_nw_ttl_t *set_nw_ttl;
    of_action_id_pop_pbb_t *pop_pbb;
    of_action_id_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_id_set_queue_t *set_queue;
    of_action_id_push_mpls_t *push_mpls;
    of_action_id_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_id_dec_nw_ttl_t *dec_nw_ttl;
    of_action_id_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_id_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    copy_ttl_in = &elt.copy_ttl_in;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_field = &elt.set_field;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    pop_pbb = &elt.pop_pbb;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    set_queue = &elt.set_queue;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;

    /* Append two instances of type of_action_id_ofdpa_set_pcpdei_from_table */

    of_action_id_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_copy_ttl_out */

    of_action_id_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_out);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_out);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_bsn_mirror */

    of_action_id_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_mirror);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_mirror);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_push_cw */

    of_action_id_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_cw);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_cw);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_copy_tc_in */

    of_action_id_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_in);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_in);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_dec_ttl_mtu */

    of_action_id_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_dec_ttl_mtu);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_dec_ttl_mtu);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_dec_mpls_ttl */

    of_action_id_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_mpls_ttl);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_mpls_ttl);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_bsn_set_tunnel_dst */

    of_action_id_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_set_tunnel_dst);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_set_tunnel_dst);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_oam_lm_tx_count */

    of_action_id_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_tx_count);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_tx_count);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_copy_tc_out */

    of_action_id_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_out);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_out);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_dscp_from_table */

    of_action_id_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_dscp_from_table);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_dscp_from_table);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_copy_ttl_in */

    of_action_id_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_in);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_in);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_group */

    of_action_id_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)group);
    value = of_action_id_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)group);
    value = of_action_id_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_push_l2hdr */

    of_action_id_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_l2hdr);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_l2hdr);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_color_based_rx_count */

    of_action_id_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_rx_count);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_rx_count);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_counter_fields */

    of_action_id_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_counter_fields);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_counter_fields);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_check_drop_status */

    of_action_id_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_check_drop_status);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_check_drop_status);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_field */

    of_action_id_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_field);
    value = of_action_id_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_field);
    value = of_action_id_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_mpls_ttl */

    of_action_id_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_mpls_ttl);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_mpls_ttl);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_pop_vlan */

    of_action_id_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_vlan);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_vlan);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table */

    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_pop_cw */

    of_action_id_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_cw);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_cw);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_qos_from_table */

    of_action_id_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_push_pbb */

    of_action_id_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_pbb);
    value = of_action_id_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_pbb);
    value = of_action_id_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_nicira_dec_ttl */

    of_action_id_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)nicira_dec_ttl);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)nicira_dec_ttl);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_oam_lm_rx_count */

    of_action_id_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_rx_count);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_rx_count);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_color_based_tx_count */

    of_action_id_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_tx_count);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_tx_count);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_pop_mpls */

    of_action_id_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_mpls);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_mpls);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_push_vlan */

    of_action_id_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_vlan);
    value = of_action_id_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_vlan);
    value = of_action_id_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_pcpdei_from_table */

    of_action_id_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table */

    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_nw_ttl */

    of_action_id_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_nw_ttl);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_nw_ttl);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_pop_pbb */

    of_action_id_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_pbb);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_pbb);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_pop_l2hdr */

    of_action_id_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_l2hdr);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_l2hdr);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_queue */

    of_action_id_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_queue);
    value = of_action_id_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_queue);
    value = of_action_id_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_push_mpls */

    of_action_id_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_mpls);
    value = of_action_id_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_mpls);
    value = of_action_id_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_class_based_rx_count */

    of_action_id_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_rx_count);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_rx_count);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_dec_nw_ttl */

    of_action_id_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_nw_ttl);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_nw_ttl);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_class_based_tx_count */

    of_action_id_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_tx_count);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_tx_count);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table */

    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_output */

    of_action_id_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)output);
    value = of_action_id_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)output);
    value = of_action_id_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_action_id generated by
 * list_setup_of_list_action_id_OF_VERSION_1_3
 */
int
list_check_of_list_action_id_OF_VERSION_1_3(
    of_list_action_id_t *list, int value)
{

    of_action_id_t elt;
    /* Declare pointers for each subclass */
    of_action_id_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_id_copy_ttl_out_t *copy_ttl_out;
    of_action_id_bsn_mirror_t *bsn_mirror;
    of_action_id_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_id_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_id_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_id_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_id_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_id_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_id_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_id_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_id_copy_ttl_in_t *copy_ttl_in;
    of_action_id_group_t *group;
    of_action_id_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_id_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_id_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_id_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_id_set_field_t *set_field;
    of_action_id_set_mpls_ttl_t *set_mpls_ttl;
    of_action_id_pop_vlan_t *pop_vlan;
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_id_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_id_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_id_push_pbb_t *push_pbb;
    of_action_id_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_id_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_id_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_id_pop_mpls_t *pop_mpls;
    of_action_id_push_vlan_t *push_vlan;
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_id_set_nw_ttl_t *set_nw_ttl;
    of_action_id_pop_pbb_t *pop_pbb;
    of_action_id_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_id_set_queue_t *set_queue;
    of_action_id_push_mpls_t *push_mpls;
    of_action_id_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_id_dec_nw_ttl_t *dec_nw_ttl;
    of_action_id_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_id_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    copy_ttl_in = &elt.copy_ttl_in;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_field = &elt.set_field;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    pop_pbb = &elt.pop_pbb;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    set_queue = &elt.set_queue;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;
    TEST_OK(of_list_action_id_first(list, &elt));

    /* Check two instances of type ofdpa_set_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 10);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 10);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 4);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_ID_COPY_TTL_OUT);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 4);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_ID_COPY_TTL_OUT);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 12);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_ID_BSN_MIRROR);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 12);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_ID_BSN_MIRROR);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_push_cw */
    TEST_ASSERT(ofdpa_push_cw->length == 10);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_ID_OFDPA_PUSH_CW);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_push_cw->length == 10);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_ID_OFDPA_PUSH_CW);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_in */
    TEST_ASSERT(ofdpa_copy_tc_in->length == 10);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_ID_OFDPA_COPY_TC_IN);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_in->length == 10);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_ID_OFDPA_COPY_TC_IN);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_dec_ttl_mtu */
    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 10);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_ID_OFDPA_DEC_TTL_MTU);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 10);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_ID_OFDPA_DEC_TTL_MTU);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 4);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_ID_DEC_MPLS_TTL);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 4);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_ID_DEC_MPLS_TTL);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 12);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_ID_BSN_SET_TUNNEL_DST);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 12);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_ID_BSN_SET_TUNNEL_DST);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_tx_count */
    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 10);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 10);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_out */
    TEST_ASSERT(ofdpa_copy_tc_out->length == 10);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_ID_OFDPA_COPY_TC_OUT);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_out->length == 10);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_ID_OFDPA_COPY_TC_OUT);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_dscp_from_table */
    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 10);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_ID_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 10);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_ID_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 4);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_ID_COPY_TTL_IN);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 4);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_ID_COPY_TTL_IN);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 4);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_ID_GROUP);
    value = of_action_id_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(group->length == 4);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_ID_GROUP);
    value = of_action_id_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_push_l2hdr */
    TEST_ASSERT(ofdpa_push_l2hdr->length == 10);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_ID_OFDPA_PUSH_L2HDR);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_push_l2hdr->length == 10);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_ID_OFDPA_PUSH_L2HDR);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_rx_count */
    TEST_ASSERT(ofdpa_color_based_rx_count->length == 10);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_rx_count->length == 10);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_counter_fields */
    TEST_ASSERT(ofdpa_set_counter_fields->length == 10);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_ID_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_counter_fields->length == 10);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_ID_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_check_drop_status */
    TEST_ASSERT(ofdpa_check_drop_status->length == 10);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_ID_OFDPA_CHECK_DROP_STATUS);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_check_drop_status->length == 10);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_ID_OFDPA_CHECK_DROP_STATUS);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_field */
    TEST_ASSERT(set_field->length == 4);
    if (set_field->wire_length_get != NULL) {
        int length;

        set_field->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_field->object_id == OF_ACTION_ID_SET_FIELD);
    value = of_action_id_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_field->length == 4);
    if (set_field->wire_length_get != NULL) {
        int length;

        set_field->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_field->object_id == OF_ACTION_ID_SET_FIELD);
    value = of_action_id_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 4);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_ID_SET_MPLS_TTL);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 4);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_ID_SET_MPLS_TTL);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 4);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_ID_POP_VLAN);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 4);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_ID_POP_VLAN);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_vpn_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 10);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 10);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_pop_cw */
    TEST_ASSERT(ofdpa_pop_cw->length == 10);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_ID_OFDPA_POP_CW);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_cw->length == 10);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_ID_OFDPA_POP_CW);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_qos_from_table */
    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 10);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 10);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type push_pbb */
    TEST_ASSERT(push_pbb->length == 4);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_ID_PUSH_PBB);
    value = of_action_id_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(push_pbb->length == 4);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_ID_PUSH_PBB);
    value = of_action_id_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 10);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_ID_NICIRA_DEC_TTL);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 10);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_ID_NICIRA_DEC_TTL);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_rx_count */
    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 10);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 10);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_tx_count */
    TEST_ASSERT(ofdpa_color_based_tx_count->length == 10);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_tx_count->length == 10);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 4);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_ID_POP_MPLS);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 4);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_ID_POP_MPLS);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 4);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_ID_PUSH_VLAN);
    value = of_action_id_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 4);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_ID_PUSH_VLAN);
    value = of_action_id_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 4);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_ID_SET_NW_TTL);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 4);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_ID_SET_NW_TTL);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type pop_pbb */
    TEST_ASSERT(pop_pbb->length == 4);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_ID_POP_PBB);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(pop_pbb->length == 4);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_ID_POP_PBB);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_pop_l2hdr */
    TEST_ASSERT(ofdpa_pop_l2hdr->length == 10);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_ID_OFDPA_POP_L2HDR);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_l2hdr->length == 10);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_ID_OFDPA_POP_L2HDR);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 4);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_ID_SET_QUEUE);
    value = of_action_id_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_queue->length == 4);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_ID_SET_QUEUE);
    value = of_action_id_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 4);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_ID_PUSH_MPLS);
    value = of_action_id_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 4);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_ID_PUSH_MPLS);
    value = of_action_id_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_rx_count */
    TEST_ASSERT(ofdpa_class_based_rx_count->length == 10);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_rx_count->length == 10);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 4);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_ID_DEC_NW_TTL);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 4);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_ID_DEC_NW_TTL);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_tx_count */
    TEST_ASSERT(ofdpa_class_based_tx_count->length == 10);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_tx_count->length == 10);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 4);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_ID_OUTPUT);
    value = of_action_id_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(output->length == 4);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_ID_OUTPUT);
    value = of_action_id_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_id_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_controller_connection with two of each type of subclass
 */
int
list_setup_of_list_bsn_controller_connection_OF_VERSION_1_3(
    of_list_bsn_controller_connection_t *list, int value)
{

    of_bsn_controller_connection_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_controller_connection */
    of_bsn_controller_connection_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_controller_connection */

    of_bsn_controller_connection_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_controller_connection_append_bind(list,
            (of_bsn_controller_connection_t *)elt_p);
    value = of_bsn_controller_connection_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_controller_connection_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_controller_connection_append_bind(list,
            (of_bsn_controller_connection_t *)elt_p);
    value = of_bsn_controller_connection_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_controller_connection generated by
 * list_setup_of_list_bsn_controller_connection_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_controller_connection_OF_VERSION_1_3(
    of_list_bsn_controller_connection_t *list, int value)
{

    of_bsn_controller_connection_t elt;
    /* No subclasses for of_bsn_controller_connection */
    of_bsn_controller_connection_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_controller_connection_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_CONTROLLER_CONNECTION);
    value = of_bsn_controller_connection_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_controller_connection_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_CONTROLLER_CONNECTION);
    value = of_bsn_controller_connection_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_controller_connection_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_debug_counter_desc_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3(
    of_list_bsn_debug_counter_desc_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_debug_counter_desc_stats_entry */
    of_bsn_debug_counter_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_debug_counter_desc_stats_entry */

    of_bsn_debug_counter_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_desc_stats_entry_append_bind(list,
            (of_bsn_debug_counter_desc_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_debug_counter_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_desc_stats_entry_append_bind(list,
            (of_bsn_debug_counter_desc_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_debug_counter_desc_stats_entry generated by
 * list_setup_of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3(
    of_list_bsn_debug_counter_desc_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_desc_stats_entry_t elt;
    /* No subclasses for of_bsn_debug_counter_desc_stats_entry */
    of_bsn_debug_counter_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_debug_counter_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_DESC_STATS_ENTRY);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_debug_counter_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_DESC_STATS_ENTRY);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_debug_counter_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_debug_counter_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3(
    of_list_bsn_debug_counter_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_debug_counter_stats_entry */
    of_bsn_debug_counter_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_debug_counter_stats_entry */

    of_bsn_debug_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_stats_entry_append_bind(list,
            (of_bsn_debug_counter_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_debug_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_stats_entry_append_bind(list,
            (of_bsn_debug_counter_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_debug_counter_stats_entry generated by
 * list_setup_of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3(
    of_list_bsn_debug_counter_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_stats_entry_t elt;
    /* No subclasses for of_bsn_debug_counter_stats_entry */
    of_bsn_debug_counter_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_debug_counter_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_STATS_ENTRY);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_debug_counter_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_STATS_ENTRY);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_debug_counter_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_flow_checksum_bucket_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *list, int value)
{

    of_bsn_flow_checksum_bucket_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_flow_checksum_bucket_stats_entry */
    of_bsn_flow_checksum_bucket_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_flow_checksum_bucket_stats_entry */

    of_bsn_flow_checksum_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_flow_checksum_bucket_stats_entry_append_bind(list,
            (of_bsn_flow_checksum_bucket_stats_entry_t *)elt_p);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_flow_checksum_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_flow_checksum_bucket_stats_entry_append_bind(list,
            (of_bsn_flow_checksum_bucket_stats_entry_t *)elt_p);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_flow_checksum_bucket_stats_entry generated by
 * list_setup_of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *list, int value)
{

    of_bsn_flow_checksum_bucket_stats_entry_t elt;
    /* No subclasses for of_bsn_flow_checksum_bucket_stats_entry */
    of_bsn_flow_checksum_bucket_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_flow_checksum_bucket_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_FLOW_CHECKSUM_BUCKET_STATS_ENTRY);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_flow_checksum_bucket_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_FLOW_CHECKSUM_BUCKET_STATS_ENTRY);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_flow_checksum_bucket_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_gentable_bucket_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_bucket_stats_entry_t *list, int value)
{

    of_bsn_gentable_bucket_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_bucket_stats_entry */
    of_bsn_gentable_bucket_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_bucket_stats_entry */

    of_bsn_gentable_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_bucket_stats_entry_append_bind(list,
            (of_bsn_gentable_bucket_stats_entry_t *)elt_p);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_bucket_stats_entry_append_bind(list,
            (of_bsn_gentable_bucket_stats_entry_t *)elt_p);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_bucket_stats_entry generated by
 * list_setup_of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_bucket_stats_entry_t *list, int value)
{

    of_bsn_gentable_bucket_stats_entry_t elt;
    /* No subclasses for of_bsn_gentable_bucket_stats_entry */
    of_bsn_gentable_bucket_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_bucket_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_BUCKET_STATS_ENTRY);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_bucket_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_BUCKET_STATS_ENTRY);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_bucket_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_gentable_desc_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_desc_stats_entry */
    of_bsn_gentable_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_desc_stats_entry */

    of_bsn_gentable_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_desc_stats_entry generated by
 * list_setup_of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_desc_stats_entry_t elt;
    /* No subclasses for of_bsn_gentable_desc_stats_entry */
    of_bsn_gentable_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_DESC_STATS_ENTRY);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_DESC_STATS_ENTRY);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_gentable_entry_desc_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_entry_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_entry_desc_stats_entry */
    of_bsn_gentable_entry_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_entry_desc_stats_entry */

    of_bsn_gentable_entry_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_entry_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_entry_desc_stats_entry generated by
 * list_setup_of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_entry_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_desc_stats_entry_t elt;
    /* No subclasses for of_bsn_gentable_entry_desc_stats_entry */
    of_bsn_gentable_entry_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_entry_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_DESC_STATS_ENTRY);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_entry_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_DESC_STATS_ENTRY);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_entry_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_gentable_entry_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_entry_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_entry_stats_entry */
    of_bsn_gentable_entry_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_entry_stats_entry */

    of_bsn_gentable_entry_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_entry_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_entry_stats_entry generated by
 * list_setup_of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_entry_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_stats_entry_t elt;
    /* No subclasses for of_bsn_gentable_entry_stats_entry */
    of_bsn_gentable_entry_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_entry_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_STATS_ENTRY);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_entry_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_STATS_ENTRY);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_entry_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_gentable_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_gentable_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_stats_entry_t *list, int value)
{

    of_bsn_gentable_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_stats_entry */
    of_bsn_gentable_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_stats_entry */

    of_bsn_gentable_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_stats_entry_append_bind(list,
            (of_bsn_gentable_stats_entry_t *)elt_p);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_stats_entry_append_bind(list,
            (of_bsn_gentable_stats_entry_t *)elt_p);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_stats_entry generated by
 * list_setup_of_list_bsn_gentable_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_gentable_stats_entry_OF_VERSION_1_3(
    of_list_bsn_gentable_stats_entry_t *list, int value)
{

    of_bsn_gentable_stats_entry_t elt;
    /* No subclasses for of_bsn_gentable_stats_entry */
    of_bsn_gentable_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_STATS_ENTRY);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_STATS_ENTRY);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_interface with two of each type of subclass
 */
int
list_setup_of_list_bsn_interface_OF_VERSION_1_3(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * list_setup_of_list_bsn_interface_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_interface_OF_VERSION_1_3(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_lacp_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_lacp_stats_entry_OF_VERSION_1_3(
    of_list_bsn_lacp_stats_entry_t *list, int value)
{

    of_bsn_lacp_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_lacp_stats_entry */
    of_bsn_lacp_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_lacp_stats_entry */

    of_bsn_lacp_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_lacp_stats_entry_append_bind(list,
            (of_bsn_lacp_stats_entry_t *)elt_p);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_lacp_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_lacp_stats_entry_append_bind(list,
            (of_bsn_lacp_stats_entry_t *)elt_p);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_lacp_stats_entry generated by
 * list_setup_of_list_bsn_lacp_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_lacp_stats_entry_OF_VERSION_1_3(
    of_list_bsn_lacp_stats_entry_t *list, int value)
{

    of_bsn_lacp_stats_entry_t elt;
    /* No subclasses for of_bsn_lacp_stats_entry */
    of_bsn_lacp_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_lacp_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_LACP_STATS_ENTRY);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_lacp_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_LACP_STATS_ENTRY);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_lacp_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_port_counter_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3(
    of_list_bsn_port_counter_stats_entry_t *list, int value)
{

    of_bsn_port_counter_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_port_counter_stats_entry */
    of_bsn_port_counter_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_port_counter_stats_entry */

    of_bsn_port_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_port_counter_stats_entry_append_bind(list,
            (of_bsn_port_counter_stats_entry_t *)elt_p);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_port_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_port_counter_stats_entry_append_bind(list,
            (of_bsn_port_counter_stats_entry_t *)elt_p);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_port_counter_stats_entry generated by
 * list_setup_of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3(
    of_list_bsn_port_counter_stats_entry_t *list, int value)
{

    of_bsn_port_counter_stats_entry_t elt;
    /* No subclasses for of_bsn_port_counter_stats_entry */
    of_bsn_port_counter_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_port_counter_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_PORT_COUNTER_STATS_ENTRY);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_port_counter_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_PORT_COUNTER_STATS_ENTRY);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_port_counter_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_switch_pipeline_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3(
    of_list_bsn_switch_pipeline_stats_entry_t *list, int value)
{

    of_bsn_switch_pipeline_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_switch_pipeline_stats_entry */
    of_bsn_switch_pipeline_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_switch_pipeline_stats_entry */

    of_bsn_switch_pipeline_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_switch_pipeline_stats_entry_append_bind(list,
            (of_bsn_switch_pipeline_stats_entry_t *)elt_p);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_switch_pipeline_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_switch_pipeline_stats_entry_append_bind(list,
            (of_bsn_switch_pipeline_stats_entry_t *)elt_p);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_switch_pipeline_stats_entry generated by
 * list_setup_of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3(
    of_list_bsn_switch_pipeline_stats_entry_t *list, int value)
{

    of_bsn_switch_pipeline_stats_entry_t elt;
    /* No subclasses for of_bsn_switch_pipeline_stats_entry */
    of_bsn_switch_pipeline_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_switch_pipeline_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_SWITCH_PIPELINE_STATS_ENTRY);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_switch_pipeline_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_SWITCH_PIPELINE_STATS_ENTRY);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_switch_pipeline_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_table_checksum_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3(
    of_list_bsn_table_checksum_stats_entry_t *list, int value)
{

    of_bsn_table_checksum_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_table_checksum_stats_entry */
    of_bsn_table_checksum_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_table_checksum_stats_entry */

    of_bsn_table_checksum_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_table_checksum_stats_entry_append_bind(list,
            (of_bsn_table_checksum_stats_entry_t *)elt_p);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_table_checksum_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_table_checksum_stats_entry_append_bind(list,
            (of_bsn_table_checksum_stats_entry_t *)elt_p);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_table_checksum_stats_entry generated by
 * list_setup_of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3(
    of_list_bsn_table_checksum_stats_entry_t *list, int value)
{

    of_bsn_table_checksum_stats_entry_t elt;
    /* No subclasses for of_bsn_table_checksum_stats_entry */
    of_bsn_table_checksum_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_table_checksum_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_TABLE_CHECKSUM_STATS_ENTRY);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_table_checksum_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_TABLE_CHECKSUM_STATS_ENTRY);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_table_checksum_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_tlv with two of each type of subclass
 */
int
list_setup_of_list_bsn_tlv_OF_VERSION_1_3(
    of_list_bsn_tlv_t *list, int value)
{

    of_bsn_tlv_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_bsn_tlv_rx_packets_t *rx_packets;
    of_bsn_tlv_reply_packets_t *reply_packets;
    of_bsn_tlv_udf_length_t *udf_length;
    of_bsn_tlv_udf_id_t *udf_id;
    of_bsn_tlv_circuit_id_t *circuit_id;
    of_bsn_tlv_request_packets_t *request_packets;
    of_bsn_tlv_unicast_query_timeout_t *unicast_query_timeout;
    of_bsn_tlv_idle_time_t *idle_time;
    of_bsn_tlv_idle_timeout_t *idle_timeout;
    of_bsn_tlv_miss_packets_t *miss_packets;
    of_bsn_tlv_mac_t *mac;
    of_bsn_tlv_ipv4_t *ipv4;
    of_bsn_tlv_vlan_vid_t *vlan_vid;
    of_bsn_tlv_idle_notification_t *idle_notification;
    of_bsn_tlv_broadcast_query_timeout_t *broadcast_query_timeout;
    of_bsn_tlv_tx_packets_t *tx_packets;
    of_bsn_tlv_port_t *port;
    of_bsn_tlv_udf_offset_t *udf_offset;
    of_bsn_tlv_udf_anchor_t *udf_anchor;

    /* Instantiate pointers for each subclass */
    rx_packets = &elt.rx_packets;
    reply_packets = &elt.reply_packets;
    udf_length = &elt.udf_length;
    udf_id = &elt.udf_id;
    circuit_id = &elt.circuit_id;
    request_packets = &elt.request_packets;
    unicast_query_timeout = &elt.unicast_query_timeout;
    idle_time = &elt.idle_time;
    idle_timeout = &elt.idle_timeout;
    miss_packets = &elt.miss_packets;
    mac = &elt.mac;
    ipv4 = &elt.ipv4;
    vlan_vid = &elt.vlan_vid;
    idle_notification = &elt.idle_notification;
    broadcast_query_timeout = &elt.broadcast_query_timeout;
    tx_packets = &elt.tx_packets;
    port = &elt.port;
    udf_offset = &elt.udf_offset;
    udf_anchor = &elt.udf_anchor;

    /* Append two instances of type of_bsn_tlv_rx_packets */

    of_bsn_tlv_rx_packets_init(rx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)rx_packets);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_populate(
        rx_packets, value);
    cur_len += rx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_rx_packets_init(rx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)rx_packets);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_populate(
        rx_packets, value);
    cur_len += rx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_reply_packets */

    of_bsn_tlv_reply_packets_init(reply_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)reply_packets);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_populate(
        reply_packets, value);
    cur_len += reply_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_reply_packets_init(reply_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)reply_packets);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_populate(
        reply_packets, value);
    cur_len += reply_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_length */

    of_bsn_tlv_udf_length_init(udf_length, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_length);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_populate(
        udf_length, value);
    cur_len += udf_length->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_length_init(udf_length, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_length);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_populate(
        udf_length, value);
    cur_len += udf_length->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_id */

    of_bsn_tlv_udf_id_init(udf_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_id);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_populate(
        udf_id, value);
    cur_len += udf_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_id_init(udf_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_id);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_populate(
        udf_id, value);
    cur_len += udf_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_circuit_id */

    of_bsn_tlv_circuit_id_init(circuit_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)circuit_id);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_populate(
        circuit_id, value);
    cur_len += circuit_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_circuit_id_init(circuit_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)circuit_id);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_populate(
        circuit_id, value);
    cur_len += circuit_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_request_packets */

    of_bsn_tlv_request_packets_init(request_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)request_packets);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_populate(
        request_packets, value);
    cur_len += request_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_request_packets_init(request_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)request_packets);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_populate(
        request_packets, value);
    cur_len += request_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_unicast_query_timeout */

    of_bsn_tlv_unicast_query_timeout_init(unicast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)unicast_query_timeout);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_populate(
        unicast_query_timeout, value);
    cur_len += unicast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_unicast_query_timeout_init(unicast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)unicast_query_timeout);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_populate(
        unicast_query_timeout, value);
    cur_len += unicast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_idle_time */

    of_bsn_tlv_idle_time_init(idle_time, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_time);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_populate(
        idle_time, value);
    cur_len += idle_time->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_idle_time_init(idle_time, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_time);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_populate(
        idle_time, value);
    cur_len += idle_time->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_idle_timeout */

    of_bsn_tlv_idle_timeout_init(idle_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_timeout);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_populate(
        idle_timeout, value);
    cur_len += idle_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_idle_timeout_init(idle_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_timeout);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_populate(
        idle_timeout, value);
    cur_len += idle_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_miss_packets */

    of_bsn_tlv_miss_packets_init(miss_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)miss_packets);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_populate(
        miss_packets, value);
    cur_len += miss_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_miss_packets_init(miss_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)miss_packets);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_populate(
        miss_packets, value);
    cur_len += miss_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_mac */

    of_bsn_tlv_mac_init(mac, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)mac);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_populate(
        mac, value);
    cur_len += mac->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_mac_init(mac, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)mac);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_populate(
        mac, value);
    cur_len += mac->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_ipv4 */

    of_bsn_tlv_ipv4_init(ipv4, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)ipv4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_populate(
        ipv4, value);
    cur_len += ipv4->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_ipv4_init(ipv4, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)ipv4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_populate(
        ipv4, value);
    cur_len += ipv4->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_vlan_vid */

    of_bsn_tlv_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)vlan_vid);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)vlan_vid);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_idle_notification */

    of_bsn_tlv_idle_notification_init(idle_notification, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_notification);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_populate(
        idle_notification, value);
    cur_len += idle_notification->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_idle_notification_init(idle_notification, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_notification);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_populate(
        idle_notification, value);
    cur_len += idle_notification->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_broadcast_query_timeout */

    of_bsn_tlv_broadcast_query_timeout_init(broadcast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)broadcast_query_timeout);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_populate(
        broadcast_query_timeout, value);
    cur_len += broadcast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_broadcast_query_timeout_init(broadcast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)broadcast_query_timeout);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_populate(
        broadcast_query_timeout, value);
    cur_len += broadcast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_tx_packets */

    of_bsn_tlv_tx_packets_init(tx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)tx_packets);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_populate(
        tx_packets, value);
    cur_len += tx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_tx_packets_init(tx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)tx_packets);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_populate(
        tx_packets, value);
    cur_len += tx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_port */

    of_bsn_tlv_port_init(port, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)port);
    value = of_bsn_tlv_port_OF_VERSION_1_3_populate(
        port, value);
    cur_len += port->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_port_init(port, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)port);
    value = of_bsn_tlv_port_OF_VERSION_1_3_populate(
        port, value);
    cur_len += port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_offset */

    of_bsn_tlv_udf_offset_init(udf_offset, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_offset);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_populate(
        udf_offset, value);
    cur_len += udf_offset->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_offset_init(udf_offset, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_offset);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_populate(
        udf_offset, value);
    cur_len += udf_offset->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_anchor */

    of_bsn_tlv_udf_anchor_init(udf_anchor, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_anchor);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_populate(
        udf_anchor, value);
    cur_len += udf_anchor->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_anchor_init(udf_anchor, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_anchor);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_populate(
        udf_anchor, value);
    cur_len += udf_anchor->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_tlv generated by
 * list_setup_of_list_bsn_tlv_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_tlv_OF_VERSION_1_3(
    of_list_bsn_tlv_t *list, int value)
{

    of_bsn_tlv_t elt;
    /* Declare pointers for each subclass */
    of_bsn_tlv_rx_packets_t *rx_packets;
    of_bsn_tlv_reply_packets_t *reply_packets;
    of_bsn_tlv_udf_length_t *udf_length;
    of_bsn_tlv_udf_id_t *udf_id;
    of_bsn_tlv_circuit_id_t *circuit_id;
    of_bsn_tlv_request_packets_t *request_packets;
    of_bsn_tlv_unicast_query_timeout_t *unicast_query_timeout;
    of_bsn_tlv_idle_time_t *idle_time;
    of_bsn_tlv_idle_timeout_t *idle_timeout;
    of_bsn_tlv_miss_packets_t *miss_packets;
    of_bsn_tlv_mac_t *mac;
    of_bsn_tlv_ipv4_t *ipv4;
    of_bsn_tlv_vlan_vid_t *vlan_vid;
    of_bsn_tlv_idle_notification_t *idle_notification;
    of_bsn_tlv_broadcast_query_timeout_t *broadcast_query_timeout;
    of_bsn_tlv_tx_packets_t *tx_packets;
    of_bsn_tlv_port_t *port;
    of_bsn_tlv_udf_offset_t *udf_offset;
    of_bsn_tlv_udf_anchor_t *udf_anchor;

    /* Instantiate pointers for each subclass */
    rx_packets = &elt.rx_packets;
    reply_packets = &elt.reply_packets;
    udf_length = &elt.udf_length;
    udf_id = &elt.udf_id;
    circuit_id = &elt.circuit_id;
    request_packets = &elt.request_packets;
    unicast_query_timeout = &elt.unicast_query_timeout;
    idle_time = &elt.idle_time;
    idle_timeout = &elt.idle_timeout;
    miss_packets = &elt.miss_packets;
    mac = &elt.mac;
    ipv4 = &elt.ipv4;
    vlan_vid = &elt.vlan_vid;
    idle_notification = &elt.idle_notification;
    broadcast_query_timeout = &elt.broadcast_query_timeout;
    tx_packets = &elt.tx_packets;
    port = &elt.port;
    udf_offset = &elt.udf_offset;
    udf_anchor = &elt.udf_anchor;
    TEST_OK(of_list_bsn_tlv_first(list, &elt));

    /* Check two instances of type rx_packets */
    TEST_ASSERT(rx_packets->length == 12);
    if (rx_packets->wire_length_get != NULL) {
        int length;

        rx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(rx_packets->object_id == OF_BSN_TLV_RX_PACKETS);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_check(
        rx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(rx_packets->length == 12);
    if (rx_packets->wire_length_get != NULL) {
        int length;

        rx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(rx_packets->object_id == OF_BSN_TLV_RX_PACKETS);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_check(
        rx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type reply_packets */
    TEST_ASSERT(reply_packets->length == 12);
    if (reply_packets->wire_length_get != NULL) {
        int length;

        reply_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(reply_packets->object_id == OF_BSN_TLV_REPLY_PACKETS);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_check(
        reply_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(reply_packets->length == 12);
    if (reply_packets->wire_length_get != NULL) {
        int length;

        reply_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(reply_packets->object_id == OF_BSN_TLV_REPLY_PACKETS);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_check(
        reply_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_length */
    TEST_ASSERT(udf_length->length == 6);
    if (udf_length->wire_length_get != NULL) {
        int length;

        udf_length->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_length->object_id == OF_BSN_TLV_UDF_LENGTH);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_check(
        udf_length, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_length->length == 6);
    if (udf_length->wire_length_get != NULL) {
        int length;

        udf_length->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_length->object_id == OF_BSN_TLV_UDF_LENGTH);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_check(
        udf_length, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_id */
    TEST_ASSERT(udf_id->length == 6);
    if (udf_id->wire_length_get != NULL) {
        int length;

        udf_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_id->object_id == OF_BSN_TLV_UDF_ID);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_check(
        udf_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_id->length == 6);
    if (udf_id->wire_length_get != NULL) {
        int length;

        udf_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_id->object_id == OF_BSN_TLV_UDF_ID);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_check(
        udf_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type circuit_id */
    TEST_ASSERT(circuit_id->object_id == OF_BSN_TLV_CIRCUIT_ID);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_check(
        circuit_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(circuit_id->object_id == OF_BSN_TLV_CIRCUIT_ID);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_check(
        circuit_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type request_packets */
    TEST_ASSERT(request_packets->length == 12);
    if (request_packets->wire_length_get != NULL) {
        int length;

        request_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(request_packets->object_id == OF_BSN_TLV_REQUEST_PACKETS);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_check(
        request_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(request_packets->length == 12);
    if (request_packets->wire_length_get != NULL) {
        int length;

        request_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(request_packets->object_id == OF_BSN_TLV_REQUEST_PACKETS);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_check(
        request_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type unicast_query_timeout */
    TEST_ASSERT(unicast_query_timeout->length == 8);
    if (unicast_query_timeout->wire_length_get != NULL) {
        int length;

        unicast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(unicast_query_timeout->object_id == OF_BSN_TLV_UNICAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_check(
        unicast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(unicast_query_timeout->length == 8);
    if (unicast_query_timeout->wire_length_get != NULL) {
        int length;

        unicast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(unicast_query_timeout->object_id == OF_BSN_TLV_UNICAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_check(
        unicast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type idle_time */
    TEST_ASSERT(idle_time->length == 12);
    if (idle_time->wire_length_get != NULL) {
        int length;

        idle_time->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(idle_time->object_id == OF_BSN_TLV_IDLE_TIME);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_check(
        idle_time, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(idle_time->length == 12);
    if (idle_time->wire_length_get != NULL) {
        int length;

        idle_time->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(idle_time->object_id == OF_BSN_TLV_IDLE_TIME);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_check(
        idle_time, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type idle_timeout */
    TEST_ASSERT(idle_timeout->length == 8);
    if (idle_timeout->wire_length_get != NULL) {
        int length;

        idle_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(idle_timeout->object_id == OF_BSN_TLV_IDLE_TIMEOUT);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_check(
        idle_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(idle_timeout->length == 8);
    if (idle_timeout->wire_length_get != NULL) {
        int length;

        idle_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(idle_timeout->object_id == OF_BSN_TLV_IDLE_TIMEOUT);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_check(
        idle_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type miss_packets */
    TEST_ASSERT(miss_packets->length == 12);
    if (miss_packets->wire_length_get != NULL) {
        int length;

        miss_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(miss_packets->object_id == OF_BSN_TLV_MISS_PACKETS);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_check(
        miss_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(miss_packets->length == 12);
    if (miss_packets->wire_length_get != NULL) {
        int length;

        miss_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(miss_packets->object_id == OF_BSN_TLV_MISS_PACKETS);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_check(
        miss_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type mac */
    TEST_ASSERT(mac->length == 10);
    if (mac->wire_length_get != NULL) {
        int length;

        mac->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(mac->object_id == OF_BSN_TLV_MAC);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_check(
        mac, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(mac->length == 10);
    if (mac->wire_length_get != NULL) {
        int length;

        mac->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(mac->object_id == OF_BSN_TLV_MAC);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_check(
        mac, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type ipv4 */
    TEST_ASSERT(ipv4->length == 8);
    if (ipv4->wire_length_get != NULL) {
        int length;

        ipv4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4->object_id == OF_BSN_TLV_IPV4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_check(
        ipv4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(ipv4->length == 8);
    if (ipv4->wire_length_get != NULL) {
        int length;

        ipv4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4->object_id == OF_BSN_TLV_IPV4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_check(
        ipv4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type vlan_vid */
    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_BSN_TLV_VLAN_VID);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_BSN_TLV_VLAN_VID);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type idle_notification */
    TEST_ASSERT(idle_notification->length == 4);
    if (idle_notification->wire_length_get != NULL) {
        int length;

        idle_notification->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(idle_notification->object_id == OF_BSN_TLV_IDLE_NOTIFICATION);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_check(
        idle_notification, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(idle_notification->length == 4);
    if (idle_notification->wire_length_get != NULL) {
        int length;

        idle_notification->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(idle_notification->object_id == OF_BSN_TLV_IDLE_NOTIFICATION);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_check(
        idle_notification, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type broadcast_query_timeout */
    TEST_ASSERT(broadcast_query_timeout->length == 8);
    if (broadcast_query_timeout->wire_length_get != NULL) {
        int length;

        broadcast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(broadcast_query_timeout->object_id == OF_BSN_TLV_BROADCAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_check(
        broadcast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(broadcast_query_timeout->length == 8);
    if (broadcast_query_timeout->wire_length_get != NULL) {
        int length;

        broadcast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(broadcast_query_timeout->object_id == OF_BSN_TLV_BROADCAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_check(
        broadcast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type tx_packets */
    TEST_ASSERT(tx_packets->length == 12);
    if (tx_packets->wire_length_get != NULL) {
        int length;

        tx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tx_packets->object_id == OF_BSN_TLV_TX_PACKETS);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_check(
        tx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(tx_packets->length == 12);
    if (tx_packets->wire_length_get != NULL) {
        int length;

        tx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tx_packets->object_id == OF_BSN_TLV_TX_PACKETS);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_check(
        tx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type port */
    TEST_ASSERT(port->length == 8);
    if (port->wire_length_get != NULL) {
        int length;

        port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(port->object_id == OF_BSN_TLV_PORT);
    value = of_bsn_tlv_port_OF_VERSION_1_3_check(
        port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(port->length == 8);
    if (port->wire_length_get != NULL) {
        int length;

        port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(port->object_id == OF_BSN_TLV_PORT);
    value = of_bsn_tlv_port_OF_VERSION_1_3_check(
        port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_offset */
    TEST_ASSERT(udf_offset->length == 6);
    if (udf_offset->wire_length_get != NULL) {
        int length;

        udf_offset->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_offset->object_id == OF_BSN_TLV_UDF_OFFSET);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_check(
        udf_offset, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_offset->length == 6);
    if (udf_offset->wire_length_get != NULL) {
        int length;

        udf_offset->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_offset->object_id == OF_BSN_TLV_UDF_OFFSET);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_check(
        udf_offset, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_anchor */
    TEST_ASSERT(udf_anchor->length == 6);
    if (udf_anchor->wire_length_get != NULL) {
        int length;

        udf_anchor->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_anchor->object_id == OF_BSN_TLV_UDF_ANCHOR);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_check(
        udf_anchor, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_anchor->length == 6);
    if (udf_anchor->wire_length_get != NULL) {
        int length;

        udf_anchor->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_anchor->object_id == OF_BSN_TLV_UDF_ANCHOR);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_check(
        udf_anchor, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_tlv_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bsn_vlan_counter_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3(
    of_list_bsn_vlan_counter_stats_entry_t *list, int value)
{

    of_bsn_vlan_counter_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_vlan_counter_stats_entry */
    of_bsn_vlan_counter_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_vlan_counter_stats_entry */

    of_bsn_vlan_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_vlan_counter_stats_entry_append_bind(list,
            (of_bsn_vlan_counter_stats_entry_t *)elt_p);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_vlan_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_vlan_counter_stats_entry_append_bind(list,
            (of_bsn_vlan_counter_stats_entry_t *)elt_p);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bsn_vlan_counter_stats_entry generated by
 * list_setup_of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3(
    of_list_bsn_vlan_counter_stats_entry_t *list, int value)
{

    of_bsn_vlan_counter_stats_entry_t elt;
    /* No subclasses for of_bsn_vlan_counter_stats_entry */
    of_bsn_vlan_counter_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_vlan_counter_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_VLAN_COUNTER_STATS_ENTRY);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_vlan_counter_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_VLAN_COUNTER_STATS_ENTRY);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_vlan_counter_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bucket with two of each type of subclass
 */
int
list_setup_of_list_bucket_OF_VERSION_1_3(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket */

    of_bucket_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bucket generated by
 * list_setup_of_list_bucket_OF_VERSION_1_3
 */
int
list_check_of_list_bucket_OF_VERSION_1_3(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_bucket_counter with two of each type of subclass
 */
int
list_setup_of_list_bucket_counter_OF_VERSION_1_3(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket_counter */

    of_bucket_counter_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_counter_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_bucket_counter generated by
 * list_setup_of_list_bucket_counter_OF_VERSION_1_3
 */
int
list_check_of_list_bucket_counter_OF_VERSION_1_3(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_counter_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_counter_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_counter_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_flow_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_flow_stats_entry_OF_VERSION_1_3(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * list_setup_of_list_flow_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_flow_stats_entry_OF_VERSION_1_3(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_group_desc_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_group_desc_stats_entry_OF_VERSION_1_3(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_desc_stats_entry */

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_group_desc_stats_entry generated by
 * list_setup_of_list_group_desc_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_group_desc_stats_entry_OF_VERSION_1_3(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_group_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_group_stats_entry_OF_VERSION_1_3(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_stats_entry */

    of_group_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_group_stats_entry generated by
 * list_setup_of_list_group_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_group_stats_entry_OF_VERSION_1_3(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_hello_elem with two of each type of subclass
 */
int
list_setup_of_list_hello_elem_OF_VERSION_1_3(
    of_list_hello_elem_t *list, int value)
{

    of_hello_elem_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_hello_elem_versionbitmap_t *versionbitmap;

    /* Instantiate pointers for each subclass */
    versionbitmap = &elt.versionbitmap;

    /* Append two instances of type of_hello_elem_versionbitmap */

    of_hello_elem_versionbitmap_init(versionbitmap, OF_VERSION_1_3, -1, 1);
    of_list_hello_elem_append_bind(list,
            (of_hello_elem_t *)versionbitmap);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_populate(
        versionbitmap, value);
    cur_len += versionbitmap->length;
    TEST_ASSERT(list->length == cur_len);

    of_hello_elem_versionbitmap_init(versionbitmap, OF_VERSION_1_3, -1, 1);
    of_list_hello_elem_append_bind(list,
            (of_hello_elem_t *)versionbitmap);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_populate(
        versionbitmap, value);
    cur_len += versionbitmap->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_hello_elem generated by
 * list_setup_of_list_hello_elem_OF_VERSION_1_3
 */
int
list_check_of_list_hello_elem_OF_VERSION_1_3(
    of_list_hello_elem_t *list, int value)
{

    of_hello_elem_t elt;
    /* Declare pointers for each subclass */
    of_hello_elem_versionbitmap_t *versionbitmap;

    /* Instantiate pointers for each subclass */
    versionbitmap = &elt.versionbitmap;
    TEST_OK(of_list_hello_elem_first(list, &elt));

    /* Check two instances of type versionbitmap */
    TEST_ASSERT(versionbitmap->object_id == OF_HELLO_ELEM_VERSIONBITMAP);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_check(
        versionbitmap, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_hello_elem_next(list, &elt));

    TEST_ASSERT(versionbitmap->object_id == OF_HELLO_ELEM_VERSIONBITMAP);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_check(
        versionbitmap, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_hello_elem_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_instruction with two of each type of subclass
 */
int
list_setup_of_list_instruction_OF_VERSION_1_3(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_write_actions_t *write_actions;
    of_instruction_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_goto_table_t *goto_table;
    of_instruction_meter_t *meter;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_bsn_deny_t *bsn_deny;
    of_instruction_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_write_metadata_t *write_metadata;
    of_instruction_bsn_permit_t *bsn_permit;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    goto_table = &elt.goto_table;
    meter = &elt.meter;
    apply_actions = &elt.apply_actions;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    bsn_permit = &elt.bsn_permit;

    /* Append two instances of type of_instruction_clear_actions */

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_disable_src_mac_check */

    of_instruction_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_src_mac_check);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_src_mac_check);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_actions */

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_arp_offload */

    of_instruction_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_arp_offload);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_arp_offload);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_disable_split_horizon_check */

    of_instruction_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_split_horizon_check);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_split_horizon_check);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_goto_table */

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_meter */

    of_instruction_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)meter);
    value = of_instruction_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)meter);
    value = of_instruction_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_apply_actions */

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_deny */

    of_instruction_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_deny);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_deny);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_dhcp_offload */

    of_instruction_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_dhcp_offload);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_dhcp_offload);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_packet_of_death */

    of_instruction_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_packet_of_death);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_packet_of_death);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_metadata */

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_permit */

    of_instruction_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_permit);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_permit);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_instruction generated by
 * list_setup_of_list_instruction_OF_VERSION_1_3
 */
int
list_check_of_list_instruction_OF_VERSION_1_3(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_write_actions_t *write_actions;
    of_instruction_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_goto_table_t *goto_table;
    of_instruction_meter_t *meter;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_bsn_deny_t *bsn_deny;
    of_instruction_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_write_metadata_t *write_metadata;
    of_instruction_bsn_permit_t *bsn_permit;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    goto_table = &elt.goto_table;
    meter = &elt.meter;
    apply_actions = &elt.apply_actions;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    bsn_permit = &elt.bsn_permit;
    TEST_OK(of_list_instruction_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_disable_src_mac_check */
    TEST_ASSERT(bsn_disable_src_mac_check->length == 16);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_disable_src_mac_check->length == 16);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_arp_offload */
    TEST_ASSERT(bsn_arp_offload->length == 16);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_BSN_ARP_OFFLOAD);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_arp_offload->length == 16);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_BSN_ARP_OFFLOAD);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_disable_split_horizon_check */
    TEST_ASSERT(bsn_disable_split_horizon_check->length == 16);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_disable_split_horizon_check->length == 16);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type meter */
    TEST_ASSERT(meter->length == 8);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_METER);
    value = of_instruction_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(meter->length == 8);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_METER);
    value = of_instruction_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_deny */
    TEST_ASSERT(bsn_deny->length == 16);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_BSN_DENY);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_deny->length == 16);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_BSN_DENY);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_dhcp_offload */
    TEST_ASSERT(bsn_dhcp_offload->length == 16);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_BSN_DHCP_OFFLOAD);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_dhcp_offload->length == 16);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_BSN_DHCP_OFFLOAD);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_packet_of_death */
    TEST_ASSERT(bsn_packet_of_death->length == 16);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_BSN_PACKET_OF_DEATH);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_packet_of_death->length == 16);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_BSN_PACKET_OF_DEATH);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_permit */
    TEST_ASSERT(bsn_permit->length == 16);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_BSN_PERMIT);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_permit->length == 16);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_BSN_PERMIT);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_instruction_id with two of each type of subclass
 */
int
list_setup_of_list_instruction_id_OF_VERSION_1_3(
    of_list_instruction_id_t *list, int value)
{

    of_instruction_id_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_id_clear_actions_t *clear_actions;
    of_instruction_id_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_id_write_actions_t *write_actions;
    of_instruction_id_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_id_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_id_apply_actions_t *apply_actions;
    of_instruction_id_meter_t *meter;
    of_instruction_id_bsn_permit_t *bsn_permit;
    of_instruction_id_bsn_deny_t *bsn_deny;
    of_instruction_id_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_id_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_id_write_metadata_t *write_metadata;
    of_instruction_id_goto_table_t *goto_table;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    apply_actions = &elt.apply_actions;
    meter = &elt.meter;
    bsn_permit = &elt.bsn_permit;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    goto_table = &elt.goto_table;

    /* Append two instances of type of_instruction_id_clear_actions */

    of_instruction_id_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)clear_actions);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)clear_actions);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_disable_src_mac_check */

    of_instruction_id_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_src_mac_check);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_src_mac_check);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_write_actions */

    of_instruction_id_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_actions);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_actions);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_arp_offload */

    of_instruction_id_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_arp_offload);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_arp_offload);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_disable_split_horizon_check */

    of_instruction_id_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_split_horizon_check);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_split_horizon_check);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_apply_actions */

    of_instruction_id_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)apply_actions);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)apply_actions);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_meter */

    of_instruction_id_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)meter);
    value = of_instruction_id_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)meter);
    value = of_instruction_id_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_permit */

    of_instruction_id_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_permit);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_permit);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_deny */

    of_instruction_id_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_deny);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_deny);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_dhcp_offload */

    of_instruction_id_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_dhcp_offload);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_dhcp_offload);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_packet_of_death */

    of_instruction_id_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_packet_of_death);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_packet_of_death);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_write_metadata */

    of_instruction_id_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_metadata);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_metadata);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_goto_table */

    of_instruction_id_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)goto_table);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)goto_table);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_instruction_id generated by
 * list_setup_of_list_instruction_id_OF_VERSION_1_3
 */
int
list_check_of_list_instruction_id_OF_VERSION_1_3(
    of_list_instruction_id_t *list, int value)
{

    of_instruction_id_t elt;
    /* Declare pointers for each subclass */
    of_instruction_id_clear_actions_t *clear_actions;
    of_instruction_id_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_id_write_actions_t *write_actions;
    of_instruction_id_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_id_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_id_apply_actions_t *apply_actions;
    of_instruction_id_meter_t *meter;
    of_instruction_id_bsn_permit_t *bsn_permit;
    of_instruction_id_bsn_deny_t *bsn_deny;
    of_instruction_id_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_id_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_id_write_metadata_t *write_metadata;
    of_instruction_id_goto_table_t *goto_table;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    apply_actions = &elt.apply_actions;
    meter = &elt.meter;
    bsn_permit = &elt.bsn_permit;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    goto_table = &elt.goto_table;
    TEST_OK(of_list_instruction_id_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 4);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_ID_CLEAR_ACTIONS);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 4);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_ID_CLEAR_ACTIONS);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_disable_src_mac_check */
    TEST_ASSERT(bsn_disable_src_mac_check->length == 12);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_disable_src_mac_check->length == 12);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->length == 4);
    if (write_actions->wire_length_get != NULL) {
        int length;

        write_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_ID_WRITE_ACTIONS);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(write_actions->length == 4);
    if (write_actions->wire_length_get != NULL) {
        int length;

        write_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_ID_WRITE_ACTIONS);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_arp_offload */
    TEST_ASSERT(bsn_arp_offload->length == 12);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_ID_BSN_ARP_OFFLOAD);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_arp_offload->length == 12);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_ID_BSN_ARP_OFFLOAD);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_disable_split_horizon_check */
    TEST_ASSERT(bsn_disable_split_horizon_check->length == 12);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_disable_split_horizon_check->length == 12);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->length == 4);
    if (apply_actions->wire_length_get != NULL) {
        int length;

        apply_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_ID_APPLY_ACTIONS);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(apply_actions->length == 4);
    if (apply_actions->wire_length_get != NULL) {
        int length;

        apply_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_ID_APPLY_ACTIONS);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type meter */
    TEST_ASSERT(meter->length == 4);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_ID_METER);
    value = of_instruction_id_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(meter->length == 4);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_ID_METER);
    value = of_instruction_id_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_permit */
    TEST_ASSERT(bsn_permit->length == 12);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_ID_BSN_PERMIT);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_permit->length == 12);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_ID_BSN_PERMIT);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_deny */
    TEST_ASSERT(bsn_deny->length == 12);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_ID_BSN_DENY);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_deny->length == 12);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_ID_BSN_DENY);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_dhcp_offload */
    TEST_ASSERT(bsn_dhcp_offload->length == 12);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_ID_BSN_DHCP_OFFLOAD);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_dhcp_offload->length == 12);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_ID_BSN_DHCP_OFFLOAD);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_packet_of_death */
    TEST_ASSERT(bsn_packet_of_death->length == 12);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_ID_BSN_PACKET_OF_DEATH);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_packet_of_death->length == 12);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_ID_BSN_PACKET_OF_DEATH);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 4);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_ID_WRITE_METADATA);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 4);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_ID_WRITE_METADATA);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 4);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_ID_GOTO_TABLE);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(goto_table->length == 4);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_ID_GOTO_TABLE);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_id_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_meter_band with two of each type of subclass
 */
int
list_setup_of_list_meter_band_OF_VERSION_1_3(
    of_list_meter_band_t *list, int value)
{

    of_meter_band_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_meter_band_dscp_remark_t *dscp_remark;
    of_meter_band_drop_t *drop;
    of_meter_band_ofdpa_color_set_t *ofdpa_color_set;

    /* Instantiate pointers for each subclass */
    dscp_remark = &elt.dscp_remark;
    drop = &elt.drop;
    ofdpa_color_set = &elt.ofdpa_color_set;

    /* Append two instances of type of_meter_band_dscp_remark */

    of_meter_band_dscp_remark_init(dscp_remark, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)dscp_remark);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_populate(
        dscp_remark, value);
    cur_len += dscp_remark->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_dscp_remark_init(dscp_remark, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)dscp_remark);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_populate(
        dscp_remark, value);
    cur_len += dscp_remark->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_meter_band_drop */

    of_meter_band_drop_init(drop, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)drop);
    value = of_meter_band_drop_OF_VERSION_1_3_populate(
        drop, value);
    cur_len += drop->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_drop_init(drop, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)drop);
    value = of_meter_band_drop_OF_VERSION_1_3_populate(
        drop, value);
    cur_len += drop->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_meter_band_ofdpa_color_set */

    of_meter_band_ofdpa_color_set_init(ofdpa_color_set, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)ofdpa_color_set);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_populate(
        ofdpa_color_set, value);
    cur_len += ofdpa_color_set->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_ofdpa_color_set_init(ofdpa_color_set, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)ofdpa_color_set);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_populate(
        ofdpa_color_set, value);
    cur_len += ofdpa_color_set->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_meter_band generated by
 * list_setup_of_list_meter_band_OF_VERSION_1_3
 */
int
list_check_of_list_meter_band_OF_VERSION_1_3(
    of_list_meter_band_t *list, int value)
{

    of_meter_band_t elt;
    /* Declare pointers for each subclass */
    of_meter_band_dscp_remark_t *dscp_remark;
    of_meter_band_drop_t *drop;
    of_meter_band_ofdpa_color_set_t *ofdpa_color_set;

    /* Instantiate pointers for each subclass */
    dscp_remark = &elt.dscp_remark;
    drop = &elt.drop;
    ofdpa_color_set = &elt.ofdpa_color_set;
    TEST_OK(of_list_meter_band_first(list, &elt));

    /* Check two instances of type dscp_remark */
    TEST_ASSERT(dscp_remark->length == 16);
    if (dscp_remark->wire_length_get != NULL) {
        int length;

        dscp_remark->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(dscp_remark->object_id == OF_METER_BAND_DSCP_REMARK);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_check(
        dscp_remark, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    TEST_ASSERT(dscp_remark->length == 16);
    if (dscp_remark->wire_length_get != NULL) {
        int length;

        dscp_remark->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(dscp_remark->object_id == OF_METER_BAND_DSCP_REMARK);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_check(
        dscp_remark, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    /* Check two instances of type drop */
    TEST_ASSERT(drop->length == 16);
    if (drop->wire_length_get != NULL) {
        int length;

        drop->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(drop->object_id == OF_METER_BAND_DROP);
    value = of_meter_band_drop_OF_VERSION_1_3_check(
        drop, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    TEST_ASSERT(drop->length == 16);
    if (drop->wire_length_get != NULL) {
        int length;

        drop->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(drop->object_id == OF_METER_BAND_DROP);
    value = of_meter_band_drop_OF_VERSION_1_3_check(
        drop, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    /* Check two instances of type ofdpa_color_set */
    TEST_ASSERT(ofdpa_color_set->length == 42);
    if (ofdpa_color_set->wire_length_get != NULL) {
        int length;

        ofdpa_color_set->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 42);
    }

    TEST_ASSERT(ofdpa_color_set->object_id == OF_METER_BAND_OFDPA_COLOR_SET);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_check(
        ofdpa_color_set, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    TEST_ASSERT(ofdpa_color_set->length == 42);
    if (ofdpa_color_set->wire_length_get != NULL) {
        int length;

        ofdpa_color_set->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 42);
    }

    TEST_ASSERT(ofdpa_color_set->object_id == OF_METER_BAND_OFDPA_COLOR_SET);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_check(
        ofdpa_color_set, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_meter_band_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_meter_band_stats with two of each type of subclass
 */
int
list_setup_of_list_meter_band_stats_OF_VERSION_1_3(
    of_list_meter_band_stats_t *list, int value)
{

    of_meter_band_stats_t elt;
    int cur_len = 0;
    /* No subclasses for of_meter_band_stats */
    of_meter_band_stats_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_meter_band_stats */

    of_meter_band_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_stats_append_bind(list,
            (of_meter_band_stats_t *)elt_p);
    value = of_meter_band_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_stats_append_bind(list,
            (of_meter_band_stats_t *)elt_p);
    value = of_meter_band_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_meter_band_stats generated by
 * list_setup_of_list_meter_band_stats_OF_VERSION_1_3
 */
int
list_check_of_list_meter_band_stats_OF_VERSION_1_3(
    of_list_meter_band_stats_t *list, int value)
{

    of_meter_band_stats_t elt;
    /* No subclasses for of_meter_band_stats */
    of_meter_band_stats_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_meter_band_stats_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_METER_BAND_STATS);
    value = of_meter_band_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_stats_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_METER_BAND_STATS);
    value = of_meter_band_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_meter_band_stats_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_meter_stats with two of each type of subclass
 */
int
list_setup_of_list_meter_stats_OF_VERSION_1_3(
    of_list_meter_stats_t *list, int value)
{

    of_meter_stats_t elt;
    int cur_len = 0;
    /* No subclasses for of_meter_stats */
    of_meter_stats_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_meter_stats */

    of_meter_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_stats_append_bind(list,
            (of_meter_stats_t *)elt_p);
    value = of_meter_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_stats_append_bind(list,
            (of_meter_stats_t *)elt_p);
    value = of_meter_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_meter_stats generated by
 * list_setup_of_list_meter_stats_OF_VERSION_1_3
 */
int
list_check_of_list_meter_stats_OF_VERSION_1_3(
    of_list_meter_stats_t *list, int value)
{

    of_meter_stats_t elt;
    /* No subclasses for of_meter_stats */
    of_meter_stats_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_meter_stats_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_METER_STATS);
    value = of_meter_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_stats_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_METER_STATS);
    value = of_meter_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_meter_stats_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_oxm with two of each type of subclass
 */
int
list_setup_of_list_oxm_OF_VERSION_1_3(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_oxm_ofdpa_mpls_ttl_t *ofdpa_mpls_ttl;
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_ofdpa_color_actions_index_t *ofdpa_color_actions_index;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_ofdpa_qos_index_t *ofdpa_qos_index;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_ofdpa_rxfcl_t *ofdpa_rxfcl;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_ofdpa_bfd_discriminator_masked_t *ofdpa_bfd_discriminator_masked;
    of_oxm_ofdpa_actset_output_t *ofdpa_actset_output;
    of_oxm_ofdpa_dei_t *ofdpa_dei;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_ofdpa_oam_y1731_mdl_t *ofdpa_oam_y1731_mdl;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_ofdpa_bfd_discriminator_t *ofdpa_bfd_discriminator;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ofdpa_mpls_data_first_nibble_t *ofdpa_mpls_data_first_nibble;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ofdpa_mpls_l2_port_masked_t *ofdpa_mpls_l2_port_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_ofdpa_protection_index_t *ofdpa_protection_index;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_ofdpa_color_t *ofdpa_color;
    of_oxm_ofdpa_vrf_t *ofdpa_vrf;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_ofdpa_lmep_id_t *ofdpa_lmep_id;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_ofdpa_mpls_next_label_is_gal_t *ofdpa_mpls_next_label_is_gal;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_ofdpa_rx_timestamp_t *ofdpa_rx_timestamp;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ofdpa_mpls_l2_port_t *ofdpa_mpls_l2_port;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_ofdpa_txfcl_t *ofdpa_txfcl;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_ofdpa_ovid_t *ofdpa_ovid;
    of_oxm_ofdpa_traffic_class_t *ofdpa_traffic_class;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_ofdpa_l3_in_port_t *ofdpa_l3_in_port;
    of_oxm_ofdpa_oam_y1731_opcode_t *ofdpa_oam_y1731_opcode;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_ofdpa_eth_subtype_t *ofdpa_eth_subtype;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_ofdpa_mpls_ach_channel_t *ofdpa_mpls_ach_channel;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    ofdpa_mpls_ttl = &elt.ofdpa_mpls_ttl;
    mpls_tc_masked = &elt.mpls_tc_masked;
    ofdpa_color_actions_index = &elt.ofdpa_color_actions_index;
    arp_sha = &elt.arp_sha;
    ofdpa_qos_index = &elt.ofdpa_qos_index;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    ofdpa_rxfcl = &elt.ofdpa_rxfcl;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    ofdpa_bfd_discriminator_masked = &elt.ofdpa_bfd_discriminator_masked;
    ofdpa_actset_output = &elt.ofdpa_actset_output;
    ofdpa_dei = &elt.ofdpa_dei;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    ofdpa_oam_y1731_mdl = &elt.ofdpa_oam_y1731_mdl;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    ofdpa_bfd_discriminator = &elt.ofdpa_bfd_discriminator;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ofdpa_mpls_data_first_nibble = &elt.ofdpa_mpls_data_first_nibble;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ofdpa_mpls_l2_port_masked = &elt.ofdpa_mpls_l2_port_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    ofdpa_protection_index = &elt.ofdpa_protection_index;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    ofdpa_color = &elt.ofdpa_color;
    ofdpa_vrf = &elt.ofdpa_vrf;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    ofdpa_lmep_id = &elt.ofdpa_lmep_id;
    in_phy_port_masked = &elt.in_phy_port_masked;
    ofdpa_mpls_next_label_is_gal = &elt.ofdpa_mpls_next_label_is_gal;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    ofdpa_rx_timestamp = &elt.ofdpa_rx_timestamp;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ofdpa_mpls_l2_port = &elt.ofdpa_mpls_l2_port;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    ofdpa_txfcl = &elt.ofdpa_txfcl;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    ofdpa_ovid = &elt.ofdpa_ovid;
    ofdpa_traffic_class = &elt.ofdpa_traffic_class;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    ofdpa_l3_in_port = &elt.ofdpa_l3_in_port;
    ofdpa_oam_y1731_opcode = &elt.ofdpa_oam_y1731_opcode;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    ofdpa_eth_subtype = &elt.ofdpa_eth_subtype;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    ofdpa_mpls_ach_channel = &elt.ofdpa_mpls_ach_channel;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;

    /* Append two instances of type of_oxm_ofdpa_mpls_ttl */

    of_oxm_ofdpa_mpls_ttl_init(ofdpa_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ttl);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_populate(
        ofdpa_mpls_ttl, value);
    cur_len += ofdpa_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_ttl_init(ofdpa_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ttl);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_populate(
        ofdpa_mpls_ttl, value);
    cur_len += ofdpa_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_tc_masked */

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_color_actions_index */

    of_oxm_ofdpa_color_actions_index_init(ofdpa_color_actions_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color_actions_index);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_populate(
        ofdpa_color_actions_index, value);
    cur_len += ofdpa_color_actions_index->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_color_actions_index_init(ofdpa_color_actions_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color_actions_index);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_populate(
        ofdpa_color_actions_index, value);
    cur_len += ofdpa_color_actions_index->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha */

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_3_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_3_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_qos_index */

    of_oxm_ofdpa_qos_index_init(ofdpa_qos_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_qos_index);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_populate(
        ofdpa_qos_index, value);
    cur_len += ofdpa_qos_index->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_qos_index_init(ofdpa_qos_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_qos_index);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_populate(
        ofdpa_qos_index, value);
    cur_len += ofdpa_qos_index->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src_masked */

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst */

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_3_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_3_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_rxfcl */

    of_oxm_ofdpa_rxfcl_init(ofdpa_rxfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rxfcl);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_populate(
        ofdpa_rxfcl, value);
    cur_len += ofdpa_rxfcl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_rxfcl_init(ofdpa_rxfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rxfcl);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_populate(
        ofdpa_rxfcl, value);
    cur_len += ofdpa_rxfcl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id */

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id */

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op */

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_3_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_3_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type */

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_3_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_3_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_bfd_discriminator_masked */

    of_oxm_ofdpa_bfd_discriminator_masked_init(ofdpa_bfd_discriminator_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator_masked);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator_masked, value);
    cur_len += ofdpa_bfd_discriminator_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_bfd_discriminator_masked_init(ofdpa_bfd_discriminator_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator_masked);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator_masked, value);
    cur_len += ofdpa_bfd_discriminator_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_actset_output */

    of_oxm_ofdpa_actset_output_init(ofdpa_actset_output, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_actset_output);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_populate(
        ofdpa_actset_output, value);
    cur_len += ofdpa_actset_output->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_actset_output_init(ofdpa_actset_output, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_actset_output);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_populate(
        ofdpa_actset_output, value);
    cur_len += ofdpa_actset_output->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_dei */

    of_oxm_ofdpa_dei_init(ofdpa_dei, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_dei);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_populate(
        ofdpa_dei, value);
    cur_len += ofdpa_dei->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_dei_init(ofdpa_dei, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_dei);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_populate(
        ofdpa_dei, value);
    cur_len += ofdpa_dei->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128_masked */

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos */

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_oam_y1731_mdl */

    of_oxm_ofdpa_oam_y1731_mdl_init(ofdpa_oam_y1731_mdl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_mdl);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_mdl, value);
    cur_len += ofdpa_oam_y1731_mdl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_oam_y1731_mdl_init(ofdpa_oam_y1731_mdl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_mdl);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_mdl, value);
    cur_len += ofdpa_oam_y1731_mdl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa */

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_3_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_3_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type_masked */

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha_masked */

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst */

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_3_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_3_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf_masked */

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target */

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type_masked */

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_bfd_discriminator */

    of_oxm_ofdpa_bfd_discriminator_init(ofdpa_bfd_discriminator, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator, value);
    cur_len += ofdpa_bfd_discriminator->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_bfd_discriminator_init(ofdpa_bfd_discriminator, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator, value);
    cur_len += ofdpa_bfd_discriminator->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed_masked */

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6_masked */

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type */

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst_masked */

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst */

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto */

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_3_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_3_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata */

    of_oxm_metadata_init(metadata, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_3_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_init(metadata, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_3_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa_masked */

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_data_first_nibble */

    of_oxm_ofdpa_mpls_data_first_nibble_init(ofdpa_mpls_data_first_nibble, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_data_first_nibble);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_populate(
        ofdpa_mpls_data_first_nibble, value);
    cur_len += ofdpa_mpls_data_first_nibble->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_data_first_nibble_init(ofdpa_mpls_data_first_nibble, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_data_first_nibble);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_populate(
        ofdpa_mpls_data_first_nibble, value);
    cur_len += ofdpa_mpls_data_first_nibble->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel */

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id_masked */

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll_masked */

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa */

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src */

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_3_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_3_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src */

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata_masked */

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_l2_port_masked */

    of_oxm_ofdpa_mpls_l2_port_masked_init(ofdpa_mpls_l2_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port_masked);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port_masked, value);
    cur_len += ofdpa_mpls_l2_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_l2_port_masked_init(ofdpa_mpls_l2_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port_masked);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port_masked, value);
    cur_len += ofdpa_mpls_l2_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll */

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id_masked */

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_protection_index */

    of_oxm_ofdpa_protection_index_init(ofdpa_protection_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_protection_index);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_populate(
        ofdpa_protection_index, value);
    cur_len += ofdpa_protection_index->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_protection_index_init(ofdpa_protection_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_protection_index);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_populate(
        ofdpa_protection_index, value);
    cur_len += ofdpa_protection_index->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id */

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid */

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha */

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_3_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_3_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp */

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7_masked */

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos_masked */

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst */

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id */

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst_masked */

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2_masked */

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port_masked */

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn */

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code_masked */

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst_masked */

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src_masked */

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst */

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst_masked */

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_color */

    of_oxm_ofdpa_color_init(ofdpa_color, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_populate(
        ofdpa_color, value);
    cur_len += ofdpa_color->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_color_init(ofdpa_color, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_populate(
        ofdpa_color, value);
    cur_len += ofdpa_color->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_vrf */

    of_oxm_ofdpa_vrf_init(ofdpa_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_vrf);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_populate(
        ofdpa_vrf, value);
    cur_len += ofdpa_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_vrf_init(ofdpa_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_vrf);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_populate(
        ofdpa_vrf, value);
    cur_len += ofdpa_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1 */

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0 */

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll */

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2 */

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5 */

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4 */

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7 */

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto_masked */

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_lmep_id */

    of_oxm_ofdpa_lmep_id_init(ofdpa_lmep_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_lmep_id);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_populate(
        ofdpa_lmep_id, value);
    cur_len += ofdpa_lmep_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_lmep_id_init(ofdpa_lmep_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_lmep_id);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_populate(
        ofdpa_lmep_id, value);
    cur_len += ofdpa_lmep_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port_masked */

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_next_label_is_gal */

    of_oxm_ofdpa_mpls_next_label_is_gal_init(ofdpa_mpls_next_label_is_gal, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_next_label_is_gal);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_populate(
        ofdpa_mpls_next_label_is_gal, value);
    cur_len += ofdpa_mpls_next_label_is_gal->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_next_label_is_gal_init(ofdpa_mpls_next_label_is_gal, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_next_label_is_gal);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_populate(
        ofdpa_mpls_next_label_is_gal, value);
    cur_len += ofdpa_mpls_next_label_is_gal->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp_masked */

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst_masked */

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp */

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src */

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst */

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src_masked */

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op_masked */

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src */

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_3_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_3_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code */

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5_masked */

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code_masked */

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src_masked */

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn_masked */

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src */

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_3_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_3_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid_masked */

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha_masked */

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_rx_timestamp */

    of_oxm_ofdpa_rx_timestamp_init(ofdpa_rx_timestamp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rx_timestamp);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_populate(
        ofdpa_rx_timestamp, value);
    cur_len += ofdpa_rx_timestamp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_rx_timestamp_init(ofdpa_rx_timestamp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rx_timestamp);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_populate(
        ofdpa_rx_timestamp, value);
    cur_len += ofdpa_rx_timestamp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src_masked */

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port */

    of_oxm_in_port_init(in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_3_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_init(in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_3_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id */

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_l2_port */

    of_oxm_ofdpa_mpls_l2_port_init(ofdpa_mpls_l2_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port, value);
    cur_len += ofdpa_mpls_l2_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_l2_port_init(ofdpa_mpls_l2_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port, value);
    cur_len += ofdpa_mpls_l2_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll_masked */

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src */

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_3_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_3_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target_masked */

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type_masked */

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0_masked */

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_txfcl */

    of_oxm_ofdpa_txfcl_init(ofdpa_txfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_txfcl);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_populate(
        ofdpa_txfcl, value);
    cur_len += ofdpa_txfcl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_txfcl_init(ofdpa_txfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_txfcl);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_populate(
        ofdpa_txfcl, value);
    cur_len += ofdpa_txfcl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3_masked */

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label */

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_3_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_3_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_ovid */

    of_oxm_ofdpa_ovid_init(ofdpa_ovid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_ovid);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_populate(
        ofdpa_ovid, value);
    cur_len += ofdpa_ovid->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_ovid_init(ofdpa_ovid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_ovid);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_populate(
        ofdpa_ovid, value);
    cur_len += ofdpa_ovid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_traffic_class */

    of_oxm_ofdpa_traffic_class_init(ofdpa_traffic_class, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_traffic_class);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_populate(
        ofdpa_traffic_class, value);
    cur_len += ofdpa_traffic_class->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_traffic_class_init(ofdpa_traffic_class, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_traffic_class);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_populate(
        ofdpa_traffic_class, value);
    cur_len += ofdpa_traffic_class->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128 */

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id_masked */

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel_masked */

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id_masked */

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp_masked */

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type */

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id_masked */

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_l3_in_port */

    of_oxm_ofdpa_l3_in_port_init(ofdpa_l3_in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_l3_in_port);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_populate(
        ofdpa_l3_in_port, value);
    cur_len += ofdpa_l3_in_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_l3_in_port_init(ofdpa_l3_in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_l3_in_port);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_populate(
        ofdpa_l3_in_port, value);
    cur_len += ofdpa_l3_in_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_oam_y1731_opcode */

    of_oxm_ofdpa_oam_y1731_opcode_init(ofdpa_oam_y1731_opcode, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_opcode);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_opcode, value);
    cur_len += ofdpa_oam_y1731_opcode->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_oam_y1731_opcode_init(ofdpa_oam_y1731_opcode, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_opcode);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_opcode, value);
    cur_len += ofdpa_oam_y1731_opcode->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_tc */

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3 */

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id_masked */

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_eth_subtype */

    of_oxm_ofdpa_eth_subtype_init(ofdpa_eth_subtype, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_eth_subtype);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_populate(
        ofdpa_eth_subtype, value);
    cur_len += ofdpa_eth_subtype->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_eth_subtype_init(ofdpa_eth_subtype, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_eth_subtype);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_populate(
        ofdpa_eth_subtype, value);
    cur_len += ofdpa_eth_subtype->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa_masked */

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst_masked */

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6 */

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code */

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf */

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1_masked */

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4_masked */

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src_masked */

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed */

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port */

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_ach_channel */

    of_oxm_ofdpa_mpls_ach_channel_init(ofdpa_mpls_ach_channel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ach_channel);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_populate(
        ofdpa_mpls_ach_channel, value);
    cur_len += ofdpa_mpls_ach_channel->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_ach_channel_init(ofdpa_mpls_ach_channel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ach_channel);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_populate(
        ofdpa_mpls_ach_channel, value);
    cur_len += ofdpa_mpls_ach_channel->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label_masked */

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id */

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_oxm generated by
 * list_setup_of_list_oxm_OF_VERSION_1_3
 */
int
list_check_of_list_oxm_OF_VERSION_1_3(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    /* Declare pointers for each subclass */
    of_oxm_ofdpa_mpls_ttl_t *ofdpa_mpls_ttl;
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_ofdpa_color_actions_index_t *ofdpa_color_actions_index;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_ofdpa_qos_index_t *ofdpa_qos_index;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_ofdpa_rxfcl_t *ofdpa_rxfcl;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_ofdpa_bfd_discriminator_masked_t *ofdpa_bfd_discriminator_masked;
    of_oxm_ofdpa_actset_output_t *ofdpa_actset_output;
    of_oxm_ofdpa_dei_t *ofdpa_dei;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_ofdpa_oam_y1731_mdl_t *ofdpa_oam_y1731_mdl;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_ofdpa_bfd_discriminator_t *ofdpa_bfd_discriminator;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ofdpa_mpls_data_first_nibble_t *ofdpa_mpls_data_first_nibble;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ofdpa_mpls_l2_port_masked_t *ofdpa_mpls_l2_port_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_ofdpa_protection_index_t *ofdpa_protection_index;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_ofdpa_color_t *ofdpa_color;
    of_oxm_ofdpa_vrf_t *ofdpa_vrf;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_ofdpa_lmep_id_t *ofdpa_lmep_id;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_ofdpa_mpls_next_label_is_gal_t *ofdpa_mpls_next_label_is_gal;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_ofdpa_rx_timestamp_t *ofdpa_rx_timestamp;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ofdpa_mpls_l2_port_t *ofdpa_mpls_l2_port;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_ofdpa_txfcl_t *ofdpa_txfcl;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_ofdpa_ovid_t *ofdpa_ovid;
    of_oxm_ofdpa_traffic_class_t *ofdpa_traffic_class;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_ofdpa_l3_in_port_t *ofdpa_l3_in_port;
    of_oxm_ofdpa_oam_y1731_opcode_t *ofdpa_oam_y1731_opcode;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_ofdpa_eth_subtype_t *ofdpa_eth_subtype;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_ofdpa_mpls_ach_channel_t *ofdpa_mpls_ach_channel;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    ofdpa_mpls_ttl = &elt.ofdpa_mpls_ttl;
    mpls_tc_masked = &elt.mpls_tc_masked;
    ofdpa_color_actions_index = &elt.ofdpa_color_actions_index;
    arp_sha = &elt.arp_sha;
    ofdpa_qos_index = &elt.ofdpa_qos_index;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    ofdpa_rxfcl = &elt.ofdpa_rxfcl;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    ofdpa_bfd_discriminator_masked = &elt.ofdpa_bfd_discriminator_masked;
    ofdpa_actset_output = &elt.ofdpa_actset_output;
    ofdpa_dei = &elt.ofdpa_dei;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    ofdpa_oam_y1731_mdl = &elt.ofdpa_oam_y1731_mdl;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    ofdpa_bfd_discriminator = &elt.ofdpa_bfd_discriminator;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ofdpa_mpls_data_first_nibble = &elt.ofdpa_mpls_data_first_nibble;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ofdpa_mpls_l2_port_masked = &elt.ofdpa_mpls_l2_port_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    ofdpa_protection_index = &elt.ofdpa_protection_index;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    ofdpa_color = &elt.ofdpa_color;
    ofdpa_vrf = &elt.ofdpa_vrf;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    ofdpa_lmep_id = &elt.ofdpa_lmep_id;
    in_phy_port_masked = &elt.in_phy_port_masked;
    ofdpa_mpls_next_label_is_gal = &elt.ofdpa_mpls_next_label_is_gal;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    ofdpa_rx_timestamp = &elt.ofdpa_rx_timestamp;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ofdpa_mpls_l2_port = &elt.ofdpa_mpls_l2_port;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    ofdpa_txfcl = &elt.ofdpa_txfcl;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    ofdpa_ovid = &elt.ofdpa_ovid;
    ofdpa_traffic_class = &elt.ofdpa_traffic_class;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    ofdpa_l3_in_port = &elt.ofdpa_l3_in_port;
    ofdpa_oam_y1731_opcode = &elt.ofdpa_oam_y1731_opcode;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    ofdpa_eth_subtype = &elt.ofdpa_eth_subtype;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    ofdpa_mpls_ach_channel = &elt.ofdpa_mpls_ach_channel;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;
    TEST_OK(of_list_oxm_first(list, &elt));

    /* Check two instances of type ofdpa_mpls_ttl */
    TEST_ASSERT(ofdpa_mpls_ttl->length == 11);
    if (ofdpa_mpls_ttl->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_ttl->object_id == OF_OXM_OFDPA_MPLS_TTL);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_check(
        ofdpa_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_ttl->length == 11);
    if (ofdpa_mpls_ttl->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_ttl->object_id == OF_OXM_OFDPA_MPLS_TTL);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_check(
        ofdpa_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_tc_masked */
    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_color_actions_index */
    TEST_ASSERT(ofdpa_color_actions_index->length == 14);
    if (ofdpa_color_actions_index->wire_length_get != NULL) {
        int length;

        ofdpa_color_actions_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_color_actions_index->object_id == OF_OXM_OFDPA_COLOR_ACTIONS_INDEX);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_check(
        ofdpa_color_actions_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_color_actions_index->length == 14);
    if (ofdpa_color_actions_index->wire_length_get != NULL) {
        int length;

        ofdpa_color_actions_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_color_actions_index->object_id == OF_OXM_OFDPA_COLOR_ACTIONS_INDEX);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_check(
        ofdpa_color_actions_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha */
    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_3_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_3_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_qos_index */
    TEST_ASSERT(ofdpa_qos_index->length == 11);
    if (ofdpa_qos_index->wire_length_get != NULL) {
        int length;

        ofdpa_qos_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_qos_index->object_id == OF_OXM_OFDPA_QOS_INDEX);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_check(
        ofdpa_qos_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_qos_index->length == 11);
    if (ofdpa_qos_index->wire_length_get != NULL) {
        int length;

        ofdpa_qos_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_qos_index->object_id == OF_OXM_OFDPA_QOS_INDEX);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_check(
        ofdpa_qos_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src_masked */
    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst */
    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_3_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_3_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_rxfcl */
    TEST_ASSERT(ofdpa_rxfcl->length == 18);
    if (ofdpa_rxfcl->wire_length_get != NULL) {
        int length;

        ofdpa_rxfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rxfcl->object_id == OF_OXM_OFDPA_RXFCL);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_check(
        ofdpa_rxfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_rxfcl->length == 18);
    if (ofdpa_rxfcl->wire_length_get != NULL) {
        int length;

        ofdpa_rxfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rxfcl->object_id == OF_OXM_OFDPA_RXFCL);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_check(
        ofdpa_rxfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id */
    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id */
    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op */
    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_3_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_3_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type */
    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_3_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_3_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_bfd_discriminator_masked */
    TEST_ASSERT(ofdpa_bfd_discriminator_masked->length == 18);
    if (ofdpa_bfd_discriminator_masked->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator_masked->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR_MASKED);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_bfd_discriminator_masked->length == 18);
    if (ofdpa_bfd_discriminator_masked->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator_masked->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR_MASKED);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_actset_output */
    TEST_ASSERT(ofdpa_actset_output->length == 14);
    if (ofdpa_actset_output->wire_length_get != NULL) {
        int length;

        ofdpa_actset_output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_actset_output->object_id == OF_OXM_OFDPA_ACTSET_OUTPUT);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_check(
        ofdpa_actset_output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_actset_output->length == 14);
    if (ofdpa_actset_output->wire_length_get != NULL) {
        int length;

        ofdpa_actset_output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_actset_output->object_id == OF_OXM_OFDPA_ACTSET_OUTPUT);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_check(
        ofdpa_actset_output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_dei */
    TEST_ASSERT(ofdpa_dei->length == 11);
    if (ofdpa_dei->wire_length_get != NULL) {
        int length;

        ofdpa_dei->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_dei->object_id == OF_OXM_OFDPA_DEI);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_check(
        ofdpa_dei, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_dei->length == 11);
    if (ofdpa_dei->wire_length_get != NULL) {
        int length;

        ofdpa_dei->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_dei->object_id == OF_OXM_OFDPA_DEI);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_check(
        ofdpa_dei, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128_masked */
    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos */
    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_oam_y1731_mdl */
    TEST_ASSERT(ofdpa_oam_y1731_mdl->length == 11);
    if (ofdpa_oam_y1731_mdl->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_mdl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_mdl->object_id == OF_OXM_OFDPA_OAM_Y1731_MDL);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_mdl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_y1731_mdl->length == 11);
    if (ofdpa_oam_y1731_mdl->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_mdl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_mdl->object_id == OF_OXM_OFDPA_OAM_Y1731_MDL);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_mdl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa */
    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_3_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_3_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type_masked */
    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha_masked */
    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst */
    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_3_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_3_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf_masked */
    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target */
    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type_masked */
    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_bfd_discriminator */
    TEST_ASSERT(ofdpa_bfd_discriminator->length == 14);
    if (ofdpa_bfd_discriminator->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_bfd_discriminator->length == 14);
    if (ofdpa_bfd_discriminator->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed_masked */
    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6_masked */
    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type */
    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst_masked */
    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst */
    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto */
    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_3_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_3_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata */
    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_3_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_3_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa_masked */
    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_data_first_nibble */
    TEST_ASSERT(ofdpa_mpls_data_first_nibble->length == 11);
    if (ofdpa_mpls_data_first_nibble->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_data_first_nibble->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_data_first_nibble->object_id == OF_OXM_OFDPA_MPLS_DATA_FIRST_NIBBLE);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_check(
        ofdpa_mpls_data_first_nibble, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_data_first_nibble->length == 11);
    if (ofdpa_mpls_data_first_nibble->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_data_first_nibble->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_data_first_nibble->object_id == OF_OXM_OFDPA_MPLS_DATA_FIRST_NIBBLE);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_check(
        ofdpa_mpls_data_first_nibble, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel */
    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id_masked */
    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll_masked */
    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa */
    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src */
    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_3_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_3_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src */
    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata_masked */
    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_l2_port_masked */
    TEST_ASSERT(ofdpa_mpls_l2_port_masked->length == 18);
    if (ofdpa_mpls_l2_port_masked->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port_masked->object_id == OF_OXM_OFDPA_MPLS_L2_PORT_MASKED);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_l2_port_masked->length == 18);
    if (ofdpa_mpls_l2_port_masked->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port_masked->object_id == OF_OXM_OFDPA_MPLS_L2_PORT_MASKED);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll */
    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id_masked */
    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_protection_index */
    TEST_ASSERT(ofdpa_protection_index->length == 11);
    if (ofdpa_protection_index->wire_length_get != NULL) {
        int length;

        ofdpa_protection_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_protection_index->object_id == OF_OXM_OFDPA_PROTECTION_INDEX);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_check(
        ofdpa_protection_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_protection_index->length == 11);
    if (ofdpa_protection_index->wire_length_get != NULL) {
        int length;

        ofdpa_protection_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_protection_index->object_id == OF_OXM_OFDPA_PROTECTION_INDEX);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_check(
        ofdpa_protection_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id */
    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid */
    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha */
    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_3_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_3_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp */
    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7_masked */
    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos_masked */
    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst */
    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id */
    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst_masked */
    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2_masked */
    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port_masked */
    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn */
    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code_masked */
    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst_masked */
    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src_masked */
    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst */
    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst_masked */
    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_color */
    TEST_ASSERT(ofdpa_color->length == 11);
    if (ofdpa_color->wire_length_get != NULL) {
        int length;

        ofdpa_color->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_color->object_id == OF_OXM_OFDPA_COLOR);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_check(
        ofdpa_color, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_color->length == 11);
    if (ofdpa_color->wire_length_get != NULL) {
        int length;

        ofdpa_color->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_color->object_id == OF_OXM_OFDPA_COLOR);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_check(
        ofdpa_color, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_vrf */
    TEST_ASSERT(ofdpa_vrf->length == 12);
    if (ofdpa_vrf->wire_length_get != NULL) {
        int length;

        ofdpa_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_vrf->object_id == OF_OXM_OFDPA_VRF);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_check(
        ofdpa_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_vrf->length == 12);
    if (ofdpa_vrf->wire_length_get != NULL) {
        int length;

        ofdpa_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_vrf->object_id == OF_OXM_OFDPA_VRF);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_check(
        ofdpa_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1 */
    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0 */
    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll */
    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2 */
    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5 */
    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4 */
    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7 */
    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto_masked */
    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_lmep_id */
    TEST_ASSERT(ofdpa_lmep_id->length == 14);
    if (ofdpa_lmep_id->wire_length_get != NULL) {
        int length;

        ofdpa_lmep_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_lmep_id->object_id == OF_OXM_OFDPA_LMEP_ID);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_check(
        ofdpa_lmep_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_lmep_id->length == 14);
    if (ofdpa_lmep_id->wire_length_get != NULL) {
        int length;

        ofdpa_lmep_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_lmep_id->object_id == OF_OXM_OFDPA_LMEP_ID);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_check(
        ofdpa_lmep_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port_masked */
    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_next_label_is_gal */
    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->length == 11);
    if (ofdpa_mpls_next_label_is_gal->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_next_label_is_gal->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->object_id == OF_OXM_OFDPA_MPLS_NEXT_LABEL_IS_GAL);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_check(
        ofdpa_mpls_next_label_is_gal, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->length == 11);
    if (ofdpa_mpls_next_label_is_gal->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_next_label_is_gal->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->object_id == OF_OXM_OFDPA_MPLS_NEXT_LABEL_IS_GAL);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_check(
        ofdpa_mpls_next_label_is_gal, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp_masked */
    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst_masked */
    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp */
    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src */
    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst */
    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src_masked */
    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op_masked */
    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src */
    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_3_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_3_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code */
    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5_masked */
    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code_masked */
    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src_masked */
    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn_masked */
    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src */
    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_3_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_3_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid_masked */
    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha_masked */
    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_rx_timestamp */
    TEST_ASSERT(ofdpa_rx_timestamp->length == 18);
    if (ofdpa_rx_timestamp->wire_length_get != NULL) {
        int length;

        ofdpa_rx_timestamp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rx_timestamp->object_id == OF_OXM_OFDPA_RX_TIMESTAMP);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_check(
        ofdpa_rx_timestamp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_rx_timestamp->length == 18);
    if (ofdpa_rx_timestamp->wire_length_get != NULL) {
        int length;

        ofdpa_rx_timestamp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rx_timestamp->object_id == OF_OXM_OFDPA_RX_TIMESTAMP);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_check(
        ofdpa_rx_timestamp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src_masked */
    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port */
    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_3_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_3_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id */
    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_l2_port */
    TEST_ASSERT(ofdpa_mpls_l2_port->length == 14);
    if (ofdpa_mpls_l2_port->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port->object_id == OF_OXM_OFDPA_MPLS_L2_PORT);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_l2_port->length == 14);
    if (ofdpa_mpls_l2_port->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port->object_id == OF_OXM_OFDPA_MPLS_L2_PORT);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll_masked */
    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src */
    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_3_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_3_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target_masked */
    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type_masked */
    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0_masked */
    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_txfcl */
    TEST_ASSERT(ofdpa_txfcl->length == 18);
    if (ofdpa_txfcl->wire_length_get != NULL) {
        int length;

        ofdpa_txfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_txfcl->object_id == OF_OXM_OFDPA_TXFCL);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_check(
        ofdpa_txfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_txfcl->length == 18);
    if (ofdpa_txfcl->wire_length_get != NULL) {
        int length;

        ofdpa_txfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_txfcl->object_id == OF_OXM_OFDPA_TXFCL);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_check(
        ofdpa_txfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3_masked */
    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label */
    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_3_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_3_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_ovid */
    TEST_ASSERT(ofdpa_ovid->length == 12);
    if (ofdpa_ovid->wire_length_get != NULL) {
        int length;

        ofdpa_ovid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_ovid->object_id == OF_OXM_OFDPA_OVID);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_check(
        ofdpa_ovid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_ovid->length == 12);
    if (ofdpa_ovid->wire_length_get != NULL) {
        int length;

        ofdpa_ovid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_ovid->object_id == OF_OXM_OFDPA_OVID);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_check(
        ofdpa_ovid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_traffic_class */
    TEST_ASSERT(ofdpa_traffic_class->length == 11);
    if (ofdpa_traffic_class->wire_length_get != NULL) {
        int length;

        ofdpa_traffic_class->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_traffic_class->object_id == OF_OXM_OFDPA_TRAFFIC_CLASS);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_check(
        ofdpa_traffic_class, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_traffic_class->length == 11);
    if (ofdpa_traffic_class->wire_length_get != NULL) {
        int length;

        ofdpa_traffic_class->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_traffic_class->object_id == OF_OXM_OFDPA_TRAFFIC_CLASS);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_check(
        ofdpa_traffic_class, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128 */
    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id_masked */
    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel_masked */
    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id_masked */
    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp_masked */
    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type */
    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id_masked */
    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_l3_in_port */
    TEST_ASSERT(ofdpa_l3_in_port->length == 14);
    if (ofdpa_l3_in_port->wire_length_get != NULL) {
        int length;

        ofdpa_l3_in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_l3_in_port->object_id == OF_OXM_OFDPA_L3_IN_PORT);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_check(
        ofdpa_l3_in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_l3_in_port->length == 14);
    if (ofdpa_l3_in_port->wire_length_get != NULL) {
        int length;

        ofdpa_l3_in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_l3_in_port->object_id == OF_OXM_OFDPA_L3_IN_PORT);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_check(
        ofdpa_l3_in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_oam_y1731_opcode */
    TEST_ASSERT(ofdpa_oam_y1731_opcode->length == 11);
    if (ofdpa_oam_y1731_opcode->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_opcode->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_opcode->object_id == OF_OXM_OFDPA_OAM_Y1731_OPCODE);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_opcode, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_y1731_opcode->length == 11);
    if (ofdpa_oam_y1731_opcode->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_opcode->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_opcode->object_id == OF_OXM_OFDPA_OAM_Y1731_OPCODE);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_opcode, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_tc */
    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3 */
    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id_masked */
    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_eth_subtype */
    TEST_ASSERT(ofdpa_eth_subtype->length == 11);
    if (ofdpa_eth_subtype->wire_length_get != NULL) {
        int length;

        ofdpa_eth_subtype->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_eth_subtype->object_id == OF_OXM_OFDPA_ETH_SUBTYPE);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_check(
        ofdpa_eth_subtype, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_eth_subtype->length == 11);
    if (ofdpa_eth_subtype->wire_length_get != NULL) {
        int length;

        ofdpa_eth_subtype->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_eth_subtype->object_id == OF_OXM_OFDPA_ETH_SUBTYPE);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_check(
        ofdpa_eth_subtype, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa_masked */
    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst_masked */
    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6 */
    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code */
    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf */
    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1_masked */
    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4_masked */
    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src_masked */
    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed */
    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port */
    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_ach_channel */
    TEST_ASSERT(ofdpa_mpls_ach_channel->length == 12);
    if (ofdpa_mpls_ach_channel->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ach_channel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_mpls_ach_channel->object_id == OF_OXM_OFDPA_MPLS_ACH_CHANNEL);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_check(
        ofdpa_mpls_ach_channel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_ach_channel->length == 12);
    if (ofdpa_mpls_ach_channel->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ach_channel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_mpls_ach_channel->object_id == OF_OXM_OFDPA_MPLS_ACH_CHANNEL);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_check(
        ofdpa_mpls_ach_channel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label_masked */
    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id */
    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_oxm_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_packet_queue with two of each type of subclass
 */
int
list_setup_of_list_packet_queue_OF_VERSION_1_3(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * list_setup_of_list_packet_queue_OF_VERSION_1_3
 */
int
list_check_of_list_packet_queue_OF_VERSION_1_3(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_desc with two of each type of subclass
 */
int
list_setup_of_list_port_desc_OF_VERSION_1_3(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * list_setup_of_list_port_desc_OF_VERSION_1_3
 */
int
list_check_of_list_port_desc_OF_VERSION_1_3(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_port_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_port_stats_entry_OF_VERSION_1_3(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * list_setup_of_list_port_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_port_stats_entry_OF_VERSION_1_3(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_prop with two of each type of subclass
 */
int
list_setup_of_list_queue_prop_OF_VERSION_1_3(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_queue_prop_max_rate */

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * list_setup_of_list_queue_prop_OF_VERSION_1_3
 */
int
list_check_of_list_queue_prop_OF_VERSION_1_3(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    /* Check two instances of type max_rate */
    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_queue_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_queue_stats_entry_OF_VERSION_1_3(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * list_setup_of_list_queue_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_queue_stats_entry_OF_VERSION_1_3(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_table_feature_prop with two of each type of subclass
 */
int
list_setup_of_list_table_feature_prop_OF_VERSION_1_3(
    of_list_table_feature_prop_t *list, int value)
{

    of_table_feature_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_table_feature_prop_next_tables_miss_t *next_tables_miss;
    of_table_feature_prop_wildcards_t *wildcards;
    of_table_feature_prop_experimenter_miss_t *experimenter_miss;
    of_table_feature_prop_write_actions_t *write_actions;
    of_table_feature_prop_write_actions_miss_t *write_actions_miss;
    of_table_feature_prop_next_tables_t *next_tables;
    of_table_feature_prop_apply_actions_miss_t *apply_actions_miss;
    of_table_feature_prop_apply_actions_t *apply_actions;
    of_table_feature_prop_apply_setfield_miss_t *apply_setfield_miss;
    of_table_feature_prop_experimenter_t *experimenter;
    of_table_feature_prop_write_setfield_miss_t *write_setfield_miss;
    of_table_feature_prop_write_setfield_t *write_setfield;
    of_table_feature_prop_instructions_miss_t *instructions_miss;
    of_table_feature_prop_apply_setfield_t *apply_setfield;
    of_table_feature_prop_match_t *match;
    of_table_feature_prop_instructions_t *instructions;

    /* Instantiate pointers for each subclass */
    next_tables_miss = &elt.next_tables_miss;
    wildcards = &elt.wildcards;
    experimenter_miss = &elt.experimenter_miss;
    write_actions = &elt.write_actions;
    write_actions_miss = &elt.write_actions_miss;
    next_tables = &elt.next_tables;
    apply_actions_miss = &elt.apply_actions_miss;
    apply_actions = &elt.apply_actions;
    apply_setfield_miss = &elt.apply_setfield_miss;
    experimenter = &elt.experimenter;
    write_setfield_miss = &elt.write_setfield_miss;
    write_setfield = &elt.write_setfield;
    instructions_miss = &elt.instructions_miss;
    apply_setfield = &elt.apply_setfield;
    match = &elt.match;
    instructions = &elt.instructions;

    /* Append two instances of type of_table_feature_prop_next_tables_miss */

    of_table_feature_prop_next_tables_miss_init(next_tables_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables_miss);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_populate(
        next_tables_miss, value);
    cur_len += next_tables_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_next_tables_miss_init(next_tables_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables_miss);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_populate(
        next_tables_miss, value);
    cur_len += next_tables_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_wildcards */

    of_table_feature_prop_wildcards_init(wildcards, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)wildcards);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_populate(
        wildcards, value);
    cur_len += wildcards->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_wildcards_init(wildcards, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)wildcards);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_populate(
        wildcards, value);
    cur_len += wildcards->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_experimenter_miss */

    of_table_feature_prop_experimenter_miss_init(experimenter_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter_miss);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_populate(
        experimenter_miss, value);
    cur_len += experimenter_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_experimenter_miss_init(experimenter_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter_miss);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_populate(
        experimenter_miss, value);
    cur_len += experimenter_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_actions */

    of_table_feature_prop_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_actions_miss */

    of_table_feature_prop_write_actions_miss_init(write_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions_miss);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_populate(
        write_actions_miss, value);
    cur_len += write_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_actions_miss_init(write_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions_miss);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_populate(
        write_actions_miss, value);
    cur_len += write_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_next_tables */

    of_table_feature_prop_next_tables_init(next_tables, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_populate(
        next_tables, value);
    cur_len += next_tables->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_next_tables_init(next_tables, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_populate(
        next_tables, value);
    cur_len += next_tables->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_actions_miss */

    of_table_feature_prop_apply_actions_miss_init(apply_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions_miss);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_populate(
        apply_actions_miss, value);
    cur_len += apply_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_actions_miss_init(apply_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions_miss);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_populate(
        apply_actions_miss, value);
    cur_len += apply_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_actions */

    of_table_feature_prop_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_setfield_miss */

    of_table_feature_prop_apply_setfield_miss_init(apply_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield_miss);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_populate(
        apply_setfield_miss, value);
    cur_len += apply_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_setfield_miss_init(apply_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield_miss);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_populate(
        apply_setfield_miss, value);
    cur_len += apply_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_experimenter */

    of_table_feature_prop_experimenter_init(experimenter, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_populate(
        experimenter, value);
    cur_len += experimenter->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_experimenter_init(experimenter, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_populate(
        experimenter, value);
    cur_len += experimenter->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_setfield_miss */

    of_table_feature_prop_write_setfield_miss_init(write_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield_miss);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_populate(
        write_setfield_miss, value);
    cur_len += write_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_setfield_miss_init(write_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield_miss);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_populate(
        write_setfield_miss, value);
    cur_len += write_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_setfield */

    of_table_feature_prop_write_setfield_init(write_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_populate(
        write_setfield, value);
    cur_len += write_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_setfield_init(write_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_populate(
        write_setfield, value);
    cur_len += write_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_instructions_miss */

    of_table_feature_prop_instructions_miss_init(instructions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions_miss);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_populate(
        instructions_miss, value);
    cur_len += instructions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_instructions_miss_init(instructions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions_miss);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_populate(
        instructions_miss, value);
    cur_len += instructions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_setfield */

    of_table_feature_prop_apply_setfield_init(apply_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_populate(
        apply_setfield, value);
    cur_len += apply_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_setfield_init(apply_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_populate(
        apply_setfield, value);
    cur_len += apply_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_match */

    of_table_feature_prop_match_init(match, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)match);
    value = of_table_feature_prop_match_OF_VERSION_1_3_populate(
        match, value);
    cur_len += match->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_match_init(match, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)match);
    value = of_table_feature_prop_match_OF_VERSION_1_3_populate(
        match, value);
    cur_len += match->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_instructions */

    of_table_feature_prop_instructions_init(instructions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_populate(
        instructions, value);
    cur_len += instructions->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_instructions_init(instructions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_populate(
        instructions, value);
    cur_len += instructions->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_table_feature_prop generated by
 * list_setup_of_list_table_feature_prop_OF_VERSION_1_3
 */
int
list_check_of_list_table_feature_prop_OF_VERSION_1_3(
    of_list_table_feature_prop_t *list, int value)
{

    of_table_feature_prop_t elt;
    /* Declare pointers for each subclass */
    of_table_feature_prop_next_tables_miss_t *next_tables_miss;
    of_table_feature_prop_wildcards_t *wildcards;
    of_table_feature_prop_experimenter_miss_t *experimenter_miss;
    of_table_feature_prop_write_actions_t *write_actions;
    of_table_feature_prop_write_actions_miss_t *write_actions_miss;
    of_table_feature_prop_next_tables_t *next_tables;
    of_table_feature_prop_apply_actions_miss_t *apply_actions_miss;
    of_table_feature_prop_apply_actions_t *apply_actions;
    of_table_feature_prop_apply_setfield_miss_t *apply_setfield_miss;
    of_table_feature_prop_experimenter_t *experimenter;
    of_table_feature_prop_write_setfield_miss_t *write_setfield_miss;
    of_table_feature_prop_write_setfield_t *write_setfield;
    of_table_feature_prop_instructions_miss_t *instructions_miss;
    of_table_feature_prop_apply_setfield_t *apply_setfield;
    of_table_feature_prop_match_t *match;
    of_table_feature_prop_instructions_t *instructions;

    /* Instantiate pointers for each subclass */
    next_tables_miss = &elt.next_tables_miss;
    wildcards = &elt.wildcards;
    experimenter_miss = &elt.experimenter_miss;
    write_actions = &elt.write_actions;
    write_actions_miss = &elt.write_actions_miss;
    next_tables = &elt.next_tables;
    apply_actions_miss = &elt.apply_actions_miss;
    apply_actions = &elt.apply_actions;
    apply_setfield_miss = &elt.apply_setfield_miss;
    experimenter = &elt.experimenter;
    write_setfield_miss = &elt.write_setfield_miss;
    write_setfield = &elt.write_setfield;
    instructions_miss = &elt.instructions_miss;
    apply_setfield = &elt.apply_setfield;
    match = &elt.match;
    instructions = &elt.instructions;
    TEST_OK(of_list_table_feature_prop_first(list, &elt));

    /* Check two instances of type next_tables_miss */
    TEST_ASSERT(next_tables_miss->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES_MISS);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_check(
        next_tables_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(next_tables_miss->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES_MISS);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_check(
        next_tables_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type wildcards */
    TEST_ASSERT(wildcards->object_id == OF_TABLE_FEATURE_PROP_WILDCARDS);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_check(
        wildcards, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(wildcards->object_id == OF_TABLE_FEATURE_PROP_WILDCARDS);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_check(
        wildcards, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type experimenter_miss */
    TEST_ASSERT(experimenter_miss->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER_MISS);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_check(
        experimenter_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(experimenter_miss->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER_MISS);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_check(
        experimenter_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_actions_miss */
    TEST_ASSERT(write_actions_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS_MISS);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_check(
        write_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_actions_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS_MISS);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_check(
        write_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type next_tables */
    TEST_ASSERT(next_tables->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_check(
        next_tables, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(next_tables->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_check(
        next_tables, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_actions_miss */
    TEST_ASSERT(apply_actions_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS_MISS);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_check(
        apply_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_actions_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS_MISS);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_check(
        apply_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_setfield_miss */
    TEST_ASSERT(apply_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD_MISS);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_check(
        apply_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD_MISS);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_check(
        apply_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type experimenter */
    TEST_ASSERT(experimenter->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_check(
        experimenter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(experimenter->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_check(
        experimenter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_setfield_miss */
    TEST_ASSERT(write_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD_MISS);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_check(
        write_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD_MISS);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_check(
        write_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_setfield */
    TEST_ASSERT(write_setfield->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_check(
        write_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_setfield->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_check(
        write_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type instructions_miss */
    TEST_ASSERT(instructions_miss->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS_MISS);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_check(
        instructions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(instructions_miss->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS_MISS);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_check(
        instructions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_setfield */
    TEST_ASSERT(apply_setfield->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_check(
        apply_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_setfield->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_check(
        apply_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type match */
    TEST_ASSERT(match->object_id == OF_TABLE_FEATURE_PROP_MATCH);
    value = of_table_feature_prop_match_OF_VERSION_1_3_check(
        match, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(match->object_id == OF_TABLE_FEATURE_PROP_MATCH);
    value = of_table_feature_prop_match_OF_VERSION_1_3_check(
        match, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type instructions */
    TEST_ASSERT(instructions->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_check(
        instructions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(instructions->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_check(
        instructions, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_feature_prop_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_table_features with two of each type of subclass
 */
int
list_setup_of_list_table_features_OF_VERSION_1_3(
    of_list_table_features_t *list, int value)
{

    of_table_features_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_features */
    of_table_features_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_features */

    of_table_features_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_features_append_bind(list,
            (of_table_features_t *)elt_p);
    value = of_table_features_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_features_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_features_append_bind(list,
            (of_table_features_t *)elt_p);
    value = of_table_features_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_table_features generated by
 * list_setup_of_list_table_features_OF_VERSION_1_3
 */
int
list_check_of_list_table_features_OF_VERSION_1_3(
    of_list_table_features_t *list, int value)
{

    of_table_features_t elt;
    /* No subclasses for of_table_features */
    of_table_features_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_features_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_FEATURES);
    value = of_table_features_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_features_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_FEATURES);
    value = of_table_features_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_features_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_table_stats_entry with two of each type of subclass
 */
int
list_setup_of_list_table_stats_entry_OF_VERSION_1_3(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * list_setup_of_list_table_stats_entry_OF_VERSION_1_3
 */
int
list_check_of_list_table_stats_entry_OF_VERSION_1_3(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_uint32 with two of each type of subclass
 */
int
list_setup_of_list_uint32_OF_VERSION_1_3(
    of_list_uint32_t *list, int value)
{

    of_uint32_t elt;
    int cur_len = 0;
    /* No subclasses for of_uint32 */
    of_uint32_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_uint32 */

    of_uint32_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint32_append_bind(list,
            (of_uint32_t *)elt_p);
    value = of_uint32_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_uint32_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint32_append_bind(list,
            (of_uint32_t *)elt_p);
    value = of_uint32_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_uint32 generated by
 * list_setup_of_list_uint32_OF_VERSION_1_3
 */
int
list_check_of_list_uint32_OF_VERSION_1_3(
    of_list_uint32_t *list, int value)
{

    of_uint32_t elt;
    /* No subclasses for of_uint32 */
    of_uint32_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_uint32_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_UINT32);
    value = of_uint32_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_uint32_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_UINT32);
    value = of_uint32_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_uint32_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_uint64 with two of each type of subclass
 */
int
list_setup_of_list_uint64_OF_VERSION_1_3(
    of_list_uint64_t *list, int value)
{

    of_uint64_t elt;
    int cur_len = 0;
    /* No subclasses for of_uint64 */
    of_uint64_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_uint64 */

    of_uint64_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint64_append_bind(list,
            (of_uint64_t *)elt_p);
    value = of_uint64_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_uint64_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint64_append_bind(list,
            (of_uint64_t *)elt_p);
    value = of_uint64_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_uint64 generated by
 * list_setup_of_list_uint64_OF_VERSION_1_3
 */
int
list_check_of_list_uint64_OF_VERSION_1_3(
    of_list_uint64_t *list, int value)
{

    of_uint64_t elt;
    /* No subclasses for of_uint64 */
    of_uint64_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_uint64_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_UINT64);
    value = of_uint64_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_uint64_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_UINT64);
    value = of_uint64_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_uint64_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Set up a list of type of_list_uint8 with two of each type of subclass
 */
int
list_setup_of_list_uint8_OF_VERSION_1_3(
    of_list_uint8_t *list, int value)
{

    of_uint8_t elt;
    int cur_len = 0;
    /* No subclasses for of_uint8 */
    of_uint8_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_uint8 */

    of_uint8_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint8_append_bind(list,
            (of_uint8_t *)elt_p);
    value = of_uint8_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_uint8_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint8_append_bind(list,
            (of_uint8_t *)elt_p);
    value = of_uint8_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);


    return value;
}

/**
 * Check a list of type of_list_uint8 generated by
 * list_setup_of_list_uint8_OF_VERSION_1_3
 */
int
list_check_of_list_uint8_OF_VERSION_1_3(
    of_list_uint8_t *list, int value)
{

    of_uint8_t elt;
    /* No subclasses for of_uint8 */
    of_uint8_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_uint8_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_UINT8);
    value = of_uint8_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_uint8_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_UINT8);
    value = of_uint8_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_uint8_next(list, &elt) == OF_ERROR_RANGE);

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_0_populate(
    of_aggregate_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_reply_xid_get(obj, &val32);
    of_aggregate_stats_reply_flags_get(obj, &val16);
    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    of_aggregate_stats_reply_flow_count_get(obj, &val32);

    value = of_aggregate_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_0_check(
    of_aggregate_stats_reply_t *obj, int value)
{

    value = of_aggregate_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_reply_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_0_populate(
    of_aggregate_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_match_t match;
    uint8_t val8;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_request_xid_get(obj, &val32);
    of_aggregate_stats_request_flags_get(obj, &val16);
    of_aggregate_stats_request_match_get(obj, &match);
    of_aggregate_stats_request_table_id_get(obj, &val8);
    of_aggregate_stats_request_out_port_get(obj, &port_no);

    value = of_aggregate_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_0_check(
    of_aggregate_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_aggregate_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_request_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_action_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_0_populate(
    of_bad_action_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_action_error_msg_xid_get(obj, &val32);
    of_bad_action_error_msg_code_get(obj, &val16);
    of_bad_action_error_msg_data_get(obj, &octets);

    value = of_bad_action_error_msg_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_action_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_0_check(
    of_bad_action_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_action_error_msg_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_action_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_action_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_action_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_request_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_0_populate(
    of_bad_request_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_request_error_msg_xid_get(obj, &val32);
    of_bad_request_error_msg_code_get(obj, &val16);
    of_bad_request_error_msg_data_get(obj, &octets);

    value = of_bad_request_error_msg_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_request_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_0_check(
    of_bad_request_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_request_error_msg_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_request_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_request_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_request_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_0_populate(
    of_barrier_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_reply_xid_get(obj, &val32);

    value = of_barrier_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_0_check(
    of_barrier_reply_t *obj, int value)
{

    value = of_barrier_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_reply_t *dup;

        TEST_ASSERT((dup = of_barrier_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_0_populate(
    of_barrier_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_request_xid_get(obj, &val32);

    value = of_barrier_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_0_check(
    of_barrier_request_t *obj, int value)
{

    value = of_barrier_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_request_t *dup;

        TEST_ASSERT((dup = of_barrier_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_0_populate(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    of_bsn_bw_clear_data_reply_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_reply_subtype_get(obj, &val32);
    of_bsn_bw_clear_data_reply_status_get(obj, &val32);

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_0_check(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_0_populate(
    of_bsn_bw_clear_data_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    of_bsn_bw_clear_data_request_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_request_subtype_get(obj, &val32);

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_0_check(
    of_bsn_bw_clear_data_request_t *obj, int value)
{

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_0_populate(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_get_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_0_check(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_0_populate(
    of_bsn_bw_enable_get_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    of_bsn_bw_enable_get_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_request_subtype_get(obj, &val32);

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_0_check(
    of_bsn_bw_enable_get_request_t *obj, int value)
{

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_0_populate(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_set_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    of_bsn_bw_enable_set_reply_status_get(obj, &val32);

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_0_check(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_0_populate(
    of_bsn_bw_enable_set_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    of_bsn_bw_enable_set_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_request_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_request_enable_get(obj, &val32);

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_0_check(
    of_bsn_bw_enable_set_request_t *obj, int value)
{

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_0_populate(
    of_bsn_get_interfaces_reply_t *obj, int value)
{
    uint32_t val32;
    of_list_bsn_interface_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    of_bsn_get_interfaces_reply_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_interface_t sub_cls;

        /* Test bind */
        of_bsn_get_interfaces_reply_interfaces_bind(obj, &sub_cls);
    }

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_interface_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_interface_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_get_interfaces_reply_interfaces_set(
        obj, list);
    of_list_bsn_interface_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_0_check(
    of_bsn_get_interfaces_reply_t *obj, int value)
{

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_interface_t *interfaces_ptr;

        interfaces_ptr = of_bsn_get_interfaces_reply_interfaces_get(obj);
        TEST_ASSERT(interfaces_ptr != NULL);
        value = of_list_bsn_interface_OF_VERSION_1_0_check(
            interfaces_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_interface_delete(interfaces_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_0_populate(
    of_bsn_get_interfaces_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    of_bsn_get_interfaces_request_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_request_subtype_get(obj, &val32);

    value = of_bsn_get_interfaces_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_0_check(
    of_bsn_get_interfaces_request_t *obj, int value)
{

    value = of_bsn_get_interfaces_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_ip_mask_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_ip_mask_reply_OF_VERSION_1_0_populate(
    of_bsn_get_ip_mask_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_ip_mask_reply_xid_get(obj, &val32);
    of_bsn_get_ip_mask_reply_experimenter_get(obj, &val32);
    of_bsn_get_ip_mask_reply_subtype_get(obj, &val32);
    of_bsn_get_ip_mask_reply_index_get(obj, &val8);
    of_bsn_get_ip_mask_reply_mask_get(obj, &val32);

    value = of_bsn_get_ip_mask_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_ip_mask_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_ip_mask_reply_OF_VERSION_1_0_check(
    of_bsn_get_ip_mask_reply_t *obj, int value)
{

    value = of_bsn_get_ip_mask_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_ip_mask_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_ip_mask_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_ip_mask_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_ip_mask_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_ip_mask_request_OF_VERSION_1_0_populate(
    of_bsn_get_ip_mask_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_ip_mask_request_xid_get(obj, &val32);
    of_bsn_get_ip_mask_request_experimenter_get(obj, &val32);
    of_bsn_get_ip_mask_request_subtype_get(obj, &val32);
    of_bsn_get_ip_mask_request_index_get(obj, &val8);

    value = of_bsn_get_ip_mask_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_ip_mask_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_ip_mask_request_OF_VERSION_1_0_check(
    of_bsn_get_ip_mask_request_t *obj, int value)
{

    value = of_bsn_get_ip_mask_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_ip_mask_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_ip_mask_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_ip_mask_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_l2_table_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_l2_table_reply_OF_VERSION_1_0_populate(
    of_bsn_get_l2_table_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_l2_table_reply_xid_get(obj, &val32);
    of_bsn_get_l2_table_reply_experimenter_get(obj, &val32);
    of_bsn_get_l2_table_reply_subtype_get(obj, &val32);
    of_bsn_get_l2_table_reply_l2_table_enable_get(obj, &val8);
    of_bsn_get_l2_table_reply_l2_table_priority_get(obj, &val16);

    value = of_bsn_get_l2_table_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_l2_table_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_l2_table_reply_OF_VERSION_1_0_check(
    of_bsn_get_l2_table_reply_t *obj, int value)
{

    value = of_bsn_get_l2_table_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_l2_table_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_l2_table_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_l2_table_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_l2_table_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_l2_table_request_OF_VERSION_1_0_populate(
    of_bsn_get_l2_table_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_l2_table_request_xid_get(obj, &val32);
    of_bsn_get_l2_table_request_experimenter_get(obj, &val32);
    of_bsn_get_l2_table_request_subtype_get(obj, &val32);

    value = of_bsn_get_l2_table_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_l2_table_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_l2_table_request_OF_VERSION_1_0_check(
    of_bsn_get_l2_table_request_t *obj, int value)
{

    value = of_bsn_get_l2_table_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_l2_table_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_l2_table_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_l2_table_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_0_populate(
    of_bsn_get_mirroring_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    of_bsn_get_mirroring_reply_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_reply_subtype_get(obj, &val32);
    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_0_check(
    of_bsn_get_mirroring_reply_t *obj, int value)
{

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_0_populate(
    of_bsn_get_mirroring_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    of_bsn_get_mirroring_request_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_request_subtype_get(obj, &val32);
    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_0_check(
    of_bsn_get_mirroring_request_t *obj, int value)
{

    value = of_bsn_get_mirroring_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_0_populate(
    of_bsn_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_header_xid_get(obj, &val32);
    of_bsn_header_experimenter_get(obj, &val32);
    of_bsn_header_subtype_get(obj, &val32);

    value = of_bsn_header_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_0_check(
    of_bsn_header_t *obj, int value)
{

    value = of_bsn_header_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_header_t *dup;

        TEST_ASSERT((dup = of_bsn_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_hybrid_get_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_hybrid_get_reply_OF_VERSION_1_0_populate(
    of_bsn_hybrid_get_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_hybrid_get_reply_xid_get(obj, &val32);
    of_bsn_hybrid_get_reply_experimenter_get(obj, &val32);
    of_bsn_hybrid_get_reply_subtype_get(obj, &val32);
    of_bsn_hybrid_get_reply_hybrid_enable_get(obj, &val8);
    of_bsn_hybrid_get_reply_hybrid_version_get(obj, &val16);

    value = of_bsn_hybrid_get_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_hybrid_get_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_hybrid_get_reply_OF_VERSION_1_0_check(
    of_bsn_hybrid_get_reply_t *obj, int value)
{

    value = of_bsn_hybrid_get_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_hybrid_get_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_hybrid_get_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_hybrid_get_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_hybrid_get_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_hybrid_get_request_OF_VERSION_1_0_populate(
    of_bsn_hybrid_get_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_hybrid_get_request_xid_get(obj, &val32);
    of_bsn_hybrid_get_request_experimenter_get(obj, &val32);
    of_bsn_hybrid_get_request_subtype_get(obj, &val32);

    value = of_bsn_hybrid_get_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_hybrid_get_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_hybrid_get_request_OF_VERSION_1_0_check(
    of_bsn_hybrid_get_request_t *obj, int value)
{

    value = of_bsn_hybrid_get_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_hybrid_get_request_t *dup;

        TEST_ASSERT((dup = of_bsn_hybrid_get_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_hybrid_get_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_0_populate(
    of_bsn_pdu_rx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    of_bsn_pdu_rx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_0_check(
    of_bsn_pdu_rx_reply_t *obj, int value)
{

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_0_populate(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    of_bsn_pdu_rx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_request_subtype_get(obj, &val32);
    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_rx_request_data_get(obj, &octets);

    value = of_bsn_pdu_rx_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_0_check(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_rx_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_timeout
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_0_populate(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    of_bsn_pdu_rx_timeout_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_timeout_subtype_get(obj, &val32);
    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_timeout
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_0_check(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_timeout_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_timeout_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_timeout_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_0_populate(
    of_bsn_pdu_tx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    of_bsn_pdu_tx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_0_check(
    of_bsn_pdu_tx_reply_t *obj, int value)
{

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_0_populate(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    of_bsn_pdu_tx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_request_subtype_get(obj, &val32);
    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_tx_request_data_get(obj, &octets);

    value = of_bsn_pdu_tx_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_0_check(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_tx_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_ip_mask
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_ip_mask_OF_VERSION_1_0_populate(
    of_bsn_set_ip_mask_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_ip_mask_xid_get(obj, &val32);
    of_bsn_set_ip_mask_experimenter_get(obj, &val32);
    of_bsn_set_ip_mask_subtype_get(obj, &val32);
    of_bsn_set_ip_mask_index_get(obj, &val8);
    of_bsn_set_ip_mask_mask_get(obj, &val32);

    value = of_bsn_set_ip_mask_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_ip_mask
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_ip_mask_OF_VERSION_1_0_check(
    of_bsn_set_ip_mask_t *obj, int value)
{

    value = of_bsn_set_ip_mask_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_ip_mask_t *dup;

        TEST_ASSERT((dup = of_bsn_set_ip_mask_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_ip_mask_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_l2_table_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_l2_table_reply_OF_VERSION_1_0_populate(
    of_bsn_set_l2_table_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_l2_table_reply_xid_get(obj, &val32);
    of_bsn_set_l2_table_reply_experimenter_get(obj, &val32);
    of_bsn_set_l2_table_reply_subtype_get(obj, &val32);
    of_bsn_set_l2_table_reply_l2_table_enable_get(obj, &val8);
    of_bsn_set_l2_table_reply_l2_table_priority_get(obj, &val16);
    of_bsn_set_l2_table_reply_status_get(obj, &val32);

    value = of_bsn_set_l2_table_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_l2_table_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_l2_table_reply_OF_VERSION_1_0_check(
    of_bsn_set_l2_table_reply_t *obj, int value)
{

    value = of_bsn_set_l2_table_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_l2_table_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_l2_table_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_l2_table_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_l2_table_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_l2_table_request_OF_VERSION_1_0_populate(
    of_bsn_set_l2_table_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_l2_table_request_xid_get(obj, &val32);
    of_bsn_set_l2_table_request_experimenter_get(obj, &val32);
    of_bsn_set_l2_table_request_subtype_get(obj, &val32);
    of_bsn_set_l2_table_request_l2_table_enable_get(obj, &val8);
    of_bsn_set_l2_table_request_l2_table_priority_get(obj, &val16);

    value = of_bsn_set_l2_table_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_l2_table_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_l2_table_request_OF_VERSION_1_0_check(
    of_bsn_set_l2_table_request_t *obj, int value)
{

    value = of_bsn_set_l2_table_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_l2_table_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_l2_table_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_l2_table_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_mirroring
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_0_populate(
    of_bsn_set_mirroring_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_mirroring_xid_get(obj, &val32);
    of_bsn_set_mirroring_experimenter_get(obj, &val32);
    of_bsn_set_mirroring_subtype_get(obj, &val32);
    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);

    value = of_bsn_set_mirroring_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_mirroring
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_0_check(
    of_bsn_set_mirroring_t *obj, int value)
{

    value = of_bsn_set_mirroring_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_mirroring_t *dup;

        TEST_ASSERT((dup = of_bsn_set_mirroring_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_mirroring_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_0_populate(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_0_check(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_0_populate(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_0_check(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_shell_command
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_shell_command_OF_VERSION_1_0_populate(
    of_bsn_shell_command_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_shell_command_xid_get(obj, &val32);
    of_bsn_shell_command_experimenter_get(obj, &val32);
    of_bsn_shell_command_subtype_get(obj, &val32);
    of_bsn_shell_command_service_get(obj, &val32);
    of_bsn_shell_command_data_get(obj, &octets);

    value = of_bsn_shell_command_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_shell_command_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_shell_command
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_shell_command_OF_VERSION_1_0_check(
    of_bsn_shell_command_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_shell_command_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_shell_command_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_shell_command_t *dup;

        TEST_ASSERT((dup = of_bsn_shell_command_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_shell_command_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_shell_output
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_shell_output_OF_VERSION_1_0_populate(
    of_bsn_shell_output_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_shell_output_xid_get(obj, &val32);
    of_bsn_shell_output_experimenter_get(obj, &val32);
    of_bsn_shell_output_subtype_get(obj, &val32);
    of_bsn_shell_output_data_get(obj, &octets);

    value = of_bsn_shell_output_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_shell_output_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_shell_output
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_shell_output_OF_VERSION_1_0_check(
    of_bsn_shell_output_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_shell_output_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_shell_output_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_shell_output_t *dup;

        TEST_ASSERT((dup = of_bsn_shell_output_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_shell_output_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_shell_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_shell_status_OF_VERSION_1_0_populate(
    of_bsn_shell_status_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_shell_status_xid_get(obj, &val32);
    of_bsn_shell_status_experimenter_get(obj, &val32);
    of_bsn_shell_status_subtype_get(obj, &val32);
    of_bsn_shell_status_status_get(obj, &val32);

    value = of_bsn_shell_status_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_shell_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_shell_status_OF_VERSION_1_0_check(
    of_bsn_shell_status_t *obj, int value)
{

    value = of_bsn_shell_status_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_shell_status_t *dup;

        TEST_ASSERT((dup = of_bsn_shell_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_shell_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_0_populate(
    of_bsn_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_reply_xid_get(obj, &val32);
    of_bsn_stats_reply_flags_get(obj, &val16);
    of_bsn_stats_reply_experimenter_get(obj, &val32);
    of_bsn_stats_reply_subtype_get(obj, &val32);

    value = of_bsn_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_0_check(
    of_bsn_stats_reply_t *obj, int value)
{

    value = of_bsn_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_0_populate(
    of_bsn_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_request_xid_get(obj, &val32);
    of_bsn_stats_request_flags_get(obj, &val16);
    of_bsn_stats_request_experimenter_get(obj, &val32);
    of_bsn_stats_request_subtype_get(obj, &val32);

    value = of_bsn_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_0_check(
    of_bsn_stats_request_t *obj, int value)
{

    value = of_bsn_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_0_populate(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_create_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_0_check(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_0_populate(
    of_bsn_virtual_port_create_request_t *obj, int value)
{
    uint32_t val32;
    of_bsn_vport_q_in_q_t *vport;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    of_bsn_virtual_port_create_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_request_subtype_get(obj, &val32);
    {
        of_bsn_vport_q_in_q_t sub_cls;

        /* Test bind */
        of_bsn_virtual_port_create_request_vport_bind(obj, &sub_cls);
    }

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    vport = of_bsn_vport_q_in_q_new(OF_VERSION_1_0);
    TEST_ASSERT(vport != NULL);
    value = of_bsn_vport_q_in_q_OF_VERSION_1_0_populate(
        vport, value);
    TEST_ASSERT(value != 0);
    of_bsn_virtual_port_create_request_vport_set(
        obj, vport);
    of_bsn_vport_q_in_q_delete(vport);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_0_check(
    of_bsn_virtual_port_create_request_t *obj, int value)
{

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_bsn_vport_q_in_q_t *vport_ptr;

        vport_ptr = of_bsn_virtual_port_create_request_vport_get(obj);
        TEST_ASSERT(vport_ptr != NULL);
        value = of_bsn_vport_q_in_q_OF_VERSION_1_0_check(
            vport_ptr, value);
        TEST_ASSERT(value != 0);
        of_bsn_vport_q_in_q_delete(vport_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_0_populate(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_0_check(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_0_populate(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_request_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_0_check(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_0_populate(
    of_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_reply_xid_get(obj, &val32);
    of_desc_stats_reply_flags_get(obj, &val16);
    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    of_desc_stats_reply_dp_desc_get(obj, &desc_str);

    value = of_desc_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_0_check(
    of_desc_stats_reply_t *obj, int value)
{

    value = of_desc_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_0_populate(
    of_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_request_xid_get(obj, &val32);
    of_desc_stats_request_flags_get(obj, &val16);

    value = of_desc_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_0_check(
    of_desc_stats_request_t *obj, int value)
{

    value = of_desc_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_0_populate(
    of_echo_reply_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_reply_xid_get(obj, &val32);
    of_echo_reply_data_get(obj, &octets);

    value = of_echo_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_0_check(
    of_echo_reply_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_reply_t *dup;

        TEST_ASSERT((dup = of_echo_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_0_populate(
    of_echo_request_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_request_xid_get(obj, &val32);
    of_echo_request_data_get(obj, &octets);

    value = of_echo_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_0_check(
    of_echo_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_request_t *dup;

        TEST_ASSERT((dup = of_echo_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_0_populate(
    of_error_msg_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_error_msg_xid_get(obj, &val32);

    value = of_error_msg_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_0_check(
    of_error_msg_t *obj, int value)
{

    value = of_error_msg_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_error_msg_t *dup;

        TEST_ASSERT((dup = of_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_0_populate(
    of_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_xid_get(obj, &val32);
    of_experimenter_experimenter_get(obj, &val32);
    of_experimenter_subtype_get(obj, &val32);
    of_experimenter_data_get(obj, &octets);

    value = of_experimenter_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_0_check(
    of_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_t *dup;

        TEST_ASSERT((dup = of_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_0_populate(
    of_experimenter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_reply_xid_get(obj, &val32);
    of_experimenter_stats_reply_flags_get(obj, &val16);
    of_experimenter_stats_reply_experimenter_get(obj, &val32);
    of_experimenter_stats_reply_data_get(obj, &octets);

    value = of_experimenter_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_reply_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_0_check(
    of_experimenter_stats_reply_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_reply_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_0_populate(
    of_experimenter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_request_xid_get(obj, &val32);
    of_experimenter_stats_request_flags_get(obj, &val16);
    of_experimenter_stats_request_experimenter_get(obj, &val32);
    of_experimenter_stats_request_data_get(obj, &octets);

    value = of_experimenter_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_0_check(
    of_experimenter_stats_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_request_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_0_populate(
    of_features_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    of_list_port_desc_t *list;

    /* Run thru accessors after new to ensure okay */
    of_features_reply_xid_get(obj, &val32);
    of_features_reply_datapath_id_get(obj, &val64);
    of_features_reply_n_buffers_get(obj, &val32);
    of_features_reply_n_tables_get(obj, &val8);
    of_features_reply_capabilities_get(obj, &val32);
    of_features_reply_actions_get(obj, &val32);
    {
        of_list_port_desc_t sub_cls;

        /* Test bind */
        of_features_reply_ports_bind(obj, &sub_cls);
    }

    value = of_features_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_desc_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_port_desc_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_features_reply_ports_set(
        obj, list);
    of_list_port_desc_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_features_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_0_check(
    of_features_reply_t *obj, int value)
{

    value = of_features_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_desc_t *ports_ptr;

        ports_ptr = of_features_reply_ports_get(obj);
        TEST_ASSERT(ports_ptr != NULL);
        value = of_list_port_desc_OF_VERSION_1_0_check(
            ports_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_desc_delete(ports_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_features_reply_t *dup;

        TEST_ASSERT((dup = of_features_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_0_populate(
    of_features_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_features_request_xid_get(obj, &val32);

    value = of_features_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_features_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_0_check(
    of_features_request_t *obj, int value)
{

    value = of_features_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_features_request_t *dup;

        TEST_ASSERT((dup = of_features_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_0_populate(
    of_flow_add_t *obj, int value)
{
    uint32_t val32;
    of_match_t match;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_add_xid_get(obj, &val32);
    of_flow_add_match_get(obj, &match);
    of_flow_add_cookie_get(obj, &val64);
    of_flow_add_idle_timeout_get(obj, &val16);
    of_flow_add_hard_timeout_get(obj, &val16);
    of_flow_add_priority_get(obj, &val16);
    of_flow_add_buffer_id_get(obj, &val32);
    of_flow_add_out_port_get(obj, &port_no);
    of_flow_add_flags_get(obj, &val16);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_flow_add_actions_bind(obj, &sub_cls);
    }

    value = of_flow_add_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_set(
        obj, &match);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_add_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_0_check(
    of_flow_add_t *obj, int value)
{
    of_match_t match;

    value = of_flow_add_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_flow_add_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_add_t *dup;

        TEST_ASSERT((dup = of_flow_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_0_populate(
    of_flow_delete_t *obj, int value)
{
    uint32_t val32;
    of_match_t match;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_xid_get(obj, &val32);
    of_flow_delete_match_get(obj, &match);
    of_flow_delete_cookie_get(obj, &val64);
    of_flow_delete_idle_timeout_get(obj, &val16);
    of_flow_delete_hard_timeout_get(obj, &val16);
    of_flow_delete_priority_get(obj, &val16);
    of_flow_delete_buffer_id_get(obj, &val32);
    of_flow_delete_out_port_get(obj, &port_no);
    of_flow_delete_flags_get(obj, &val16);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_flow_delete_actions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_set(
        obj, &match);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_0_check(
    of_flow_delete_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_flow_delete_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_t *dup;

        TEST_ASSERT((dup = of_flow_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_0_populate(
    of_flow_delete_strict_t *obj, int value)
{
    uint32_t val32;
    of_match_t match;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_strict_xid_get(obj, &val32);
    of_flow_delete_strict_match_get(obj, &match);
    of_flow_delete_strict_cookie_get(obj, &val64);
    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    of_flow_delete_strict_priority_get(obj, &val16);
    of_flow_delete_strict_buffer_id_get(obj, &val32);
    of_flow_delete_strict_out_port_get(obj, &port_no);
    of_flow_delete_strict_flags_get(obj, &val16);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_flow_delete_strict_actions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_strict_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_set(
        obj, &match);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_0_check(
    of_flow_delete_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_strict_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_flow_delete_strict_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_strict_t *dup;

        TEST_ASSERT((dup = of_flow_delete_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_0_populate(
    of_flow_mod_t *obj, int value)
{
    uint32_t val32;
    of_match_t match;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_xid_get(obj, &val32);
    of_flow_mod_match_get(obj, &match);
    of_flow_mod_cookie_get(obj, &val64);
    of_flow_mod_idle_timeout_get(obj, &val16);
    of_flow_mod_hard_timeout_get(obj, &val16);
    of_flow_mod_priority_get(obj, &val16);
    of_flow_mod_buffer_id_get(obj, &val32);
    of_flow_mod_out_port_get(obj, &port_no);
    of_flow_mod_flags_get(obj, &val16);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_flow_mod_actions_bind(obj, &sub_cls);
    }

    value = of_flow_mod_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_set(
        obj, &match);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_0_check(
    of_flow_mod_t *obj, int value)
{
    of_match_t match;

    value = of_flow_mod_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_flow_mod_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_t *dup;

        TEST_ASSERT((dup = of_flow_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_0_populate(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_flow_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_0_check(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_flow_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_0_populate(
    of_flow_modify_t *obj, int value)
{
    uint32_t val32;
    of_match_t match;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_xid_get(obj, &val32);
    of_flow_modify_match_get(obj, &match);
    of_flow_modify_cookie_get(obj, &val64);
    of_flow_modify_idle_timeout_get(obj, &val16);
    of_flow_modify_hard_timeout_get(obj, &val16);
    of_flow_modify_priority_get(obj, &val16);
    of_flow_modify_buffer_id_get(obj, &val32);
    of_flow_modify_out_port_get(obj, &port_no);
    of_flow_modify_flags_get(obj, &val16);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_flow_modify_actions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_set(
        obj, &match);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_0_check(
    of_flow_modify_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_flow_modify_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_t *dup;

        TEST_ASSERT((dup = of_flow_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_0_populate(
    of_flow_modify_strict_t *obj, int value)
{
    uint32_t val32;
    of_match_t match;
    uint64_t val64;
    uint16_t val16;
    of_port_no_t port_no;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_strict_xid_get(obj, &val32);
    of_flow_modify_strict_match_get(obj, &match);
    of_flow_modify_strict_cookie_get(obj, &val64);
    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    of_flow_modify_strict_priority_get(obj, &val16);
    of_flow_modify_strict_buffer_id_get(obj, &val32);
    of_flow_modify_strict_out_port_get(obj, &port_no);
    of_flow_modify_strict_flags_get(obj, &val16);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_flow_modify_strict_actions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_strict_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_set(
        obj, &match);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_0_check(
    of_flow_modify_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_strict_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_flow_modify_strict_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_strict_t *dup;

        TEST_ASSERT((dup = of_flow_modify_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_removed
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_0_populate(
    of_flow_removed_t *obj, int value)
{
    uint32_t val32;
    of_match_t match;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_flow_removed_xid_get(obj, &val32);
    of_flow_removed_match_get(obj, &match);
    of_flow_removed_cookie_get(obj, &val64);
    of_flow_removed_priority_get(obj, &val16);
    of_flow_removed_reason_get(obj, &val8);
    of_flow_removed_duration_sec_get(obj, &val32);
    of_flow_removed_duration_nsec_get(obj, &val32);
    of_flow_removed_idle_timeout_get(obj, &val16);
    of_flow_removed_packet_count_get(obj, &val64);
    of_flow_removed_byte_count_get(obj, &val64);

    value = of_flow_removed_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_removed
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_0_check(
    of_flow_removed_t *obj, int value)
{
    of_match_t match;

    value = of_flow_removed_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_removed_t *dup;

        TEST_ASSERT((dup = of_flow_removed_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_removed_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_0_populate(
    of_flow_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_flow_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_reply_xid_get(obj, &val32);
    of_flow_stats_reply_flags_get(obj, &val16);
    {
        of_list_flow_stats_entry_t sub_cls;

        /* Test bind */
        of_flow_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_flow_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_flow_stats_entry_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_flow_stats_entry_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_reply_entries_set(
        obj, list);
    of_list_flow_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_0_check(
    of_flow_stats_reply_t *obj, int value)
{

    value = of_flow_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_flow_stats_entry_t *entries_ptr;

        entries_ptr = of_flow_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_flow_stats_entry_OF_VERSION_1_0_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_flow_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_reply_t *dup;

        TEST_ASSERT((dup = of_flow_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_0_populate(
    of_flow_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_match_t match;
    uint8_t val8;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_request_xid_get(obj, &val32);
    of_flow_stats_request_flags_get(obj, &val16);
    of_flow_stats_request_match_get(obj, &match);
    of_flow_stats_request_table_id_get(obj, &val8);
    of_flow_stats_request_out_port_get(obj, &port_no);

    value = of_flow_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_0_check(
    of_flow_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_request_t *dup;

        TEST_ASSERT((dup = of_flow_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_0_populate(
    of_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_get_config_reply_xid_get(obj, &val32);
    of_get_config_reply_flags_get(obj, &val16);
    of_get_config_reply_miss_send_len_get(obj, &val16);

    value = of_get_config_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_0_check(
    of_get_config_reply_t *obj, int value)
{

    value = of_get_config_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_0_populate(
    of_get_config_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_get_config_request_xid_get(obj, &val32);

    value = of_get_config_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_0_check(
    of_get_config_request_t *obj, int value)
{

    value = of_get_config_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_request_t *dup;

        TEST_ASSERT((dup = of_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_0_populate(
    of_hello_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_hello_xid_get(obj, &val32);

    value = of_hello_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_hello
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_0_check(
    of_hello_t *obj, int value)
{

    value = of_hello_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_t *dup;

        TEST_ASSERT((dup = of_hello_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_0_populate(
    of_hello_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_hello_failed_error_msg_xid_get(obj, &val32);
    of_hello_failed_error_msg_code_get(obj, &val16);
    of_hello_failed_error_msg_data_get(obj, &octets);

    value = of_hello_failed_error_msg_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_hello_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_0_check(
    of_hello_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_hello_failed_error_msg_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_hello_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_nicira_controller_role_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_nicira_controller_role_reply_OF_VERSION_1_0_populate(
    of_nicira_controller_role_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_nicira_controller_role_reply_xid_get(obj, &val32);
    of_nicira_controller_role_reply_experimenter_get(obj, &val32);
    of_nicira_controller_role_reply_subtype_get(obj, &val32);
    of_nicira_controller_role_reply_role_get(obj, &val32);

    value = of_nicira_controller_role_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_nicira_controller_role_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_nicira_controller_role_reply_OF_VERSION_1_0_check(
    of_nicira_controller_role_reply_t *obj, int value)
{

    value = of_nicira_controller_role_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_nicira_controller_role_reply_t *dup;

        TEST_ASSERT((dup = of_nicira_controller_role_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_nicira_controller_role_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_nicira_controller_role_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_nicira_controller_role_request_OF_VERSION_1_0_populate(
    of_nicira_controller_role_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_nicira_controller_role_request_xid_get(obj, &val32);
    of_nicira_controller_role_request_experimenter_get(obj, &val32);
    of_nicira_controller_role_request_subtype_get(obj, &val32);
    of_nicira_controller_role_request_role_get(obj, &val32);

    value = of_nicira_controller_role_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_nicira_controller_role_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_nicira_controller_role_request_OF_VERSION_1_0_check(
    of_nicira_controller_role_request_t *obj, int value)
{

    value = of_nicira_controller_role_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_nicira_controller_role_request_t *dup;

        TEST_ASSERT((dup = of_nicira_controller_role_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_nicira_controller_role_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_nicira_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_0_populate(
    of_nicira_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_nicira_header_xid_get(obj, &val32);
    of_nicira_header_experimenter_get(obj, &val32);
    of_nicira_header_subtype_get(obj, &val32);

    value = of_nicira_header_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_nicira_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_0_check(
    of_nicira_header_t *obj, int value)
{

    value = of_nicira_header_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_nicira_header_t *dup;

        TEST_ASSERT((dup = of_nicira_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_nicira_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_0_populate(
    of_packet_in_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_in_xid_get(obj, &val32);
    of_packet_in_buffer_id_get(obj, &val32);
    of_packet_in_total_len_get(obj, &val16);
    of_packet_in_in_port_get(obj, &port_no);
    of_packet_in_reason_get(obj, &val8);
    of_packet_in_data_get(obj, &octets);

    value = of_packet_in_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_in_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_0_check(
    of_packet_in_t *obj, int value)
{
    of_octets_t octets;

    value = of_packet_in_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_packet_in_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_in_t *dup;

        TEST_ASSERT((dup = of_packet_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_0_populate(
    of_packet_out_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_action_t *list;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_out_xid_get(obj, &val32);
    of_packet_out_buffer_id_get(obj, &val32);
    of_packet_out_in_port_get(obj, &port_no);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_packet_out_actions_bind(obj, &sub_cls);
    }
    of_packet_out_data_get(obj, &octets);

    value = of_packet_out_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_out_actions_set(
        obj, list);
    of_list_action_delete(list);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_out_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_0_check(
    of_packet_out_t *obj, int value)
{
    of_octets_t octets;

    value = of_packet_out_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_packet_out_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }
    of_packet_out_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_out_t *dup;

        TEST_ASSERT((dup = of_packet_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_0_populate(
    of_port_mod_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_xid_get(obj, &val32);
    of_port_mod_port_no_get(obj, &port_no);
    of_port_mod_hw_addr_get(obj, &mac_addr);
    of_port_mod_config_get(obj, &val32);
    of_port_mod_mask_get(obj, &val32);
    of_port_mod_advertise_get(obj, &val32);

    value = of_port_mod_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_0_check(
    of_port_mod_t *obj, int value)
{

    value = of_port_mod_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_t *dup;

        TEST_ASSERT((dup = of_port_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_0_populate(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    of_port_mod_failed_error_msg_code_get(obj, &val16);
    of_port_mod_failed_error_msg_data_get(obj, &octets);

    value = of_port_mod_failed_error_msg_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_port_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_0_check(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_port_mod_failed_error_msg_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_port_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_0_populate(
    of_port_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_port_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_reply_xid_get(obj, &val32);
    of_port_stats_reply_flags_get(obj, &val16);
    {
        of_list_port_stats_entry_t sub_cls;

        /* Test bind */
        of_port_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_port_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_stats_entry_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_port_stats_entry_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_port_stats_reply_entries_set(
        obj, list);
    of_list_port_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_0_check(
    of_port_stats_reply_t *obj, int value)
{

    value = of_port_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_stats_entry_t *entries_ptr;

        entries_ptr = of_port_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_port_stats_entry_OF_VERSION_1_0_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_reply_t *dup;

        TEST_ASSERT((dup = of_port_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_0_populate(
    of_port_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_request_xid_get(obj, &val32);
    of_port_stats_request_flags_get(obj, &val16);
    of_port_stats_request_port_no_get(obj, &port_no);

    value = of_port_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_0_check(
    of_port_stats_request_t *obj, int value)
{

    value = of_port_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_request_t *dup;

        TEST_ASSERT((dup = of_port_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_0_populate(
    of_port_status_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_port_desc_t *port_desc;

    /* Run thru accessors after new to ensure okay */
    of_port_status_xid_get(obj, &val32);
    of_port_status_reason_get(obj, &val8);
    {
        of_port_desc_t sub_cls;

        /* Test bind */
        of_port_status_desc_bind(obj, &sub_cls);
    }

    value = of_port_status_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    port_desc = of_port_desc_new(OF_VERSION_1_0);
    TEST_ASSERT(port_desc != NULL);
    value = of_port_desc_OF_VERSION_1_0_populate(
        port_desc, value);
    TEST_ASSERT(value != 0);
    of_port_status_desc_set(
        obj, port_desc);
    of_port_desc_delete(port_desc);

    return value;
}

/**
 * Check all members of an object of type of_port_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_0_check(
    of_port_status_t *obj, int value)
{

    value = of_port_status_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_port_desc_t *desc_ptr;

        desc_ptr = of_port_status_desc_get(obj);
        TEST_ASSERT(desc_ptr != NULL);
        value = of_port_desc_OF_VERSION_1_0_check(
            desc_ptr, value);
        TEST_ASSERT(value != 0);
        of_port_desc_delete(desc_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_status_t *dup;

        TEST_ASSERT((dup = of_port_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_0_populate(
    of_queue_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_packet_queue_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_reply_xid_get(obj, &val32);
    of_queue_get_config_reply_port_get(obj, &port_no);
    {
        of_list_packet_queue_t sub_cls;

        /* Test bind */
        of_queue_get_config_reply_queues_bind(obj, &sub_cls);
    }

    value = of_queue_get_config_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_packet_queue_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_packet_queue_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_get_config_reply_queues_set(
        obj, list);
    of_list_packet_queue_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_0_check(
    of_queue_get_config_reply_t *obj, int value)
{

    value = of_queue_get_config_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_packet_queue_t *queues_ptr;

        queues_ptr = of_queue_get_config_reply_queues_get(obj);
        TEST_ASSERT(queues_ptr != NULL);
        value = of_list_packet_queue_OF_VERSION_1_0_check(
            queues_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_packet_queue_delete(queues_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_0_populate(
    of_queue_get_config_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_request_xid_get(obj, &val32);
    of_queue_get_config_request_port_get(obj, &port_no);

    value = of_queue_get_config_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_0_check(
    of_queue_get_config_request_t *obj, int value)
{

    value = of_queue_get_config_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_request_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_op_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_0_populate(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    of_queue_op_failed_error_msg_code_get(obj, &val16);
    of_queue_op_failed_error_msg_data_get(obj, &octets);

    value = of_queue_op_failed_error_msg_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_queue_op_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_0_check(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_queue_op_failed_error_msg_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_op_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_queue_op_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_op_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_0_populate(
    of_queue_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_queue_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_reply_xid_get(obj, &val32);
    of_queue_stats_reply_flags_get(obj, &val16);
    {
        of_list_queue_stats_entry_t sub_cls;

        /* Test bind */
        of_queue_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_queue_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_stats_entry_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_stats_entry_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_stats_reply_entries_set(
        obj, list);
    of_list_queue_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_0_check(
    of_queue_stats_reply_t *obj, int value)
{

    value = of_queue_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_stats_entry_t *entries_ptr;

        entries_ptr = of_queue_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_queue_stats_entry_OF_VERSION_1_0_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_reply_t *dup;

        TEST_ASSERT((dup = of_queue_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_0_populate(
    of_queue_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_request_xid_get(obj, &val32);
    of_queue_stats_request_flags_get(obj, &val16);
    of_queue_stats_request_port_no_get(obj, &port_no);
    of_queue_stats_request_queue_id_get(obj, &val32);

    value = of_queue_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_0_check(
    of_queue_stats_request_t *obj, int value)
{

    value = of_queue_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_request_t *dup;

        TEST_ASSERT((dup = of_queue_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_set_config
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_0_populate(
    of_set_config_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_set_config_xid_get(obj, &val32);
    of_set_config_flags_get(obj, &val16);
    of_set_config_miss_send_len_get(obj, &val16);

    value = of_set_config_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_set_config
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_0_check(
    of_set_config_t *obj, int value)
{

    value = of_set_config_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_set_config_t *dup;

        TEST_ASSERT((dup = of_set_config_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_set_config_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_0_populate(
    of_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_reply_xid_get(obj, &val32);
    of_stats_reply_flags_get(obj, &val16);

    value = of_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_0_check(
    of_stats_reply_t *obj, int value)
{

    value = of_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_reply_t *dup;

        TEST_ASSERT((dup = of_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_0_populate(
    of_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_request_xid_get(obj, &val32);
    of_stats_request_flags_get(obj, &val16);

    value = of_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_0_check(
    of_stats_request_t *obj, int value)
{

    value = of_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_request_t *dup;

        TEST_ASSERT((dup = of_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_0_populate(
    of_table_mod_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_table_mod_xid_get(obj, &val32);
    of_table_mod_table_id_get(obj, &val8);
    of_table_mod_config_get(obj, &val32);

    value = of_table_mod_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_0_check(
    of_table_mod_t *obj, int value)
{

    value = of_table_mod_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_mod_t *dup;

        TEST_ASSERT((dup = of_table_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_0_populate(
    of_table_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_table_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_reply_xid_get(obj, &val32);
    of_table_stats_reply_flags_get(obj, &val16);
    {
        of_list_table_stats_entry_t sub_cls;

        /* Test bind */
        of_table_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_table_stats_reply_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_table_stats_entry_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_table_stats_entry_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_stats_reply_entries_set(
        obj, list);
    of_list_table_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_0_check(
    of_table_stats_reply_t *obj, int value)
{

    value = of_table_stats_reply_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_table_stats_entry_t *entries_ptr;

        entries_ptr = of_table_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_table_stats_entry_OF_VERSION_1_0_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_table_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_reply_t *dup;

        TEST_ASSERT((dup = of_table_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_0_populate(
    of_table_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_request_xid_get(obj, &val32);
    of_table_stats_request_flags_get(obj, &val16);

    value = of_table_stats_request_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_0_check(
    of_table_stats_request_t *obj, int value)
{

    value = of_table_stats_request_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_request_t *dup;

        TEST_ASSERT((dup = of_table_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_0_populate(
    of_action_bsn_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_experimenter_get(obj, &val32);
    of_action_bsn_subtype_get(obj, &val32);

    value = of_action_bsn_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_0_check(
    of_action_bsn_t *obj, int value)
{

    value = of_action_bsn_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_t *dup;

        TEST_ASSERT((dup = of_action_bsn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_mirror
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_0_populate(
    of_action_bsn_mirror_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_mirror_experimenter_get(obj, &val32);
    of_action_bsn_mirror_subtype_get(obj, &val32);
    of_action_bsn_mirror_dest_port_get(obj, &val32);
    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    of_action_bsn_mirror_copy_stage_get(obj, &val8);

    value = of_action_bsn_mirror_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_mirror
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_0_check(
    of_action_bsn_mirror_t *obj, int value)
{

    value = of_action_bsn_mirror_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_mirror_t *dup;

        TEST_ASSERT((dup = of_action_bsn_mirror_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_mirror_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_set_tunnel_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_populate(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_set_tunnel_dst_experimenter_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_subtype_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_set_tunnel_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_check(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_set_tunnel_dst_t *dup;

        TEST_ASSERT((dup = of_action_bsn_set_tunnel_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_set_tunnel_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_enqueue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_enqueue_OF_VERSION_1_0_populate(
    of_action_enqueue_t *obj, int value)
{
    of_port_no_t port_no;
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_enqueue_port_get(obj, &port_no);
    of_action_enqueue_queue_id_get(obj, &val32);

    value = of_action_enqueue_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_enqueue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_enqueue_OF_VERSION_1_0_check(
    of_action_enqueue_t *obj, int value)
{

    value = of_action_enqueue_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_enqueue_t *dup;

        TEST_ASSERT((dup = of_action_enqueue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_enqueue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_0_populate(
    of_action_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_action_experimenter_experimenter_get(obj, &val32);
    of_action_experimenter_data_get(obj, &octets);

    value = of_action_experimenter_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_action_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_0_check(
    of_action_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_action_experimenter_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_experimenter_t *dup;

        TEST_ASSERT((dup = of_action_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_0_populate(
    of_action_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_header_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_0_check(
    of_action_header_t *obj, int value)
{

    value = of_action_header_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_header_t *dup;

        TEST_ASSERT((dup = of_action_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_0_populate(
    of_action_nicira_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_experimenter_get(obj, &val32);
    of_action_nicira_subtype_get(obj, &val16);

    value = of_action_nicira_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_0_check(
    of_action_nicira_t *obj, int value)
{

    value = of_action_nicira_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira_dec_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_0_populate(
    of_action_nicira_dec_ttl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_dec_ttl_experimenter_get(obj, &val32);
    of_action_nicira_dec_ttl_subtype_get(obj, &val16);

    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira_dec_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_0_check(
    of_action_nicira_dec_ttl_t *obj, int value)
{

    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_dec_ttl_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dec_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_dec_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_output
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_0_populate(
    of_action_output_t *obj, int value)
{
    of_port_no_t port_no;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_output_port_get(obj, &port_no);
    of_action_output_max_len_get(obj, &val16);

    value = of_action_output_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_output
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_0_check(
    of_action_output_t *obj, int value)
{

    value = of_action_output_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_output_t *dup;

        TEST_ASSERT((dup = of_action_output_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_output_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_dl_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_dst_OF_VERSION_1_0_populate(
    of_action_set_dl_dst_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_action_set_dl_dst_dl_addr_get(obj, &mac_addr);

    value = of_action_set_dl_dst_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_dl_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_dst_OF_VERSION_1_0_check(
    of_action_set_dl_dst_t *obj, int value)
{

    value = of_action_set_dl_dst_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_dl_dst_t *dup;

        TEST_ASSERT((dup = of_action_set_dl_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_dl_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_dl_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_src_OF_VERSION_1_0_populate(
    of_action_set_dl_src_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_action_set_dl_src_dl_addr_get(obj, &mac_addr);

    value = of_action_set_dl_src_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_dl_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_src_OF_VERSION_1_0_check(
    of_action_set_dl_src_t *obj, int value)
{

    value = of_action_set_dl_src_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_dl_src_t *dup;

        TEST_ASSERT((dup = of_action_set_dl_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_dl_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_dst_OF_VERSION_1_0_populate(
    of_action_set_nw_dst_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_dst_nw_addr_get(obj, &val32);

    value = of_action_set_nw_dst_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_dst_OF_VERSION_1_0_check(
    of_action_set_nw_dst_t *obj, int value)
{

    value = of_action_set_nw_dst_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_dst_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_src_OF_VERSION_1_0_populate(
    of_action_set_nw_src_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_src_nw_addr_get(obj, &val32);

    value = of_action_set_nw_src_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_src_OF_VERSION_1_0_check(
    of_action_set_nw_src_t *obj, int value)
{

    value = of_action_set_nw_src_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_src_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_tos
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_tos_OF_VERSION_1_0_populate(
    of_action_set_nw_tos_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_tos_nw_tos_get(obj, &val8);

    value = of_action_set_nw_tos_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_tos
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_tos_OF_VERSION_1_0_check(
    of_action_set_nw_tos_t *obj, int value)
{

    value = of_action_set_nw_tos_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_tos_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_tos_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_tos_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_tp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_dst_OF_VERSION_1_0_populate(
    of_action_set_tp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_set_tp_dst_tp_port_get(obj, &val16);

    value = of_action_set_tp_dst_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_tp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_dst_OF_VERSION_1_0_check(
    of_action_set_tp_dst_t *obj, int value)
{

    value = of_action_set_tp_dst_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_tp_dst_t *dup;

        TEST_ASSERT((dup = of_action_set_tp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_tp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_tp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_src_OF_VERSION_1_0_populate(
    of_action_set_tp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_set_tp_src_tp_port_get(obj, &val16);

    value = of_action_set_tp_src_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_tp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_src_OF_VERSION_1_0_check(
    of_action_set_tp_src_t *obj, int value)
{

    value = of_action_set_tp_src_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_tp_src_t *dup;

        TEST_ASSERT((dup = of_action_set_tp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_tp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_vlan_pcp
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_pcp_OF_VERSION_1_0_populate(
    of_action_set_vlan_pcp_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_vlan_pcp_vlan_pcp_get(obj, &val8);

    value = of_action_set_vlan_pcp_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_vlan_pcp
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_pcp_OF_VERSION_1_0_check(
    of_action_set_vlan_pcp_t *obj, int value)
{

    value = of_action_set_vlan_pcp_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_vlan_pcp_t *dup;

        TEST_ASSERT((dup = of_action_set_vlan_pcp_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_vlan_pcp_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_vlan_vid
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_vid_OF_VERSION_1_0_populate(
    of_action_set_vlan_vid_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_set_vlan_vid_vlan_vid_get(obj, &val16);

    value = of_action_set_vlan_vid_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_vlan_vid
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_vid_OF_VERSION_1_0_check(
    of_action_set_vlan_vid_t *obj, int value)
{

    value = of_action_set_vlan_vid_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_vlan_vid_t *dup;

        TEST_ASSERT((dup = of_action_set_vlan_vid_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_vlan_vid_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_strip_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_strip_vlan_OF_VERSION_1_0_populate(
    of_action_strip_vlan_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_strip_vlan_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_strip_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_strip_vlan_OF_VERSION_1_0_check(
    of_action_strip_vlan_t *obj, int value)
{

    value = of_action_strip_vlan_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_strip_vlan_t *dup;

        TEST_ASSERT((dup = of_action_strip_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_strip_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_interface
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_0_populate(
    of_bsn_interface_t *obj, int value)
{
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    of_bsn_interface_name_get(obj, &port_name);
    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);

    value = of_bsn_interface_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_interface
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_0_check(
    of_bsn_interface_t *obj, int value)
{

    value = of_bsn_interface_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_bsn_interface_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_interface_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_0_populate(
    of_bsn_vport_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_bsn_vport_header_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_0_check(
    of_bsn_vport_header_t *obj, int value)
{

    value = of_bsn_vport_header_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_header_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_q_in_q
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_0_populate(
    of_bsn_vport_q_in_q_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    /* Run thru accessors after new to ensure okay */
    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);

    value = of_bsn_vport_q_in_q_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_q_in_q
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_0_check(
    of_bsn_vport_q_in_q_t *obj, int value)
{

    value = of_bsn_vport_q_in_q_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_q_in_q_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_q_in_q_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_q_in_q_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_0_populate(
    of_flow_stats_entry_t *obj, int value)
{
    uint8_t val8;
    of_match_t match;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_entry_table_id_get(obj, &val8);
    of_flow_stats_entry_match_get(obj, &match);
    of_flow_stats_entry_duration_sec_get(obj, &val32);
    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    of_flow_stats_entry_priority_get(obj, &val16);
    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    of_flow_stats_entry_cookie_get(obj, &val64);
    of_flow_stats_entry_packet_count_get(obj, &val64);
    of_flow_stats_entry_byte_count_get(obj, &val64);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_flow_stats_entry_actions_bind(obj, &sub_cls);
    }

    value = of_flow_stats_entry_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_0, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_set(
        obj, &match);

    list = of_list_action_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_0_check(
    of_flow_stats_entry_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_entry_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_0, value);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_flow_stats_entry_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_0_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_flow_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_0_populate(
    of_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_header_xid_get(obj, &val32);

    value = of_header_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_0_check(
    of_header_t *obj, int value)
{

    value = of_header_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_header_t *dup;

        TEST_ASSERT((dup = of_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_match_v1
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_match_v1_OF_VERSION_1_0_populate(
    of_match_v1_t *obj, int value)
{
    of_wc_bmap_t wc_bmap;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    uint16_t val16;
    uint8_t val8;
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_match_v1_wildcards_get(obj, &wc_bmap);
    of_match_v1_in_port_get(obj, &port_no);
    of_match_v1_eth_src_get(obj, &mac_addr);
    of_match_v1_eth_dst_get(obj, &mac_addr);
    of_match_v1_vlan_vid_get(obj, &val16);
    of_match_v1_vlan_pcp_get(obj, &val8);
    of_match_v1_eth_type_get(obj, &val16);
    of_match_v1_ip_dscp_get(obj, &val8);
    of_match_v1_ip_proto_get(obj, &val8);
    of_match_v1_ipv4_src_get(obj, &ipv4);
    of_match_v1_ipv4_dst_get(obj, &ipv4);
    of_match_v1_tcp_src_get(obj, &val16);
    of_match_v1_tcp_dst_get(obj, &val16);

    value = of_match_v1_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_match_v1
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_match_v1_OF_VERSION_1_0_check(
    of_match_v1_t *obj, int value)
{

    value = of_match_v1_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_match_v1_t *dup;

        TEST_ASSERT((dup = of_match_v1_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_match_v1_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_0_populate(
    of_packet_queue_t *obj, int value)
{
    uint32_t val32;
    of_list_queue_prop_t *list;

    /* Run thru accessors after new to ensure okay */
    of_packet_queue_queue_id_get(obj, &val32);
    {
        of_list_queue_prop_t sub_cls;

        /* Test bind */
        of_packet_queue_properties_bind(obj, &sub_cls);
    }

    value = of_packet_queue_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_prop_new(OF_VERSION_1_0);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_prop_OF_VERSION_1_0_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_queue_properties_set(
        obj, list);
    of_list_queue_prop_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_packet_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_0_check(
    of_packet_queue_t *obj, int value)
{

    value = of_packet_queue_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_prop_t *properties_ptr;

        properties_ptr = of_packet_queue_properties_get(obj);
        TEST_ASSERT(properties_ptr != NULL);
        value = of_list_queue_prop_OF_VERSION_1_0_check(
            properties_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_prop_delete(properties_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_queue_t *dup;

        TEST_ASSERT((dup = of_packet_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_desc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_0_populate(
    of_port_desc_t *obj, int value)
{
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_port_desc_port_no_get(obj, &port_no);
    of_port_desc_hw_addr_get(obj, &mac_addr);
    of_port_desc_name_get(obj, &port_name);
    of_port_desc_config_get(obj, &val32);
    of_port_desc_state_get(obj, &val32);
    of_port_desc_curr_get(obj, &val32);
    of_port_desc_advertised_get(obj, &val32);
    of_port_desc_supported_get(obj, &val32);
    of_port_desc_peer_get(obj, &val32);

    value = of_port_desc_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_desc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_0_check(
    of_port_desc_t *obj, int value)
{

    value = of_port_desc_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_desc_t *dup;

        TEST_ASSERT((dup = of_port_desc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_desc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_0_populate(
    of_port_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_entry_port_no_get(obj, &port_no);
    of_port_stats_entry_rx_packets_get(obj, &val64);
    of_port_stats_entry_tx_packets_get(obj, &val64);
    of_port_stats_entry_rx_bytes_get(obj, &val64);
    of_port_stats_entry_tx_bytes_get(obj, &val64);
    of_port_stats_entry_rx_dropped_get(obj, &val64);
    of_port_stats_entry_tx_dropped_get(obj, &val64);
    of_port_stats_entry_rx_errors_get(obj, &val64);
    of_port_stats_entry_tx_errors_get(obj, &val64);
    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    of_port_stats_entry_rx_over_err_get(obj, &val64);
    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    of_port_stats_entry_collisions_get(obj, &val64);

    value = of_port_stats_entry_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_0_check(
    of_port_stats_entry_t *obj, int value)
{

    value = of_port_stats_entry_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_port_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_0_populate(
    of_queue_prop_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_queue_prop_header_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_0_check(
    of_queue_prop_header_t *obj, int value)
{

    value = of_queue_prop_header_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_header_t *dup;

        TEST_ASSERT((dup = of_queue_prop_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_min_rate
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_0_populate(
    of_queue_prop_min_rate_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_min_rate_rate_get(obj, &val16);

    value = of_queue_prop_min_rate_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_min_rate
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_0_check(
    of_queue_prop_min_rate_t *obj, int value)
{

    value = of_queue_prop_min_rate_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_min_rate_t *dup;

        TEST_ASSERT((dup = of_queue_prop_min_rate_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_min_rate_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_0_populate(
    of_queue_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_entry_port_no_get(obj, &port_no);
    of_queue_stats_entry_queue_id_get(obj, &val32);
    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    of_queue_stats_entry_tx_packets_get(obj, &val64);
    of_queue_stats_entry_tx_errors_get(obj, &val64);

    value = of_queue_stats_entry_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_0_check(
    of_queue_stats_entry_t *obj, int value)
{

    value = of_queue_stats_entry_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_queue_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_0_populate(
    of_table_stats_entry_t *obj, int value)
{
    uint8_t val8;
    of_table_name_t table_name;
    of_wc_bmap_t wc_bmap;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_entry_table_id_get(obj, &val8);
    of_table_stats_entry_name_get(obj, &table_name);
    of_table_stats_entry_wildcards_get(obj, &wc_bmap);
    of_table_stats_entry_max_entries_get(obj, &val32);
    of_table_stats_entry_active_count_get(obj, &val32);
    of_table_stats_entry_lookup_count_get(obj, &val64);
    of_table_stats_entry_matched_count_get(obj, &val64);

    value = of_table_stats_entry_OF_VERSION_1_0_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_0_check(
    of_table_stats_entry_t *obj, int value)
{

    value = of_table_stats_entry_OF_VERSION_1_0_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_table_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_action with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_0_populate(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_enqueue_t *enqueue;
    of_action_strip_vlan_t *strip_vlan;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_set_tp_src_t *set_tp_src;
    of_action_set_dl_src_t *set_dl_src;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    set_vlan_pcp = &elt.set_vlan_pcp;
    enqueue = &elt.enqueue;
    strip_vlan = &elt.strip_vlan;
    set_dl_dst = &elt.set_dl_dst;
    set_tp_src = &elt.set_tp_src;
    set_dl_src = &elt.set_dl_src;
    output = &elt.output;

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_0_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_0_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_tos */

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_0_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_0_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_dst */

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_0_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_0_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_src */

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_0_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_0_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_vid */

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_dst */

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_0_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_0_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_pcp */

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_enqueue */

    of_action_enqueue_init(enqueue, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)enqueue);
    value = of_action_enqueue_OF_VERSION_1_0_populate(
        enqueue, value);
    cur_len += enqueue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_enqueue_init(enqueue, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)enqueue);
    value = of_action_enqueue_OF_VERSION_1_0_populate(
        enqueue, value);
    cur_len += enqueue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_strip_vlan */

    of_action_strip_vlan_init(strip_vlan, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)strip_vlan);
    value = of_action_strip_vlan_OF_VERSION_1_0_populate(
        strip_vlan, value);
    cur_len += strip_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_strip_vlan_init(strip_vlan, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)strip_vlan);
    value = of_action_strip_vlan_OF_VERSION_1_0_populate(
        strip_vlan, value);
    cur_len += strip_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_dst */

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_0_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_0_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_src */

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_0_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_0_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_src */

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_0_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_0_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_0_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_0, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_0_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_action generated by
 * of_list_action_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_0_check(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_enqueue_t *enqueue;
    of_action_strip_vlan_t *strip_vlan;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_set_tp_src_t *set_tp_src;
    of_action_set_dl_src_t *set_dl_src;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    set_vlan_pcp = &elt.set_vlan_pcp;
    enqueue = &elt.enqueue;
    strip_vlan = &elt.strip_vlan;
    set_dl_dst = &elt.set_dl_dst;
    set_tp_src = &elt.set_tp_src;
    set_dl_src = &elt.set_dl_src;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_0_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_0_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_tos */
    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_0_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_0_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_0_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_dst */
    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_0_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_0_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_src */
    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_0_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_0_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_vid */
    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_0_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_0_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_dst */
    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_0_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_0_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_pcp */
    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_0_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type enqueue */
    TEST_ASSERT(enqueue->length == 16);
    if (enqueue->wire_length_get != NULL) {
        int length;

        enqueue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(enqueue->object_id == OF_ACTION_ENQUEUE);
    value = of_action_enqueue_OF_VERSION_1_0_check(
        enqueue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(enqueue->length == 16);
    if (enqueue->wire_length_get != NULL) {
        int length;

        enqueue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(enqueue->object_id == OF_ACTION_ENQUEUE);
    value = of_action_enqueue_OF_VERSION_1_0_check(
        enqueue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type strip_vlan */
    TEST_ASSERT(strip_vlan->length == 8);
    if (strip_vlan->wire_length_get != NULL) {
        int length;

        strip_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(strip_vlan->object_id == OF_ACTION_STRIP_VLAN);
    value = of_action_strip_vlan_OF_VERSION_1_0_check(
        strip_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(strip_vlan->length == 8);
    if (strip_vlan->wire_length_get != NULL) {
        int length;

        strip_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(strip_vlan->object_id == OF_ACTION_STRIP_VLAN);
    value = of_action_strip_vlan_OF_VERSION_1_0_check(
        strip_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_dst */
    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_0_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_0_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_src */
    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_0_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_0_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_src */
    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_0_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_0_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 8);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_0_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 8);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_0_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_ACTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 30);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_action_t *dup;

        TEST_ASSERT((dup = of_list_action_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_action_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_interface with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_0_populate(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * of_list_bsn_interface_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_0_check(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_INTERFACE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_list_bsn_interface_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_interface_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_flow_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_0_populate(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * of_list_flow_stats_entry_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_0_check(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_FLOW_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_flow_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_flow_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_packet_queue with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_0_populate(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * of_list_packet_queue_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_0_check(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PACKET_QUEUE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_packet_queue_t *dup;

        TEST_ASSERT((dup = of_list_packet_queue_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_packet_queue_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_desc with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_0_populate(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * of_list_port_desc_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_0_check(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_DESC_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_desc_t *dup;

        TEST_ASSERT((dup = of_list_port_desc_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_desc_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_0_populate(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * of_list_port_stats_entry_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_0_check(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_port_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_prop with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_0_populate(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_0, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_0, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * of_list_queue_prop_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_0_check(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_0_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_PROP_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_prop_t *dup;

        TEST_ASSERT((dup = of_list_queue_prop_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_prop_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_0_populate(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * of_list_queue_stats_entry_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_0_check(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_queue_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_table_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_0_populate(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_0, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_0_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * of_list_table_stats_entry_OF_VERSION_1_0_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_0_check(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_0_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_TABLE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_table_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_table_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_1_populate(
    of_aggregate_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_reply_xid_get(obj, &val32);
    of_aggregate_stats_reply_flags_get(obj, &val16);
    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    of_aggregate_stats_reply_flow_count_get(obj, &val32);

    value = of_aggregate_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_1_check(
    of_aggregate_stats_reply_t *obj, int value)
{

    value = of_aggregate_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_reply_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_1_populate(
    of_aggregate_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_request_xid_get(obj, &val32);
    of_aggregate_stats_request_flags_get(obj, &val16);
    of_aggregate_stats_request_table_id_get(obj, &val8);
    of_aggregate_stats_request_out_port_get(obj, &port_no);
    of_aggregate_stats_request_out_group_get(obj, &val32);
    of_aggregate_stats_request_cookie_get(obj, &val64);
    of_aggregate_stats_request_cookie_mask_get(obj, &val64);
    of_aggregate_stats_request_match_get(obj, &match);

    value = of_aggregate_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_1_check(
    of_aggregate_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_aggregate_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_request_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_action_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_1_populate(
    of_bad_action_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_action_error_msg_xid_get(obj, &val32);
    of_bad_action_error_msg_code_get(obj, &val16);
    of_bad_action_error_msg_data_get(obj, &octets);

    value = of_bad_action_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_action_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_1_check(
    of_bad_action_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_action_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_action_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_action_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_action_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_instruction_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_instruction_error_msg_OF_VERSION_1_1_populate(
    of_bad_instruction_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_instruction_error_msg_xid_get(obj, &val32);
    of_bad_instruction_error_msg_code_get(obj, &val16);
    of_bad_instruction_error_msg_data_get(obj, &octets);

    value = of_bad_instruction_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_instruction_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_instruction_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_instruction_error_msg_OF_VERSION_1_1_check(
    of_bad_instruction_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_instruction_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_instruction_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_instruction_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_instruction_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_instruction_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_match_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_match_error_msg_OF_VERSION_1_1_populate(
    of_bad_match_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_match_error_msg_xid_get(obj, &val32);
    of_bad_match_error_msg_code_get(obj, &val16);
    of_bad_match_error_msg_data_get(obj, &octets);

    value = of_bad_match_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_match_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_match_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_match_error_msg_OF_VERSION_1_1_check(
    of_bad_match_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_match_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_match_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_match_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_match_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_match_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_request_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_1_populate(
    of_bad_request_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_request_error_msg_xid_get(obj, &val32);
    of_bad_request_error_msg_code_get(obj, &val16);
    of_bad_request_error_msg_data_get(obj, &octets);

    value = of_bad_request_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_request_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_1_check(
    of_bad_request_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_request_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_request_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_request_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_request_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_1_populate(
    of_barrier_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_reply_xid_get(obj, &val32);

    value = of_barrier_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_1_check(
    of_barrier_reply_t *obj, int value)
{

    value = of_barrier_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_reply_t *dup;

        TEST_ASSERT((dup = of_barrier_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_1_populate(
    of_barrier_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_request_xid_get(obj, &val32);

    value = of_barrier_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_1_check(
    of_barrier_request_t *obj, int value)
{

    value = of_barrier_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_request_t *dup;

        TEST_ASSERT((dup = of_barrier_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_1_populate(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    of_bsn_bw_clear_data_reply_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_reply_subtype_get(obj, &val32);
    of_bsn_bw_clear_data_reply_status_get(obj, &val32);

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_1_check(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_1_populate(
    of_bsn_bw_clear_data_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    of_bsn_bw_clear_data_request_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_request_subtype_get(obj, &val32);

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_1_check(
    of_bsn_bw_clear_data_request_t *obj, int value)
{

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_1_populate(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_get_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_1_check(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_1_populate(
    of_bsn_bw_enable_get_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    of_bsn_bw_enable_get_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_request_subtype_get(obj, &val32);

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_1_check(
    of_bsn_bw_enable_get_request_t *obj, int value)
{

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_1_populate(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_set_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    of_bsn_bw_enable_set_reply_status_get(obj, &val32);

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_1_check(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_1_populate(
    of_bsn_bw_enable_set_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    of_bsn_bw_enable_set_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_request_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_request_enable_get(obj, &val32);

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_1_check(
    of_bsn_bw_enable_set_request_t *obj, int value)
{

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_1_populate(
    of_bsn_get_interfaces_reply_t *obj, int value)
{
    uint32_t val32;
    of_list_bsn_interface_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    of_bsn_get_interfaces_reply_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_interface_t sub_cls;

        /* Test bind */
        of_bsn_get_interfaces_reply_interfaces_bind(obj, &sub_cls);
    }

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_interface_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_interface_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_get_interfaces_reply_interfaces_set(
        obj, list);
    of_list_bsn_interface_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_1_check(
    of_bsn_get_interfaces_reply_t *obj, int value)
{

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_interface_t *interfaces_ptr;

        interfaces_ptr = of_bsn_get_interfaces_reply_interfaces_get(obj);
        TEST_ASSERT(interfaces_ptr != NULL);
        value = of_list_bsn_interface_OF_VERSION_1_1_check(
            interfaces_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_interface_delete(interfaces_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_1_populate(
    of_bsn_get_interfaces_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    of_bsn_get_interfaces_request_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_request_subtype_get(obj, &val32);

    value = of_bsn_get_interfaces_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_1_check(
    of_bsn_get_interfaces_request_t *obj, int value)
{

    value = of_bsn_get_interfaces_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_1_populate(
    of_bsn_get_mirroring_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    of_bsn_get_mirroring_reply_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_reply_subtype_get(obj, &val32);
    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_1_check(
    of_bsn_get_mirroring_reply_t *obj, int value)
{

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_1_populate(
    of_bsn_get_mirroring_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    of_bsn_get_mirroring_request_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_request_subtype_get(obj, &val32);
    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_1_check(
    of_bsn_get_mirroring_request_t *obj, int value)
{

    value = of_bsn_get_mirroring_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_1_populate(
    of_bsn_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_header_xid_get(obj, &val32);
    of_bsn_header_experimenter_get(obj, &val32);
    of_bsn_header_subtype_get(obj, &val32);

    value = of_bsn_header_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_1_check(
    of_bsn_header_t *obj, int value)
{

    value = of_bsn_header_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_header_t *dup;

        TEST_ASSERT((dup = of_bsn_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_1_populate(
    of_bsn_pdu_rx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    of_bsn_pdu_rx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_1_check(
    of_bsn_pdu_rx_reply_t *obj, int value)
{

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_1_populate(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    of_bsn_pdu_rx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_request_subtype_get(obj, &val32);
    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_rx_request_data_get(obj, &octets);

    value = of_bsn_pdu_rx_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_1_check(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_rx_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_timeout
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_1_populate(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    of_bsn_pdu_rx_timeout_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_timeout_subtype_get(obj, &val32);
    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_timeout
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_1_check(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_timeout_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_timeout_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_timeout_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_1_populate(
    of_bsn_pdu_tx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    of_bsn_pdu_tx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_1_check(
    of_bsn_pdu_tx_reply_t *obj, int value)
{

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_1_populate(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    of_bsn_pdu_tx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_request_subtype_get(obj, &val32);
    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_tx_request_data_get(obj, &octets);

    value = of_bsn_pdu_tx_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_1_check(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_tx_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_mirroring
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_1_populate(
    of_bsn_set_mirroring_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_mirroring_xid_get(obj, &val32);
    of_bsn_set_mirroring_experimenter_get(obj, &val32);
    of_bsn_set_mirroring_subtype_get(obj, &val32);
    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);

    value = of_bsn_set_mirroring_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_mirroring
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_1_check(
    of_bsn_set_mirroring_t *obj, int value)
{

    value = of_bsn_set_mirroring_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_mirroring_t *dup;

        TEST_ASSERT((dup = of_bsn_set_mirroring_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_mirroring_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_1_populate(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_1_check(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_1_populate(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_1_check(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_1_populate(
    of_bsn_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_reply_xid_get(obj, &val32);
    of_bsn_stats_reply_flags_get(obj, &val16);
    of_bsn_stats_reply_experimenter_get(obj, &val32);
    of_bsn_stats_reply_subtype_get(obj, &val32);

    value = of_bsn_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_1_check(
    of_bsn_stats_reply_t *obj, int value)
{

    value = of_bsn_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_1_populate(
    of_bsn_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_request_xid_get(obj, &val32);
    of_bsn_stats_request_flags_get(obj, &val16);
    of_bsn_stats_request_experimenter_get(obj, &val32);
    of_bsn_stats_request_subtype_get(obj, &val32);

    value = of_bsn_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_1_check(
    of_bsn_stats_request_t *obj, int value)
{

    value = of_bsn_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_1_populate(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_create_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_1_check(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_1_populate(
    of_bsn_virtual_port_create_request_t *obj, int value)
{
    uint32_t val32;
    of_bsn_vport_q_in_q_t *vport;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    of_bsn_virtual_port_create_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_request_subtype_get(obj, &val32);
    {
        of_bsn_vport_q_in_q_t sub_cls;

        /* Test bind */
        of_bsn_virtual_port_create_request_vport_bind(obj, &sub_cls);
    }

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    vport = of_bsn_vport_q_in_q_new(OF_VERSION_1_1);
    TEST_ASSERT(vport != NULL);
    value = of_bsn_vport_q_in_q_OF_VERSION_1_1_populate(
        vport, value);
    TEST_ASSERT(value != 0);
    of_bsn_virtual_port_create_request_vport_set(
        obj, vport);
    of_bsn_vport_q_in_q_delete(vport);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_1_check(
    of_bsn_virtual_port_create_request_t *obj, int value)
{

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_bsn_vport_q_in_q_t *vport_ptr;

        vport_ptr = of_bsn_virtual_port_create_request_vport_get(obj);
        TEST_ASSERT(vport_ptr != NULL);
        value = of_bsn_vport_q_in_q_OF_VERSION_1_1_check(
            vport_ptr, value);
        TEST_ASSERT(value != 0);
        of_bsn_vport_q_in_q_delete(vport_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_1_populate(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_1_check(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_1_populate(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_request_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_1_check(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_1_populate(
    of_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_reply_xid_get(obj, &val32);
    of_desc_stats_reply_flags_get(obj, &val16);
    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    of_desc_stats_reply_dp_desc_get(obj, &desc_str);

    value = of_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_1_check(
    of_desc_stats_reply_t *obj, int value)
{

    value = of_desc_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_1_populate(
    of_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_request_xid_get(obj, &val32);
    of_desc_stats_request_flags_get(obj, &val16);

    value = of_desc_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_1_check(
    of_desc_stats_request_t *obj, int value)
{

    value = of_desc_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_1_populate(
    of_echo_reply_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_reply_xid_get(obj, &val32);
    of_echo_reply_data_get(obj, &octets);

    value = of_echo_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_1_check(
    of_echo_reply_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_reply_t *dup;

        TEST_ASSERT((dup = of_echo_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_1_populate(
    of_echo_request_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_request_xid_get(obj, &val32);
    of_echo_request_data_get(obj, &octets);

    value = of_echo_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_1_check(
    of_echo_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_request_t *dup;

        TEST_ASSERT((dup = of_echo_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_1_populate(
    of_error_msg_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_error_msg_xid_get(obj, &val32);

    value = of_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_1_check(
    of_error_msg_t *obj, int value)
{

    value = of_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_error_msg_t *dup;

        TEST_ASSERT((dup = of_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_1_populate(
    of_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_xid_get(obj, &val32);
    of_experimenter_experimenter_get(obj, &val32);
    of_experimenter_subtype_get(obj, &val32);
    of_experimenter_data_get(obj, &octets);

    value = of_experimenter_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_1_check(
    of_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_t *dup;

        TEST_ASSERT((dup = of_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_1_populate(
    of_experimenter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_reply_xid_get(obj, &val32);
    of_experimenter_stats_reply_flags_get(obj, &val16);
    of_experimenter_stats_reply_experimenter_get(obj, &val32);
    of_experimenter_stats_reply_data_get(obj, &octets);

    value = of_experimenter_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_reply_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_1_check(
    of_experimenter_stats_reply_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_reply_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_1_populate(
    of_experimenter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_request_xid_get(obj, &val32);
    of_experimenter_stats_request_flags_get(obj, &val16);
    of_experimenter_stats_request_experimenter_get(obj, &val32);
    of_experimenter_stats_request_data_get(obj, &octets);

    value = of_experimenter_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_1_check(
    of_experimenter_stats_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_request_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_1_populate(
    of_features_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    of_list_port_desc_t *list;

    /* Run thru accessors after new to ensure okay */
    of_features_reply_xid_get(obj, &val32);
    of_features_reply_datapath_id_get(obj, &val64);
    of_features_reply_n_buffers_get(obj, &val32);
    of_features_reply_n_tables_get(obj, &val8);
    of_features_reply_capabilities_get(obj, &val32);
    of_features_reply_reserved_get(obj, &val32);
    {
        of_list_port_desc_t sub_cls;

        /* Test bind */
        of_features_reply_ports_bind(obj, &sub_cls);
    }

    value = of_features_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_desc_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_port_desc_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_features_reply_ports_set(
        obj, list);
    of_list_port_desc_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_features_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_1_check(
    of_features_reply_t *obj, int value)
{

    value = of_features_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_desc_t *ports_ptr;

        ports_ptr = of_features_reply_ports_get(obj);
        TEST_ASSERT(ports_ptr != NULL);
        value = of_list_port_desc_OF_VERSION_1_1_check(
            ports_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_desc_delete(ports_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_features_reply_t *dup;

        TEST_ASSERT((dup = of_features_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_1_populate(
    of_features_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_features_request_xid_get(obj, &val32);

    value = of_features_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_features_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_1_check(
    of_features_request_t *obj, int value)
{

    value = of_features_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_features_request_t *dup;

        TEST_ASSERT((dup = of_features_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_1_populate(
    of_flow_add_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_add_xid_get(obj, &val32);
    of_flow_add_cookie_get(obj, &val64);
    of_flow_add_cookie_mask_get(obj, &val64);
    of_flow_add_table_id_get(obj, &val8);
    of_flow_add_idle_timeout_get(obj, &val16);
    of_flow_add_hard_timeout_get(obj, &val16);
    of_flow_add_priority_get(obj, &val16);
    of_flow_add_buffer_id_get(obj, &val32);
    of_flow_add_out_port_get(obj, &port_no);
    of_flow_add_out_group_get(obj, &val32);
    of_flow_add_flags_get(obj, &val16);
    of_flow_add_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_add_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_add_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_add_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_1_check(
    of_flow_add_t *obj, int value)
{
    of_match_t match;

    value = of_flow_add_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_add_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_1_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_add_t *dup;

        TEST_ASSERT((dup = of_flow_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_1_populate(
    of_flow_delete_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_xid_get(obj, &val32);
    of_flow_delete_cookie_get(obj, &val64);
    of_flow_delete_cookie_mask_get(obj, &val64);
    of_flow_delete_table_id_get(obj, &val8);
    of_flow_delete_idle_timeout_get(obj, &val16);
    of_flow_delete_hard_timeout_get(obj, &val16);
    of_flow_delete_priority_get(obj, &val16);
    of_flow_delete_buffer_id_get(obj, &val32);
    of_flow_delete_out_port_get(obj, &port_no);
    of_flow_delete_out_group_get(obj, &val32);
    of_flow_delete_flags_get(obj, &val16);
    of_flow_delete_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_delete_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_1_check(
    of_flow_delete_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_delete_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_1_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_t *dup;

        TEST_ASSERT((dup = of_flow_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_1_populate(
    of_flow_delete_strict_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_strict_xid_get(obj, &val32);
    of_flow_delete_strict_cookie_get(obj, &val64);
    of_flow_delete_strict_cookie_mask_get(obj, &val64);
    of_flow_delete_strict_table_id_get(obj, &val8);
    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    of_flow_delete_strict_priority_get(obj, &val16);
    of_flow_delete_strict_buffer_id_get(obj, &val32);
    of_flow_delete_strict_out_port_get(obj, &port_no);
    of_flow_delete_strict_out_group_get(obj, &val32);
    of_flow_delete_strict_flags_get(obj, &val16);
    of_flow_delete_strict_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_delete_strict_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_strict_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_1_check(
    of_flow_delete_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_strict_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_delete_strict_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_1_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_strict_t *dup;

        TEST_ASSERT((dup = of_flow_delete_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_1_populate(
    of_flow_mod_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_xid_get(obj, &val32);
    of_flow_mod_cookie_get(obj, &val64);
    of_flow_mod_cookie_mask_get(obj, &val64);
    of_flow_mod_table_id_get(obj, &val8);
    of_flow_mod_idle_timeout_get(obj, &val16);
    of_flow_mod_hard_timeout_get(obj, &val16);
    of_flow_mod_priority_get(obj, &val16);
    of_flow_mod_buffer_id_get(obj, &val32);
    of_flow_mod_out_port_get(obj, &port_no);
    of_flow_mod_out_group_get(obj, &val32);
    of_flow_mod_flags_get(obj, &val16);
    of_flow_mod_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_mod_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_mod_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_1_check(
    of_flow_mod_t *obj, int value)
{
    of_match_t match;

    value = of_flow_mod_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_mod_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_1_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_t *dup;

        TEST_ASSERT((dup = of_flow_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_1_populate(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_flow_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_1_check(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_flow_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_1_populate(
    of_flow_modify_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_xid_get(obj, &val32);
    of_flow_modify_cookie_get(obj, &val64);
    of_flow_modify_cookie_mask_get(obj, &val64);
    of_flow_modify_table_id_get(obj, &val8);
    of_flow_modify_idle_timeout_get(obj, &val16);
    of_flow_modify_hard_timeout_get(obj, &val16);
    of_flow_modify_priority_get(obj, &val16);
    of_flow_modify_buffer_id_get(obj, &val32);
    of_flow_modify_out_port_get(obj, &port_no);
    of_flow_modify_out_group_get(obj, &val32);
    of_flow_modify_flags_get(obj, &val16);
    of_flow_modify_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_modify_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_1_check(
    of_flow_modify_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_modify_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_1_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_t *dup;

        TEST_ASSERT((dup = of_flow_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_1_populate(
    of_flow_modify_strict_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_strict_xid_get(obj, &val32);
    of_flow_modify_strict_cookie_get(obj, &val64);
    of_flow_modify_strict_cookie_mask_get(obj, &val64);
    of_flow_modify_strict_table_id_get(obj, &val8);
    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    of_flow_modify_strict_priority_get(obj, &val16);
    of_flow_modify_strict_buffer_id_get(obj, &val32);
    of_flow_modify_strict_out_port_get(obj, &port_no);
    of_flow_modify_strict_out_group_get(obj, &val32);
    of_flow_modify_strict_flags_get(obj, &val16);
    of_flow_modify_strict_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_modify_strict_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_strict_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_1_check(
    of_flow_modify_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_strict_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_modify_strict_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_1_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_strict_t *dup;

        TEST_ASSERT((dup = of_flow_modify_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_removed
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_1_populate(
    of_flow_removed_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_flow_removed_xid_get(obj, &val32);
    of_flow_removed_cookie_get(obj, &val64);
    of_flow_removed_priority_get(obj, &val16);
    of_flow_removed_reason_get(obj, &val8);
    of_flow_removed_table_id_get(obj, &val8);
    of_flow_removed_duration_sec_get(obj, &val32);
    of_flow_removed_duration_nsec_get(obj, &val32);
    of_flow_removed_idle_timeout_get(obj, &val16);
    of_flow_removed_packet_count_get(obj, &val64);
    of_flow_removed_byte_count_get(obj, &val64);
    of_flow_removed_match_get(obj, &match);

    value = of_flow_removed_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_removed
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_1_check(
    of_flow_removed_t *obj, int value)
{
    of_match_t match;

    value = of_flow_removed_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_removed_t *dup;

        TEST_ASSERT((dup = of_flow_removed_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_removed_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_1_populate(
    of_flow_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_flow_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_reply_xid_get(obj, &val32);
    of_flow_stats_reply_flags_get(obj, &val16);
    {
        of_list_flow_stats_entry_t sub_cls;

        /* Test bind */
        of_flow_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_flow_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_flow_stats_entry_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_flow_stats_entry_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_reply_entries_set(
        obj, list);
    of_list_flow_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_1_check(
    of_flow_stats_reply_t *obj, int value)
{

    value = of_flow_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_flow_stats_entry_t *entries_ptr;

        entries_ptr = of_flow_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_flow_stats_entry_OF_VERSION_1_1_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_flow_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_reply_t *dup;

        TEST_ASSERT((dup = of_flow_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_1_populate(
    of_flow_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_request_xid_get(obj, &val32);
    of_flow_stats_request_flags_get(obj, &val16);
    of_flow_stats_request_table_id_get(obj, &val8);
    of_flow_stats_request_out_port_get(obj, &port_no);
    of_flow_stats_request_out_group_get(obj, &val32);
    of_flow_stats_request_cookie_get(obj, &val64);
    of_flow_stats_request_cookie_mask_get(obj, &val64);
    of_flow_stats_request_match_get(obj, &match);

    value = of_flow_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_1_check(
    of_flow_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_request_t *dup;

        TEST_ASSERT((dup = of_flow_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_1_populate(
    of_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_get_config_reply_xid_get(obj, &val32);
    of_get_config_reply_flags_get(obj, &val16);
    of_get_config_reply_miss_send_len_get(obj, &val16);

    value = of_get_config_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_1_check(
    of_get_config_reply_t *obj, int value)
{

    value = of_get_config_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_1_populate(
    of_get_config_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_get_config_request_xid_get(obj, &val32);

    value = of_get_config_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_1_check(
    of_get_config_request_t *obj, int value)
{

    value = of_get_config_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_request_t *dup;

        TEST_ASSERT((dup = of_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_add_OF_VERSION_1_1_populate(
    of_group_add_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_add_xid_get(obj, &val32);
    of_group_add_group_type_get(obj, &val8);
    of_group_add_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_add_buckets_bind(obj, &sub_cls);
    }

    value = of_group_add_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_add_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_add_OF_VERSION_1_1_check(
    of_group_add_t *obj, int value)
{

    value = of_group_add_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_add_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_1_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_add_t *dup;

        TEST_ASSERT((dup = of_group_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_delete_OF_VERSION_1_1_populate(
    of_group_delete_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_delete_xid_get(obj, &val32);
    of_group_delete_group_type_get(obj, &val8);
    of_group_delete_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_delete_buckets_bind(obj, &sub_cls);
    }

    value = of_group_delete_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_delete_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_delete_OF_VERSION_1_1_check(
    of_group_delete_t *obj, int value)
{

    value = of_group_delete_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_delete_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_1_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_delete_t *dup;

        TEST_ASSERT((dup = of_group_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_reply_OF_VERSION_1_1_populate(
    of_group_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_group_desc_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_reply_xid_get(obj, &val32);
    of_group_desc_stats_reply_flags_get(obj, &val16);
    {
        of_list_group_desc_stats_entry_t sub_cls;

        /* Test bind */
        of_group_desc_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_group_desc_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_group_desc_stats_entry_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_group_desc_stats_entry_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_desc_stats_reply_entries_set(
        obj, list);
    of_list_group_desc_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_reply_OF_VERSION_1_1_check(
    of_group_desc_stats_reply_t *obj, int value)
{

    value = of_group_desc_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_group_desc_stats_entry_t *entries_ptr;

        entries_ptr = of_group_desc_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_group_desc_stats_entry_OF_VERSION_1_1_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_group_desc_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_request_OF_VERSION_1_1_populate(
    of_group_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_request_xid_get(obj, &val32);
    of_group_desc_stats_request_flags_get(obj, &val16);

    value = of_group_desc_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_request_OF_VERSION_1_1_check(
    of_group_desc_stats_request_t *obj, int value)
{

    value = of_group_desc_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_mod_OF_VERSION_1_1_populate(
    of_group_mod_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_mod_xid_get(obj, &val32);
    of_group_mod_group_type_get(obj, &val8);
    of_group_mod_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_mod_buckets_bind(obj, &sub_cls);
    }

    value = of_group_mod_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_mod_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_mod_OF_VERSION_1_1_check(
    of_group_mod_t *obj, int value)
{

    value = of_group_mod_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_mod_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_1_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_mod_t *dup;

        TEST_ASSERT((dup = of_group_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_mod_failed_error_msg_OF_VERSION_1_1_populate(
    of_group_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_group_mod_failed_error_msg_xid_get(obj, &val32);
    of_group_mod_failed_error_msg_code_get(obj, &val16);
    of_group_mod_failed_error_msg_data_get(obj, &octets);

    value = of_group_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_group_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_group_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_mod_failed_error_msg_OF_VERSION_1_1_check(
    of_group_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_group_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_group_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_group_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_modify_OF_VERSION_1_1_populate(
    of_group_modify_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_modify_xid_get(obj, &val32);
    of_group_modify_group_type_get(obj, &val8);
    of_group_modify_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_modify_buckets_bind(obj, &sub_cls);
    }

    value = of_group_modify_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_modify_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_modify_OF_VERSION_1_1_check(
    of_group_modify_t *obj, int value)
{

    value = of_group_modify_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_modify_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_1_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_modify_t *dup;

        TEST_ASSERT((dup = of_group_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_reply_OF_VERSION_1_1_populate(
    of_group_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_group_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_reply_xid_get(obj, &val32);
    of_group_stats_reply_flags_get(obj, &val16);
    {
        of_list_group_stats_entry_t sub_cls;

        /* Test bind */
        of_group_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_group_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_group_stats_entry_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_group_stats_entry_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_stats_reply_entries_set(
        obj, list);
    of_list_group_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_reply_OF_VERSION_1_1_check(
    of_group_stats_reply_t *obj, int value)
{

    value = of_group_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_group_stats_entry_t *entries_ptr;

        entries_ptr = of_group_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_group_stats_entry_OF_VERSION_1_1_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_group_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_request_OF_VERSION_1_1_populate(
    of_group_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_request_xid_get(obj, &val32);
    of_group_stats_request_flags_get(obj, &val16);
    of_group_stats_request_group_id_get(obj, &val32);

    value = of_group_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_request_OF_VERSION_1_1_check(
    of_group_stats_request_t *obj, int value)
{

    value = of_group_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_1_populate(
    of_hello_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_hello_xid_get(obj, &val32);

    value = of_hello_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_hello
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_1_check(
    of_hello_t *obj, int value)
{

    value = of_hello_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_t *dup;

        TEST_ASSERT((dup = of_hello_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_1_populate(
    of_hello_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_hello_failed_error_msg_xid_get(obj, &val32);
    of_hello_failed_error_msg_code_get(obj, &val16);
    of_hello_failed_error_msg_data_get(obj, &octets);

    value = of_hello_failed_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_hello_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_1_check(
    of_hello_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_hello_failed_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_hello_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_nicira_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_1_populate(
    of_nicira_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_nicira_header_xid_get(obj, &val32);
    of_nicira_header_experimenter_get(obj, &val32);
    of_nicira_header_subtype_get(obj, &val32);

    value = of_nicira_header_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_nicira_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_1_check(
    of_nicira_header_t *obj, int value)
{

    value = of_nicira_header_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_nicira_header_t *dup;

        TEST_ASSERT((dup = of_nicira_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_nicira_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_1_populate(
    of_packet_in_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint16_t val16;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_in_xid_get(obj, &val32);
    of_packet_in_buffer_id_get(obj, &val32);
    of_packet_in_in_port_get(obj, &port_no);
    of_packet_in_in_phy_port_get(obj, &port_no);
    of_packet_in_total_len_get(obj, &val16);
    of_packet_in_reason_get(obj, &val8);
    of_packet_in_table_id_get(obj, &val8);
    of_packet_in_data_get(obj, &octets);

    value = of_packet_in_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_in_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_1_check(
    of_packet_in_t *obj, int value)
{
    of_octets_t octets;

    value = of_packet_in_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_packet_in_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_in_t *dup;

        TEST_ASSERT((dup = of_packet_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_1_populate(
    of_packet_out_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_action_t *list;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_out_xid_get(obj, &val32);
    of_packet_out_buffer_id_get(obj, &val32);
    of_packet_out_in_port_get(obj, &port_no);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_packet_out_actions_bind(obj, &sub_cls);
    }
    of_packet_out_data_get(obj, &octets);

    value = of_packet_out_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_out_actions_set(
        obj, list);
    of_list_action_delete(list);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_out_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_1_check(
    of_packet_out_t *obj, int value)
{
    of_octets_t octets;

    value = of_packet_out_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_packet_out_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_1_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }
    of_packet_out_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_out_t *dup;

        TEST_ASSERT((dup = of_packet_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_1_populate(
    of_port_mod_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_xid_get(obj, &val32);
    of_port_mod_port_no_get(obj, &port_no);
    of_port_mod_hw_addr_get(obj, &mac_addr);
    of_port_mod_config_get(obj, &val32);
    of_port_mod_mask_get(obj, &val32);
    of_port_mod_advertise_get(obj, &val32);

    value = of_port_mod_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_1_check(
    of_port_mod_t *obj, int value)
{

    value = of_port_mod_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_t *dup;

        TEST_ASSERT((dup = of_port_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_1_populate(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    of_port_mod_failed_error_msg_code_get(obj, &val16);
    of_port_mod_failed_error_msg_data_get(obj, &octets);

    value = of_port_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_port_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_1_check(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_port_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_port_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_1_populate(
    of_port_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_port_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_reply_xid_get(obj, &val32);
    of_port_stats_reply_flags_get(obj, &val16);
    {
        of_list_port_stats_entry_t sub_cls;

        /* Test bind */
        of_port_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_port_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_stats_entry_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_port_stats_entry_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_port_stats_reply_entries_set(
        obj, list);
    of_list_port_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_1_check(
    of_port_stats_reply_t *obj, int value)
{

    value = of_port_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_stats_entry_t *entries_ptr;

        entries_ptr = of_port_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_port_stats_entry_OF_VERSION_1_1_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_reply_t *dup;

        TEST_ASSERT((dup = of_port_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_1_populate(
    of_port_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_request_xid_get(obj, &val32);
    of_port_stats_request_flags_get(obj, &val16);
    of_port_stats_request_port_no_get(obj, &port_no);

    value = of_port_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_1_check(
    of_port_stats_request_t *obj, int value)
{

    value = of_port_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_request_t *dup;

        TEST_ASSERT((dup = of_port_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_1_populate(
    of_port_status_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_port_desc_t *port_desc;

    /* Run thru accessors after new to ensure okay */
    of_port_status_xid_get(obj, &val32);
    of_port_status_reason_get(obj, &val8);
    {
        of_port_desc_t sub_cls;

        /* Test bind */
        of_port_status_desc_bind(obj, &sub_cls);
    }

    value = of_port_status_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    port_desc = of_port_desc_new(OF_VERSION_1_1);
    TEST_ASSERT(port_desc != NULL);
    value = of_port_desc_OF_VERSION_1_1_populate(
        port_desc, value);
    TEST_ASSERT(value != 0);
    of_port_status_desc_set(
        obj, port_desc);
    of_port_desc_delete(port_desc);

    return value;
}

/**
 * Check all members of an object of type of_port_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_1_check(
    of_port_status_t *obj, int value)
{

    value = of_port_status_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_port_desc_t *desc_ptr;

        desc_ptr = of_port_status_desc_get(obj);
        TEST_ASSERT(desc_ptr != NULL);
        value = of_port_desc_OF_VERSION_1_1_check(
            desc_ptr, value);
        TEST_ASSERT(value != 0);
        of_port_desc_delete(desc_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_status_t *dup;

        TEST_ASSERT((dup = of_port_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_1_populate(
    of_queue_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_packet_queue_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_reply_xid_get(obj, &val32);
    of_queue_get_config_reply_port_get(obj, &port_no);
    {
        of_list_packet_queue_t sub_cls;

        /* Test bind */
        of_queue_get_config_reply_queues_bind(obj, &sub_cls);
    }

    value = of_queue_get_config_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_packet_queue_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_packet_queue_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_get_config_reply_queues_set(
        obj, list);
    of_list_packet_queue_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_1_check(
    of_queue_get_config_reply_t *obj, int value)
{

    value = of_queue_get_config_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_packet_queue_t *queues_ptr;

        queues_ptr = of_queue_get_config_reply_queues_get(obj);
        TEST_ASSERT(queues_ptr != NULL);
        value = of_list_packet_queue_OF_VERSION_1_1_check(
            queues_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_packet_queue_delete(queues_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_1_populate(
    of_queue_get_config_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_request_xid_get(obj, &val32);
    of_queue_get_config_request_port_get(obj, &port_no);

    value = of_queue_get_config_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_1_check(
    of_queue_get_config_request_t *obj, int value)
{

    value = of_queue_get_config_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_request_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_op_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_1_populate(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    of_queue_op_failed_error_msg_code_get(obj, &val16);
    of_queue_op_failed_error_msg_data_get(obj, &octets);

    value = of_queue_op_failed_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_queue_op_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_1_check(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_queue_op_failed_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_op_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_queue_op_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_op_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_1_populate(
    of_queue_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_queue_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_reply_xid_get(obj, &val32);
    of_queue_stats_reply_flags_get(obj, &val16);
    {
        of_list_queue_stats_entry_t sub_cls;

        /* Test bind */
        of_queue_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_queue_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_stats_entry_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_stats_entry_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_stats_reply_entries_set(
        obj, list);
    of_list_queue_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_1_check(
    of_queue_stats_reply_t *obj, int value)
{

    value = of_queue_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_stats_entry_t *entries_ptr;

        entries_ptr = of_queue_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_queue_stats_entry_OF_VERSION_1_1_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_reply_t *dup;

        TEST_ASSERT((dup = of_queue_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_1_populate(
    of_queue_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_request_xid_get(obj, &val32);
    of_queue_stats_request_flags_get(obj, &val16);
    of_queue_stats_request_port_no_get(obj, &port_no);
    of_queue_stats_request_queue_id_get(obj, &val32);

    value = of_queue_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_1_check(
    of_queue_stats_request_t *obj, int value)
{

    value = of_queue_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_request_t *dup;

        TEST_ASSERT((dup = of_queue_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_set_config
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_1_populate(
    of_set_config_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_set_config_xid_get(obj, &val32);
    of_set_config_flags_get(obj, &val16);
    of_set_config_miss_send_len_get(obj, &val16);

    value = of_set_config_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_set_config
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_1_check(
    of_set_config_t *obj, int value)
{

    value = of_set_config_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_set_config_t *dup;

        TEST_ASSERT((dup = of_set_config_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_set_config_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_1_populate(
    of_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_reply_xid_get(obj, &val32);
    of_stats_reply_flags_get(obj, &val16);

    value = of_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_1_check(
    of_stats_reply_t *obj, int value)
{

    value = of_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_reply_t *dup;

        TEST_ASSERT((dup = of_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_1_populate(
    of_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_request_xid_get(obj, &val32);
    of_stats_request_flags_get(obj, &val16);

    value = of_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_1_check(
    of_stats_request_t *obj, int value)
{

    value = of_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_request_t *dup;

        TEST_ASSERT((dup = of_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_switch_config_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_switch_config_failed_error_msg_OF_VERSION_1_1_populate(
    of_switch_config_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_switch_config_failed_error_msg_xid_get(obj, &val32);
    of_switch_config_failed_error_msg_code_get(obj, &val16);
    of_switch_config_failed_error_msg_data_get(obj, &octets);

    value = of_switch_config_failed_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_switch_config_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_switch_config_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_switch_config_failed_error_msg_OF_VERSION_1_1_check(
    of_switch_config_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_switch_config_failed_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_switch_config_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_switch_config_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_switch_config_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_switch_config_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_1_populate(
    of_table_mod_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_table_mod_xid_get(obj, &val32);
    of_table_mod_table_id_get(obj, &val8);
    of_table_mod_config_get(obj, &val32);

    value = of_table_mod_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_1_check(
    of_table_mod_t *obj, int value)
{

    value = of_table_mod_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_mod_t *dup;

        TEST_ASSERT((dup = of_table_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_mod_failed_error_msg_OF_VERSION_1_1_populate(
    of_table_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_table_mod_failed_error_msg_xid_get(obj, &val32);
    of_table_mod_failed_error_msg_code_get(obj, &val16);
    of_table_mod_failed_error_msg_data_get(obj, &octets);

    value = of_table_mod_failed_error_msg_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_table_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_table_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_mod_failed_error_msg_OF_VERSION_1_1_check(
    of_table_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_table_mod_failed_error_msg_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_table_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_table_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_1_populate(
    of_table_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_table_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_reply_xid_get(obj, &val32);
    of_table_stats_reply_flags_get(obj, &val16);
    {
        of_list_table_stats_entry_t sub_cls;

        /* Test bind */
        of_table_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_table_stats_reply_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_table_stats_entry_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_table_stats_entry_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_stats_reply_entries_set(
        obj, list);
    of_list_table_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_1_check(
    of_table_stats_reply_t *obj, int value)
{

    value = of_table_stats_reply_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_table_stats_entry_t *entries_ptr;

        entries_ptr = of_table_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_table_stats_entry_OF_VERSION_1_1_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_table_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_reply_t *dup;

        TEST_ASSERT((dup = of_table_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_1_populate(
    of_table_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_request_xid_get(obj, &val32);
    of_table_stats_request_flags_get(obj, &val16);

    value = of_table_stats_request_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_1_check(
    of_table_stats_request_t *obj, int value)
{

    value = of_table_stats_request_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_request_t *dup;

        TEST_ASSERT((dup = of_table_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_1_populate(
    of_action_bsn_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_experimenter_get(obj, &val32);
    of_action_bsn_subtype_get(obj, &val32);

    value = of_action_bsn_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_1_check(
    of_action_bsn_t *obj, int value)
{

    value = of_action_bsn_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_t *dup;

        TEST_ASSERT((dup = of_action_bsn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_mirror
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_1_populate(
    of_action_bsn_mirror_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_mirror_experimenter_get(obj, &val32);
    of_action_bsn_mirror_subtype_get(obj, &val32);
    of_action_bsn_mirror_dest_port_get(obj, &val32);
    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    of_action_bsn_mirror_copy_stage_get(obj, &val8);

    value = of_action_bsn_mirror_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_mirror
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_1_check(
    of_action_bsn_mirror_t *obj, int value)
{

    value = of_action_bsn_mirror_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_mirror_t *dup;

        TEST_ASSERT((dup = of_action_bsn_mirror_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_mirror_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_set_tunnel_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_populate(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_set_tunnel_dst_experimenter_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_subtype_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_set_tunnel_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_check(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_set_tunnel_dst_t *dup;

        TEST_ASSERT((dup = of_action_bsn_set_tunnel_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_set_tunnel_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_copy_ttl_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_in_OF_VERSION_1_1_populate(
    of_action_copy_ttl_in_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_copy_ttl_in_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_copy_ttl_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_in_OF_VERSION_1_1_check(
    of_action_copy_ttl_in_t *obj, int value)
{

    value = of_action_copy_ttl_in_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_copy_ttl_in_t *dup;

        TEST_ASSERT((dup = of_action_copy_ttl_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_copy_ttl_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_copy_ttl_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_out_OF_VERSION_1_1_populate(
    of_action_copy_ttl_out_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_copy_ttl_out_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_copy_ttl_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_out_OF_VERSION_1_1_check(
    of_action_copy_ttl_out_t *obj, int value)
{

    value = of_action_copy_ttl_out_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_copy_ttl_out_t *dup;

        TEST_ASSERT((dup = of_action_copy_ttl_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_copy_ttl_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_dec_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_dec_mpls_ttl_OF_VERSION_1_1_populate(
    of_action_dec_mpls_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_dec_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_dec_mpls_ttl_OF_VERSION_1_1_check(
    of_action_dec_mpls_ttl_t *obj, int value)
{

    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_dec_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_dec_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_dec_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_dec_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_dec_nw_ttl_OF_VERSION_1_1_populate(
    of_action_dec_nw_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_dec_nw_ttl_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_dec_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_dec_nw_ttl_OF_VERSION_1_1_check(
    of_action_dec_nw_ttl_t *obj, int value)
{

    value = of_action_dec_nw_ttl_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_dec_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_dec_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_dec_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_1_populate(
    of_action_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_action_experimenter_experimenter_get(obj, &val32);
    of_action_experimenter_data_get(obj, &octets);

    value = of_action_experimenter_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_action_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_1_check(
    of_action_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_action_experimenter_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_experimenter_t *dup;

        TEST_ASSERT((dup = of_action_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_group
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_group_OF_VERSION_1_1_populate(
    of_action_group_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_group_group_id_get(obj, &val32);

    value = of_action_group_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_group
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_group_OF_VERSION_1_1_check(
    of_action_group_t *obj, int value)
{

    value = of_action_group_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_group_t *dup;

        TEST_ASSERT((dup = of_action_group_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_group_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_1_populate(
    of_action_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_header_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_1_check(
    of_action_header_t *obj, int value)
{

    value = of_action_header_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_header_t *dup;

        TEST_ASSERT((dup = of_action_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_1_populate(
    of_action_nicira_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_experimenter_get(obj, &val32);
    of_action_nicira_subtype_get(obj, &val16);

    value = of_action_nicira_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_1_check(
    of_action_nicira_t *obj, int value)
{

    value = of_action_nicira_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira_dec_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_1_populate(
    of_action_nicira_dec_ttl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_dec_ttl_experimenter_get(obj, &val32);
    of_action_nicira_dec_ttl_subtype_get(obj, &val16);

    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira_dec_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_1_check(
    of_action_nicira_dec_ttl_t *obj, int value)
{

    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_dec_ttl_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dec_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_dec_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_output
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_1_populate(
    of_action_output_t *obj, int value)
{
    of_port_no_t port_no;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_output_port_get(obj, &port_no);
    of_action_output_max_len_get(obj, &val16);

    value = of_action_output_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_output
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_1_check(
    of_action_output_t *obj, int value)
{

    value = of_action_output_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_output_t *dup;

        TEST_ASSERT((dup = of_action_output_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_output_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_pop_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_pop_mpls_OF_VERSION_1_1_populate(
    of_action_pop_mpls_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_pop_mpls_ethertype_get(obj, &val16);

    value = of_action_pop_mpls_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_pop_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_pop_mpls_OF_VERSION_1_1_check(
    of_action_pop_mpls_t *obj, int value)
{

    value = of_action_pop_mpls_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_pop_mpls_t *dup;

        TEST_ASSERT((dup = of_action_pop_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_pop_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_pop_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_pop_vlan_OF_VERSION_1_1_populate(
    of_action_pop_vlan_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_pop_vlan_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_pop_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_pop_vlan_OF_VERSION_1_1_check(
    of_action_pop_vlan_t *obj, int value)
{

    value = of_action_pop_vlan_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_pop_vlan_t *dup;

        TEST_ASSERT((dup = of_action_pop_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_pop_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_push_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_push_mpls_OF_VERSION_1_1_populate(
    of_action_push_mpls_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_push_mpls_ethertype_get(obj, &val16);

    value = of_action_push_mpls_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_push_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_push_mpls_OF_VERSION_1_1_check(
    of_action_push_mpls_t *obj, int value)
{

    value = of_action_push_mpls_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_push_mpls_t *dup;

        TEST_ASSERT((dup = of_action_push_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_push_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_push_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_push_vlan_OF_VERSION_1_1_populate(
    of_action_push_vlan_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_push_vlan_ethertype_get(obj, &val16);

    value = of_action_push_vlan_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_push_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_push_vlan_OF_VERSION_1_1_check(
    of_action_push_vlan_t *obj, int value)
{

    value = of_action_push_vlan_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_push_vlan_t *dup;

        TEST_ASSERT((dup = of_action_push_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_push_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_dl_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_dst_OF_VERSION_1_1_populate(
    of_action_set_dl_dst_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_action_set_dl_dst_dl_addr_get(obj, &mac_addr);

    value = of_action_set_dl_dst_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_dl_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_dst_OF_VERSION_1_1_check(
    of_action_set_dl_dst_t *obj, int value)
{

    value = of_action_set_dl_dst_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_dl_dst_t *dup;

        TEST_ASSERT((dup = of_action_set_dl_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_dl_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_dl_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_src_OF_VERSION_1_1_populate(
    of_action_set_dl_src_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_action_set_dl_src_dl_addr_get(obj, &mac_addr);

    value = of_action_set_dl_src_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_dl_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_dl_src_OF_VERSION_1_1_check(
    of_action_set_dl_src_t *obj, int value)
{

    value = of_action_set_dl_src_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_dl_src_t *dup;

        TEST_ASSERT((dup = of_action_set_dl_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_dl_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_mpls_label
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_label_OF_VERSION_1_1_populate(
    of_action_set_mpls_label_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_mpls_label_mpls_label_get(obj, &val32);

    value = of_action_set_mpls_label_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_mpls_label
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_label_OF_VERSION_1_1_check(
    of_action_set_mpls_label_t *obj, int value)
{

    value = of_action_set_mpls_label_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_mpls_label_t *dup;

        TEST_ASSERT((dup = of_action_set_mpls_label_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_mpls_label_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_mpls_tc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_tc_OF_VERSION_1_1_populate(
    of_action_set_mpls_tc_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_mpls_tc_mpls_tc_get(obj, &val8);

    value = of_action_set_mpls_tc_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_mpls_tc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_tc_OF_VERSION_1_1_check(
    of_action_set_mpls_tc_t *obj, int value)
{

    value = of_action_set_mpls_tc_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_mpls_tc_t *dup;

        TEST_ASSERT((dup = of_action_set_mpls_tc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_mpls_tc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_ttl_OF_VERSION_1_1_populate(
    of_action_set_mpls_ttl_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_mpls_ttl_mpls_ttl_get(obj, &val8);

    value = of_action_set_mpls_ttl_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_ttl_OF_VERSION_1_1_check(
    of_action_set_mpls_ttl_t *obj, int value)
{

    value = of_action_set_mpls_ttl_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_set_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_dst_OF_VERSION_1_1_populate(
    of_action_set_nw_dst_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_dst_nw_addr_get(obj, &val32);

    value = of_action_set_nw_dst_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_dst_OF_VERSION_1_1_check(
    of_action_set_nw_dst_t *obj, int value)
{

    value = of_action_set_nw_dst_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_dst_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_ecn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ecn_OF_VERSION_1_1_populate(
    of_action_set_nw_ecn_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_ecn_nw_ecn_get(obj, &val8);

    value = of_action_set_nw_ecn_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_ecn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ecn_OF_VERSION_1_1_check(
    of_action_set_nw_ecn_t *obj, int value)
{

    value = of_action_set_nw_ecn_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_ecn_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_ecn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_ecn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_src_OF_VERSION_1_1_populate(
    of_action_set_nw_src_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_src_nw_addr_get(obj, &val32);

    value = of_action_set_nw_src_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_src_OF_VERSION_1_1_check(
    of_action_set_nw_src_t *obj, int value)
{

    value = of_action_set_nw_src_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_src_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_tos
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_tos_OF_VERSION_1_1_populate(
    of_action_set_nw_tos_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_tos_nw_tos_get(obj, &val8);

    value = of_action_set_nw_tos_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_tos
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_tos_OF_VERSION_1_1_check(
    of_action_set_nw_tos_t *obj, int value)
{

    value = of_action_set_nw_tos_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_tos_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_tos_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_tos_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ttl_OF_VERSION_1_1_populate(
    of_action_set_nw_ttl_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_ttl_nw_ttl_get(obj, &val8);

    value = of_action_set_nw_ttl_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ttl_OF_VERSION_1_1_check(
    of_action_set_nw_ttl_t *obj, int value)
{

    value = of_action_set_nw_ttl_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_queue_OF_VERSION_1_1_populate(
    of_action_set_queue_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_queue_queue_id_get(obj, &val32);

    value = of_action_set_queue_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_queue_OF_VERSION_1_1_check(
    of_action_set_queue_t *obj, int value)
{

    value = of_action_set_queue_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_queue_t *dup;

        TEST_ASSERT((dup = of_action_set_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_tp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_dst_OF_VERSION_1_1_populate(
    of_action_set_tp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_set_tp_dst_tp_port_get(obj, &val16);

    value = of_action_set_tp_dst_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_tp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_dst_OF_VERSION_1_1_check(
    of_action_set_tp_dst_t *obj, int value)
{

    value = of_action_set_tp_dst_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_tp_dst_t *dup;

        TEST_ASSERT((dup = of_action_set_tp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_tp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_tp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_src_OF_VERSION_1_1_populate(
    of_action_set_tp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_set_tp_src_tp_port_get(obj, &val16);

    value = of_action_set_tp_src_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_tp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_tp_src_OF_VERSION_1_1_check(
    of_action_set_tp_src_t *obj, int value)
{

    value = of_action_set_tp_src_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_tp_src_t *dup;

        TEST_ASSERT((dup = of_action_set_tp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_tp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_vlan_pcp
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_pcp_OF_VERSION_1_1_populate(
    of_action_set_vlan_pcp_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_vlan_pcp_vlan_pcp_get(obj, &val8);

    value = of_action_set_vlan_pcp_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_vlan_pcp
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_pcp_OF_VERSION_1_1_check(
    of_action_set_vlan_pcp_t *obj, int value)
{

    value = of_action_set_vlan_pcp_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_vlan_pcp_t *dup;

        TEST_ASSERT((dup = of_action_set_vlan_pcp_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_vlan_pcp_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_vlan_vid
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_vid_OF_VERSION_1_1_populate(
    of_action_set_vlan_vid_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_set_vlan_vid_vlan_vid_get(obj, &val16);

    value = of_action_set_vlan_vid_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_vlan_vid
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_vlan_vid_OF_VERSION_1_1_check(
    of_action_set_vlan_vid_t *obj, int value)
{

    value = of_action_set_vlan_vid_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_vlan_vid_t *dup;

        TEST_ASSERT((dup = of_action_set_vlan_vid_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_vlan_vid_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_interface
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_1_populate(
    of_bsn_interface_t *obj, int value)
{
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    of_bsn_interface_name_get(obj, &port_name);
    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);

    value = of_bsn_interface_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_interface
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_1_check(
    of_bsn_interface_t *obj, int value)
{

    value = of_bsn_interface_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_bsn_interface_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_interface_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_1_populate(
    of_bsn_vport_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_bsn_vport_header_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_1_check(
    of_bsn_vport_header_t *obj, int value)
{

    value = of_bsn_vport_header_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_header_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_q_in_q
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_1_populate(
    of_bsn_vport_q_in_q_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    /* Run thru accessors after new to ensure okay */
    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);

    value = of_bsn_vport_q_in_q_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_q_in_q
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_1_check(
    of_bsn_vport_q_in_q_t *obj, int value)
{

    value = of_bsn_vport_q_in_q_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_q_in_q_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_q_in_q_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_q_in_q_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bucket
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bucket_OF_VERSION_1_1_populate(
    of_bucket_t *obj, int value)
{
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bucket_weight_get(obj, &val16);
    of_bucket_watch_port_get(obj, &port_no);
    of_bucket_watch_group_get(obj, &val32);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_bucket_actions_bind(obj, &sub_cls);
    }

    value = of_bucket_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bucket_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bucket
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bucket_OF_VERSION_1_1_check(
    of_bucket_t *obj, int value)
{

    value = of_bucket_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_bucket_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_1_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bucket_t *dup;

        TEST_ASSERT((dup = of_bucket_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bucket_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bucket_counter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bucket_counter_OF_VERSION_1_1_populate(
    of_bucket_counter_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bucket_counter_packet_count_get(obj, &val64);
    of_bucket_counter_byte_count_get(obj, &val64);

    value = of_bucket_counter_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bucket_counter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bucket_counter_OF_VERSION_1_1_check(
    of_bucket_counter_t *obj, int value)
{

    value = of_bucket_counter_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bucket_counter_t *dup;

        TEST_ASSERT((dup = of_bucket_counter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bucket_counter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_1_populate(
    of_flow_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_entry_table_id_get(obj, &val8);
    of_flow_stats_entry_duration_sec_get(obj, &val32);
    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    of_flow_stats_entry_priority_get(obj, &val16);
    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    of_flow_stats_entry_cookie_get(obj, &val64);
    of_flow_stats_entry_packet_count_get(obj, &val64);
    of_flow_stats_entry_byte_count_get(obj, &val64);
    of_flow_stats_entry_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_stats_entry_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_stats_entry_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_1, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_1_check(
    of_flow_stats_entry_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_entry_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_1, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_stats_entry_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_1_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_flow_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_entry_OF_VERSION_1_1_populate(
    of_group_desc_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_entry_group_type_get(obj, &val8);
    of_group_desc_stats_entry_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_desc_stats_entry_buckets_bind(obj, &sub_cls);
    }

    value = of_group_desc_stats_entry_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_desc_stats_entry_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_entry_OF_VERSION_1_1_check(
    of_group_desc_stats_entry_t *obj, int value)
{

    value = of_group_desc_stats_entry_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_desc_stats_entry_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_1_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_entry_OF_VERSION_1_1_populate(
    of_group_stats_entry_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    of_list_bucket_counter_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_entry_group_id_get(obj, &val32);
    of_group_stats_entry_ref_count_get(obj, &val32);
    of_group_stats_entry_packet_count_get(obj, &val64);
    of_group_stats_entry_byte_count_get(obj, &val64);
    {
        of_list_bucket_counter_t sub_cls;

        /* Test bind */
        of_group_stats_entry_bucket_stats_bind(obj, &sub_cls);
    }

    value = of_group_stats_entry_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_counter_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_counter_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_stats_entry_bucket_stats_set(
        obj, list);
    of_list_bucket_counter_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_entry_OF_VERSION_1_1_check(
    of_group_stats_entry_t *obj, int value)
{

    value = of_group_stats_entry_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_counter_t *bucket_stats_ptr;

        bucket_stats_ptr = of_group_stats_entry_bucket_stats_get(obj);
        TEST_ASSERT(bucket_stats_ptr != NULL);
        value = of_list_bucket_counter_OF_VERSION_1_1_check(
            bucket_stats_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_counter_delete(bucket_stats_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_entry_t *dup;

        TEST_ASSERT((dup = of_group_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_1_populate(
    of_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_header_xid_get(obj, &val32);

    value = of_header_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_1_check(
    of_header_t *obj, int value)
{

    value = of_header_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_header_t *dup;

        TEST_ASSERT((dup = of_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_apply_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_apply_actions_OF_VERSION_1_1_populate(
    of_instruction_apply_actions_t *obj, int value)
{
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_instruction_apply_actions_actions_bind(obj, &sub_cls);
    }

    value = of_instruction_apply_actions_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_instruction_apply_actions_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_instruction_apply_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_apply_actions_OF_VERSION_1_1_check(
    of_instruction_apply_actions_t *obj, int value)
{

    value = of_instruction_apply_actions_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_instruction_apply_actions_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_1_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_apply_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_apply_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_apply_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_clear_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_clear_actions_OF_VERSION_1_1_populate(
    of_instruction_clear_actions_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_clear_actions_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_clear_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_clear_actions_OF_VERSION_1_1_check(
    of_instruction_clear_actions_t *obj, int value)
{

    value = of_instruction_clear_actions_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_clear_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_clear_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_clear_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_experimenter_OF_VERSION_1_1_populate(
    of_instruction_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_instruction_experimenter_experimenter_get(obj, &val32);
    of_instruction_experimenter_data_get(obj, &octets);

    value = of_instruction_experimenter_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_instruction_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_instruction_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_experimenter_OF_VERSION_1_1_check(
    of_instruction_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_instruction_experimenter_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_instruction_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_experimenter_t *dup;

        TEST_ASSERT((dup = of_instruction_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_goto_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_goto_table_OF_VERSION_1_1_populate(
    of_instruction_goto_table_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_instruction_goto_table_table_id_get(obj, &val8);

    value = of_instruction_goto_table_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_goto_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_goto_table_OF_VERSION_1_1_check(
    of_instruction_goto_table_t *obj, int value)
{

    value = of_instruction_goto_table_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_goto_table_t *dup;

        TEST_ASSERT((dup = of_instruction_goto_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_goto_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_header_OF_VERSION_1_1_populate(
    of_instruction_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_header_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_header_OF_VERSION_1_1_check(
    of_instruction_header_t *obj, int value)
{

    value = of_instruction_header_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_header_t *dup;

        TEST_ASSERT((dup = of_instruction_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_write_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_actions_OF_VERSION_1_1_populate(
    of_instruction_write_actions_t *obj, int value)
{
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_instruction_write_actions_actions_bind(obj, &sub_cls);
    }

    value = of_instruction_write_actions_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_instruction_write_actions_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_instruction_write_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_actions_OF_VERSION_1_1_check(
    of_instruction_write_actions_t *obj, int value)
{

    value = of_instruction_write_actions_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_instruction_write_actions_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_1_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_write_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_write_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_write_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_write_metadata
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_metadata_OF_VERSION_1_1_populate(
    of_instruction_write_metadata_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_instruction_write_metadata_metadata_get(obj, &val64);
    of_instruction_write_metadata_metadata_mask_get(obj, &val64);

    value = of_instruction_write_metadata_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_write_metadata
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_metadata_OF_VERSION_1_1_check(
    of_instruction_write_metadata_t *obj, int value)
{

    value = of_instruction_write_metadata_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_write_metadata_t *dup;

        TEST_ASSERT((dup = of_instruction_write_metadata_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_write_metadata_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_match_v2
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_match_v2_OF_VERSION_1_1_populate(
    of_match_v2_t *obj, int value)
{
    of_port_no_t port_no;
    of_wc_bmap_t wc_bmap;
    of_mac_addr_t mac_addr;
    uint16_t val16;
    uint8_t val8;
    of_ipv4_t ipv4;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_match_v2_in_port_get(obj, &port_no);
    of_match_v2_wildcards_get(obj, &wc_bmap);
    of_match_v2_eth_src_get(obj, &mac_addr);
    of_match_v2_eth_src_mask_get(obj, &mac_addr);
    of_match_v2_eth_dst_get(obj, &mac_addr);
    of_match_v2_eth_dst_mask_get(obj, &mac_addr);
    of_match_v2_vlan_vid_get(obj, &val16);
    of_match_v2_vlan_pcp_get(obj, &val8);
    of_match_v2_eth_type_get(obj, &val16);
    of_match_v2_ip_dscp_get(obj, &val8);
    of_match_v2_ip_proto_get(obj, &val8);
    of_match_v2_ipv4_src_get(obj, &ipv4);
    of_match_v2_ipv4_src_mask_get(obj, &ipv4);
    of_match_v2_ipv4_dst_get(obj, &ipv4);
    of_match_v2_ipv4_dst_mask_get(obj, &ipv4);
    of_match_v2_tcp_src_get(obj, &val16);
    of_match_v2_tcp_dst_get(obj, &val16);
    of_match_v2_mpls_label_get(obj, &val32);
    of_match_v2_mpls_tc_get(obj, &val8);
    of_match_v2_metadata_get(obj, &val64);
    of_match_v2_metadata_mask_get(obj, &val64);

    value = of_match_v2_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_match_v2
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_match_v2_OF_VERSION_1_1_check(
    of_match_v2_t *obj, int value)
{

    value = of_match_v2_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_match_v2_t *dup;

        TEST_ASSERT((dup = of_match_v2_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_match_v2_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_1_populate(
    of_packet_queue_t *obj, int value)
{
    uint32_t val32;
    of_list_queue_prop_t *list;

    /* Run thru accessors after new to ensure okay */
    of_packet_queue_queue_id_get(obj, &val32);
    {
        of_list_queue_prop_t sub_cls;

        /* Test bind */
        of_packet_queue_properties_bind(obj, &sub_cls);
    }

    value = of_packet_queue_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_prop_new(OF_VERSION_1_1);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_prop_OF_VERSION_1_1_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_queue_properties_set(
        obj, list);
    of_list_queue_prop_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_packet_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_1_check(
    of_packet_queue_t *obj, int value)
{

    value = of_packet_queue_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_prop_t *properties_ptr;

        properties_ptr = of_packet_queue_properties_get(obj);
        TEST_ASSERT(properties_ptr != NULL);
        value = of_list_queue_prop_OF_VERSION_1_1_check(
            properties_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_prop_delete(properties_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_queue_t *dup;

        TEST_ASSERT((dup = of_packet_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_desc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_1_populate(
    of_port_desc_t *obj, int value)
{
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_port_desc_port_no_get(obj, &port_no);
    of_port_desc_hw_addr_get(obj, &mac_addr);
    of_port_desc_name_get(obj, &port_name);
    of_port_desc_config_get(obj, &val32);
    of_port_desc_state_get(obj, &val32);
    of_port_desc_curr_get(obj, &val32);
    of_port_desc_advertised_get(obj, &val32);
    of_port_desc_supported_get(obj, &val32);
    of_port_desc_peer_get(obj, &val32);
    of_port_desc_curr_speed_get(obj, &val32);
    of_port_desc_max_speed_get(obj, &val32);

    value = of_port_desc_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_desc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_1_check(
    of_port_desc_t *obj, int value)
{

    value = of_port_desc_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_desc_t *dup;

        TEST_ASSERT((dup = of_port_desc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_desc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_1_populate(
    of_port_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_entry_port_no_get(obj, &port_no);
    of_port_stats_entry_rx_packets_get(obj, &val64);
    of_port_stats_entry_tx_packets_get(obj, &val64);
    of_port_stats_entry_rx_bytes_get(obj, &val64);
    of_port_stats_entry_tx_bytes_get(obj, &val64);
    of_port_stats_entry_rx_dropped_get(obj, &val64);
    of_port_stats_entry_tx_dropped_get(obj, &val64);
    of_port_stats_entry_rx_errors_get(obj, &val64);
    of_port_stats_entry_tx_errors_get(obj, &val64);
    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    of_port_stats_entry_rx_over_err_get(obj, &val64);
    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    of_port_stats_entry_collisions_get(obj, &val64);

    value = of_port_stats_entry_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_1_check(
    of_port_stats_entry_t *obj, int value)
{

    value = of_port_stats_entry_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_port_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_1_populate(
    of_queue_prop_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_queue_prop_header_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_1_check(
    of_queue_prop_header_t *obj, int value)
{

    value = of_queue_prop_header_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_header_t *dup;

        TEST_ASSERT((dup = of_queue_prop_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_min_rate
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_1_populate(
    of_queue_prop_min_rate_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_min_rate_rate_get(obj, &val16);

    value = of_queue_prop_min_rate_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_min_rate
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_1_check(
    of_queue_prop_min_rate_t *obj, int value)
{

    value = of_queue_prop_min_rate_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_min_rate_t *dup;

        TEST_ASSERT((dup = of_queue_prop_min_rate_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_min_rate_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_1_populate(
    of_queue_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_entry_port_no_get(obj, &port_no);
    of_queue_stats_entry_queue_id_get(obj, &val32);
    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    of_queue_stats_entry_tx_packets_get(obj, &val64);
    of_queue_stats_entry_tx_errors_get(obj, &val64);

    value = of_queue_stats_entry_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_1_check(
    of_queue_stats_entry_t *obj, int value)
{

    value = of_queue_stats_entry_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_queue_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_1_populate(
    of_table_stats_entry_t *obj, int value)
{
    uint8_t val8;
    of_table_name_t table_name;
    of_wc_bmap_t wc_bmap;
    of_match_bmap_t match_bmap;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_entry_table_id_get(obj, &val8);
    of_table_stats_entry_name_get(obj, &table_name);
    of_table_stats_entry_wildcards_get(obj, &wc_bmap);
    of_table_stats_entry_match_get(obj, &match_bmap);
    of_table_stats_entry_instructions_get(obj, &val32);
    of_table_stats_entry_write_actions_get(obj, &val32);
    of_table_stats_entry_apply_actions_get(obj, &val32);
    of_table_stats_entry_config_get(obj, &val32);
    of_table_stats_entry_max_entries_get(obj, &val32);
    of_table_stats_entry_active_count_get(obj, &val32);
    of_table_stats_entry_lookup_count_get(obj, &val64);
    of_table_stats_entry_matched_count_get(obj, &val64);

    value = of_table_stats_entry_OF_VERSION_1_1_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_1_check(
    of_table_stats_entry_t *obj, int value)
{

    value = of_table_stats_entry_OF_VERSION_1_1_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_table_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_action with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_1_populate(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_set_mpls_tc_t *set_mpls_tc;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_mpls_label_t *set_mpls_label;
    of_action_group_t *group;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_set_nw_ecn_t *set_nw_ecn;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_push_mpls_t *push_mpls;
    of_action_set_tp_src_t *set_tp_src;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_dl_src_t *set_dl_src;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    set_mpls_tc = &elt.set_mpls_tc;
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_mpls_label = &elt.set_mpls_label;
    group = &elt.group;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_vlan_pcp = &elt.set_vlan_pcp;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    set_nw_ecn = &elt.set_nw_ecn;
    set_dl_dst = &elt.set_dl_dst;
    push_mpls = &elt.push_mpls;
    set_tp_src = &elt.set_tp_src;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_dl_src = &elt.set_dl_src;
    set_queue = &elt.set_queue;
    output = &elt.output;

    /* Append two instances of type of_action_copy_ttl_out */

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_tc */

    of_action_set_mpls_tc_init(set_mpls_tc, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_tc);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_populate(
        set_mpls_tc, value);
    cur_len += set_mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_tc_init(set_mpls_tc, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_tc);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_populate(
        set_mpls_tc, value);
    cur_len += set_mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_1_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_1_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_tos */

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_1_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_tos_init(set_nw_tos, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_tos);
    value = of_action_set_nw_tos_OF_VERSION_1_1_populate(
        set_nw_tos, value);
    cur_len += set_nw_tos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_mpls_ttl */

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_dst */

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_1_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_dst_init(set_nw_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_dst);
    value = of_action_set_nw_dst_OF_VERSION_1_1_populate(
        set_nw_dst, value);
    cur_len += set_nw_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_label */

    of_action_set_mpls_label_init(set_mpls_label, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_label);
    value = of_action_set_mpls_label_OF_VERSION_1_1_populate(
        set_mpls_label, value);
    cur_len += set_mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_label_init(set_mpls_label, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_label);
    value = of_action_set_mpls_label_OF_VERSION_1_1_populate(
        set_mpls_label, value);
    cur_len += set_mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_group */

    of_action_group_init(group, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_1_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_group_init(group, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_1_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_src */

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_1_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_src_init(set_nw_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_src);
    value = of_action_set_nw_src_OF_VERSION_1_1_populate(
        set_nw_src, value);
    cur_len += set_nw_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_vid */

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_vid_init(set_vlan_vid, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_vid);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_populate(
        set_vlan_vid, value);
    cur_len += set_vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_ttl */

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_vlan */

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_1_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_1_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_dst */

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_1_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_dst_init(set_tp_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_dst);
    value = of_action_set_tp_dst_OF_VERSION_1_1_populate(
        set_tp_dst, value);
    cur_len += set_tp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_mpls */

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_1_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_1_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_vlan */

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_1_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_1_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_vlan_pcp */

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_vlan_pcp_init(set_vlan_pcp, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_vlan_pcp);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_populate(
        set_vlan_pcp, value);
    cur_len += set_vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ttl */

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_in */

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ecn */

    of_action_set_nw_ecn_init(set_nw_ecn, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ecn);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_populate(
        set_nw_ecn, value);
    cur_len += set_nw_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ecn_init(set_nw_ecn, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ecn);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_populate(
        set_nw_ecn, value);
    cur_len += set_nw_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_dst */

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_1_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_dst_init(set_dl_dst, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_dst);
    value = of_action_set_dl_dst_OF_VERSION_1_1_populate(
        set_dl_dst, value);
    cur_len += set_dl_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_mpls */

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_1_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_1_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_tp_src */

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_1_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_tp_src_init(set_tp_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_tp_src);
    value = of_action_set_tp_src_OF_VERSION_1_1_populate(
        set_tp_src, value);
    cur_len += set_tp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_nw_ttl */

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_dl_src */

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_1_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_dl_src_init(set_dl_src, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_dl_src);
    value = of_action_set_dl_src_OF_VERSION_1_1_populate(
        set_dl_src, value);
    cur_len += set_dl_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_queue */

    of_action_set_queue_init(set_queue, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_1_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_queue_init(set_queue, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_1_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_1_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_1, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_1_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_action generated by
 * of_list_action_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_1_check(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_set_mpls_tc_t *set_mpls_tc;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_nw_tos_t *set_nw_tos;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_set_nw_dst_t *set_nw_dst;
    of_action_set_mpls_label_t *set_mpls_label;
    of_action_group_t *group;
    of_action_set_nw_src_t *set_nw_src;
    of_action_set_vlan_vid_t *set_vlan_vid;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_set_tp_dst_t *set_tp_dst;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_vlan_pcp_t *set_vlan_pcp;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_set_nw_ecn_t *set_nw_ecn;
    of_action_set_dl_dst_t *set_dl_dst;
    of_action_push_mpls_t *push_mpls;
    of_action_set_tp_src_t *set_tp_src;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_dl_src_t *set_dl_src;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    set_mpls_tc = &elt.set_mpls_tc;
    bsn_mirror = &elt.bsn_mirror;
    set_nw_tos = &elt.set_nw_tos;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    set_nw_dst = &elt.set_nw_dst;
    set_mpls_label = &elt.set_mpls_label;
    group = &elt.group;
    set_nw_src = &elt.set_nw_src;
    set_vlan_vid = &elt.set_vlan_vid;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    set_tp_dst = &elt.set_tp_dst;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_vlan_pcp = &elt.set_vlan_pcp;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    set_nw_ecn = &elt.set_nw_ecn;
    set_dl_dst = &elt.set_dl_dst;
    push_mpls = &elt.push_mpls;
    set_tp_src = &elt.set_tp_src;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_dl_src = &elt.set_dl_src;
    set_queue = &elt.set_queue;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_1_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_tc */
    TEST_ASSERT(set_mpls_tc->length == 8);
    if (set_mpls_tc->wire_length_get != NULL) {
        int length;

        set_mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_tc->object_id == OF_ACTION_SET_MPLS_TC);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_check(
        set_mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_tc->length == 8);
    if (set_mpls_tc->wire_length_get != NULL) {
        int length;

        set_mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_tc->object_id == OF_ACTION_SET_MPLS_TC);
    value = of_action_set_mpls_tc_OF_VERSION_1_1_check(
        set_mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_1_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_1_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_tos */
    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_1_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_tos->length == 8);
    if (set_nw_tos->wire_length_get != NULL) {
        int length;

        set_nw_tos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_tos->object_id == OF_ACTION_SET_NW_TOS);
    value = of_action_set_nw_tos_OF_VERSION_1_1_check(
        set_nw_tos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_1_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_1_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_dst */
    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_1_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_dst->length == 8);
    if (set_nw_dst->wire_length_get != NULL) {
        int length;

        set_nw_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_dst->object_id == OF_ACTION_SET_NW_DST);
    value = of_action_set_nw_dst_OF_VERSION_1_1_check(
        set_nw_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_label */
    TEST_ASSERT(set_mpls_label->length == 8);
    if (set_mpls_label->wire_length_get != NULL) {
        int length;

        set_mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_label->object_id == OF_ACTION_SET_MPLS_LABEL);
    value = of_action_set_mpls_label_OF_VERSION_1_1_check(
        set_mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_label->length == 8);
    if (set_mpls_label->wire_length_get != NULL) {
        int length;

        set_mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_label->object_id == OF_ACTION_SET_MPLS_LABEL);
    value = of_action_set_mpls_label_OF_VERSION_1_1_check(
        set_mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_1_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_1_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_src */
    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_1_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_src->length == 8);
    if (set_nw_src->wire_length_get != NULL) {
        int length;

        set_nw_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_src->object_id == OF_ACTION_SET_NW_SRC);
    value = of_action_set_nw_src_OF_VERSION_1_1_check(
        set_nw_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_vid */
    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_vid->length == 8);
    if (set_vlan_vid->wire_length_get != NULL) {
        int length;

        set_vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_vid->object_id == OF_ACTION_SET_VLAN_VID);
    value = of_action_set_vlan_vid_OF_VERSION_1_1_check(
        set_vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_1_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_1_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_1_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_1_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_dst */
    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_1_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_dst->length == 8);
    if (set_tp_dst->wire_length_get != NULL) {
        int length;

        set_tp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_dst->object_id == OF_ACTION_SET_TP_DST);
    value = of_action_set_tp_dst_OF_VERSION_1_1_check(
        set_tp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_1_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_1_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_1_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_1_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_vlan_pcp */
    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_vlan_pcp->length == 8);
    if (set_vlan_pcp->wire_length_get != NULL) {
        int length;

        set_vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_vlan_pcp->object_id == OF_ACTION_SET_VLAN_PCP);
    value = of_action_set_vlan_pcp_OF_VERSION_1_1_check(
        set_vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_1_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_1_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ecn */
    TEST_ASSERT(set_nw_ecn->length == 8);
    if (set_nw_ecn->wire_length_get != NULL) {
        int length;

        set_nw_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ecn->object_id == OF_ACTION_SET_NW_ECN);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_check(
        set_nw_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ecn->length == 8);
    if (set_nw_ecn->wire_length_get != NULL) {
        int length;

        set_nw_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ecn->object_id == OF_ACTION_SET_NW_ECN);
    value = of_action_set_nw_ecn_OF_VERSION_1_1_check(
        set_nw_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_dst */
    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_1_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_dst->length == 16);
    if (set_dl_dst->wire_length_get != NULL) {
        int length;

        set_dl_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_dst->object_id == OF_ACTION_SET_DL_DST);
    value = of_action_set_dl_dst_OF_VERSION_1_1_check(
        set_dl_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_1_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_1_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_tp_src */
    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_1_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_tp_src->length == 8);
    if (set_tp_src->wire_length_get != NULL) {
        int length;

        set_tp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_tp_src->object_id == OF_ACTION_SET_TP_SRC);
    value = of_action_set_tp_src_OF_VERSION_1_1_check(
        set_tp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_1_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_dl_src */
    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_1_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_dl_src->length == 16);
    if (set_dl_src->wire_length_get != NULL) {
        int length;

        set_dl_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(set_dl_src->object_id == OF_ACTION_SET_DL_SRC);
    value = of_action_set_dl_src_OF_VERSION_1_1_check(
        set_dl_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_1_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_1_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_1_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_1_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_ACTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 56);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_action_t *dup;

        TEST_ASSERT((dup = of_list_action_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_action_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_interface with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_1_populate(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * of_list_bsn_interface_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_1_check(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_INTERFACE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_list_bsn_interface_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_interface_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bucket with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_OF_VERSION_1_1_populate(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket */

    of_bucket_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bucket generated by
 * of_list_bucket_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_OF_VERSION_1_1_check(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BUCKET_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bucket_t *dup;

        TEST_ASSERT((dup = of_list_bucket_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bucket_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bucket_counter with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_counter_OF_VERSION_1_1_populate(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket_counter */

    of_bucket_counter_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_counter_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bucket_counter generated by
 * of_list_bucket_counter_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_counter_OF_VERSION_1_1_check(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_counter_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_counter_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_counter_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BUCKET_COUNTER_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bucket_counter_t *dup;

        TEST_ASSERT((dup = of_list_bucket_counter_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bucket_counter_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_flow_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_1_populate(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * of_list_flow_stats_entry_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_1_check(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_FLOW_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_flow_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_flow_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_group_desc_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_group_desc_stats_entry_OF_VERSION_1_1_populate(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_desc_stats_entry */

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_group_desc_stats_entry generated by
 * of_list_group_desc_stats_entry_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_group_desc_stats_entry_OF_VERSION_1_1_check(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_GROUP_DESC_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_group_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_group_desc_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_group_desc_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_group_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_group_stats_entry_OF_VERSION_1_1_populate(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_stats_entry */

    of_group_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_group_stats_entry generated by
 * of_list_group_stats_entry_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_group_stats_entry_OF_VERSION_1_1_check(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_GROUP_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_group_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_group_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_group_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_instruction with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_OF_VERSION_1_1_populate(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;

    /* Append two instances of type of_instruction_clear_actions */

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_1_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_1_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_actions */

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_1_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_1_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_goto_table */

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_1_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_1_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_apply_actions */

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_1_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_1_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_metadata */

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_1_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_1, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_1_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_instruction generated by
 * of_list_instruction_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_OF_VERSION_1_1_check(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;
    TEST_OK(of_list_instruction_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_1_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_1_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_1_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_1_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_1_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_1_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_1_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_1_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_1_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_1_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_INSTRUCTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 10);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_instruction_t *dup;

        TEST_ASSERT((dup = of_list_instruction_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_instruction_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_packet_queue with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_1_populate(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * of_list_packet_queue_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_1_check(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PACKET_QUEUE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_packet_queue_t *dup;

        TEST_ASSERT((dup = of_list_packet_queue_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_packet_queue_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_desc with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_1_populate(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * of_list_port_desc_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_1_check(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_DESC_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_desc_t *dup;

        TEST_ASSERT((dup = of_list_port_desc_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_desc_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_1_populate(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * of_list_port_stats_entry_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_1_check(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_port_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_prop with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_1_populate(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_1, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_1, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * of_list_queue_prop_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_1_check(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_1_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_PROP_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_prop_t *dup;

        TEST_ASSERT((dup = of_list_queue_prop_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_prop_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_1_populate(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * of_list_queue_stats_entry_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_1_check(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_queue_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_table_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_1_populate(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_1, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_1_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * of_list_table_stats_entry_OF_VERSION_1_1_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_1_check(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_1_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_TABLE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_table_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_table_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_2_populate(
    of_aggregate_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_reply_xid_get(obj, &val32);
    of_aggregate_stats_reply_flags_get(obj, &val16);
    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    of_aggregate_stats_reply_flow_count_get(obj, &val32);

    value = of_aggregate_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_2_check(
    of_aggregate_stats_reply_t *obj, int value)
{

    value = of_aggregate_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_reply_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_2_populate(
    of_aggregate_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_request_xid_get(obj, &val32);
    of_aggregate_stats_request_flags_get(obj, &val16);
    of_aggregate_stats_request_table_id_get(obj, &val8);
    of_aggregate_stats_request_out_port_get(obj, &port_no);
    of_aggregate_stats_request_out_group_get(obj, &val32);
    of_aggregate_stats_request_cookie_get(obj, &val64);
    of_aggregate_stats_request_cookie_mask_get(obj, &val64);
    of_aggregate_stats_request_match_get(obj, &match);

    value = of_aggregate_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_2_check(
    of_aggregate_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_aggregate_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_request_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_action_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_2_populate(
    of_bad_action_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_action_error_msg_xid_get(obj, &val32);
    of_bad_action_error_msg_code_get(obj, &val16);
    of_bad_action_error_msg_data_get(obj, &octets);

    value = of_bad_action_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_action_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_2_check(
    of_bad_action_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_action_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_action_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_action_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_action_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_instruction_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_instruction_error_msg_OF_VERSION_1_2_populate(
    of_bad_instruction_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_instruction_error_msg_xid_get(obj, &val32);
    of_bad_instruction_error_msg_code_get(obj, &val16);
    of_bad_instruction_error_msg_data_get(obj, &octets);

    value = of_bad_instruction_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_instruction_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_instruction_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_instruction_error_msg_OF_VERSION_1_2_check(
    of_bad_instruction_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_instruction_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_instruction_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_instruction_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_instruction_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_instruction_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_match_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_match_error_msg_OF_VERSION_1_2_populate(
    of_bad_match_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_match_error_msg_xid_get(obj, &val32);
    of_bad_match_error_msg_code_get(obj, &val16);
    of_bad_match_error_msg_data_get(obj, &octets);

    value = of_bad_match_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_match_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_match_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_match_error_msg_OF_VERSION_1_2_check(
    of_bad_match_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_match_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_match_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_match_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_match_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_match_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_request_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_2_populate(
    of_bad_request_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_request_error_msg_xid_get(obj, &val32);
    of_bad_request_error_msg_code_get(obj, &val16);
    of_bad_request_error_msg_data_get(obj, &octets);

    value = of_bad_request_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_request_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_2_check(
    of_bad_request_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_request_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_request_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_request_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_request_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_2_populate(
    of_barrier_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_reply_xid_get(obj, &val32);

    value = of_barrier_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_2_check(
    of_barrier_reply_t *obj, int value)
{

    value = of_barrier_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_reply_t *dup;

        TEST_ASSERT((dup = of_barrier_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_2_populate(
    of_barrier_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_request_xid_get(obj, &val32);

    value = of_barrier_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_2_check(
    of_barrier_request_t *obj, int value)
{

    value = of_barrier_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_request_t *dup;

        TEST_ASSERT((dup = of_barrier_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_2_populate(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    of_bsn_bw_clear_data_reply_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_reply_subtype_get(obj, &val32);
    of_bsn_bw_clear_data_reply_status_get(obj, &val32);

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_2_check(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_2_populate(
    of_bsn_bw_clear_data_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    of_bsn_bw_clear_data_request_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_request_subtype_get(obj, &val32);

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_2_check(
    of_bsn_bw_clear_data_request_t *obj, int value)
{

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_2_populate(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_get_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_2_check(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_2_populate(
    of_bsn_bw_enable_get_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    of_bsn_bw_enable_get_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_request_subtype_get(obj, &val32);

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_2_check(
    of_bsn_bw_enable_get_request_t *obj, int value)
{

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_2_populate(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_set_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    of_bsn_bw_enable_set_reply_status_get(obj, &val32);

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_2_check(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_2_populate(
    of_bsn_bw_enable_set_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    of_bsn_bw_enable_set_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_request_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_request_enable_get(obj, &val32);

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_2_check(
    of_bsn_bw_enable_set_request_t *obj, int value)
{

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_2_populate(
    of_bsn_get_interfaces_reply_t *obj, int value)
{
    uint32_t val32;
    of_list_bsn_interface_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    of_bsn_get_interfaces_reply_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_interface_t sub_cls;

        /* Test bind */
        of_bsn_get_interfaces_reply_interfaces_bind(obj, &sub_cls);
    }

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_interface_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_interface_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_get_interfaces_reply_interfaces_set(
        obj, list);
    of_list_bsn_interface_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_2_check(
    of_bsn_get_interfaces_reply_t *obj, int value)
{

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_interface_t *interfaces_ptr;

        interfaces_ptr = of_bsn_get_interfaces_reply_interfaces_get(obj);
        TEST_ASSERT(interfaces_ptr != NULL);
        value = of_list_bsn_interface_OF_VERSION_1_2_check(
            interfaces_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_interface_delete(interfaces_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_2_populate(
    of_bsn_get_interfaces_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    of_bsn_get_interfaces_request_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_request_subtype_get(obj, &val32);

    value = of_bsn_get_interfaces_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_2_check(
    of_bsn_get_interfaces_request_t *obj, int value)
{

    value = of_bsn_get_interfaces_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_2_populate(
    of_bsn_get_mirroring_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    of_bsn_get_mirroring_reply_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_reply_subtype_get(obj, &val32);
    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_2_check(
    of_bsn_get_mirroring_reply_t *obj, int value)
{

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_2_populate(
    of_bsn_get_mirroring_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    of_bsn_get_mirroring_request_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_request_subtype_get(obj, &val32);
    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_2_check(
    of_bsn_get_mirroring_request_t *obj, int value)
{

    value = of_bsn_get_mirroring_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_2_populate(
    of_bsn_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_header_xid_get(obj, &val32);
    of_bsn_header_experimenter_get(obj, &val32);
    of_bsn_header_subtype_get(obj, &val32);

    value = of_bsn_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_2_check(
    of_bsn_header_t *obj, int value)
{

    value = of_bsn_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_header_t *dup;

        TEST_ASSERT((dup = of_bsn_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_2_populate(
    of_bsn_pdu_rx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    of_bsn_pdu_rx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_2_check(
    of_bsn_pdu_rx_reply_t *obj, int value)
{

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_2_populate(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    of_bsn_pdu_rx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_request_subtype_get(obj, &val32);
    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_rx_request_data_get(obj, &octets);

    value = of_bsn_pdu_rx_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_2_check(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_rx_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_timeout
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_2_populate(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    of_bsn_pdu_rx_timeout_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_timeout_subtype_get(obj, &val32);
    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_timeout
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_2_check(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_timeout_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_timeout_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_timeout_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_2_populate(
    of_bsn_pdu_tx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    of_bsn_pdu_tx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_2_check(
    of_bsn_pdu_tx_reply_t *obj, int value)
{

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_2_populate(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    of_bsn_pdu_tx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_request_subtype_get(obj, &val32);
    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_tx_request_data_get(obj, &octets);

    value = of_bsn_pdu_tx_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_2_check(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_tx_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_mirroring
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_2_populate(
    of_bsn_set_mirroring_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_mirroring_xid_get(obj, &val32);
    of_bsn_set_mirroring_experimenter_get(obj, &val32);
    of_bsn_set_mirroring_subtype_get(obj, &val32);
    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);

    value = of_bsn_set_mirroring_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_mirroring
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_2_check(
    of_bsn_set_mirroring_t *obj, int value)
{

    value = of_bsn_set_mirroring_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_mirroring_t *dup;

        TEST_ASSERT((dup = of_bsn_set_mirroring_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_mirroring_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_2_populate(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_2_check(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_2_populate(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_2_check(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_2_populate(
    of_bsn_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_reply_xid_get(obj, &val32);
    of_bsn_stats_reply_flags_get(obj, &val16);
    of_bsn_stats_reply_experimenter_get(obj, &val32);
    of_bsn_stats_reply_subtype_get(obj, &val32);

    value = of_bsn_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_2_check(
    of_bsn_stats_reply_t *obj, int value)
{

    value = of_bsn_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_2_populate(
    of_bsn_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_request_xid_get(obj, &val32);
    of_bsn_stats_request_flags_get(obj, &val16);
    of_bsn_stats_request_experimenter_get(obj, &val32);
    of_bsn_stats_request_subtype_get(obj, &val32);

    value = of_bsn_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_2_check(
    of_bsn_stats_request_t *obj, int value)
{

    value = of_bsn_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_2_populate(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_create_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_2_check(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_2_populate(
    of_bsn_virtual_port_create_request_t *obj, int value)
{
    uint32_t val32;
    of_bsn_vport_q_in_q_t *vport;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    of_bsn_virtual_port_create_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_request_subtype_get(obj, &val32);
    {
        of_bsn_vport_q_in_q_t sub_cls;

        /* Test bind */
        of_bsn_virtual_port_create_request_vport_bind(obj, &sub_cls);
    }

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    vport = of_bsn_vport_q_in_q_new(OF_VERSION_1_2);
    TEST_ASSERT(vport != NULL);
    value = of_bsn_vport_q_in_q_OF_VERSION_1_2_populate(
        vport, value);
    TEST_ASSERT(value != 0);
    of_bsn_virtual_port_create_request_vport_set(
        obj, vport);
    of_bsn_vport_q_in_q_delete(vport);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_2_check(
    of_bsn_virtual_port_create_request_t *obj, int value)
{

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_bsn_vport_q_in_q_t *vport_ptr;

        vport_ptr = of_bsn_virtual_port_create_request_vport_get(obj);
        TEST_ASSERT(vport_ptr != NULL);
        value = of_bsn_vport_q_in_q_OF_VERSION_1_2_check(
            vport_ptr, value);
        TEST_ASSERT(value != 0);
        of_bsn_vport_q_in_q_delete(vport_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_2_populate(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_2_check(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_2_populate(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_request_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_2_check(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_2_populate(
    of_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_reply_xid_get(obj, &val32);
    of_desc_stats_reply_flags_get(obj, &val16);
    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    of_desc_stats_reply_dp_desc_get(obj, &desc_str);

    value = of_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_2_check(
    of_desc_stats_reply_t *obj, int value)
{

    value = of_desc_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_2_populate(
    of_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_request_xid_get(obj, &val32);
    of_desc_stats_request_flags_get(obj, &val16);

    value = of_desc_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_2_check(
    of_desc_stats_request_t *obj, int value)
{

    value = of_desc_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_2_populate(
    of_echo_reply_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_reply_xid_get(obj, &val32);
    of_echo_reply_data_get(obj, &octets);

    value = of_echo_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_2_check(
    of_echo_reply_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_reply_t *dup;

        TEST_ASSERT((dup = of_echo_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_2_populate(
    of_echo_request_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_request_xid_get(obj, &val32);
    of_echo_request_data_get(obj, &octets);

    value = of_echo_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_2_check(
    of_echo_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_request_t *dup;

        TEST_ASSERT((dup = of_echo_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_2_populate(
    of_error_msg_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_error_msg_xid_get(obj, &val32);

    value = of_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_2_check(
    of_error_msg_t *obj, int value)
{

    value = of_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_error_msg_t *dup;

        TEST_ASSERT((dup = of_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_2_populate(
    of_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_xid_get(obj, &val32);
    of_experimenter_experimenter_get(obj, &val32);
    of_experimenter_subtype_get(obj, &val32);
    of_experimenter_data_get(obj, &octets);

    value = of_experimenter_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_2_check(
    of_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_t *dup;

        TEST_ASSERT((dup = of_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_error_msg_OF_VERSION_1_2_populate(
    of_experimenter_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_error_msg_xid_get(obj, &val32);
    of_experimenter_error_msg_subtype_get(obj, &val16);
    of_experimenter_error_msg_experimenter_get(obj, &val32);
    of_experimenter_error_msg_data_get(obj, &octets);

    value = of_experimenter_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_error_msg_OF_VERSION_1_2_check(
    of_experimenter_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_error_msg_t *dup;

        TEST_ASSERT((dup = of_experimenter_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_2_populate(
    of_experimenter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_reply_xid_get(obj, &val32);
    of_experimenter_stats_reply_flags_get(obj, &val16);
    of_experimenter_stats_reply_experimenter_get(obj, &val32);
    of_experimenter_stats_reply_subtype_get(obj, &val32);
    of_experimenter_stats_reply_data_get(obj, &octets);

    value = of_experimenter_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_reply_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_2_check(
    of_experimenter_stats_reply_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_reply_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_2_populate(
    of_experimenter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_request_xid_get(obj, &val32);
    of_experimenter_stats_request_flags_get(obj, &val16);
    of_experimenter_stats_request_experimenter_get(obj, &val32);
    of_experimenter_stats_request_subtype_get(obj, &val32);
    of_experimenter_stats_request_data_get(obj, &octets);

    value = of_experimenter_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_2_check(
    of_experimenter_stats_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_stats_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_request_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_2_populate(
    of_features_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    of_list_port_desc_t *list;

    /* Run thru accessors after new to ensure okay */
    of_features_reply_xid_get(obj, &val32);
    of_features_reply_datapath_id_get(obj, &val64);
    of_features_reply_n_buffers_get(obj, &val32);
    of_features_reply_n_tables_get(obj, &val8);
    of_features_reply_capabilities_get(obj, &val32);
    of_features_reply_reserved_get(obj, &val32);
    {
        of_list_port_desc_t sub_cls;

        /* Test bind */
        of_features_reply_ports_bind(obj, &sub_cls);
    }

    value = of_features_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_desc_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_port_desc_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_features_reply_ports_set(
        obj, list);
    of_list_port_desc_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_features_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_2_check(
    of_features_reply_t *obj, int value)
{

    value = of_features_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_desc_t *ports_ptr;

        ports_ptr = of_features_reply_ports_get(obj);
        TEST_ASSERT(ports_ptr != NULL);
        value = of_list_port_desc_OF_VERSION_1_2_check(
            ports_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_desc_delete(ports_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_features_reply_t *dup;

        TEST_ASSERT((dup = of_features_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_2_populate(
    of_features_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_features_request_xid_get(obj, &val32);

    value = of_features_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_features_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_2_check(
    of_features_request_t *obj, int value)
{

    value = of_features_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_features_request_t *dup;

        TEST_ASSERT((dup = of_features_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_2_populate(
    of_flow_add_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_add_xid_get(obj, &val32);
    of_flow_add_cookie_get(obj, &val64);
    of_flow_add_cookie_mask_get(obj, &val64);
    of_flow_add_table_id_get(obj, &val8);
    of_flow_add_idle_timeout_get(obj, &val16);
    of_flow_add_hard_timeout_get(obj, &val16);
    of_flow_add_priority_get(obj, &val16);
    of_flow_add_buffer_id_get(obj, &val32);
    of_flow_add_out_port_get(obj, &port_no);
    of_flow_add_out_group_get(obj, &val32);
    of_flow_add_flags_get(obj, &val16);
    of_flow_add_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_add_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_add_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_add_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_2_check(
    of_flow_add_t *obj, int value)
{
    of_match_t match;

    value = of_flow_add_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_add_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_2_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_add_t *dup;

        TEST_ASSERT((dup = of_flow_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_2_populate(
    of_flow_delete_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_xid_get(obj, &val32);
    of_flow_delete_cookie_get(obj, &val64);
    of_flow_delete_cookie_mask_get(obj, &val64);
    of_flow_delete_table_id_get(obj, &val8);
    of_flow_delete_idle_timeout_get(obj, &val16);
    of_flow_delete_hard_timeout_get(obj, &val16);
    of_flow_delete_priority_get(obj, &val16);
    of_flow_delete_buffer_id_get(obj, &val32);
    of_flow_delete_out_port_get(obj, &port_no);
    of_flow_delete_out_group_get(obj, &val32);
    of_flow_delete_flags_get(obj, &val16);
    of_flow_delete_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_delete_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_2_check(
    of_flow_delete_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_delete_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_2_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_t *dup;

        TEST_ASSERT((dup = of_flow_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_2_populate(
    of_flow_delete_strict_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_strict_xid_get(obj, &val32);
    of_flow_delete_strict_cookie_get(obj, &val64);
    of_flow_delete_strict_cookie_mask_get(obj, &val64);
    of_flow_delete_strict_table_id_get(obj, &val8);
    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    of_flow_delete_strict_priority_get(obj, &val16);
    of_flow_delete_strict_buffer_id_get(obj, &val32);
    of_flow_delete_strict_out_port_get(obj, &port_no);
    of_flow_delete_strict_out_group_get(obj, &val32);
    of_flow_delete_strict_flags_get(obj, &val16);
    of_flow_delete_strict_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_delete_strict_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_strict_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_2_check(
    of_flow_delete_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_strict_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_delete_strict_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_2_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_strict_t *dup;

        TEST_ASSERT((dup = of_flow_delete_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_2_populate(
    of_flow_mod_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_xid_get(obj, &val32);
    of_flow_mod_cookie_get(obj, &val64);
    of_flow_mod_cookie_mask_get(obj, &val64);
    of_flow_mod_table_id_get(obj, &val8);
    of_flow_mod_idle_timeout_get(obj, &val16);
    of_flow_mod_hard_timeout_get(obj, &val16);
    of_flow_mod_priority_get(obj, &val16);
    of_flow_mod_buffer_id_get(obj, &val32);
    of_flow_mod_out_port_get(obj, &port_no);
    of_flow_mod_out_group_get(obj, &val32);
    of_flow_mod_flags_get(obj, &val16);
    of_flow_mod_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_mod_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_mod_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_2_check(
    of_flow_mod_t *obj, int value)
{
    of_match_t match;

    value = of_flow_mod_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_mod_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_2_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_t *dup;

        TEST_ASSERT((dup = of_flow_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_2_populate(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_flow_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_2_check(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_flow_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_2_populate(
    of_flow_modify_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_xid_get(obj, &val32);
    of_flow_modify_cookie_get(obj, &val64);
    of_flow_modify_cookie_mask_get(obj, &val64);
    of_flow_modify_table_id_get(obj, &val8);
    of_flow_modify_idle_timeout_get(obj, &val16);
    of_flow_modify_hard_timeout_get(obj, &val16);
    of_flow_modify_priority_get(obj, &val16);
    of_flow_modify_buffer_id_get(obj, &val32);
    of_flow_modify_out_port_get(obj, &port_no);
    of_flow_modify_out_group_get(obj, &val32);
    of_flow_modify_flags_get(obj, &val16);
    of_flow_modify_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_modify_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_2_check(
    of_flow_modify_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_modify_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_2_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_t *dup;

        TEST_ASSERT((dup = of_flow_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_2_populate(
    of_flow_modify_strict_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_strict_xid_get(obj, &val32);
    of_flow_modify_strict_cookie_get(obj, &val64);
    of_flow_modify_strict_cookie_mask_get(obj, &val64);
    of_flow_modify_strict_table_id_get(obj, &val8);
    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    of_flow_modify_strict_priority_get(obj, &val16);
    of_flow_modify_strict_buffer_id_get(obj, &val32);
    of_flow_modify_strict_out_port_get(obj, &port_no);
    of_flow_modify_strict_out_group_get(obj, &val32);
    of_flow_modify_strict_flags_get(obj, &val16);
    of_flow_modify_strict_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_modify_strict_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_strict_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_2_check(
    of_flow_modify_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_strict_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_modify_strict_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_2_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_strict_t *dup;

        TEST_ASSERT((dup = of_flow_modify_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_removed
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_2_populate(
    of_flow_removed_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_flow_removed_xid_get(obj, &val32);
    of_flow_removed_cookie_get(obj, &val64);
    of_flow_removed_priority_get(obj, &val16);
    of_flow_removed_reason_get(obj, &val8);
    of_flow_removed_table_id_get(obj, &val8);
    of_flow_removed_duration_sec_get(obj, &val32);
    of_flow_removed_duration_nsec_get(obj, &val32);
    of_flow_removed_idle_timeout_get(obj, &val16);
    of_flow_removed_hard_timeout_get(obj, &val16);
    of_flow_removed_packet_count_get(obj, &val64);
    of_flow_removed_byte_count_get(obj, &val64);
    of_flow_removed_match_get(obj, &match);

    value = of_flow_removed_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_removed
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_2_check(
    of_flow_removed_t *obj, int value)
{
    of_match_t match;

    value = of_flow_removed_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_removed_t *dup;

        TEST_ASSERT((dup = of_flow_removed_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_removed_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_2_populate(
    of_flow_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_flow_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_reply_xid_get(obj, &val32);
    of_flow_stats_reply_flags_get(obj, &val16);
    {
        of_list_flow_stats_entry_t sub_cls;

        /* Test bind */
        of_flow_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_flow_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_flow_stats_entry_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_flow_stats_entry_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_reply_entries_set(
        obj, list);
    of_list_flow_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_2_check(
    of_flow_stats_reply_t *obj, int value)
{

    value = of_flow_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_flow_stats_entry_t *entries_ptr;

        entries_ptr = of_flow_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_flow_stats_entry_OF_VERSION_1_2_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_flow_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_reply_t *dup;

        TEST_ASSERT((dup = of_flow_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_2_populate(
    of_flow_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_request_xid_get(obj, &val32);
    of_flow_stats_request_flags_get(obj, &val16);
    of_flow_stats_request_table_id_get(obj, &val8);
    of_flow_stats_request_out_port_get(obj, &port_no);
    of_flow_stats_request_out_group_get(obj, &val32);
    of_flow_stats_request_cookie_get(obj, &val64);
    of_flow_stats_request_cookie_mask_get(obj, &val64);
    of_flow_stats_request_match_get(obj, &match);

    value = of_flow_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_2_check(
    of_flow_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_request_t *dup;

        TEST_ASSERT((dup = of_flow_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_2_populate(
    of_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_get_config_reply_xid_get(obj, &val32);
    of_get_config_reply_flags_get(obj, &val16);
    of_get_config_reply_miss_send_len_get(obj, &val16);

    value = of_get_config_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_2_check(
    of_get_config_reply_t *obj, int value)
{

    value = of_get_config_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_2_populate(
    of_get_config_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_get_config_request_xid_get(obj, &val32);

    value = of_get_config_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_2_check(
    of_get_config_request_t *obj, int value)
{

    value = of_get_config_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_request_t *dup;

        TEST_ASSERT((dup = of_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_add_OF_VERSION_1_2_populate(
    of_group_add_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_add_xid_get(obj, &val32);
    of_group_add_group_type_get(obj, &val8);
    of_group_add_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_add_buckets_bind(obj, &sub_cls);
    }

    value = of_group_add_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_add_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_add_OF_VERSION_1_2_check(
    of_group_add_t *obj, int value)
{

    value = of_group_add_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_add_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_2_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_add_t *dup;

        TEST_ASSERT((dup = of_group_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_delete_OF_VERSION_1_2_populate(
    of_group_delete_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_delete_xid_get(obj, &val32);
    of_group_delete_group_type_get(obj, &val8);
    of_group_delete_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_delete_buckets_bind(obj, &sub_cls);
    }

    value = of_group_delete_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_delete_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_delete_OF_VERSION_1_2_check(
    of_group_delete_t *obj, int value)
{

    value = of_group_delete_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_delete_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_2_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_delete_t *dup;

        TEST_ASSERT((dup = of_group_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_reply_OF_VERSION_1_2_populate(
    of_group_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_group_desc_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_reply_xid_get(obj, &val32);
    of_group_desc_stats_reply_flags_get(obj, &val16);
    {
        of_list_group_desc_stats_entry_t sub_cls;

        /* Test bind */
        of_group_desc_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_group_desc_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_group_desc_stats_entry_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_group_desc_stats_entry_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_desc_stats_reply_entries_set(
        obj, list);
    of_list_group_desc_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_reply_OF_VERSION_1_2_check(
    of_group_desc_stats_reply_t *obj, int value)
{

    value = of_group_desc_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_group_desc_stats_entry_t *entries_ptr;

        entries_ptr = of_group_desc_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_group_desc_stats_entry_OF_VERSION_1_2_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_group_desc_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_request_OF_VERSION_1_2_populate(
    of_group_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_request_xid_get(obj, &val32);
    of_group_desc_stats_request_flags_get(obj, &val16);

    value = of_group_desc_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_request_OF_VERSION_1_2_check(
    of_group_desc_stats_request_t *obj, int value)
{

    value = of_group_desc_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_features_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_reply_OF_VERSION_1_2_populate(
    of_group_features_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_features_stats_reply_xid_get(obj, &val32);
    of_group_features_stats_reply_flags_get(obj, &val16);
    of_group_features_stats_reply_types_get(obj, &val32);
    of_group_features_stats_reply_capabilities_get(obj, &val32);
    of_group_features_stats_reply_max_groups_all_get(obj, &val32);
    of_group_features_stats_reply_max_groups_select_get(obj, &val32);
    of_group_features_stats_reply_max_groups_indirect_get(obj, &val32);
    of_group_features_stats_reply_max_groups_ff_get(obj, &val32);
    of_group_features_stats_reply_actions_all_get(obj, &val32);
    of_group_features_stats_reply_actions_select_get(obj, &val32);
    of_group_features_stats_reply_actions_indirect_get(obj, &val32);
    of_group_features_stats_reply_actions_ff_get(obj, &val32);

    value = of_group_features_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_features_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_reply_OF_VERSION_1_2_check(
    of_group_features_stats_reply_t *obj, int value)
{

    value = of_group_features_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_features_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_features_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_features_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_features_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_request_OF_VERSION_1_2_populate(
    of_group_features_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_features_stats_request_xid_get(obj, &val32);
    of_group_features_stats_request_flags_get(obj, &val16);

    value = of_group_features_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_features_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_request_OF_VERSION_1_2_check(
    of_group_features_stats_request_t *obj, int value)
{

    value = of_group_features_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_features_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_features_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_features_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_mod_OF_VERSION_1_2_populate(
    of_group_mod_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_mod_xid_get(obj, &val32);
    of_group_mod_group_type_get(obj, &val8);
    of_group_mod_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_mod_buckets_bind(obj, &sub_cls);
    }

    value = of_group_mod_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_mod_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_mod_OF_VERSION_1_2_check(
    of_group_mod_t *obj, int value)
{

    value = of_group_mod_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_mod_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_2_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_mod_t *dup;

        TEST_ASSERT((dup = of_group_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_mod_failed_error_msg_OF_VERSION_1_2_populate(
    of_group_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_group_mod_failed_error_msg_xid_get(obj, &val32);
    of_group_mod_failed_error_msg_code_get(obj, &val16);
    of_group_mod_failed_error_msg_data_get(obj, &octets);

    value = of_group_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_group_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_group_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_mod_failed_error_msg_OF_VERSION_1_2_check(
    of_group_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_group_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_group_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_group_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_modify_OF_VERSION_1_2_populate(
    of_group_modify_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_modify_xid_get(obj, &val32);
    of_group_modify_group_type_get(obj, &val8);
    of_group_modify_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_modify_buckets_bind(obj, &sub_cls);
    }

    value = of_group_modify_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_modify_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_modify_OF_VERSION_1_2_check(
    of_group_modify_t *obj, int value)
{

    value = of_group_modify_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_modify_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_2_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_modify_t *dup;

        TEST_ASSERT((dup = of_group_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_reply_OF_VERSION_1_2_populate(
    of_group_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_group_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_reply_xid_get(obj, &val32);
    of_group_stats_reply_flags_get(obj, &val16);
    {
        of_list_group_stats_entry_t sub_cls;

        /* Test bind */
        of_group_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_group_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_group_stats_entry_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_group_stats_entry_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_stats_reply_entries_set(
        obj, list);
    of_list_group_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_reply_OF_VERSION_1_2_check(
    of_group_stats_reply_t *obj, int value)
{

    value = of_group_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_group_stats_entry_t *entries_ptr;

        entries_ptr = of_group_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_group_stats_entry_OF_VERSION_1_2_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_group_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_request_OF_VERSION_1_2_populate(
    of_group_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_request_xid_get(obj, &val32);
    of_group_stats_request_flags_get(obj, &val16);
    of_group_stats_request_group_id_get(obj, &val32);

    value = of_group_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_request_OF_VERSION_1_2_check(
    of_group_stats_request_t *obj, int value)
{

    value = of_group_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_2_populate(
    of_hello_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_hello_xid_get(obj, &val32);

    value = of_hello_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_hello
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_2_check(
    of_hello_t *obj, int value)
{

    value = of_hello_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_t *dup;

        TEST_ASSERT((dup = of_hello_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_2_populate(
    of_hello_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_hello_failed_error_msg_xid_get(obj, &val32);
    of_hello_failed_error_msg_code_get(obj, &val16);
    of_hello_failed_error_msg_data_get(obj, &octets);

    value = of_hello_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_hello_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_2_check(
    of_hello_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_hello_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_hello_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_nicira_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_2_populate(
    of_nicira_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_nicira_header_xid_get(obj, &val32);
    of_nicira_header_experimenter_get(obj, &val32);
    of_nicira_header_subtype_get(obj, &val32);

    value = of_nicira_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_nicira_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_2_check(
    of_nicira_header_t *obj, int value)
{

    value = of_nicira_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_nicira_header_t *dup;

        TEST_ASSERT((dup = of_nicira_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_nicira_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_2_populate(
    of_packet_in_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_match_t match;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_in_xid_get(obj, &val32);
    of_packet_in_buffer_id_get(obj, &val32);
    of_packet_in_total_len_get(obj, &val16);
    of_packet_in_reason_get(obj, &val8);
    of_packet_in_table_id_get(obj, &val8);
    of_packet_in_match_get(obj, &match);
    of_packet_in_data_get(obj, &octets);

    value = of_packet_in_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_packet_in_match_set(
        obj, &match);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_in_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_2_check(
    of_packet_in_t *obj, int value)
{
    of_match_t match;
    of_octets_t octets;

    value = of_packet_in_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_packet_in_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);
    of_packet_in_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_in_t *dup;

        TEST_ASSERT((dup = of_packet_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_2_populate(
    of_packet_out_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_action_t *list;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_out_xid_get(obj, &val32);
    of_packet_out_buffer_id_get(obj, &val32);
    of_packet_out_in_port_get(obj, &port_no);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_packet_out_actions_bind(obj, &sub_cls);
    }
    of_packet_out_data_get(obj, &octets);

    value = of_packet_out_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_out_actions_set(
        obj, list);
    of_list_action_delete(list);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_out_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_2_check(
    of_packet_out_t *obj, int value)
{
    of_octets_t octets;

    value = of_packet_out_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_packet_out_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_2_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }
    of_packet_out_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_out_t *dup;

        TEST_ASSERT((dup = of_packet_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_2_populate(
    of_port_mod_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_xid_get(obj, &val32);
    of_port_mod_port_no_get(obj, &port_no);
    of_port_mod_hw_addr_get(obj, &mac_addr);
    of_port_mod_config_get(obj, &val32);
    of_port_mod_mask_get(obj, &val32);
    of_port_mod_advertise_get(obj, &val32);

    value = of_port_mod_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_2_check(
    of_port_mod_t *obj, int value)
{

    value = of_port_mod_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_t *dup;

        TEST_ASSERT((dup = of_port_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_2_populate(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    of_port_mod_failed_error_msg_code_get(obj, &val16);
    of_port_mod_failed_error_msg_data_get(obj, &octets);

    value = of_port_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_port_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_2_check(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_port_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_port_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_2_populate(
    of_port_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_port_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_reply_xid_get(obj, &val32);
    of_port_stats_reply_flags_get(obj, &val16);
    {
        of_list_port_stats_entry_t sub_cls;

        /* Test bind */
        of_port_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_port_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_stats_entry_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_port_stats_entry_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_port_stats_reply_entries_set(
        obj, list);
    of_list_port_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_2_check(
    of_port_stats_reply_t *obj, int value)
{

    value = of_port_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_stats_entry_t *entries_ptr;

        entries_ptr = of_port_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_port_stats_entry_OF_VERSION_1_2_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_reply_t *dup;

        TEST_ASSERT((dup = of_port_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_2_populate(
    of_port_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_request_xid_get(obj, &val32);
    of_port_stats_request_flags_get(obj, &val16);
    of_port_stats_request_port_no_get(obj, &port_no);

    value = of_port_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_2_check(
    of_port_stats_request_t *obj, int value)
{

    value = of_port_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_request_t *dup;

        TEST_ASSERT((dup = of_port_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_2_populate(
    of_port_status_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_port_desc_t *port_desc;

    /* Run thru accessors after new to ensure okay */
    of_port_status_xid_get(obj, &val32);
    of_port_status_reason_get(obj, &val8);
    {
        of_port_desc_t sub_cls;

        /* Test bind */
        of_port_status_desc_bind(obj, &sub_cls);
    }

    value = of_port_status_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    port_desc = of_port_desc_new(OF_VERSION_1_2);
    TEST_ASSERT(port_desc != NULL);
    value = of_port_desc_OF_VERSION_1_2_populate(
        port_desc, value);
    TEST_ASSERT(value != 0);
    of_port_status_desc_set(
        obj, port_desc);
    of_port_desc_delete(port_desc);

    return value;
}

/**
 * Check all members of an object of type of_port_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_2_check(
    of_port_status_t *obj, int value)
{

    value = of_port_status_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_port_desc_t *desc_ptr;

        desc_ptr = of_port_status_desc_get(obj);
        TEST_ASSERT(desc_ptr != NULL);
        value = of_port_desc_OF_VERSION_1_2_check(
            desc_ptr, value);
        TEST_ASSERT(value != 0);
        of_port_desc_delete(desc_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_status_t *dup;

        TEST_ASSERT((dup = of_port_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_2_populate(
    of_queue_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_packet_queue_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_reply_xid_get(obj, &val32);
    of_queue_get_config_reply_port_get(obj, &port_no);
    {
        of_list_packet_queue_t sub_cls;

        /* Test bind */
        of_queue_get_config_reply_queues_bind(obj, &sub_cls);
    }

    value = of_queue_get_config_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_packet_queue_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_packet_queue_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_get_config_reply_queues_set(
        obj, list);
    of_list_packet_queue_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_2_check(
    of_queue_get_config_reply_t *obj, int value)
{

    value = of_queue_get_config_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_packet_queue_t *queues_ptr;

        queues_ptr = of_queue_get_config_reply_queues_get(obj);
        TEST_ASSERT(queues_ptr != NULL);
        value = of_list_packet_queue_OF_VERSION_1_2_check(
            queues_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_packet_queue_delete(queues_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_2_populate(
    of_queue_get_config_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_request_xid_get(obj, &val32);
    of_queue_get_config_request_port_get(obj, &port_no);

    value = of_queue_get_config_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_2_check(
    of_queue_get_config_request_t *obj, int value)
{

    value = of_queue_get_config_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_request_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_op_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_2_populate(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    of_queue_op_failed_error_msg_code_get(obj, &val16);
    of_queue_op_failed_error_msg_data_get(obj, &octets);

    value = of_queue_op_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_queue_op_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_2_check(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_queue_op_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_op_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_queue_op_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_op_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_2_populate(
    of_queue_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_queue_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_reply_xid_get(obj, &val32);
    of_queue_stats_reply_flags_get(obj, &val16);
    {
        of_list_queue_stats_entry_t sub_cls;

        /* Test bind */
        of_queue_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_queue_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_stats_entry_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_stats_entry_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_stats_reply_entries_set(
        obj, list);
    of_list_queue_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_2_check(
    of_queue_stats_reply_t *obj, int value)
{

    value = of_queue_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_stats_entry_t *entries_ptr;

        entries_ptr = of_queue_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_queue_stats_entry_OF_VERSION_1_2_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_reply_t *dup;

        TEST_ASSERT((dup = of_queue_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_2_populate(
    of_queue_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_request_xid_get(obj, &val32);
    of_queue_stats_request_flags_get(obj, &val16);
    of_queue_stats_request_port_no_get(obj, &port_no);
    of_queue_stats_request_queue_id_get(obj, &val32);

    value = of_queue_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_2_check(
    of_queue_stats_request_t *obj, int value)
{

    value = of_queue_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_request_t *dup;

        TEST_ASSERT((dup = of_queue_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_role_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_role_reply_OF_VERSION_1_2_populate(
    of_role_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_role_reply_xid_get(obj, &val32);
    of_role_reply_role_get(obj, &val32);
    of_role_reply_generation_id_get(obj, &val64);

    value = of_role_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_role_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_role_reply_OF_VERSION_1_2_check(
    of_role_reply_t *obj, int value)
{

    value = of_role_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_role_reply_t *dup;

        TEST_ASSERT((dup = of_role_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_role_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_role_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_role_request_OF_VERSION_1_2_populate(
    of_role_request_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_role_request_xid_get(obj, &val32);
    of_role_request_role_get(obj, &val32);
    of_role_request_generation_id_get(obj, &val64);

    value = of_role_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_role_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_role_request_OF_VERSION_1_2_check(
    of_role_request_t *obj, int value)
{

    value = of_role_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_role_request_t *dup;

        TEST_ASSERT((dup = of_role_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_role_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_role_request_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_role_request_failed_error_msg_OF_VERSION_1_2_populate(
    of_role_request_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_role_request_failed_error_msg_xid_get(obj, &val32);
    of_role_request_failed_error_msg_code_get(obj, &val16);
    of_role_request_failed_error_msg_data_get(obj, &octets);

    value = of_role_request_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_role_request_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_role_request_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_role_request_failed_error_msg_OF_VERSION_1_2_check(
    of_role_request_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_role_request_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_role_request_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_role_request_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_role_request_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_role_request_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_set_config
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_2_populate(
    of_set_config_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_set_config_xid_get(obj, &val32);
    of_set_config_flags_get(obj, &val16);
    of_set_config_miss_send_len_get(obj, &val16);

    value = of_set_config_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_set_config
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_2_check(
    of_set_config_t *obj, int value)
{

    value = of_set_config_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_set_config_t *dup;

        TEST_ASSERT((dup = of_set_config_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_set_config_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_2_populate(
    of_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_reply_xid_get(obj, &val32);
    of_stats_reply_flags_get(obj, &val16);

    value = of_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_2_check(
    of_stats_reply_t *obj, int value)
{

    value = of_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_reply_t *dup;

        TEST_ASSERT((dup = of_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_2_populate(
    of_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_request_xid_get(obj, &val32);
    of_stats_request_flags_get(obj, &val16);

    value = of_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_2_check(
    of_stats_request_t *obj, int value)
{

    value = of_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_request_t *dup;

        TEST_ASSERT((dup = of_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_switch_config_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_switch_config_failed_error_msg_OF_VERSION_1_2_populate(
    of_switch_config_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_switch_config_failed_error_msg_xid_get(obj, &val32);
    of_switch_config_failed_error_msg_code_get(obj, &val16);
    of_switch_config_failed_error_msg_data_get(obj, &octets);

    value = of_switch_config_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_switch_config_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_switch_config_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_switch_config_failed_error_msg_OF_VERSION_1_2_check(
    of_switch_config_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_switch_config_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_switch_config_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_switch_config_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_switch_config_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_switch_config_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_2_populate(
    of_table_mod_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_table_mod_xid_get(obj, &val32);
    of_table_mod_table_id_get(obj, &val8);
    of_table_mod_config_get(obj, &val32);

    value = of_table_mod_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_2_check(
    of_table_mod_t *obj, int value)
{

    value = of_table_mod_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_mod_t *dup;

        TEST_ASSERT((dup = of_table_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_mod_failed_error_msg_OF_VERSION_1_2_populate(
    of_table_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_table_mod_failed_error_msg_xid_get(obj, &val32);
    of_table_mod_failed_error_msg_code_get(obj, &val16);
    of_table_mod_failed_error_msg_data_get(obj, &octets);

    value = of_table_mod_failed_error_msg_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_table_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_table_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_mod_failed_error_msg_OF_VERSION_1_2_check(
    of_table_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_table_mod_failed_error_msg_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_table_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_table_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_2_populate(
    of_table_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_table_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_reply_xid_get(obj, &val32);
    of_table_stats_reply_flags_get(obj, &val16);
    {
        of_list_table_stats_entry_t sub_cls;

        /* Test bind */
        of_table_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_table_stats_reply_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_table_stats_entry_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_table_stats_entry_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_stats_reply_entries_set(
        obj, list);
    of_list_table_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_2_check(
    of_table_stats_reply_t *obj, int value)
{

    value = of_table_stats_reply_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_table_stats_entry_t *entries_ptr;

        entries_ptr = of_table_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_table_stats_entry_OF_VERSION_1_2_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_table_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_reply_t *dup;

        TEST_ASSERT((dup = of_table_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_2_populate(
    of_table_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_request_xid_get(obj, &val32);
    of_table_stats_request_flags_get(obj, &val16);

    value = of_table_stats_request_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_2_check(
    of_table_stats_request_t *obj, int value)
{

    value = of_table_stats_request_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_request_t *dup;

        TEST_ASSERT((dup = of_table_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_2_populate(
    of_action_bsn_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_experimenter_get(obj, &val32);
    of_action_bsn_subtype_get(obj, &val32);

    value = of_action_bsn_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_2_check(
    of_action_bsn_t *obj, int value)
{

    value = of_action_bsn_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_t *dup;

        TEST_ASSERT((dup = of_action_bsn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_mirror
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_2_populate(
    of_action_bsn_mirror_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_mirror_experimenter_get(obj, &val32);
    of_action_bsn_mirror_subtype_get(obj, &val32);
    of_action_bsn_mirror_dest_port_get(obj, &val32);
    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    of_action_bsn_mirror_copy_stage_get(obj, &val8);

    value = of_action_bsn_mirror_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_mirror
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_2_check(
    of_action_bsn_mirror_t *obj, int value)
{

    value = of_action_bsn_mirror_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_mirror_t *dup;

        TEST_ASSERT((dup = of_action_bsn_mirror_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_mirror_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_set_tunnel_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_populate(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_set_tunnel_dst_experimenter_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_subtype_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_set_tunnel_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_check(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_set_tunnel_dst_t *dup;

        TEST_ASSERT((dup = of_action_bsn_set_tunnel_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_set_tunnel_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_copy_ttl_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_in_OF_VERSION_1_2_populate(
    of_action_copy_ttl_in_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_copy_ttl_in_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_copy_ttl_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_in_OF_VERSION_1_2_check(
    of_action_copy_ttl_in_t *obj, int value)
{

    value = of_action_copy_ttl_in_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_copy_ttl_in_t *dup;

        TEST_ASSERT((dup = of_action_copy_ttl_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_copy_ttl_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_copy_ttl_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_out_OF_VERSION_1_2_populate(
    of_action_copy_ttl_out_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_copy_ttl_out_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_copy_ttl_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_out_OF_VERSION_1_2_check(
    of_action_copy_ttl_out_t *obj, int value)
{

    value = of_action_copy_ttl_out_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_copy_ttl_out_t *dup;

        TEST_ASSERT((dup = of_action_copy_ttl_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_copy_ttl_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_dec_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_dec_mpls_ttl_OF_VERSION_1_2_populate(
    of_action_dec_mpls_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_dec_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_dec_mpls_ttl_OF_VERSION_1_2_check(
    of_action_dec_mpls_ttl_t *obj, int value)
{

    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_dec_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_dec_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_dec_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_dec_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_dec_nw_ttl_OF_VERSION_1_2_populate(
    of_action_dec_nw_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_dec_nw_ttl_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_dec_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_dec_nw_ttl_OF_VERSION_1_2_check(
    of_action_dec_nw_ttl_t *obj, int value)
{

    value = of_action_dec_nw_ttl_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_dec_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_dec_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_dec_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_2_populate(
    of_action_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_action_experimenter_experimenter_get(obj, &val32);
    of_action_experimenter_data_get(obj, &octets);

    value = of_action_experimenter_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_action_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_2_check(
    of_action_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_action_experimenter_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_experimenter_t *dup;

        TEST_ASSERT((dup = of_action_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_group
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_group_OF_VERSION_1_2_populate(
    of_action_group_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_group_group_id_get(obj, &val32);

    value = of_action_group_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_group
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_group_OF_VERSION_1_2_check(
    of_action_group_t *obj, int value)
{

    value = of_action_group_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_group_t *dup;

        TEST_ASSERT((dup = of_action_group_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_group_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_2_populate(
    of_action_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_2_check(
    of_action_header_t *obj, int value)
{

    value = of_action_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_header_t *dup;

        TEST_ASSERT((dup = of_action_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_2_populate(
    of_action_nicira_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_experimenter_get(obj, &val32);
    of_action_nicira_subtype_get(obj, &val16);

    value = of_action_nicira_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_2_check(
    of_action_nicira_t *obj, int value)
{

    value = of_action_nicira_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira_dec_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_2_populate(
    of_action_nicira_dec_ttl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_dec_ttl_experimenter_get(obj, &val32);
    of_action_nicira_dec_ttl_subtype_get(obj, &val16);

    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira_dec_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_2_check(
    of_action_nicira_dec_ttl_t *obj, int value)
{

    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_dec_ttl_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dec_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_dec_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_output
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_2_populate(
    of_action_output_t *obj, int value)
{
    of_port_no_t port_no;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_output_port_get(obj, &port_no);
    of_action_output_max_len_get(obj, &val16);

    value = of_action_output_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_output
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_2_check(
    of_action_output_t *obj, int value)
{

    value = of_action_output_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_output_t *dup;

        TEST_ASSERT((dup = of_action_output_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_output_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_pop_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_pop_mpls_OF_VERSION_1_2_populate(
    of_action_pop_mpls_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_pop_mpls_ethertype_get(obj, &val16);

    value = of_action_pop_mpls_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_pop_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_pop_mpls_OF_VERSION_1_2_check(
    of_action_pop_mpls_t *obj, int value)
{

    value = of_action_pop_mpls_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_pop_mpls_t *dup;

        TEST_ASSERT((dup = of_action_pop_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_pop_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_pop_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_pop_vlan_OF_VERSION_1_2_populate(
    of_action_pop_vlan_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_pop_vlan_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_pop_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_pop_vlan_OF_VERSION_1_2_check(
    of_action_pop_vlan_t *obj, int value)
{

    value = of_action_pop_vlan_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_pop_vlan_t *dup;

        TEST_ASSERT((dup = of_action_pop_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_pop_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_push_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_push_mpls_OF_VERSION_1_2_populate(
    of_action_push_mpls_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_push_mpls_ethertype_get(obj, &val16);

    value = of_action_push_mpls_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_push_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_push_mpls_OF_VERSION_1_2_check(
    of_action_push_mpls_t *obj, int value)
{

    value = of_action_push_mpls_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_push_mpls_t *dup;

        TEST_ASSERT((dup = of_action_push_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_push_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_push_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_push_vlan_OF_VERSION_1_2_populate(
    of_action_push_vlan_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_push_vlan_ethertype_get(obj, &val16);

    value = of_action_push_vlan_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_push_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_push_vlan_OF_VERSION_1_2_check(
    of_action_push_vlan_t *obj, int value)
{

    value = of_action_push_vlan_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_push_vlan_t *dup;

        TEST_ASSERT((dup = of_action_push_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_push_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_field
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_field_OF_VERSION_1_2_populate(
    of_action_set_field_t *obj, int value)
{
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_action_set_field_field_get(obj, &octets);

    value = of_action_set_field_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_action_set_field_field_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_action_set_field
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_field_OF_VERSION_1_2_check(
    of_action_set_field_t *obj, int value)
{
    of_octets_t octets;

    value = of_action_set_field_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_action_set_field_field_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_field_t *dup;

        TEST_ASSERT((dup = of_action_set_field_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_field_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_ttl_OF_VERSION_1_2_populate(
    of_action_set_mpls_ttl_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_mpls_ttl_mpls_ttl_get(obj, &val8);

    value = of_action_set_mpls_ttl_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_ttl_OF_VERSION_1_2_check(
    of_action_set_mpls_ttl_t *obj, int value)
{

    value = of_action_set_mpls_ttl_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_set_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ttl_OF_VERSION_1_2_populate(
    of_action_set_nw_ttl_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_ttl_nw_ttl_get(obj, &val8);

    value = of_action_set_nw_ttl_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ttl_OF_VERSION_1_2_check(
    of_action_set_nw_ttl_t *obj, int value)
{

    value = of_action_set_nw_ttl_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_queue_OF_VERSION_1_2_populate(
    of_action_set_queue_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_queue_queue_id_get(obj, &val32);

    value = of_action_set_queue_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_queue_OF_VERSION_1_2_check(
    of_action_set_queue_t *obj, int value)
{

    value = of_action_set_queue_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_queue_t *dup;

        TEST_ASSERT((dup = of_action_set_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_interface
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_2_populate(
    of_bsn_interface_t *obj, int value)
{
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    of_bsn_interface_name_get(obj, &port_name);
    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);

    value = of_bsn_interface_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_interface
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_2_check(
    of_bsn_interface_t *obj, int value)
{

    value = of_bsn_interface_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_bsn_interface_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_interface_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_2_populate(
    of_bsn_vport_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_bsn_vport_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_2_check(
    of_bsn_vport_header_t *obj, int value)
{

    value = of_bsn_vport_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_header_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_q_in_q
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_2_populate(
    of_bsn_vport_q_in_q_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    /* Run thru accessors after new to ensure okay */
    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);

    value = of_bsn_vport_q_in_q_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_q_in_q
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_2_check(
    of_bsn_vport_q_in_q_t *obj, int value)
{

    value = of_bsn_vport_q_in_q_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_q_in_q_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_q_in_q_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_q_in_q_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bucket
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bucket_OF_VERSION_1_2_populate(
    of_bucket_t *obj, int value)
{
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bucket_weight_get(obj, &val16);
    of_bucket_watch_port_get(obj, &port_no);
    of_bucket_watch_group_get(obj, &val32);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_bucket_actions_bind(obj, &sub_cls);
    }

    value = of_bucket_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bucket_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bucket
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bucket_OF_VERSION_1_2_check(
    of_bucket_t *obj, int value)
{

    value = of_bucket_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_bucket_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_2_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bucket_t *dup;

        TEST_ASSERT((dup = of_bucket_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bucket_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bucket_counter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bucket_counter_OF_VERSION_1_2_populate(
    of_bucket_counter_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bucket_counter_packet_count_get(obj, &val64);
    of_bucket_counter_byte_count_get(obj, &val64);

    value = of_bucket_counter_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bucket_counter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bucket_counter_OF_VERSION_1_2_check(
    of_bucket_counter_t *obj, int value)
{

    value = of_bucket_counter_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bucket_counter_t *dup;

        TEST_ASSERT((dup = of_bucket_counter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bucket_counter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_2_populate(
    of_flow_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_entry_table_id_get(obj, &val8);
    of_flow_stats_entry_duration_sec_get(obj, &val32);
    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    of_flow_stats_entry_priority_get(obj, &val16);
    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    of_flow_stats_entry_cookie_get(obj, &val64);
    of_flow_stats_entry_packet_count_get(obj, &val64);
    of_flow_stats_entry_byte_count_get(obj, &val64);
    of_flow_stats_entry_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_stats_entry_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_stats_entry_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_2, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_2_check(
    of_flow_stats_entry_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_entry_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_2, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_stats_entry_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_2_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_flow_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_entry_OF_VERSION_1_2_populate(
    of_group_desc_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_entry_group_type_get(obj, &val8);
    of_group_desc_stats_entry_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_desc_stats_entry_buckets_bind(obj, &sub_cls);
    }

    value = of_group_desc_stats_entry_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_desc_stats_entry_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_entry_OF_VERSION_1_2_check(
    of_group_desc_stats_entry_t *obj, int value)
{

    value = of_group_desc_stats_entry_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_desc_stats_entry_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_2_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_entry_OF_VERSION_1_2_populate(
    of_group_stats_entry_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    of_list_bucket_counter_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_entry_group_id_get(obj, &val32);
    of_group_stats_entry_ref_count_get(obj, &val32);
    of_group_stats_entry_packet_count_get(obj, &val64);
    of_group_stats_entry_byte_count_get(obj, &val64);
    {
        of_list_bucket_counter_t sub_cls;

        /* Test bind */
        of_group_stats_entry_bucket_stats_bind(obj, &sub_cls);
    }

    value = of_group_stats_entry_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_counter_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_counter_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_stats_entry_bucket_stats_set(
        obj, list);
    of_list_bucket_counter_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_entry_OF_VERSION_1_2_check(
    of_group_stats_entry_t *obj, int value)
{

    value = of_group_stats_entry_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_counter_t *bucket_stats_ptr;

        bucket_stats_ptr = of_group_stats_entry_bucket_stats_get(obj);
        TEST_ASSERT(bucket_stats_ptr != NULL);
        value = of_list_bucket_counter_OF_VERSION_1_2_check(
            bucket_stats_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_counter_delete(bucket_stats_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_entry_t *dup;

        TEST_ASSERT((dup = of_group_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_2_populate(
    of_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_header_xid_get(obj, &val32);

    value = of_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_2_check(
    of_header_t *obj, int value)
{

    value = of_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_header_t *dup;

        TEST_ASSERT((dup = of_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_apply_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_apply_actions_OF_VERSION_1_2_populate(
    of_instruction_apply_actions_t *obj, int value)
{
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_instruction_apply_actions_actions_bind(obj, &sub_cls);
    }

    value = of_instruction_apply_actions_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_instruction_apply_actions_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_instruction_apply_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_apply_actions_OF_VERSION_1_2_check(
    of_instruction_apply_actions_t *obj, int value)
{

    value = of_instruction_apply_actions_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_instruction_apply_actions_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_2_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_apply_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_apply_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_apply_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_clear_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_clear_actions_OF_VERSION_1_2_populate(
    of_instruction_clear_actions_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_clear_actions_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_clear_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_clear_actions_OF_VERSION_1_2_check(
    of_instruction_clear_actions_t *obj, int value)
{

    value = of_instruction_clear_actions_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_clear_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_clear_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_clear_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_experimenter_OF_VERSION_1_2_populate(
    of_instruction_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_instruction_experimenter_experimenter_get(obj, &val32);
    of_instruction_experimenter_data_get(obj, &octets);

    value = of_instruction_experimenter_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_instruction_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_instruction_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_experimenter_OF_VERSION_1_2_check(
    of_instruction_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_instruction_experimenter_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_instruction_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_experimenter_t *dup;

        TEST_ASSERT((dup = of_instruction_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_goto_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_goto_table_OF_VERSION_1_2_populate(
    of_instruction_goto_table_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_instruction_goto_table_table_id_get(obj, &val8);

    value = of_instruction_goto_table_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_goto_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_goto_table_OF_VERSION_1_2_check(
    of_instruction_goto_table_t *obj, int value)
{

    value = of_instruction_goto_table_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_goto_table_t *dup;

        TEST_ASSERT((dup = of_instruction_goto_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_goto_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_header_OF_VERSION_1_2_populate(
    of_instruction_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_header_OF_VERSION_1_2_check(
    of_instruction_header_t *obj, int value)
{

    value = of_instruction_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_header_t *dup;

        TEST_ASSERT((dup = of_instruction_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_write_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_actions_OF_VERSION_1_2_populate(
    of_instruction_write_actions_t *obj, int value)
{
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_instruction_write_actions_actions_bind(obj, &sub_cls);
    }

    value = of_instruction_write_actions_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_instruction_write_actions_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_instruction_write_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_actions_OF_VERSION_1_2_check(
    of_instruction_write_actions_t *obj, int value)
{

    value = of_instruction_write_actions_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_instruction_write_actions_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_2_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_write_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_write_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_write_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_write_metadata
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_metadata_OF_VERSION_1_2_populate(
    of_instruction_write_metadata_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_instruction_write_metadata_metadata_get(obj, &val64);
    of_instruction_write_metadata_metadata_mask_get(obj, &val64);

    value = of_instruction_write_metadata_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_write_metadata
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_metadata_OF_VERSION_1_2_check(
    of_instruction_write_metadata_t *obj, int value)
{

    value = of_instruction_write_metadata_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_write_metadata_t *dup;

        TEST_ASSERT((dup = of_instruction_write_metadata_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_write_metadata_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_match_v3
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_match_v3_OF_VERSION_1_2_populate(
    of_match_v3_t *obj, int value)
{
    of_list_oxm_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_oxm_t sub_cls;

        /* Test bind */
        of_match_v3_oxm_list_bind(obj, &sub_cls);
    }

    value = of_match_v3_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_oxm_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_oxm_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_match_v3_oxm_list_set(
        obj, list);
    of_list_oxm_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_match_v3
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_match_v3_OF_VERSION_1_2_check(
    of_match_v3_t *obj, int value)
{

    value = of_match_v3_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_oxm_t *oxm_list_ptr;

        oxm_list_ptr = of_match_v3_oxm_list_get(obj);
        TEST_ASSERT(oxm_list_ptr != NULL);
        value = of_list_oxm_OF_VERSION_1_2_check(
            oxm_list_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_oxm_delete(oxm_list_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_match_v3_t *dup;

        TEST_ASSERT((dup = of_match_v3_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_match_v3_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_op
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_OF_VERSION_1_2_populate(
    of_oxm_arp_op_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_op_value_get(obj, &val16);

    value = of_oxm_arp_op_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_op
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_OF_VERSION_1_2_check(
    of_oxm_arp_op_t *obj, int value)
{

    value = of_oxm_arp_op_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_op_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_op_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_op_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_op_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_masked_OF_VERSION_1_2_populate(
    of_oxm_arp_op_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_op_masked_value_get(obj, &val16);
    of_oxm_arp_op_masked_value_mask_get(obj, &val16);

    value = of_oxm_arp_op_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_op_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_masked_OF_VERSION_1_2_check(
    of_oxm_arp_op_masked_t *obj, int value)
{

    value = of_oxm_arp_op_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_op_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_op_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_op_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_sha
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_OF_VERSION_1_2_populate(
    of_oxm_arp_sha_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_sha_value_get(obj, &mac_addr);

    value = of_oxm_arp_sha_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_sha
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_OF_VERSION_1_2_check(
    of_oxm_arp_sha_t *obj, int value)
{

    value = of_oxm_arp_sha_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_sha_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_sha_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_sha_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_sha_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_masked_OF_VERSION_1_2_populate(
    of_oxm_arp_sha_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_sha_masked_value_get(obj, &mac_addr);
    of_oxm_arp_sha_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_sha_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_masked_OF_VERSION_1_2_check(
    of_oxm_arp_sha_masked_t *obj, int value)
{

    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_sha_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_sha_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_sha_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_spa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_OF_VERSION_1_2_populate(
    of_oxm_arp_spa_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_spa_value_get(obj, &val32);

    value = of_oxm_arp_spa_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_spa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_OF_VERSION_1_2_check(
    of_oxm_arp_spa_t *obj, int value)
{

    value = of_oxm_arp_spa_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_spa_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_spa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_spa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_spa_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_masked_OF_VERSION_1_2_populate(
    of_oxm_arp_spa_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_spa_masked_value_get(obj, &val32);
    of_oxm_arp_spa_masked_value_mask_get(obj, &val32);

    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_spa_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_masked_OF_VERSION_1_2_check(
    of_oxm_arp_spa_masked_t *obj, int value)
{

    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_spa_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_spa_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_spa_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tha
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_OF_VERSION_1_2_populate(
    of_oxm_arp_tha_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tha_value_get(obj, &mac_addr);

    value = of_oxm_arp_tha_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tha
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_OF_VERSION_1_2_check(
    of_oxm_arp_tha_t *obj, int value)
{

    value = of_oxm_arp_tha_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tha_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tha_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tha_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tha_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_masked_OF_VERSION_1_2_populate(
    of_oxm_arp_tha_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tha_masked_value_get(obj, &mac_addr);
    of_oxm_arp_tha_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tha_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_masked_OF_VERSION_1_2_check(
    of_oxm_arp_tha_masked_t *obj, int value)
{

    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tha_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tha_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tha_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_OF_VERSION_1_2_populate(
    of_oxm_arp_tpa_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tpa_value_get(obj, &val32);

    value = of_oxm_arp_tpa_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_OF_VERSION_1_2_check(
    of_oxm_arp_tpa_t *obj, int value)
{

    value = of_oxm_arp_tpa_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tpa_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tpa_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_masked_OF_VERSION_1_2_populate(
    of_oxm_arp_tpa_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tpa_masked_value_get(obj, &val32);
    of_oxm_arp_tpa_masked_value_mask_get(obj, &val32);

    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tpa_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_masked_OF_VERSION_1_2_check(
    of_oxm_arp_tpa_masked_t *obj, int value)
{

    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tpa_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tpa_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tpa_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_egr_port_group_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_populate(
    of_oxm_bsn_egr_port_group_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_egr_port_group_id_value_get(obj, &val32);

    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_egr_port_group_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_check(
    of_oxm_bsn_egr_port_group_id_t *obj, int value)
{

    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_egr_port_group_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_egr_port_group_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_egr_port_group_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_egr_port_group_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_egr_port_group_id_masked_value_get(obj, &val32);
    of_oxm_bsn_egr_port_group_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_egr_port_group_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_egr_port_group_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_egr_port_group_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_egr_port_group_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_global_vrf_allowed
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_populate(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_global_vrf_allowed_value_get(obj, &val8);

    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_global_vrf_allowed
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_check(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value)
{

    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_global_vrf_allowed_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_global_vrf_allowed_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_global_vrf_allowed_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_global_vrf_allowed_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_global_vrf_allowed_masked_value_get(obj, &val8);
    of_oxm_bsn_global_vrf_allowed_masked_value_mask_get(obj, &val8);

    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_global_vrf_allowed_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value)
{

    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_global_vrf_allowed_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_global_vrf_allowed_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_global_vrf_allowed_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_in_ports_128
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_OF_VERSION_1_2_populate(
    of_oxm_bsn_in_ports_128_t *obj, int value)
{
    of_bitmap_128_t bitmap_128;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_in_ports_128_value_get(obj, &bitmap_128);

    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_in_ports_128
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_OF_VERSION_1_2_check(
    of_oxm_bsn_in_ports_128_t *obj, int value)
{

    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_in_ports_128_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_in_ports_128_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_in_ports_128_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_in_ports_128_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value)
{
    of_bitmap_128_t bitmap_128;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_in_ports_128_masked_value_get(obj, &bitmap_128);
    of_oxm_bsn_in_ports_128_masked_value_mask_get(obj, &bitmap_128);

    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_in_ports_128_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value)
{

    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_in_ports_128_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_in_ports_128_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_in_ports_128_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_dst_class_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_populate(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_dst_class_id_value_get(obj, &val32);

    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_dst_class_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_check(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value)
{

    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_dst_class_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_dst_class_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_dst_class_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_dst_class_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_dst_class_id_masked_value_get(obj, &val32);
    of_oxm_bsn_l3_dst_class_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_dst_class_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_dst_class_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_dst_class_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_dst_class_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_interface_class_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_populate(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_interface_class_id_value_get(obj, &val32);

    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_interface_class_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_check(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value)
{

    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_interface_class_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_interface_class_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_interface_class_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_interface_class_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_interface_class_id_masked_value_get(obj, &val32);
    of_oxm_bsn_l3_interface_class_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_interface_class_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_interface_class_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_interface_class_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_interface_class_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_src_class_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_populate(
    of_oxm_bsn_l3_src_class_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_src_class_id_value_get(obj, &val32);

    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_src_class_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_check(
    of_oxm_bsn_l3_src_class_id_t *obj, int value)
{

    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_src_class_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_src_class_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_src_class_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_src_class_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_src_class_id_masked_value_get(obj, &val32);
    of_oxm_bsn_l3_src_class_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_src_class_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_src_class_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_src_class_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_src_class_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_lag_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_OF_VERSION_1_2_populate(
    of_oxm_bsn_lag_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_lag_id_value_get(obj, &val32);

    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_lag_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_OF_VERSION_1_2_check(
    of_oxm_bsn_lag_id_t *obj, int value)
{

    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_lag_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_lag_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_lag_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_lag_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_lag_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_lag_id_masked_value_get(obj, &val32);
    of_oxm_bsn_lag_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_lag_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_lag_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_lag_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_lag_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_lag_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf0
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf0_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf0_value_get(obj, &val32);

    value = of_oxm_bsn_udf0_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf0
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_OF_VERSION_1_2_check(
    of_oxm_bsn_udf0_t *obj, int value)
{

    value = of_oxm_bsn_udf0_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf0_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf0_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf0_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf0_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf0_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf0_masked_value_get(obj, &val32);
    of_oxm_bsn_udf0_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf0_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf0_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf0_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf0_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf0_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf1
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf1_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf1_value_get(obj, &val32);

    value = of_oxm_bsn_udf1_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf1
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_OF_VERSION_1_2_check(
    of_oxm_bsn_udf1_t *obj, int value)
{

    value = of_oxm_bsn_udf1_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf1_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf1_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf1_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf1_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf1_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf1_masked_value_get(obj, &val32);
    of_oxm_bsn_udf1_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf1_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf1_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf1_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf1_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf1_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf2
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf2_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf2_value_get(obj, &val32);

    value = of_oxm_bsn_udf2_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf2
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_OF_VERSION_1_2_check(
    of_oxm_bsn_udf2_t *obj, int value)
{

    value = of_oxm_bsn_udf2_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf2_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf2_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf2_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf2_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf2_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf2_masked_value_get(obj, &val32);
    of_oxm_bsn_udf2_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf2_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf2_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf2_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf2_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf2_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf3
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf3_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf3_value_get(obj, &val32);

    value = of_oxm_bsn_udf3_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf3
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_OF_VERSION_1_2_check(
    of_oxm_bsn_udf3_t *obj, int value)
{

    value = of_oxm_bsn_udf3_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf3_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf3_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf3_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf3_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf3_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf3_masked_value_get(obj, &val32);
    of_oxm_bsn_udf3_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf3_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf3_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf3_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf3_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf3_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf4
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf4_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf4_value_get(obj, &val32);

    value = of_oxm_bsn_udf4_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf4
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_OF_VERSION_1_2_check(
    of_oxm_bsn_udf4_t *obj, int value)
{

    value = of_oxm_bsn_udf4_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf4_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf4_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf4_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf4_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf4_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf4_masked_value_get(obj, &val32);
    of_oxm_bsn_udf4_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf4_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf4_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf4_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf4_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf4_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf5
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf5_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf5_value_get(obj, &val32);

    value = of_oxm_bsn_udf5_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf5
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_OF_VERSION_1_2_check(
    of_oxm_bsn_udf5_t *obj, int value)
{

    value = of_oxm_bsn_udf5_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf5_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf5_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf5_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf5_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf5_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf5_masked_value_get(obj, &val32);
    of_oxm_bsn_udf5_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf5_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf5_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf5_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf5_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf5_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf6
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf6_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf6_value_get(obj, &val32);

    value = of_oxm_bsn_udf6_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf6
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_OF_VERSION_1_2_check(
    of_oxm_bsn_udf6_t *obj, int value)
{

    value = of_oxm_bsn_udf6_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf6_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf6_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf6_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf6_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf6_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf6_masked_value_get(obj, &val32);
    of_oxm_bsn_udf6_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf6_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf6_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf6_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf6_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf6_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf7
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf7_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf7_value_get(obj, &val32);

    value = of_oxm_bsn_udf7_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf7
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_OF_VERSION_1_2_check(
    of_oxm_bsn_udf7_t *obj, int value)
{

    value = of_oxm_bsn_udf7_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf7_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf7_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf7_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf7_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_udf7_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf7_masked_value_get(obj, &val32);
    of_oxm_bsn_udf7_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf7_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_udf7_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf7_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf7_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf7_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_vrf
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_OF_VERSION_1_2_populate(
    of_oxm_bsn_vrf_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_vrf_value_get(obj, &val32);

    value = of_oxm_bsn_vrf_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_vrf
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_OF_VERSION_1_2_check(
    of_oxm_bsn_vrf_t *obj, int value)
{

    value = of_oxm_bsn_vrf_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_vrf_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_vrf_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_vrf_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_vrf_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_masked_OF_VERSION_1_2_populate(
    of_oxm_bsn_vrf_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_vrf_masked_value_get(obj, &val32);
    of_oxm_bsn_vrf_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_vrf_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_masked_OF_VERSION_1_2_check(
    of_oxm_bsn_vrf_masked_t *obj, int value)
{

    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_vrf_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_vrf_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_vrf_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_OF_VERSION_1_2_populate(
    of_oxm_eth_dst_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_dst_value_get(obj, &mac_addr);

    value = of_oxm_eth_dst_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_OF_VERSION_1_2_check(
    of_oxm_eth_dst_t *obj, int value)
{

    value = of_oxm_eth_dst_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_masked_OF_VERSION_1_2_populate(
    of_oxm_eth_dst_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_dst_masked_value_get(obj, &mac_addr);
    of_oxm_eth_dst_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_masked_OF_VERSION_1_2_check(
    of_oxm_eth_dst_masked_t *obj, int value)
{

    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_OF_VERSION_1_2_populate(
    of_oxm_eth_src_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_src_value_get(obj, &mac_addr);

    value = of_oxm_eth_src_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_OF_VERSION_1_2_check(
    of_oxm_eth_src_t *obj, int value)
{

    value = of_oxm_eth_src_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_src_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_masked_OF_VERSION_1_2_populate(
    of_oxm_eth_src_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_src_masked_value_get(obj, &mac_addr);
    of_oxm_eth_src_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_eth_src_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_masked_OF_VERSION_1_2_check(
    of_oxm_eth_src_masked_t *obj, int value)
{

    value = of_oxm_eth_src_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_type
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_OF_VERSION_1_2_populate(
    of_oxm_eth_type_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_type_value_get(obj, &val16);

    value = of_oxm_eth_type_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_type
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_OF_VERSION_1_2_check(
    of_oxm_eth_type_t *obj, int value)
{

    value = of_oxm_eth_type_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_type_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_type_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_type_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_type_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_masked_OF_VERSION_1_2_populate(
    of_oxm_eth_type_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_type_masked_value_get(obj, &val16);
    of_oxm_eth_type_masked_value_mask_get(obj, &val16);

    value = of_oxm_eth_type_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_type_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_masked_OF_VERSION_1_2_check(
    of_oxm_eth_type_masked_t *obj, int value)
{

    value = of_oxm_eth_type_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_type_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_type_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_type_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_header_OF_VERSION_1_2_populate(
    of_oxm_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_oxm_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_header_OF_VERSION_1_2_check(
    of_oxm_header_t *obj, int value)
{

    value = of_oxm_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_header_t *dup;

        TEST_ASSERT((dup = of_oxm_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_code
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_OF_VERSION_1_2_populate(
    of_oxm_icmpv4_code_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_code_value_get(obj, &val8);

    value = of_oxm_icmpv4_code_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_code
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_OF_VERSION_1_2_check(
    of_oxm_icmpv4_code_t *obj, int value)
{

    value = of_oxm_icmpv4_code_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_code_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_code_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_code_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_code_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_masked_OF_VERSION_1_2_populate(
    of_oxm_icmpv4_code_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_code_masked_value_get(obj, &val8);
    of_oxm_icmpv4_code_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_code_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_masked_OF_VERSION_1_2_check(
    of_oxm_icmpv4_code_masked_t *obj, int value)
{

    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_code_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_code_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_code_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_type
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_OF_VERSION_1_2_populate(
    of_oxm_icmpv4_type_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_type_value_get(obj, &val8);

    value = of_oxm_icmpv4_type_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_type
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_OF_VERSION_1_2_check(
    of_oxm_icmpv4_type_t *obj, int value)
{

    value = of_oxm_icmpv4_type_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_type_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_type_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_type_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_type_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_masked_OF_VERSION_1_2_populate(
    of_oxm_icmpv4_type_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_type_masked_value_get(obj, &val8);
    of_oxm_icmpv4_type_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_type_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_masked_OF_VERSION_1_2_check(
    of_oxm_icmpv4_type_masked_t *obj, int value)
{

    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_type_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_type_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_type_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_code
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_OF_VERSION_1_2_populate(
    of_oxm_icmpv6_code_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_code_value_get(obj, &val8);

    value = of_oxm_icmpv6_code_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_code
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_OF_VERSION_1_2_check(
    of_oxm_icmpv6_code_t *obj, int value)
{

    value = of_oxm_icmpv6_code_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_code_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_code_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_code_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_code_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_masked_OF_VERSION_1_2_populate(
    of_oxm_icmpv6_code_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_code_masked_value_get(obj, &val8);
    of_oxm_icmpv6_code_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_code_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_masked_OF_VERSION_1_2_check(
    of_oxm_icmpv6_code_masked_t *obj, int value)
{

    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_code_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_code_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_code_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_type
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_OF_VERSION_1_2_populate(
    of_oxm_icmpv6_type_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_type_value_get(obj, &val8);

    value = of_oxm_icmpv6_type_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_type
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_OF_VERSION_1_2_check(
    of_oxm_icmpv6_type_t *obj, int value)
{

    value = of_oxm_icmpv6_type_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_type_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_type_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_type_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_type_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_masked_OF_VERSION_1_2_populate(
    of_oxm_icmpv6_type_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_type_masked_value_get(obj, &val8);
    of_oxm_icmpv6_type_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_type_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_masked_OF_VERSION_1_2_check(
    of_oxm_icmpv6_type_masked_t *obj, int value)
{

    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_type_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_type_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_type_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_phy_port
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_OF_VERSION_1_2_populate(
    of_oxm_in_phy_port_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_phy_port_value_get(obj, &port_no);

    value = of_oxm_in_phy_port_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_phy_port
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_OF_VERSION_1_2_check(
    of_oxm_in_phy_port_t *obj, int value)
{

    value = of_oxm_in_phy_port_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_phy_port_t *dup;

        TEST_ASSERT((dup = of_oxm_in_phy_port_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_phy_port_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_phy_port_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_masked_OF_VERSION_1_2_populate(
    of_oxm_in_phy_port_masked_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_phy_port_masked_value_get(obj, &port_no);
    of_oxm_in_phy_port_masked_value_mask_get(obj, &port_no);

    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_phy_port_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_masked_OF_VERSION_1_2_check(
    of_oxm_in_phy_port_masked_t *obj, int value)
{

    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_phy_port_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_in_phy_port_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_phy_port_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_port
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_OF_VERSION_1_2_populate(
    of_oxm_in_port_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_port_value_get(obj, &port_no);

    value = of_oxm_in_port_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_port
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_OF_VERSION_1_2_check(
    of_oxm_in_port_t *obj, int value)
{

    value = of_oxm_in_port_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_port_t *dup;

        TEST_ASSERT((dup = of_oxm_in_port_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_port_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_port_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_masked_OF_VERSION_1_2_populate(
    of_oxm_in_port_masked_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_port_masked_value_get(obj, &port_no);
    of_oxm_in_port_masked_value_mask_get(obj, &port_no);

    value = of_oxm_in_port_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_port_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_masked_OF_VERSION_1_2_check(
    of_oxm_in_port_masked_t *obj, int value)
{

    value = of_oxm_in_port_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_port_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_in_port_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_port_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_dscp
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_OF_VERSION_1_2_populate(
    of_oxm_ip_dscp_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_dscp_value_get(obj, &val8);

    value = of_oxm_ip_dscp_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_dscp
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_OF_VERSION_1_2_check(
    of_oxm_ip_dscp_t *obj, int value)
{

    value = of_oxm_ip_dscp_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_dscp_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_dscp_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_dscp_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_dscp_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_masked_OF_VERSION_1_2_populate(
    of_oxm_ip_dscp_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_dscp_masked_value_get(obj, &val8);
    of_oxm_ip_dscp_masked_value_mask_get(obj, &val8);

    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_dscp_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_masked_OF_VERSION_1_2_check(
    of_oxm_ip_dscp_masked_t *obj, int value)
{

    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_dscp_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_dscp_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_dscp_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_ecn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_OF_VERSION_1_2_populate(
    of_oxm_ip_ecn_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_ecn_value_get(obj, &val8);

    value = of_oxm_ip_ecn_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_ecn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_OF_VERSION_1_2_check(
    of_oxm_ip_ecn_t *obj, int value)
{

    value = of_oxm_ip_ecn_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_ecn_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_ecn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_ecn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_ecn_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_masked_OF_VERSION_1_2_populate(
    of_oxm_ip_ecn_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_ecn_masked_value_get(obj, &val8);
    of_oxm_ip_ecn_masked_value_mask_get(obj, &val8);

    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_ecn_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_masked_OF_VERSION_1_2_check(
    of_oxm_ip_ecn_masked_t *obj, int value)
{

    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_ecn_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_ecn_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_ecn_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_proto
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_OF_VERSION_1_2_populate(
    of_oxm_ip_proto_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_proto_value_get(obj, &val8);

    value = of_oxm_ip_proto_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_proto
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_OF_VERSION_1_2_check(
    of_oxm_ip_proto_t *obj, int value)
{

    value = of_oxm_ip_proto_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_proto_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_proto_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_proto_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_proto_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_masked_OF_VERSION_1_2_populate(
    of_oxm_ip_proto_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_proto_masked_value_get(obj, &val8);
    of_oxm_ip_proto_masked_value_mask_get(obj, &val8);

    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_proto_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_masked_OF_VERSION_1_2_check(
    of_oxm_ip_proto_masked_t *obj, int value)
{

    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_proto_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_proto_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_proto_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_OF_VERSION_1_2_populate(
    of_oxm_ipv4_dst_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_dst_value_get(obj, &ipv4);

    value = of_oxm_ipv4_dst_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_OF_VERSION_1_2_check(
    of_oxm_ipv4_dst_t *obj, int value)
{

    value = of_oxm_ipv4_dst_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv4_dst_masked_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_dst_masked_value_get(obj, &ipv4);
    of_oxm_ipv4_dst_masked_value_mask_get(obj, &ipv4);

    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_masked_OF_VERSION_1_2_check(
    of_oxm_ipv4_dst_masked_t *obj, int value)
{

    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_OF_VERSION_1_2_populate(
    of_oxm_ipv4_src_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_src_value_get(obj, &ipv4);

    value = of_oxm_ipv4_src_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_OF_VERSION_1_2_check(
    of_oxm_ipv4_src_t *obj, int value)
{

    value = of_oxm_ipv4_src_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_src_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv4_src_masked_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_src_masked_value_get(obj, &ipv4);
    of_oxm_ipv4_src_masked_value_mask_get(obj, &ipv4);

    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_masked_OF_VERSION_1_2_check(
    of_oxm_ipv4_src_masked_t *obj, int value)
{

    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_OF_VERSION_1_2_populate(
    of_oxm_ipv6_dst_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_dst_value_get(obj, &ipv6);

    value = of_oxm_ipv6_dst_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_OF_VERSION_1_2_check(
    of_oxm_ipv6_dst_t *obj, int value)
{

    value = of_oxm_ipv6_dst_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv6_dst_masked_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_dst_masked_value_get(obj, &ipv6);
    of_oxm_ipv6_dst_masked_value_mask_get(obj, &ipv6);

    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_masked_OF_VERSION_1_2_check(
    of_oxm_ipv6_dst_masked_t *obj, int value)
{

    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_flabel
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_OF_VERSION_1_2_populate(
    of_oxm_ipv6_flabel_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_flabel_value_get(obj, &val32);

    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_flabel
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_OF_VERSION_1_2_check(
    of_oxm_ipv6_flabel_t *obj, int value)
{

    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_flabel_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_flabel_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_flabel_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_flabel_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv6_flabel_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_flabel_masked_value_get(obj, &val32);
    of_oxm_ipv6_flabel_masked_value_mask_get(obj, &val32);

    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_flabel_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_check(
    of_oxm_ipv6_flabel_masked_t *obj, int value)
{

    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_flabel_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_flabel_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_flabel_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_sll
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_OF_VERSION_1_2_populate(
    of_oxm_ipv6_nd_sll_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_sll_value_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_sll
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_OF_VERSION_1_2_check(
    of_oxm_ipv6_nd_sll_t *obj, int value)
{

    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_sll_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_sll_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_sll_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_sll_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_sll_masked_value_get(obj, &mac_addr);
    of_oxm_ipv6_nd_sll_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_sll_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_check(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value)
{

    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_sll_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_sll_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_sll_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_target
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_OF_VERSION_1_2_populate(
    of_oxm_ipv6_nd_target_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_target_value_get(obj, &ipv6);

    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_target
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_OF_VERSION_1_2_check(
    of_oxm_ipv6_nd_target_t *obj, int value)
{

    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_target_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_target_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_target_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_target_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv6_nd_target_masked_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_target_masked_value_get(obj, &ipv6);
    of_oxm_ipv6_nd_target_masked_value_mask_get(obj, &ipv6);

    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_target_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_check(
    of_oxm_ipv6_nd_target_masked_t *obj, int value)
{

    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_target_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_target_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_target_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_tll
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_OF_VERSION_1_2_populate(
    of_oxm_ipv6_nd_tll_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_tll_value_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_tll
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_OF_VERSION_1_2_check(
    of_oxm_ipv6_nd_tll_t *obj, int value)
{

    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_tll_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_tll_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_tll_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_tll_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_tll_masked_value_get(obj, &mac_addr);
    of_oxm_ipv6_nd_tll_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_tll_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_check(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value)
{

    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_tll_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_tll_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_tll_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_OF_VERSION_1_2_populate(
    of_oxm_ipv6_src_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_src_value_get(obj, &ipv6);

    value = of_oxm_ipv6_src_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_OF_VERSION_1_2_check(
    of_oxm_ipv6_src_t *obj, int value)
{

    value = of_oxm_ipv6_src_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_src_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_masked_OF_VERSION_1_2_populate(
    of_oxm_ipv6_src_masked_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_src_masked_value_get(obj, &ipv6);
    of_oxm_ipv6_src_masked_value_mask_get(obj, &ipv6);

    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_masked_OF_VERSION_1_2_check(
    of_oxm_ipv6_src_masked_t *obj, int value)
{

    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_metadata
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_OF_VERSION_1_2_populate(
    of_oxm_metadata_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_metadata_value_get(obj, &val64);

    value = of_oxm_metadata_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_metadata
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_OF_VERSION_1_2_check(
    of_oxm_metadata_t *obj, int value)
{

    value = of_oxm_metadata_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_metadata_t *dup;

        TEST_ASSERT((dup = of_oxm_metadata_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_metadata_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_metadata_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_masked_OF_VERSION_1_2_populate(
    of_oxm_metadata_masked_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_metadata_masked_value_get(obj, &val64);
    of_oxm_metadata_masked_value_mask_get(obj, &val64);

    value = of_oxm_metadata_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_metadata_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_masked_OF_VERSION_1_2_check(
    of_oxm_metadata_masked_t *obj, int value)
{

    value = of_oxm_metadata_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_metadata_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_metadata_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_metadata_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_bos
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_OF_VERSION_1_2_populate(
    of_oxm_mpls_bos_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_bos_value_get(obj, &val8);

    value = of_oxm_mpls_bos_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_bos
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_OF_VERSION_1_2_check(
    of_oxm_mpls_bos_t *obj, int value)
{

    value = of_oxm_mpls_bos_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_bos_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_bos_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_bos_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_bos_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_masked_OF_VERSION_1_2_populate(
    of_oxm_mpls_bos_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_bos_masked_value_get(obj, &val8);
    of_oxm_mpls_bos_masked_value_mask_get(obj, &val8);

    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_bos_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_masked_OF_VERSION_1_2_check(
    of_oxm_mpls_bos_masked_t *obj, int value)
{

    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_bos_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_bos_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_bos_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_label
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_OF_VERSION_1_2_populate(
    of_oxm_mpls_label_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_label_value_get(obj, &val32);

    value = of_oxm_mpls_label_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_label
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_OF_VERSION_1_2_check(
    of_oxm_mpls_label_t *obj, int value)
{

    value = of_oxm_mpls_label_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_label_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_label_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_label_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_label_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_masked_OF_VERSION_1_2_populate(
    of_oxm_mpls_label_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_label_masked_value_get(obj, &val32);
    of_oxm_mpls_label_masked_value_mask_get(obj, &val32);

    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_label_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_masked_OF_VERSION_1_2_check(
    of_oxm_mpls_label_masked_t *obj, int value)
{

    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_label_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_label_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_label_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_tc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_OF_VERSION_1_2_populate(
    of_oxm_mpls_tc_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_tc_value_get(obj, &val8);

    value = of_oxm_mpls_tc_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_tc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_OF_VERSION_1_2_check(
    of_oxm_mpls_tc_t *obj, int value)
{

    value = of_oxm_mpls_tc_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_tc_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_tc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_tc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_tc_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_masked_OF_VERSION_1_2_populate(
    of_oxm_mpls_tc_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_tc_masked_value_get(obj, &val8);
    of_oxm_mpls_tc_masked_value_mask_get(obj, &val8);

    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_tc_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_masked_OF_VERSION_1_2_check(
    of_oxm_mpls_tc_masked_t *obj, int value)
{

    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_tc_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_tc_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_tc_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_OF_VERSION_1_2_populate(
    of_oxm_sctp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_dst_value_get(obj, &val16);

    value = of_oxm_sctp_dst_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_OF_VERSION_1_2_check(
    of_oxm_sctp_dst_t *obj, int value)
{

    value = of_oxm_sctp_dst_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_masked_OF_VERSION_1_2_populate(
    of_oxm_sctp_dst_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_dst_masked_value_get(obj, &val16);
    of_oxm_sctp_dst_masked_value_mask_get(obj, &val16);

    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_masked_OF_VERSION_1_2_check(
    of_oxm_sctp_dst_masked_t *obj, int value)
{

    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_OF_VERSION_1_2_populate(
    of_oxm_sctp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_src_value_get(obj, &val16);

    value = of_oxm_sctp_src_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_OF_VERSION_1_2_check(
    of_oxm_sctp_src_t *obj, int value)
{

    value = of_oxm_sctp_src_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_src_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_masked_OF_VERSION_1_2_populate(
    of_oxm_sctp_src_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_src_masked_value_get(obj, &val16);
    of_oxm_sctp_src_masked_value_mask_get(obj, &val16);

    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_masked_OF_VERSION_1_2_check(
    of_oxm_sctp_src_masked_t *obj, int value)
{

    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_OF_VERSION_1_2_populate(
    of_oxm_tcp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_dst_value_get(obj, &val16);

    value = of_oxm_tcp_dst_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_OF_VERSION_1_2_check(
    of_oxm_tcp_dst_t *obj, int value)
{

    value = of_oxm_tcp_dst_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_masked_OF_VERSION_1_2_populate(
    of_oxm_tcp_dst_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_dst_masked_value_get(obj, &val16);
    of_oxm_tcp_dst_masked_value_mask_get(obj, &val16);

    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_masked_OF_VERSION_1_2_check(
    of_oxm_tcp_dst_masked_t *obj, int value)
{

    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_OF_VERSION_1_2_populate(
    of_oxm_tcp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_src_value_get(obj, &val16);

    value = of_oxm_tcp_src_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_OF_VERSION_1_2_check(
    of_oxm_tcp_src_t *obj, int value)
{

    value = of_oxm_tcp_src_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_src_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_masked_OF_VERSION_1_2_populate(
    of_oxm_tcp_src_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_src_masked_value_get(obj, &val16);
    of_oxm_tcp_src_masked_value_mask_get(obj, &val16);

    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_masked_OF_VERSION_1_2_check(
    of_oxm_tcp_src_masked_t *obj, int value)
{

    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tunnel_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_OF_VERSION_1_2_populate(
    of_oxm_tunnel_id_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tunnel_id_value_get(obj, &val64);

    value = of_oxm_tunnel_id_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tunnel_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_OF_VERSION_1_2_check(
    of_oxm_tunnel_id_t *obj, int value)
{

    value = of_oxm_tunnel_id_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tunnel_id_t *dup;

        TEST_ASSERT((dup = of_oxm_tunnel_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tunnel_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tunnel_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_masked_OF_VERSION_1_2_populate(
    of_oxm_tunnel_id_masked_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tunnel_id_masked_value_get(obj, &val64);
    of_oxm_tunnel_id_masked_value_mask_get(obj, &val64);

    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tunnel_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_masked_OF_VERSION_1_2_check(
    of_oxm_tunnel_id_masked_t *obj, int value)
{

    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tunnel_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_tunnel_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tunnel_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_OF_VERSION_1_2_populate(
    of_oxm_udp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_dst_value_get(obj, &val16);

    value = of_oxm_udp_dst_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_OF_VERSION_1_2_check(
    of_oxm_udp_dst_t *obj, int value)
{

    value = of_oxm_udp_dst_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_masked_OF_VERSION_1_2_populate(
    of_oxm_udp_dst_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_dst_masked_value_get(obj, &val16);
    of_oxm_udp_dst_masked_value_mask_get(obj, &val16);

    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_masked_OF_VERSION_1_2_check(
    of_oxm_udp_dst_masked_t *obj, int value)
{

    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_OF_VERSION_1_2_populate(
    of_oxm_udp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_src_value_get(obj, &val16);

    value = of_oxm_udp_src_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_OF_VERSION_1_2_check(
    of_oxm_udp_src_t *obj, int value)
{

    value = of_oxm_udp_src_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_src_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_masked_OF_VERSION_1_2_populate(
    of_oxm_udp_src_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_src_masked_value_get(obj, &val16);
    of_oxm_udp_src_masked_value_mask_get(obj, &val16);

    value = of_oxm_udp_src_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_masked_OF_VERSION_1_2_check(
    of_oxm_udp_src_masked_t *obj, int value)
{

    value = of_oxm_udp_src_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_pcp
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_OF_VERSION_1_2_populate(
    of_oxm_vlan_pcp_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_pcp_value_get(obj, &val8);

    value = of_oxm_vlan_pcp_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_pcp
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_OF_VERSION_1_2_check(
    of_oxm_vlan_pcp_t *obj, int value)
{

    value = of_oxm_vlan_pcp_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_pcp_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_pcp_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_pcp_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_pcp_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_masked_OF_VERSION_1_2_populate(
    of_oxm_vlan_pcp_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_pcp_masked_value_get(obj, &val8);
    of_oxm_vlan_pcp_masked_value_mask_get(obj, &val8);

    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_pcp_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_masked_OF_VERSION_1_2_check(
    of_oxm_vlan_pcp_masked_t *obj, int value)
{

    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_pcp_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_pcp_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_pcp_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_vid
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_OF_VERSION_1_2_populate(
    of_oxm_vlan_vid_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_vid_value_get(obj, &val16);

    value = of_oxm_vlan_vid_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_vid
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_OF_VERSION_1_2_check(
    of_oxm_vlan_vid_t *obj, int value)
{

    value = of_oxm_vlan_vid_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_vid_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_vid_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_vid_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_vid_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_masked_OF_VERSION_1_2_populate(
    of_oxm_vlan_vid_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_vid_masked_value_get(obj, &val16);
    of_oxm_vlan_vid_masked_value_mask_get(obj, &val16);

    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_vid_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_masked_OF_VERSION_1_2_check(
    of_oxm_vlan_vid_masked_t *obj, int value)
{

    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_vid_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_vid_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_vid_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_2_populate(
    of_packet_queue_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_queue_prop_t *list;

    /* Run thru accessors after new to ensure okay */
    of_packet_queue_queue_id_get(obj, &val32);
    of_packet_queue_port_get(obj, &port_no);
    {
        of_list_queue_prop_t sub_cls;

        /* Test bind */
        of_packet_queue_properties_bind(obj, &sub_cls);
    }

    value = of_packet_queue_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_prop_new(OF_VERSION_1_2);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_prop_OF_VERSION_1_2_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_queue_properties_set(
        obj, list);
    of_list_queue_prop_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_packet_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_2_check(
    of_packet_queue_t *obj, int value)
{

    value = of_packet_queue_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_prop_t *properties_ptr;

        properties_ptr = of_packet_queue_properties_get(obj);
        TEST_ASSERT(properties_ptr != NULL);
        value = of_list_queue_prop_OF_VERSION_1_2_check(
            properties_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_prop_delete(properties_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_queue_t *dup;

        TEST_ASSERT((dup = of_packet_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_desc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_2_populate(
    of_port_desc_t *obj, int value)
{
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_port_desc_port_no_get(obj, &port_no);
    of_port_desc_hw_addr_get(obj, &mac_addr);
    of_port_desc_name_get(obj, &port_name);
    of_port_desc_config_get(obj, &val32);
    of_port_desc_state_get(obj, &val32);
    of_port_desc_curr_get(obj, &val32);
    of_port_desc_advertised_get(obj, &val32);
    of_port_desc_supported_get(obj, &val32);
    of_port_desc_peer_get(obj, &val32);
    of_port_desc_curr_speed_get(obj, &val32);
    of_port_desc_max_speed_get(obj, &val32);

    value = of_port_desc_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_desc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_2_check(
    of_port_desc_t *obj, int value)
{

    value = of_port_desc_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_desc_t *dup;

        TEST_ASSERT((dup = of_port_desc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_desc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_2_populate(
    of_port_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_entry_port_no_get(obj, &port_no);
    of_port_stats_entry_rx_packets_get(obj, &val64);
    of_port_stats_entry_tx_packets_get(obj, &val64);
    of_port_stats_entry_rx_bytes_get(obj, &val64);
    of_port_stats_entry_tx_bytes_get(obj, &val64);
    of_port_stats_entry_rx_dropped_get(obj, &val64);
    of_port_stats_entry_tx_dropped_get(obj, &val64);
    of_port_stats_entry_rx_errors_get(obj, &val64);
    of_port_stats_entry_tx_errors_get(obj, &val64);
    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    of_port_stats_entry_rx_over_err_get(obj, &val64);
    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    of_port_stats_entry_collisions_get(obj, &val64);

    value = of_port_stats_entry_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_2_check(
    of_port_stats_entry_t *obj, int value)
{

    value = of_port_stats_entry_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_port_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_experimenter_OF_VERSION_1_2_populate(
    of_queue_prop_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_experimenter_experimenter_get(obj, &val32);
    of_queue_prop_experimenter_data_get(obj, &octets);

    value = of_queue_prop_experimenter_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_queue_prop_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_experimenter_OF_VERSION_1_2_check(
    of_queue_prop_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_queue_prop_experimenter_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_queue_prop_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_experimenter_t *dup;

        TEST_ASSERT((dup = of_queue_prop_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_2_populate(
    of_queue_prop_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_queue_prop_header_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_2_check(
    of_queue_prop_header_t *obj, int value)
{

    value = of_queue_prop_header_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_header_t *dup;

        TEST_ASSERT((dup = of_queue_prop_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_max_rate
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_max_rate_OF_VERSION_1_2_populate(
    of_queue_prop_max_rate_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_max_rate_rate_get(obj, &val16);

    value = of_queue_prop_max_rate_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_max_rate
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_max_rate_OF_VERSION_1_2_check(
    of_queue_prop_max_rate_t *obj, int value)
{

    value = of_queue_prop_max_rate_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_max_rate_t *dup;

        TEST_ASSERT((dup = of_queue_prop_max_rate_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_max_rate_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_min_rate
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_2_populate(
    of_queue_prop_min_rate_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_min_rate_rate_get(obj, &val16);

    value = of_queue_prop_min_rate_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_min_rate
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_2_check(
    of_queue_prop_min_rate_t *obj, int value)
{

    value = of_queue_prop_min_rate_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_min_rate_t *dup;

        TEST_ASSERT((dup = of_queue_prop_min_rate_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_min_rate_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_2_populate(
    of_queue_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_entry_port_no_get(obj, &port_no);
    of_queue_stats_entry_queue_id_get(obj, &val32);
    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    of_queue_stats_entry_tx_packets_get(obj, &val64);
    of_queue_stats_entry_tx_errors_get(obj, &val64);

    value = of_queue_stats_entry_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_2_check(
    of_queue_stats_entry_t *obj, int value)
{

    value = of_queue_stats_entry_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_queue_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_2_populate(
    of_table_stats_entry_t *obj, int value)
{
    uint8_t val8;
    of_table_name_t table_name;
    of_match_bmap_t match_bmap;
    of_wc_bmap_t wc_bmap;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_entry_table_id_get(obj, &val8);
    of_table_stats_entry_name_get(obj, &table_name);
    of_table_stats_entry_match_get(obj, &match_bmap);
    of_table_stats_entry_wildcards_get(obj, &wc_bmap);
    of_table_stats_entry_write_actions_get(obj, &val32);
    of_table_stats_entry_apply_actions_get(obj, &val32);
    of_table_stats_entry_write_setfields_get(obj, &val64);
    of_table_stats_entry_apply_setfields_get(obj, &val64);
    of_table_stats_entry_metadata_match_get(obj, &val64);
    of_table_stats_entry_metadata_write_get(obj, &val64);
    of_table_stats_entry_instructions_get(obj, &val32);
    of_table_stats_entry_config_get(obj, &val32);
    of_table_stats_entry_max_entries_get(obj, &val32);
    of_table_stats_entry_active_count_get(obj, &val32);
    of_table_stats_entry_lookup_count_get(obj, &val64);
    of_table_stats_entry_matched_count_get(obj, &val64);

    value = of_table_stats_entry_OF_VERSION_1_2_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_2_check(
    of_table_stats_entry_t *obj, int value)
{

    value = of_table_stats_entry_OF_VERSION_1_2_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_table_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_action with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_2_populate(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_group_t *group;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_push_mpls_t *push_mpls;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    group = &elt.group;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    push_mpls = &elt.push_mpls;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_queue = &elt.set_queue;
    output = &elt.output;

    /* Append two instances of type of_action_copy_ttl_out */

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_2_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_2_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_field */

    of_action_set_field_init(set_field, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_2_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_field_init(set_field, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_2_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_mpls_ttl */

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_group */

    of_action_group_init(group, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_2_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_group_init(group, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_2_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_ttl */

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_vlan */

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_2_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_2_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_mpls */

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_2_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_2_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_vlan */

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_2_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_2_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ttl */

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_in */

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_mpls */

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_2_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_2_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_nw_ttl */

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_queue */

    of_action_set_queue_init(set_queue, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_2_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_queue_init(set_queue, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_2_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_2_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_2, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_2_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_action generated by
 * of_list_action_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_2_check(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_group_t *group;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_push_mpls_t *push_mpls;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_set_queue_t *set_queue;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    group = &elt.group;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    push_mpls = &elt.push_mpls;
    dec_nw_ttl = &elt.dec_nw_ttl;
    set_queue = &elt.set_queue;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_2_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_2_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_2_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_field */
    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_2_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_2_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_2_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_2_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_2_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_2_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_2_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_2_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_2_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_2_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_2_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_2_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_2_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_2_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_2_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_2_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_2_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_2_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_2_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_2_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_2_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_2_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_2_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_ACTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 34);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_action_t *dup;

        TEST_ASSERT((dup = of_list_action_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_action_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_interface with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_2_populate(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * of_list_bsn_interface_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_2_check(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_INTERFACE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_list_bsn_interface_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_interface_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bucket with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_OF_VERSION_1_2_populate(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket */

    of_bucket_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bucket generated by
 * of_list_bucket_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_OF_VERSION_1_2_check(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BUCKET_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bucket_t *dup;

        TEST_ASSERT((dup = of_list_bucket_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bucket_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bucket_counter with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_counter_OF_VERSION_1_2_populate(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket_counter */

    of_bucket_counter_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_counter_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bucket_counter generated by
 * of_list_bucket_counter_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_counter_OF_VERSION_1_2_check(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_counter_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_counter_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_counter_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BUCKET_COUNTER_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bucket_counter_t *dup;

        TEST_ASSERT((dup = of_list_bucket_counter_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bucket_counter_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_flow_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_2_populate(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * of_list_flow_stats_entry_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_2_check(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_FLOW_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_flow_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_flow_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_group_desc_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_group_desc_stats_entry_OF_VERSION_1_2_populate(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_desc_stats_entry */

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_group_desc_stats_entry generated by
 * of_list_group_desc_stats_entry_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_group_desc_stats_entry_OF_VERSION_1_2_check(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_GROUP_DESC_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_group_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_group_desc_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_group_desc_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_group_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_group_stats_entry_OF_VERSION_1_2_populate(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_stats_entry */

    of_group_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_group_stats_entry generated by
 * of_list_group_stats_entry_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_group_stats_entry_OF_VERSION_1_2_check(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_GROUP_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_group_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_group_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_group_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_instruction with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_OF_VERSION_1_2_populate(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;

    /* Append two instances of type of_instruction_clear_actions */

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_2_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_2_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_actions */

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_2_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_2_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_goto_table */

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_2_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_2_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_apply_actions */

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_2_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_2_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_metadata */

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_2_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_2, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_2_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_instruction generated by
 * of_list_instruction_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_OF_VERSION_1_2_check(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_write_actions_t *write_actions;
    of_instruction_goto_table_t *goto_table;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_write_metadata_t *write_metadata;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    write_actions = &elt.write_actions;
    goto_table = &elt.goto_table;
    apply_actions = &elt.apply_actions;
    write_metadata = &elt.write_metadata;
    TEST_OK(of_list_instruction_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_2_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_2_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_2_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_2_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_2_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_2_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_2_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_2_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_2_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_2_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_INSTRUCTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 10);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_instruction_t *dup;

        TEST_ASSERT((dup = of_list_instruction_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_instruction_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_oxm with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_oxm_OF_VERSION_1_2_populate(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    mpls_tc_masked = &elt.mpls_tc_masked;
    arp_sha = &elt.arp_sha;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    in_phy_port_masked = &elt.in_phy_port_masked;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;

    /* Append two instances of type of_oxm_mpls_tc_masked */

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha */

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_2_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_2_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src_masked */

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst */

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_2_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_2_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id */

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id */

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op */

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_2_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_2_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type */

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_2_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_2_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128_masked */

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos */

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa */

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_2_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_2_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type_masked */

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha_masked */

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst */

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_2_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_2_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf_masked */

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target */

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type_masked */

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed_masked */

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6_masked */

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type */

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst_masked */

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst */

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto */

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_2_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_2_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata */

    of_oxm_metadata_init(metadata, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_2_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_init(metadata, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_2_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa_masked */

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel */

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id_masked */

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll_masked */

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa */

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src */

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_2_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_2_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src */

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata_masked */

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll */

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id_masked */

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id */

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid */

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha */

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_2_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_2_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp */

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7_masked */

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos_masked */

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst */

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id */

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst_masked */

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2_masked */

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port_masked */

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn */

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code_masked */

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst_masked */

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src_masked */

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst */

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst_masked */

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1 */

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0 */

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll */

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2 */

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5 */

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4 */

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7 */

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto_masked */

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port_masked */

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp_masked */

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst_masked */

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp */

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src */

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst */

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src_masked */

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op_masked */

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src */

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_2_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_2_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code */

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5_masked */

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code_masked */

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src_masked */

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn_masked */

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src */

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_2_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_2_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid_masked */

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha_masked */

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src_masked */

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port */

    of_oxm_in_port_init(in_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_2_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_init(in_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_2_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id */

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll_masked */

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src */

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_2_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_2_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target_masked */

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type_masked */

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0_masked */

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3_masked */

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label */

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_2_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_2_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128 */

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id_masked */

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel_masked */

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id_masked */

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp_masked */

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type */

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id_masked */

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_tc */

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3 */

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id_masked */

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa_masked */

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst_masked */

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6 */

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code */

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf */

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1_masked */

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4_masked */

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src_masked */

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed */

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port */

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label_masked */

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id */

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_2, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_oxm generated by
 * of_list_oxm_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_oxm_OF_VERSION_1_2_check(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    mpls_tc_masked = &elt.mpls_tc_masked;
    arp_sha = &elt.arp_sha;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    in_phy_port_masked = &elt.in_phy_port_masked;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;
    TEST_OK(of_list_oxm_first(list, &elt));

    /* Check two instances of type mpls_tc_masked */
    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_2_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha */
    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_2_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_2_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src_masked */
    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_2_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst */
    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_2_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_2_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id */
    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_2_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id */
    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_2_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op */
    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_2_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_2_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type */
    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_2_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_2_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128_masked */
    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_2_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos */
    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_2_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa */
    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_2_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_2_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type_masked */
    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_2_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha_masked */
    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_2_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst */
    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_2_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_2_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf_masked */
    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_2_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target */
    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_2_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type_masked */
    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_2_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed_masked */
    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6_masked */
    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_2_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type */
    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_2_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst_masked */
    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_2_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst */
    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_2_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto */
    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_2_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_2_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata */
    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_2_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_2_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa_masked */
    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_2_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel */
    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_2_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id_masked */
    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_2_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll_masked */
    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_2_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa */
    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_2_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src */
    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_2_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_2_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src */
    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_2_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata_masked */
    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_2_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll */
    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_2_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id_masked */
    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_2_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id */
    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_2_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid */
    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_2_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha */
    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_2_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_2_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp */
    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_2_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7_masked */
    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_2_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos_masked */
    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_2_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst */
    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_2_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id */
    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_2_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst_masked */
    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_2_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2_masked */
    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_2_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port_masked */
    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_2_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn */
    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_2_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code_masked */
    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_2_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst_masked */
    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_2_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src_masked */
    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_2_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst */
    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_2_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst_masked */
    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_2_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1 */
    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_2_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0 */
    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_2_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll */
    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_2_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2 */
    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_2_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5 */
    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_2_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4 */
    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_2_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7 */
    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_2_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto_masked */
    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_2_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port_masked */
    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_2_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp_masked */
    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_2_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst_masked */
    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_2_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp */
    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_2_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src */
    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_2_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst */
    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_2_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src_masked */
    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_2_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op_masked */
    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_2_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src */
    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_2_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_2_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code */
    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_2_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5_masked */
    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_2_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code_masked */
    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_2_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src_masked */
    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_2_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn_masked */
    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_2_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src */
    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_2_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_2_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid_masked */
    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_2_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha_masked */
    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_2_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src_masked */
    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_2_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port */
    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_2_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_2_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id */
    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll_masked */
    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_2_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src */
    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_2_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_2_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target_masked */
    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_2_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type_masked */
    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_2_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0_masked */
    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_2_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3_masked */
    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_2_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label */
    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_2_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_2_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128 */
    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_2_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id_masked */
    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel_masked */
    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_2_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id_masked */
    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_2_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp_masked */
    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_2_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type */
    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_2_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id_masked */
    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_tc */
    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_2_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3 */
    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_2_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id_masked */
    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa_masked */
    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_2_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst_masked */
    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_2_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6 */
    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_2_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code */
    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_2_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf */
    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_2_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1_masked */
    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_2_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4_masked */
    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_2_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src_masked */
    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_2_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed */
    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_2_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port */
    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_2_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label_masked */
    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_2_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id */
    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_2_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_oxm_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_OXM_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 216);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_oxm_t *dup;

        TEST_ASSERT((dup = of_list_oxm_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_oxm_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_packet_queue with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_2_populate(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * of_list_packet_queue_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_2_check(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PACKET_QUEUE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_packet_queue_t *dup;

        TEST_ASSERT((dup = of_list_packet_queue_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_packet_queue_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_desc with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_2_populate(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * of_list_port_desc_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_2_check(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_DESC_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_desc_t *dup;

        TEST_ASSERT((dup = of_list_port_desc_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_desc_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_2_populate(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * of_list_port_stats_entry_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_2_check(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_port_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_prop with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_2_populate(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_queue_prop_max_rate */

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_2, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * of_list_queue_prop_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_2_check(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_2_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    /* Check two instances of type max_rate */
    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_2_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_PROP_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 4);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_prop_t *dup;

        TEST_ASSERT((dup = of_list_queue_prop_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_prop_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_2_populate(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * of_list_queue_stats_entry_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_2_check(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_queue_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_table_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_2_populate(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_2, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_2_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * of_list_table_stats_entry_OF_VERSION_1_2_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_2_check(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_2_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_TABLE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_table_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_table_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_3_populate(
    of_aggregate_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_reply_xid_get(obj, &val32);
    of_aggregate_stats_reply_flags_get(obj, &val16);
    of_aggregate_stats_reply_packet_count_get(obj, &val64);
    of_aggregate_stats_reply_byte_count_get(obj, &val64);
    of_aggregate_stats_reply_flow_count_get(obj, &val32);

    value = of_aggregate_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_reply_OF_VERSION_1_3_check(
    of_aggregate_stats_reply_t *obj, int value)
{

    value = of_aggregate_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_reply_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_aggregate_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_3_populate(
    of_aggregate_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_aggregate_stats_request_xid_get(obj, &val32);
    of_aggregate_stats_request_flags_get(obj, &val16);
    of_aggregate_stats_request_table_id_get(obj, &val8);
    of_aggregate_stats_request_out_port_get(obj, &port_no);
    of_aggregate_stats_request_out_group_get(obj, &val32);
    of_aggregate_stats_request_cookie_get(obj, &val64);
    of_aggregate_stats_request_cookie_mask_get(obj, &val64);
    of_aggregate_stats_request_match_get(obj, &match);

    value = of_aggregate_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_aggregate_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_aggregate_stats_request_OF_VERSION_1_3_check(
    of_aggregate_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_aggregate_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_aggregate_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_aggregate_stats_request_t *dup;

        TEST_ASSERT((dup = of_aggregate_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_aggregate_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_async_get_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_async_get_reply_OF_VERSION_1_3_populate(
    of_async_get_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_async_get_reply_xid_get(obj, &val32);
    of_async_get_reply_packet_in_mask_equal_master_get(obj, &val32);
    of_async_get_reply_packet_in_mask_slave_get(obj, &val32);
    of_async_get_reply_port_status_mask_equal_master_get(obj, &val32);
    of_async_get_reply_port_status_mask_slave_get(obj, &val32);
    of_async_get_reply_flow_removed_mask_equal_master_get(obj, &val32);
    of_async_get_reply_flow_removed_mask_slave_get(obj, &val32);

    value = of_async_get_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_async_get_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_async_get_reply_OF_VERSION_1_3_check(
    of_async_get_reply_t *obj, int value)
{

    value = of_async_get_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_async_get_reply_t *dup;

        TEST_ASSERT((dup = of_async_get_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_async_get_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_async_get_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_async_get_request_OF_VERSION_1_3_populate(
    of_async_get_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_async_get_request_xid_get(obj, &val32);
    of_async_get_request_packet_in_mask_equal_master_get(obj, &val32);
    of_async_get_request_packet_in_mask_slave_get(obj, &val32);
    of_async_get_request_port_status_mask_equal_master_get(obj, &val32);
    of_async_get_request_port_status_mask_slave_get(obj, &val32);
    of_async_get_request_flow_removed_mask_equal_master_get(obj, &val32);
    of_async_get_request_flow_removed_mask_slave_get(obj, &val32);

    value = of_async_get_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_async_get_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_async_get_request_OF_VERSION_1_3_check(
    of_async_get_request_t *obj, int value)
{

    value = of_async_get_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_async_get_request_t *dup;

        TEST_ASSERT((dup = of_async_get_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_async_get_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_async_set
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_async_set_OF_VERSION_1_3_populate(
    of_async_set_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_async_set_xid_get(obj, &val32);
    of_async_set_packet_in_mask_equal_master_get(obj, &val32);
    of_async_set_packet_in_mask_slave_get(obj, &val32);
    of_async_set_port_status_mask_equal_master_get(obj, &val32);
    of_async_set_port_status_mask_slave_get(obj, &val32);
    of_async_set_flow_removed_mask_equal_master_get(obj, &val32);
    of_async_set_flow_removed_mask_slave_get(obj, &val32);

    value = of_async_set_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_async_set
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_async_set_OF_VERSION_1_3_check(
    of_async_set_t *obj, int value)
{

    value = of_async_set_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_async_set_t *dup;

        TEST_ASSERT((dup = of_async_set_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_async_set_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_action_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_3_populate(
    of_bad_action_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_action_error_msg_xid_get(obj, &val32);
    of_bad_action_error_msg_code_get(obj, &val16);
    of_bad_action_error_msg_data_get(obj, &octets);

    value = of_bad_action_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_action_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_action_error_msg_OF_VERSION_1_3_check(
    of_bad_action_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_action_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_action_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_action_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_action_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_action_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_instruction_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_instruction_error_msg_OF_VERSION_1_3_populate(
    of_bad_instruction_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_instruction_error_msg_xid_get(obj, &val32);
    of_bad_instruction_error_msg_code_get(obj, &val16);
    of_bad_instruction_error_msg_data_get(obj, &octets);

    value = of_bad_instruction_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_instruction_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_instruction_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_instruction_error_msg_OF_VERSION_1_3_check(
    of_bad_instruction_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_instruction_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_instruction_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_instruction_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_instruction_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_instruction_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_match_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_match_error_msg_OF_VERSION_1_3_populate(
    of_bad_match_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_match_error_msg_xid_get(obj, &val32);
    of_bad_match_error_msg_code_get(obj, &val16);
    of_bad_match_error_msg_data_get(obj, &octets);

    value = of_bad_match_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_match_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_match_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_match_error_msg_OF_VERSION_1_3_check(
    of_bad_match_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_match_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_match_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_match_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_match_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_match_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bad_request_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_3_populate(
    of_bad_request_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bad_request_error_msg_xid_get(obj, &val32);
    of_bad_request_error_msg_code_get(obj, &val16);
    of_bad_request_error_msg_data_get(obj, &octets);

    value = of_bad_request_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bad_request_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bad_request_error_msg_OF_VERSION_1_3_check(
    of_bad_request_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_bad_request_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bad_request_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bad_request_error_msg_t *dup;

        TEST_ASSERT((dup = of_bad_request_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bad_request_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_3_populate(
    of_barrier_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_reply_xid_get(obj, &val32);

    value = of_barrier_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_reply_OF_VERSION_1_3_check(
    of_barrier_reply_t *obj, int value)
{

    value = of_barrier_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_reply_t *dup;

        TEST_ASSERT((dup = of_barrier_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_barrier_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_3_populate(
    of_barrier_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_barrier_request_xid_get(obj, &val32);

    value = of_barrier_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_barrier_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_barrier_request_OF_VERSION_1_3_check(
    of_barrier_request_t *obj, int value)
{

    value = of_barrier_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_barrier_request_t *dup;

        TEST_ASSERT((dup = of_barrier_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_barrier_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_arp_idle
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_arp_idle_OF_VERSION_1_3_populate(
    of_bsn_arp_idle_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_bsn_arp_idle_xid_get(obj, &val32);
    of_bsn_arp_idle_experimenter_get(obj, &val32);
    of_bsn_arp_idle_subtype_get(obj, &val32);
    of_bsn_arp_idle_vlan_vid_get(obj, &val16);
    of_bsn_arp_idle_ipv4_addr_get(obj, &ipv4);

    value = of_bsn_arp_idle_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_arp_idle
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_arp_idle_OF_VERSION_1_3_check(
    of_bsn_arp_idle_t *obj, int value)
{

    value = of_bsn_arp_idle_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_arp_idle_t *dup;

        TEST_ASSERT((dup = of_bsn_arp_idle_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_arp_idle_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_3_populate(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_reply_xid_get(obj, &val32);
    of_bsn_bw_clear_data_reply_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_reply_subtype_get(obj, &val32);
    of_bsn_bw_clear_data_reply_status_get(obj, &val32);

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_reply_OF_VERSION_1_3_check(
    of_bsn_bw_clear_data_reply_t *obj, int value)
{

    value = of_bsn_bw_clear_data_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_clear_data_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_3_populate(
    of_bsn_bw_clear_data_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_clear_data_request_xid_get(obj, &val32);
    of_bsn_bw_clear_data_request_experimenter_get(obj, &val32);
    of_bsn_bw_clear_data_request_subtype_get(obj, &val32);

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_clear_data_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_clear_data_request_OF_VERSION_1_3_check(
    of_bsn_bw_clear_data_request_t *obj, int value)
{

    value = of_bsn_bw_clear_data_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_clear_data_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_clear_data_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_clear_data_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_3_populate(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_get_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_get_reply_enabled_get(obj, &val32);

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_reply_OF_VERSION_1_3_check(
    of_bsn_bw_enable_get_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_get_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_get_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_3_populate(
    of_bsn_bw_enable_get_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_get_request_xid_get(obj, &val32);
    of_bsn_bw_enable_get_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_get_request_subtype_get(obj, &val32);

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_get_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_get_request_OF_VERSION_1_3_check(
    of_bsn_bw_enable_get_request_t *obj, int value)
{

    value = of_bsn_bw_enable_get_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_get_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_get_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_get_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_3_populate(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_reply_xid_get(obj, &val32);
    of_bsn_bw_enable_set_reply_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_reply_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_reply_enable_get(obj, &val32);
    of_bsn_bw_enable_set_reply_status_get(obj, &val32);

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_reply_OF_VERSION_1_3_check(
    of_bsn_bw_enable_set_reply_t *obj, int value)
{

    value = of_bsn_bw_enable_set_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_bw_enable_set_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_3_populate(
    of_bsn_bw_enable_set_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_bw_enable_set_request_xid_get(obj, &val32);
    of_bsn_bw_enable_set_request_experimenter_get(obj, &val32);
    of_bsn_bw_enable_set_request_subtype_get(obj, &val32);
    of_bsn_bw_enable_set_request_enable_get(obj, &val32);

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_bw_enable_set_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_bw_enable_set_request_OF_VERSION_1_3_check(
    of_bsn_bw_enable_set_request_t *obj, int value)
{

    value = of_bsn_bw_enable_set_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_bw_enable_set_request_t *dup;

        TEST_ASSERT((dup = of_bsn_bw_enable_set_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_bw_enable_set_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_controller_connections_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_controller_connections_reply_OF_VERSION_1_3_populate(
    of_bsn_controller_connections_reply_t *obj, int value)
{
    uint32_t val32;
    of_list_bsn_controller_connection_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_controller_connections_reply_xid_get(obj, &val32);
    of_bsn_controller_connections_reply_experimenter_get(obj, &val32);
    of_bsn_controller_connections_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_controller_connection_t sub_cls;

        /* Test bind */
        of_bsn_controller_connections_reply_connections_bind(obj, &sub_cls);
    }

    value = of_bsn_controller_connections_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_controller_connection_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_controller_connection_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_controller_connections_reply_connections_set(
        obj, list);
    of_list_bsn_controller_connection_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_controller_connections_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_controller_connections_reply_OF_VERSION_1_3_check(
    of_bsn_controller_connections_reply_t *obj, int value)
{

    value = of_bsn_controller_connections_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_controller_connection_t *connections_ptr;

        connections_ptr = of_bsn_controller_connections_reply_connections_get(obj);
        TEST_ASSERT(connections_ptr != NULL);
        value = of_list_bsn_controller_connection_OF_VERSION_1_3_check(
            connections_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_controller_connection_delete(connections_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_controller_connections_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_controller_connections_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_controller_connections_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_controller_connections_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_controller_connections_request_OF_VERSION_1_3_populate(
    of_bsn_controller_connections_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_controller_connections_request_xid_get(obj, &val32);
    of_bsn_controller_connections_request_experimenter_get(obj, &val32);
    of_bsn_controller_connections_request_subtype_get(obj, &val32);

    value = of_bsn_controller_connections_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_controller_connections_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_controller_connections_request_OF_VERSION_1_3_check(
    of_bsn_controller_connections_request_t *obj, int value)
{

    value = of_bsn_controller_connections_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_controller_connections_request_t *dup;

        TEST_ASSERT((dup = of_bsn_controller_connections_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_controller_connections_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_debug_counter_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_debug_counter_desc_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_debug_counter_desc_stats_reply_xid_get(obj, &val32);
    of_bsn_debug_counter_desc_stats_reply_flags_get(obj, &val16);
    of_bsn_debug_counter_desc_stats_reply_experimenter_get(obj, &val32);
    of_bsn_debug_counter_desc_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_debug_counter_desc_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_debug_counter_desc_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_debug_counter_desc_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_debug_counter_desc_stats_reply_entries_set(
        obj, list);
    of_list_bsn_debug_counter_desc_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_debug_counter_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_3_check(
    of_bsn_debug_counter_desc_stats_reply_t *obj, int value)
{

    value = of_bsn_debug_counter_desc_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_debug_counter_desc_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_debug_counter_desc_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_debug_counter_desc_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_debug_counter_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_debug_counter_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_debug_counter_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_debug_counter_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_3_populate(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_debug_counter_desc_stats_request_xid_get(obj, &val32);
    of_bsn_debug_counter_desc_stats_request_flags_get(obj, &val16);
    of_bsn_debug_counter_desc_stats_request_experimenter_get(obj, &val32);
    of_bsn_debug_counter_desc_stats_request_subtype_get(obj, &val32);

    value = of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_debug_counter_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_3_check(
    of_bsn_debug_counter_desc_stats_request_t *obj, int value)
{

    value = of_bsn_debug_counter_desc_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_debug_counter_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_debug_counter_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_debug_counter_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_debug_counter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_debug_counter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_debug_counter_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_debug_counter_stats_reply_xid_get(obj, &val32);
    of_bsn_debug_counter_stats_reply_flags_get(obj, &val16);
    of_bsn_debug_counter_stats_reply_experimenter_get(obj, &val32);
    of_bsn_debug_counter_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_debug_counter_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_debug_counter_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_debug_counter_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_debug_counter_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_debug_counter_stats_reply_entries_set(
        obj, list);
    of_list_bsn_debug_counter_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_debug_counter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_stats_reply_OF_VERSION_1_3_check(
    of_bsn_debug_counter_stats_reply_t *obj, int value)
{

    value = of_bsn_debug_counter_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_debug_counter_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_debug_counter_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_debug_counter_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_debug_counter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_debug_counter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_debug_counter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_debug_counter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_stats_request_OF_VERSION_1_3_populate(
    of_bsn_debug_counter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_debug_counter_stats_request_xid_get(obj, &val32);
    of_bsn_debug_counter_stats_request_flags_get(obj, &val16);
    of_bsn_debug_counter_stats_request_experimenter_get(obj, &val32);
    of_bsn_debug_counter_stats_request_subtype_get(obj, &val32);

    value = of_bsn_debug_counter_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_debug_counter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_stats_request_OF_VERSION_1_3_check(
    of_bsn_debug_counter_stats_request_t *obj, int value)
{

    value = of_bsn_debug_counter_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_debug_counter_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_debug_counter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_debug_counter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_checksum_bucket_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_flow_checksum_bucket_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_checksum_bucket_stats_reply_xid_get(obj, &val32);
    of_bsn_flow_checksum_bucket_stats_reply_flags_get(obj, &val16);
    of_bsn_flow_checksum_bucket_stats_reply_experimenter_get(obj, &val32);
    of_bsn_flow_checksum_bucket_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_flow_checksum_bucket_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_flow_checksum_bucket_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_flow_checksum_bucket_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_flow_checksum_bucket_stats_reply_entries_set(
        obj, list);
    of_list_bsn_flow_checksum_bucket_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_checksum_bucket_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_3_check(
    of_bsn_flow_checksum_bucket_stats_reply_t *obj, int value)
{

    value = of_bsn_flow_checksum_bucket_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_flow_checksum_bucket_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_flow_checksum_bucket_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_flow_checksum_bucket_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_checksum_bucket_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_checksum_bucket_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_checksum_bucket_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_checksum_bucket_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_3_populate(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_checksum_bucket_stats_request_xid_get(obj, &val32);
    of_bsn_flow_checksum_bucket_stats_request_flags_get(obj, &val16);
    of_bsn_flow_checksum_bucket_stats_request_experimenter_get(obj, &val32);
    of_bsn_flow_checksum_bucket_stats_request_subtype_get(obj, &val32);
    of_bsn_flow_checksum_bucket_stats_request_table_id_get(obj, &val8);

    value = of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_checksum_bucket_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_3_check(
    of_bsn_flow_checksum_bucket_stats_request_t *obj, int value)
{

    value = of_bsn_flow_checksum_bucket_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_checksum_bucket_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_checksum_bucket_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_checksum_bucket_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_idle
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_OF_VERSION_1_3_populate(
    of_bsn_flow_idle_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_idle_xid_get(obj, &val32);
    of_bsn_flow_idle_experimenter_get(obj, &val32);
    of_bsn_flow_idle_subtype_get(obj, &val32);
    of_bsn_flow_idle_cookie_get(obj, &val64);
    of_bsn_flow_idle_priority_get(obj, &val16);
    of_bsn_flow_idle_table_id_get(obj, &val8);
    of_bsn_flow_idle_match_get(obj, &match);

    value = of_bsn_flow_idle_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_bsn_flow_idle_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_idle
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_OF_VERSION_1_3_check(
    of_bsn_flow_idle_t *obj, int value)
{
    of_match_t match;

    value = of_bsn_flow_idle_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_flow_idle_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_idle_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_idle_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_idle_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_idle_enable_get_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_3_populate(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_idle_enable_get_reply_xid_get(obj, &val32);
    of_bsn_flow_idle_enable_get_reply_experimenter_get(obj, &val32);
    of_bsn_flow_idle_enable_get_reply_subtype_get(obj, &val32);
    of_bsn_flow_idle_enable_get_reply_enabled_get(obj, &val32);

    value = of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_idle_enable_get_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_3_check(
    of_bsn_flow_idle_enable_get_reply_t *obj, int value)
{

    value = of_bsn_flow_idle_enable_get_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_idle_enable_get_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_idle_enable_get_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_idle_enable_get_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_idle_enable_get_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_get_request_OF_VERSION_1_3_populate(
    of_bsn_flow_idle_enable_get_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_idle_enable_get_request_xid_get(obj, &val32);
    of_bsn_flow_idle_enable_get_request_experimenter_get(obj, &val32);
    of_bsn_flow_idle_enable_get_request_subtype_get(obj, &val32);

    value = of_bsn_flow_idle_enable_get_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_idle_enable_get_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_get_request_OF_VERSION_1_3_check(
    of_bsn_flow_idle_enable_get_request_t *obj, int value)
{

    value = of_bsn_flow_idle_enable_get_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_idle_enable_get_request_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_idle_enable_get_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_idle_enable_get_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_idle_enable_set_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_3_populate(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_idle_enable_set_reply_xid_get(obj, &val32);
    of_bsn_flow_idle_enable_set_reply_experimenter_get(obj, &val32);
    of_bsn_flow_idle_enable_set_reply_subtype_get(obj, &val32);
    of_bsn_flow_idle_enable_set_reply_enable_get(obj, &val32);
    of_bsn_flow_idle_enable_set_reply_status_get(obj, &val32);

    value = of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_idle_enable_set_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_3_check(
    of_bsn_flow_idle_enable_set_reply_t *obj, int value)
{

    value = of_bsn_flow_idle_enable_set_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_idle_enable_set_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_idle_enable_set_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_idle_enable_set_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_idle_enable_set_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_set_request_OF_VERSION_1_3_populate(
    of_bsn_flow_idle_enable_set_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_idle_enable_set_request_xid_get(obj, &val32);
    of_bsn_flow_idle_enable_set_request_experimenter_get(obj, &val32);
    of_bsn_flow_idle_enable_set_request_subtype_get(obj, &val32);
    of_bsn_flow_idle_enable_set_request_enable_get(obj, &val32);

    value = of_bsn_flow_idle_enable_set_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_idle_enable_set_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_idle_enable_set_request_OF_VERSION_1_3_check(
    of_bsn_flow_idle_enable_set_request_t *obj, int value)
{

    value = of_bsn_flow_idle_enable_set_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_idle_enable_set_request_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_idle_enable_set_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_idle_enable_set_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_bucket_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_gentable_bucket_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_bucket_stats_reply_xid_get(obj, &val32);
    of_bsn_gentable_bucket_stats_reply_flags_get(obj, &val16);
    of_bsn_gentable_bucket_stats_reply_experimenter_get(obj, &val32);
    of_bsn_gentable_bucket_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_gentable_bucket_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_gentable_bucket_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_gentable_bucket_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_bucket_stats_reply_entries_set(
        obj, list);
    of_list_bsn_gentable_bucket_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_bucket_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_3_check(
    of_bsn_gentable_bucket_stats_reply_t *obj, int value)
{

    value = of_bsn_gentable_bucket_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_gentable_bucket_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_gentable_bucket_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_gentable_bucket_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_bucket_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_bucket_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_bucket_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_bucket_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_bucket_stats_request_OF_VERSION_1_3_populate(
    of_bsn_gentable_bucket_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_bucket_stats_request_xid_get(obj, &val32);
    of_bsn_gentable_bucket_stats_request_flags_get(obj, &val16);
    of_bsn_gentable_bucket_stats_request_experimenter_get(obj, &val32);
    of_bsn_gentable_bucket_stats_request_subtype_get(obj, &val32);
    of_bsn_gentable_bucket_stats_request_table_id_get(obj, &val16);

    value = of_bsn_gentable_bucket_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_bucket_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_bucket_stats_request_OF_VERSION_1_3_check(
    of_bsn_gentable_bucket_stats_request_t *obj, int value)
{

    value = of_bsn_gentable_bucket_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_bucket_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_bucket_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_bucket_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_clear_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_clear_reply_OF_VERSION_1_3_populate(
    of_bsn_gentable_clear_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_clear_reply_xid_get(obj, &val32);
    of_bsn_gentable_clear_reply_experimenter_get(obj, &val32);
    of_bsn_gentable_clear_reply_subtype_get(obj, &val32);
    of_bsn_gentable_clear_reply_table_id_get(obj, &val16);
    of_bsn_gentable_clear_reply_deleted_count_get(obj, &val32);
    of_bsn_gentable_clear_reply_error_count_get(obj, &val32);

    value = of_bsn_gentable_clear_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_clear_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_clear_reply_OF_VERSION_1_3_check(
    of_bsn_gentable_clear_reply_t *obj, int value)
{

    value = of_bsn_gentable_clear_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_clear_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_clear_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_clear_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_clear_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_clear_request_OF_VERSION_1_3_populate(
    of_bsn_gentable_clear_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_clear_request_xid_get(obj, &val32);
    of_bsn_gentable_clear_request_experimenter_get(obj, &val32);
    of_bsn_gentable_clear_request_subtype_get(obj, &val32);
    of_bsn_gentable_clear_request_table_id_get(obj, &val16);
    of_bsn_gentable_clear_request_checksum_get(obj, &checksum_128);
    of_bsn_gentable_clear_request_checksum_mask_get(obj, &checksum_128);

    value = of_bsn_gentable_clear_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_clear_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_clear_request_OF_VERSION_1_3_check(
    of_bsn_gentable_clear_request_t *obj, int value)
{

    value = of_bsn_gentable_clear_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_clear_request_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_clear_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_clear_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_desc_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_gentable_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_gentable_desc_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_desc_stats_reply_xid_get(obj, &val32);
    of_bsn_gentable_desc_stats_reply_flags_get(obj, &val16);
    of_bsn_gentable_desc_stats_reply_experimenter_get(obj, &val32);
    of_bsn_gentable_desc_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_gentable_desc_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_gentable_desc_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_gentable_desc_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_desc_stats_reply_entries_set(
        obj, list);
    of_list_bsn_gentable_desc_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_desc_stats_reply_OF_VERSION_1_3_check(
    of_bsn_gentable_desc_stats_reply_t *obj, int value)
{

    value = of_bsn_gentable_desc_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_gentable_desc_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_gentable_desc_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_gentable_desc_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_desc_stats_request_OF_VERSION_1_3_populate(
    of_bsn_gentable_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_desc_stats_request_xid_get(obj, &val32);
    of_bsn_gentable_desc_stats_request_flags_get(obj, &val16);
    of_bsn_gentable_desc_stats_request_experimenter_get(obj, &val32);
    of_bsn_gentable_desc_stats_request_subtype_get(obj, &val32);

    value = of_bsn_gentable_desc_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_desc_stats_request_OF_VERSION_1_3_check(
    of_bsn_gentable_desc_stats_request_t *obj, int value)
{

    value = of_bsn_gentable_desc_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_add_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_add_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;
    of_list_bsn_tlv_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_entry_add_xid_get(obj, &val32);
    of_bsn_gentable_entry_add_experimenter_get(obj, &val32);
    of_bsn_gentable_entry_add_subtype_get(obj, &val32);
    of_bsn_gentable_entry_add_table_id_get(obj, &val16);
    of_bsn_gentable_entry_add_checksum_get(obj, &checksum_128);
    {
        of_list_bsn_tlv_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_add_key_bind(obj, &sub_cls);
    }
    {
        of_list_bsn_tlv_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_add_value_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_entry_add_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_tlv_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_tlv_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_add_key_set(
        obj, list);
    of_list_bsn_tlv_delete(list);

    list = of_list_bsn_tlv_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_tlv_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_add_value_set(
        obj, list);
    of_list_bsn_tlv_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_add_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_add_t *obj, int value)
{

    value = of_bsn_gentable_entry_add_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_tlv_t *key_ptr;

        key_ptr = of_bsn_gentable_entry_add_key_get(obj);
        TEST_ASSERT(key_ptr != NULL);
        value = of_list_bsn_tlv_OF_VERSION_1_3_check(
            key_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_tlv_delete(key_ptr);
    }

    { /* Use get/delete to access on check */
        of_list_bsn_tlv_t *value_ptr;

        value_ptr = of_bsn_gentable_entry_add_value_get(obj);
        TEST_ASSERT(value_ptr != NULL);
        value = of_list_bsn_tlv_OF_VERSION_1_3_check(
            value_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_tlv_delete(value_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_add_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_delete_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_delete_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_tlv_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_entry_delete_xid_get(obj, &val32);
    of_bsn_gentable_entry_delete_experimenter_get(obj, &val32);
    of_bsn_gentable_entry_delete_subtype_get(obj, &val32);
    of_bsn_gentable_entry_delete_table_id_get(obj, &val16);
    {
        of_list_bsn_tlv_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_delete_key_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_entry_delete_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_tlv_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_tlv_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_delete_key_set(
        obj, list);
    of_list_bsn_tlv_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_delete_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_delete_t *obj, int value)
{

    value = of_bsn_gentable_entry_delete_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_tlv_t *key_ptr;

        key_ptr = of_bsn_gentable_entry_delete_key_get(obj);
        TEST_ASSERT(key_ptr != NULL);
        value = of_list_bsn_tlv_OF_VERSION_1_3_check(
            key_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_tlv_delete(key_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_delete_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_gentable_entry_desc_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_entry_desc_stats_reply_xid_get(obj, &val32);
    of_bsn_gentable_entry_desc_stats_reply_flags_get(obj, &val16);
    of_bsn_gentable_entry_desc_stats_reply_experimenter_get(obj, &val32);
    of_bsn_gentable_entry_desc_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_gentable_entry_desc_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_desc_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_gentable_entry_desc_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_desc_stats_reply_entries_set(
        obj, list);
    of_list_bsn_gentable_entry_desc_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_desc_stats_reply_t *obj, int value)
{

    value = of_bsn_gentable_entry_desc_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_gentable_entry_desc_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_gentable_entry_desc_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_gentable_entry_desc_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_entry_desc_stats_request_xid_get(obj, &val32);
    of_bsn_gentable_entry_desc_stats_request_flags_get(obj, &val16);
    of_bsn_gentable_entry_desc_stats_request_experimenter_get(obj, &val32);
    of_bsn_gentable_entry_desc_stats_request_subtype_get(obj, &val32);
    of_bsn_gentable_entry_desc_stats_request_table_id_get(obj, &val16);
    of_bsn_gentable_entry_desc_stats_request_checksum_get(obj, &checksum_128);
    of_bsn_gentable_entry_desc_stats_request_checksum_mask_get(obj, &checksum_128);

    value = of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_desc_stats_request_t *obj, int value)
{

    value = of_bsn_gentable_entry_desc_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_gentable_entry_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_entry_stats_reply_xid_get(obj, &val32);
    of_bsn_gentable_entry_stats_reply_flags_get(obj, &val16);
    of_bsn_gentable_entry_stats_reply_experimenter_get(obj, &val32);
    of_bsn_gentable_entry_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_gentable_entry_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_entry_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_gentable_entry_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_stats_reply_entries_set(
        obj, list);
    of_list_bsn_gentable_entry_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_stats_reply_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_stats_reply_t *obj, int value)
{

    value = of_bsn_gentable_entry_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_gentable_entry_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_gentable_entry_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_gentable_entry_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_stats_request_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_checksum_128_t checksum_128;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_entry_stats_request_xid_get(obj, &val32);
    of_bsn_gentable_entry_stats_request_flags_get(obj, &val16);
    of_bsn_gentable_entry_stats_request_experimenter_get(obj, &val32);
    of_bsn_gentable_entry_stats_request_subtype_get(obj, &val32);
    of_bsn_gentable_entry_stats_request_table_id_get(obj, &val16);
    of_bsn_gentable_entry_stats_request_checksum_get(obj, &checksum_128);
    of_bsn_gentable_entry_stats_request_checksum_mask_get(obj, &checksum_128);

    value = of_bsn_gentable_entry_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_stats_request_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_stats_request_t *obj, int value)
{

    value = of_bsn_gentable_entry_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_set_buckets_size
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_set_buckets_size_OF_VERSION_1_3_populate(
    of_bsn_gentable_set_buckets_size_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_set_buckets_size_xid_get(obj, &val32);
    of_bsn_gentable_set_buckets_size_experimenter_get(obj, &val32);
    of_bsn_gentable_set_buckets_size_subtype_get(obj, &val32);
    of_bsn_gentable_set_buckets_size_table_id_get(obj, &val16);
    of_bsn_gentable_set_buckets_size_buckets_size_get(obj, &val32);

    value = of_bsn_gentable_set_buckets_size_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_set_buckets_size
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_set_buckets_size_OF_VERSION_1_3_check(
    of_bsn_gentable_set_buckets_size_t *obj, int value)
{

    value = of_bsn_gentable_set_buckets_size_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_set_buckets_size_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_set_buckets_size_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_set_buckets_size_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_gentable_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_gentable_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_stats_reply_xid_get(obj, &val32);
    of_bsn_gentable_stats_reply_flags_get(obj, &val16);
    of_bsn_gentable_stats_reply_experimenter_get(obj, &val32);
    of_bsn_gentable_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_gentable_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_gentable_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_gentable_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_gentable_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_stats_reply_entries_set(
        obj, list);
    of_list_bsn_gentable_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_stats_reply_OF_VERSION_1_3_check(
    of_bsn_gentable_stats_reply_t *obj, int value)
{

    value = of_bsn_gentable_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_gentable_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_gentable_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_gentable_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_gentable_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_stats_request_OF_VERSION_1_3_populate(
    of_bsn_gentable_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_stats_request_xid_get(obj, &val32);
    of_bsn_gentable_stats_request_flags_get(obj, &val16);
    of_bsn_gentable_stats_request_experimenter_get(obj, &val32);
    of_bsn_gentable_stats_request_subtype_get(obj, &val32);

    value = of_bsn_gentable_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_stats_request_OF_VERSION_1_3_check(
    of_bsn_gentable_stats_request_t *obj, int value)
{

    value = of_bsn_gentable_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_3_populate(
    of_bsn_get_interfaces_reply_t *obj, int value)
{
    uint32_t val32;
    of_list_bsn_interface_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_reply_xid_get(obj, &val32);
    of_bsn_get_interfaces_reply_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_interface_t sub_cls;

        /* Test bind */
        of_bsn_get_interfaces_reply_interfaces_bind(obj, &sub_cls);
    }

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_interface_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_interface_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_get_interfaces_reply_interfaces_set(
        obj, list);
    of_list_bsn_interface_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_reply_OF_VERSION_1_3_check(
    of_bsn_get_interfaces_reply_t *obj, int value)
{

    value = of_bsn_get_interfaces_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_interface_t *interfaces_ptr;

        interfaces_ptr = of_bsn_get_interfaces_reply_interfaces_get(obj);
        TEST_ASSERT(interfaces_ptr != NULL);
        value = of_list_bsn_interface_OF_VERSION_1_3_check(
            interfaces_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_interface_delete(interfaces_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_interfaces_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_3_populate(
    of_bsn_get_interfaces_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_interfaces_request_xid_get(obj, &val32);
    of_bsn_get_interfaces_request_experimenter_get(obj, &val32);
    of_bsn_get_interfaces_request_subtype_get(obj, &val32);

    value = of_bsn_get_interfaces_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_interfaces_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_interfaces_request_OF_VERSION_1_3_check(
    of_bsn_get_interfaces_request_t *obj, int value)
{

    value = of_bsn_get_interfaces_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_interfaces_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_interfaces_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_interfaces_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_3_populate(
    of_bsn_get_mirroring_reply_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_reply_xid_get(obj, &val32);
    of_bsn_get_mirroring_reply_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_reply_subtype_get(obj, &val32);
    of_bsn_get_mirroring_reply_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_reply_OF_VERSION_1_3_check(
    of_bsn_get_mirroring_reply_t *obj, int value)
{

    value = of_bsn_get_mirroring_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_mirroring_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_3_populate(
    of_bsn_get_mirroring_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_mirroring_request_xid_get(obj, &val32);
    of_bsn_get_mirroring_request_experimenter_get(obj, &val32);
    of_bsn_get_mirroring_request_subtype_get(obj, &val32);
    of_bsn_get_mirroring_request_report_mirror_ports_get(obj, &val8);

    value = of_bsn_get_mirroring_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_mirroring_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_mirroring_request_OF_VERSION_1_3_check(
    of_bsn_get_mirroring_request_t *obj, int value)
{

    value = of_bsn_get_mirroring_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_mirroring_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_mirroring_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_mirroring_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_switch_pipeline_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_switch_pipeline_reply_OF_VERSION_1_3_populate(
    of_bsn_get_switch_pipeline_reply_t *obj, int value)
{
    uint32_t val32;
    of_desc_str_t desc_str;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_switch_pipeline_reply_xid_get(obj, &val32);
    of_bsn_get_switch_pipeline_reply_experimenter_get(obj, &val32);
    of_bsn_get_switch_pipeline_reply_subtype_get(obj, &val32);
    of_bsn_get_switch_pipeline_reply_pipeline_get(obj, &desc_str);

    value = of_bsn_get_switch_pipeline_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_switch_pipeline_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_switch_pipeline_reply_OF_VERSION_1_3_check(
    of_bsn_get_switch_pipeline_reply_t *obj, int value)
{

    value = of_bsn_get_switch_pipeline_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_switch_pipeline_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_get_switch_pipeline_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_switch_pipeline_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_get_switch_pipeline_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_switch_pipeline_request_OF_VERSION_1_3_populate(
    of_bsn_get_switch_pipeline_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_get_switch_pipeline_request_xid_get(obj, &val32);
    of_bsn_get_switch_pipeline_request_experimenter_get(obj, &val32);
    of_bsn_get_switch_pipeline_request_subtype_get(obj, &val32);

    value = of_bsn_get_switch_pipeline_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_get_switch_pipeline_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_get_switch_pipeline_request_OF_VERSION_1_3_check(
    of_bsn_get_switch_pipeline_request_t *obj, int value)
{

    value = of_bsn_get_switch_pipeline_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_get_switch_pipeline_request_t *dup;

        TEST_ASSERT((dup = of_bsn_get_switch_pipeline_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_get_switch_pipeline_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_3_populate(
    of_bsn_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_header_xid_get(obj, &val32);
    of_bsn_header_experimenter_get(obj, &val32);
    of_bsn_header_subtype_get(obj, &val32);

    value = of_bsn_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_header_OF_VERSION_1_3_check(
    of_bsn_header_t *obj, int value)
{

    value = of_bsn_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_header_t *dup;

        TEST_ASSERT((dup = of_bsn_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_lacp_convergence_notif
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_convergence_notif_OF_VERSION_1_3_populate(
    of_bsn_lacp_convergence_notif_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_bsn_lacp_convergence_notif_xid_get(obj, &val32);
    of_bsn_lacp_convergence_notif_experimenter_get(obj, &val32);
    of_bsn_lacp_convergence_notif_subtype_get(obj, &val32);
    of_bsn_lacp_convergence_notif_convergence_status_get(obj, &val8);
    of_bsn_lacp_convergence_notif_port_no_get(obj, &port_no);
    of_bsn_lacp_convergence_notif_actor_sys_priority_get(obj, &val16);
    of_bsn_lacp_convergence_notif_actor_sys_mac_get(obj, &mac_addr);
    of_bsn_lacp_convergence_notif_actor_port_priority_get(obj, &val16);
    of_bsn_lacp_convergence_notif_actor_port_num_get(obj, &val16);
    of_bsn_lacp_convergence_notif_actor_key_get(obj, &val16);
    of_bsn_lacp_convergence_notif_partner_sys_priority_get(obj, &val16);
    of_bsn_lacp_convergence_notif_partner_sys_mac_get(obj, &mac_addr);
    of_bsn_lacp_convergence_notif_partner_port_priority_get(obj, &val16);
    of_bsn_lacp_convergence_notif_partner_port_num_get(obj, &val16);
    of_bsn_lacp_convergence_notif_partner_key_get(obj, &val16);

    value = of_bsn_lacp_convergence_notif_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_lacp_convergence_notif
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_convergence_notif_OF_VERSION_1_3_check(
    of_bsn_lacp_convergence_notif_t *obj, int value)
{

    value = of_bsn_lacp_convergence_notif_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_lacp_convergence_notif_t *dup;

        TEST_ASSERT((dup = of_bsn_lacp_convergence_notif_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_lacp_convergence_notif_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_lacp_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_lacp_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_lacp_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_lacp_stats_reply_xid_get(obj, &val32);
    of_bsn_lacp_stats_reply_flags_get(obj, &val16);
    of_bsn_lacp_stats_reply_experimenter_get(obj, &val32);
    of_bsn_lacp_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_lacp_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_lacp_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_lacp_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_lacp_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_lacp_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_lacp_stats_reply_entries_set(
        obj, list);
    of_list_bsn_lacp_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_lacp_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_stats_reply_OF_VERSION_1_3_check(
    of_bsn_lacp_stats_reply_t *obj, int value)
{

    value = of_bsn_lacp_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_lacp_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_lacp_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_lacp_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_lacp_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_lacp_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_lacp_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_lacp_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_lacp_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_stats_request_OF_VERSION_1_3_populate(
    of_bsn_lacp_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_lacp_stats_request_xid_get(obj, &val32);
    of_bsn_lacp_stats_request_flags_get(obj, &val16);
    of_bsn_lacp_stats_request_experimenter_get(obj, &val32);
    of_bsn_lacp_stats_request_subtype_get(obj, &val32);

    value = of_bsn_lacp_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_lacp_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_stats_request_OF_VERSION_1_3_check(
    of_bsn_lacp_stats_request_t *obj, int value)
{

    value = of_bsn_lacp_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_lacp_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_lacp_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_lacp_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_3_populate(
    of_bsn_pdu_rx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_reply_xid_get(obj, &val32);
    of_bsn_pdu_rx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_rx_reply_status_get(obj, &val32);
    of_bsn_pdu_rx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_reply_OF_VERSION_1_3_check(
    of_bsn_pdu_rx_reply_t *obj, int value)
{

    value = of_bsn_pdu_rx_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_3_populate(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_request_xid_get(obj, &val32);
    of_bsn_pdu_rx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_request_subtype_get(obj, &val32);
    of_bsn_pdu_rx_request_timeout_ms_get(obj, &val32);
    of_bsn_pdu_rx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_rx_request_data_get(obj, &octets);

    value = of_bsn_pdu_rx_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_request_OF_VERSION_1_3_check(
    of_bsn_pdu_rx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_rx_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_rx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_rx_timeout
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_3_populate(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_rx_timeout_xid_get(obj, &val32);
    of_bsn_pdu_rx_timeout_experimenter_get(obj, &val32);
    of_bsn_pdu_rx_timeout_subtype_get(obj, &val32);
    of_bsn_pdu_rx_timeout_port_no_get(obj, &port_no);
    of_bsn_pdu_rx_timeout_slot_num_get(obj, &val8);

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_rx_timeout
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_rx_timeout_OF_VERSION_1_3_check(
    of_bsn_pdu_rx_timeout_t *obj, int value)
{

    value = of_bsn_pdu_rx_timeout_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_rx_timeout_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_rx_timeout_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_rx_timeout_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_3_populate(
    of_bsn_pdu_tx_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_reply_xid_get(obj, &val32);
    of_bsn_pdu_tx_reply_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_reply_subtype_get(obj, &val32);
    of_bsn_pdu_tx_reply_status_get(obj, &val32);
    of_bsn_pdu_tx_reply_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_reply_slot_num_get(obj, &val8);

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_reply_OF_VERSION_1_3_check(
    of_bsn_pdu_tx_reply_t *obj, int value)
{

    value = of_bsn_pdu_tx_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_pdu_tx_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_3_populate(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    uint8_t val8;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_pdu_tx_request_xid_get(obj, &val32);
    of_bsn_pdu_tx_request_experimenter_get(obj, &val32);
    of_bsn_pdu_tx_request_subtype_get(obj, &val32);
    of_bsn_pdu_tx_request_tx_interval_ms_get(obj, &val32);
    of_bsn_pdu_tx_request_port_no_get(obj, &port_no);
    of_bsn_pdu_tx_request_slot_num_get(obj, &val8);
    of_bsn_pdu_tx_request_data_get(obj, &octets);

    value = of_bsn_pdu_tx_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_pdu_tx_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_pdu_tx_request_OF_VERSION_1_3_check(
    of_bsn_pdu_tx_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_pdu_tx_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_pdu_tx_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_pdu_tx_request_t *dup;

        TEST_ASSERT((dup = of_bsn_pdu_tx_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_pdu_tx_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_port_counter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_port_counter_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_port_counter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_port_counter_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_port_counter_stats_reply_xid_get(obj, &val32);
    of_bsn_port_counter_stats_reply_flags_get(obj, &val16);
    of_bsn_port_counter_stats_reply_experimenter_get(obj, &val32);
    of_bsn_port_counter_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_port_counter_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_port_counter_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_port_counter_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_port_counter_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_port_counter_stats_reply_entries_set(
        obj, list);
    of_list_bsn_port_counter_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_port_counter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_port_counter_stats_reply_OF_VERSION_1_3_check(
    of_bsn_port_counter_stats_reply_t *obj, int value)
{

    value = of_bsn_port_counter_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_port_counter_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_port_counter_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_port_counter_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_port_counter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_port_counter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_port_counter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_port_counter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_port_counter_stats_request_OF_VERSION_1_3_populate(
    of_bsn_port_counter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_bsn_port_counter_stats_request_xid_get(obj, &val32);
    of_bsn_port_counter_stats_request_flags_get(obj, &val16);
    of_bsn_port_counter_stats_request_experimenter_get(obj, &val32);
    of_bsn_port_counter_stats_request_subtype_get(obj, &val32);
    of_bsn_port_counter_stats_request_port_no_get(obj, &port_no);

    value = of_bsn_port_counter_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_port_counter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_port_counter_stats_request_OF_VERSION_1_3_check(
    of_bsn_port_counter_stats_request_t *obj, int value)
{

    value = of_bsn_port_counter_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_port_counter_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_port_counter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_port_counter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_role_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_role_status_OF_VERSION_1_3_populate(
    of_bsn_role_status_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_role_status_xid_get(obj, &val32);
    of_bsn_role_status_experimenter_get(obj, &val32);
    of_bsn_role_status_subtype_get(obj, &val32);
    of_bsn_role_status_role_get(obj, &val32);
    of_bsn_role_status_reason_get(obj, &val8);
    of_bsn_role_status_generation_id_get(obj, &val64);

    value = of_bsn_role_status_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_role_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_role_status_OF_VERSION_1_3_check(
    of_bsn_role_status_t *obj, int value)
{

    value = of_bsn_role_status_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_role_status_t *dup;

        TEST_ASSERT((dup = of_bsn_role_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_role_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_aux_cxns_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_aux_cxns_reply_OF_VERSION_1_3_populate(
    of_bsn_set_aux_cxns_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_aux_cxns_reply_xid_get(obj, &val32);
    of_bsn_set_aux_cxns_reply_experimenter_get(obj, &val32);
    of_bsn_set_aux_cxns_reply_subtype_get(obj, &val32);
    of_bsn_set_aux_cxns_reply_num_aux_get(obj, &val32);
    of_bsn_set_aux_cxns_reply_status_get(obj, &val32);

    value = of_bsn_set_aux_cxns_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_aux_cxns_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_aux_cxns_reply_OF_VERSION_1_3_check(
    of_bsn_set_aux_cxns_reply_t *obj, int value)
{

    value = of_bsn_set_aux_cxns_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_aux_cxns_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_aux_cxns_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_aux_cxns_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_aux_cxns_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_aux_cxns_request_OF_VERSION_1_3_populate(
    of_bsn_set_aux_cxns_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_aux_cxns_request_xid_get(obj, &val32);
    of_bsn_set_aux_cxns_request_experimenter_get(obj, &val32);
    of_bsn_set_aux_cxns_request_subtype_get(obj, &val32);
    of_bsn_set_aux_cxns_request_num_aux_get(obj, &val32);

    value = of_bsn_set_aux_cxns_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_aux_cxns_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_aux_cxns_request_OF_VERSION_1_3_check(
    of_bsn_set_aux_cxns_request_t *obj, int value)
{

    value = of_bsn_set_aux_cxns_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_aux_cxns_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_aux_cxns_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_aux_cxns_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_lacp_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_lacp_reply_OF_VERSION_1_3_populate(
    of_bsn_set_lacp_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_lacp_reply_xid_get(obj, &val32);
    of_bsn_set_lacp_reply_experimenter_get(obj, &val32);
    of_bsn_set_lacp_reply_subtype_get(obj, &val32);
    of_bsn_set_lacp_reply_status_get(obj, &val32);
    of_bsn_set_lacp_reply_port_no_get(obj, &port_no);

    value = of_bsn_set_lacp_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_lacp_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_lacp_reply_OF_VERSION_1_3_check(
    of_bsn_set_lacp_reply_t *obj, int value)
{

    value = of_bsn_set_lacp_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_lacp_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_lacp_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_lacp_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_lacp_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_lacp_request_OF_VERSION_1_3_populate(
    of_bsn_set_lacp_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_lacp_request_xid_get(obj, &val32);
    of_bsn_set_lacp_request_experimenter_get(obj, &val32);
    of_bsn_set_lacp_request_subtype_get(obj, &val32);
    of_bsn_set_lacp_request_enabled_get(obj, &val8);
    of_bsn_set_lacp_request_port_no_get(obj, &port_no);
    of_bsn_set_lacp_request_actor_sys_priority_get(obj, &val16);
    of_bsn_set_lacp_request_actor_sys_mac_get(obj, &mac_addr);
    of_bsn_set_lacp_request_actor_port_priority_get(obj, &val16);
    of_bsn_set_lacp_request_actor_port_num_get(obj, &val16);
    of_bsn_set_lacp_request_actor_key_get(obj, &val16);

    value = of_bsn_set_lacp_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_lacp_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_lacp_request_OF_VERSION_1_3_check(
    of_bsn_set_lacp_request_t *obj, int value)
{

    value = of_bsn_set_lacp_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_lacp_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_lacp_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_lacp_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_mirroring
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_3_populate(
    of_bsn_set_mirroring_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_mirroring_xid_get(obj, &val32);
    of_bsn_set_mirroring_experimenter_get(obj, &val32);
    of_bsn_set_mirroring_subtype_get(obj, &val32);
    of_bsn_set_mirroring_report_mirror_ports_get(obj, &val8);

    value = of_bsn_set_mirroring_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_mirroring
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_mirroring_OF_VERSION_1_3_check(
    of_bsn_set_mirroring_t *obj, int value)
{

    value = of_bsn_set_mirroring_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_mirroring_t *dup;

        TEST_ASSERT((dup = of_bsn_set_mirroring_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_mirroring_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_3_populate(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_reply_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_reply_status_get(obj, &val32);

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_reply_OF_VERSION_1_3_check(
    of_bsn_set_pktin_suppression_reply_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_pktin_suppression_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_3_populate(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_pktin_suppression_request_xid_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_experimenter_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_subtype_get(obj, &val32);
    of_bsn_set_pktin_suppression_request_enabled_get(obj, &val8);
    of_bsn_set_pktin_suppression_request_idle_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_hard_timeout_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_priority_get(obj, &val16);
    of_bsn_set_pktin_suppression_request_cookie_get(obj, &val64);

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_pktin_suppression_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_pktin_suppression_request_OF_VERSION_1_3_check(
    of_bsn_set_pktin_suppression_request_t *obj, int value)
{

    value = of_bsn_set_pktin_suppression_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_pktin_suppression_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_pktin_suppression_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_pktin_suppression_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_switch_pipeline_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_switch_pipeline_reply_OF_VERSION_1_3_populate(
    of_bsn_set_switch_pipeline_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_switch_pipeline_reply_xid_get(obj, &val32);
    of_bsn_set_switch_pipeline_reply_experimenter_get(obj, &val32);
    of_bsn_set_switch_pipeline_reply_subtype_get(obj, &val32);
    of_bsn_set_switch_pipeline_reply_status_get(obj, &val32);

    value = of_bsn_set_switch_pipeline_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_switch_pipeline_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_switch_pipeline_reply_OF_VERSION_1_3_check(
    of_bsn_set_switch_pipeline_reply_t *obj, int value)
{

    value = of_bsn_set_switch_pipeline_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_switch_pipeline_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_set_switch_pipeline_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_switch_pipeline_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_set_switch_pipeline_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_switch_pipeline_request_OF_VERSION_1_3_populate(
    of_bsn_set_switch_pipeline_request_t *obj, int value)
{
    uint32_t val32;
    of_desc_str_t desc_str;

    /* Run thru accessors after new to ensure okay */
    of_bsn_set_switch_pipeline_request_xid_get(obj, &val32);
    of_bsn_set_switch_pipeline_request_experimenter_get(obj, &val32);
    of_bsn_set_switch_pipeline_request_subtype_get(obj, &val32);
    of_bsn_set_switch_pipeline_request_pipeline_get(obj, &desc_str);

    value = of_bsn_set_switch_pipeline_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_set_switch_pipeline_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_set_switch_pipeline_request_OF_VERSION_1_3_check(
    of_bsn_set_switch_pipeline_request_t *obj, int value)
{

    value = of_bsn_set_switch_pipeline_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_set_switch_pipeline_request_t *dup;

        TEST_ASSERT((dup = of_bsn_set_switch_pipeline_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_set_switch_pipeline_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_reply_xid_get(obj, &val32);
    of_bsn_stats_reply_flags_get(obj, &val16);
    of_bsn_stats_reply_experimenter_get(obj, &val32);
    of_bsn_stats_reply_subtype_get(obj, &val32);

    value = of_bsn_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_reply_OF_VERSION_1_3_check(
    of_bsn_stats_reply_t *obj, int value)
{

    value = of_bsn_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_3_populate(
    of_bsn_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_stats_request_xid_get(obj, &val32);
    of_bsn_stats_request_flags_get(obj, &val16);
    of_bsn_stats_request_experimenter_get(obj, &val32);
    of_bsn_stats_request_subtype_get(obj, &val32);

    value = of_bsn_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_stats_request_OF_VERSION_1_3_check(
    of_bsn_stats_request_t *obj, int value)
{

    value = of_bsn_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_switch_pipeline_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_switch_pipeline_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_switch_pipeline_stats_reply_xid_get(obj, &val32);
    of_bsn_switch_pipeline_stats_reply_flags_get(obj, &val16);
    of_bsn_switch_pipeline_stats_reply_experimenter_get(obj, &val32);
    of_bsn_switch_pipeline_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_switch_pipeline_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_switch_pipeline_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_switch_pipeline_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_switch_pipeline_stats_reply_entries_set(
        obj, list);
    of_list_bsn_switch_pipeline_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_switch_pipeline_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_3_check(
    of_bsn_switch_pipeline_stats_reply_t *obj, int value)
{

    value = of_bsn_switch_pipeline_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_switch_pipeline_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_switch_pipeline_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_switch_pipeline_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_switch_pipeline_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_switch_pipeline_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_switch_pipeline_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_switch_pipeline_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_switch_pipeline_stats_request_OF_VERSION_1_3_populate(
    of_bsn_switch_pipeline_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_switch_pipeline_stats_request_xid_get(obj, &val32);
    of_bsn_switch_pipeline_stats_request_flags_get(obj, &val16);
    of_bsn_switch_pipeline_stats_request_experimenter_get(obj, &val32);
    of_bsn_switch_pipeline_stats_request_subtype_get(obj, &val32);

    value = of_bsn_switch_pipeline_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_switch_pipeline_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_switch_pipeline_stats_request_OF_VERSION_1_3_check(
    of_bsn_switch_pipeline_stats_request_t *obj, int value)
{

    value = of_bsn_switch_pipeline_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_switch_pipeline_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_switch_pipeline_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_switch_pipeline_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_table_checksum_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_checksum_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_table_checksum_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_table_checksum_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_table_checksum_stats_reply_xid_get(obj, &val32);
    of_bsn_table_checksum_stats_reply_flags_get(obj, &val16);
    of_bsn_table_checksum_stats_reply_experimenter_get(obj, &val32);
    of_bsn_table_checksum_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_table_checksum_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_table_checksum_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_table_checksum_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_table_checksum_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_table_checksum_stats_reply_entries_set(
        obj, list);
    of_list_bsn_table_checksum_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_table_checksum_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_checksum_stats_reply_OF_VERSION_1_3_check(
    of_bsn_table_checksum_stats_reply_t *obj, int value)
{

    value = of_bsn_table_checksum_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_table_checksum_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_table_checksum_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_table_checksum_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_table_checksum_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_table_checksum_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_table_checksum_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_table_checksum_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_checksum_stats_request_OF_VERSION_1_3_populate(
    of_bsn_table_checksum_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_table_checksum_stats_request_xid_get(obj, &val32);
    of_bsn_table_checksum_stats_request_flags_get(obj, &val16);
    of_bsn_table_checksum_stats_request_experimenter_get(obj, &val32);
    of_bsn_table_checksum_stats_request_subtype_get(obj, &val32);

    value = of_bsn_table_checksum_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_table_checksum_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_checksum_stats_request_OF_VERSION_1_3_check(
    of_bsn_table_checksum_stats_request_t *obj, int value)
{

    value = of_bsn_table_checksum_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_table_checksum_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_table_checksum_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_table_checksum_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_table_set_buckets_size
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_set_buckets_size_OF_VERSION_1_3_populate(
    of_bsn_table_set_buckets_size_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_table_set_buckets_size_xid_get(obj, &val32);
    of_bsn_table_set_buckets_size_experimenter_get(obj, &val32);
    of_bsn_table_set_buckets_size_subtype_get(obj, &val32);
    of_bsn_table_set_buckets_size_table_id_get(obj, &val16);
    of_bsn_table_set_buckets_size_buckets_size_get(obj, &val32);

    value = of_bsn_table_set_buckets_size_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_table_set_buckets_size
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_set_buckets_size_OF_VERSION_1_3_check(
    of_bsn_table_set_buckets_size_t *obj, int value)
{

    value = of_bsn_table_set_buckets_size_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_table_set_buckets_size_t *dup;

        TEST_ASSERT((dup = of_bsn_table_set_buckets_size_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_table_set_buckets_size_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_time_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_time_reply_OF_VERSION_1_3_populate(
    of_bsn_time_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_time_reply_xid_get(obj, &val32);
    of_bsn_time_reply_experimenter_get(obj, &val32);
    of_bsn_time_reply_subtype_get(obj, &val32);
    of_bsn_time_reply_time_ms_get(obj, &val64);

    value = of_bsn_time_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_time_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_time_reply_OF_VERSION_1_3_check(
    of_bsn_time_reply_t *obj, int value)
{

    value = of_bsn_time_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_time_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_time_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_time_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_time_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_time_request_OF_VERSION_1_3_populate(
    of_bsn_time_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_time_request_xid_get(obj, &val32);
    of_bsn_time_request_experimenter_get(obj, &val32);
    of_bsn_time_request_subtype_get(obj, &val32);

    value = of_bsn_time_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_time_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_time_request_OF_VERSION_1_3_check(
    of_bsn_time_request_t *obj, int value)
{

    value = of_bsn_time_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_time_request_t *dup;

        TEST_ASSERT((dup = of_bsn_time_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_time_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_3_populate(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_create_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_create_reply_status_get(obj, &val32);
    of_bsn_virtual_port_create_reply_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_reply_OF_VERSION_1_3_check(
    of_bsn_virtual_port_create_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_create_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_create_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_3_populate(
    of_bsn_virtual_port_create_request_t *obj, int value)
{
    uint32_t val32;
    of_bsn_vport_q_in_q_t *vport;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_create_request_xid_get(obj, &val32);
    of_bsn_virtual_port_create_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_create_request_subtype_get(obj, &val32);
    {
        of_bsn_vport_q_in_q_t sub_cls;

        /* Test bind */
        of_bsn_virtual_port_create_request_vport_bind(obj, &sub_cls);
    }

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    vport = of_bsn_vport_q_in_q_new(OF_VERSION_1_3);
    TEST_ASSERT(vport != NULL);
    value = of_bsn_vport_q_in_q_OF_VERSION_1_3_populate(
        vport, value);
    TEST_ASSERT(value != 0);
    of_bsn_virtual_port_create_request_vport_set(
        obj, vport);
    of_bsn_vport_q_in_q_delete(vport);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_create_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_create_request_OF_VERSION_1_3_check(
    of_bsn_virtual_port_create_request_t *obj, int value)
{

    value = of_bsn_virtual_port_create_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_bsn_vport_q_in_q_t *vport_ptr;

        vport_ptr = of_bsn_virtual_port_create_request_vport_get(obj);
        TEST_ASSERT(vport_ptr != NULL);
        value = of_bsn_vport_q_in_q_OF_VERSION_1_3_check(
            vport_ptr, value);
        TEST_ASSERT(value != 0);
        of_bsn_vport_q_in_q_delete(vport_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_create_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_create_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_create_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_3_populate(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_reply_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_reply_status_get(obj, &val32);

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_reply_OF_VERSION_1_3_check(
    of_bsn_virtual_port_remove_reply_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_virtual_port_remove_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_3_populate(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_virtual_port_remove_request_xid_get(obj, &val32);
    of_bsn_virtual_port_remove_request_experimenter_get(obj, &val32);
    of_bsn_virtual_port_remove_request_subtype_get(obj, &val32);
    of_bsn_virtual_port_remove_request_vport_no_get(obj, &val32);

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_virtual_port_remove_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_virtual_port_remove_request_OF_VERSION_1_3_check(
    of_bsn_virtual_port_remove_request_t *obj, int value)
{

    value = of_bsn_virtual_port_remove_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_virtual_port_remove_request_t *dup;

        TEST_ASSERT((dup = of_bsn_virtual_port_remove_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_virtual_port_remove_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vlan_counter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vlan_counter_stats_reply_OF_VERSION_1_3_populate(
    of_bsn_vlan_counter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_bsn_vlan_counter_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_vlan_counter_stats_reply_xid_get(obj, &val32);
    of_bsn_vlan_counter_stats_reply_flags_get(obj, &val16);
    of_bsn_vlan_counter_stats_reply_experimenter_get(obj, &val32);
    of_bsn_vlan_counter_stats_reply_subtype_get(obj, &val32);
    {
        of_list_bsn_vlan_counter_stats_entry_t sub_cls;

        /* Test bind */
        of_bsn_vlan_counter_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_bsn_vlan_counter_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_vlan_counter_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_vlan_counter_stats_reply_entries_set(
        obj, list);
    of_list_bsn_vlan_counter_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vlan_counter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vlan_counter_stats_reply_OF_VERSION_1_3_check(
    of_bsn_vlan_counter_stats_reply_t *obj, int value)
{

    value = of_bsn_vlan_counter_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_vlan_counter_stats_entry_t *entries_ptr;

        entries_ptr = of_bsn_vlan_counter_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_vlan_counter_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vlan_counter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_bsn_vlan_counter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vlan_counter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vlan_counter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vlan_counter_stats_request_OF_VERSION_1_3_populate(
    of_bsn_vlan_counter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_vlan_counter_stats_request_xid_get(obj, &val32);
    of_bsn_vlan_counter_stats_request_flags_get(obj, &val16);
    of_bsn_vlan_counter_stats_request_experimenter_get(obj, &val32);
    of_bsn_vlan_counter_stats_request_subtype_get(obj, &val32);
    of_bsn_vlan_counter_stats_request_vlan_vid_get(obj, &val16);

    value = of_bsn_vlan_counter_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vlan_counter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vlan_counter_stats_request_OF_VERSION_1_3_check(
    of_bsn_vlan_counter_stats_request_t *obj, int value)
{

    value = of_bsn_vlan_counter_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vlan_counter_stats_request_t *dup;

        TEST_ASSERT((dup = of_bsn_vlan_counter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vlan_counter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_3_populate(
    of_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_desc_str_t desc_str;
    of_serial_num_t ser_num;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_reply_xid_get(obj, &val32);
    of_desc_stats_reply_flags_get(obj, &val16);
    of_desc_stats_reply_mfr_desc_get(obj, &desc_str);
    of_desc_stats_reply_hw_desc_get(obj, &desc_str);
    of_desc_stats_reply_sw_desc_get(obj, &desc_str);
    of_desc_stats_reply_serial_num_get(obj, &ser_num);
    of_desc_stats_reply_dp_desc_get(obj, &desc_str);

    value = of_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_reply_OF_VERSION_1_3_check(
    of_desc_stats_reply_t *obj, int value)
{

    value = of_desc_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_3_populate(
    of_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_desc_stats_request_xid_get(obj, &val32);
    of_desc_stats_request_flags_get(obj, &val16);

    value = of_desc_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_desc_stats_request_OF_VERSION_1_3_check(
    of_desc_stats_request_t *obj, int value)
{

    value = of_desc_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_3_populate(
    of_echo_reply_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_reply_xid_get(obj, &val32);
    of_echo_reply_data_get(obj, &octets);

    value = of_echo_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_reply_OF_VERSION_1_3_check(
    of_echo_reply_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_reply_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_reply_t *dup;

        TEST_ASSERT((dup = of_echo_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_echo_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_3_populate(
    of_echo_request_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_echo_request_xid_get(obj, &val32);
    of_echo_request_data_get(obj, &octets);

    value = of_echo_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_echo_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_echo_request_OF_VERSION_1_3_check(
    of_echo_request_t *obj, int value)
{
    of_octets_t octets;

    value = of_echo_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_echo_request_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_echo_request_t *dup;

        TEST_ASSERT((dup = of_echo_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_echo_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_3_populate(
    of_error_msg_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_error_msg_xid_get(obj, &val32);

    value = of_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_error_msg_OF_VERSION_1_3_check(
    of_error_msg_t *obj, int value)
{

    value = of_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_error_msg_t *dup;

        TEST_ASSERT((dup = of_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_3_populate(
    of_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_xid_get(obj, &val32);
    of_experimenter_experimenter_get(obj, &val32);
    of_experimenter_subtype_get(obj, &val32);
    of_experimenter_data_get(obj, &octets);

    value = of_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_OF_VERSION_1_3_check(
    of_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_t *dup;

        TEST_ASSERT((dup = of_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_error_msg_OF_VERSION_1_3_populate(
    of_experimenter_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_error_msg_xid_get(obj, &val32);
    of_experimenter_error_msg_subtype_get(obj, &val16);
    of_experimenter_error_msg_experimenter_get(obj, &val32);
    of_experimenter_error_msg_data_get(obj, &octets);

    value = of_experimenter_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_error_msg_OF_VERSION_1_3_check(
    of_experimenter_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_error_msg_t *dup;

        TEST_ASSERT((dup = of_experimenter_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_multipart_reply_ofdpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_3_populate(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_multipart_reply_ofdpa_xid_get(obj, &val32);
    of_experimenter_multipart_reply_ofdpa_flags_get(obj, &val16);
    of_experimenter_multipart_reply_ofdpa_experimenter_get(obj, &val32);
    of_experimenter_multipart_reply_ofdpa_subtype_get(obj, &val32);

    value = of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_experimenter_multipart_reply_ofdpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_3_check(
    of_experimenter_multipart_reply_ofdpa_t *obj, int value)
{

    value = of_experimenter_multipart_reply_ofdpa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_multipart_reply_ofdpa_t *dup;

        TEST_ASSERT((dup = of_experimenter_multipart_reply_ofdpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_multipart_reply_ofdpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_multipart_request_ofdpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_multipart_request_ofdpa_OF_VERSION_1_3_populate(
    of_experimenter_multipart_request_ofdpa_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_multipart_request_ofdpa_xid_get(obj, &val32);
    of_experimenter_multipart_request_ofdpa_flags_get(obj, &val16);
    of_experimenter_multipart_request_ofdpa_experimenter_get(obj, &val32);
    of_experimenter_multipart_request_ofdpa_subtype_get(obj, &val32);

    value = of_experimenter_multipart_request_ofdpa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_experimenter_multipart_request_ofdpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_multipart_request_ofdpa_OF_VERSION_1_3_check(
    of_experimenter_multipart_request_ofdpa_t *obj, int value)
{

    value = of_experimenter_multipart_request_ofdpa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_multipart_request_ofdpa_t *dup;

        TEST_ASSERT((dup = of_experimenter_multipart_request_ofdpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_multipart_request_ofdpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_ofdpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_ofdpa_OF_VERSION_1_3_populate(
    of_experimenter_ofdpa_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_ofdpa_xid_get(obj, &val32);
    of_experimenter_ofdpa_experimenter_get(obj, &val32);
    of_experimenter_ofdpa_subtype_get(obj, &val32);
    of_experimenter_ofdpa_data_get(obj, &octets);

    value = of_experimenter_ofdpa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_experimenter_ofdpa_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_experimenter_ofdpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_ofdpa_OF_VERSION_1_3_check(
    of_experimenter_ofdpa_t *obj, int value)
{
    of_octets_t octets;

    value = of_experimenter_ofdpa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_experimenter_ofdpa_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_ofdpa_t *dup;

        TEST_ASSERT((dup = of_experimenter_ofdpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_ofdpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_3_populate(
    of_experimenter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_reply_xid_get(obj, &val32);
    of_experimenter_stats_reply_flags_get(obj, &val16);
    of_experimenter_stats_reply_experimenter_get(obj, &val32);
    of_experimenter_stats_reply_subtype_get(obj, &val32);

    value = of_experimenter_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_reply_OF_VERSION_1_3_check(
    of_experimenter_stats_reply_t *obj, int value)
{

    value = of_experimenter_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_experimenter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_3_populate(
    of_experimenter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_experimenter_stats_request_xid_get(obj, &val32);
    of_experimenter_stats_request_flags_get(obj, &val16);
    of_experimenter_stats_request_experimenter_get(obj, &val32);
    of_experimenter_stats_request_subtype_get(obj, &val32);

    value = of_experimenter_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_experimenter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_experimenter_stats_request_OF_VERSION_1_3_check(
    of_experimenter_stats_request_t *obj, int value)
{

    value = of_experimenter_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_experimenter_stats_request_t *dup;

        TEST_ASSERT((dup = of_experimenter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_experimenter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_3_populate(
    of_features_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    of_list_port_desc_t *list;

    /* Run thru accessors after new to ensure okay */
    of_features_reply_xid_get(obj, &val32);
    of_features_reply_datapath_id_get(obj, &val64);
    of_features_reply_n_buffers_get(obj, &val32);
    of_features_reply_n_tables_get(obj, &val8);
    of_features_reply_auxiliary_id_get(obj, &val8);
    of_features_reply_capabilities_get(obj, &val32);
    of_features_reply_reserved_get(obj, &val32);
    {
        of_list_port_desc_t sub_cls;

        /* Test bind */
        of_features_reply_ports_bind(obj, &sub_cls);
    }

    value = of_features_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_desc_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_port_desc_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_features_reply_ports_set(
        obj, list);
    of_list_port_desc_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_features_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_reply_OF_VERSION_1_3_check(
    of_features_reply_t *obj, int value)
{

    value = of_features_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_desc_t *ports_ptr;

        ports_ptr = of_features_reply_ports_get(obj);
        TEST_ASSERT(ports_ptr != NULL);
        value = of_list_port_desc_OF_VERSION_1_3_check(
            ports_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_desc_delete(ports_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_features_reply_t *dup;

        TEST_ASSERT((dup = of_features_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_features_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_3_populate(
    of_features_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_features_request_xid_get(obj, &val32);

    value = of_features_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_features_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_features_request_OF_VERSION_1_3_check(
    of_features_request_t *obj, int value)
{

    value = of_features_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_features_request_t *dup;

        TEST_ASSERT((dup = of_features_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_features_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_3_populate(
    of_flow_add_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_add_xid_get(obj, &val32);
    of_flow_add_cookie_get(obj, &val64);
    of_flow_add_cookie_mask_get(obj, &val64);
    of_flow_add_table_id_get(obj, &val8);
    of_flow_add_idle_timeout_get(obj, &val16);
    of_flow_add_hard_timeout_get(obj, &val16);
    of_flow_add_priority_get(obj, &val16);
    of_flow_add_buffer_id_get(obj, &val32);
    of_flow_add_out_port_get(obj, &port_no);
    of_flow_add_out_group_get(obj, &val32);
    of_flow_add_flags_get(obj, &val16);
    of_flow_add_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_add_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_add_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_add_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_add_OF_VERSION_1_3_check(
    of_flow_add_t *obj, int value)
{
    of_match_t match;

    value = of_flow_add_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_add_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_add_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_3_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_add_t *dup;

        TEST_ASSERT((dup = of_flow_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_3_populate(
    of_flow_delete_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_xid_get(obj, &val32);
    of_flow_delete_cookie_get(obj, &val64);
    of_flow_delete_cookie_mask_get(obj, &val64);
    of_flow_delete_table_id_get(obj, &val8);
    of_flow_delete_idle_timeout_get(obj, &val16);
    of_flow_delete_hard_timeout_get(obj, &val16);
    of_flow_delete_priority_get(obj, &val16);
    of_flow_delete_buffer_id_get(obj, &val32);
    of_flow_delete_out_port_get(obj, &port_no);
    of_flow_delete_out_group_get(obj, &val32);
    of_flow_delete_flags_get(obj, &val16);
    of_flow_delete_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_delete_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_OF_VERSION_1_3_check(
    of_flow_delete_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_delete_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_3_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_t *dup;

        TEST_ASSERT((dup = of_flow_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_delete_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_3_populate(
    of_flow_delete_strict_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_delete_strict_xid_get(obj, &val32);
    of_flow_delete_strict_cookie_get(obj, &val64);
    of_flow_delete_strict_cookie_mask_get(obj, &val64);
    of_flow_delete_strict_table_id_get(obj, &val8);
    of_flow_delete_strict_idle_timeout_get(obj, &val16);
    of_flow_delete_strict_hard_timeout_get(obj, &val16);
    of_flow_delete_strict_priority_get(obj, &val16);
    of_flow_delete_strict_buffer_id_get(obj, &val32);
    of_flow_delete_strict_out_port_get(obj, &port_no);
    of_flow_delete_strict_out_group_get(obj, &val32);
    of_flow_delete_strict_flags_get(obj, &val16);
    of_flow_delete_strict_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_delete_strict_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_delete_strict_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_delete_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_delete_strict_OF_VERSION_1_3_check(
    of_flow_delete_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_delete_strict_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_delete_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_delete_strict_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_3_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_delete_strict_t *dup;

        TEST_ASSERT((dup = of_flow_delete_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_delete_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_3_populate(
    of_flow_mod_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_xid_get(obj, &val32);
    of_flow_mod_cookie_get(obj, &val64);
    of_flow_mod_cookie_mask_get(obj, &val64);
    of_flow_mod_table_id_get(obj, &val8);
    of_flow_mod_idle_timeout_get(obj, &val16);
    of_flow_mod_hard_timeout_get(obj, &val16);
    of_flow_mod_priority_get(obj, &val16);
    of_flow_mod_buffer_id_get(obj, &val32);
    of_flow_mod_out_port_get(obj, &port_no);
    of_flow_mod_out_group_get(obj, &val32);
    of_flow_mod_flags_get(obj, &val16);
    of_flow_mod_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_mod_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_mod_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_OF_VERSION_1_3_check(
    of_flow_mod_t *obj, int value)
{
    of_match_t match;

    value = of_flow_mod_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_mod_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_3_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_t *dup;

        TEST_ASSERT((dup = of_flow_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_3_populate(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_flow_mod_failed_error_msg_xid_get(obj, &val32);
    of_flow_mod_failed_error_msg_code_get(obj, &val16);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_flow_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_mod_failed_error_msg_OF_VERSION_1_3_check(
    of_flow_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_flow_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_flow_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_3_populate(
    of_flow_modify_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_xid_get(obj, &val32);
    of_flow_modify_cookie_get(obj, &val64);
    of_flow_modify_cookie_mask_get(obj, &val64);
    of_flow_modify_table_id_get(obj, &val8);
    of_flow_modify_idle_timeout_get(obj, &val16);
    of_flow_modify_hard_timeout_get(obj, &val16);
    of_flow_modify_priority_get(obj, &val16);
    of_flow_modify_buffer_id_get(obj, &val32);
    of_flow_modify_out_port_get(obj, &port_no);
    of_flow_modify_out_group_get(obj, &val32);
    of_flow_modify_flags_get(obj, &val16);
    of_flow_modify_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_modify_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_OF_VERSION_1_3_check(
    of_flow_modify_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_modify_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_3_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_t *dup;

        TEST_ASSERT((dup = of_flow_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_modify_strict
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_3_populate(
    of_flow_modify_strict_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;
    uint16_t val16;
    of_port_no_t port_no;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_modify_strict_xid_get(obj, &val32);
    of_flow_modify_strict_cookie_get(obj, &val64);
    of_flow_modify_strict_cookie_mask_get(obj, &val64);
    of_flow_modify_strict_table_id_get(obj, &val8);
    of_flow_modify_strict_idle_timeout_get(obj, &val16);
    of_flow_modify_strict_hard_timeout_get(obj, &val16);
    of_flow_modify_strict_priority_get(obj, &val16);
    of_flow_modify_strict_buffer_id_get(obj, &val32);
    of_flow_modify_strict_out_port_get(obj, &port_no);
    of_flow_modify_strict_out_group_get(obj, &val32);
    of_flow_modify_strict_flags_get(obj, &val16);
    of_flow_modify_strict_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_modify_strict_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_modify_strict_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_modify_strict
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_modify_strict_OF_VERSION_1_3_check(
    of_flow_modify_strict_t *obj, int value)
{
    of_match_t match;

    value = of_flow_modify_strict_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_modify_strict_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_modify_strict_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_3_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_modify_strict_t *dup;

        TEST_ASSERT((dup = of_flow_modify_strict_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_modify_strict_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_removed
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_3_populate(
    of_flow_removed_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint16_t val16;
    uint8_t val8;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_flow_removed_xid_get(obj, &val32);
    of_flow_removed_cookie_get(obj, &val64);
    of_flow_removed_priority_get(obj, &val16);
    of_flow_removed_reason_get(obj, &val8);
    of_flow_removed_table_id_get(obj, &val8);
    of_flow_removed_duration_sec_get(obj, &val32);
    of_flow_removed_duration_nsec_get(obj, &val32);
    of_flow_removed_idle_timeout_get(obj, &val16);
    of_flow_removed_hard_timeout_get(obj, &val16);
    of_flow_removed_packet_count_get(obj, &val64);
    of_flow_removed_byte_count_get(obj, &val64);
    of_flow_removed_match_get(obj, &match);

    value = of_flow_removed_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_removed
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_removed_OF_VERSION_1_3_check(
    of_flow_removed_t *obj, int value)
{
    of_match_t match;

    value = of_flow_removed_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_removed_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_removed_t *dup;

        TEST_ASSERT((dup = of_flow_removed_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_removed_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_3_populate(
    of_flow_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_flow_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_reply_xid_get(obj, &val32);
    of_flow_stats_reply_flags_get(obj, &val16);
    {
        of_list_flow_stats_entry_t sub_cls;

        /* Test bind */
        of_flow_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_flow_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_flow_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_flow_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_reply_entries_set(
        obj, list);
    of_list_flow_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_reply_OF_VERSION_1_3_check(
    of_flow_stats_reply_t *obj, int value)
{

    value = of_flow_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_flow_stats_entry_t *entries_ptr;

        entries_ptr = of_flow_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_flow_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_flow_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_reply_t *dup;

        TEST_ASSERT((dup = of_flow_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_3_populate(
    of_flow_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    of_port_no_t port_no;
    uint64_t val64;
    of_match_t match;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_request_xid_get(obj, &val32);
    of_flow_stats_request_flags_get(obj, &val16);
    of_flow_stats_request_table_id_get(obj, &val8);
    of_flow_stats_request_out_port_get(obj, &port_no);
    of_flow_stats_request_out_group_get(obj, &val32);
    of_flow_stats_request_cookie_get(obj, &val64);
    of_flow_stats_request_cookie_mask_get(obj, &val64);
    of_flow_stats_request_match_get(obj, &match);

    value = of_flow_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_set(
        obj, &match);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_request_OF_VERSION_1_3_check(
    of_flow_stats_request_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_request_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_request_t *dup;

        TEST_ASSERT((dup = of_flow_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_3_populate(
    of_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_get_config_reply_xid_get(obj, &val32);
    of_get_config_reply_flags_get(obj, &val16);
    of_get_config_reply_miss_send_len_get(obj, &val16);

    value = of_get_config_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_reply_OF_VERSION_1_3_check(
    of_get_config_reply_t *obj, int value)
{

    value = of_get_config_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_3_populate(
    of_get_config_request_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_get_config_request_xid_get(obj, &val32);

    value = of_get_config_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_get_config_request_OF_VERSION_1_3_check(
    of_get_config_request_t *obj, int value)
{

    value = of_get_config_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_get_config_request_t *dup;

        TEST_ASSERT((dup = of_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_add_OF_VERSION_1_3_populate(
    of_group_add_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_add_xid_get(obj, &val32);
    of_group_add_group_type_get(obj, &val8);
    of_group_add_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_add_buckets_bind(obj, &sub_cls);
    }

    value = of_group_add_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_add_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_add_OF_VERSION_1_3_check(
    of_group_add_t *obj, int value)
{

    value = of_group_add_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_add_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_3_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_add_t *dup;

        TEST_ASSERT((dup = of_group_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_delete_OF_VERSION_1_3_populate(
    of_group_delete_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_delete_xid_get(obj, &val32);
    of_group_delete_group_type_get(obj, &val8);
    of_group_delete_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_delete_buckets_bind(obj, &sub_cls);
    }

    value = of_group_delete_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_delete_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_delete_OF_VERSION_1_3_check(
    of_group_delete_t *obj, int value)
{

    value = of_group_delete_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_delete_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_3_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_delete_t *dup;

        TEST_ASSERT((dup = of_group_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_reply_OF_VERSION_1_3_populate(
    of_group_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_group_desc_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_reply_xid_get(obj, &val32);
    of_group_desc_stats_reply_flags_get(obj, &val16);
    {
        of_list_group_desc_stats_entry_t sub_cls;

        /* Test bind */
        of_group_desc_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_group_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_group_desc_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_group_desc_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_desc_stats_reply_entries_set(
        obj, list);
    of_list_group_desc_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_reply_OF_VERSION_1_3_check(
    of_group_desc_stats_reply_t *obj, int value)
{

    value = of_group_desc_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_group_desc_stats_entry_t *entries_ptr;

        entries_ptr = of_group_desc_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_group_desc_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_group_desc_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_request_OF_VERSION_1_3_populate(
    of_group_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_request_xid_get(obj, &val32);
    of_group_desc_stats_request_flags_get(obj, &val16);

    value = of_group_desc_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_request_OF_VERSION_1_3_check(
    of_group_desc_stats_request_t *obj, int value)
{

    value = of_group_desc_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_features_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_reply_OF_VERSION_1_3_populate(
    of_group_features_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_features_stats_reply_xid_get(obj, &val32);
    of_group_features_stats_reply_flags_get(obj, &val16);
    of_group_features_stats_reply_types_get(obj, &val32);
    of_group_features_stats_reply_capabilities_get(obj, &val32);
    of_group_features_stats_reply_max_groups_all_get(obj, &val32);
    of_group_features_stats_reply_max_groups_select_get(obj, &val32);
    of_group_features_stats_reply_max_groups_indirect_get(obj, &val32);
    of_group_features_stats_reply_max_groups_ff_get(obj, &val32);
    of_group_features_stats_reply_actions_all_get(obj, &val32);
    of_group_features_stats_reply_actions_select_get(obj, &val32);
    of_group_features_stats_reply_actions_indirect_get(obj, &val32);
    of_group_features_stats_reply_actions_ff_get(obj, &val32);

    value = of_group_features_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_features_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_reply_OF_VERSION_1_3_check(
    of_group_features_stats_reply_t *obj, int value)
{

    value = of_group_features_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_features_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_features_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_features_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_features_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_request_OF_VERSION_1_3_populate(
    of_group_features_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_features_stats_request_xid_get(obj, &val32);
    of_group_features_stats_request_flags_get(obj, &val16);

    value = of_group_features_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_features_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_features_stats_request_OF_VERSION_1_3_check(
    of_group_features_stats_request_t *obj, int value)
{

    value = of_group_features_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_features_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_features_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_features_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_mod_OF_VERSION_1_3_populate(
    of_group_mod_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_mod_xid_get(obj, &val32);
    of_group_mod_group_type_get(obj, &val8);
    of_group_mod_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_mod_buckets_bind(obj, &sub_cls);
    }

    value = of_group_mod_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_mod_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_mod_OF_VERSION_1_3_check(
    of_group_mod_t *obj, int value)
{

    value = of_group_mod_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_mod_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_3_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_mod_t *dup;

        TEST_ASSERT((dup = of_group_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_mod_failed_error_msg_OF_VERSION_1_3_populate(
    of_group_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_group_mod_failed_error_msg_xid_get(obj, &val32);
    of_group_mod_failed_error_msg_code_get(obj, &val16);
    of_group_mod_failed_error_msg_data_get(obj, &octets);

    value = of_group_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_group_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_group_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_mod_failed_error_msg_OF_VERSION_1_3_check(
    of_group_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_group_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_group_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_group_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_modify_OF_VERSION_1_3_populate(
    of_group_modify_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_modify_xid_get(obj, &val32);
    of_group_modify_group_type_get(obj, &val8);
    of_group_modify_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_modify_buckets_bind(obj, &sub_cls);
    }

    value = of_group_modify_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_modify_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_modify_OF_VERSION_1_3_check(
    of_group_modify_t *obj, int value)
{

    value = of_group_modify_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_modify_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_3_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_modify_t *dup;

        TEST_ASSERT((dup = of_group_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_reply_OF_VERSION_1_3_populate(
    of_group_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_group_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_reply_xid_get(obj, &val32);
    of_group_stats_reply_flags_get(obj, &val16);
    {
        of_list_group_stats_entry_t sub_cls;

        /* Test bind */
        of_group_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_group_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_group_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_group_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_stats_reply_entries_set(
        obj, list);
    of_list_group_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_reply_OF_VERSION_1_3_check(
    of_group_stats_reply_t *obj, int value)
{

    value = of_group_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_group_stats_entry_t *entries_ptr;

        entries_ptr = of_group_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_group_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_group_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_reply_t *dup;

        TEST_ASSERT((dup = of_group_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_request_OF_VERSION_1_3_populate(
    of_group_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_request_xid_get(obj, &val32);
    of_group_stats_request_flags_get(obj, &val16);
    of_group_stats_request_group_id_get(obj, &val32);

    value = of_group_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_request_OF_VERSION_1_3_check(
    of_group_stats_request_t *obj, int value)
{

    value = of_group_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_request_t *dup;

        TEST_ASSERT((dup = of_group_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_3_populate(
    of_hello_t *obj, int value)
{
    uint32_t val32;
    of_list_hello_elem_t *list;

    /* Run thru accessors after new to ensure okay */
    of_hello_xid_get(obj, &val32);
    {
        of_list_hello_elem_t sub_cls;

        /* Test bind */
        of_hello_elements_bind(obj, &sub_cls);
    }

    value = of_hello_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_hello_elem_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_hello_elem_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_hello_elements_set(
        obj, list);
    of_list_hello_elem_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_hello
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_OF_VERSION_1_3_check(
    of_hello_t *obj, int value)
{

    value = of_hello_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_hello_elem_t *elements_ptr;

        elements_ptr = of_hello_elements_get(obj);
        TEST_ASSERT(elements_ptr != NULL);
        value = of_list_hello_elem_OF_VERSION_1_3_check(
            elements_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_hello_elem_delete(elements_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_t *dup;

        TEST_ASSERT((dup = of_hello_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_3_populate(
    of_hello_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_hello_failed_error_msg_xid_get(obj, &val32);
    of_hello_failed_error_msg_code_get(obj, &val16);
    of_hello_failed_error_msg_data_get(obj, &octets);

    value = of_hello_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_hello_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_failed_error_msg_OF_VERSION_1_3_check(
    of_hello_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_hello_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_hello_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_hello_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_add
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_add_OF_VERSION_1_3_populate(
    of_meter_add_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_meter_band_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_add_xid_get(obj, &val32);
    of_meter_add_flags_get(obj, &val16);
    of_meter_add_meter_id_get(obj, &val32);
    {
        of_list_meter_band_t sub_cls;

        /* Test bind */
        of_meter_add_meters_bind(obj, &sub_cls);
    }

    value = of_meter_add_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_band_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_band_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_add_meters_set(
        obj, list);
    of_list_meter_band_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_add
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_add_OF_VERSION_1_3_check(
    of_meter_add_t *obj, int value)
{

    value = of_meter_add_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_band_t *meters_ptr;

        meters_ptr = of_meter_add_meters_get(obj);
        TEST_ASSERT(meters_ptr != NULL);
        value = of_list_meter_band_OF_VERSION_1_3_check(
            meters_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_band_delete(meters_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_add_t *dup;

        TEST_ASSERT((dup = of_meter_add_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_add_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_band_ofdpa_color_set
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_band_ofdpa_color_set_OF_VERSION_1_3_populate(
    of_meter_band_ofdpa_color_set_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_meter_band_ofdpa_color_set_xid_get(obj, &val32);
    of_meter_band_ofdpa_color_set_meter_command_get(obj, &val16);
    of_meter_band_ofdpa_color_set_meter_flags_get(obj, &val16);
    of_meter_band_ofdpa_color_set_meter_id_get(obj, &val32);
    of_meter_band_ofdpa_color_set_mb_type_get(obj, &val16);
    of_meter_band_ofdpa_color_set_rate_get(obj, &val32);
    of_meter_band_ofdpa_color_set_burst_size_get(obj, &val32);
    of_meter_band_ofdpa_color_set_experimenter_get(obj, &val32);
    of_meter_band_ofdpa_color_set_subtype_get(obj, &val32);
    of_meter_band_ofdpa_color_set_mode_get(obj, &val8);
    of_meter_band_ofdpa_color_set_color_aware_get(obj, &val8);
    of_meter_band_ofdpa_color_set_color_get(obj, &val8);

    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_band_ofdpa_color_set
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_band_ofdpa_color_set_OF_VERSION_1_3_check(
    of_meter_band_ofdpa_color_set_t *obj, int value)
{

    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_band_ofdpa_color_set_t *dup;

        TEST_ASSERT((dup = of_meter_band_ofdpa_color_set_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_band_ofdpa_color_set_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_config_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_config_stats_reply_OF_VERSION_1_3_populate(
    of_meter_config_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_meter_band_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_config_stats_reply_xid_get(obj, &val32);
    of_meter_config_stats_reply_flags_get(obj, &val16);
    {
        of_list_meter_band_t sub_cls;

        /* Test bind */
        of_meter_config_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_meter_config_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_band_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_band_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_config_stats_reply_entries_set(
        obj, list);
    of_list_meter_band_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_config_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_config_stats_reply_OF_VERSION_1_3_check(
    of_meter_config_stats_reply_t *obj, int value)
{

    value = of_meter_config_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_band_t *entries_ptr;

        entries_ptr = of_meter_config_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_meter_band_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_band_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_config_stats_reply_t *dup;

        TEST_ASSERT((dup = of_meter_config_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_config_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_config_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_config_stats_request_OF_VERSION_1_3_populate(
    of_meter_config_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_meter_config_stats_request_xid_get(obj, &val32);
    of_meter_config_stats_request_flags_get(obj, &val16);
    of_meter_config_stats_request_meter_id_get(obj, &val32);

    value = of_meter_config_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_config_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_config_stats_request_OF_VERSION_1_3_check(
    of_meter_config_stats_request_t *obj, int value)
{

    value = of_meter_config_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_config_stats_request_t *dup;

        TEST_ASSERT((dup = of_meter_config_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_config_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_delete
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_delete_OF_VERSION_1_3_populate(
    of_meter_delete_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_meter_band_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_delete_xid_get(obj, &val32);
    of_meter_delete_flags_get(obj, &val16);
    of_meter_delete_meter_id_get(obj, &val32);
    {
        of_list_meter_band_t sub_cls;

        /* Test bind */
        of_meter_delete_meters_bind(obj, &sub_cls);
    }

    value = of_meter_delete_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_band_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_band_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_delete_meters_set(
        obj, list);
    of_list_meter_band_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_delete
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_delete_OF_VERSION_1_3_check(
    of_meter_delete_t *obj, int value)
{

    value = of_meter_delete_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_band_t *meters_ptr;

        meters_ptr = of_meter_delete_meters_get(obj);
        TEST_ASSERT(meters_ptr != NULL);
        value = of_list_meter_band_OF_VERSION_1_3_check(
            meters_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_band_delete(meters_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_delete_t *dup;

        TEST_ASSERT((dup = of_meter_delete_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_delete_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_features_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_features_stats_reply_OF_VERSION_1_3_populate(
    of_meter_features_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_meter_features_t *features;

    /* Run thru accessors after new to ensure okay */
    of_meter_features_stats_reply_xid_get(obj, &val32);
    of_meter_features_stats_reply_flags_get(obj, &val16);
    {
        of_meter_features_t sub_cls;

        /* Test bind */
        of_meter_features_stats_reply_features_bind(obj, &sub_cls);
    }

    value = of_meter_features_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    features = of_meter_features_new(OF_VERSION_1_3);
    TEST_ASSERT(features != NULL);
    value = of_meter_features_OF_VERSION_1_3_populate(
        features, value);
    TEST_ASSERT(value != 0);
    of_meter_features_stats_reply_features_set(
        obj, features);
    of_meter_features_delete(features);

    return value;
}

/**
 * Check all members of an object of type of_meter_features_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_features_stats_reply_OF_VERSION_1_3_check(
    of_meter_features_stats_reply_t *obj, int value)
{

    value = of_meter_features_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_meter_features_t *features_ptr;

        features_ptr = of_meter_features_stats_reply_features_get(obj);
        TEST_ASSERT(features_ptr != NULL);
        value = of_meter_features_OF_VERSION_1_3_check(
            features_ptr, value);
        TEST_ASSERT(value != 0);
        of_meter_features_delete(features_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_features_stats_reply_t *dup;

        TEST_ASSERT((dup = of_meter_features_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_features_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_features_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_features_stats_request_OF_VERSION_1_3_populate(
    of_meter_features_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_meter_features_stats_request_xid_get(obj, &val32);
    of_meter_features_stats_request_flags_get(obj, &val16);

    value = of_meter_features_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_features_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_features_stats_request_OF_VERSION_1_3_check(
    of_meter_features_stats_request_t *obj, int value)
{

    value = of_meter_features_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_features_stats_request_t *dup;

        TEST_ASSERT((dup = of_meter_features_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_features_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_mod_OF_VERSION_1_3_populate(
    of_meter_mod_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_meter_band_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_mod_xid_get(obj, &val32);
    of_meter_mod_flags_get(obj, &val16);
    of_meter_mod_meter_id_get(obj, &val32);
    {
        of_list_meter_band_t sub_cls;

        /* Test bind */
        of_meter_mod_meters_bind(obj, &sub_cls);
    }

    value = of_meter_mod_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_band_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_band_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_mod_meters_set(
        obj, list);
    of_list_meter_band_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_mod_OF_VERSION_1_3_check(
    of_meter_mod_t *obj, int value)
{

    value = of_meter_mod_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_band_t *meters_ptr;

        meters_ptr = of_meter_mod_meters_get(obj);
        TEST_ASSERT(meters_ptr != NULL);
        value = of_list_meter_band_OF_VERSION_1_3_check(
            meters_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_band_delete(meters_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_mod_t *dup;

        TEST_ASSERT((dup = of_meter_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_mod_failed_error_msg_OF_VERSION_1_3_populate(
    of_meter_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_meter_mod_failed_error_msg_xid_get(obj, &val32);
    of_meter_mod_failed_error_msg_code_get(obj, &val16);
    of_meter_mod_failed_error_msg_data_get(obj, &octets);

    value = of_meter_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_meter_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_meter_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_mod_failed_error_msg_OF_VERSION_1_3_check(
    of_meter_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_meter_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_meter_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_meter_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_modify
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_modify_OF_VERSION_1_3_populate(
    of_meter_modify_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_meter_band_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_modify_xid_get(obj, &val32);
    of_meter_modify_flags_get(obj, &val16);
    of_meter_modify_meter_id_get(obj, &val32);
    {
        of_list_meter_band_t sub_cls;

        /* Test bind */
        of_meter_modify_meters_bind(obj, &sub_cls);
    }

    value = of_meter_modify_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_band_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_band_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_modify_meters_set(
        obj, list);
    of_list_meter_band_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_modify
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_modify_OF_VERSION_1_3_check(
    of_meter_modify_t *obj, int value)
{

    value = of_meter_modify_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_band_t *meters_ptr;

        meters_ptr = of_meter_modify_meters_get(obj);
        TEST_ASSERT(meters_ptr != NULL);
        value = of_list_meter_band_OF_VERSION_1_3_check(
            meters_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_band_delete(meters_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_modify_t *dup;

        TEST_ASSERT((dup = of_meter_modify_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_modify_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_stats_reply_OF_VERSION_1_3_populate(
    of_meter_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_meter_stats_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_stats_reply_xid_get(obj, &val32);
    of_meter_stats_reply_flags_get(obj, &val16);
    {
        of_list_meter_stats_t sub_cls;

        /* Test bind */
        of_meter_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_meter_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_stats_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_stats_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_stats_reply_entries_set(
        obj, list);
    of_list_meter_stats_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_stats_reply_OF_VERSION_1_3_check(
    of_meter_stats_reply_t *obj, int value)
{

    value = of_meter_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_stats_t *entries_ptr;

        entries_ptr = of_meter_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_meter_stats_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_stats_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_stats_reply_t *dup;

        TEST_ASSERT((dup = of_meter_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_stats_request_OF_VERSION_1_3_populate(
    of_meter_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_meter_stats_request_xid_get(obj, &val32);
    of_meter_stats_request_flags_get(obj, &val16);
    of_meter_stats_request_meter_id_get(obj, &val32);

    value = of_meter_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_stats_request_OF_VERSION_1_3_check(
    of_meter_stats_request_t *obj, int value)
{

    value = of_meter_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_stats_request_t *dup;

        TEST_ASSERT((dup = of_meter_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_nicira_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_3_populate(
    of_nicira_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_nicira_header_xid_get(obj, &val32);
    of_nicira_header_experimenter_get(obj, &val32);
    of_nicira_header_subtype_get(obj, &val32);

    value = of_nicira_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_nicira_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_nicira_header_OF_VERSION_1_3_check(
    of_nicira_header_t *obj, int value)
{

    value = of_nicira_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_nicira_header_t *dup;

        TEST_ASSERT((dup = of_nicira_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_nicira_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_3_populate(
    of_packet_in_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;
    uint64_t val64;
    of_match_t match;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_in_xid_get(obj, &val32);
    of_packet_in_buffer_id_get(obj, &val32);
    of_packet_in_total_len_get(obj, &val16);
    of_packet_in_reason_get(obj, &val8);
    of_packet_in_table_id_get(obj, &val8);
    of_packet_in_cookie_get(obj, &val64);
    of_packet_in_match_get(obj, &match);
    of_packet_in_data_get(obj, &octets);

    value = of_packet_in_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_packet_in_match_set(
        obj, &match);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_in_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_in_OF_VERSION_1_3_check(
    of_packet_in_t *obj, int value)
{
    of_match_t match;
    of_octets_t octets;

    value = of_packet_in_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_packet_in_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);
    of_packet_in_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_in_t *dup;

        TEST_ASSERT((dup = of_packet_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_3_populate(
    of_packet_out_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_action_t *list;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_packet_out_xid_get(obj, &val32);
    of_packet_out_buffer_id_get(obj, &val32);
    of_packet_out_in_port_get(obj, &port_no);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_packet_out_actions_bind(obj, &sub_cls);
    }
    of_packet_out_data_get(obj, &octets);

    value = of_packet_out_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_out_actions_set(
        obj, list);
    of_list_action_delete(list);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_packet_out_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_packet_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_out_OF_VERSION_1_3_check(
    of_packet_out_t *obj, int value)
{
    of_octets_t octets;

    value = of_packet_out_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_packet_out_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_3_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }
    of_packet_out_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_out_t *dup;

        TEST_ASSERT((dup = of_packet_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_desc_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_desc_stats_reply_OF_VERSION_1_3_populate(
    of_port_desc_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_port_desc_t *list;

    /* Run thru accessors after new to ensure okay */
    of_port_desc_stats_reply_xid_get(obj, &val32);
    of_port_desc_stats_reply_flags_get(obj, &val16);
    {
        of_list_port_desc_t sub_cls;

        /* Test bind */
        of_port_desc_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_port_desc_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_desc_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_port_desc_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_port_desc_stats_reply_entries_set(
        obj, list);
    of_list_port_desc_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_port_desc_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_desc_stats_reply_OF_VERSION_1_3_check(
    of_port_desc_stats_reply_t *obj, int value)
{

    value = of_port_desc_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_desc_t *entries_ptr;

        entries_ptr = of_port_desc_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_port_desc_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_desc_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_desc_stats_reply_t *dup;

        TEST_ASSERT((dup = of_port_desc_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_desc_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_desc_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_desc_stats_request_OF_VERSION_1_3_populate(
    of_port_desc_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_port_desc_stats_request_xid_get(obj, &val32);
    of_port_desc_stats_request_flags_get(obj, &val16);

    value = of_port_desc_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_desc_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_desc_stats_request_OF_VERSION_1_3_check(
    of_port_desc_stats_request_t *obj, int value)
{

    value = of_port_desc_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_desc_stats_request_t *dup;

        TEST_ASSERT((dup = of_port_desc_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_desc_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_3_populate(
    of_port_mod_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_xid_get(obj, &val32);
    of_port_mod_port_no_get(obj, &port_no);
    of_port_mod_hw_addr_get(obj, &mac_addr);
    of_port_mod_config_get(obj, &val32);
    of_port_mod_mask_get(obj, &val32);
    of_port_mod_advertise_get(obj, &val32);

    value = of_port_mod_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_OF_VERSION_1_3_check(
    of_port_mod_t *obj, int value)
{

    value = of_port_mod_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_t *dup;

        TEST_ASSERT((dup = of_port_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_3_populate(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_port_mod_failed_error_msg_xid_get(obj, &val32);
    of_port_mod_failed_error_msg_code_get(obj, &val16);
    of_port_mod_failed_error_msg_data_get(obj, &octets);

    value = of_port_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_port_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_mod_failed_error_msg_OF_VERSION_1_3_check(
    of_port_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_port_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_port_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_port_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_3_populate(
    of_port_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_port_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_reply_xid_get(obj, &val32);
    of_port_stats_reply_flags_get(obj, &val16);
    {
        of_list_port_stats_entry_t sub_cls;

        /* Test bind */
        of_port_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_port_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_port_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_port_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_port_stats_reply_entries_set(
        obj, list);
    of_list_port_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_reply_OF_VERSION_1_3_check(
    of_port_stats_reply_t *obj, int value)
{

    value = of_port_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_port_stats_entry_t *entries_ptr;

        entries_ptr = of_port_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_port_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_port_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_reply_t *dup;

        TEST_ASSERT((dup = of_port_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_3_populate(
    of_port_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_request_xid_get(obj, &val32);
    of_port_stats_request_flags_get(obj, &val16);
    of_port_stats_request_port_no_get(obj, &port_no);

    value = of_port_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_request_OF_VERSION_1_3_check(
    of_port_stats_request_t *obj, int value)
{

    value = of_port_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_request_t *dup;

        TEST_ASSERT((dup = of_port_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_3_populate(
    of_port_status_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;
    of_port_desc_t *port_desc;

    /* Run thru accessors after new to ensure okay */
    of_port_status_xid_get(obj, &val32);
    of_port_status_reason_get(obj, &val8);
    {
        of_port_desc_t sub_cls;

        /* Test bind */
        of_port_status_desc_bind(obj, &sub_cls);
    }

    value = of_port_status_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    port_desc = of_port_desc_new(OF_VERSION_1_3);
    TEST_ASSERT(port_desc != NULL);
    value = of_port_desc_OF_VERSION_1_3_populate(
        port_desc, value);
    TEST_ASSERT(value != 0);
    of_port_status_desc_set(
        obj, port_desc);
    of_port_desc_delete(port_desc);

    return value;
}

/**
 * Check all members of an object of type of_port_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_status_OF_VERSION_1_3_check(
    of_port_status_t *obj, int value)
{

    value = of_port_status_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_port_desc_t *desc_ptr;

        desc_ptr = of_port_status_desc_get(obj);
        TEST_ASSERT(desc_ptr != NULL);
        value = of_port_desc_OF_VERSION_1_3_check(
            desc_ptr, value);
        TEST_ASSERT(value != 0);
        of_port_desc_delete(desc_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_port_status_t *dup;

        TEST_ASSERT((dup = of_port_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_3_populate(
    of_queue_get_config_reply_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_packet_queue_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_reply_xid_get(obj, &val32);
    of_queue_get_config_reply_port_get(obj, &port_no);
    {
        of_list_packet_queue_t sub_cls;

        /* Test bind */
        of_queue_get_config_reply_queues_bind(obj, &sub_cls);
    }

    value = of_queue_get_config_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_packet_queue_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_packet_queue_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_get_config_reply_queues_set(
        obj, list);
    of_list_packet_queue_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_reply_OF_VERSION_1_3_check(
    of_queue_get_config_reply_t *obj, int value)
{

    value = of_queue_get_config_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_packet_queue_t *queues_ptr;

        queues_ptr = of_queue_get_config_reply_queues_get(obj);
        TEST_ASSERT(queues_ptr != NULL);
        value = of_list_packet_queue_OF_VERSION_1_3_check(
            queues_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_packet_queue_delete(queues_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_reply_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_get_config_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_3_populate(
    of_queue_get_config_request_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_get_config_request_xid_get(obj, &val32);
    of_queue_get_config_request_port_get(obj, &port_no);

    value = of_queue_get_config_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_get_config_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_get_config_request_OF_VERSION_1_3_check(
    of_queue_get_config_request_t *obj, int value)
{

    value = of_queue_get_config_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_get_config_request_t *dup;

        TEST_ASSERT((dup = of_queue_get_config_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_get_config_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_op_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_3_populate(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_queue_op_failed_error_msg_xid_get(obj, &val32);
    of_queue_op_failed_error_msg_code_get(obj, &val16);
    of_queue_op_failed_error_msg_data_get(obj, &octets);

    value = of_queue_op_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_queue_op_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_op_failed_error_msg_OF_VERSION_1_3_check(
    of_queue_op_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_queue_op_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_queue_op_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_op_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_queue_op_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_op_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_3_populate(
    of_queue_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_queue_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_reply_xid_get(obj, &val32);
    of_queue_stats_reply_flags_get(obj, &val16);
    {
        of_list_queue_stats_entry_t sub_cls;

        /* Test bind */
        of_queue_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_queue_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_queue_stats_reply_entries_set(
        obj, list);
    of_list_queue_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_reply_OF_VERSION_1_3_check(
    of_queue_stats_reply_t *obj, int value)
{

    value = of_queue_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_stats_entry_t *entries_ptr;

        entries_ptr = of_queue_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_queue_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_reply_t *dup;

        TEST_ASSERT((dup = of_queue_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_3_populate(
    of_queue_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_request_xid_get(obj, &val32);
    of_queue_stats_request_flags_get(obj, &val16);
    of_queue_stats_request_port_no_get(obj, &port_no);
    of_queue_stats_request_queue_id_get(obj, &val32);

    value = of_queue_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_request_OF_VERSION_1_3_check(
    of_queue_stats_request_t *obj, int value)
{

    value = of_queue_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_request_t *dup;

        TEST_ASSERT((dup = of_queue_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_role_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_role_reply_OF_VERSION_1_3_populate(
    of_role_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_role_reply_xid_get(obj, &val32);
    of_role_reply_role_get(obj, &val32);
    of_role_reply_generation_id_get(obj, &val64);

    value = of_role_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_role_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_role_reply_OF_VERSION_1_3_check(
    of_role_reply_t *obj, int value)
{

    value = of_role_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_role_reply_t *dup;

        TEST_ASSERT((dup = of_role_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_role_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_role_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_role_request_OF_VERSION_1_3_populate(
    of_role_request_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_role_request_xid_get(obj, &val32);
    of_role_request_role_get(obj, &val32);
    of_role_request_generation_id_get(obj, &val64);

    value = of_role_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_role_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_role_request_OF_VERSION_1_3_check(
    of_role_request_t *obj, int value)
{

    value = of_role_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_role_request_t *dup;

        TEST_ASSERT((dup = of_role_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_role_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_role_request_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_role_request_failed_error_msg_OF_VERSION_1_3_populate(
    of_role_request_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_role_request_failed_error_msg_xid_get(obj, &val32);
    of_role_request_failed_error_msg_code_get(obj, &val16);
    of_role_request_failed_error_msg_data_get(obj, &octets);

    value = of_role_request_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_role_request_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_role_request_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_role_request_failed_error_msg_OF_VERSION_1_3_check(
    of_role_request_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_role_request_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_role_request_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_role_request_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_role_request_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_role_request_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_set_config
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_3_populate(
    of_set_config_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_set_config_xid_get(obj, &val32);
    of_set_config_flags_get(obj, &val16);
    of_set_config_miss_send_len_get(obj, &val16);

    value = of_set_config_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_set_config
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_set_config_OF_VERSION_1_3_check(
    of_set_config_t *obj, int value)
{

    value = of_set_config_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_set_config_t *dup;

        TEST_ASSERT((dup = of_set_config_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_set_config_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_3_populate(
    of_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_reply_xid_get(obj, &val32);
    of_stats_reply_flags_get(obj, &val16);

    value = of_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_reply_OF_VERSION_1_3_check(
    of_stats_reply_t *obj, int value)
{

    value = of_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_reply_t *dup;

        TEST_ASSERT((dup = of_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_3_populate(
    of_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_stats_request_xid_get(obj, &val32);
    of_stats_request_flags_get(obj, &val16);

    value = of_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_stats_request_OF_VERSION_1_3_check(
    of_stats_request_t *obj, int value)
{

    value = of_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_stats_request_t *dup;

        TEST_ASSERT((dup = of_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_switch_config_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_switch_config_failed_error_msg_OF_VERSION_1_3_populate(
    of_switch_config_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_switch_config_failed_error_msg_xid_get(obj, &val32);
    of_switch_config_failed_error_msg_code_get(obj, &val16);
    of_switch_config_failed_error_msg_data_get(obj, &octets);

    value = of_switch_config_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_switch_config_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_switch_config_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_switch_config_failed_error_msg_OF_VERSION_1_3_check(
    of_switch_config_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_switch_config_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_switch_config_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_switch_config_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_switch_config_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_switch_config_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_features_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_features_failed_error_msg_OF_VERSION_1_3_populate(
    of_table_features_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_table_features_failed_error_msg_xid_get(obj, &val32);
    of_table_features_failed_error_msg_code_get(obj, &val16);
    of_table_features_failed_error_msg_data_get(obj, &octets);

    value = of_table_features_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_table_features_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_table_features_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_features_failed_error_msg_OF_VERSION_1_3_check(
    of_table_features_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_table_features_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_table_features_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_features_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_table_features_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_features_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_features_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_features_stats_reply_OF_VERSION_1_3_populate(
    of_table_features_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_table_features_t *list;

    /* Run thru accessors after new to ensure okay */
    of_table_features_stats_reply_xid_get(obj, &val32);
    of_table_features_stats_reply_flags_get(obj, &val16);
    {
        of_list_table_features_t sub_cls;

        /* Test bind */
        of_table_features_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_table_features_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_table_features_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_table_features_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_features_stats_reply_entries_set(
        obj, list);
    of_list_table_features_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_features_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_features_stats_reply_OF_VERSION_1_3_check(
    of_table_features_stats_reply_t *obj, int value)
{

    value = of_table_features_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_table_features_t *entries_ptr;

        entries_ptr = of_table_features_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_table_features_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_table_features_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_features_stats_reply_t *dup;

        TEST_ASSERT((dup = of_table_features_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_features_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_features_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_features_stats_request_OF_VERSION_1_3_populate(
    of_table_features_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_table_features_t *list;

    /* Run thru accessors after new to ensure okay */
    of_table_features_stats_request_xid_get(obj, &val32);
    of_table_features_stats_request_flags_get(obj, &val16);
    {
        of_list_table_features_t sub_cls;

        /* Test bind */
        of_table_features_stats_request_entries_bind(obj, &sub_cls);
    }

    value = of_table_features_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_table_features_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_table_features_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_features_stats_request_entries_set(
        obj, list);
    of_list_table_features_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_features_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_features_stats_request_OF_VERSION_1_3_check(
    of_table_features_stats_request_t *obj, int value)
{

    value = of_table_features_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_table_features_t *entries_ptr;

        entries_ptr = of_table_features_stats_request_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_table_features_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_table_features_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_features_stats_request_t *dup;

        TEST_ASSERT((dup = of_table_features_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_features_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_3_populate(
    of_table_mod_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_table_mod_xid_get(obj, &val32);
    of_table_mod_table_id_get(obj, &val8);
    of_table_mod_config_get(obj, &val32);

    value = of_table_mod_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_mod_OF_VERSION_1_3_check(
    of_table_mod_t *obj, int value)
{

    value = of_table_mod_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_mod_t *dup;

        TEST_ASSERT((dup = of_table_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_mod_failed_error_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_mod_failed_error_msg_OF_VERSION_1_3_populate(
    of_table_mod_failed_error_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_table_mod_failed_error_msg_xid_get(obj, &val32);
    of_table_mod_failed_error_msg_code_get(obj, &val16);
    of_table_mod_failed_error_msg_data_get(obj, &octets);

    value = of_table_mod_failed_error_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_table_mod_failed_error_msg_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_table_mod_failed_error_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_mod_failed_error_msg_OF_VERSION_1_3_check(
    of_table_mod_failed_error_msg_t *obj, int value)
{
    of_octets_t octets;

    value = of_table_mod_failed_error_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_table_mod_failed_error_msg_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_mod_failed_error_msg_t *dup;

        TEST_ASSERT((dup = of_table_mod_failed_error_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_mod_failed_error_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_3_populate(
    of_table_stats_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_list_table_stats_entry_t *list;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_reply_xid_get(obj, &val32);
    of_table_stats_reply_flags_get(obj, &val16);
    {
        of_list_table_stats_entry_t sub_cls;

        /* Test bind */
        of_table_stats_reply_entries_bind(obj, &sub_cls);
    }

    value = of_table_stats_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_table_stats_entry_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_table_stats_entry_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_stats_reply_entries_set(
        obj, list);
    of_list_table_stats_entry_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_reply_OF_VERSION_1_3_check(
    of_table_stats_reply_t *obj, int value)
{

    value = of_table_stats_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_table_stats_entry_t *entries_ptr;

        entries_ptr = of_table_stats_reply_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_table_stats_entry_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_table_stats_entry_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_reply_t *dup;

        TEST_ASSERT((dup = of_table_stats_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_3_populate(
    of_table_stats_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_request_xid_get(obj, &val32);
    of_table_stats_request_flags_get(obj, &val16);

    value = of_table_stats_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_request_OF_VERSION_1_3_check(
    of_table_stats_request_t *obj, int value)
{

    value = of_table_stats_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_request_t *dup;

        TEST_ASSERT((dup = of_table_stats_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_class_based_ctr_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_class_based_ctr_mod_msg_xid_get(obj, &val32);
    ofdpa_class_based_ctr_mod_msg_experimenter_get(obj, &val32);
    ofdpa_class_based_ctr_mod_msg_subtype_get(obj, &val32);
    ofdpa_class_based_ctr_mod_msg_block_index_get(obj, &val32);
    ofdpa_class_based_ctr_mod_msg_transmit_packets_get(obj, &val64);
    ofdpa_class_based_ctr_mod_msg_receive_packets_get(obj, &val64);
    ofdpa_class_based_ctr_mod_msg_traffic_class_get(obj, &val8);

    value = ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_class_based_ctr_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_3_check(
    ofdpa_class_based_ctr_mod_msg_t *obj, int value)
{

    value = ofdpa_class_based_ctr_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_class_based_ctr_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_class_based_ctr_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_class_based_ctr_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_class_based_ctr_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_class_based_ctr_multipart_reply_xid_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_reply_subtype_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_reply_block_index_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_reply_transmit_packets_get(obj, &val64);
    ofdpa_class_based_ctr_multipart_reply_receive_packets_get(obj, &val64);
    ofdpa_class_based_ctr_multipart_reply_reference_count_get(obj, &val64);
    ofdpa_class_based_ctr_multipart_reply_traffic_class_get(obj, &val8);

    value = ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_class_based_ctr_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_class_based_ctr_multipart_reply_t *obj, int value)
{

    value = ofdpa_class_based_ctr_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_class_based_ctr_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_class_based_ctr_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_class_based_ctr_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_class_based_ctr_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_class_based_ctr_multipart_request_xid_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_request_experimenter_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_request_subtype_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_request_block_index_get(obj, &val32);
    ofdpa_class_based_ctr_multipart_request_traffic_class_get(obj, &val8);

    value = ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_class_based_ctr_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_3_check(
    ofdpa_class_based_ctr_multipart_request_t *obj, int value)
{

    value = ofdpa_class_based_ctr_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_class_based_ctr_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_class_based_ctr_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_class_based_ctr_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_color_based_ctr_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_color_based_ctr_mod_msg_xid_get(obj, &val32);
    ofdpa_color_based_ctr_mod_msg_experimenter_get(obj, &val32);
    ofdpa_color_based_ctr_mod_msg_subtype_get(obj, &val32);
    ofdpa_color_based_ctr_mod_msg_block_index_get(obj, &val32);
    ofdpa_color_based_ctr_mod_msg_transmit_packets_get(obj, &val64);
    ofdpa_color_based_ctr_mod_msg_receive_packets_get(obj, &val64);
    ofdpa_color_based_ctr_mod_msg_color_get(obj, &val8);

    value = ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_color_based_ctr_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_3_check(
    ofdpa_color_based_ctr_mod_msg_t *obj, int value)
{

    value = ofdpa_color_based_ctr_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_color_based_ctr_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_color_based_ctr_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_color_based_ctr_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_color_based_ctr_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_color_based_ctr_multipart_reply_xid_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_reply_subtype_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_reply_block_index_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_reply_transmit_packets_get(obj, &val64);
    ofdpa_color_based_ctr_multipart_reply_receive_packets_get(obj, &val64);
    ofdpa_color_based_ctr_multipart_reply_color_get(obj, &val8);

    value = ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_color_based_ctr_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_color_based_ctr_multipart_reply_t *obj, int value)
{

    value = ofdpa_color_based_ctr_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_color_based_ctr_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_color_based_ctr_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_color_based_ctr_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_color_based_ctr_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_color_based_ctr_multipart_request_xid_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_request_experimenter_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_request_subtype_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_request_block_index_get(obj, &val32);
    ofdpa_color_based_ctr_multipart_request_transmit_packets_get(obj, &val64);
    ofdpa_color_based_ctr_multipart_request_receive_packets_get(obj, &val64);
    ofdpa_color_based_ctr_multipart_request_color_get(obj, &val8);

    value = ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_color_based_ctr_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_3_check(
    ofdpa_color_based_ctr_multipart_request_t *obj, int value)
{

    value = ofdpa_color_based_ctr_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_color_based_ctr_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_color_based_ctr_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_color_based_ctr_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_l2_interface_remark_action_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_l2_interface_remark_action_mod_msg_xid_get(obj, &val32);
    ofdpa_l2_interface_remark_action_mod_msg_experimenter_get(obj, &val32);
    ofdpa_l2_interface_remark_action_mod_msg_subtype_get(obj, &val32);
    ofdpa_l2_interface_remark_action_mod_msg_index_get(obj, &val32);
    ofdpa_l2_interface_remark_action_mod_msg_traffic_class_get(obj, &val8);
    ofdpa_l2_interface_remark_action_mod_msg_color_get(obj, &val8);
    ofdpa_l2_interface_remark_action_mod_msg_dscp_get(obj, &val8);
    ofdpa_l2_interface_remark_action_mod_msg_vlan_pcp_get(obj, &val8);
    ofdpa_l2_interface_remark_action_mod_msg_vlan_dei_get(obj, &val8);

    value = ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_l2_interface_remark_action_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_3_check(
    ofdpa_l2_interface_remark_action_mod_msg_t *obj, int value)
{

    value = ofdpa_l2_interface_remark_action_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_l2_interface_remark_action_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_l2_interface_remark_action_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_l2_interface_remark_action_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_l2_interface_remark_action_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_l2_interface_remark_action_multipart_reply_xid_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_reply_flags_get(obj, &val16);
    ofdpa_l2_interface_remark_action_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_reply_subtype_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_reply_index_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_reply_traffic_class_get(obj, &val8);
    ofdpa_l2_interface_remark_action_multipart_reply_mpls_tc_get(obj, &val8);
    ofdpa_l2_interface_remark_action_multipart_reply_vlan_pcp_get(obj, &val8);
    ofdpa_l2_interface_remark_action_multipart_reply_vlan_dei_get(obj, &val8);

    value = ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_l2_interface_remark_action_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_l2_interface_remark_action_multipart_reply_t *obj, int value)
{

    value = ofdpa_l2_interface_remark_action_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_l2_interface_remark_action_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_l2_interface_remark_action_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_l2_interface_remark_action_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_l2_interface_remark_action_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_l2_interface_remark_action_multipart_request_xid_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_request_flags_get(obj, &val16);
    ofdpa_l2_interface_remark_action_multipart_request_experimenter_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_request_subtype_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_request_index_get(obj, &val32);
    ofdpa_l2_interface_remark_action_multipart_request_traffic_class_get(obj, &val8);
    ofdpa_l2_interface_remark_action_multipart_request_mpls_tc_get(obj, &val8);
    ofdpa_l2_interface_remark_action_multipart_request_vlan_pcp_get(obj, &val8);
    ofdpa_l2_interface_remark_action_multipart_request_vlan_dei_get(obj, &val8);

    value = ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_l2_interface_remark_action_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_3_check(
    ofdpa_l2_interface_remark_action_multipart_request_t *obj, int value)
{

    value = ofdpa_l2_interface_remark_action_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_l2_interface_remark_action_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_l2_interface_remark_action_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_l2_interface_remark_action_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_set_qos_action_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_set_qos_action_mod_msg_xid_get(obj, &val32);
    ofdpa_mpls_set_qos_action_mod_msg_experimenter_get(obj, &val32);
    ofdpa_mpls_set_qos_action_mod_msg_subtype_get(obj, &val32);
    ofdpa_mpls_set_qos_action_mod_msg_qos_index_get(obj, &val8);
    ofdpa_mpls_set_qos_action_mod_msg_mpls_tc_get(obj, &val8);
    ofdpa_mpls_set_qos_action_mod_msg_traffic_class_get(obj, &val8);
    ofdpa_mpls_set_qos_action_mod_msg_color_get(obj, &val8);

    value = ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_set_qos_action_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_3_check(
    ofdpa_mpls_set_qos_action_mod_msg_t *obj, int value)
{

    value = ofdpa_mpls_set_qos_action_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_set_qos_action_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_set_qos_action_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_set_qos_action_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_set_qos_action_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_set_qos_action_multipart_reply_xid_get(obj, &val32);
    ofdpa_mpls_set_qos_action_multipart_reply_flags_get(obj, &val16);
    ofdpa_mpls_set_qos_action_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_mpls_set_qos_action_multipart_reply_subtype_get(obj, &val32);
    ofdpa_mpls_set_qos_action_multipart_reply_qos_index_get(obj, &val8);
    ofdpa_mpls_set_qos_action_multipart_reply_mpls_tc_get(obj, &val8);
    ofdpa_mpls_set_qos_action_multipart_reply_traffic_class_get(obj, &val8);
    ofdpa_mpls_set_qos_action_multipart_reply_color_get(obj, &val8);

    value = ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_set_qos_action_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_mpls_set_qos_action_multipart_reply_t *obj, int value)
{

    value = ofdpa_mpls_set_qos_action_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_set_qos_action_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_set_qos_action_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_set_qos_action_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_set_qos_action_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_set_qos_action_multipart_request_xid_get(obj, &val32);
    ofdpa_mpls_set_qos_action_multipart_request_flags_get(obj, &val16);
    ofdpa_mpls_set_qos_action_multipart_request_experimenter_get(obj, &val32);
    ofdpa_mpls_set_qos_action_multipart_request_subtype_get(obj, &val32);
    ofdpa_mpls_set_qos_action_multipart_request_qos_index_get(obj, &val8);
    ofdpa_mpls_set_qos_action_multipart_request_mpls_tc_get(obj, &val8);
    ofdpa_mpls_set_qos_action_multipart_request_traffic_class_get(obj, &val8);
    ofdpa_mpls_set_qos_action_multipart_request_color_get(obj, &val8);

    value = ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_set_qos_action_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_3_check(
    ofdpa_mpls_set_qos_action_multipart_request_t *obj, int value)
{

    value = ofdpa_mpls_set_qos_action_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_set_qos_action_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_set_qos_action_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_set_qos_action_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_tunnel_label_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_tunnel_label_multipart_reply_xid_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_reply_flags_get(obj, &val16);
    ofdpa_mpls_tunnel_label_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_reply_subtype_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_reply_index_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_reply_traffic_class_get(obj, &val8);
    ofdpa_mpls_tunnel_label_multipart_reply_mpls_tc_get(obj, &val8);
    ofdpa_mpls_tunnel_label_multipart_reply_vlan_pcp_get(obj, &val8);
    ofdpa_mpls_tunnel_label_multipart_reply_vlan_dei_get(obj, &val8);

    value = ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_tunnel_label_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_mpls_tunnel_label_multipart_reply_t *obj, int value)
{

    value = ofdpa_mpls_tunnel_label_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_tunnel_label_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_tunnel_label_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_tunnel_label_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_tunnel_label_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_tunnel_label_multipart_request_xid_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_request_flags_get(obj, &val16);
    ofdpa_mpls_tunnel_label_multipart_request_experimenter_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_request_subtype_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_request_index_get(obj, &val32);
    ofdpa_mpls_tunnel_label_multipart_request_traffic_class_get(obj, &val8);
    ofdpa_mpls_tunnel_label_multipart_request_mpls_tc_get(obj, &val8);
    ofdpa_mpls_tunnel_label_multipart_request_vlan_pcp_get(obj, &val8);
    ofdpa_mpls_tunnel_label_multipart_request_vlan_dei_get(obj, &val8);

    value = ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_tunnel_label_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_3_check(
    ofdpa_mpls_tunnel_label_multipart_request_t *obj, int value)
{

    value = ofdpa_mpls_tunnel_label_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_tunnel_label_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_tunnel_label_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_tunnel_label_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_tunnel_label_remark_action_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_xid_get(obj, &val32);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_experimenter_get(obj, &val32);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_subtype_get(obj, &val32);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_index_get(obj, &val32);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_traffic_class_get(obj, &val8);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_color_get(obj, &val8);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_mpls_tc_get(obj, &val8);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_vlan_pcp_get(obj, &val8);
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_vlan_dei_get(obj, &val8);

    value = ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_tunnel_label_remark_action_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_3_check(
    ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *obj, int value)
{

    value = ofdpa_mpls_tunnel_label_remark_action_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_tunnel_label_remark_action_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_tunnel_label_remark_action_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_vpn_label_remark_action_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_vpn_label_remark_action_mod_msg_xid_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_experimenter_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_subtype_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_index_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_traffic_class_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_color_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_mpls_tc_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_vlan_pcp_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_mod_msg_vlan_dei_get(obj, &val8);

    value = ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_vpn_label_remark_action_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_3_check(
    ofdpa_mpls_vpn_label_remark_action_mod_msg_t *obj, int value)
{

    value = ofdpa_mpls_vpn_label_remark_action_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_vpn_label_remark_action_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_vpn_label_remark_action_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_vpn_label_remark_action_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_vpn_label_remark_action_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_xid_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_flags_get(obj, &val16);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_subtype_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_index_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_traffic_class_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_mpls_tc_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_vlan_pcp_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_vlan_dei_get(obj, &val8);

    value = ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_vpn_label_remark_action_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *obj, int value)
{

    value = ofdpa_mpls_vpn_label_remark_action_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_vpn_label_remark_action_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_vpn_label_remark_action_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_mpls_vpn_label_remark_action_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_mpls_vpn_label_remark_action_multipart_request_xid_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_flags_get(obj, &val16);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_experimenter_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_subtype_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_index_get(obj, &val32);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_traffic_class_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_mpls_tc_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_vlan_pcp_get(obj, &val8);
    ofdpa_mpls_vpn_label_remark_action_multipart_request_vlan_dei_get(obj, &val8);

    value = ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_mpls_vpn_label_remark_action_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_3_check(
    ofdpa_mpls_vpn_label_remark_action_multipart_request_t *obj, int value)
{

    value = ofdpa_mpls_vpn_label_remark_action_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_mpls_vpn_label_remark_action_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_mpls_vpn_label_remark_action_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_mpls_vpn_label_remark_action_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_oam_data_plane_ctr_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_oam_data_plane_ctr_mod_msg_xid_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_mod_msg_experimenter_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_mod_msg_subtype_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_mod_msg_lmep_id_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_mod_msg_transmit_packets_get(obj, &val64);
    ofdpa_oam_data_plane_ctr_mod_msg_receive_packets_get(obj, &val64);
    ofdpa_oam_data_plane_ctr_mod_msg_traffic_class_get(obj, &val8);

    value = ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_oam_data_plane_ctr_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_3_check(
    ofdpa_oam_data_plane_ctr_mod_msg_t *obj, int value)
{

    value = ofdpa_oam_data_plane_ctr_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_oam_data_plane_ctr_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_oam_data_plane_ctr_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_oam_data_plane_ctr_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_oam_data_plane_ctr_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_oam_data_plane_ctr_multipart_reply_xid_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_reply_flags_get(obj, &val16);
    ofdpa_oam_data_plane_ctr_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_reply_subtype_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_reply_lmep_id_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_reply_traffic_class_get(obj, &val8);

    value = ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_oam_data_plane_ctr_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_oam_data_plane_ctr_multipart_reply_t *obj, int value)
{

    value = ofdpa_oam_data_plane_ctr_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_oam_data_plane_ctr_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_oam_data_plane_ctr_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_oam_data_plane_ctr_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_oam_data_plane_ctr_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_oam_data_plane_ctr_multipart_request_xid_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_request_flags_get(obj, &val16);
    ofdpa_oam_data_plane_ctr_multipart_request_experimenter_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_request_subtype_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_request_lmep_id_get(obj, &val32);
    ofdpa_oam_data_plane_ctr_multipart_request_transmit_packets_get(obj, &val64);
    ofdpa_oam_data_plane_ctr_multipart_request_receive_packets_get(obj, &val64);
    ofdpa_oam_data_plane_ctr_multipart_request_reference_count_get(obj, &val64);
    ofdpa_oam_data_plane_ctr_multipart_request_traffic_class_get(obj, &val8);

    value = ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_oam_data_plane_ctr_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_3_check(
    ofdpa_oam_data_plane_ctr_multipart_request_t *obj, int value)
{

    value = ofdpa_oam_data_plane_ctr_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_oam_data_plane_ctr_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_oam_data_plane_ctr_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_oam_data_plane_ctr_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_oam_drop_status_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_oam_drop_status_mod_msg_xid_get(obj, &val32);
    ofdpa_oam_drop_status_mod_msg_experimenter_get(obj, &val32);
    ofdpa_oam_drop_status_mod_msg_subtype_get(obj, &val32);
    ofdpa_oam_drop_status_mod_msg_index_get(obj, &val32);
    ofdpa_oam_drop_status_mod_msg_entry_type_get(obj, &val8);
    ofdpa_oam_drop_status_mod_msg_drop_status_get(obj, &val8);

    value = ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_oam_drop_status_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_3_check(
    ofdpa_oam_drop_status_mod_msg_t *obj, int value)
{

    value = ofdpa_oam_drop_status_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_oam_drop_status_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_oam_drop_status_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_oam_drop_status_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_oam_drop_status_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_oam_drop_status_multipart_reply_xid_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_reply_flags_get(obj, &val16);
    ofdpa_oam_drop_status_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_reply_subtype_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_reply_index_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_reply_entry_type_get(obj, &val8);
    ofdpa_oam_drop_status_multipart_reply_drop_status_get(obj, &val8);

    value = ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_oam_drop_status_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_oam_drop_status_multipart_reply_t *obj, int value)
{

    value = ofdpa_oam_drop_status_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_oam_drop_status_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_oam_drop_status_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_oam_drop_status_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_oam_drop_status_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    ofdpa_oam_drop_status_multipart_request_xid_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_request_flags_get(obj, &val16);
    ofdpa_oam_drop_status_multipart_request_experimenter_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_request_subtype_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_request_index_get(obj, &val32);
    ofdpa_oam_drop_status_multipart_request_entry_type_get(obj, &val8);
    ofdpa_oam_drop_status_multipart_request_drop_status_get(obj, &val8);

    value = ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_oam_drop_status_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_3_check(
    ofdpa_oam_drop_status_multipart_request_t *obj, int value)
{

    value = ofdpa_oam_drop_status_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_oam_drop_status_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_oam_drop_status_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_oam_drop_status_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_port_experimenter_prop_scheduling
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_3_populate(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value)
{
    uint32_t val32;
    of_mac_addr_t mac_addr;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_port_experimenter_prop_scheduling_xid_get(obj, &val32);
    ofdpa_port_experimenter_prop_scheduling_port_no_get(obj, &val32);
    ofdpa_port_experimenter_prop_scheduling_hw_addr_get(obj, &mac_addr);
    ofdpa_port_experimenter_prop_scheduling_config_get(obj, &val32);
    ofdpa_port_experimenter_prop_scheduling_mask_get(obj, &val32);
    ofdpa_port_experimenter_prop_scheduling_pm_type_get(obj, &val16);
    ofdpa_port_experimenter_prop_scheduling_experimenter_get(obj, &val32);
    ofdpa_port_experimenter_prop_scheduling_subtype_get(obj, &val32);
    ofdpa_port_experimenter_prop_scheduling_mode_get(obj, &val16);
    ofdpa_port_experimenter_prop_scheduling_parameter_get(obj, &val16);

    value = ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_port_experimenter_prop_scheduling
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_3_check(
    ofdpa_port_experimenter_prop_scheduling_t *obj, int value)
{

    value = ofdpa_port_experimenter_prop_scheduling_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_port_experimenter_prop_scheduling_t *dup;

        TEST_ASSERT((dup = ofdpa_port_experimenter_prop_scheduling_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_port_experimenter_prop_scheduling_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_queue_desc_prop_congestion_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_queue_desc_prop_congestion_multipart_reply_xid_get(obj, &val32);
    ofdpa_queue_desc_prop_congestion_multipart_reply_flags_get(obj, &val16);
    ofdpa_queue_desc_prop_congestion_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_queue_desc_prop_congestion_multipart_reply_subtype_get(obj, &val32);
    ofdpa_queue_desc_prop_congestion_multipart_reply_mode_get(obj, &val16);

    value = ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_queue_desc_prop_congestion_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_queue_desc_prop_congestion_multipart_reply_t *obj, int value)
{

    value = ofdpa_queue_desc_prop_congestion_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_queue_desc_prop_congestion_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_queue_desc_prop_congestion_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_queue_desc_prop_congestion_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_queue_desc_prop_congestion_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_queue_desc_prop_congestion_multipart_request_xid_get(obj, &val32);
    ofdpa_queue_desc_prop_congestion_multipart_request_flags_get(obj, &val16);
    ofdpa_queue_desc_prop_congestion_multipart_request_experimenter_get(obj, &val32);
    ofdpa_queue_desc_prop_congestion_multipart_request_subtype_get(obj, &val32);
    ofdpa_queue_desc_prop_congestion_multipart_request_mode_get(obj, &val16);

    value = ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_queue_desc_prop_congestion_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_3_check(
    ofdpa_queue_desc_prop_congestion_multipart_request_t *obj, int value)
{

    value = ofdpa_queue_desc_prop_congestion_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_queue_desc_prop_congestion_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_queue_desc_prop_congestion_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_queue_desc_prop_congestion_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_queue_desc_prop_parent_multipart_reply
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_3_populate(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_queue_desc_prop_parent_multipart_reply_xid_get(obj, &val32);
    ofdpa_queue_desc_prop_parent_multipart_reply_flags_get(obj, &val16);
    ofdpa_queue_desc_prop_parent_multipart_reply_experimenter_get(obj, &val32);
    ofdpa_queue_desc_prop_parent_multipart_reply_subtype_get(obj, &val32);
    ofdpa_queue_desc_prop_parent_multipart_reply_scheduler_level_get(obj, &val16);

    value = ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_queue_desc_prop_parent_multipart_reply
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_3_check(
    ofdpa_queue_desc_prop_parent_multipart_reply_t *obj, int value)
{

    value = ofdpa_queue_desc_prop_parent_multipart_reply_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_queue_desc_prop_parent_multipart_reply_t *dup;

        TEST_ASSERT((dup = ofdpa_queue_desc_prop_parent_multipart_reply_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_queue_desc_prop_parent_multipart_reply_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_queue_desc_prop_parent_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_queue_desc_prop_parent_multipart_request_xid_get(obj, &val32);
    ofdpa_queue_desc_prop_parent_multipart_request_flags_get(obj, &val16);
    ofdpa_queue_desc_prop_parent_multipart_request_experimenter_get(obj, &val32);
    ofdpa_queue_desc_prop_parent_multipart_request_subtype_get(obj, &val32);
    ofdpa_queue_desc_prop_parent_multipart_request_scheduler_level_get(obj, &val16);

    value = ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_queue_desc_prop_parent_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_3_check(
    ofdpa_queue_desc_prop_parent_multipart_request_t *obj, int value)
{

    value = ofdpa_queue_desc_prop_parent_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_queue_desc_prop_parent_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_queue_desc_prop_parent_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_queue_desc_prop_parent_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_queue_mod
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_mod_OF_VERSION_1_3_populate(
    ofdpa_queue_mod_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    ofdpa_queue_mod_xid_get(obj, &val32);
    ofdpa_queue_mod_experimenter_get(obj, &val32);
    ofdpa_queue_mod_subtype_get(obj, &val32);
    ofdpa_queue_mod_port_no_get(obj, &val32);
    ofdpa_queue_mod_queue_id_get(obj, &val32);

    value = ofdpa_queue_mod_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_queue_mod
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_queue_mod_OF_VERSION_1_3_check(
    ofdpa_queue_mod_t *obj, int value)
{

    value = ofdpa_queue_mod_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_queue_mod_t *dup;

        TEST_ASSERT((dup = ofdpa_queue_mod_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_queue_mod_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_scheduler_node_mod_msg
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_scheduler_node_mod_msg_OF_VERSION_1_3_populate(
    ofdpa_scheduler_node_mod_msg_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_scheduler_node_mod_msg_xid_get(obj, &val32);
    ofdpa_scheduler_node_mod_msg_experimenter_get(obj, &val32);
    ofdpa_scheduler_node_mod_msg_subtype_get(obj, &val32);
    ofdpa_scheduler_node_mod_msg_level_get(obj, &val16);
    ofdpa_scheduler_node_mod_msg_port_get(obj, &val32);
    ofdpa_scheduler_node_mod_msg_max_rate_get(obj, &val16);
    ofdpa_scheduler_node_mod_msg_min_rate_get(obj, &val16);
    ofdpa_scheduler_node_mod_msg_mode_get(obj, &val16);
    ofdpa_scheduler_node_mod_msg_weight_get(obj, &val16);

    value = ofdpa_scheduler_node_mod_msg_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_scheduler_node_mod_msg
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_scheduler_node_mod_msg_OF_VERSION_1_3_check(
    ofdpa_scheduler_node_mod_msg_t *obj, int value)
{

    value = ofdpa_scheduler_node_mod_msg_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_scheduler_node_mod_msg_t *dup;

        TEST_ASSERT((dup = ofdpa_scheduler_node_mod_msg_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_scheduler_node_mod_msg_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_scheduler_node_multipart_request
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_scheduler_node_multipart_request_OF_VERSION_1_3_populate(
    ofdpa_scheduler_node_multipart_request_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_scheduler_node_multipart_request_xid_get(obj, &val32);
    ofdpa_scheduler_node_multipart_request_flags_get(obj, &val16);
    ofdpa_scheduler_node_multipart_request_experimenter_get(obj, &val32);
    ofdpa_scheduler_node_multipart_request_subtype_get(obj, &val32);
    ofdpa_scheduler_node_multipart_request_port_get(obj, &val32);
    ofdpa_scheduler_node_multipart_request_level_get(obj, &val16);
    ofdpa_scheduler_node_multipart_request_max_rate_get(obj, &val16);
    ofdpa_scheduler_node_multipart_request_min_rate_get(obj, &val16);
    ofdpa_scheduler_node_multipart_request_mode_get(obj, &val16);
    ofdpa_scheduler_node_multipart_request_weight_get(obj, &val16);

    value = ofdpa_scheduler_node_multipart_request_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_scheduler_node_multipart_request
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_scheduler_node_multipart_request_OF_VERSION_1_3_check(
    ofdpa_scheduler_node_multipart_request_t *obj, int value)
{

    value = ofdpa_scheduler_node_multipart_request_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_scheduler_node_multipart_request_t *dup;

        TEST_ASSERT((dup = ofdpa_scheduler_node_multipart_request_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_scheduler_node_multipart_request_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofdpa_scheduler_node_multipart_response
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofdpa_scheduler_node_multipart_response_OF_VERSION_1_3_populate(
    ofdpa_scheduler_node_multipart_response_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    ofdpa_scheduler_node_multipart_response_xid_get(obj, &val32);
    ofdpa_scheduler_node_multipart_response_flags_get(obj, &val16);
    ofdpa_scheduler_node_multipart_response_experimenter_get(obj, &val32);
    ofdpa_scheduler_node_multipart_response_subtype_get(obj, &val32);
    ofdpa_scheduler_node_multipart_response_port_get(obj, &val32);
    ofdpa_scheduler_node_multipart_response_level_get(obj, &val16);
    ofdpa_scheduler_node_multipart_response_max_rate_get(obj, &val16);
    ofdpa_scheduler_node_multipart_response_min_rate_get(obj, &val16);
    ofdpa_scheduler_node_multipart_response_mode_get(obj, &val16);
    ofdpa_scheduler_node_multipart_response_weight_get(obj, &val16);

    value = ofdpa_scheduler_node_multipart_response_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofdpa_scheduler_node_multipart_response
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofdpa_scheduler_node_multipart_response_OF_VERSION_1_3_check(
    ofdpa_scheduler_node_multipart_response_t *obj, int value)
{

    value = ofdpa_scheduler_node_multipart_response_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofdpa_scheduler_node_multipart_response_t *dup;

        TEST_ASSERT((dup = ofdpa_scheduler_node_multipart_response_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofdpa_scheduler_node_multipart_response_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_3_populate(
    of_action_bsn_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_experimenter_get(obj, &val32);
    of_action_bsn_subtype_get(obj, &val32);

    value = of_action_bsn_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_OF_VERSION_1_3_check(
    of_action_bsn_t *obj, int value)
{

    value = of_action_bsn_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_t *dup;

        TEST_ASSERT((dup = of_action_bsn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_mirror
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_3_populate(
    of_action_bsn_mirror_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_mirror_experimenter_get(obj, &val32);
    of_action_bsn_mirror_subtype_get(obj, &val32);
    of_action_bsn_mirror_dest_port_get(obj, &val32);
    of_action_bsn_mirror_vlan_tag_get(obj, &val32);
    of_action_bsn_mirror_copy_stage_get(obj, &val8);

    value = of_action_bsn_mirror_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_mirror
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_mirror_OF_VERSION_1_3_check(
    of_action_bsn_mirror_t *obj, int value)
{

    value = of_action_bsn_mirror_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_mirror_t *dup;

        TEST_ASSERT((dup = of_action_bsn_mirror_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_mirror_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_bsn_set_tunnel_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_bsn_set_tunnel_dst_experimenter_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_subtype_get(obj, &val32);
    of_action_bsn_set_tunnel_dst_dst_get(obj, &val32);

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_bsn_set_tunnel_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
    of_action_bsn_set_tunnel_dst_t *obj, int value)
{

    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_bsn_set_tunnel_dst_t *dup;

        TEST_ASSERT((dup = of_action_bsn_set_tunnel_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_bsn_set_tunnel_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_copy_ttl_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_in_OF_VERSION_1_3_populate(
    of_action_copy_ttl_in_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_copy_ttl_in_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_copy_ttl_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_in_OF_VERSION_1_3_check(
    of_action_copy_ttl_in_t *obj, int value)
{

    value = of_action_copy_ttl_in_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_copy_ttl_in_t *dup;

        TEST_ASSERT((dup = of_action_copy_ttl_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_copy_ttl_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_copy_ttl_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_out_OF_VERSION_1_3_populate(
    of_action_copy_ttl_out_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_copy_ttl_out_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_copy_ttl_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_copy_ttl_out_OF_VERSION_1_3_check(
    of_action_copy_ttl_out_t *obj, int value)
{

    value = of_action_copy_ttl_out_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_copy_ttl_out_t *dup;

        TEST_ASSERT((dup = of_action_copy_ttl_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_copy_ttl_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_dec_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_dec_mpls_ttl_OF_VERSION_1_3_populate(
    of_action_dec_mpls_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_dec_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_dec_mpls_ttl_OF_VERSION_1_3_check(
    of_action_dec_mpls_ttl_t *obj, int value)
{

    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_dec_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_dec_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_dec_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_dec_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_dec_nw_ttl_OF_VERSION_1_3_populate(
    of_action_dec_nw_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_dec_nw_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_dec_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_dec_nw_ttl_OF_VERSION_1_3_check(
    of_action_dec_nw_ttl_t *obj, int value)
{

    value = of_action_dec_nw_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_dec_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_dec_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_dec_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_3_populate(
    of_action_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_action_experimenter_experimenter_get(obj, &val32);
    of_action_experimenter_data_get(obj, &octets);

    value = of_action_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_action_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_experimenter_OF_VERSION_1_3_check(
    of_action_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_action_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_action_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_experimenter_t *dup;

        TEST_ASSERT((dup = of_action_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_group
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_group_OF_VERSION_1_3_populate(
    of_action_group_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_group_group_id_get(obj, &val32);

    value = of_action_group_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_group
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_group_OF_VERSION_1_3_check(
    of_action_group_t *obj, int value)
{

    value = of_action_group_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_group_t *dup;

        TEST_ASSERT((dup = of_action_group_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_group_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_3_populate(
    of_action_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_header_OF_VERSION_1_3_check(
    of_action_header_t *obj, int value)
{

    value = of_action_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_header_t *dup;

        TEST_ASSERT((dup = of_action_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_bsn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_bsn_OF_VERSION_1_3_populate(
    of_action_id_bsn_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_id_bsn_experimenter_get(obj, &val32);
    of_action_id_bsn_subtype_get(obj, &val32);

    value = of_action_id_bsn_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_bsn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_bsn_OF_VERSION_1_3_check(
    of_action_id_bsn_t *obj, int value)
{

    value = of_action_id_bsn_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_bsn_t *dup;

        TEST_ASSERT((dup = of_action_id_bsn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_bsn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_bsn_mirror
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_bsn_mirror_OF_VERSION_1_3_populate(
    of_action_id_bsn_mirror_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_id_bsn_mirror_experimenter_get(obj, &val32);
    of_action_id_bsn_mirror_subtype_get(obj, &val32);

    value = of_action_id_bsn_mirror_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_bsn_mirror
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_bsn_mirror_OF_VERSION_1_3_check(
    of_action_id_bsn_mirror_t *obj, int value)
{

    value = of_action_id_bsn_mirror_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_bsn_mirror_t *dup;

        TEST_ASSERT((dup = of_action_id_bsn_mirror_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_bsn_mirror_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_bsn_set_tunnel_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_id_bsn_set_tunnel_dst_experimenter_get(obj, &val32);
    of_action_id_bsn_set_tunnel_dst_subtype_get(obj, &val32);

    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_bsn_set_tunnel_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
    of_action_id_bsn_set_tunnel_dst_t *obj, int value)
{

    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_bsn_set_tunnel_dst_t *dup;

        TEST_ASSERT((dup = of_action_id_bsn_set_tunnel_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_bsn_set_tunnel_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_copy_ttl_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_copy_ttl_in_OF_VERSION_1_3_populate(
    of_action_id_copy_ttl_in_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_copy_ttl_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_copy_ttl_in_OF_VERSION_1_3_check(
    of_action_id_copy_ttl_in_t *obj, int value)
{

    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_copy_ttl_in_t *dup;

        TEST_ASSERT((dup = of_action_id_copy_ttl_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_copy_ttl_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_copy_ttl_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_copy_ttl_out_OF_VERSION_1_3_populate(
    of_action_id_copy_ttl_out_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_copy_ttl_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_copy_ttl_out_OF_VERSION_1_3_check(
    of_action_id_copy_ttl_out_t *obj, int value)
{

    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_copy_ttl_out_t *dup;

        TEST_ASSERT((dup = of_action_id_copy_ttl_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_copy_ttl_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_dec_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_dec_mpls_ttl_OF_VERSION_1_3_populate(
    of_action_id_dec_mpls_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_dec_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_dec_mpls_ttl_OF_VERSION_1_3_check(
    of_action_id_dec_mpls_ttl_t *obj, int value)
{

    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_dec_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_id_dec_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_dec_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_dec_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_dec_nw_ttl_OF_VERSION_1_3_populate(
    of_action_id_dec_nw_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_dec_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_dec_nw_ttl_OF_VERSION_1_3_check(
    of_action_id_dec_nw_ttl_t *obj, int value)
{

    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_dec_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_id_dec_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_dec_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_experimenter_OF_VERSION_1_3_populate(
    of_action_id_experimenter_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_id_experimenter_experimenter_get(obj, &val32);

    value = of_action_id_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_experimenter_OF_VERSION_1_3_check(
    of_action_id_experimenter_t *obj, int value)
{

    value = of_action_id_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_experimenter_t *dup;

        TEST_ASSERT((dup = of_action_id_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_group
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_group_OF_VERSION_1_3_populate(
    of_action_id_group_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_group_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_group
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_group_OF_VERSION_1_3_check(
    of_action_id_group_t *obj, int value)
{

    value = of_action_id_group_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_group_t *dup;

        TEST_ASSERT((dup = of_action_id_group_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_group_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_header_OF_VERSION_1_3_populate(
    of_action_id_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_header_OF_VERSION_1_3_check(
    of_action_id_header_t *obj, int value)
{

    value = of_action_id_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_header_t *dup;

        TEST_ASSERT((dup = of_action_id_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_nicira
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_nicira_OF_VERSION_1_3_populate(
    of_action_id_nicira_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_nicira_experimenter_get(obj, &val32);
    of_action_id_nicira_subtype_get(obj, &val16);

    value = of_action_id_nicira_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_nicira
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_nicira_OF_VERSION_1_3_check(
    of_action_id_nicira_t *obj, int value)
{

    value = of_action_id_nicira_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_nicira_t *dup;

        TEST_ASSERT((dup = of_action_id_nicira_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_nicira_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_nicira_dec_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_nicira_dec_ttl_OF_VERSION_1_3_populate(
    of_action_id_nicira_dec_ttl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_nicira_dec_ttl_experimenter_get(obj, &val32);
    of_action_id_nicira_dec_ttl_subtype_get(obj, &val16);

    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_nicira_dec_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_nicira_dec_ttl_OF_VERSION_1_3_check(
    of_action_id_nicira_dec_ttl_t *obj, int value)
{

    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_nicira_dec_ttl_t *dup;

        TEST_ASSERT((dup = of_action_id_nicira_dec_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_nicira_dec_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_experimenter_get(obj, &val32);
    of_action_id_ofdpa_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_OF_VERSION_1_3_check(
    of_action_id_ofdpa_t *obj, int value)
{

    value = of_action_id_ofdpa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_check_drop_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_check_drop_status_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_check_drop_status_experimenter_get(obj, &val32);
    of_action_id_ofdpa_check_drop_status_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_check_drop_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_check(
    of_action_id_ofdpa_check_drop_status_t *obj, int value)
{

    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_check_drop_status_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_check_drop_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_check_drop_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_class_based_rx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_class_based_rx_count_experimenter_get(obj, &val32);
    of_action_id_ofdpa_class_based_rx_count_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_class_based_rx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
    of_action_id_ofdpa_class_based_rx_count_t *obj, int value)
{

    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_class_based_rx_count_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_class_based_rx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_class_based_rx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_class_based_tx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_class_based_tx_count_experimenter_get(obj, &val32);
    of_action_id_ofdpa_class_based_tx_count_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_class_based_tx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
    of_action_id_ofdpa_class_based_tx_count_t *obj, int value)
{

    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_class_based_tx_count_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_class_based_tx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_class_based_tx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_color_based_rx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_color_based_rx_count_experimenter_get(obj, &val32);
    of_action_id_ofdpa_color_based_rx_count_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_color_based_rx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
    of_action_id_ofdpa_color_based_rx_count_t *obj, int value)
{

    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_color_based_rx_count_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_color_based_rx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_color_based_rx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_color_based_tx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_color_based_tx_count_experimenter_get(obj, &val32);
    of_action_id_ofdpa_color_based_tx_count_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_color_based_tx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
    of_action_id_ofdpa_color_based_tx_count_t *obj, int value)
{

    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_color_based_tx_count_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_color_based_tx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_color_based_tx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_copy_tc_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_copy_tc_in_experimenter_get(obj, &val32);
    of_action_id_ofdpa_copy_tc_in_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_copy_tc_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
    of_action_id_ofdpa_copy_tc_in_t *obj, int value)
{

    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_copy_tc_in_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_copy_tc_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_copy_tc_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_copy_tc_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_copy_tc_out_experimenter_get(obj, &val32);
    of_action_id_ofdpa_copy_tc_out_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_copy_tc_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
    of_action_id_ofdpa_copy_tc_out_t *obj, int value)
{

    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_copy_tc_out_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_copy_tc_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_copy_tc_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_dec_ttl_mtu
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_dec_ttl_mtu_experimenter_get(obj, &val32);
    of_action_id_ofdpa_dec_ttl_mtu_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_dec_ttl_mtu
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
    of_action_id_ofdpa_dec_ttl_mtu_t *obj, int value)
{

    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_dec_ttl_mtu_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_dec_ttl_mtu_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_dec_ttl_mtu_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_oam_lm_rx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_oam_lm_rx_count_experimenter_get(obj, &val32);
    of_action_id_ofdpa_oam_lm_rx_count_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_oam_lm_rx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
    of_action_id_ofdpa_oam_lm_rx_count_t *obj, int value)
{

    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_oam_lm_rx_count_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_oam_lm_rx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_oam_lm_rx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_oam_lm_tx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_oam_lm_tx_count_experimenter_get(obj, &val32);
    of_action_id_ofdpa_oam_lm_tx_count_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_oam_lm_tx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
    of_action_id_ofdpa_oam_lm_tx_count_t *obj, int value)
{

    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_oam_lm_tx_count_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_oam_lm_tx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_oam_lm_tx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_pop_cw
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_pop_cw_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_pop_cw_experimenter_get(obj, &val32);
    of_action_id_ofdpa_pop_cw_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_pop_cw
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_check(
    of_action_id_ofdpa_pop_cw_t *obj, int value)
{

    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_pop_cw_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_pop_cw_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_pop_cw_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_pop_l2hdr
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_pop_l2hdr_experimenter_get(obj, &val32);
    of_action_id_ofdpa_pop_l2hdr_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_pop_l2hdr
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
    of_action_id_ofdpa_pop_l2hdr_t *obj, int value)
{

    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_pop_l2hdr_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_pop_l2hdr_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_pop_l2hdr_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_push_cw
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_push_cw_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_push_cw_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_push_cw_experimenter_get(obj, &val32);
    of_action_id_ofdpa_push_cw_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_push_cw
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_push_cw_OF_VERSION_1_3_check(
    of_action_id_ofdpa_push_cw_t *obj, int value)
{

    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_push_cw_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_push_cw_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_push_cw_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_push_l2hdr
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_push_l2hdr_experimenter_get(obj, &val32);
    of_action_id_ofdpa_push_l2hdr_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_push_l2hdr
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
    of_action_id_ofdpa_push_l2hdr_t *obj, int value)
{

    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_push_l2hdr_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_push_l2hdr_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_push_l2hdr_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_counter_fields
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_counter_fields_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_counter_fields_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_counter_fields
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_counter_fields_t *obj, int value)
{

    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_counter_fields_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_counter_fields_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_counter_fields_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_dscp_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_dscp_from_table_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_dscp_from_table_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_dscp_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_dscp_from_table_t *obj, int value)
{

    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_dscp_from_table_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_dscp_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_dscp_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_mpls_pcpdei_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_mpls_pcpdei_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value)
{

    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_mpls_pcpdei_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value)
{

    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_mpls_qos_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_mpls_qos_from_table_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_mpls_qos_from_table_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_mpls_qos_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_mpls_qos_from_table_t *obj, int value)
{

    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_mpls_qos_from_table_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_mpls_qos_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_mpls_qos_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value)
{

    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value)
{

    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_ofdpa_set_pcpdei_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_id_ofdpa_set_pcpdei_from_table_experimenter_get(obj, &val32);
    of_action_id_ofdpa_set_pcpdei_from_table_subtype_get(obj, &val16);

    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_ofdpa_set_pcpdei_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
    of_action_id_ofdpa_set_pcpdei_from_table_t *obj, int value)
{

    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_ofdpa_set_pcpdei_from_table_t *dup;

        TEST_ASSERT((dup = of_action_id_ofdpa_set_pcpdei_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_ofdpa_set_pcpdei_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_output
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_output_OF_VERSION_1_3_populate(
    of_action_id_output_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_output_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_output
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_output_OF_VERSION_1_3_check(
    of_action_id_output_t *obj, int value)
{

    value = of_action_id_output_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_output_t *dup;

        TEST_ASSERT((dup = of_action_id_output_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_output_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_pop_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_pop_mpls_OF_VERSION_1_3_populate(
    of_action_id_pop_mpls_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_pop_mpls_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_pop_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_pop_mpls_OF_VERSION_1_3_check(
    of_action_id_pop_mpls_t *obj, int value)
{

    value = of_action_id_pop_mpls_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_pop_mpls_t *dup;

        TEST_ASSERT((dup = of_action_id_pop_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_pop_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_pop_pbb
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_pop_pbb_OF_VERSION_1_3_populate(
    of_action_id_pop_pbb_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_pop_pbb_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_pop_pbb
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_pop_pbb_OF_VERSION_1_3_check(
    of_action_id_pop_pbb_t *obj, int value)
{

    value = of_action_id_pop_pbb_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_pop_pbb_t *dup;

        TEST_ASSERT((dup = of_action_id_pop_pbb_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_pop_pbb_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_pop_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_pop_vlan_OF_VERSION_1_3_populate(
    of_action_id_pop_vlan_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_pop_vlan_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_pop_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_pop_vlan_OF_VERSION_1_3_check(
    of_action_id_pop_vlan_t *obj, int value)
{

    value = of_action_id_pop_vlan_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_pop_vlan_t *dup;

        TEST_ASSERT((dup = of_action_id_pop_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_pop_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_push_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_push_mpls_OF_VERSION_1_3_populate(
    of_action_id_push_mpls_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_push_mpls_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_push_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_push_mpls_OF_VERSION_1_3_check(
    of_action_id_push_mpls_t *obj, int value)
{

    value = of_action_id_push_mpls_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_push_mpls_t *dup;

        TEST_ASSERT((dup = of_action_id_push_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_push_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_push_pbb
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_push_pbb_OF_VERSION_1_3_populate(
    of_action_id_push_pbb_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_push_pbb_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_push_pbb
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_push_pbb_OF_VERSION_1_3_check(
    of_action_id_push_pbb_t *obj, int value)
{

    value = of_action_id_push_pbb_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_push_pbb_t *dup;

        TEST_ASSERT((dup = of_action_id_push_pbb_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_push_pbb_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_push_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_push_vlan_OF_VERSION_1_3_populate(
    of_action_id_push_vlan_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_push_vlan_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_push_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_push_vlan_OF_VERSION_1_3_check(
    of_action_id_push_vlan_t *obj, int value)
{

    value = of_action_id_push_vlan_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_push_vlan_t *dup;

        TEST_ASSERT((dup = of_action_id_push_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_push_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_set_field
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_field_OF_VERSION_1_3_populate(
    of_action_id_set_field_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_set_field_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_set_field
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_field_OF_VERSION_1_3_check(
    of_action_id_set_field_t *obj, int value)
{

    value = of_action_id_set_field_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_set_field_t *dup;

        TEST_ASSERT((dup = of_action_id_set_field_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_set_field_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_set_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_mpls_ttl_OF_VERSION_1_3_populate(
    of_action_id_set_mpls_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_set_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_mpls_ttl_OF_VERSION_1_3_check(
    of_action_id_set_mpls_ttl_t *obj, int value)
{

    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_set_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_id_set_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_set_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_set_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_nw_ttl_OF_VERSION_1_3_populate(
    of_action_id_set_nw_ttl_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_set_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_nw_ttl_OF_VERSION_1_3_check(
    of_action_id_set_nw_ttl_t *obj, int value)
{

    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_set_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_id_set_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_set_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_id_set_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_queue_OF_VERSION_1_3_populate(
    of_action_id_set_queue_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_id_set_queue_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_id_set_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_id_set_queue_OF_VERSION_1_3_check(
    of_action_id_set_queue_t *obj, int value)
{

    value = of_action_id_set_queue_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_id_set_queue_t *dup;

        TEST_ASSERT((dup = of_action_id_set_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_id_set_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_3_populate(
    of_action_nicira_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_experimenter_get(obj, &val32);
    of_action_nicira_subtype_get(obj, &val16);

    value = of_action_nicira_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_OF_VERSION_1_3_check(
    of_action_nicira_t *obj, int value)
{

    value = of_action_nicira_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_nicira_dec_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_3_populate(
    of_action_nicira_dec_ttl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_nicira_dec_ttl_experimenter_get(obj, &val32);
    of_action_nicira_dec_ttl_subtype_get(obj, &val16);

    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_nicira_dec_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_nicira_dec_ttl_OF_VERSION_1_3_check(
    of_action_nicira_dec_ttl_t *obj, int value)
{

    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_nicira_dec_ttl_t *dup;

        TEST_ASSERT((dup = of_action_nicira_dec_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_nicira_dec_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_OF_VERSION_1_3_populate(
    of_action_ofdpa_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_experimenter_get(obj, &val32);
    of_action_ofdpa_subtype_get(obj, &val16);

    value = of_action_ofdpa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_OF_VERSION_1_3_check(
    of_action_ofdpa_t *obj, int value)
{

    value = of_action_ofdpa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_check_drop_status
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
    of_action_ofdpa_check_drop_status_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_check_drop_status_experimenter_get(obj, &val32);
    of_action_ofdpa_check_drop_status_subtype_get(obj, &val16);
    of_action_ofdpa_check_drop_status_drop_type_get(obj, &val16);
    of_action_ofdpa_check_drop_status_drop_index_get(obj, &val32);

    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_check_drop_status
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_check_drop_status_OF_VERSION_1_3_check(
    of_action_ofdpa_check_drop_status_t *obj, int value)
{

    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_check_drop_status_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_check_drop_status_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_check_drop_status_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_class_based_rx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
    of_action_ofdpa_class_based_rx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_class_based_rx_count_experimenter_get(obj, &val32);
    of_action_ofdpa_class_based_rx_count_subtype_get(obj, &val16);
    of_action_ofdpa_class_based_rx_count_traffic_class_get(obj, &val16);
    of_action_ofdpa_class_based_rx_count_index_get(obj, &val32);

    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_class_based_rx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
    of_action_ofdpa_class_based_rx_count_t *obj, int value)
{

    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_class_based_rx_count_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_class_based_rx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_class_based_rx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_class_based_tx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
    of_action_ofdpa_class_based_tx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_class_based_tx_count_experimenter_get(obj, &val32);
    of_action_ofdpa_class_based_tx_count_subtype_get(obj, &val16);
    of_action_ofdpa_class_based_tx_count_traffic_class_get(obj, &val16);
    of_action_ofdpa_class_based_tx_count_index_get(obj, &val32);

    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_class_based_tx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
    of_action_ofdpa_class_based_tx_count_t *obj, int value)
{

    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_class_based_tx_count_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_class_based_tx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_class_based_tx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_color_based_rx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
    of_action_ofdpa_color_based_rx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_color_based_rx_count_experimenter_get(obj, &val32);
    of_action_ofdpa_color_based_rx_count_subtype_get(obj, &val16);
    of_action_ofdpa_color_based_rx_count_color_get(obj, &val16);
    of_action_ofdpa_color_based_rx_count_index_get(obj, &val32);

    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_color_based_rx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
    of_action_ofdpa_color_based_rx_count_t *obj, int value)
{

    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_color_based_rx_count_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_color_based_rx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_color_based_rx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_color_based_tx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
    of_action_ofdpa_color_based_tx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_color_based_tx_count_experimenter_get(obj, &val32);
    of_action_ofdpa_color_based_tx_count_subtype_get(obj, &val16);
    of_action_ofdpa_color_based_tx_count_color_get(obj, &val16);
    of_action_ofdpa_color_based_tx_count_index_get(obj, &val32);

    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_color_based_tx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
    of_action_ofdpa_color_based_tx_count_t *obj, int value)
{

    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_color_based_tx_count_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_color_based_tx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_color_based_tx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_copy_tc_in
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
    of_action_ofdpa_copy_tc_in_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_copy_tc_in_experimenter_get(obj, &val32);
    of_action_ofdpa_copy_tc_in_subtype_get(obj, &val16);

    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_copy_tc_in
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
    of_action_ofdpa_copy_tc_in_t *obj, int value)
{

    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_copy_tc_in_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_copy_tc_in_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_copy_tc_in_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_copy_tc_out
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
    of_action_ofdpa_copy_tc_out_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_copy_tc_out_experimenter_get(obj, &val32);
    of_action_ofdpa_copy_tc_out_subtype_get(obj, &val16);

    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_copy_tc_out
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
    of_action_ofdpa_copy_tc_out_t *obj, int value)
{

    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_copy_tc_out_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_copy_tc_out_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_copy_tc_out_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_dec_ttl_mtu
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_dec_ttl_mtu_experimenter_get(obj, &val32);
    of_action_ofdpa_dec_ttl_mtu_subtype_get(obj, &val16);

    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_dec_ttl_mtu
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
    of_action_ofdpa_dec_ttl_mtu_t *obj, int value)
{

    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_dec_ttl_mtu_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_dec_ttl_mtu_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_dec_ttl_mtu_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_oam_lm_rx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_oam_lm_rx_count_experimenter_get(obj, &val32);
    of_action_ofdpa_oam_lm_rx_count_subtype_get(obj, &val16);
    of_action_ofdpa_oam_lm_rx_count_traffic_class_get(obj, &val16);
    of_action_ofdpa_oam_lm_rx_count_lmep_id_get(obj, &val32);

    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_oam_lm_rx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
    of_action_ofdpa_oam_lm_rx_count_t *obj, int value)
{

    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_oam_lm_rx_count_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_oam_lm_rx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_oam_lm_rx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_oam_lm_tx_count
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_oam_lm_tx_count_experimenter_get(obj, &val32);
    of_action_ofdpa_oam_lm_tx_count_subtype_get(obj, &val16);
    of_action_ofdpa_oam_lm_tx_count_traffic_class_get(obj, &val16);
    of_action_ofdpa_oam_lm_tx_count_lmep_id_get(obj, &val32);

    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_oam_lm_tx_count
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
    of_action_ofdpa_oam_lm_tx_count_t *obj, int value)
{

    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_oam_lm_tx_count_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_oam_lm_tx_count_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_oam_lm_tx_count_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_pop_cw
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_pop_cw_OF_VERSION_1_3_populate(
    of_action_ofdpa_pop_cw_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_pop_cw_experimenter_get(obj, &val32);
    of_action_ofdpa_pop_cw_subtype_get(obj, &val16);

    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_pop_cw
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_pop_cw_OF_VERSION_1_3_check(
    of_action_ofdpa_pop_cw_t *obj, int value)
{

    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_pop_cw_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_pop_cw_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_pop_cw_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_pop_l2hdr
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
    of_action_ofdpa_pop_l2hdr_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_pop_l2hdr_experimenter_get(obj, &val32);
    of_action_ofdpa_pop_l2hdr_subtype_get(obj, &val16);

    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_pop_l2hdr
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
    of_action_ofdpa_pop_l2hdr_t *obj, int value)
{

    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_pop_l2hdr_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_pop_l2hdr_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_pop_l2hdr_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_push_cw
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_push_cw_OF_VERSION_1_3_populate(
    of_action_ofdpa_push_cw_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_push_cw_experimenter_get(obj, &val32);
    of_action_ofdpa_push_cw_subtype_get(obj, &val16);

    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_push_cw
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_push_cw_OF_VERSION_1_3_check(
    of_action_ofdpa_push_cw_t *obj, int value)
{

    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_push_cw_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_push_cw_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_push_cw_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_push_l2hdr
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
    of_action_ofdpa_push_l2hdr_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_push_l2hdr_experimenter_get(obj, &val32);
    of_action_ofdpa_push_l2hdr_subtype_get(obj, &val16);

    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_push_l2hdr
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
    of_action_ofdpa_push_l2hdr_t *obj, int value)
{

    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_push_l2hdr_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_push_l2hdr_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_push_l2hdr_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_counter_fields
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_counter_fields_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_counter_fields_experimenter_get(obj, &val32);
    of_action_ofdpa_set_counter_fields_subtype_get(obj, &val16);
    of_action_ofdpa_set_counter_fields_traffic_class_get(obj, &val16);
    of_action_ofdpa_set_counter_fields_lmep_id_get(obj, &val32);

    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_counter_fields
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
    of_action_ofdpa_set_counter_fields_t *obj, int value)
{

    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_counter_fields_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_counter_fields_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_counter_fields_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_dscp_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_dscp_from_table_experimenter_get(obj, &val32);
    of_action_ofdpa_set_dscp_from_table_subtype_get(obj, &val16);
    of_action_ofdpa_set_dscp_from_table_qos_index_get(obj, &val8);
    of_action_ofdpa_set_dscp_from_table_traffic_class_get(obj, &val8);
    of_action_ofdpa_set_dscp_from_table_color_get(obj, &val8);

    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_dscp_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
    of_action_ofdpa_set_dscp_from_table_t *obj, int value)
{

    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_dscp_from_table_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_dscp_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_dscp_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_mpls_pcpdei_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_mpls_pcpdei_from_table_experimenter_get(obj, &val32);
    of_action_ofdpa_set_mpls_pcpdei_from_table_subtype_get(obj, &val16);
    of_action_ofdpa_set_mpls_pcpdei_from_table_qos_index_get(obj, &val8);
    of_action_ofdpa_set_mpls_pcpdei_from_table_traffic_class_get(obj, &val8);
    of_action_ofdpa_set_mpls_pcpdei_from_table_color_get(obj, &val8);
    of_action_ofdpa_set_mpls_pcpdei_from_table_pcp_dei_get(obj, &val8);

    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_mpls_pcpdei_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *obj, int value)
{

    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_mpls_pcpdei_from_table_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_mpls_pcpdei_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_mpls_pcpdei_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_experimenter_get(obj, &val32);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_subtype_get(obj, &val16);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_qos_index_get(obj, &val8);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_traffic_class_get(obj, &val8);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_color_get(obj, &val8);
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_pcp_dei_get(obj, &val8);

    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *obj, int value)
{

    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_mpls_qos_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_mpls_qos_from_table_experimenter_get(obj, &val32);
    of_action_ofdpa_set_mpls_qos_from_table_subtype_get(obj, &val16);
    of_action_ofdpa_set_mpls_qos_from_table_qos_index_get(obj, &val8);
    of_action_ofdpa_set_mpls_qos_from_table_mpls_tc_get(obj, &val8);

    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_mpls_qos_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
    of_action_ofdpa_set_mpls_qos_from_table_t *obj, int value)
{

    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_mpls_qos_from_table_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_mpls_qos_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_mpls_qos_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_mpls_tc_from_tunnel_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_experimenter_get(obj, &val32);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_subtype_get(obj, &val16);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_qos_index_get(obj, &val8);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_traffic_class_get(obj, &val8);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_color_get(obj, &val8);
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_mpls_tc_get(obj, &val8);

    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_mpls_tc_from_tunnel_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *obj, int value)
{

    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_mpls_tc_from_tunnel_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_mpls_tc_from_vpn_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_mpls_tc_from_vpn_table_experimenter_get(obj, &val32);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_subtype_get(obj, &val16);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_qos_index_get(obj, &val8);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_traffic_class_get(obj, &val8);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_color_get(obj, &val8);
    of_action_ofdpa_set_mpls_tc_from_vpn_table_mpls_tc_get(obj, &val8);

    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_mpls_tc_from_vpn_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *obj, int value)
{

    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_mpls_tc_from_vpn_table_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_mpls_tc_from_vpn_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_mpls_tc_from_vpn_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_ofdpa_set_pcpdei_from_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_ofdpa_set_pcpdei_from_table_experimenter_get(obj, &val32);
    of_action_ofdpa_set_pcpdei_from_table_subtype_get(obj, &val16);
    of_action_ofdpa_set_pcpdei_from_table_qos_index_get(obj, &val8);
    of_action_ofdpa_set_pcpdei_from_table_traffic_class_get(obj, &val8);
    of_action_ofdpa_set_pcpdei_from_table_color_get(obj, &val8);

    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_ofdpa_set_pcpdei_from_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
    of_action_ofdpa_set_pcpdei_from_table_t *obj, int value)
{

    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_ofdpa_set_pcpdei_from_table_t *dup;

        TEST_ASSERT((dup = of_action_ofdpa_set_pcpdei_from_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_ofdpa_set_pcpdei_from_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_output
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_3_populate(
    of_action_output_t *obj, int value)
{
    of_port_no_t port_no;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_output_port_get(obj, &port_no);
    of_action_output_max_len_get(obj, &val16);

    value = of_action_output_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_output
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_output_OF_VERSION_1_3_check(
    of_action_output_t *obj, int value)
{

    value = of_action_output_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_output_t *dup;

        TEST_ASSERT((dup = of_action_output_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_output_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_pop_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_pop_mpls_OF_VERSION_1_3_populate(
    of_action_pop_mpls_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_pop_mpls_ethertype_get(obj, &val16);

    value = of_action_pop_mpls_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_pop_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_pop_mpls_OF_VERSION_1_3_check(
    of_action_pop_mpls_t *obj, int value)
{

    value = of_action_pop_mpls_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_pop_mpls_t *dup;

        TEST_ASSERT((dup = of_action_pop_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_pop_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_pop_pbb
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_pop_pbb_OF_VERSION_1_3_populate(
    of_action_pop_pbb_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_pop_pbb_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_pop_pbb
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_pop_pbb_OF_VERSION_1_3_check(
    of_action_pop_pbb_t *obj, int value)
{

    value = of_action_pop_pbb_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_pop_pbb_t *dup;

        TEST_ASSERT((dup = of_action_pop_pbb_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_pop_pbb_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_pop_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_pop_vlan_OF_VERSION_1_3_populate(
    of_action_pop_vlan_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_action_pop_vlan_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_pop_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_pop_vlan_OF_VERSION_1_3_check(
    of_action_pop_vlan_t *obj, int value)
{

    value = of_action_pop_vlan_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_pop_vlan_t *dup;

        TEST_ASSERT((dup = of_action_pop_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_pop_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_push_mpls
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_push_mpls_OF_VERSION_1_3_populate(
    of_action_push_mpls_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_push_mpls_ethertype_get(obj, &val16);

    value = of_action_push_mpls_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_push_mpls
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_push_mpls_OF_VERSION_1_3_check(
    of_action_push_mpls_t *obj, int value)
{

    value = of_action_push_mpls_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_push_mpls_t *dup;

        TEST_ASSERT((dup = of_action_push_mpls_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_push_mpls_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_push_pbb
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_push_pbb_OF_VERSION_1_3_populate(
    of_action_push_pbb_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_push_pbb_ethertype_get(obj, &val16);

    value = of_action_push_pbb_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_push_pbb
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_push_pbb_OF_VERSION_1_3_check(
    of_action_push_pbb_t *obj, int value)
{

    value = of_action_push_pbb_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_push_pbb_t *dup;

        TEST_ASSERT((dup = of_action_push_pbb_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_push_pbb_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_push_vlan
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_push_vlan_OF_VERSION_1_3_populate(
    of_action_push_vlan_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_action_push_vlan_ethertype_get(obj, &val16);

    value = of_action_push_vlan_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_push_vlan
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_push_vlan_OF_VERSION_1_3_check(
    of_action_push_vlan_t *obj, int value)
{

    value = of_action_push_vlan_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_push_vlan_t *dup;

        TEST_ASSERT((dup = of_action_push_vlan_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_push_vlan_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_field
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_field_OF_VERSION_1_3_populate(
    of_action_set_field_t *obj, int value)
{
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_action_set_field_field_get(obj, &octets);

    value = of_action_set_field_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_action_set_field_field_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_action_set_field
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_field_OF_VERSION_1_3_check(
    of_action_set_field_t *obj, int value)
{
    of_octets_t octets;

    value = of_action_set_field_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_action_set_field_field_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_field_t *dup;

        TEST_ASSERT((dup = of_action_set_field_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_field_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_ttl_OF_VERSION_1_3_populate(
    of_action_set_mpls_ttl_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_mpls_ttl_mpls_ttl_get(obj, &val8);

    value = of_action_set_mpls_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_mpls_ttl_OF_VERSION_1_3_check(
    of_action_set_mpls_ttl_t *obj, int value)
{

    value = of_action_set_mpls_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_action_set_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_nw_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ttl_OF_VERSION_1_3_populate(
    of_action_set_nw_ttl_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_action_set_nw_ttl_nw_ttl_get(obj, &val8);

    value = of_action_set_nw_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_nw_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_nw_ttl_OF_VERSION_1_3_check(
    of_action_set_nw_ttl_t *obj, int value)
{

    value = of_action_set_nw_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_nw_ttl_t *dup;

        TEST_ASSERT((dup = of_action_set_nw_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_nw_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_action_set_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_action_set_queue_OF_VERSION_1_3_populate(
    of_action_set_queue_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_action_set_queue_queue_id_get(obj, &val32);

    value = of_action_set_queue_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_action_set_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_action_set_queue_OF_VERSION_1_3_check(
    of_action_set_queue_t *obj, int value)
{

    value = of_action_set_queue_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_action_set_queue_t *dup;

        TEST_ASSERT((dup = of_action_set_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_action_set_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_controller_connection
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_controller_connection_OF_VERSION_1_3_populate(
    of_bsn_controller_connection_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    of_desc_str_t desc_str;

    /* Run thru accessors after new to ensure okay */
    of_bsn_controller_connection_state_get(obj, &val8);
    of_bsn_controller_connection_auxiliary_id_get(obj, &val8);
    of_bsn_controller_connection_role_get(obj, &val32);
    of_bsn_controller_connection_uri_get(obj, &desc_str);

    value = of_bsn_controller_connection_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_controller_connection
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_controller_connection_OF_VERSION_1_3_check(
    of_bsn_controller_connection_t *obj, int value)
{

    value = of_bsn_controller_connection_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_controller_connection_t *dup;

        TEST_ASSERT((dup = of_bsn_controller_connection_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_controller_connection_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_debug_counter_desc_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value)
{
    uint64_t val64;
    of_str64_t str64;
    of_desc_str_t desc_str;

    /* Run thru accessors after new to ensure okay */
    of_bsn_debug_counter_desc_stats_entry_counter_id_get(obj, &val64);
    of_bsn_debug_counter_desc_stats_entry_name_get(obj, &str64);
    of_bsn_debug_counter_desc_stats_entry_description_get(obj, &desc_str);

    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_debug_counter_desc_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check(
    of_bsn_debug_counter_desc_stats_entry_t *obj, int value)
{

    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_debug_counter_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_debug_counter_desc_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_debug_counter_desc_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_debug_counter_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_debug_counter_stats_entry_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_debug_counter_stats_entry_counter_id_get(obj, &val64);
    of_bsn_debug_counter_stats_entry_value_get(obj, &val64);

    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_debug_counter_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check(
    of_bsn_debug_counter_stats_entry_t *obj, int value)
{

    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_debug_counter_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_debug_counter_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_debug_counter_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_flow_checksum_bucket_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_flow_checksum_bucket_stats_entry_checksum_get(obj, &val64);

    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_flow_checksum_bucket_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check(
    of_bsn_flow_checksum_bucket_stats_entry_t *obj, int value)
{

    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_flow_checksum_bucket_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_flow_checksum_bucket_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_flow_checksum_bucket_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_bucket_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value)
{
    of_checksum_128_t checksum_128;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_bucket_stats_entry_checksum_get(obj, &checksum_128);

    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_bucket_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check(
    of_bsn_gentable_bucket_stats_entry_t *obj, int value)
{

    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_bucket_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_bucket_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_bucket_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_desc_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_gentable_desc_stats_entry_t *obj, int value)
{
    uint16_t val16;
    of_table_name_t table_name;
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_desc_stats_entry_table_id_get(obj, &val16);
    of_bsn_gentable_desc_stats_entry_name_get(obj, &table_name);
    of_bsn_gentable_desc_stats_entry_buckets_size_get(obj, &val32);
    of_bsn_gentable_desc_stats_entry_max_entries_get(obj, &val32);

    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_desc_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check(
    of_bsn_gentable_desc_stats_entry_t *obj, int value)
{

    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_desc_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_desc_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_desc_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value)
{
    of_checksum_128_t checksum_128;
    of_list_bsn_tlv_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_entry_desc_stats_entry_checksum_get(obj, &checksum_128);
    {
        of_list_bsn_tlv_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_desc_stats_entry_key_bind(obj, &sub_cls);
    }
    {
        of_list_bsn_tlv_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_desc_stats_entry_value_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_tlv_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_tlv_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_desc_stats_entry_key_set(
        obj, list);
    of_list_bsn_tlv_delete(list);

    list = of_list_bsn_tlv_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_tlv_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_desc_stats_entry_value_set(
        obj, list);
    of_list_bsn_tlv_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_desc_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_desc_stats_entry_t *obj, int value)
{

    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_tlv_t *key_ptr;

        key_ptr = of_bsn_gentable_entry_desc_stats_entry_key_get(obj);
        TEST_ASSERT(key_ptr != NULL);
        value = of_list_bsn_tlv_OF_VERSION_1_3_check(
            key_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_tlv_delete(key_ptr);
    }

    { /* Use get/delete to access on check */
        of_list_bsn_tlv_t *value_ptr;

        value_ptr = of_bsn_gentable_entry_desc_stats_entry_value_get(obj);
        TEST_ASSERT(value_ptr != NULL);
        value = of_list_bsn_tlv_OF_VERSION_1_3_check(
            value_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_tlv_delete(value_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_desc_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_desc_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_entry_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_gentable_entry_stats_entry_t *obj, int value)
{
    of_list_bsn_tlv_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_bsn_tlv_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_stats_entry_key_bind(obj, &sub_cls);
    }
    {
        of_list_bsn_tlv_t sub_cls;

        /* Test bind */
        of_bsn_gentable_entry_stats_entry_stats_bind(obj, &sub_cls);
    }

    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bsn_tlv_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_tlv_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_stats_entry_key_set(
        obj, list);
    of_list_bsn_tlv_delete(list);

    list = of_list_bsn_tlv_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bsn_tlv_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_gentable_entry_stats_entry_stats_set(
        obj, list);
    of_list_bsn_tlv_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_entry_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check(
    of_bsn_gentable_entry_stats_entry_t *obj, int value)
{

    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bsn_tlv_t *key_ptr;

        key_ptr = of_bsn_gentable_entry_stats_entry_key_get(obj);
        TEST_ASSERT(key_ptr != NULL);
        value = of_list_bsn_tlv_OF_VERSION_1_3_check(
            key_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_tlv_delete(key_ptr);
    }

    { /* Use get/delete to access on check */
        of_list_bsn_tlv_t *stats_ptr;

        stats_ptr = of_bsn_gentable_entry_stats_entry_stats_get(obj);
        TEST_ASSERT(stats_ptr != NULL);
        value = of_list_bsn_tlv_OF_VERSION_1_3_check(
            stats_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bsn_tlv_delete(stats_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_entry_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_entry_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_entry_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_gentable_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_gentable_stats_entry_t *obj, int value)
{
    uint16_t val16;
    uint32_t val32;
    of_checksum_128_t checksum_128;

    /* Run thru accessors after new to ensure okay */
    of_bsn_gentable_stats_entry_table_id_get(obj, &val16);
    of_bsn_gentable_stats_entry_entry_count_get(obj, &val32);
    of_bsn_gentable_stats_entry_checksum_get(obj, &checksum_128);

    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_gentable_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_gentable_stats_entry_OF_VERSION_1_3_check(
    of_bsn_gentable_stats_entry_t *obj, int value)
{

    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_gentable_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_gentable_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_gentable_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_interface
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_3_populate(
    of_bsn_interface_t *obj, int value)
{
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_bsn_interface_hw_addr_get(obj, &mac_addr);
    of_bsn_interface_name_get(obj, &port_name);
    of_bsn_interface_ipv4_addr_get(obj, &ipv4);
    of_bsn_interface_ipv4_netmask_get(obj, &ipv4);

    value = of_bsn_interface_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_interface
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_interface_OF_VERSION_1_3_check(
    of_bsn_interface_t *obj, int value)
{

    value = of_bsn_interface_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_bsn_interface_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_interface_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_lacp_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_lacp_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint16_t val16;
    of_mac_addr_t mac_addr;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_bsn_lacp_stats_entry_port_no_get(obj, &port_no);
    of_bsn_lacp_stats_entry_actor_sys_priority_get(obj, &val16);
    of_bsn_lacp_stats_entry_actor_sys_mac_get(obj, &mac_addr);
    of_bsn_lacp_stats_entry_actor_port_priority_get(obj, &val16);
    of_bsn_lacp_stats_entry_actor_port_num_get(obj, &val16);
    of_bsn_lacp_stats_entry_actor_key_get(obj, &val16);
    of_bsn_lacp_stats_entry_convergence_status_get(obj, &val8);
    of_bsn_lacp_stats_entry_partner_sys_priority_get(obj, &val16);
    of_bsn_lacp_stats_entry_partner_sys_mac_get(obj, &mac_addr);
    of_bsn_lacp_stats_entry_partner_port_priority_get(obj, &val16);
    of_bsn_lacp_stats_entry_partner_port_num_get(obj, &val16);
    of_bsn_lacp_stats_entry_partner_key_get(obj, &val16);

    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_lacp_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_lacp_stats_entry_OF_VERSION_1_3_check(
    of_bsn_lacp_stats_entry_t *obj, int value)
{

    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_lacp_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_lacp_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_lacp_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_port_counter_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_port_counter_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    of_list_uint64_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_port_counter_stats_entry_port_no_get(obj, &port_no);
    {
        of_list_uint64_t sub_cls;

        /* Test bind */
        of_bsn_port_counter_stats_entry_values_bind(obj, &sub_cls);
    }

    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint64_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint64_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_port_counter_stats_entry_values_set(
        obj, list);
    of_list_uint64_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_port_counter_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_port_counter_stats_entry_OF_VERSION_1_3_check(
    of_bsn_port_counter_stats_entry_t *obj, int value)
{

    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint64_t *values_ptr;

        values_ptr = of_bsn_port_counter_stats_entry_values_get(obj);
        TEST_ASSERT(values_ptr != NULL);
        value = of_list_uint64_OF_VERSION_1_3_check(
            values_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint64_delete(values_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_port_counter_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_port_counter_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_port_counter_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_switch_pipeline_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value)
{
    of_desc_str_t desc_str;

    /* Run thru accessors after new to ensure okay */
    of_bsn_switch_pipeline_stats_entry_pipeline_get(obj, &desc_str);

    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_switch_pipeline_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check(
    of_bsn_switch_pipeline_stats_entry_t *obj, int value)
{

    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_switch_pipeline_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_switch_pipeline_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_switch_pipeline_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_table_checksum_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_table_checksum_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_table_checksum_stats_entry_table_id_get(obj, &val8);
    of_bsn_table_checksum_stats_entry_checksum_get(obj, &val64);

    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_table_checksum_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check(
    of_bsn_table_checksum_stats_entry_t *obj, int value)
{

    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_table_checksum_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_table_checksum_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_table_checksum_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_broadcast_query_timeout
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_populate(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_broadcast_query_timeout_value_get(obj, &val32);

    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_broadcast_query_timeout
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_check(
    of_bsn_tlv_broadcast_query_timeout_t *obj, int value)
{

    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_broadcast_query_timeout_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_broadcast_query_timeout_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_broadcast_query_timeout_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_circuit_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_circuit_id_OF_VERSION_1_3_populate(
    of_bsn_tlv_circuit_id_t *obj, int value)
{
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_circuit_id_value_get(obj, &octets);

    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_bsn_tlv_circuit_id_value_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_circuit_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_circuit_id_OF_VERSION_1_3_check(
    of_bsn_tlv_circuit_id_t *obj, int value)
{
    of_octets_t octets;

    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_bsn_tlv_circuit_id_value_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_circuit_id_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_circuit_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_circuit_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_header_OF_VERSION_1_3_populate(
    of_bsn_tlv_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_bsn_tlv_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_header_OF_VERSION_1_3_check(
    of_bsn_tlv_header_t *obj, int value)
{

    value = of_bsn_tlv_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_header_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_idle_notification
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_idle_notification_OF_VERSION_1_3_populate(
    of_bsn_tlv_idle_notification_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_idle_notification
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_idle_notification_OF_VERSION_1_3_check(
    of_bsn_tlv_idle_notification_t *obj, int value)
{

    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_idle_notification_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_idle_notification_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_idle_notification_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_idle_time
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_idle_time_OF_VERSION_1_3_populate(
    of_bsn_tlv_idle_time_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_idle_time_value_get(obj, &val64);

    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_idle_time
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_idle_time_OF_VERSION_1_3_check(
    of_bsn_tlv_idle_time_t *obj, int value)
{

    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_idle_time_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_idle_time_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_idle_time_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_idle_timeout
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_idle_timeout_OF_VERSION_1_3_populate(
    of_bsn_tlv_idle_timeout_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_idle_timeout_value_get(obj, &val32);

    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_idle_timeout
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_idle_timeout_OF_VERSION_1_3_check(
    of_bsn_tlv_idle_timeout_t *obj, int value)
{

    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_idle_timeout_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_idle_timeout_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_idle_timeout_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_ipv4
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_ipv4_OF_VERSION_1_3_populate(
    of_bsn_tlv_ipv4_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_ipv4_value_get(obj, &ipv4);

    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_ipv4
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_ipv4_OF_VERSION_1_3_check(
    of_bsn_tlv_ipv4_t *obj, int value)
{

    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_ipv4_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_ipv4_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_ipv4_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_mac
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_mac_OF_VERSION_1_3_populate(
    of_bsn_tlv_mac_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_mac_value_get(obj, &mac_addr);

    value = of_bsn_tlv_mac_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_mac
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_mac_OF_VERSION_1_3_check(
    of_bsn_tlv_mac_t *obj, int value)
{

    value = of_bsn_tlv_mac_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_mac_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_mac_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_mac_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_miss_packets
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_miss_packets_OF_VERSION_1_3_populate(
    of_bsn_tlv_miss_packets_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_miss_packets_value_get(obj, &val64);

    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_miss_packets
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_miss_packets_OF_VERSION_1_3_check(
    of_bsn_tlv_miss_packets_t *obj, int value)
{

    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_miss_packets_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_miss_packets_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_miss_packets_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_port
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_port_OF_VERSION_1_3_populate(
    of_bsn_tlv_port_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_port_value_get(obj, &port_no);

    value = of_bsn_tlv_port_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_port
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_port_OF_VERSION_1_3_check(
    of_bsn_tlv_port_t *obj, int value)
{

    value = of_bsn_tlv_port_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_port_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_port_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_port_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_reply_packets
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_reply_packets_OF_VERSION_1_3_populate(
    of_bsn_tlv_reply_packets_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_reply_packets_value_get(obj, &val64);

    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_reply_packets
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_reply_packets_OF_VERSION_1_3_check(
    of_bsn_tlv_reply_packets_t *obj, int value)
{

    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_reply_packets_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_reply_packets_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_reply_packets_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_request_packets
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_request_packets_OF_VERSION_1_3_populate(
    of_bsn_tlv_request_packets_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_request_packets_value_get(obj, &val64);

    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_request_packets
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_request_packets_OF_VERSION_1_3_check(
    of_bsn_tlv_request_packets_t *obj, int value)
{

    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_request_packets_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_request_packets_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_request_packets_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_rx_packets
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_rx_packets_OF_VERSION_1_3_populate(
    of_bsn_tlv_rx_packets_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_rx_packets_value_get(obj, &val64);

    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_rx_packets
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_rx_packets_OF_VERSION_1_3_check(
    of_bsn_tlv_rx_packets_t *obj, int value)
{

    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_rx_packets_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_rx_packets_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_rx_packets_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_tx_packets
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_tx_packets_OF_VERSION_1_3_populate(
    of_bsn_tlv_tx_packets_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_tx_packets_value_get(obj, &val64);

    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_tx_packets
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_tx_packets_OF_VERSION_1_3_check(
    of_bsn_tlv_tx_packets_t *obj, int value)
{

    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_tx_packets_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_tx_packets_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_tx_packets_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_udf_anchor
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_anchor_OF_VERSION_1_3_populate(
    of_bsn_tlv_udf_anchor_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_udf_anchor_value_get(obj, &val16);

    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_udf_anchor
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_anchor_OF_VERSION_1_3_check(
    of_bsn_tlv_udf_anchor_t *obj, int value)
{

    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_udf_anchor_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_udf_anchor_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_udf_anchor_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_udf_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_id_OF_VERSION_1_3_populate(
    of_bsn_tlv_udf_id_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_udf_id_value_get(obj, &val16);

    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_udf_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_id_OF_VERSION_1_3_check(
    of_bsn_tlv_udf_id_t *obj, int value)
{

    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_udf_id_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_udf_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_udf_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_udf_length
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_length_OF_VERSION_1_3_populate(
    of_bsn_tlv_udf_length_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_udf_length_value_get(obj, &val16);

    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_udf_length
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_length_OF_VERSION_1_3_check(
    of_bsn_tlv_udf_length_t *obj, int value)
{

    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_udf_length_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_udf_length_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_udf_length_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_udf_offset
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_offset_OF_VERSION_1_3_populate(
    of_bsn_tlv_udf_offset_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_udf_offset_value_get(obj, &val16);

    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_udf_offset
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_udf_offset_OF_VERSION_1_3_check(
    of_bsn_tlv_udf_offset_t *obj, int value)
{

    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_udf_offset_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_udf_offset_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_udf_offset_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_unicast_query_timeout
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_populate(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_unicast_query_timeout_value_get(obj, &val32);

    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_unicast_query_timeout
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_check(
    of_bsn_tlv_unicast_query_timeout_t *obj, int value)
{

    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_unicast_query_timeout_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_unicast_query_timeout_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_unicast_query_timeout_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_tlv_vlan_vid
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_vlan_vid_OF_VERSION_1_3_populate(
    of_bsn_tlv_vlan_vid_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_bsn_tlv_vlan_vid_value_get(obj, &val16);

    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_tlv_vlan_vid
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_tlv_vlan_vid_OF_VERSION_1_3_check(
    of_bsn_tlv_vlan_vid_t *obj, int value)
{

    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_tlv_vlan_vid_t *dup;

        TEST_ASSERT((dup = of_bsn_tlv_vlan_vid_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_tlv_vlan_vid_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vlan_counter_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate(
    of_bsn_vlan_counter_stats_entry_t *obj, int value)
{
    uint16_t val16;
    of_list_uint64_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bsn_vlan_counter_stats_entry_vlan_vid_get(obj, &val16);
    {
        of_list_uint64_t sub_cls;

        /* Test bind */
        of_bsn_vlan_counter_stats_entry_values_bind(obj, &sub_cls);
    }

    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint64_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint64_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bsn_vlan_counter_stats_entry_values_set(
        obj, list);
    of_list_uint64_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vlan_counter_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check(
    of_bsn_vlan_counter_stats_entry_t *obj, int value)
{

    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint64_t *values_ptr;

        values_ptr = of_bsn_vlan_counter_stats_entry_values_get(obj);
        TEST_ASSERT(values_ptr != NULL);
        value = of_list_uint64_OF_VERSION_1_3_check(
            values_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint64_delete(values_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vlan_counter_stats_entry_t *dup;

        TEST_ASSERT((dup = of_bsn_vlan_counter_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vlan_counter_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_3_populate(
    of_bsn_vport_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_bsn_vport_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_header_OF_VERSION_1_3_check(
    of_bsn_vport_header_t *obj, int value)
{

    value = of_bsn_vport_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_header_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bsn_vport_q_in_q
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_3_populate(
    of_bsn_vport_q_in_q_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_port_name_t port_name;

    /* Run thru accessors after new to ensure okay */
    of_bsn_vport_q_in_q_port_no_get(obj, &val32);
    of_bsn_vport_q_in_q_ingress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_ingress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_tpid_get(obj, &val16);
    of_bsn_vport_q_in_q_egress_vlan_id_get(obj, &val16);
    of_bsn_vport_q_in_q_if_name_get(obj, &port_name);

    value = of_bsn_vport_q_in_q_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bsn_vport_q_in_q
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bsn_vport_q_in_q_OF_VERSION_1_3_check(
    of_bsn_vport_q_in_q_t *obj, int value)
{

    value = of_bsn_vport_q_in_q_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bsn_vport_q_in_q_t *dup;

        TEST_ASSERT((dup = of_bsn_vport_q_in_q_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bsn_vport_q_in_q_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bucket
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bucket_OF_VERSION_1_3_populate(
    of_bucket_t *obj, int value)
{
    uint16_t val16;
    of_port_no_t port_no;
    uint32_t val32;
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    of_bucket_weight_get(obj, &val16);
    of_bucket_watch_port_get(obj, &port_no);
    of_bucket_watch_group_get(obj, &val32);
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_bucket_actions_bind(obj, &sub_cls);
    }

    value = of_bucket_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_bucket_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_bucket
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bucket_OF_VERSION_1_3_check(
    of_bucket_t *obj, int value)
{

    value = of_bucket_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_bucket_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_3_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_bucket_t *dup;

        TEST_ASSERT((dup = of_bucket_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bucket_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_bucket_counter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_bucket_counter_OF_VERSION_1_3_populate(
    of_bucket_counter_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_bucket_counter_packet_count_get(obj, &val64);
    of_bucket_counter_byte_count_get(obj, &val64);

    value = of_bucket_counter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_bucket_counter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_bucket_counter_OF_VERSION_1_3_check(
    of_bucket_counter_t *obj, int value)
{

    value = of_bucket_counter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_bucket_counter_t *dup;

        TEST_ASSERT((dup = of_bucket_counter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_bucket_counter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_flow_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_3_populate(
    of_flow_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;
    of_match_t match;
    of_list_instruction_t *list;

    /* Run thru accessors after new to ensure okay */
    of_flow_stats_entry_table_id_get(obj, &val8);
    of_flow_stats_entry_duration_sec_get(obj, &val32);
    of_flow_stats_entry_duration_nsec_get(obj, &val32);
    of_flow_stats_entry_priority_get(obj, &val16);
    of_flow_stats_entry_idle_timeout_get(obj, &val16);
    of_flow_stats_entry_hard_timeout_get(obj, &val16);
    of_flow_stats_entry_flags_get(obj, &val16);
    of_flow_stats_entry_cookie_get(obj, &val64);
    of_flow_stats_entry_packet_count_get(obj, &val64);
    of_flow_stats_entry_byte_count_get(obj, &val64);
    of_flow_stats_entry_match_get(obj, &match);
    {
        of_list_instruction_t sub_cls;

        /* Test bind */
        of_flow_stats_entry_instructions_bind(obj, &sub_cls);
    }

    value = of_flow_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_match_populate(&match, OF_VERSION_1_3, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_set(
        obj, &match);

    list = of_list_instruction_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_instructions_set(
        obj, list);
    of_list_instruction_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_flow_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_flow_stats_entry_OF_VERSION_1_3_check(
    of_flow_stats_entry_t *obj, int value)
{
    of_match_t match;

    value = of_flow_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_flow_stats_entry_match_get(obj, &match);
    value = of_match_check(&match, OF_VERSION_1_3, value);

    { /* Use get/delete to access on check */
        of_list_instruction_t *instructions_ptr;

        instructions_ptr = of_flow_stats_entry_instructions_get(obj);
        TEST_ASSERT(instructions_ptr != NULL);
        value = of_list_instruction_OF_VERSION_1_3_check(
            instructions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_delete(instructions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_flow_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_flow_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_desc_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_entry_OF_VERSION_1_3_populate(
    of_group_desc_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    of_list_bucket_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_desc_stats_entry_group_type_get(obj, &val8);
    of_group_desc_stats_entry_group_id_get(obj, &val32);
    {
        of_list_bucket_t sub_cls;

        /* Test bind */
        of_group_desc_stats_entry_buckets_bind(obj, &sub_cls);
    }

    value = of_group_desc_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_desc_stats_entry_buckets_set(
        obj, list);
    of_list_bucket_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_desc_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_desc_stats_entry_OF_VERSION_1_3_check(
    of_group_desc_stats_entry_t *obj, int value)
{

    value = of_group_desc_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_t *buckets_ptr;

        buckets_ptr = of_group_desc_stats_entry_buckets_get(obj);
        TEST_ASSERT(buckets_ptr != NULL);
        value = of_list_bucket_OF_VERSION_1_3_check(
            buckets_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_delete(buckets_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_group_desc_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_desc_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_group_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_group_stats_entry_OF_VERSION_1_3_populate(
    of_group_stats_entry_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    of_list_bucket_counter_t *list;

    /* Run thru accessors after new to ensure okay */
    of_group_stats_entry_group_id_get(obj, &val32);
    of_group_stats_entry_ref_count_get(obj, &val32);
    of_group_stats_entry_packet_count_get(obj, &val64);
    of_group_stats_entry_byte_count_get(obj, &val64);
    of_group_stats_entry_duration_sec_get(obj, &val32);
    of_group_stats_entry_duration_nsec_get(obj, &val32);
    {
        of_list_bucket_counter_t sub_cls;

        /* Test bind */
        of_group_stats_entry_bucket_stats_bind(obj, &sub_cls);
    }

    value = of_group_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_bucket_counter_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_bucket_counter_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_group_stats_entry_bucket_stats_set(
        obj, list);
    of_list_bucket_counter_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_group_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_group_stats_entry_OF_VERSION_1_3_check(
    of_group_stats_entry_t *obj, int value)
{

    value = of_group_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_bucket_counter_t *bucket_stats_ptr;

        bucket_stats_ptr = of_group_stats_entry_bucket_stats_get(obj);
        TEST_ASSERT(bucket_stats_ptr != NULL);
        value = of_list_bucket_counter_OF_VERSION_1_3_check(
            bucket_stats_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_bucket_counter_delete(bucket_stats_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_group_stats_entry_t *dup;

        TEST_ASSERT((dup = of_group_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_group_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_3_populate(
    of_header_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_header_xid_get(obj, &val32);

    value = of_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_header_OF_VERSION_1_3_check(
    of_header_t *obj, int value)
{

    value = of_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_header_t *dup;

        TEST_ASSERT((dup = of_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello_elem_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_elem_header_OF_VERSION_1_3_populate(
    of_hello_elem_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_hello_elem_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_hello_elem_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_elem_header_OF_VERSION_1_3_check(
    of_hello_elem_header_t *obj, int value)
{

    value = of_hello_elem_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_elem_header_t *dup;

        TEST_ASSERT((dup = of_hello_elem_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_elem_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_hello_elem_versionbitmap
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_hello_elem_versionbitmap_OF_VERSION_1_3_populate(
    of_hello_elem_versionbitmap_t *obj, int value)
{
    of_list_uint32_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint32_t sub_cls;

        /* Test bind */
        of_hello_elem_versionbitmap_bitmaps_bind(obj, &sub_cls);
    }

    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint32_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint32_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_hello_elem_versionbitmap_bitmaps_set(
        obj, list);
    of_list_uint32_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_hello_elem_versionbitmap
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_hello_elem_versionbitmap_OF_VERSION_1_3_check(
    of_hello_elem_versionbitmap_t *obj, int value)
{

    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint32_t *bitmaps_ptr;

        bitmaps_ptr = of_hello_elem_versionbitmap_bitmaps_get(obj);
        TEST_ASSERT(bitmaps_ptr != NULL);
        value = of_list_uint32_OF_VERSION_1_3_check(
            bitmaps_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint32_delete(bitmaps_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_hello_elem_versionbitmap_t *dup;

        TEST_ASSERT((dup = of_hello_elem_versionbitmap_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_hello_elem_versionbitmap_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_apply_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_apply_actions_OF_VERSION_1_3_populate(
    of_instruction_apply_actions_t *obj, int value)
{
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_instruction_apply_actions_actions_bind(obj, &sub_cls);
    }

    value = of_instruction_apply_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_instruction_apply_actions_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_instruction_apply_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_apply_actions_OF_VERSION_1_3_check(
    of_instruction_apply_actions_t *obj, int value)
{

    value = of_instruction_apply_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_instruction_apply_actions_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_3_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_apply_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_apply_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_apply_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_OF_VERSION_1_3_populate(
    of_instruction_bsn_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_experimenter_get(obj, &val32);
    of_instruction_bsn_subtype_get(obj, &val32);

    value = of_instruction_bsn_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_OF_VERSION_1_3_check(
    of_instruction_bsn_t *obj, int value)
{

    value = of_instruction_bsn_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn_arp_offload
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_arp_offload_OF_VERSION_1_3_populate(
    of_instruction_bsn_arp_offload_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_arp_offload_experimenter_get(obj, &val32);
    of_instruction_bsn_arp_offload_subtype_get(obj, &val32);

    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn_arp_offload
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_arp_offload_OF_VERSION_1_3_check(
    of_instruction_bsn_arp_offload_t *obj, int value)
{

    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_arp_offload_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_arp_offload_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_arp_offload_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn_deny
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_deny_OF_VERSION_1_3_populate(
    of_instruction_bsn_deny_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_deny_experimenter_get(obj, &val32);
    of_instruction_bsn_deny_subtype_get(obj, &val32);

    value = of_instruction_bsn_deny_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn_deny
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_deny_OF_VERSION_1_3_check(
    of_instruction_bsn_deny_t *obj, int value)
{

    value = of_instruction_bsn_deny_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_deny_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_deny_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_deny_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn_dhcp_offload
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_populate(
    of_instruction_bsn_dhcp_offload_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_dhcp_offload_experimenter_get(obj, &val32);
    of_instruction_bsn_dhcp_offload_subtype_get(obj, &val32);

    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn_dhcp_offload
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_check(
    of_instruction_bsn_dhcp_offload_t *obj, int value)
{

    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_dhcp_offload_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_dhcp_offload_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_dhcp_offload_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn_disable_split_horizon_check
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_disable_split_horizon_check_experimenter_get(obj, &val32);
    of_instruction_bsn_disable_split_horizon_check_subtype_get(obj, &val32);

    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn_disable_split_horizon_check
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
    of_instruction_bsn_disable_split_horizon_check_t *obj, int value)
{

    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_disable_split_horizon_check_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_disable_split_horizon_check_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_disable_split_horizon_check_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn_disable_src_mac_check
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_disable_src_mac_check_experimenter_get(obj, &val32);
    of_instruction_bsn_disable_src_mac_check_subtype_get(obj, &val32);

    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn_disable_src_mac_check
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
    of_instruction_bsn_disable_src_mac_check_t *obj, int value)
{

    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_disable_src_mac_check_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_disable_src_mac_check_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_disable_src_mac_check_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn_packet_of_death
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_packet_of_death_OF_VERSION_1_3_populate(
    of_instruction_bsn_packet_of_death_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_packet_of_death_experimenter_get(obj, &val32);
    of_instruction_bsn_packet_of_death_subtype_get(obj, &val32);

    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn_packet_of_death
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_packet_of_death_OF_VERSION_1_3_check(
    of_instruction_bsn_packet_of_death_t *obj, int value)
{

    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_packet_of_death_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_packet_of_death_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_packet_of_death_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_bsn_permit
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_permit_OF_VERSION_1_3_populate(
    of_instruction_bsn_permit_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_bsn_permit_experimenter_get(obj, &val32);
    of_instruction_bsn_permit_subtype_get(obj, &val32);

    value = of_instruction_bsn_permit_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_bsn_permit
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_bsn_permit_OF_VERSION_1_3_check(
    of_instruction_bsn_permit_t *obj, int value)
{

    value = of_instruction_bsn_permit_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_bsn_permit_t *dup;

        TEST_ASSERT((dup = of_instruction_bsn_permit_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_bsn_permit_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_clear_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_clear_actions_OF_VERSION_1_3_populate(
    of_instruction_clear_actions_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_clear_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_clear_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_clear_actions_OF_VERSION_1_3_check(
    of_instruction_clear_actions_t *obj, int value)
{

    value = of_instruction_clear_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_clear_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_clear_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_clear_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_experimenter_OF_VERSION_1_3_populate(
    of_instruction_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_instruction_experimenter_experimenter_get(obj, &val32);
    of_instruction_experimenter_data_get(obj, &octets);

    value = of_instruction_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_instruction_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_instruction_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_experimenter_OF_VERSION_1_3_check(
    of_instruction_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_instruction_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_instruction_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_experimenter_t *dup;

        TEST_ASSERT((dup = of_instruction_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_goto_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_goto_table_OF_VERSION_1_3_populate(
    of_instruction_goto_table_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_instruction_goto_table_table_id_get(obj, &val8);

    value = of_instruction_goto_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_goto_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_goto_table_OF_VERSION_1_3_check(
    of_instruction_goto_table_t *obj, int value)
{

    value = of_instruction_goto_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_goto_table_t *dup;

        TEST_ASSERT((dup = of_instruction_goto_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_goto_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_header_OF_VERSION_1_3_populate(
    of_instruction_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_header_OF_VERSION_1_3_check(
    of_instruction_header_t *obj, int value)
{

    value = of_instruction_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_header_t *dup;

        TEST_ASSERT((dup = of_instruction_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_apply_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_apply_actions_OF_VERSION_1_3_populate(
    of_instruction_id_apply_actions_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_id_apply_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_apply_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_apply_actions_OF_VERSION_1_3_check(
    of_instruction_id_apply_actions_t *obj, int value)
{

    value = of_instruction_id_apply_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_apply_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_id_apply_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_apply_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_experimenter_get(obj, &val32);
    of_instruction_id_bsn_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_OF_VERSION_1_3_check(
    of_instruction_id_bsn_t *obj, int value)
{

    value = of_instruction_id_bsn_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn_arp_offload
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_arp_offload_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_arp_offload_experimenter_get(obj, &val32);
    of_instruction_id_bsn_arp_offload_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn_arp_offload
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_check(
    of_instruction_id_bsn_arp_offload_t *obj, int value)
{

    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_arp_offload_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_arp_offload_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_arp_offload_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn_deny
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_deny_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_deny_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_deny_experimenter_get(obj, &val32);
    of_instruction_id_bsn_deny_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn_deny
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_deny_OF_VERSION_1_3_check(
    of_instruction_id_bsn_deny_t *obj, int value)
{

    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_deny_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_deny_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_deny_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn_dhcp_offload
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_dhcp_offload_experimenter_get(obj, &val32);
    of_instruction_id_bsn_dhcp_offload_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn_dhcp_offload
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_check(
    of_instruction_id_bsn_dhcp_offload_t *obj, int value)
{

    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_dhcp_offload_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_dhcp_offload_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_dhcp_offload_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn_disable_split_horizon_check
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_disable_split_horizon_check_experimenter_get(obj, &val32);
    of_instruction_id_bsn_disable_split_horizon_check_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn_disable_split_horizon_check
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
    of_instruction_id_bsn_disable_split_horizon_check_t *obj, int value)
{

    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_disable_split_horizon_check_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_disable_split_horizon_check_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_disable_split_horizon_check_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn_disable_src_mac_check
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_disable_src_mac_check_experimenter_get(obj, &val32);
    of_instruction_id_bsn_disable_src_mac_check_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn_disable_src_mac_check
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
    of_instruction_id_bsn_disable_src_mac_check_t *obj, int value)
{

    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_disable_src_mac_check_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_disable_src_mac_check_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_disable_src_mac_check_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn_packet_of_death
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_packet_of_death_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_packet_of_death_experimenter_get(obj, &val32);
    of_instruction_id_bsn_packet_of_death_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn_packet_of_death
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_check(
    of_instruction_id_bsn_packet_of_death_t *obj, int value)
{

    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_packet_of_death_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_packet_of_death_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_packet_of_death_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_bsn_permit
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_permit_OF_VERSION_1_3_populate(
    of_instruction_id_bsn_permit_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_bsn_permit_experimenter_get(obj, &val32);
    of_instruction_id_bsn_permit_subtype_get(obj, &val32);

    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_bsn_permit
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_bsn_permit_OF_VERSION_1_3_check(
    of_instruction_id_bsn_permit_t *obj, int value)
{

    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_bsn_permit_t *dup;

        TEST_ASSERT((dup = of_instruction_id_bsn_permit_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_bsn_permit_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_clear_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_clear_actions_OF_VERSION_1_3_populate(
    of_instruction_id_clear_actions_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_id_clear_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_clear_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_clear_actions_OF_VERSION_1_3_check(
    of_instruction_id_clear_actions_t *obj, int value)
{

    value = of_instruction_id_clear_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_clear_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_id_clear_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_clear_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_experimenter_OF_VERSION_1_3_populate(
    of_instruction_id_experimenter_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_id_experimenter_experimenter_get(obj, &val32);

    value = of_instruction_id_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_experimenter_OF_VERSION_1_3_check(
    of_instruction_id_experimenter_t *obj, int value)
{

    value = of_instruction_id_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_experimenter_t *dup;

        TEST_ASSERT((dup = of_instruction_id_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_goto_table
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_goto_table_OF_VERSION_1_3_populate(
    of_instruction_id_goto_table_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_id_goto_table_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_goto_table
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_goto_table_OF_VERSION_1_3_check(
    of_instruction_id_goto_table_t *obj, int value)
{

    value = of_instruction_id_goto_table_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_goto_table_t *dup;

        TEST_ASSERT((dup = of_instruction_id_goto_table_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_goto_table_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_header_OF_VERSION_1_3_populate(
    of_instruction_id_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_id_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_header_OF_VERSION_1_3_check(
    of_instruction_id_header_t *obj, int value)
{

    value = of_instruction_id_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_header_t *dup;

        TEST_ASSERT((dup = of_instruction_id_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_meter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_meter_OF_VERSION_1_3_populate(
    of_instruction_id_meter_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_id_meter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_meter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_meter_OF_VERSION_1_3_check(
    of_instruction_id_meter_t *obj, int value)
{

    value = of_instruction_id_meter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_meter_t *dup;

        TEST_ASSERT((dup = of_instruction_id_meter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_meter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_write_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_write_actions_OF_VERSION_1_3_populate(
    of_instruction_id_write_actions_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_id_write_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_write_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_write_actions_OF_VERSION_1_3_check(
    of_instruction_id_write_actions_t *obj, int value)
{

    value = of_instruction_id_write_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_write_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_id_write_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_write_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_id_write_metadata
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_write_metadata_OF_VERSION_1_3_populate(
    of_instruction_id_write_metadata_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_instruction_id_write_metadata_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_id_write_metadata
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_id_write_metadata_OF_VERSION_1_3_check(
    of_instruction_id_write_metadata_t *obj, int value)
{

    value = of_instruction_id_write_metadata_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_id_write_metadata_t *dup;

        TEST_ASSERT((dup = of_instruction_id_write_metadata_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_id_write_metadata_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_meter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_meter_OF_VERSION_1_3_populate(
    of_instruction_meter_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_instruction_meter_meter_id_get(obj, &val32);

    value = of_instruction_meter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_meter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_meter_OF_VERSION_1_3_check(
    of_instruction_meter_t *obj, int value)
{

    value = of_instruction_meter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_meter_t *dup;

        TEST_ASSERT((dup = of_instruction_meter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_meter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_write_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_actions_OF_VERSION_1_3_populate(
    of_instruction_write_actions_t *obj, int value)
{
    of_list_action_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_t sub_cls;

        /* Test bind */
        of_instruction_write_actions_actions_bind(obj, &sub_cls);
    }

    value = of_instruction_write_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_instruction_write_actions_actions_set(
        obj, list);
    of_list_action_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_instruction_write_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_actions_OF_VERSION_1_3_check(
    of_instruction_write_actions_t *obj, int value)
{

    value = of_instruction_write_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_t *actions_ptr;

        actions_ptr = of_instruction_write_actions_actions_get(obj);
        TEST_ASSERT(actions_ptr != NULL);
        value = of_list_action_OF_VERSION_1_3_check(
            actions_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_delete(actions_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_write_actions_t *dup;

        TEST_ASSERT((dup = of_instruction_write_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_write_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_instruction_write_metadata
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_metadata_OF_VERSION_1_3_populate(
    of_instruction_write_metadata_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_instruction_write_metadata_metadata_get(obj, &val64);
    of_instruction_write_metadata_metadata_mask_get(obj, &val64);

    value = of_instruction_write_metadata_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_instruction_write_metadata
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_instruction_write_metadata_OF_VERSION_1_3_check(
    of_instruction_write_metadata_t *obj, int value)
{

    value = of_instruction_write_metadata_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_instruction_write_metadata_t *dup;

        TEST_ASSERT((dup = of_instruction_write_metadata_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_instruction_write_metadata_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_match_v3
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_match_v3_OF_VERSION_1_3_populate(
    of_match_v3_t *obj, int value)
{
    of_list_oxm_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_oxm_t sub_cls;

        /* Test bind */
        of_match_v3_oxm_list_bind(obj, &sub_cls);
    }

    value = of_match_v3_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_oxm_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_oxm_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_match_v3_oxm_list_set(
        obj, list);
    of_list_oxm_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_match_v3
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_match_v3_OF_VERSION_1_3_check(
    of_match_v3_t *obj, int value)
{

    value = of_match_v3_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_oxm_t *oxm_list_ptr;

        oxm_list_ptr = of_match_v3_oxm_list_get(obj);
        TEST_ASSERT(oxm_list_ptr != NULL);
        value = of_list_oxm_OF_VERSION_1_3_check(
            oxm_list_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_oxm_delete(oxm_list_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_match_v3_t *dup;

        TEST_ASSERT((dup = of_match_v3_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_match_v3_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_band_drop
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_band_drop_OF_VERSION_1_3_populate(
    of_meter_band_drop_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_meter_band_drop_rate_get(obj, &val32);
    of_meter_band_drop_burst_size_get(obj, &val32);

    value = of_meter_band_drop_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_band_drop
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_band_drop_OF_VERSION_1_3_check(
    of_meter_band_drop_t *obj, int value)
{

    value = of_meter_band_drop_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_band_drop_t *dup;

        TEST_ASSERT((dup = of_meter_band_drop_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_band_drop_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_band_dscp_remark
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_band_dscp_remark_OF_VERSION_1_3_populate(
    of_meter_band_dscp_remark_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_meter_band_dscp_remark_rate_get(obj, &val32);
    of_meter_band_dscp_remark_burst_size_get(obj, &val32);
    of_meter_band_dscp_remark_prec_level_get(obj, &val8);

    value = of_meter_band_dscp_remark_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_band_dscp_remark
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_band_dscp_remark_OF_VERSION_1_3_check(
    of_meter_band_dscp_remark_t *obj, int value)
{

    value = of_meter_band_dscp_remark_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_band_dscp_remark_t *dup;

        TEST_ASSERT((dup = of_meter_band_dscp_remark_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_band_dscp_remark_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_band_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_band_experimenter_OF_VERSION_1_3_populate(
    of_meter_band_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_meter_band_experimenter_rate_get(obj, &val32);
    of_meter_band_experimenter_burst_size_get(obj, &val32);
    of_meter_band_experimenter_experimenter_get(obj, &val32);
    of_meter_band_experimenter_data_get(obj, &octets);

    value = of_meter_band_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_meter_band_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_meter_band_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_band_experimenter_OF_VERSION_1_3_check(
    of_meter_band_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_meter_band_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_meter_band_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_band_experimenter_t *dup;

        TEST_ASSERT((dup = of_meter_band_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_band_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_band_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_band_header_OF_VERSION_1_3_populate(
    of_meter_band_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_meter_band_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_band_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_band_header_OF_VERSION_1_3_check(
    of_meter_band_header_t *obj, int value)
{

    value = of_meter_band_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_band_header_t *dup;

        TEST_ASSERT((dup = of_meter_band_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_band_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_band_stats
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_band_stats_OF_VERSION_1_3_populate(
    of_meter_band_stats_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_meter_band_stats_packet_band_count_get(obj, &val64);
    of_meter_band_stats_byte_band_count_get(obj, &val64);

    value = of_meter_band_stats_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_band_stats
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_band_stats_OF_VERSION_1_3_check(
    of_meter_band_stats_t *obj, int value)
{

    value = of_meter_band_stats_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_band_stats_t *dup;

        TEST_ASSERT((dup = of_meter_band_stats_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_band_stats_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_config
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_config_OF_VERSION_1_3_populate(
    of_meter_config_t *obj, int value)
{
    uint16_t val16;
    uint32_t val32;
    of_list_meter_band_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_config_flags_get(obj, &val16);
    of_meter_config_meter_id_get(obj, &val32);
    {
        of_list_meter_band_t sub_cls;

        /* Test bind */
        of_meter_config_entries_bind(obj, &sub_cls);
    }

    value = of_meter_config_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_band_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_band_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_config_entries_set(
        obj, list);
    of_list_meter_band_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_config
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_config_OF_VERSION_1_3_check(
    of_meter_config_t *obj, int value)
{

    value = of_meter_config_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_band_t *entries_ptr;

        entries_ptr = of_meter_config_entries_get(obj);
        TEST_ASSERT(entries_ptr != NULL);
        value = of_list_meter_band_OF_VERSION_1_3_check(
            entries_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_band_delete(entries_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_config_t *dup;

        TEST_ASSERT((dup = of_meter_config_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_config_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_features
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_features_OF_VERSION_1_3_populate(
    of_meter_features_t *obj, int value)
{
    uint32_t val32;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_meter_features_max_meter_get(obj, &val32);
    of_meter_features_band_types_get(obj, &val32);
    of_meter_features_capabilities_get(obj, &val32);
    of_meter_features_max_bands_get(obj, &val8);
    of_meter_features_max_color_get(obj, &val8);

    value = of_meter_features_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_meter_features
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_features_OF_VERSION_1_3_check(
    of_meter_features_t *obj, int value)
{

    value = of_meter_features_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_features_t *dup;

        TEST_ASSERT((dup = of_meter_features_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_features_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_meter_stats
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_meter_stats_OF_VERSION_1_3_populate(
    of_meter_stats_t *obj, int value)
{
    uint32_t val32;
    uint64_t val64;
    of_list_meter_band_stats_t *list;

    /* Run thru accessors after new to ensure okay */
    of_meter_stats_meter_id_get(obj, &val32);
    of_meter_stats_flow_count_get(obj, &val32);
    of_meter_stats_packet_in_count_get(obj, &val64);
    of_meter_stats_byte_in_count_get(obj, &val64);
    of_meter_stats_duration_sec_get(obj, &val32);
    of_meter_stats_duration_nsec_get(obj, &val32);
    {
        of_list_meter_band_stats_t sub_cls;

        /* Test bind */
        of_meter_stats_band_stats_bind(obj, &sub_cls);
    }

    value = of_meter_stats_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_meter_band_stats_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_meter_band_stats_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_meter_stats_band_stats_set(
        obj, list);
    of_list_meter_band_stats_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_meter_stats
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_meter_stats_OF_VERSION_1_3_check(
    of_meter_stats_t *obj, int value)
{

    value = of_meter_stats_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_meter_band_stats_t *band_stats_ptr;

        band_stats_ptr = of_meter_stats_band_stats_get(obj);
        TEST_ASSERT(band_stats_ptr != NULL);
        value = of_list_meter_band_stats_OF_VERSION_1_3_check(
            band_stats_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_meter_band_stats_delete(band_stats_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_meter_stats_t *dup;

        TEST_ASSERT((dup = of_meter_stats_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_meter_stats_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_op
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_OF_VERSION_1_3_populate(
    of_oxm_arp_op_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_op_value_get(obj, &val16);

    value = of_oxm_arp_op_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_op
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_OF_VERSION_1_3_check(
    of_oxm_arp_op_t *obj, int value)
{

    value = of_oxm_arp_op_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_op_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_op_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_op_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_op_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_masked_OF_VERSION_1_3_populate(
    of_oxm_arp_op_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_op_masked_value_get(obj, &val16);
    of_oxm_arp_op_masked_value_mask_get(obj, &val16);

    value = of_oxm_arp_op_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_op_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_op_masked_OF_VERSION_1_3_check(
    of_oxm_arp_op_masked_t *obj, int value)
{

    value = of_oxm_arp_op_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_op_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_op_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_op_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_sha
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_OF_VERSION_1_3_populate(
    of_oxm_arp_sha_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_sha_value_get(obj, &mac_addr);

    value = of_oxm_arp_sha_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_sha
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_OF_VERSION_1_3_check(
    of_oxm_arp_sha_t *obj, int value)
{

    value = of_oxm_arp_sha_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_sha_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_sha_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_sha_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_sha_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_masked_OF_VERSION_1_3_populate(
    of_oxm_arp_sha_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_sha_masked_value_get(obj, &mac_addr);
    of_oxm_arp_sha_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_sha_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_sha_masked_OF_VERSION_1_3_check(
    of_oxm_arp_sha_masked_t *obj, int value)
{

    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_sha_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_sha_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_sha_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_spa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_OF_VERSION_1_3_populate(
    of_oxm_arp_spa_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_spa_value_get(obj, &val32);

    value = of_oxm_arp_spa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_spa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_OF_VERSION_1_3_check(
    of_oxm_arp_spa_t *obj, int value)
{

    value = of_oxm_arp_spa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_spa_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_spa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_spa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_spa_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_masked_OF_VERSION_1_3_populate(
    of_oxm_arp_spa_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_spa_masked_value_get(obj, &val32);
    of_oxm_arp_spa_masked_value_mask_get(obj, &val32);

    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_spa_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_spa_masked_OF_VERSION_1_3_check(
    of_oxm_arp_spa_masked_t *obj, int value)
{

    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_spa_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_spa_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_spa_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tha
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_OF_VERSION_1_3_populate(
    of_oxm_arp_tha_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tha_value_get(obj, &mac_addr);

    value = of_oxm_arp_tha_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tha
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_OF_VERSION_1_3_check(
    of_oxm_arp_tha_t *obj, int value)
{

    value = of_oxm_arp_tha_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tha_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tha_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tha_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tha_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_masked_OF_VERSION_1_3_populate(
    of_oxm_arp_tha_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tha_masked_value_get(obj, &mac_addr);
    of_oxm_arp_tha_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tha_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tha_masked_OF_VERSION_1_3_check(
    of_oxm_arp_tha_masked_t *obj, int value)
{

    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tha_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tha_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tha_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_OF_VERSION_1_3_populate(
    of_oxm_arp_tpa_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tpa_value_get(obj, &val32);

    value = of_oxm_arp_tpa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_OF_VERSION_1_3_check(
    of_oxm_arp_tpa_t *obj, int value)
{

    value = of_oxm_arp_tpa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tpa_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_arp_tpa_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_masked_OF_VERSION_1_3_populate(
    of_oxm_arp_tpa_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_arp_tpa_masked_value_get(obj, &val32);
    of_oxm_arp_tpa_masked_value_mask_get(obj, &val32);

    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_arp_tpa_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_arp_tpa_masked_OF_VERSION_1_3_check(
    of_oxm_arp_tpa_masked_t *obj, int value)
{

    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_arp_tpa_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_arp_tpa_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_arp_tpa_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_egr_port_group_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_populate(
    of_oxm_bsn_egr_port_group_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_egr_port_group_id_value_get(obj, &val32);

    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_egr_port_group_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_check(
    of_oxm_bsn_egr_port_group_id_t *obj, int value)
{

    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_egr_port_group_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_egr_port_group_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_egr_port_group_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_egr_port_group_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_egr_port_group_id_masked_value_get(obj, &val32);
    of_oxm_bsn_egr_port_group_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_egr_port_group_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_egr_port_group_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_egr_port_group_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_egr_port_group_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_egr_port_group_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_global_vrf_allowed
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_populate(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_global_vrf_allowed_value_get(obj, &val8);

    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_global_vrf_allowed
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_check(
    of_oxm_bsn_global_vrf_allowed_t *obj, int value)
{

    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_global_vrf_allowed_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_global_vrf_allowed_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_global_vrf_allowed_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_global_vrf_allowed_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_global_vrf_allowed_masked_value_get(obj, &val8);
    of_oxm_bsn_global_vrf_allowed_masked_value_mask_get(obj, &val8);

    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_global_vrf_allowed_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_global_vrf_allowed_masked_t *obj, int value)
{

    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_global_vrf_allowed_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_global_vrf_allowed_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_global_vrf_allowed_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_in_ports_128
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_OF_VERSION_1_3_populate(
    of_oxm_bsn_in_ports_128_t *obj, int value)
{
    of_bitmap_128_t bitmap_128;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_in_ports_128_value_get(obj, &bitmap_128);

    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_in_ports_128
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_OF_VERSION_1_3_check(
    of_oxm_bsn_in_ports_128_t *obj, int value)
{

    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_in_ports_128_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_in_ports_128_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_in_ports_128_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_in_ports_128_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value)
{
    of_bitmap_128_t bitmap_128;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_in_ports_128_masked_value_get(obj, &bitmap_128);
    of_oxm_bsn_in_ports_128_masked_value_mask_get(obj, &bitmap_128);

    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_in_ports_128_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_in_ports_128_masked_t *obj, int value)
{

    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_in_ports_128_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_in_ports_128_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_in_ports_128_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_dst_class_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_populate(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_dst_class_id_value_get(obj, &val32);

    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_dst_class_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_check(
    of_oxm_bsn_l3_dst_class_id_t *obj, int value)
{

    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_dst_class_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_dst_class_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_dst_class_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_dst_class_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_dst_class_id_masked_value_get(obj, &val32);
    of_oxm_bsn_l3_dst_class_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_dst_class_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_l3_dst_class_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_dst_class_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_dst_class_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_dst_class_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_interface_class_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_populate(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_interface_class_id_value_get(obj, &val32);

    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_interface_class_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_check(
    of_oxm_bsn_l3_interface_class_id_t *obj, int value)
{

    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_interface_class_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_interface_class_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_interface_class_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_interface_class_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_interface_class_id_masked_value_get(obj, &val32);
    of_oxm_bsn_l3_interface_class_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_interface_class_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_l3_interface_class_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_interface_class_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_interface_class_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_interface_class_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_src_class_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_populate(
    of_oxm_bsn_l3_src_class_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_src_class_id_value_get(obj, &val32);

    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_src_class_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_check(
    of_oxm_bsn_l3_src_class_id_t *obj, int value)
{

    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_src_class_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_src_class_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_src_class_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_l3_src_class_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_l3_src_class_id_masked_value_get(obj, &val32);
    of_oxm_bsn_l3_src_class_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_l3_src_class_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_l3_src_class_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_l3_src_class_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_l3_src_class_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_l3_src_class_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_lag_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_OF_VERSION_1_3_populate(
    of_oxm_bsn_lag_id_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_lag_id_value_get(obj, &val32);

    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_lag_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_OF_VERSION_1_3_check(
    of_oxm_bsn_lag_id_t *obj, int value)
{

    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_lag_id_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_lag_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_lag_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_lag_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_lag_id_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_lag_id_masked_value_get(obj, &val32);
    of_oxm_bsn_lag_id_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_lag_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_lag_id_masked_t *obj, int value)
{

    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_lag_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_lag_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_lag_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf0
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf0_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf0_value_get(obj, &val32);

    value = of_oxm_bsn_udf0_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf0
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_OF_VERSION_1_3_check(
    of_oxm_bsn_udf0_t *obj, int value)
{

    value = of_oxm_bsn_udf0_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf0_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf0_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf0_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf0_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf0_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf0_masked_value_get(obj, &val32);
    of_oxm_bsn_udf0_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf0_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf0_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf0_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf0_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf0_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf0_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf1
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf1_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf1_value_get(obj, &val32);

    value = of_oxm_bsn_udf1_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf1
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_OF_VERSION_1_3_check(
    of_oxm_bsn_udf1_t *obj, int value)
{

    value = of_oxm_bsn_udf1_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf1_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf1_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf1_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf1_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf1_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf1_masked_value_get(obj, &val32);
    of_oxm_bsn_udf1_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf1_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf1_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf1_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf1_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf1_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf1_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf2
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf2_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf2_value_get(obj, &val32);

    value = of_oxm_bsn_udf2_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf2
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_OF_VERSION_1_3_check(
    of_oxm_bsn_udf2_t *obj, int value)
{

    value = of_oxm_bsn_udf2_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf2_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf2_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf2_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf2_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf2_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf2_masked_value_get(obj, &val32);
    of_oxm_bsn_udf2_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf2_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf2_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf2_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf2_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf2_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf2_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf3
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf3_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf3_value_get(obj, &val32);

    value = of_oxm_bsn_udf3_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf3
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_OF_VERSION_1_3_check(
    of_oxm_bsn_udf3_t *obj, int value)
{

    value = of_oxm_bsn_udf3_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf3_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf3_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf3_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf3_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf3_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf3_masked_value_get(obj, &val32);
    of_oxm_bsn_udf3_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf3_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf3_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf3_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf3_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf3_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf3_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf4
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf4_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf4_value_get(obj, &val32);

    value = of_oxm_bsn_udf4_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf4
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_OF_VERSION_1_3_check(
    of_oxm_bsn_udf4_t *obj, int value)
{

    value = of_oxm_bsn_udf4_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf4_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf4_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf4_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf4_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf4_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf4_masked_value_get(obj, &val32);
    of_oxm_bsn_udf4_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf4_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf4_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf4_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf4_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf4_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf4_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf5
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf5_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf5_value_get(obj, &val32);

    value = of_oxm_bsn_udf5_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf5
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_OF_VERSION_1_3_check(
    of_oxm_bsn_udf5_t *obj, int value)
{

    value = of_oxm_bsn_udf5_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf5_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf5_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf5_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf5_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf5_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf5_masked_value_get(obj, &val32);
    of_oxm_bsn_udf5_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf5_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf5_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf5_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf5_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf5_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf5_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf6
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf6_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf6_value_get(obj, &val32);

    value = of_oxm_bsn_udf6_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf6
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_OF_VERSION_1_3_check(
    of_oxm_bsn_udf6_t *obj, int value)
{

    value = of_oxm_bsn_udf6_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf6_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf6_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf6_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf6_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf6_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf6_masked_value_get(obj, &val32);
    of_oxm_bsn_udf6_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf6_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf6_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf6_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf6_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf6_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf6_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf7
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf7_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf7_value_get(obj, &val32);

    value = of_oxm_bsn_udf7_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf7
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_OF_VERSION_1_3_check(
    of_oxm_bsn_udf7_t *obj, int value)
{

    value = of_oxm_bsn_udf7_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf7_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf7_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf7_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_udf7_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_udf7_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_udf7_masked_value_get(obj, &val32);
    of_oxm_bsn_udf7_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_udf7_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_udf7_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_udf7_masked_t *obj, int value)
{

    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_udf7_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_udf7_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_udf7_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_vrf
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_OF_VERSION_1_3_populate(
    of_oxm_bsn_vrf_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_vrf_value_get(obj, &val32);

    value = of_oxm_bsn_vrf_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_vrf
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_OF_VERSION_1_3_check(
    of_oxm_bsn_vrf_t *obj, int value)
{

    value = of_oxm_bsn_vrf_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_vrf_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_vrf_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_vrf_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_bsn_vrf_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_masked_OF_VERSION_1_3_populate(
    of_oxm_bsn_vrf_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_bsn_vrf_masked_value_get(obj, &val32);
    of_oxm_bsn_vrf_masked_value_mask_get(obj, &val32);

    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_bsn_vrf_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_bsn_vrf_masked_OF_VERSION_1_3_check(
    of_oxm_bsn_vrf_masked_t *obj, int value)
{

    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_bsn_vrf_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_bsn_vrf_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_bsn_vrf_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_OF_VERSION_1_3_populate(
    of_oxm_eth_dst_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_dst_value_get(obj, &mac_addr);

    value = of_oxm_eth_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_OF_VERSION_1_3_check(
    of_oxm_eth_dst_t *obj, int value)
{

    value = of_oxm_eth_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_masked_OF_VERSION_1_3_populate(
    of_oxm_eth_dst_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_dst_masked_value_get(obj, &mac_addr);
    of_oxm_eth_dst_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_dst_masked_OF_VERSION_1_3_check(
    of_oxm_eth_dst_masked_t *obj, int value)
{

    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_OF_VERSION_1_3_populate(
    of_oxm_eth_src_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_src_value_get(obj, &mac_addr);

    value = of_oxm_eth_src_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_OF_VERSION_1_3_check(
    of_oxm_eth_src_t *obj, int value)
{

    value = of_oxm_eth_src_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_src_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_masked_OF_VERSION_1_3_populate(
    of_oxm_eth_src_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_src_masked_value_get(obj, &mac_addr);
    of_oxm_eth_src_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_eth_src_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_src_masked_OF_VERSION_1_3_check(
    of_oxm_eth_src_masked_t *obj, int value)
{

    value = of_oxm_eth_src_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_type
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_OF_VERSION_1_3_populate(
    of_oxm_eth_type_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_type_value_get(obj, &val16);

    value = of_oxm_eth_type_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_type
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_OF_VERSION_1_3_check(
    of_oxm_eth_type_t *obj, int value)
{

    value = of_oxm_eth_type_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_type_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_type_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_type_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_eth_type_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_masked_OF_VERSION_1_3_populate(
    of_oxm_eth_type_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_eth_type_masked_value_get(obj, &val16);
    of_oxm_eth_type_masked_value_mask_get(obj, &val16);

    value = of_oxm_eth_type_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_eth_type_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_eth_type_masked_OF_VERSION_1_3_check(
    of_oxm_eth_type_masked_t *obj, int value)
{

    value = of_oxm_eth_type_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_eth_type_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_eth_type_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_eth_type_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_experimenter_OF_VERSION_1_3_populate(
    of_oxm_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_oxm_experimenter_experimenter_get(obj, &val32);
    of_oxm_experimenter_data_get(obj, &octets);

    value = of_oxm_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_oxm_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_oxm_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_experimenter_OF_VERSION_1_3_check(
    of_oxm_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_oxm_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_oxm_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_experimenter_t *dup;

        TEST_ASSERT((dup = of_oxm_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_header_OF_VERSION_1_3_populate(
    of_oxm_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_oxm_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_header_OF_VERSION_1_3_check(
    of_oxm_header_t *obj, int value)
{

    value = of_oxm_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_header_t *dup;

        TEST_ASSERT((dup = of_oxm_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_code
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_OF_VERSION_1_3_populate(
    of_oxm_icmpv4_code_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_code_value_get(obj, &val8);

    value = of_oxm_icmpv4_code_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_code
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_OF_VERSION_1_3_check(
    of_oxm_icmpv4_code_t *obj, int value)
{

    value = of_oxm_icmpv4_code_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_code_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_code_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_code_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_code_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_masked_OF_VERSION_1_3_populate(
    of_oxm_icmpv4_code_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_code_masked_value_get(obj, &val8);
    of_oxm_icmpv4_code_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_code_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_code_masked_OF_VERSION_1_3_check(
    of_oxm_icmpv4_code_masked_t *obj, int value)
{

    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_code_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_code_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_code_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_type
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_OF_VERSION_1_3_populate(
    of_oxm_icmpv4_type_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_type_value_get(obj, &val8);

    value = of_oxm_icmpv4_type_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_type
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_OF_VERSION_1_3_check(
    of_oxm_icmpv4_type_t *obj, int value)
{

    value = of_oxm_icmpv4_type_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_type_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_type_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_type_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv4_type_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_masked_OF_VERSION_1_3_populate(
    of_oxm_icmpv4_type_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv4_type_masked_value_get(obj, &val8);
    of_oxm_icmpv4_type_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv4_type_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv4_type_masked_OF_VERSION_1_3_check(
    of_oxm_icmpv4_type_masked_t *obj, int value)
{

    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv4_type_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv4_type_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv4_type_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_code
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_OF_VERSION_1_3_populate(
    of_oxm_icmpv6_code_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_code_value_get(obj, &val8);

    value = of_oxm_icmpv6_code_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_code
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_OF_VERSION_1_3_check(
    of_oxm_icmpv6_code_t *obj, int value)
{

    value = of_oxm_icmpv6_code_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_code_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_code_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_code_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_code_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_masked_OF_VERSION_1_3_populate(
    of_oxm_icmpv6_code_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_code_masked_value_get(obj, &val8);
    of_oxm_icmpv6_code_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_code_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_code_masked_OF_VERSION_1_3_check(
    of_oxm_icmpv6_code_masked_t *obj, int value)
{

    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_code_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_code_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_code_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_type
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_OF_VERSION_1_3_populate(
    of_oxm_icmpv6_type_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_type_value_get(obj, &val8);

    value = of_oxm_icmpv6_type_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_type
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_OF_VERSION_1_3_check(
    of_oxm_icmpv6_type_t *obj, int value)
{

    value = of_oxm_icmpv6_type_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_type_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_type_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_type_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_icmpv6_type_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_masked_OF_VERSION_1_3_populate(
    of_oxm_icmpv6_type_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_icmpv6_type_masked_value_get(obj, &val8);
    of_oxm_icmpv6_type_masked_value_mask_get(obj, &val8);

    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_icmpv6_type_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_icmpv6_type_masked_OF_VERSION_1_3_check(
    of_oxm_icmpv6_type_masked_t *obj, int value)
{

    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_icmpv6_type_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_icmpv6_type_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_icmpv6_type_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_phy_port
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_OF_VERSION_1_3_populate(
    of_oxm_in_phy_port_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_phy_port_value_get(obj, &port_no);

    value = of_oxm_in_phy_port_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_phy_port
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_OF_VERSION_1_3_check(
    of_oxm_in_phy_port_t *obj, int value)
{

    value = of_oxm_in_phy_port_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_phy_port_t *dup;

        TEST_ASSERT((dup = of_oxm_in_phy_port_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_phy_port_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_phy_port_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_masked_OF_VERSION_1_3_populate(
    of_oxm_in_phy_port_masked_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_phy_port_masked_value_get(obj, &port_no);
    of_oxm_in_phy_port_masked_value_mask_get(obj, &port_no);

    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_phy_port_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_phy_port_masked_OF_VERSION_1_3_check(
    of_oxm_in_phy_port_masked_t *obj, int value)
{

    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_phy_port_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_in_phy_port_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_phy_port_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_port
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_OF_VERSION_1_3_populate(
    of_oxm_in_port_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_port_value_get(obj, &port_no);

    value = of_oxm_in_port_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_port
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_OF_VERSION_1_3_check(
    of_oxm_in_port_t *obj, int value)
{

    value = of_oxm_in_port_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_port_t *dup;

        TEST_ASSERT((dup = of_oxm_in_port_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_port_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_in_port_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_masked_OF_VERSION_1_3_populate(
    of_oxm_in_port_masked_t *obj, int value)
{
    of_port_no_t port_no;

    /* Run thru accessors after new to ensure okay */
    of_oxm_in_port_masked_value_get(obj, &port_no);
    of_oxm_in_port_masked_value_mask_get(obj, &port_no);

    value = of_oxm_in_port_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_in_port_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_in_port_masked_OF_VERSION_1_3_check(
    of_oxm_in_port_masked_t *obj, int value)
{

    value = of_oxm_in_port_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_in_port_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_in_port_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_in_port_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_dscp
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_OF_VERSION_1_3_populate(
    of_oxm_ip_dscp_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_dscp_value_get(obj, &val8);

    value = of_oxm_ip_dscp_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_dscp
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_OF_VERSION_1_3_check(
    of_oxm_ip_dscp_t *obj, int value)
{

    value = of_oxm_ip_dscp_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_dscp_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_dscp_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_dscp_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_dscp_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_masked_OF_VERSION_1_3_populate(
    of_oxm_ip_dscp_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_dscp_masked_value_get(obj, &val8);
    of_oxm_ip_dscp_masked_value_mask_get(obj, &val8);

    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_dscp_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_dscp_masked_OF_VERSION_1_3_check(
    of_oxm_ip_dscp_masked_t *obj, int value)
{

    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_dscp_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_dscp_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_dscp_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_ecn
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_OF_VERSION_1_3_populate(
    of_oxm_ip_ecn_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_ecn_value_get(obj, &val8);

    value = of_oxm_ip_ecn_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_ecn
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_OF_VERSION_1_3_check(
    of_oxm_ip_ecn_t *obj, int value)
{

    value = of_oxm_ip_ecn_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_ecn_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_ecn_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_ecn_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_ecn_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_masked_OF_VERSION_1_3_populate(
    of_oxm_ip_ecn_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_ecn_masked_value_get(obj, &val8);
    of_oxm_ip_ecn_masked_value_mask_get(obj, &val8);

    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_ecn_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_ecn_masked_OF_VERSION_1_3_check(
    of_oxm_ip_ecn_masked_t *obj, int value)
{

    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_ecn_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_ecn_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_ecn_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_proto
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_OF_VERSION_1_3_populate(
    of_oxm_ip_proto_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_proto_value_get(obj, &val8);

    value = of_oxm_ip_proto_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_proto
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_OF_VERSION_1_3_check(
    of_oxm_ip_proto_t *obj, int value)
{

    value = of_oxm_ip_proto_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_proto_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_proto_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_proto_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ip_proto_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_masked_OF_VERSION_1_3_populate(
    of_oxm_ip_proto_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ip_proto_masked_value_get(obj, &val8);
    of_oxm_ip_proto_masked_value_mask_get(obj, &val8);

    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ip_proto_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ip_proto_masked_OF_VERSION_1_3_check(
    of_oxm_ip_proto_masked_t *obj, int value)
{

    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ip_proto_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ip_proto_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ip_proto_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_OF_VERSION_1_3_populate(
    of_oxm_ipv4_dst_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_dst_value_get(obj, &ipv4);

    value = of_oxm_ipv4_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_OF_VERSION_1_3_check(
    of_oxm_ipv4_dst_t *obj, int value)
{

    value = of_oxm_ipv4_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv4_dst_masked_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_dst_masked_value_get(obj, &ipv4);
    of_oxm_ipv4_dst_masked_value_mask_get(obj, &ipv4);

    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_dst_masked_OF_VERSION_1_3_check(
    of_oxm_ipv4_dst_masked_t *obj, int value)
{

    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_OF_VERSION_1_3_populate(
    of_oxm_ipv4_src_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_src_value_get(obj, &ipv4);

    value = of_oxm_ipv4_src_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_OF_VERSION_1_3_check(
    of_oxm_ipv4_src_t *obj, int value)
{

    value = of_oxm_ipv4_src_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_src_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv4_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv4_src_masked_t *obj, int value)
{
    of_ipv4_t ipv4;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv4_src_masked_value_get(obj, &ipv4);
    of_oxm_ipv4_src_masked_value_mask_get(obj, &ipv4);

    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv4_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv4_src_masked_OF_VERSION_1_3_check(
    of_oxm_ipv4_src_masked_t *obj, int value)
{

    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv4_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv4_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv4_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_OF_VERSION_1_3_populate(
    of_oxm_ipv6_dst_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_dst_value_get(obj, &ipv6);

    value = of_oxm_ipv6_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_OF_VERSION_1_3_check(
    of_oxm_ipv6_dst_t *obj, int value)
{

    value = of_oxm_ipv6_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv6_dst_masked_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_dst_masked_value_get(obj, &ipv6);
    of_oxm_ipv6_dst_masked_value_mask_get(obj, &ipv6);

    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_dst_masked_OF_VERSION_1_3_check(
    of_oxm_ipv6_dst_masked_t *obj, int value)
{

    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_flabel
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_OF_VERSION_1_3_populate(
    of_oxm_ipv6_flabel_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_flabel_value_get(obj, &val32);

    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_flabel
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_OF_VERSION_1_3_check(
    of_oxm_ipv6_flabel_t *obj, int value)
{

    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_flabel_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_flabel_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_flabel_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_flabel_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv6_flabel_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_flabel_masked_value_get(obj, &val32);
    of_oxm_ipv6_flabel_masked_value_mask_get(obj, &val32);

    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_flabel_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_check(
    of_oxm_ipv6_flabel_masked_t *obj, int value)
{

    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_flabel_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_flabel_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_flabel_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_sll
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_OF_VERSION_1_3_populate(
    of_oxm_ipv6_nd_sll_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_sll_value_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_sll
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_OF_VERSION_1_3_check(
    of_oxm_ipv6_nd_sll_t *obj, int value)
{

    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_sll_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_sll_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_sll_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_sll_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_sll_masked_value_get(obj, &mac_addr);
    of_oxm_ipv6_nd_sll_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_sll_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_check(
    of_oxm_ipv6_nd_sll_masked_t *obj, int value)
{

    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_sll_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_sll_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_sll_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_target
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_OF_VERSION_1_3_populate(
    of_oxm_ipv6_nd_target_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_target_value_get(obj, &ipv6);

    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_target
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_OF_VERSION_1_3_check(
    of_oxm_ipv6_nd_target_t *obj, int value)
{

    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_target_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_target_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_target_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_target_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv6_nd_target_masked_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_target_masked_value_get(obj, &ipv6);
    of_oxm_ipv6_nd_target_masked_value_mask_get(obj, &ipv6);

    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_target_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_check(
    of_oxm_ipv6_nd_target_masked_t *obj, int value)
{

    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_target_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_target_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_target_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_tll
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_OF_VERSION_1_3_populate(
    of_oxm_ipv6_nd_tll_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_tll_value_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_tll
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_OF_VERSION_1_3_check(
    of_oxm_ipv6_nd_tll_t *obj, int value)
{

    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_tll_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_tll_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_tll_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_nd_tll_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value)
{
    of_mac_addr_t mac_addr;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_nd_tll_masked_value_get(obj, &mac_addr);
    of_oxm_ipv6_nd_tll_masked_value_mask_get(obj, &mac_addr);

    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_nd_tll_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_check(
    of_oxm_ipv6_nd_tll_masked_t *obj, int value)
{

    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_nd_tll_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_nd_tll_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_nd_tll_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_OF_VERSION_1_3_populate(
    of_oxm_ipv6_src_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_src_value_get(obj, &ipv6);

    value = of_oxm_ipv6_src_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_OF_VERSION_1_3_check(
    of_oxm_ipv6_src_t *obj, int value)
{

    value = of_oxm_ipv6_src_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_src_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ipv6_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_masked_OF_VERSION_1_3_populate(
    of_oxm_ipv6_src_masked_t *obj, int value)
{
    of_ipv6_t ipv6;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ipv6_src_masked_value_get(obj, &ipv6);
    of_oxm_ipv6_src_masked_value_mask_get(obj, &ipv6);

    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ipv6_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ipv6_src_masked_OF_VERSION_1_3_check(
    of_oxm_ipv6_src_masked_t *obj, int value)
{

    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ipv6_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ipv6_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ipv6_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_metadata
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_OF_VERSION_1_3_populate(
    of_oxm_metadata_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_metadata_value_get(obj, &val64);

    value = of_oxm_metadata_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_metadata
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_OF_VERSION_1_3_check(
    of_oxm_metadata_t *obj, int value)
{

    value = of_oxm_metadata_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_metadata_t *dup;

        TEST_ASSERT((dup = of_oxm_metadata_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_metadata_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_metadata_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_masked_OF_VERSION_1_3_populate(
    of_oxm_metadata_masked_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_metadata_masked_value_get(obj, &val64);
    of_oxm_metadata_masked_value_mask_get(obj, &val64);

    value = of_oxm_metadata_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_metadata_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_metadata_masked_OF_VERSION_1_3_check(
    of_oxm_metadata_masked_t *obj, int value)
{

    value = of_oxm_metadata_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_metadata_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_metadata_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_metadata_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_bos
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_OF_VERSION_1_3_populate(
    of_oxm_mpls_bos_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_bos_value_get(obj, &val8);

    value = of_oxm_mpls_bos_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_bos
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_OF_VERSION_1_3_check(
    of_oxm_mpls_bos_t *obj, int value)
{

    value = of_oxm_mpls_bos_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_bos_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_bos_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_bos_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_bos_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_masked_OF_VERSION_1_3_populate(
    of_oxm_mpls_bos_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_bos_masked_value_get(obj, &val8);
    of_oxm_mpls_bos_masked_value_mask_get(obj, &val8);

    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_bos_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_bos_masked_OF_VERSION_1_3_check(
    of_oxm_mpls_bos_masked_t *obj, int value)
{

    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_bos_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_bos_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_bos_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_label
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_OF_VERSION_1_3_populate(
    of_oxm_mpls_label_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_label_value_get(obj, &val32);

    value = of_oxm_mpls_label_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_label
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_OF_VERSION_1_3_check(
    of_oxm_mpls_label_t *obj, int value)
{

    value = of_oxm_mpls_label_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_label_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_label_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_label_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_label_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_masked_OF_VERSION_1_3_populate(
    of_oxm_mpls_label_masked_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_label_masked_value_get(obj, &val32);
    of_oxm_mpls_label_masked_value_mask_get(obj, &val32);

    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_label_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_label_masked_OF_VERSION_1_3_check(
    of_oxm_mpls_label_masked_t *obj, int value)
{

    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_label_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_label_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_label_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_tc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_OF_VERSION_1_3_populate(
    of_oxm_mpls_tc_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_tc_value_get(obj, &val8);

    value = of_oxm_mpls_tc_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_tc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_OF_VERSION_1_3_check(
    of_oxm_mpls_tc_t *obj, int value)
{

    value = of_oxm_mpls_tc_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_tc_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_tc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_tc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_mpls_tc_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_masked_OF_VERSION_1_3_populate(
    of_oxm_mpls_tc_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_mpls_tc_masked_value_get(obj, &val8);
    of_oxm_mpls_tc_masked_value_mask_get(obj, &val8);

    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_mpls_tc_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_mpls_tc_masked_OF_VERSION_1_3_check(
    of_oxm_mpls_tc_masked_t *obj, int value)
{

    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_mpls_tc_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_mpls_tc_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_mpls_tc_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_experimenter_get(obj, &val32);
    of_oxm_ofdpa_subtype_get(obj, &val16);
    of_oxm_ofdpa_data_get(obj, &octets);

    value = of_oxm_ofdpa_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_oxm_ofdpa_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_OF_VERSION_1_3_check(
    of_oxm_ofdpa_t *obj, int value)
{
    of_octets_t octets;

    value = of_oxm_ofdpa_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_oxm_ofdpa_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_actset_output
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_actset_output_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_actset_output_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_actset_output_experimenter_get(obj, &val32);
    of_oxm_ofdpa_actset_output_subtype_get(obj, &val16);
    of_oxm_ofdpa_actset_output_value_get(obj, &val32);

    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_actset_output
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_actset_output_OF_VERSION_1_3_check(
    of_oxm_ofdpa_actset_output_t *obj, int value)
{

    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_actset_output_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_actset_output_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_actset_output_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_bfd_discriminator
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_bfd_discriminator_experimenter_get(obj, &val32);
    of_oxm_ofdpa_bfd_discriminator_subtype_get(obj, &val16);
    of_oxm_ofdpa_bfd_discriminator_value_get(obj, &val32);

    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_bfd_discriminator
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_check(
    of_oxm_ofdpa_bfd_discriminator_t *obj, int value)
{

    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_bfd_discriminator_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_bfd_discriminator_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_bfd_discriminator_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_bfd_discriminator_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_bfd_discriminator_masked_experimenter_get(obj, &val32);
    of_oxm_ofdpa_bfd_discriminator_masked_subtype_get(obj, &val16);
    of_oxm_ofdpa_bfd_discriminator_masked_value_get(obj, &val32);
    of_oxm_ofdpa_bfd_discriminator_masked_value_mask_get(obj, &val32);

    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_bfd_discriminator_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_check(
    of_oxm_ofdpa_bfd_discriminator_masked_t *obj, int value)
{

    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_bfd_discriminator_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_bfd_discriminator_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_bfd_discriminator_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_color
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_color_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_color_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_color_experimenter_get(obj, &val32);
    of_oxm_ofdpa_color_subtype_get(obj, &val16);
    of_oxm_ofdpa_color_value_get(obj, &val8);

    value = of_oxm_ofdpa_color_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_color
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_color_OF_VERSION_1_3_check(
    of_oxm_ofdpa_color_t *obj, int value)
{

    value = of_oxm_ofdpa_color_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_color_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_color_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_color_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_color_actions_index
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_color_actions_index_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_color_actions_index_experimenter_get(obj, &val32);
    of_oxm_ofdpa_color_actions_index_subtype_get(obj, &val16);
    of_oxm_ofdpa_color_actions_index_value_get(obj, &val32);

    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_color_actions_index
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_check(
    of_oxm_ofdpa_color_actions_index_t *obj, int value)
{

    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_color_actions_index_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_color_actions_index_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_color_actions_index_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_dei
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_dei_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_dei_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_dei_experimenter_get(obj, &val32);
    of_oxm_ofdpa_dei_subtype_get(obj, &val16);
    of_oxm_ofdpa_dei_value_get(obj, &val8);

    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_dei
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_dei_OF_VERSION_1_3_check(
    of_oxm_ofdpa_dei_t *obj, int value)
{

    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_dei_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_dei_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_dei_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_eth_subtype
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_eth_subtype_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_eth_subtype_experimenter_get(obj, &val32);
    of_oxm_ofdpa_eth_subtype_subtype_get(obj, &val16);
    of_oxm_ofdpa_eth_subtype_value_get(obj, &val8);

    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_eth_subtype
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_check(
    of_oxm_ofdpa_eth_subtype_t *obj, int value)
{

    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_eth_subtype_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_eth_subtype_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_eth_subtype_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_l3_in_port
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_l3_in_port_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_l3_in_port_experimenter_get(obj, &val32);
    of_oxm_ofdpa_l3_in_port_subtype_get(obj, &val16);
    of_oxm_ofdpa_l3_in_port_value_get(obj, &val32);

    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_l3_in_port
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_check(
    of_oxm_ofdpa_l3_in_port_t *obj, int value)
{

    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_l3_in_port_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_l3_in_port_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_l3_in_port_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_lmep_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_lmep_id_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_lmep_id_experimenter_get(obj, &val32);
    of_oxm_ofdpa_lmep_id_subtype_get(obj, &val16);
    of_oxm_ofdpa_lmep_id_value_get(obj, &val32);

    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_lmep_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_check(
    of_oxm_ofdpa_lmep_id_t *obj, int value)
{

    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_lmep_id_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_lmep_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_lmep_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_mpls_ach_channel
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_mpls_ach_channel_experimenter_get(obj, &val32);
    of_oxm_ofdpa_mpls_ach_channel_subtype_get(obj, &val16);
    of_oxm_ofdpa_mpls_ach_channel_value_get(obj, &val16);

    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_mpls_ach_channel
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_check(
    of_oxm_ofdpa_mpls_ach_channel_t *obj, int value)
{

    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_mpls_ach_channel_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_mpls_ach_channel_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_mpls_ach_channel_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_mpls_data_first_nibble
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_mpls_data_first_nibble_experimenter_get(obj, &val32);
    of_oxm_ofdpa_mpls_data_first_nibble_subtype_get(obj, &val16);
    of_oxm_ofdpa_mpls_data_first_nibble_value_get(obj, &val8);

    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_mpls_data_first_nibble
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_check(
    of_oxm_ofdpa_mpls_data_first_nibble_t *obj, int value)
{

    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_mpls_data_first_nibble_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_mpls_data_first_nibble_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_mpls_data_first_nibble_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_mpls_l2_port
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_mpls_l2_port_experimenter_get(obj, &val32);
    of_oxm_ofdpa_mpls_l2_port_subtype_get(obj, &val16);
    of_oxm_ofdpa_mpls_l2_port_value_get(obj, &val32);

    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_mpls_l2_port
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_check(
    of_oxm_ofdpa_mpls_l2_port_t *obj, int value)
{

    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_mpls_l2_port_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_mpls_l2_port_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_mpls_l2_port_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_mpls_l2_port_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_mpls_l2_port_masked_experimenter_get(obj, &val32);
    of_oxm_ofdpa_mpls_l2_port_masked_subtype_get(obj, &val16);
    of_oxm_ofdpa_mpls_l2_port_masked_value_get(obj, &val32);
    of_oxm_ofdpa_mpls_l2_port_masked_value_mask_get(obj, &val32);

    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_mpls_l2_port_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_check(
    of_oxm_ofdpa_mpls_l2_port_masked_t *obj, int value)
{

    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_mpls_l2_port_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_mpls_l2_port_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_mpls_l2_port_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_mpls_next_label_is_gal
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_mpls_next_label_is_gal_experimenter_get(obj, &val32);
    of_oxm_ofdpa_mpls_next_label_is_gal_subtype_get(obj, &val16);
    of_oxm_ofdpa_mpls_next_label_is_gal_value_get(obj, &val8);

    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_mpls_next_label_is_gal
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_check(
    of_oxm_ofdpa_mpls_next_label_is_gal_t *obj, int value)
{

    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_mpls_next_label_is_gal_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_mpls_next_label_is_gal_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_mpls_next_label_is_gal_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_mpls_ttl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_mpls_ttl_experimenter_get(obj, &val32);
    of_oxm_ofdpa_mpls_ttl_subtype_get(obj, &val16);
    of_oxm_ofdpa_mpls_ttl_value_get(obj, &val8);

    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_mpls_ttl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_check(
    of_oxm_ofdpa_mpls_ttl_t *obj, int value)
{

    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_mpls_ttl_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_mpls_ttl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_mpls_ttl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_oam_y1731_mdl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_oam_y1731_mdl_experimenter_get(obj, &val32);
    of_oxm_ofdpa_oam_y1731_mdl_subtype_get(obj, &val16);
    of_oxm_ofdpa_oam_y1731_mdl_value_get(obj, &val8);

    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_oam_y1731_mdl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_check(
    of_oxm_ofdpa_oam_y1731_mdl_t *obj, int value)
{

    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_oam_y1731_mdl_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_oam_y1731_mdl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_oam_y1731_mdl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_oam_y1731_opcode
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_oam_y1731_opcode_experimenter_get(obj, &val32);
    of_oxm_ofdpa_oam_y1731_opcode_subtype_get(obj, &val16);
    of_oxm_ofdpa_oam_y1731_opcode_value_get(obj, &val8);

    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_oam_y1731_opcode
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_check(
    of_oxm_ofdpa_oam_y1731_opcode_t *obj, int value)
{

    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_oam_y1731_opcode_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_oam_y1731_opcode_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_oam_y1731_opcode_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_ovid
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_ovid_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_ovid_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_ovid_experimenter_get(obj, &val32);
    of_oxm_ofdpa_ovid_subtype_get(obj, &val16);
    of_oxm_ofdpa_ovid_value_get(obj, &val16);

    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_ovid
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_ovid_OF_VERSION_1_3_check(
    of_oxm_ofdpa_ovid_t *obj, int value)
{

    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_ovid_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_ovid_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_ovid_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_protection_index
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_protection_index_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_protection_index_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_protection_index_experimenter_get(obj, &val32);
    of_oxm_ofdpa_protection_index_subtype_get(obj, &val16);
    of_oxm_ofdpa_protection_index_value_get(obj, &val8);

    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_protection_index
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_protection_index_OF_VERSION_1_3_check(
    of_oxm_ofdpa_protection_index_t *obj, int value)
{

    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_protection_index_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_protection_index_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_protection_index_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_qos_index
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_qos_index_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_qos_index_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_qos_index_experimenter_get(obj, &val32);
    of_oxm_ofdpa_qos_index_subtype_get(obj, &val16);
    of_oxm_ofdpa_qos_index_value_get(obj, &val8);

    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_qos_index
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_qos_index_OF_VERSION_1_3_check(
    of_oxm_ofdpa_qos_index_t *obj, int value)
{

    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_qos_index_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_qos_index_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_qos_index_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_rx_timestamp
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_rx_timestamp_experimenter_get(obj, &val32);
    of_oxm_ofdpa_rx_timestamp_subtype_get(obj, &val16);
    of_oxm_ofdpa_rx_timestamp_value_get(obj, &val64);

    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_rx_timestamp
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_check(
    of_oxm_ofdpa_rx_timestamp_t *obj, int value)
{

    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_rx_timestamp_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_rx_timestamp_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_rx_timestamp_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_rxfcl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_rxfcl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_rxfcl_experimenter_get(obj, &val32);
    of_oxm_ofdpa_rxfcl_subtype_get(obj, &val16);
    of_oxm_ofdpa_rxfcl_value_get(obj, &val64);

    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_rxfcl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_check(
    of_oxm_ofdpa_rxfcl_t *obj, int value)
{

    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_rxfcl_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_rxfcl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_rxfcl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_traffic_class
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_traffic_class_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_traffic_class_experimenter_get(obj, &val32);
    of_oxm_ofdpa_traffic_class_subtype_get(obj, &val16);
    of_oxm_ofdpa_traffic_class_value_get(obj, &val8);

    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_traffic_class
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_check(
    of_oxm_ofdpa_traffic_class_t *obj, int value)
{

    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_traffic_class_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_traffic_class_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_traffic_class_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_txfcl
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_txfcl_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_txfcl_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_txfcl_experimenter_get(obj, &val32);
    of_oxm_ofdpa_txfcl_subtype_get(obj, &val16);
    of_oxm_ofdpa_txfcl_value_get(obj, &val64);

    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_txfcl
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_txfcl_OF_VERSION_1_3_check(
    of_oxm_ofdpa_txfcl_t *obj, int value)
{

    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_txfcl_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_txfcl_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_txfcl_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_ofdpa_vrf
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_vrf_OF_VERSION_1_3_populate(
    of_oxm_ofdpa_vrf_t *obj, int value)
{
    uint32_t val32;
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_ofdpa_vrf_experimenter_get(obj, &val32);
    of_oxm_ofdpa_vrf_subtype_get(obj, &val16);
    of_oxm_ofdpa_vrf_value_get(obj, &val16);

    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_ofdpa_vrf
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_ofdpa_vrf_OF_VERSION_1_3_check(
    of_oxm_ofdpa_vrf_t *obj, int value)
{

    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_ofdpa_vrf_t *dup;

        TEST_ASSERT((dup = of_oxm_ofdpa_vrf_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_ofdpa_vrf_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_OF_VERSION_1_3_populate(
    of_oxm_sctp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_dst_value_get(obj, &val16);

    value = of_oxm_sctp_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_OF_VERSION_1_3_check(
    of_oxm_sctp_dst_t *obj, int value)
{

    value = of_oxm_sctp_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_masked_OF_VERSION_1_3_populate(
    of_oxm_sctp_dst_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_dst_masked_value_get(obj, &val16);
    of_oxm_sctp_dst_masked_value_mask_get(obj, &val16);

    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_dst_masked_OF_VERSION_1_3_check(
    of_oxm_sctp_dst_masked_t *obj, int value)
{

    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_OF_VERSION_1_3_populate(
    of_oxm_sctp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_src_value_get(obj, &val16);

    value = of_oxm_sctp_src_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_OF_VERSION_1_3_check(
    of_oxm_sctp_src_t *obj, int value)
{

    value = of_oxm_sctp_src_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_src_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_sctp_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_masked_OF_VERSION_1_3_populate(
    of_oxm_sctp_src_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_sctp_src_masked_value_get(obj, &val16);
    of_oxm_sctp_src_masked_value_mask_get(obj, &val16);

    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_sctp_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_sctp_src_masked_OF_VERSION_1_3_check(
    of_oxm_sctp_src_masked_t *obj, int value)
{

    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_sctp_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_sctp_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_sctp_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_OF_VERSION_1_3_populate(
    of_oxm_tcp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_dst_value_get(obj, &val16);

    value = of_oxm_tcp_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_OF_VERSION_1_3_check(
    of_oxm_tcp_dst_t *obj, int value)
{

    value = of_oxm_tcp_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_masked_OF_VERSION_1_3_populate(
    of_oxm_tcp_dst_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_dst_masked_value_get(obj, &val16);
    of_oxm_tcp_dst_masked_value_mask_get(obj, &val16);

    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_dst_masked_OF_VERSION_1_3_check(
    of_oxm_tcp_dst_masked_t *obj, int value)
{

    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_OF_VERSION_1_3_populate(
    of_oxm_tcp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_src_value_get(obj, &val16);

    value = of_oxm_tcp_src_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_OF_VERSION_1_3_check(
    of_oxm_tcp_src_t *obj, int value)
{

    value = of_oxm_tcp_src_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_src_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tcp_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_masked_OF_VERSION_1_3_populate(
    of_oxm_tcp_src_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tcp_src_masked_value_get(obj, &val16);
    of_oxm_tcp_src_masked_value_mask_get(obj, &val16);

    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tcp_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tcp_src_masked_OF_VERSION_1_3_check(
    of_oxm_tcp_src_masked_t *obj, int value)
{

    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tcp_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_tcp_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tcp_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tunnel_id
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_OF_VERSION_1_3_populate(
    of_oxm_tunnel_id_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tunnel_id_value_get(obj, &val64);

    value = of_oxm_tunnel_id_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tunnel_id
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_OF_VERSION_1_3_check(
    of_oxm_tunnel_id_t *obj, int value)
{

    value = of_oxm_tunnel_id_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tunnel_id_t *dup;

        TEST_ASSERT((dup = of_oxm_tunnel_id_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tunnel_id_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_tunnel_id_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_masked_OF_VERSION_1_3_populate(
    of_oxm_tunnel_id_masked_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_oxm_tunnel_id_masked_value_get(obj, &val64);
    of_oxm_tunnel_id_masked_value_mask_get(obj, &val64);

    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_tunnel_id_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_tunnel_id_masked_OF_VERSION_1_3_check(
    of_oxm_tunnel_id_masked_t *obj, int value)
{

    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_tunnel_id_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_tunnel_id_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_tunnel_id_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_dst
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_OF_VERSION_1_3_populate(
    of_oxm_udp_dst_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_dst_value_get(obj, &val16);

    value = of_oxm_udp_dst_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_dst
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_OF_VERSION_1_3_check(
    of_oxm_udp_dst_t *obj, int value)
{

    value = of_oxm_udp_dst_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_dst_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_dst_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_dst_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_dst_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_masked_OF_VERSION_1_3_populate(
    of_oxm_udp_dst_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_dst_masked_value_get(obj, &val16);
    of_oxm_udp_dst_masked_value_mask_get(obj, &val16);

    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_dst_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_dst_masked_OF_VERSION_1_3_check(
    of_oxm_udp_dst_masked_t *obj, int value)
{

    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_dst_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_dst_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_dst_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_src
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_OF_VERSION_1_3_populate(
    of_oxm_udp_src_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_src_value_get(obj, &val16);

    value = of_oxm_udp_src_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_src
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_OF_VERSION_1_3_check(
    of_oxm_udp_src_t *obj, int value)
{

    value = of_oxm_udp_src_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_src_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_src_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_src_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_udp_src_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_masked_OF_VERSION_1_3_populate(
    of_oxm_udp_src_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_udp_src_masked_value_get(obj, &val16);
    of_oxm_udp_src_masked_value_mask_get(obj, &val16);

    value = of_oxm_udp_src_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_udp_src_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_udp_src_masked_OF_VERSION_1_3_check(
    of_oxm_udp_src_masked_t *obj, int value)
{

    value = of_oxm_udp_src_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_udp_src_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_udp_src_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_udp_src_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_pcp
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_OF_VERSION_1_3_populate(
    of_oxm_vlan_pcp_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_pcp_value_get(obj, &val8);

    value = of_oxm_vlan_pcp_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_pcp
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_OF_VERSION_1_3_check(
    of_oxm_vlan_pcp_t *obj, int value)
{

    value = of_oxm_vlan_pcp_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_pcp_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_pcp_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_pcp_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_pcp_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_masked_OF_VERSION_1_3_populate(
    of_oxm_vlan_pcp_masked_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_pcp_masked_value_get(obj, &val8);
    of_oxm_vlan_pcp_masked_value_mask_get(obj, &val8);

    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_pcp_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_pcp_masked_OF_VERSION_1_3_check(
    of_oxm_vlan_pcp_masked_t *obj, int value)
{

    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_pcp_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_pcp_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_pcp_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_vid
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_OF_VERSION_1_3_populate(
    of_oxm_vlan_vid_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_vid_value_get(obj, &val16);

    value = of_oxm_vlan_vid_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_vid
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_OF_VERSION_1_3_check(
    of_oxm_vlan_vid_t *obj, int value)
{

    value = of_oxm_vlan_vid_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_vid_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_vid_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_vid_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_oxm_vlan_vid_masked
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_masked_OF_VERSION_1_3_populate(
    of_oxm_vlan_vid_masked_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_oxm_vlan_vid_masked_value_get(obj, &val16);
    of_oxm_vlan_vid_masked_value_mask_get(obj, &val16);

    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_oxm_vlan_vid_masked
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_oxm_vlan_vid_masked_OF_VERSION_1_3_check(
    of_oxm_vlan_vid_masked_t *obj, int value)
{

    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_oxm_vlan_vid_masked_t *dup;

        TEST_ASSERT((dup = of_oxm_vlan_vid_masked_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_oxm_vlan_vid_masked_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_packet_queue
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_3_populate(
    of_packet_queue_t *obj, int value)
{
    uint32_t val32;
    of_port_no_t port_no;
    of_list_queue_prop_t *list;

    /* Run thru accessors after new to ensure okay */
    of_packet_queue_queue_id_get(obj, &val32);
    of_packet_queue_port_get(obj, &port_no);
    {
        of_list_queue_prop_t sub_cls;

        /* Test bind */
        of_packet_queue_properties_bind(obj, &sub_cls);
    }

    value = of_packet_queue_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_queue_prop_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_queue_prop_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_packet_queue_properties_set(
        obj, list);
    of_list_queue_prop_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_packet_queue
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_packet_queue_OF_VERSION_1_3_check(
    of_packet_queue_t *obj, int value)
{

    value = of_packet_queue_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_queue_prop_t *properties_ptr;

        properties_ptr = of_packet_queue_properties_get(obj);
        TEST_ASSERT(properties_ptr != NULL);
        value = of_list_queue_prop_OF_VERSION_1_3_check(
            properties_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_queue_prop_delete(properties_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_packet_queue_t *dup;

        TEST_ASSERT((dup = of_packet_queue_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_packet_queue_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_desc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_3_populate(
    of_port_desc_t *obj, int value)
{
    of_port_no_t port_no;
    of_mac_addr_t mac_addr;
    of_port_name_t port_name;
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_port_desc_port_no_get(obj, &port_no);
    of_port_desc_hw_addr_get(obj, &mac_addr);
    of_port_desc_name_get(obj, &port_name);
    of_port_desc_config_get(obj, &val32);
    of_port_desc_state_get(obj, &val32);
    of_port_desc_curr_get(obj, &val32);
    of_port_desc_advertised_get(obj, &val32);
    of_port_desc_supported_get(obj, &val32);
    of_port_desc_peer_get(obj, &val32);
    of_port_desc_curr_speed_get(obj, &val32);
    of_port_desc_max_speed_get(obj, &val32);

    value = of_port_desc_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_desc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_desc_OF_VERSION_1_3_check(
    of_port_desc_t *obj, int value)
{

    value = of_port_desc_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_desc_t *dup;

        TEST_ASSERT((dup = of_port_desc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_desc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_port_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_3_populate(
    of_port_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint64_t val64;
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_port_stats_entry_port_no_get(obj, &port_no);
    of_port_stats_entry_rx_packets_get(obj, &val64);
    of_port_stats_entry_tx_packets_get(obj, &val64);
    of_port_stats_entry_rx_bytes_get(obj, &val64);
    of_port_stats_entry_tx_bytes_get(obj, &val64);
    of_port_stats_entry_rx_dropped_get(obj, &val64);
    of_port_stats_entry_tx_dropped_get(obj, &val64);
    of_port_stats_entry_rx_errors_get(obj, &val64);
    of_port_stats_entry_tx_errors_get(obj, &val64);
    of_port_stats_entry_rx_frame_err_get(obj, &val64);
    of_port_stats_entry_rx_over_err_get(obj, &val64);
    of_port_stats_entry_rx_crc_err_get(obj, &val64);
    of_port_stats_entry_collisions_get(obj, &val64);
    of_port_stats_entry_duration_sec_get(obj, &val32);
    of_port_stats_entry_duration_nsec_get(obj, &val32);

    value = of_port_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_port_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_port_stats_entry_OF_VERSION_1_3_check(
    of_port_stats_entry_t *obj, int value)
{

    value = of_port_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_port_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_port_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_experimenter_OF_VERSION_1_3_populate(
    of_queue_prop_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_experimenter_experimenter_get(obj, &val32);
    of_queue_prop_experimenter_data_get(obj, &octets);

    value = of_queue_prop_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_queue_prop_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_experimenter_OF_VERSION_1_3_check(
    of_queue_prop_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_queue_prop_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_queue_prop_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_experimenter_t *dup;

        TEST_ASSERT((dup = of_queue_prop_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_3_populate(
    of_queue_prop_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_queue_prop_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_header_OF_VERSION_1_3_check(
    of_queue_prop_header_t *obj, int value)
{

    value = of_queue_prop_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_header_t *dup;

        TEST_ASSERT((dup = of_queue_prop_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_max_rate
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_max_rate_OF_VERSION_1_3_populate(
    of_queue_prop_max_rate_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_max_rate_rate_get(obj, &val16);

    value = of_queue_prop_max_rate_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_max_rate
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_max_rate_OF_VERSION_1_3_check(
    of_queue_prop_max_rate_t *obj, int value)
{

    value = of_queue_prop_max_rate_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_max_rate_t *dup;

        TEST_ASSERT((dup = of_queue_prop_max_rate_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_max_rate_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_prop_min_rate
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_3_populate(
    of_queue_prop_min_rate_t *obj, int value)
{
    uint16_t val16;

    /* Run thru accessors after new to ensure okay */
    of_queue_prop_min_rate_rate_get(obj, &val16);

    value = of_queue_prop_min_rate_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_prop_min_rate
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_prop_min_rate_OF_VERSION_1_3_check(
    of_queue_prop_min_rate_t *obj, int value)
{

    value = of_queue_prop_min_rate_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_prop_min_rate_t *dup;

        TEST_ASSERT((dup = of_queue_prop_min_rate_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_prop_min_rate_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_queue_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_3_populate(
    of_queue_stats_entry_t *obj, int value)
{
    of_port_no_t port_no;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_queue_stats_entry_port_no_get(obj, &port_no);
    of_queue_stats_entry_queue_id_get(obj, &val32);
    of_queue_stats_entry_tx_bytes_get(obj, &val64);
    of_queue_stats_entry_tx_packets_get(obj, &val64);
    of_queue_stats_entry_tx_errors_get(obj, &val64);
    of_queue_stats_entry_duration_sec_get(obj, &val32);
    of_queue_stats_entry_duration_nsec_get(obj, &val32);

    value = of_queue_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_queue_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_queue_stats_entry_OF_VERSION_1_3_check(
    of_queue_stats_entry_t *obj, int value)
{

    value = of_queue_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_queue_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_queue_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_apply_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_actions_OF_VERSION_1_3_populate(
    of_table_feature_prop_apply_actions_t *obj, int value)
{
    of_list_action_id_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_id_t sub_cls;

        /* Test bind */
        of_table_feature_prop_apply_actions_action_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_id_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_id_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_apply_actions_action_ids_set(
        obj, list);
    of_list_action_id_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_apply_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_actions_OF_VERSION_1_3_check(
    of_table_feature_prop_apply_actions_t *obj, int value)
{

    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_id_t *action_ids_ptr;

        action_ids_ptr = of_table_feature_prop_apply_actions_action_ids_get(obj);
        TEST_ASSERT(action_ids_ptr != NULL);
        value = of_list_action_id_OF_VERSION_1_3_check(
            action_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_id_delete(action_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_apply_actions_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_apply_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_apply_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_apply_actions_miss
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_populate(
    of_table_feature_prop_apply_actions_miss_t *obj, int value)
{
    of_list_action_id_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_id_t sub_cls;

        /* Test bind */
        of_table_feature_prop_apply_actions_miss_action_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_id_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_id_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_apply_actions_miss_action_ids_set(
        obj, list);
    of_list_action_id_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_apply_actions_miss
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_check(
    of_table_feature_prop_apply_actions_miss_t *obj, int value)
{

    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_id_t *action_ids_ptr;

        action_ids_ptr = of_table_feature_prop_apply_actions_miss_action_ids_get(obj);
        TEST_ASSERT(action_ids_ptr != NULL);
        value = of_list_action_id_OF_VERSION_1_3_check(
            action_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_id_delete(action_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_apply_actions_miss_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_apply_actions_miss_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_apply_actions_miss_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_apply_setfield
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_setfield_OF_VERSION_1_3_populate(
    of_table_feature_prop_apply_setfield_t *obj, int value)
{
    of_list_uint32_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint32_t sub_cls;

        /* Test bind */
        of_table_feature_prop_apply_setfield_oxm_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint32_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint32_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_apply_setfield_oxm_ids_set(
        obj, list);
    of_list_uint32_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_apply_setfield
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_setfield_OF_VERSION_1_3_check(
    of_table_feature_prop_apply_setfield_t *obj, int value)
{

    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint32_t *oxm_ids_ptr;

        oxm_ids_ptr = of_table_feature_prop_apply_setfield_oxm_ids_get(obj);
        TEST_ASSERT(oxm_ids_ptr != NULL);
        value = of_list_uint32_OF_VERSION_1_3_check(
            oxm_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint32_delete(oxm_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_apply_setfield_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_apply_setfield_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_apply_setfield_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_apply_setfield_miss
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_populate(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value)
{
    of_list_uint32_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint32_t sub_cls;

        /* Test bind */
        of_table_feature_prop_apply_setfield_miss_oxm_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint32_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint32_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_apply_setfield_miss_oxm_ids_set(
        obj, list);
    of_list_uint32_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_apply_setfield_miss
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_check(
    of_table_feature_prop_apply_setfield_miss_t *obj, int value)
{

    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint32_t *oxm_ids_ptr;

        oxm_ids_ptr = of_table_feature_prop_apply_setfield_miss_oxm_ids_get(obj);
        TEST_ASSERT(oxm_ids_ptr != NULL);
        value = of_list_uint32_OF_VERSION_1_3_check(
            oxm_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint32_delete(oxm_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_apply_setfield_miss_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_apply_setfield_miss_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_apply_setfield_miss_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_experimenter
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_experimenter_OF_VERSION_1_3_populate(
    of_table_feature_prop_experimenter_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_table_feature_prop_experimenter_experimenter_get(obj, &val32);
    of_table_feature_prop_experimenter_subtype_get(obj, &val32);
    of_table_feature_prop_experimenter_experimenter_data_get(obj, &octets);

    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_experimenter_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_experimenter
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_experimenter_OF_VERSION_1_3_check(
    of_table_feature_prop_experimenter_t *obj, int value)
{
    of_octets_t octets;

    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_experimenter_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_experimenter_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_experimenter_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_experimenter_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_experimenter_miss
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_populate(
    of_table_feature_prop_experimenter_miss_t *obj, int value)
{
    uint32_t val32;
    of_octets_t octets;

    /* Run thru accessors after new to ensure okay */
    of_table_feature_prop_experimenter_miss_experimenter_get(obj, &val32);
    of_table_feature_prop_experimenter_miss_subtype_get(obj, &val32);
    of_table_feature_prop_experimenter_miss_experimenter_data_get(obj, &octets);

    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    value = of_octets_populate(&octets, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_experimenter_miss_experimenter_data_set(
        obj, &octets);
    if (octets.bytes) {
        FREE(octets.data);
    }

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_experimenter_miss
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_check(
    of_table_feature_prop_experimenter_miss_t *obj, int value)
{
    of_octets_t octets;

    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_experimenter_miss_experimenter_data_get(obj, &octets);
    value = of_octets_check(&octets, value);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_experimenter_miss_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_experimenter_miss_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_experimenter_miss_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_header
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_header_OF_VERSION_1_3_populate(
    of_table_feature_prop_header_t *obj, int value)
{

    /* Run thru accessors after new to ensure okay */

    value = of_table_feature_prop_header_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_header
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_header_OF_VERSION_1_3_check(
    of_table_feature_prop_header_t *obj, int value)
{

    value = of_table_feature_prop_header_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_header_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_header_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_header_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_instructions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_instructions_OF_VERSION_1_3_populate(
    of_table_feature_prop_instructions_t *obj, int value)
{
    of_list_instruction_id_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_instruction_id_t sub_cls;

        /* Test bind */
        of_table_feature_prop_instructions_instruction_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_instructions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_instruction_id_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_id_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_instructions_instruction_ids_set(
        obj, list);
    of_list_instruction_id_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_instructions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_instructions_OF_VERSION_1_3_check(
    of_table_feature_prop_instructions_t *obj, int value)
{

    value = of_table_feature_prop_instructions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_instruction_id_t *instruction_ids_ptr;

        instruction_ids_ptr = of_table_feature_prop_instructions_instruction_ids_get(obj);
        TEST_ASSERT(instruction_ids_ptr != NULL);
        value = of_list_instruction_id_OF_VERSION_1_3_check(
            instruction_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_id_delete(instruction_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_instructions_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_instructions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_instructions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_instructions_miss
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_instructions_miss_OF_VERSION_1_3_populate(
    of_table_feature_prop_instructions_miss_t *obj, int value)
{
    of_list_instruction_id_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_instruction_id_t sub_cls;

        /* Test bind */
        of_table_feature_prop_instructions_miss_instruction_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_instruction_id_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_instruction_id_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_instructions_miss_instruction_ids_set(
        obj, list);
    of_list_instruction_id_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_instructions_miss
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_instructions_miss_OF_VERSION_1_3_check(
    of_table_feature_prop_instructions_miss_t *obj, int value)
{

    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_instruction_id_t *instruction_ids_ptr;

        instruction_ids_ptr = of_table_feature_prop_instructions_miss_instruction_ids_get(obj);
        TEST_ASSERT(instruction_ids_ptr != NULL);
        value = of_list_instruction_id_OF_VERSION_1_3_check(
            instruction_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_instruction_id_delete(instruction_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_instructions_miss_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_instructions_miss_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_instructions_miss_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_match
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_match_OF_VERSION_1_3_populate(
    of_table_feature_prop_match_t *obj, int value)
{
    of_list_uint32_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint32_t sub_cls;

        /* Test bind */
        of_table_feature_prop_match_oxm_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_match_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint32_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint32_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_match_oxm_ids_set(
        obj, list);
    of_list_uint32_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_match
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_match_OF_VERSION_1_3_check(
    of_table_feature_prop_match_t *obj, int value)
{

    value = of_table_feature_prop_match_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint32_t *oxm_ids_ptr;

        oxm_ids_ptr = of_table_feature_prop_match_oxm_ids_get(obj);
        TEST_ASSERT(oxm_ids_ptr != NULL);
        value = of_list_uint32_OF_VERSION_1_3_check(
            oxm_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint32_delete(oxm_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_match_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_match_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_match_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_next_tables
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_next_tables_OF_VERSION_1_3_populate(
    of_table_feature_prop_next_tables_t *obj, int value)
{
    of_list_uint8_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint8_t sub_cls;

        /* Test bind */
        of_table_feature_prop_next_tables_next_table_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint8_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint8_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_next_tables_next_table_ids_set(
        obj, list);
    of_list_uint8_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_next_tables
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_next_tables_OF_VERSION_1_3_check(
    of_table_feature_prop_next_tables_t *obj, int value)
{

    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint8_t *next_table_ids_ptr;

        next_table_ids_ptr = of_table_feature_prop_next_tables_next_table_ids_get(obj);
        TEST_ASSERT(next_table_ids_ptr != NULL);
        value = of_list_uint8_OF_VERSION_1_3_check(
            next_table_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint8_delete(next_table_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_next_tables_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_next_tables_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_next_tables_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_next_tables_miss
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_populate(
    of_table_feature_prop_next_tables_miss_t *obj, int value)
{
    of_list_uint8_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint8_t sub_cls;

        /* Test bind */
        of_table_feature_prop_next_tables_miss_next_table_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint8_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint8_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_next_tables_miss_next_table_ids_set(
        obj, list);
    of_list_uint8_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_next_tables_miss
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_check(
    of_table_feature_prop_next_tables_miss_t *obj, int value)
{

    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint8_t *next_table_ids_ptr;

        next_table_ids_ptr = of_table_feature_prop_next_tables_miss_next_table_ids_get(obj);
        TEST_ASSERT(next_table_ids_ptr != NULL);
        value = of_list_uint8_OF_VERSION_1_3_check(
            next_table_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint8_delete(next_table_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_next_tables_miss_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_next_tables_miss_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_next_tables_miss_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_wildcards
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_wildcards_OF_VERSION_1_3_populate(
    of_table_feature_prop_wildcards_t *obj, int value)
{
    of_list_uint32_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint32_t sub_cls;

        /* Test bind */
        of_table_feature_prop_wildcards_oxm_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint32_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint32_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_wildcards_oxm_ids_set(
        obj, list);
    of_list_uint32_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_wildcards
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_wildcards_OF_VERSION_1_3_check(
    of_table_feature_prop_wildcards_t *obj, int value)
{

    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint32_t *oxm_ids_ptr;

        oxm_ids_ptr = of_table_feature_prop_wildcards_oxm_ids_get(obj);
        TEST_ASSERT(oxm_ids_ptr != NULL);
        value = of_list_uint32_OF_VERSION_1_3_check(
            oxm_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint32_delete(oxm_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_wildcards_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_wildcards_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_wildcards_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_write_actions
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_actions_OF_VERSION_1_3_populate(
    of_table_feature_prop_write_actions_t *obj, int value)
{
    of_list_action_id_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_id_t sub_cls;

        /* Test bind */
        of_table_feature_prop_write_actions_action_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_id_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_id_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_write_actions_action_ids_set(
        obj, list);
    of_list_action_id_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_write_actions
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_actions_OF_VERSION_1_3_check(
    of_table_feature_prop_write_actions_t *obj, int value)
{

    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_id_t *action_ids_ptr;

        action_ids_ptr = of_table_feature_prop_write_actions_action_ids_get(obj);
        TEST_ASSERT(action_ids_ptr != NULL);
        value = of_list_action_id_OF_VERSION_1_3_check(
            action_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_id_delete(action_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_write_actions_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_write_actions_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_write_actions_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_write_actions_miss
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_populate(
    of_table_feature_prop_write_actions_miss_t *obj, int value)
{
    of_list_action_id_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_action_id_t sub_cls;

        /* Test bind */
        of_table_feature_prop_write_actions_miss_action_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_action_id_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_action_id_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_write_actions_miss_action_ids_set(
        obj, list);
    of_list_action_id_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_write_actions_miss
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_check(
    of_table_feature_prop_write_actions_miss_t *obj, int value)
{

    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_action_id_t *action_ids_ptr;

        action_ids_ptr = of_table_feature_prop_write_actions_miss_action_ids_get(obj);
        TEST_ASSERT(action_ids_ptr != NULL);
        value = of_list_action_id_OF_VERSION_1_3_check(
            action_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_action_id_delete(action_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_write_actions_miss_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_write_actions_miss_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_write_actions_miss_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_write_setfield
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_setfield_OF_VERSION_1_3_populate(
    of_table_feature_prop_write_setfield_t *obj, int value)
{
    of_list_uint32_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint32_t sub_cls;

        /* Test bind */
        of_table_feature_prop_write_setfield_oxm_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint32_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint32_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_write_setfield_oxm_ids_set(
        obj, list);
    of_list_uint32_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_write_setfield
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_setfield_OF_VERSION_1_3_check(
    of_table_feature_prop_write_setfield_t *obj, int value)
{

    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint32_t *oxm_ids_ptr;

        oxm_ids_ptr = of_table_feature_prop_write_setfield_oxm_ids_get(obj);
        TEST_ASSERT(oxm_ids_ptr != NULL);
        value = of_list_uint32_OF_VERSION_1_3_check(
            oxm_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint32_delete(oxm_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_write_setfield_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_write_setfield_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_write_setfield_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_feature_prop_write_setfield_miss
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_populate(
    of_table_feature_prop_write_setfield_miss_t *obj, int value)
{
    of_list_uint32_t *list;

    /* Run thru accessors after new to ensure okay */
    {
        of_list_uint32_t sub_cls;

        /* Test bind */
        of_table_feature_prop_write_setfield_miss_oxm_ids_bind(obj, &sub_cls);
    }

    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_uint32_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_uint32_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_feature_prop_write_setfield_miss_oxm_ids_set(
        obj, list);
    of_list_uint32_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_feature_prop_write_setfield_miss
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_check(
    of_table_feature_prop_write_setfield_miss_t *obj, int value)
{

    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_uint32_t *oxm_ids_ptr;

        oxm_ids_ptr = of_table_feature_prop_write_setfield_miss_oxm_ids_get(obj);
        TEST_ASSERT(oxm_ids_ptr != NULL);
        value = of_list_uint32_OF_VERSION_1_3_check(
            oxm_ids_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_uint32_delete(oxm_ids_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_feature_prop_write_setfield_miss_t *dup;

        TEST_ASSERT((dup = of_table_feature_prop_write_setfield_miss_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_feature_prop_write_setfield_miss_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_features
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_features_OF_VERSION_1_3_populate(
    of_table_features_t *obj, int value)
{
    uint8_t val8;
    of_table_name_t table_name;
    uint64_t val64;
    uint32_t val32;
    of_list_table_feature_prop_t *list;

    /* Run thru accessors after new to ensure okay */
    of_table_features_table_id_get(obj, &val8);
    of_table_features_name_get(obj, &table_name);
    of_table_features_metadata_match_get(obj, &val64);
    of_table_features_metadata_write_get(obj, &val64);
    of_table_features_config_get(obj, &val32);
    of_table_features_max_entries_get(obj, &val32);
    {
        of_list_table_feature_prop_t sub_cls;

        /* Test bind */
        of_table_features_properties_bind(obj, &sub_cls);
    }

    value = of_table_features_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    list = of_list_table_feature_prop_new(OF_VERSION_1_3);
    TEST_ASSERT(list != NULL);
    value = of_list_table_feature_prop_OF_VERSION_1_3_populate(
        list, value);
    TEST_ASSERT(value != 0);
    of_table_features_properties_set(
        obj, list);
    of_list_table_feature_prop_delete(list);

    return value;
}

/**
 * Check all members of an object of type of_table_features
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_features_OF_VERSION_1_3_check(
    of_table_features_t *obj, int value)
{

    value = of_table_features_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    { /* Use get/delete to access on check */
        of_list_table_feature_prop_t *properties_ptr;

        properties_ptr = of_table_features_properties_get(obj);
        TEST_ASSERT(properties_ptr != NULL);
        value = of_list_table_feature_prop_OF_VERSION_1_3_check(
            properties_ptr, value);
        TEST_ASSERT(value != 0);
        of_list_table_feature_prop_delete(properties_ptr);
    }

    /* We shoehorn a test of the dup functions here */
    {
        of_table_features_t *dup;

        TEST_ASSERT((dup = of_table_features_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_features_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_table_stats_entry
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_3_populate(
    of_table_stats_entry_t *obj, int value)
{
    uint8_t val8;
    uint32_t val32;
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_table_stats_entry_table_id_get(obj, &val8);
    of_table_stats_entry_active_count_get(obj, &val32);
    of_table_stats_entry_lookup_count_get(obj, &val64);
    of_table_stats_entry_matched_count_get(obj, &val64);

    value = of_table_stats_entry_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_table_stats_entry
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_table_stats_entry_OF_VERSION_1_3_check(
    of_table_stats_entry_t *obj, int value)
{

    value = of_table_stats_entry_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_table_stats_entry_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_table_stats_entry_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_uint32
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_uint32_OF_VERSION_1_3_populate(
    of_uint32_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    of_uint32_value_get(obj, &val32);

    value = of_uint32_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_uint32
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_uint32_OF_VERSION_1_3_check(
    of_uint32_t *obj, int value)
{

    value = of_uint32_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_uint32_t *dup;

        TEST_ASSERT((dup = of_uint32_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_uint32_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_uint64
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_uint64_OF_VERSION_1_3_populate(
    of_uint64_t *obj, int value)
{
    uint64_t val64;

    /* Run thru accessors after new to ensure okay */
    of_uint64_value_get(obj, &val64);

    value = of_uint64_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_uint64
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_uint64_OF_VERSION_1_3_check(
    of_uint64_t *obj, int value)
{

    value = of_uint64_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_uint64_t *dup;

        TEST_ASSERT((dup = of_uint64_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_uint64_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type of_uint8
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
of_uint8_OF_VERSION_1_3_populate(
    of_uint8_t *obj, int value)
{
    uint8_t val8;

    /* Run thru accessors after new to ensure okay */
    of_uint8_value_get(obj, &val8);

    value = of_uint8_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type of_uint8
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
of_uint8_OF_VERSION_1_3_check(
    of_uint8_t *obj, int value)
{

    value = of_uint8_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        of_uint8_t *dup;

        TEST_ASSERT((dup = of_uint8_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_uint8_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate all members of an object of type ofp_queue_desc
 * with incrementing values
 * @param obj Pointer to an object to populate
 * @param value The seed value to use in populating the object
 * @returns The value after increments for this object's values
 */

int
ofp_queue_desc_OF_VERSION_1_3_populate(
    ofp_queue_desc_t *obj, int value)
{
    uint32_t val32;

    /* Run thru accessors after new to ensure okay */
    ofp_queue_desc_port_no_get(obj, &val32);
    ofp_queue_desc_queue_id_get(obj, &val32);

    value = ofp_queue_desc_OF_VERSION_1_3_populate_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    return value;
}

/**
 * Check all members of an object of type ofp_queue_desc
 * populated by the above function
 * @param obj Pointer to an object to check
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */

int
ofp_queue_desc_OF_VERSION_1_3_check(
    ofp_queue_desc_t *obj, int value)
{

    value = ofp_queue_desc_OF_VERSION_1_3_check_scalars(
        obj, value);
    TEST_ASSERT(value != 0);

    /* We shoehorn a test of the dup functions here */
    {
        ofp_queue_desc_t *dup;

        TEST_ASSERT((dup = ofp_queue_desc_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (ofp_queue_desc_t *)
            of_object_dup(obj)) != NULL);
        TEST_ASSERT(dup->length == obj->length);
        TEST_ASSERT(dup->object_id == obj->object_id);
        TEST_ASSERT(dup->version == obj->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(obj, 0), obj->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_action with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_3_populate(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_pop_pbb_t *pop_pbb;
    of_action_group_t *group;
    of_action_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_push_pbb_t *push_pbb;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_push_mpls_t *push_mpls;
    of_action_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_set_queue_t *set_queue;
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    pop_pbb = &elt.pop_pbb;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_queue = &elt.set_queue;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;

    /* Append two instances of type of_action_ofdpa_set_pcpdei_from_table */

    of_action_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_out */

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_out);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_mirror */

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_mirror);
    value = of_action_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_field */

    of_action_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_field);
    value = of_action_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_push_cw */

    of_action_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_cw);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_cw);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_copy_tc_in */

    of_action_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_in);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_in);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_dec_ttl_mtu */

    of_action_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_dec_ttl_mtu);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_dec_ttl_mtu);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_mpls_ttl */

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_mpls_ttl);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_bsn_set_tunnel_dst */

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)bsn_set_tunnel_dst);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_copy_tc_out */

    of_action_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_out);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_copy_tc_out);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_dscp_from_table */

    of_action_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_dscp_from_table);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_dscp_from_table);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_pbb */

    of_action_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_pbb);
    value = of_action_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_pbb);
    value = of_action_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_group */

    of_action_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)group);
    value = of_action_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_push_l2hdr */

    of_action_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_l2hdr);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_push_l2hdr);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_color_based_rx_count */

    of_action_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_rx_count);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_rx_count);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_oam_lm_tx_count */

    of_action_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_tx_count);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_tx_count);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_mpls_ttl */

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_mpls_ttl);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_vlan */

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_vlan);
    value = of_action_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_counter_fields */

    of_action_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_counter_fields);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_counter_fields);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_tc_from_vpn_table */

    of_action_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_pop_cw */

    of_action_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_cw);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_cw);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_qos_from_table */

    of_action_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_pbb */

    of_action_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_pbb);
    value = of_action_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_pbb);
    value = of_action_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_nicira_dec_ttl */

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)nicira_dec_ttl);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_oam_lm_rx_count */

    of_action_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_rx_count);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_oam_lm_rx_count);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_color_based_tx_count */

    of_action_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_tx_count);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_color_based_tx_count);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_pop_mpls */

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)pop_mpls);
    value = of_action_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_vlan */

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_vlan);
    value = of_action_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_pcpdei_from_table */

    of_action_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_tc_from_tunnel_table */

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_nw_ttl */

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_nw_ttl);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_copy_ttl_in */

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)copy_ttl_in);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_pop_l2hdr */

    of_action_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_l2hdr);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_pop_l2hdr);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_push_mpls */

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)push_mpls);
    value = of_action_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_class_based_rx_count */

    of_action_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_rx_count);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_rx_count);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_dec_nw_ttl */

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)dec_nw_ttl);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_class_based_tx_count */

    of_action_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_tx_count);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_class_based_tx_count);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_check_drop_status */

    of_action_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_check_drop_status);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_check_drop_status);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_set_queue */

    of_action_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)set_queue);
    value = of_action_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table */

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_output */

    of_action_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_append_bind(list,
            (of_action_t *)output);
    value = of_action_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_action generated by
 * of_list_action_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_action_OF_VERSION_1_3_check(
    of_list_action_t *list, int value)
{

    of_action_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_action_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_copy_ttl_out_t *copy_ttl_out;
    of_action_bsn_mirror_t *bsn_mirror;
    of_action_set_field_t *set_field;
    of_action_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_pop_pbb_t *pop_pbb;
    of_action_group_t *group;
    of_action_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_set_mpls_ttl_t *set_mpls_ttl;
    of_action_pop_vlan_t *pop_vlan;
    of_action_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_push_pbb_t *push_pbb;
    of_action_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_pop_mpls_t *pop_mpls;
    of_action_push_vlan_t *push_vlan;
    of_action_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_set_nw_ttl_t *set_nw_ttl;
    of_action_copy_ttl_in_t *copy_ttl_in;
    of_action_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_push_mpls_t *push_mpls;
    of_action_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_dec_nw_ttl_t *dec_nw_ttl;
    of_action_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_set_queue_t *set_queue;
    of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    set_field = &elt.set_field;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    pop_pbb = &elt.pop_pbb;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    copy_ttl_in = &elt.copy_ttl_in;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_queue = &elt.set_queue;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;
    TEST_OK(of_list_action_first(list, &elt));

    /* Check two instances of type ofdpa_set_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 16);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 16);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 8);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_COPY_TTL_OUT);
    value = of_action_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 24);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_BSN_MIRROR);
    value = of_action_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_field */
    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_field->object_id == OF_ACTION_SET_FIELD);
    value = of_action_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_push_cw */
    TEST_ASSERT(ofdpa_push_cw->length == 16);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_OFDPA_PUSH_CW);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_push_cw->length == 16);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_OFDPA_PUSH_CW);
    value = of_action_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_in */
    TEST_ASSERT(ofdpa_copy_tc_in->length == 16);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_OFDPA_COPY_TC_IN);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_in->length == 16);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_OFDPA_COPY_TC_IN);
    value = of_action_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_dec_ttl_mtu */
    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 16);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_OFDPA_DEC_TTL_MTU);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 16);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_OFDPA_DEC_TTL_MTU);
    value = of_action_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 8);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_DEC_MPLS_TTL);
    value = of_action_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 16);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_BSN_SET_TUNNEL_DST);
    value = of_action_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_out */
    TEST_ASSERT(ofdpa_copy_tc_out->length == 16);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_OFDPA_COPY_TC_OUT);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_out->length == 16);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_OFDPA_COPY_TC_OUT);
    value = of_action_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_dscp_from_table */
    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 16);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 16);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_pbb */
    TEST_ASSERT(pop_pbb->length == 8);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_POP_PBB);
    value = of_action_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_pbb->length == 8);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_POP_PBB);
    value = of_action_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(group->length == 8);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_GROUP);
    value = of_action_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_push_l2hdr */
    TEST_ASSERT(ofdpa_push_l2hdr->length == 16);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_OFDPA_PUSH_L2HDR);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_push_l2hdr->length == 16);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_OFDPA_PUSH_L2HDR);
    value = of_action_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_rx_count */
    TEST_ASSERT(ofdpa_color_based_rx_count->length == 16);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_rx_count->length == 16);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_tx_count */
    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 16);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 16);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 8);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_SET_MPLS_TTL);
    value = of_action_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 8);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_POP_VLAN);
    value = of_action_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_counter_fields */
    TEST_ASSERT(ofdpa_set_counter_fields->length == 18);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_counter_fields->length == 18);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_vpn_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 16);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 16);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_pop_cw */
    TEST_ASSERT(ofdpa_pop_cw->length == 16);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_OFDPA_POP_CW);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_cw->length == 16);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_OFDPA_POP_CW);
    value = of_action_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_qos_from_table */
    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 16);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 16);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_pbb */
    TEST_ASSERT(push_pbb->length == 8);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_PUSH_PBB);
    value = of_action_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_pbb->length == 8);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_PUSH_PBB);
    value = of_action_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 16);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_NICIRA_DEC_TTL);
    value = of_action_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_rx_count */
    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 16);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 16);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_tx_count */
    TEST_ASSERT(ofdpa_color_based_tx_count->length == 16);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_tx_count->length == 16);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 8);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_POP_MPLS);
    value = of_action_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 8);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_PUSH_VLAN);
    value = of_action_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 8);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_SET_NW_TTL);
    value = of_action_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 8);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_COPY_TTL_IN);
    value = of_action_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_pop_l2hdr */
    TEST_ASSERT(ofdpa_pop_l2hdr->length == 16);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_OFDPA_POP_L2HDR);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_l2hdr->length == 16);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_OFDPA_POP_L2HDR);
    value = of_action_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 8);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_PUSH_MPLS);
    value = of_action_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_rx_count */
    TEST_ASSERT(ofdpa_class_based_rx_count->length == 16);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_rx_count->length == 16);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 8);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_DEC_NW_TTL);
    value = of_action_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_tx_count */
    TEST_ASSERT(ofdpa_class_based_tx_count->length == 16);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_tx_count->length == 16);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_check_drop_status */
    TEST_ASSERT(ofdpa_check_drop_status->length == 16);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_OFDPA_CHECK_DROP_STATUS);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_check_drop_status->length == 16);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_OFDPA_CHECK_DROP_STATUS);
    value = of_action_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(set_queue->length == 8);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_SET_QUEUE);
    value = of_action_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 16);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_next(list, &elt));

    TEST_ASSERT(output->length == 16);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_OUTPUT);
    value = of_action_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_ACTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 82);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_action_t *dup;

        TEST_ASSERT((dup = of_list_action_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_action_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_action_id with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_action_id_OF_VERSION_1_3_populate(
    of_list_action_id_t *list, int value)
{

    of_action_id_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_action_id_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_id_copy_ttl_out_t *copy_ttl_out;
    of_action_id_bsn_mirror_t *bsn_mirror;
    of_action_id_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_id_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_id_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_id_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_id_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_id_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_id_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_id_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_id_copy_ttl_in_t *copy_ttl_in;
    of_action_id_group_t *group;
    of_action_id_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_id_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_id_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_id_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_id_set_field_t *set_field;
    of_action_id_set_mpls_ttl_t *set_mpls_ttl;
    of_action_id_pop_vlan_t *pop_vlan;
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_id_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_id_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_id_push_pbb_t *push_pbb;
    of_action_id_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_id_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_id_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_id_pop_mpls_t *pop_mpls;
    of_action_id_push_vlan_t *push_vlan;
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_id_set_nw_ttl_t *set_nw_ttl;
    of_action_id_pop_pbb_t *pop_pbb;
    of_action_id_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_id_set_queue_t *set_queue;
    of_action_id_push_mpls_t *push_mpls;
    of_action_id_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_id_dec_nw_ttl_t *dec_nw_ttl;
    of_action_id_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_id_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    copy_ttl_in = &elt.copy_ttl_in;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_field = &elt.set_field;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    pop_pbb = &elt.pop_pbb;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    set_queue = &elt.set_queue;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;

    /* Append two instances of type of_action_id_ofdpa_set_pcpdei_from_table */

    of_action_id_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_pcpdei_from_table_init(ofdpa_set_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_pcpdei_from_table);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_pcpdei_from_table, value);
    cur_len += ofdpa_set_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_copy_ttl_out */

    of_action_id_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_out);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_copy_ttl_out_init(copy_ttl_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_out);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_populate(
        copy_ttl_out, value);
    cur_len += copy_ttl_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_bsn_mirror */

    of_action_id_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_mirror);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_bsn_mirror_init(bsn_mirror, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_mirror);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_populate(
        bsn_mirror, value);
    cur_len += bsn_mirror->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_push_cw */

    of_action_id_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_cw);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_push_cw_init(ofdpa_push_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_cw);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_populate(
        ofdpa_push_cw, value);
    cur_len += ofdpa_push_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_copy_tc_in */

    of_action_id_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_in);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_copy_tc_in_init(ofdpa_copy_tc_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_in);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_in, value);
    cur_len += ofdpa_copy_tc_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_dec_ttl_mtu */

    of_action_id_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_dec_ttl_mtu);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_dec_ttl_mtu_init(ofdpa_dec_ttl_mtu, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_dec_ttl_mtu);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_populate(
        ofdpa_dec_ttl_mtu, value);
    cur_len += ofdpa_dec_ttl_mtu->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_dec_mpls_ttl */

    of_action_id_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_mpls_ttl);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_dec_mpls_ttl_init(dec_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_mpls_ttl);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_populate(
        dec_mpls_ttl, value);
    cur_len += dec_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_bsn_set_tunnel_dst */

    of_action_id_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_set_tunnel_dst);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_bsn_set_tunnel_dst_init(bsn_set_tunnel_dst, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)bsn_set_tunnel_dst);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_populate(
        bsn_set_tunnel_dst, value);
    cur_len += bsn_set_tunnel_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_oam_lm_tx_count */

    of_action_id_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_tx_count);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_oam_lm_tx_count_init(ofdpa_oam_lm_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_tx_count);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_tx_count, value);
    cur_len += ofdpa_oam_lm_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_copy_tc_out */

    of_action_id_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_out);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_copy_tc_out_init(ofdpa_copy_tc_out, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_copy_tc_out);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_populate(
        ofdpa_copy_tc_out, value);
    cur_len += ofdpa_copy_tc_out->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_dscp_from_table */

    of_action_id_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_dscp_from_table);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_dscp_from_table_init(ofdpa_set_dscp_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_dscp_from_table);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_dscp_from_table, value);
    cur_len += ofdpa_set_dscp_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_copy_ttl_in */

    of_action_id_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_in);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_copy_ttl_in_init(copy_ttl_in, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)copy_ttl_in);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_populate(
        copy_ttl_in, value);
    cur_len += copy_ttl_in->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_group */

    of_action_id_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)group);
    value = of_action_id_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_group_init(group, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)group);
    value = of_action_id_group_OF_VERSION_1_3_populate(
        group, value);
    cur_len += group->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_push_l2hdr */

    of_action_id_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_l2hdr);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_push_l2hdr_init(ofdpa_push_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_push_l2hdr);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_push_l2hdr, value);
    cur_len += ofdpa_push_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_color_based_rx_count */

    of_action_id_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_rx_count);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_color_based_rx_count_init(ofdpa_color_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_rx_count);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_rx_count, value);
    cur_len += ofdpa_color_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_counter_fields */

    of_action_id_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_counter_fields);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_counter_fields_init(ofdpa_set_counter_fields, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_counter_fields);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_populate(
        ofdpa_set_counter_fields, value);
    cur_len += ofdpa_set_counter_fields->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_check_drop_status */

    of_action_id_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_check_drop_status);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_check_drop_status_init(ofdpa_check_drop_status, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_check_drop_status);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_populate(
        ofdpa_check_drop_status, value);
    cur_len += ofdpa_check_drop_status->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_field */

    of_action_id_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_field);
    value = of_action_id_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_field_init(set_field, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_field);
    value = of_action_id_set_field_OF_VERSION_1_3_populate(
        set_field, value);
    cur_len += set_field->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_mpls_ttl */

    of_action_id_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_mpls_ttl);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_mpls_ttl_init(set_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_mpls_ttl);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_populate(
        set_mpls_ttl, value);
    cur_len += set_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_pop_vlan */

    of_action_id_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_vlan);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_pop_vlan_init(pop_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_vlan);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_populate(
        pop_vlan, value);
    cur_len += pop_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_tc_from_vpn_table */

    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_init(ofdpa_set_mpls_tc_from_vpn_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_vpn_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    cur_len += ofdpa_set_mpls_tc_from_vpn_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_pop_cw */

    of_action_id_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_cw);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_pop_cw_init(ofdpa_pop_cw, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_cw);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_populate(
        ofdpa_pop_cw, value);
    cur_len += ofdpa_pop_cw->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_qos_from_table */

    of_action_id_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_qos_from_table_init(ofdpa_set_mpls_qos_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_qos_from_table);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_qos_from_table, value);
    cur_len += ofdpa_set_mpls_qos_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_push_pbb */

    of_action_id_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_pbb);
    value = of_action_id_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_push_pbb_init(push_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_pbb);
    value = of_action_id_push_pbb_OF_VERSION_1_3_populate(
        push_pbb, value);
    cur_len += push_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_nicira_dec_ttl */

    of_action_id_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)nicira_dec_ttl);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_nicira_dec_ttl_init(nicira_dec_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)nicira_dec_ttl);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_populate(
        nicira_dec_ttl, value);
    cur_len += nicira_dec_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_oam_lm_rx_count */

    of_action_id_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_rx_count);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_oam_lm_rx_count_init(ofdpa_oam_lm_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_oam_lm_rx_count);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_populate(
        ofdpa_oam_lm_rx_count, value);
    cur_len += ofdpa_oam_lm_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_color_based_tx_count */

    of_action_id_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_tx_count);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_color_based_tx_count_init(ofdpa_color_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_color_based_tx_count);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_color_based_tx_count, value);
    cur_len += ofdpa_color_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_pop_mpls */

    of_action_id_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_mpls);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_pop_mpls_init(pop_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_mpls);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_populate(
        pop_mpls, value);
    cur_len += pop_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_push_vlan */

    of_action_id_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_vlan);
    value = of_action_id_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_push_vlan_init(push_vlan, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_vlan);
    value = of_action_id_push_vlan_OF_VERSION_1_3_populate(
        push_vlan, value);
    cur_len += push_vlan->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_pcpdei_from_table */

    of_action_id_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_pcpdei_from_table_init(ofdpa_set_mpls_pcpdei_from_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_tc_from_tunnel_table */

    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_init(ofdpa_set_mpls_tc_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_tc_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_tc_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_nw_ttl */

    of_action_id_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_nw_ttl);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_nw_ttl_init(set_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_nw_ttl);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_populate(
        set_nw_ttl, value);
    cur_len += set_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_pop_pbb */

    of_action_id_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_pbb);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_pop_pbb_init(pop_pbb, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)pop_pbb);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_populate(
        pop_pbb, value);
    cur_len += pop_pbb->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_pop_l2hdr */

    of_action_id_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_l2hdr);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_pop_l2hdr_init(ofdpa_pop_l2hdr, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_pop_l2hdr);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_populate(
        ofdpa_pop_l2hdr, value);
    cur_len += ofdpa_pop_l2hdr->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_set_queue */

    of_action_id_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_queue);
    value = of_action_id_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_set_queue_init(set_queue, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)set_queue);
    value = of_action_id_set_queue_OF_VERSION_1_3_populate(
        set_queue, value);
    cur_len += set_queue->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_push_mpls */

    of_action_id_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_mpls);
    value = of_action_id_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_push_mpls_init(push_mpls, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)push_mpls);
    value = of_action_id_push_mpls_OF_VERSION_1_3_populate(
        push_mpls, value);
    cur_len += push_mpls->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_class_based_rx_count */

    of_action_id_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_rx_count);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_class_based_rx_count_init(ofdpa_class_based_rx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_rx_count);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_rx_count, value);
    cur_len += ofdpa_class_based_rx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_dec_nw_ttl */

    of_action_id_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_nw_ttl);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_dec_nw_ttl_init(dec_nw_ttl, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)dec_nw_ttl);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_populate(
        dec_nw_ttl, value);
    cur_len += dec_nw_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_class_based_tx_count */

    of_action_id_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_tx_count);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_class_based_tx_count_init(ofdpa_class_based_tx_count, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_class_based_tx_count);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_populate(
        ofdpa_class_based_tx_count, value);
    cur_len += ofdpa_class_based_tx_count->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table */

    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_init(ofdpa_set_mpls_pcpdei_from_tunnel_table, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)ofdpa_set_mpls_pcpdei_from_tunnel_table);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_populate(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    cur_len += ofdpa_set_mpls_pcpdei_from_tunnel_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_action_id_output */

    of_action_id_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)output);
    value = of_action_id_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    of_action_id_output_init(output, OF_VERSION_1_3, -1, 1);
    of_list_action_id_append_bind(list,
            (of_action_id_t *)output);
    value = of_action_id_output_OF_VERSION_1_3_populate(
        output, value);
    cur_len += output->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_action_id generated by
 * of_list_action_id_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_action_id_OF_VERSION_1_3_check(
    of_list_action_id_t *list, int value)
{

    of_action_id_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_action_id_ofdpa_set_pcpdei_from_table_t *ofdpa_set_pcpdei_from_table;
    of_action_id_copy_ttl_out_t *copy_ttl_out;
    of_action_id_bsn_mirror_t *bsn_mirror;
    of_action_id_ofdpa_push_cw_t *ofdpa_push_cw;
    of_action_id_ofdpa_copy_tc_in_t *ofdpa_copy_tc_in;
    of_action_id_ofdpa_dec_ttl_mtu_t *ofdpa_dec_ttl_mtu;
    of_action_id_dec_mpls_ttl_t *dec_mpls_ttl;
    of_action_id_bsn_set_tunnel_dst_t *bsn_set_tunnel_dst;
    of_action_id_ofdpa_oam_lm_tx_count_t *ofdpa_oam_lm_tx_count;
    of_action_id_ofdpa_copy_tc_out_t *ofdpa_copy_tc_out;
    of_action_id_ofdpa_set_dscp_from_table_t *ofdpa_set_dscp_from_table;
    of_action_id_copy_ttl_in_t *copy_ttl_in;
    of_action_id_group_t *group;
    of_action_id_ofdpa_push_l2hdr_t *ofdpa_push_l2hdr;
    of_action_id_ofdpa_color_based_rx_count_t *ofdpa_color_based_rx_count;
    of_action_id_ofdpa_set_counter_fields_t *ofdpa_set_counter_fields;
    of_action_id_ofdpa_check_drop_status_t *ofdpa_check_drop_status;
    of_action_id_set_field_t *set_field;
    of_action_id_set_mpls_ttl_t *set_mpls_ttl;
    of_action_id_pop_vlan_t *pop_vlan;
    of_action_id_ofdpa_set_mpls_tc_from_vpn_table_t *ofdpa_set_mpls_tc_from_vpn_table;
    of_action_id_ofdpa_pop_cw_t *ofdpa_pop_cw;
    of_action_id_ofdpa_set_mpls_qos_from_table_t *ofdpa_set_mpls_qos_from_table;
    of_action_id_push_pbb_t *push_pbb;
    of_action_id_nicira_dec_ttl_t *nicira_dec_ttl;
    of_action_id_ofdpa_oam_lm_rx_count_t *ofdpa_oam_lm_rx_count;
    of_action_id_ofdpa_color_based_tx_count_t *ofdpa_color_based_tx_count;
    of_action_id_pop_mpls_t *pop_mpls;
    of_action_id_push_vlan_t *push_vlan;
    of_action_id_ofdpa_set_mpls_pcpdei_from_table_t *ofdpa_set_mpls_pcpdei_from_table;
    of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_t *ofdpa_set_mpls_tc_from_tunnel_table;
    of_action_id_set_nw_ttl_t *set_nw_ttl;
    of_action_id_pop_pbb_t *pop_pbb;
    of_action_id_ofdpa_pop_l2hdr_t *ofdpa_pop_l2hdr;
    of_action_id_set_queue_t *set_queue;
    of_action_id_push_mpls_t *push_mpls;
    of_action_id_ofdpa_class_based_rx_count_t *ofdpa_class_based_rx_count;
    of_action_id_dec_nw_ttl_t *dec_nw_ttl;
    of_action_id_ofdpa_class_based_tx_count_t *ofdpa_class_based_tx_count;
    of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_t *ofdpa_set_mpls_pcpdei_from_tunnel_table;
    of_action_id_output_t *output;

    /* Instantiate pointers for each subclass */
    ofdpa_set_pcpdei_from_table = &elt.ofdpa_set_pcpdei_from_table;
    copy_ttl_out = &elt.copy_ttl_out;
    bsn_mirror = &elt.bsn_mirror;
    ofdpa_push_cw = &elt.ofdpa_push_cw;
    ofdpa_copy_tc_in = &elt.ofdpa_copy_tc_in;
    ofdpa_dec_ttl_mtu = &elt.ofdpa_dec_ttl_mtu;
    dec_mpls_ttl = &elt.dec_mpls_ttl;
    bsn_set_tunnel_dst = &elt.bsn_set_tunnel_dst;
    ofdpa_oam_lm_tx_count = &elt.ofdpa_oam_lm_tx_count;
    ofdpa_copy_tc_out = &elt.ofdpa_copy_tc_out;
    ofdpa_set_dscp_from_table = &elt.ofdpa_set_dscp_from_table;
    copy_ttl_in = &elt.copy_ttl_in;
    group = &elt.group;
    ofdpa_push_l2hdr = &elt.ofdpa_push_l2hdr;
    ofdpa_color_based_rx_count = &elt.ofdpa_color_based_rx_count;
    ofdpa_set_counter_fields = &elt.ofdpa_set_counter_fields;
    ofdpa_check_drop_status = &elt.ofdpa_check_drop_status;
    set_field = &elt.set_field;
    set_mpls_ttl = &elt.set_mpls_ttl;
    pop_vlan = &elt.pop_vlan;
    ofdpa_set_mpls_tc_from_vpn_table = &elt.ofdpa_set_mpls_tc_from_vpn_table;
    ofdpa_pop_cw = &elt.ofdpa_pop_cw;
    ofdpa_set_mpls_qos_from_table = &elt.ofdpa_set_mpls_qos_from_table;
    push_pbb = &elt.push_pbb;
    nicira_dec_ttl = &elt.nicira_dec_ttl;
    ofdpa_oam_lm_rx_count = &elt.ofdpa_oam_lm_rx_count;
    ofdpa_color_based_tx_count = &elt.ofdpa_color_based_tx_count;
    pop_mpls = &elt.pop_mpls;
    push_vlan = &elt.push_vlan;
    ofdpa_set_mpls_pcpdei_from_table = &elt.ofdpa_set_mpls_pcpdei_from_table;
    ofdpa_set_mpls_tc_from_tunnel_table = &elt.ofdpa_set_mpls_tc_from_tunnel_table;
    set_nw_ttl = &elt.set_nw_ttl;
    pop_pbb = &elt.pop_pbb;
    ofdpa_pop_l2hdr = &elt.ofdpa_pop_l2hdr;
    set_queue = &elt.set_queue;
    push_mpls = &elt.push_mpls;
    ofdpa_class_based_rx_count = &elt.ofdpa_class_based_rx_count;
    dec_nw_ttl = &elt.dec_nw_ttl;
    ofdpa_class_based_tx_count = &elt.ofdpa_class_based_tx_count;
    ofdpa_set_mpls_pcpdei_from_tunnel_table = &elt.ofdpa_set_mpls_pcpdei_from_tunnel_table;
    output = &elt.output;
    TEST_OK(of_list_action_id_first(list, &elt));

    /* Check two instances of type ofdpa_set_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 10);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->length == 10);
    if (ofdpa_set_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type copy_ttl_out */
    TEST_ASSERT(copy_ttl_out->length == 4);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_ID_COPY_TTL_OUT);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(copy_ttl_out->length == 4);
    if (copy_ttl_out->wire_length_get != NULL) {
        int length;

        copy_ttl_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_out->object_id == OF_ACTION_ID_COPY_TTL_OUT);
    value = of_action_id_copy_ttl_out_OF_VERSION_1_3_check(
        copy_ttl_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type bsn_mirror */
    TEST_ASSERT(bsn_mirror->length == 12);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_ID_BSN_MIRROR);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(bsn_mirror->length == 12);
    if (bsn_mirror->wire_length_get != NULL) {
        int length;

        bsn_mirror->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_mirror->object_id == OF_ACTION_ID_BSN_MIRROR);
    value = of_action_id_bsn_mirror_OF_VERSION_1_3_check(
        bsn_mirror, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_push_cw */
    TEST_ASSERT(ofdpa_push_cw->length == 10);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_ID_OFDPA_PUSH_CW);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_push_cw->length == 10);
    if (ofdpa_push_cw->wire_length_get != NULL) {
        int length;

        ofdpa_push_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_cw->object_id == OF_ACTION_ID_OFDPA_PUSH_CW);
    value = of_action_id_ofdpa_push_cw_OF_VERSION_1_3_check(
        ofdpa_push_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_in */
    TEST_ASSERT(ofdpa_copy_tc_in->length == 10);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_ID_OFDPA_COPY_TC_IN);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_in->length == 10);
    if (ofdpa_copy_tc_in->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_in->object_id == OF_ACTION_ID_OFDPA_COPY_TC_IN);
    value = of_action_id_ofdpa_copy_tc_in_OF_VERSION_1_3_check(
        ofdpa_copy_tc_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_dec_ttl_mtu */
    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 10);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_ID_OFDPA_DEC_TTL_MTU);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_dec_ttl_mtu->length == 10);
    if (ofdpa_dec_ttl_mtu->wire_length_get != NULL) {
        int length;

        ofdpa_dec_ttl_mtu->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_dec_ttl_mtu->object_id == OF_ACTION_ID_OFDPA_DEC_TTL_MTU);
    value = of_action_id_ofdpa_dec_ttl_mtu_OF_VERSION_1_3_check(
        ofdpa_dec_ttl_mtu, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type dec_mpls_ttl */
    TEST_ASSERT(dec_mpls_ttl->length == 4);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_ID_DEC_MPLS_TTL);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(dec_mpls_ttl->length == 4);
    if (dec_mpls_ttl->wire_length_get != NULL) {
        int length;

        dec_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_mpls_ttl->object_id == OF_ACTION_ID_DEC_MPLS_TTL);
    value = of_action_id_dec_mpls_ttl_OF_VERSION_1_3_check(
        dec_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type bsn_set_tunnel_dst */
    TEST_ASSERT(bsn_set_tunnel_dst->length == 12);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_ID_BSN_SET_TUNNEL_DST);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(bsn_set_tunnel_dst->length == 12);
    if (bsn_set_tunnel_dst->wire_length_get != NULL) {
        int length;

        bsn_set_tunnel_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_set_tunnel_dst->object_id == OF_ACTION_ID_BSN_SET_TUNNEL_DST);
    value = of_action_id_bsn_set_tunnel_dst_OF_VERSION_1_3_check(
        bsn_set_tunnel_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_tx_count */
    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 10);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_tx_count->length == 10);
    if (ofdpa_oam_lm_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_tx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_TX_COUNT);
    value = of_action_id_ofdpa_oam_lm_tx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_copy_tc_out */
    TEST_ASSERT(ofdpa_copy_tc_out->length == 10);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_ID_OFDPA_COPY_TC_OUT);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_copy_tc_out->length == 10);
    if (ofdpa_copy_tc_out->wire_length_get != NULL) {
        int length;

        ofdpa_copy_tc_out->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_copy_tc_out->object_id == OF_ACTION_ID_OFDPA_COPY_TC_OUT);
    value = of_action_id_ofdpa_copy_tc_out_OF_VERSION_1_3_check(
        ofdpa_copy_tc_out, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_dscp_from_table */
    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 10);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_ID_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_dscp_from_table->length == 10);
    if (ofdpa_set_dscp_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_dscp_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_dscp_from_table->object_id == OF_ACTION_ID_OFDPA_SET_DSCP_FROM_TABLE);
    value = of_action_id_ofdpa_set_dscp_from_table_OF_VERSION_1_3_check(
        ofdpa_set_dscp_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type copy_ttl_in */
    TEST_ASSERT(copy_ttl_in->length == 4);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_ID_COPY_TTL_IN);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(copy_ttl_in->length == 4);
    if (copy_ttl_in->wire_length_get != NULL) {
        int length;

        copy_ttl_in->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(copy_ttl_in->object_id == OF_ACTION_ID_COPY_TTL_IN);
    value = of_action_id_copy_ttl_in_OF_VERSION_1_3_check(
        copy_ttl_in, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type group */
    TEST_ASSERT(group->length == 4);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_ID_GROUP);
    value = of_action_id_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(group->length == 4);
    if (group->wire_length_get != NULL) {
        int length;

        group->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(group->object_id == OF_ACTION_ID_GROUP);
    value = of_action_id_group_OF_VERSION_1_3_check(
        group, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_push_l2hdr */
    TEST_ASSERT(ofdpa_push_l2hdr->length == 10);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_ID_OFDPA_PUSH_L2HDR);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_push_l2hdr->length == 10);
    if (ofdpa_push_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_push_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_push_l2hdr->object_id == OF_ACTION_ID_OFDPA_PUSH_L2HDR);
    value = of_action_id_ofdpa_push_l2hdr_OF_VERSION_1_3_check(
        ofdpa_push_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_rx_count */
    TEST_ASSERT(ofdpa_color_based_rx_count->length == 10);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_rx_count->length == 10);
    if (ofdpa_color_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_rx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_RX_COUNT);
    value = of_action_id_ofdpa_color_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_counter_fields */
    TEST_ASSERT(ofdpa_set_counter_fields->length == 10);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_ID_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_counter_fields->length == 10);
    if (ofdpa_set_counter_fields->wire_length_get != NULL) {
        int length;

        ofdpa_set_counter_fields->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_counter_fields->object_id == OF_ACTION_ID_OFDPA_SET_COUNTER_FIELDS);
    value = of_action_id_ofdpa_set_counter_fields_OF_VERSION_1_3_check(
        ofdpa_set_counter_fields, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_check_drop_status */
    TEST_ASSERT(ofdpa_check_drop_status->length == 10);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_ID_OFDPA_CHECK_DROP_STATUS);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_check_drop_status->length == 10);
    if (ofdpa_check_drop_status->wire_length_get != NULL) {
        int length;

        ofdpa_check_drop_status->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_check_drop_status->object_id == OF_ACTION_ID_OFDPA_CHECK_DROP_STATUS);
    value = of_action_id_ofdpa_check_drop_status_OF_VERSION_1_3_check(
        ofdpa_check_drop_status, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_field */
    TEST_ASSERT(set_field->length == 4);
    if (set_field->wire_length_get != NULL) {
        int length;

        set_field->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_field->object_id == OF_ACTION_ID_SET_FIELD);
    value = of_action_id_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_field->length == 4);
    if (set_field->wire_length_get != NULL) {
        int length;

        set_field->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_field->object_id == OF_ACTION_ID_SET_FIELD);
    value = of_action_id_set_field_OF_VERSION_1_3_check(
        set_field, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_mpls_ttl */
    TEST_ASSERT(set_mpls_ttl->length == 4);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_ID_SET_MPLS_TTL);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_mpls_ttl->length == 4);
    if (set_mpls_ttl->wire_length_get != NULL) {
        int length;

        set_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_mpls_ttl->object_id == OF_ACTION_ID_SET_MPLS_TTL);
    value = of_action_id_set_mpls_ttl_OF_VERSION_1_3_check(
        set_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type pop_vlan */
    TEST_ASSERT(pop_vlan->length == 4);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_ID_POP_VLAN);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(pop_vlan->length == 4);
    if (pop_vlan->wire_length_get != NULL) {
        int length;

        pop_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_vlan->object_id == OF_ACTION_ID_POP_VLAN);
    value = of_action_id_pop_vlan_OF_VERSION_1_3_check(
        pop_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_vpn_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 10);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->length == 10);
    if (ofdpa_set_mpls_tc_from_vpn_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_vpn_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_vpn_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_VPN_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_vpn_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_vpn_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_pop_cw */
    TEST_ASSERT(ofdpa_pop_cw->length == 10);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_ID_OFDPA_POP_CW);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_cw->length == 10);
    if (ofdpa_pop_cw->wire_length_get != NULL) {
        int length;

        ofdpa_pop_cw->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_cw->object_id == OF_ACTION_ID_OFDPA_POP_CW);
    value = of_action_id_ofdpa_pop_cw_OF_VERSION_1_3_check(
        ofdpa_pop_cw, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_qos_from_table */
    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 10);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->length == 10);
    if (ofdpa_set_mpls_qos_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_qos_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_qos_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_QOS_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_qos_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_qos_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type push_pbb */
    TEST_ASSERT(push_pbb->length == 4);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_ID_PUSH_PBB);
    value = of_action_id_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(push_pbb->length == 4);
    if (push_pbb->wire_length_get != NULL) {
        int length;

        push_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_pbb->object_id == OF_ACTION_ID_PUSH_PBB);
    value = of_action_id_push_pbb_OF_VERSION_1_3_check(
        push_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type nicira_dec_ttl */
    TEST_ASSERT(nicira_dec_ttl->length == 10);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_ID_NICIRA_DEC_TTL);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(nicira_dec_ttl->length == 10);
    if (nicira_dec_ttl->wire_length_get != NULL) {
        int length;

        nicira_dec_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(nicira_dec_ttl->object_id == OF_ACTION_ID_NICIRA_DEC_TTL);
    value = of_action_id_nicira_dec_ttl_OF_VERSION_1_3_check(
        nicira_dec_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_oam_lm_rx_count */
    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 10);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_lm_rx_count->length == 10);
    if (ofdpa_oam_lm_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_oam_lm_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_oam_lm_rx_count->object_id == OF_ACTION_ID_OFDPA_OAM_LM_RX_COUNT);
    value = of_action_id_ofdpa_oam_lm_rx_count_OF_VERSION_1_3_check(
        ofdpa_oam_lm_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_color_based_tx_count */
    TEST_ASSERT(ofdpa_color_based_tx_count->length == 10);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_color_based_tx_count->length == 10);
    if (ofdpa_color_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_color_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_color_based_tx_count->object_id == OF_ACTION_ID_OFDPA_COLOR_BASED_TX_COUNT);
    value = of_action_id_ofdpa_color_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_color_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type pop_mpls */
    TEST_ASSERT(pop_mpls->length == 4);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_ID_POP_MPLS);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(pop_mpls->length == 4);
    if (pop_mpls->wire_length_get != NULL) {
        int length;

        pop_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_mpls->object_id == OF_ACTION_ID_POP_MPLS);
    value = of_action_id_pop_mpls_OF_VERSION_1_3_check(
        pop_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type push_vlan */
    TEST_ASSERT(push_vlan->length == 4);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_ID_PUSH_VLAN);
    value = of_action_id_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(push_vlan->length == 4);
    if (push_vlan->wire_length_get != NULL) {
        int length;

        push_vlan->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_vlan->object_id == OF_ACTION_ID_PUSH_VLAN);
    value = of_action_id_push_vlan_OF_VERSION_1_3_check(
        push_vlan, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_tc_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_tc_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_tc_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_TC_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_tc_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_nw_ttl */
    TEST_ASSERT(set_nw_ttl->length == 4);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_ID_SET_NW_TTL);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_nw_ttl->length == 4);
    if (set_nw_ttl->wire_length_get != NULL) {
        int length;

        set_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_nw_ttl->object_id == OF_ACTION_ID_SET_NW_TTL);
    value = of_action_id_set_nw_ttl_OF_VERSION_1_3_check(
        set_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type pop_pbb */
    TEST_ASSERT(pop_pbb->length == 4);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_ID_POP_PBB);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(pop_pbb->length == 4);
    if (pop_pbb->wire_length_get != NULL) {
        int length;

        pop_pbb->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(pop_pbb->object_id == OF_ACTION_ID_POP_PBB);
    value = of_action_id_pop_pbb_OF_VERSION_1_3_check(
        pop_pbb, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_pop_l2hdr */
    TEST_ASSERT(ofdpa_pop_l2hdr->length == 10);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_ID_OFDPA_POP_L2HDR);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_pop_l2hdr->length == 10);
    if (ofdpa_pop_l2hdr->wire_length_get != NULL) {
        int length;

        ofdpa_pop_l2hdr->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_pop_l2hdr->object_id == OF_ACTION_ID_OFDPA_POP_L2HDR);
    value = of_action_id_ofdpa_pop_l2hdr_OF_VERSION_1_3_check(
        ofdpa_pop_l2hdr, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type set_queue */
    TEST_ASSERT(set_queue->length == 4);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_ID_SET_QUEUE);
    value = of_action_id_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(set_queue->length == 4);
    if (set_queue->wire_length_get != NULL) {
        int length;

        set_queue->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(set_queue->object_id == OF_ACTION_ID_SET_QUEUE);
    value = of_action_id_set_queue_OF_VERSION_1_3_check(
        set_queue, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type push_mpls */
    TEST_ASSERT(push_mpls->length == 4);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_ID_PUSH_MPLS);
    value = of_action_id_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(push_mpls->length == 4);
    if (push_mpls->wire_length_get != NULL) {
        int length;

        push_mpls->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(push_mpls->object_id == OF_ACTION_ID_PUSH_MPLS);
    value = of_action_id_push_mpls_OF_VERSION_1_3_check(
        push_mpls, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_rx_count */
    TEST_ASSERT(ofdpa_class_based_rx_count->length == 10);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_rx_count->length == 10);
    if (ofdpa_class_based_rx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_rx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_rx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_RX_COUNT);
    value = of_action_id_ofdpa_class_based_rx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_rx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type dec_nw_ttl */
    TEST_ASSERT(dec_nw_ttl->length == 4);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_ID_DEC_NW_TTL);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(dec_nw_ttl->length == 4);
    if (dec_nw_ttl->wire_length_get != NULL) {
        int length;

        dec_nw_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(dec_nw_ttl->object_id == OF_ACTION_ID_DEC_NW_TTL);
    value = of_action_id_dec_nw_ttl_OF_VERSION_1_3_check(
        dec_nw_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_class_based_tx_count */
    TEST_ASSERT(ofdpa_class_based_tx_count->length == 10);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_class_based_tx_count->length == 10);
    if (ofdpa_class_based_tx_count->wire_length_get != NULL) {
        int length;

        ofdpa_class_based_tx_count->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_class_based_tx_count->object_id == OF_ACTION_ID_OFDPA_CLASS_BASED_TX_COUNT);
    value = of_action_id_ofdpa_class_based_tx_count_OF_VERSION_1_3_check(
        ofdpa_class_based_tx_count, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type ofdpa_set_mpls_pcpdei_from_tunnel_table */
    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->length == 10);
    if (ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get != NULL) {
        int length;

        ofdpa_set_mpls_pcpdei_from_tunnel_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ofdpa_set_mpls_pcpdei_from_tunnel_table->object_id == OF_ACTION_ID_OFDPA_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE);
    value = of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_OF_VERSION_1_3_check(
        ofdpa_set_mpls_pcpdei_from_tunnel_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    /* Check two instances of type output */
    TEST_ASSERT(output->length == 4);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_ID_OUTPUT);
    value = of_action_id_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_action_id_next(list, &elt));

    TEST_ASSERT(output->length == 4);
    if (output->wire_length_get != NULL) {
        int length;

        output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(output->object_id == OF_ACTION_ID_OUTPUT);
    value = of_action_id_output_OF_VERSION_1_3_check(
        output, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_action_id_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_ACTION_ID_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 82);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_action_id_t *dup;

        TEST_ASSERT((dup = of_list_action_id_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_action_id_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_controller_connection with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_controller_connection_OF_VERSION_1_3_populate(
    of_list_bsn_controller_connection_t *list, int value)
{

    of_bsn_controller_connection_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_controller_connection */
    of_bsn_controller_connection_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_controller_connection */

    of_bsn_controller_connection_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_controller_connection_append_bind(list,
            (of_bsn_controller_connection_t *)elt_p);
    value = of_bsn_controller_connection_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_controller_connection_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_controller_connection_append_bind(list,
            (of_bsn_controller_connection_t *)elt_p);
    value = of_bsn_controller_connection_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_controller_connection generated by
 * of_list_bsn_controller_connection_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_controller_connection_OF_VERSION_1_3_check(
    of_list_bsn_controller_connection_t *list, int value)
{

    of_bsn_controller_connection_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_controller_connection */
    of_bsn_controller_connection_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_controller_connection_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_CONTROLLER_CONNECTION);
    value = of_bsn_controller_connection_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_controller_connection_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_CONTROLLER_CONNECTION);
    value = of_bsn_controller_connection_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_controller_connection_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_CONTROLLER_CONNECTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_controller_connection_t *dup;

        TEST_ASSERT((dup = of_list_bsn_controller_connection_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_controller_connection_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_debug_counter_desc_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_debug_counter_desc_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_debug_counter_desc_stats_entry */
    of_bsn_debug_counter_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_debug_counter_desc_stats_entry */

    of_bsn_debug_counter_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_desc_stats_entry_append_bind(list,
            (of_bsn_debug_counter_desc_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_debug_counter_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_desc_stats_entry_append_bind(list,
            (of_bsn_debug_counter_desc_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_debug_counter_desc_stats_entry generated by
 * of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_debug_counter_desc_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_desc_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_debug_counter_desc_stats_entry */
    of_bsn_debug_counter_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_debug_counter_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_DESC_STATS_ENTRY);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_debug_counter_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_DESC_STATS_ENTRY);
    value = of_bsn_debug_counter_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_debug_counter_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_DEBUG_COUNTER_DESC_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_debug_counter_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_debug_counter_desc_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_debug_counter_desc_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_debug_counter_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_debug_counter_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_debug_counter_stats_entry */
    of_bsn_debug_counter_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_debug_counter_stats_entry */

    of_bsn_debug_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_stats_entry_append_bind(list,
            (of_bsn_debug_counter_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_debug_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_debug_counter_stats_entry_append_bind(list,
            (of_bsn_debug_counter_stats_entry_t *)elt_p);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_debug_counter_stats_entry generated by
 * of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_debug_counter_stats_entry_t *list, int value)
{

    of_bsn_debug_counter_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_debug_counter_stats_entry */
    of_bsn_debug_counter_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_debug_counter_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_STATS_ENTRY);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_debug_counter_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_DEBUG_COUNTER_STATS_ENTRY);
    value = of_bsn_debug_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_debug_counter_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_DEBUG_COUNTER_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_debug_counter_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_debug_counter_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_debug_counter_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_flow_checksum_bucket_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *list, int value)
{

    of_bsn_flow_checksum_bucket_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_flow_checksum_bucket_stats_entry */
    of_bsn_flow_checksum_bucket_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_flow_checksum_bucket_stats_entry */

    of_bsn_flow_checksum_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_flow_checksum_bucket_stats_entry_append_bind(list,
            (of_bsn_flow_checksum_bucket_stats_entry_t *)elt_p);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_flow_checksum_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_flow_checksum_bucket_stats_entry_append_bind(list,
            (of_bsn_flow_checksum_bucket_stats_entry_t *)elt_p);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_flow_checksum_bucket_stats_entry generated by
 * of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_flow_checksum_bucket_stats_entry_t *list, int value)
{

    of_bsn_flow_checksum_bucket_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_flow_checksum_bucket_stats_entry */
    of_bsn_flow_checksum_bucket_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_flow_checksum_bucket_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_FLOW_CHECKSUM_BUCKET_STATS_ENTRY);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_flow_checksum_bucket_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_FLOW_CHECKSUM_BUCKET_STATS_ENTRY);
    value = of_bsn_flow_checksum_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_flow_checksum_bucket_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_FLOW_CHECKSUM_BUCKET_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_flow_checksum_bucket_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_flow_checksum_bucket_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_flow_checksum_bucket_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_gentable_bucket_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_gentable_bucket_stats_entry_t *list, int value)
{

    of_bsn_gentable_bucket_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_bucket_stats_entry */
    of_bsn_gentable_bucket_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_bucket_stats_entry */

    of_bsn_gentable_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_bucket_stats_entry_append_bind(list,
            (of_bsn_gentable_bucket_stats_entry_t *)elt_p);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_bucket_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_bucket_stats_entry_append_bind(list,
            (of_bsn_gentable_bucket_stats_entry_t *)elt_p);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_bucket_stats_entry generated by
 * of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_gentable_bucket_stats_entry_t *list, int value)
{

    of_bsn_gentable_bucket_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_gentable_bucket_stats_entry */
    of_bsn_gentable_bucket_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_bucket_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_BUCKET_STATS_ENTRY);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_bucket_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_BUCKET_STATS_ENTRY);
    value = of_bsn_gentable_bucket_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_bucket_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_GENTABLE_BUCKET_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_gentable_bucket_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_gentable_bucket_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_gentable_bucket_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_gentable_desc_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_gentable_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_desc_stats_entry */
    of_bsn_gentable_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_desc_stats_entry */

    of_bsn_gentable_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_desc_stats_entry generated by
 * of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_gentable_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_desc_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_gentable_desc_stats_entry */
    of_bsn_gentable_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_DESC_STATS_ENTRY);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_DESC_STATS_ENTRY);
    value = of_bsn_gentable_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_GENTABLE_DESC_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_gentable_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_gentable_desc_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_gentable_desc_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_gentable_entry_desc_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_gentable_entry_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_entry_desc_stats_entry */
    of_bsn_gentable_entry_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_entry_desc_stats_entry */

    of_bsn_gentable_entry_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_entry_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_desc_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_desc_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_entry_desc_stats_entry generated by
 * of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_gentable_entry_desc_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_desc_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_gentable_entry_desc_stats_entry */
    of_bsn_gentable_entry_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_entry_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_DESC_STATS_ENTRY);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_entry_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_DESC_STATS_ENTRY);
    value = of_bsn_gentable_entry_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_entry_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_GENTABLE_ENTRY_DESC_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_gentable_entry_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_gentable_entry_desc_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_gentable_entry_desc_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_gentable_entry_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_gentable_entry_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_entry_stats_entry */
    of_bsn_gentable_entry_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_entry_stats_entry */

    of_bsn_gentable_entry_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_entry_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_entry_stats_entry_append_bind(list,
            (of_bsn_gentable_entry_stats_entry_t *)elt_p);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_entry_stats_entry generated by
 * of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_gentable_entry_stats_entry_t *list, int value)
{

    of_bsn_gentable_entry_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_gentable_entry_stats_entry */
    of_bsn_gentable_entry_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_entry_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_STATS_ENTRY);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_entry_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_ENTRY_STATS_ENTRY);
    value = of_bsn_gentable_entry_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_entry_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_GENTABLE_ENTRY_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_gentable_entry_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_gentable_entry_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_gentable_entry_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_gentable_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_gentable_stats_entry_t *list, int value)
{

    of_bsn_gentable_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_gentable_stats_entry */
    of_bsn_gentable_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_gentable_stats_entry */

    of_bsn_gentable_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_stats_entry_append_bind(list,
            (of_bsn_gentable_stats_entry_t *)elt_p);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_gentable_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_gentable_stats_entry_append_bind(list,
            (of_bsn_gentable_stats_entry_t *)elt_p);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_gentable_stats_entry generated by
 * of_list_bsn_gentable_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_gentable_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_gentable_stats_entry_t *list, int value)
{

    of_bsn_gentable_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_gentable_stats_entry */
    of_bsn_gentable_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_gentable_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_STATS_ENTRY);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_gentable_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_GENTABLE_STATS_ENTRY);
    value = of_bsn_gentable_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_gentable_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_GENTABLE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_gentable_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_gentable_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_gentable_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_interface with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_3_populate(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_interface */

    of_bsn_interface_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_interface_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_interface_append_bind(list,
            (of_bsn_interface_t *)elt_p);
    value = of_bsn_interface_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_interface generated by
 * of_list_bsn_interface_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_interface_OF_VERSION_1_3_check(
    of_list_bsn_interface_t *list, int value)
{

    of_bsn_interface_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_interface */
    of_bsn_interface_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_interface_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_interface_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_INTERFACE);
    value = of_bsn_interface_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_interface_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_INTERFACE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_interface_t *dup;

        TEST_ASSERT((dup = of_list_bsn_interface_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_interface_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_lacp_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_lacp_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_lacp_stats_entry_t *list, int value)
{

    of_bsn_lacp_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_lacp_stats_entry */
    of_bsn_lacp_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_lacp_stats_entry */

    of_bsn_lacp_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_lacp_stats_entry_append_bind(list,
            (of_bsn_lacp_stats_entry_t *)elt_p);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_lacp_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_lacp_stats_entry_append_bind(list,
            (of_bsn_lacp_stats_entry_t *)elt_p);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_lacp_stats_entry generated by
 * of_list_bsn_lacp_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_lacp_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_lacp_stats_entry_t *list, int value)
{

    of_bsn_lacp_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_lacp_stats_entry */
    of_bsn_lacp_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_lacp_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_LACP_STATS_ENTRY);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_lacp_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_LACP_STATS_ENTRY);
    value = of_bsn_lacp_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_lacp_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_LACP_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_lacp_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_lacp_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_lacp_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_port_counter_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_port_counter_stats_entry_t *list, int value)
{

    of_bsn_port_counter_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_port_counter_stats_entry */
    of_bsn_port_counter_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_port_counter_stats_entry */

    of_bsn_port_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_port_counter_stats_entry_append_bind(list,
            (of_bsn_port_counter_stats_entry_t *)elt_p);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_port_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_port_counter_stats_entry_append_bind(list,
            (of_bsn_port_counter_stats_entry_t *)elt_p);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_port_counter_stats_entry generated by
 * of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_port_counter_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_port_counter_stats_entry_t *list, int value)
{

    of_bsn_port_counter_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_port_counter_stats_entry */
    of_bsn_port_counter_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_port_counter_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_PORT_COUNTER_STATS_ENTRY);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_port_counter_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_PORT_COUNTER_STATS_ENTRY);
    value = of_bsn_port_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_port_counter_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_PORT_COUNTER_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_port_counter_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_port_counter_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_port_counter_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_switch_pipeline_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_switch_pipeline_stats_entry_t *list, int value)
{

    of_bsn_switch_pipeline_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_switch_pipeline_stats_entry */
    of_bsn_switch_pipeline_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_switch_pipeline_stats_entry */

    of_bsn_switch_pipeline_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_switch_pipeline_stats_entry_append_bind(list,
            (of_bsn_switch_pipeline_stats_entry_t *)elt_p);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_switch_pipeline_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_switch_pipeline_stats_entry_append_bind(list,
            (of_bsn_switch_pipeline_stats_entry_t *)elt_p);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_switch_pipeline_stats_entry generated by
 * of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_switch_pipeline_stats_entry_t *list, int value)
{

    of_bsn_switch_pipeline_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_switch_pipeline_stats_entry */
    of_bsn_switch_pipeline_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_switch_pipeline_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_SWITCH_PIPELINE_STATS_ENTRY);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_switch_pipeline_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_SWITCH_PIPELINE_STATS_ENTRY);
    value = of_bsn_switch_pipeline_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_switch_pipeline_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_SWITCH_PIPELINE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_switch_pipeline_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_switch_pipeline_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_switch_pipeline_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_table_checksum_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_table_checksum_stats_entry_t *list, int value)
{

    of_bsn_table_checksum_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_table_checksum_stats_entry */
    of_bsn_table_checksum_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_table_checksum_stats_entry */

    of_bsn_table_checksum_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_table_checksum_stats_entry_append_bind(list,
            (of_bsn_table_checksum_stats_entry_t *)elt_p);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_table_checksum_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_table_checksum_stats_entry_append_bind(list,
            (of_bsn_table_checksum_stats_entry_t *)elt_p);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_table_checksum_stats_entry generated by
 * of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_table_checksum_stats_entry_t *list, int value)
{

    of_bsn_table_checksum_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_table_checksum_stats_entry */
    of_bsn_table_checksum_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_table_checksum_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_TABLE_CHECKSUM_STATS_ENTRY);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_table_checksum_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_TABLE_CHECKSUM_STATS_ENTRY);
    value = of_bsn_table_checksum_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_table_checksum_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_TABLE_CHECKSUM_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_table_checksum_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_table_checksum_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_table_checksum_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_tlv with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_tlv_OF_VERSION_1_3_populate(
    of_list_bsn_tlv_t *list, int value)
{

    of_bsn_tlv_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_bsn_tlv_rx_packets_t *rx_packets;
    of_bsn_tlv_reply_packets_t *reply_packets;
    of_bsn_tlv_udf_length_t *udf_length;
    of_bsn_tlv_udf_id_t *udf_id;
    of_bsn_tlv_circuit_id_t *circuit_id;
    of_bsn_tlv_request_packets_t *request_packets;
    of_bsn_tlv_unicast_query_timeout_t *unicast_query_timeout;
    of_bsn_tlv_idle_time_t *idle_time;
    of_bsn_tlv_idle_timeout_t *idle_timeout;
    of_bsn_tlv_miss_packets_t *miss_packets;
    of_bsn_tlv_mac_t *mac;
    of_bsn_tlv_ipv4_t *ipv4;
    of_bsn_tlv_vlan_vid_t *vlan_vid;
    of_bsn_tlv_idle_notification_t *idle_notification;
    of_bsn_tlv_broadcast_query_timeout_t *broadcast_query_timeout;
    of_bsn_tlv_tx_packets_t *tx_packets;
    of_bsn_tlv_port_t *port;
    of_bsn_tlv_udf_offset_t *udf_offset;
    of_bsn_tlv_udf_anchor_t *udf_anchor;

    /* Instantiate pointers for each subclass */
    rx_packets = &elt.rx_packets;
    reply_packets = &elt.reply_packets;
    udf_length = &elt.udf_length;
    udf_id = &elt.udf_id;
    circuit_id = &elt.circuit_id;
    request_packets = &elt.request_packets;
    unicast_query_timeout = &elt.unicast_query_timeout;
    idle_time = &elt.idle_time;
    idle_timeout = &elt.idle_timeout;
    miss_packets = &elt.miss_packets;
    mac = &elt.mac;
    ipv4 = &elt.ipv4;
    vlan_vid = &elt.vlan_vid;
    idle_notification = &elt.idle_notification;
    broadcast_query_timeout = &elt.broadcast_query_timeout;
    tx_packets = &elt.tx_packets;
    port = &elt.port;
    udf_offset = &elt.udf_offset;
    udf_anchor = &elt.udf_anchor;

    /* Append two instances of type of_bsn_tlv_rx_packets */

    of_bsn_tlv_rx_packets_init(rx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)rx_packets);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_populate(
        rx_packets, value);
    cur_len += rx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_rx_packets_init(rx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)rx_packets);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_populate(
        rx_packets, value);
    cur_len += rx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_reply_packets */

    of_bsn_tlv_reply_packets_init(reply_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)reply_packets);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_populate(
        reply_packets, value);
    cur_len += reply_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_reply_packets_init(reply_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)reply_packets);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_populate(
        reply_packets, value);
    cur_len += reply_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_length */

    of_bsn_tlv_udf_length_init(udf_length, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_length);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_populate(
        udf_length, value);
    cur_len += udf_length->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_length_init(udf_length, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_length);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_populate(
        udf_length, value);
    cur_len += udf_length->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_id */

    of_bsn_tlv_udf_id_init(udf_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_id);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_populate(
        udf_id, value);
    cur_len += udf_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_id_init(udf_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_id);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_populate(
        udf_id, value);
    cur_len += udf_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_circuit_id */

    of_bsn_tlv_circuit_id_init(circuit_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)circuit_id);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_populate(
        circuit_id, value);
    cur_len += circuit_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_circuit_id_init(circuit_id, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)circuit_id);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_populate(
        circuit_id, value);
    cur_len += circuit_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_request_packets */

    of_bsn_tlv_request_packets_init(request_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)request_packets);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_populate(
        request_packets, value);
    cur_len += request_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_request_packets_init(request_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)request_packets);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_populate(
        request_packets, value);
    cur_len += request_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_unicast_query_timeout */

    of_bsn_tlv_unicast_query_timeout_init(unicast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)unicast_query_timeout);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_populate(
        unicast_query_timeout, value);
    cur_len += unicast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_unicast_query_timeout_init(unicast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)unicast_query_timeout);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_populate(
        unicast_query_timeout, value);
    cur_len += unicast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_idle_time */

    of_bsn_tlv_idle_time_init(idle_time, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_time);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_populate(
        idle_time, value);
    cur_len += idle_time->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_idle_time_init(idle_time, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_time);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_populate(
        idle_time, value);
    cur_len += idle_time->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_idle_timeout */

    of_bsn_tlv_idle_timeout_init(idle_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_timeout);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_populate(
        idle_timeout, value);
    cur_len += idle_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_idle_timeout_init(idle_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_timeout);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_populate(
        idle_timeout, value);
    cur_len += idle_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_miss_packets */

    of_bsn_tlv_miss_packets_init(miss_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)miss_packets);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_populate(
        miss_packets, value);
    cur_len += miss_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_miss_packets_init(miss_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)miss_packets);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_populate(
        miss_packets, value);
    cur_len += miss_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_mac */

    of_bsn_tlv_mac_init(mac, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)mac);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_populate(
        mac, value);
    cur_len += mac->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_mac_init(mac, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)mac);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_populate(
        mac, value);
    cur_len += mac->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_ipv4 */

    of_bsn_tlv_ipv4_init(ipv4, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)ipv4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_populate(
        ipv4, value);
    cur_len += ipv4->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_ipv4_init(ipv4, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)ipv4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_populate(
        ipv4, value);
    cur_len += ipv4->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_vlan_vid */

    of_bsn_tlv_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)vlan_vid);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)vlan_vid);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_idle_notification */

    of_bsn_tlv_idle_notification_init(idle_notification, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_notification);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_populate(
        idle_notification, value);
    cur_len += idle_notification->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_idle_notification_init(idle_notification, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)idle_notification);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_populate(
        idle_notification, value);
    cur_len += idle_notification->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_broadcast_query_timeout */

    of_bsn_tlv_broadcast_query_timeout_init(broadcast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)broadcast_query_timeout);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_populate(
        broadcast_query_timeout, value);
    cur_len += broadcast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_broadcast_query_timeout_init(broadcast_query_timeout, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)broadcast_query_timeout);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_populate(
        broadcast_query_timeout, value);
    cur_len += broadcast_query_timeout->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_tx_packets */

    of_bsn_tlv_tx_packets_init(tx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)tx_packets);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_populate(
        tx_packets, value);
    cur_len += tx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_tx_packets_init(tx_packets, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)tx_packets);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_populate(
        tx_packets, value);
    cur_len += tx_packets->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_port */

    of_bsn_tlv_port_init(port, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)port);
    value = of_bsn_tlv_port_OF_VERSION_1_3_populate(
        port, value);
    cur_len += port->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_port_init(port, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)port);
    value = of_bsn_tlv_port_OF_VERSION_1_3_populate(
        port, value);
    cur_len += port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_offset */

    of_bsn_tlv_udf_offset_init(udf_offset, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_offset);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_populate(
        udf_offset, value);
    cur_len += udf_offset->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_offset_init(udf_offset, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_offset);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_populate(
        udf_offset, value);
    cur_len += udf_offset->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_bsn_tlv_udf_anchor */

    of_bsn_tlv_udf_anchor_init(udf_anchor, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_anchor);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_populate(
        udf_anchor, value);
    cur_len += udf_anchor->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_tlv_udf_anchor_init(udf_anchor, OF_VERSION_1_3, -1, 1);
    of_list_bsn_tlv_append_bind(list,
            (of_bsn_tlv_t *)udf_anchor);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_populate(
        udf_anchor, value);
    cur_len += udf_anchor->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_tlv generated by
 * of_list_bsn_tlv_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_tlv_OF_VERSION_1_3_check(
    of_list_bsn_tlv_t *list, int value)
{

    of_bsn_tlv_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_bsn_tlv_rx_packets_t *rx_packets;
    of_bsn_tlv_reply_packets_t *reply_packets;
    of_bsn_tlv_udf_length_t *udf_length;
    of_bsn_tlv_udf_id_t *udf_id;
    of_bsn_tlv_circuit_id_t *circuit_id;
    of_bsn_tlv_request_packets_t *request_packets;
    of_bsn_tlv_unicast_query_timeout_t *unicast_query_timeout;
    of_bsn_tlv_idle_time_t *idle_time;
    of_bsn_tlv_idle_timeout_t *idle_timeout;
    of_bsn_tlv_miss_packets_t *miss_packets;
    of_bsn_tlv_mac_t *mac;
    of_bsn_tlv_ipv4_t *ipv4;
    of_bsn_tlv_vlan_vid_t *vlan_vid;
    of_bsn_tlv_idle_notification_t *idle_notification;
    of_bsn_tlv_broadcast_query_timeout_t *broadcast_query_timeout;
    of_bsn_tlv_tx_packets_t *tx_packets;
    of_bsn_tlv_port_t *port;
    of_bsn_tlv_udf_offset_t *udf_offset;
    of_bsn_tlv_udf_anchor_t *udf_anchor;

    /* Instantiate pointers for each subclass */
    rx_packets = &elt.rx_packets;
    reply_packets = &elt.reply_packets;
    udf_length = &elt.udf_length;
    udf_id = &elt.udf_id;
    circuit_id = &elt.circuit_id;
    request_packets = &elt.request_packets;
    unicast_query_timeout = &elt.unicast_query_timeout;
    idle_time = &elt.idle_time;
    idle_timeout = &elt.idle_timeout;
    miss_packets = &elt.miss_packets;
    mac = &elt.mac;
    ipv4 = &elt.ipv4;
    vlan_vid = &elt.vlan_vid;
    idle_notification = &elt.idle_notification;
    broadcast_query_timeout = &elt.broadcast_query_timeout;
    tx_packets = &elt.tx_packets;
    port = &elt.port;
    udf_offset = &elt.udf_offset;
    udf_anchor = &elt.udf_anchor;
    TEST_OK(of_list_bsn_tlv_first(list, &elt));

    /* Check two instances of type rx_packets */
    TEST_ASSERT(rx_packets->length == 12);
    if (rx_packets->wire_length_get != NULL) {
        int length;

        rx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(rx_packets->object_id == OF_BSN_TLV_RX_PACKETS);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_check(
        rx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(rx_packets->length == 12);
    if (rx_packets->wire_length_get != NULL) {
        int length;

        rx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(rx_packets->object_id == OF_BSN_TLV_RX_PACKETS);
    value = of_bsn_tlv_rx_packets_OF_VERSION_1_3_check(
        rx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type reply_packets */
    TEST_ASSERT(reply_packets->length == 12);
    if (reply_packets->wire_length_get != NULL) {
        int length;

        reply_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(reply_packets->object_id == OF_BSN_TLV_REPLY_PACKETS);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_check(
        reply_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(reply_packets->length == 12);
    if (reply_packets->wire_length_get != NULL) {
        int length;

        reply_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(reply_packets->object_id == OF_BSN_TLV_REPLY_PACKETS);
    value = of_bsn_tlv_reply_packets_OF_VERSION_1_3_check(
        reply_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_length */
    TEST_ASSERT(udf_length->length == 6);
    if (udf_length->wire_length_get != NULL) {
        int length;

        udf_length->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_length->object_id == OF_BSN_TLV_UDF_LENGTH);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_check(
        udf_length, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_length->length == 6);
    if (udf_length->wire_length_get != NULL) {
        int length;

        udf_length->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_length->object_id == OF_BSN_TLV_UDF_LENGTH);
    value = of_bsn_tlv_udf_length_OF_VERSION_1_3_check(
        udf_length, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_id */
    TEST_ASSERT(udf_id->length == 6);
    if (udf_id->wire_length_get != NULL) {
        int length;

        udf_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_id->object_id == OF_BSN_TLV_UDF_ID);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_check(
        udf_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_id->length == 6);
    if (udf_id->wire_length_get != NULL) {
        int length;

        udf_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_id->object_id == OF_BSN_TLV_UDF_ID);
    value = of_bsn_tlv_udf_id_OF_VERSION_1_3_check(
        udf_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type circuit_id */
    TEST_ASSERT(circuit_id->object_id == OF_BSN_TLV_CIRCUIT_ID);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_check(
        circuit_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(circuit_id->object_id == OF_BSN_TLV_CIRCUIT_ID);
    value = of_bsn_tlv_circuit_id_OF_VERSION_1_3_check(
        circuit_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type request_packets */
    TEST_ASSERT(request_packets->length == 12);
    if (request_packets->wire_length_get != NULL) {
        int length;

        request_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(request_packets->object_id == OF_BSN_TLV_REQUEST_PACKETS);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_check(
        request_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(request_packets->length == 12);
    if (request_packets->wire_length_get != NULL) {
        int length;

        request_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(request_packets->object_id == OF_BSN_TLV_REQUEST_PACKETS);
    value = of_bsn_tlv_request_packets_OF_VERSION_1_3_check(
        request_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type unicast_query_timeout */
    TEST_ASSERT(unicast_query_timeout->length == 8);
    if (unicast_query_timeout->wire_length_get != NULL) {
        int length;

        unicast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(unicast_query_timeout->object_id == OF_BSN_TLV_UNICAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_check(
        unicast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(unicast_query_timeout->length == 8);
    if (unicast_query_timeout->wire_length_get != NULL) {
        int length;

        unicast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(unicast_query_timeout->object_id == OF_BSN_TLV_UNICAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_unicast_query_timeout_OF_VERSION_1_3_check(
        unicast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type idle_time */
    TEST_ASSERT(idle_time->length == 12);
    if (idle_time->wire_length_get != NULL) {
        int length;

        idle_time->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(idle_time->object_id == OF_BSN_TLV_IDLE_TIME);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_check(
        idle_time, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(idle_time->length == 12);
    if (idle_time->wire_length_get != NULL) {
        int length;

        idle_time->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(idle_time->object_id == OF_BSN_TLV_IDLE_TIME);
    value = of_bsn_tlv_idle_time_OF_VERSION_1_3_check(
        idle_time, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type idle_timeout */
    TEST_ASSERT(idle_timeout->length == 8);
    if (idle_timeout->wire_length_get != NULL) {
        int length;

        idle_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(idle_timeout->object_id == OF_BSN_TLV_IDLE_TIMEOUT);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_check(
        idle_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(idle_timeout->length == 8);
    if (idle_timeout->wire_length_get != NULL) {
        int length;

        idle_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(idle_timeout->object_id == OF_BSN_TLV_IDLE_TIMEOUT);
    value = of_bsn_tlv_idle_timeout_OF_VERSION_1_3_check(
        idle_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type miss_packets */
    TEST_ASSERT(miss_packets->length == 12);
    if (miss_packets->wire_length_get != NULL) {
        int length;

        miss_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(miss_packets->object_id == OF_BSN_TLV_MISS_PACKETS);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_check(
        miss_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(miss_packets->length == 12);
    if (miss_packets->wire_length_get != NULL) {
        int length;

        miss_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(miss_packets->object_id == OF_BSN_TLV_MISS_PACKETS);
    value = of_bsn_tlv_miss_packets_OF_VERSION_1_3_check(
        miss_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type mac */
    TEST_ASSERT(mac->length == 10);
    if (mac->wire_length_get != NULL) {
        int length;

        mac->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(mac->object_id == OF_BSN_TLV_MAC);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_check(
        mac, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(mac->length == 10);
    if (mac->wire_length_get != NULL) {
        int length;

        mac->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(mac->object_id == OF_BSN_TLV_MAC);
    value = of_bsn_tlv_mac_OF_VERSION_1_3_check(
        mac, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type ipv4 */
    TEST_ASSERT(ipv4->length == 8);
    if (ipv4->wire_length_get != NULL) {
        int length;

        ipv4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4->object_id == OF_BSN_TLV_IPV4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_check(
        ipv4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(ipv4->length == 8);
    if (ipv4->wire_length_get != NULL) {
        int length;

        ipv4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4->object_id == OF_BSN_TLV_IPV4);
    value = of_bsn_tlv_ipv4_OF_VERSION_1_3_check(
        ipv4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type vlan_vid */
    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_BSN_TLV_VLAN_VID);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_BSN_TLV_VLAN_VID);
    value = of_bsn_tlv_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type idle_notification */
    TEST_ASSERT(idle_notification->length == 4);
    if (idle_notification->wire_length_get != NULL) {
        int length;

        idle_notification->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(idle_notification->object_id == OF_BSN_TLV_IDLE_NOTIFICATION);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_check(
        idle_notification, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(idle_notification->length == 4);
    if (idle_notification->wire_length_get != NULL) {
        int length;

        idle_notification->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(idle_notification->object_id == OF_BSN_TLV_IDLE_NOTIFICATION);
    value = of_bsn_tlv_idle_notification_OF_VERSION_1_3_check(
        idle_notification, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type broadcast_query_timeout */
    TEST_ASSERT(broadcast_query_timeout->length == 8);
    if (broadcast_query_timeout->wire_length_get != NULL) {
        int length;

        broadcast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(broadcast_query_timeout->object_id == OF_BSN_TLV_BROADCAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_check(
        broadcast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(broadcast_query_timeout->length == 8);
    if (broadcast_query_timeout->wire_length_get != NULL) {
        int length;

        broadcast_query_timeout->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(broadcast_query_timeout->object_id == OF_BSN_TLV_BROADCAST_QUERY_TIMEOUT);
    value = of_bsn_tlv_broadcast_query_timeout_OF_VERSION_1_3_check(
        broadcast_query_timeout, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type tx_packets */
    TEST_ASSERT(tx_packets->length == 12);
    if (tx_packets->wire_length_get != NULL) {
        int length;

        tx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tx_packets->object_id == OF_BSN_TLV_TX_PACKETS);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_check(
        tx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(tx_packets->length == 12);
    if (tx_packets->wire_length_get != NULL) {
        int length;

        tx_packets->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tx_packets->object_id == OF_BSN_TLV_TX_PACKETS);
    value = of_bsn_tlv_tx_packets_OF_VERSION_1_3_check(
        tx_packets, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type port */
    TEST_ASSERT(port->length == 8);
    if (port->wire_length_get != NULL) {
        int length;

        port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(port->object_id == OF_BSN_TLV_PORT);
    value = of_bsn_tlv_port_OF_VERSION_1_3_check(
        port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(port->length == 8);
    if (port->wire_length_get != NULL) {
        int length;

        port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(port->object_id == OF_BSN_TLV_PORT);
    value = of_bsn_tlv_port_OF_VERSION_1_3_check(
        port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_offset */
    TEST_ASSERT(udf_offset->length == 6);
    if (udf_offset->wire_length_get != NULL) {
        int length;

        udf_offset->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_offset->object_id == OF_BSN_TLV_UDF_OFFSET);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_check(
        udf_offset, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_offset->length == 6);
    if (udf_offset->wire_length_get != NULL) {
        int length;

        udf_offset->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_offset->object_id == OF_BSN_TLV_UDF_OFFSET);
    value = of_bsn_tlv_udf_offset_OF_VERSION_1_3_check(
        udf_offset, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    /* Check two instances of type udf_anchor */
    TEST_ASSERT(udf_anchor->length == 6);
    if (udf_anchor->wire_length_get != NULL) {
        int length;

        udf_anchor->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_anchor->object_id == OF_BSN_TLV_UDF_ANCHOR);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_check(
        udf_anchor, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_tlv_next(list, &elt));

    TEST_ASSERT(udf_anchor->length == 6);
    if (udf_anchor->wire_length_get != NULL) {
        int length;

        udf_anchor->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udf_anchor->object_id == OF_BSN_TLV_UDF_ANCHOR);
    value = of_bsn_tlv_udf_anchor_OF_VERSION_1_3_check(
        udf_anchor, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_tlv_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_TLV_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 38);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_tlv_t *dup;

        TEST_ASSERT((dup = of_list_bsn_tlv_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_tlv_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bsn_vlan_counter_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate(
    of_list_bsn_vlan_counter_stats_entry_t *list, int value)
{

    of_bsn_vlan_counter_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_bsn_vlan_counter_stats_entry */
    of_bsn_vlan_counter_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bsn_vlan_counter_stats_entry */

    of_bsn_vlan_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_vlan_counter_stats_entry_append_bind(list,
            (of_bsn_vlan_counter_stats_entry_t *)elt_p);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bsn_vlan_counter_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bsn_vlan_counter_stats_entry_append_bind(list,
            (of_bsn_vlan_counter_stats_entry_t *)elt_p);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bsn_vlan_counter_stats_entry generated by
 * of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check(
    of_list_bsn_vlan_counter_stats_entry_t *list, int value)
{

    of_bsn_vlan_counter_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bsn_vlan_counter_stats_entry */
    of_bsn_vlan_counter_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bsn_vlan_counter_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BSN_VLAN_COUNTER_STATS_ENTRY);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bsn_vlan_counter_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BSN_VLAN_COUNTER_STATS_ENTRY);
    value = of_bsn_vlan_counter_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bsn_vlan_counter_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BSN_VLAN_COUNTER_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bsn_vlan_counter_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_bsn_vlan_counter_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bsn_vlan_counter_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bucket with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_OF_VERSION_1_3_populate(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket */

    of_bucket_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_append_bind(list,
            (of_bucket_t *)elt_p);
    value = of_bucket_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bucket generated by
 * of_list_bucket_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_OF_VERSION_1_3_check(
    of_list_bucket_t *list, int value)
{

    of_bucket_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bucket */
    of_bucket_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET);
    value = of_bucket_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BUCKET_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bucket_t *dup;

        TEST_ASSERT((dup = of_list_bucket_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bucket_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_bucket_counter with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_counter_OF_VERSION_1_3_populate(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int cur_len = 0;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_bucket_counter */

    of_bucket_counter_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_bucket_counter_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_bucket_counter_append_bind(list,
            (of_bucket_counter_t *)elt_p);
    value = of_bucket_counter_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_bucket_counter generated by
 * of_list_bucket_counter_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_bucket_counter_OF_VERSION_1_3_check(
    of_list_bucket_counter_t *list, int value)
{

    of_bucket_counter_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_bucket_counter */
    of_bucket_counter_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_bucket_counter_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_bucket_counter_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_BUCKET_COUNTER);
    value = of_bucket_counter_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_bucket_counter_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_BUCKET_COUNTER_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_bucket_counter_t *dup;

        TEST_ASSERT((dup = of_list_bucket_counter_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_bucket_counter_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_flow_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_3_populate(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_flow_stats_entry */

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_flow_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_flow_stats_entry_append_bind(list,
            (of_flow_stats_entry_t *)elt_p);
    value = of_flow_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_flow_stats_entry generated by
 * of_list_flow_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_flow_stats_entry_OF_VERSION_1_3_check(
    of_list_flow_stats_entry_t *list, int value)
{

    of_flow_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_flow_stats_entry */
    of_flow_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_flow_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_flow_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_FLOW_STATS_ENTRY);
    value = of_flow_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_flow_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_FLOW_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_flow_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_flow_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_flow_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_group_desc_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_group_desc_stats_entry_OF_VERSION_1_3_populate(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_desc_stats_entry */

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_desc_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_desc_stats_entry_append_bind(list,
            (of_group_desc_stats_entry_t *)elt_p);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_group_desc_stats_entry generated by
 * of_list_group_desc_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_group_desc_stats_entry_OF_VERSION_1_3_check(
    of_list_group_desc_stats_entry_t *list, int value)
{

    of_group_desc_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_group_desc_stats_entry */
    of_group_desc_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_desc_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_desc_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_DESC_STATS_ENTRY);
    value = of_group_desc_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_desc_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_GROUP_DESC_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_group_desc_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_group_desc_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_group_desc_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_group_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_group_stats_entry_OF_VERSION_1_3_populate(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_group_stats_entry */

    of_group_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_group_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_group_stats_entry_append_bind(list,
            (of_group_stats_entry_t *)elt_p);
    value = of_group_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_group_stats_entry generated by
 * of_list_group_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_group_stats_entry_OF_VERSION_1_3_check(
    of_list_group_stats_entry_t *list, int value)
{

    of_group_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_group_stats_entry */
    of_group_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_group_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_group_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_GROUP_STATS_ENTRY);
    value = of_group_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_group_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_GROUP_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_group_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_group_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_group_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_hello_elem with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_hello_elem_OF_VERSION_1_3_populate(
    of_list_hello_elem_t *list, int value)
{

    of_hello_elem_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_hello_elem_versionbitmap_t *versionbitmap;

    /* Instantiate pointers for each subclass */
    versionbitmap = &elt.versionbitmap;

    /* Append two instances of type of_hello_elem_versionbitmap */

    of_hello_elem_versionbitmap_init(versionbitmap, OF_VERSION_1_3, -1, 1);
    of_list_hello_elem_append_bind(list,
            (of_hello_elem_t *)versionbitmap);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_populate(
        versionbitmap, value);
    cur_len += versionbitmap->length;
    TEST_ASSERT(list->length == cur_len);

    of_hello_elem_versionbitmap_init(versionbitmap, OF_VERSION_1_3, -1, 1);
    of_list_hello_elem_append_bind(list,
            (of_hello_elem_t *)versionbitmap);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_populate(
        versionbitmap, value);
    cur_len += versionbitmap->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_hello_elem generated by
 * of_list_hello_elem_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_hello_elem_OF_VERSION_1_3_check(
    of_list_hello_elem_t *list, int value)
{

    of_hello_elem_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_hello_elem_versionbitmap_t *versionbitmap;

    /* Instantiate pointers for each subclass */
    versionbitmap = &elt.versionbitmap;
    TEST_OK(of_list_hello_elem_first(list, &elt));

    /* Check two instances of type versionbitmap */
    TEST_ASSERT(versionbitmap->object_id == OF_HELLO_ELEM_VERSIONBITMAP);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_check(
        versionbitmap, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_hello_elem_next(list, &elt));

    TEST_ASSERT(versionbitmap->object_id == OF_HELLO_ELEM_VERSIONBITMAP);
    value = of_hello_elem_versionbitmap_OF_VERSION_1_3_check(
        versionbitmap, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_hello_elem_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_HELLO_ELEM_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_hello_elem_t *dup;

        TEST_ASSERT((dup = of_list_hello_elem_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_hello_elem_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_instruction with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_OF_VERSION_1_3_populate(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_write_actions_t *write_actions;
    of_instruction_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_goto_table_t *goto_table;
    of_instruction_meter_t *meter;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_bsn_deny_t *bsn_deny;
    of_instruction_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_write_metadata_t *write_metadata;
    of_instruction_bsn_permit_t *bsn_permit;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    goto_table = &elt.goto_table;
    meter = &elt.meter;
    apply_actions = &elt.apply_actions;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    bsn_permit = &elt.bsn_permit;

    /* Append two instances of type of_instruction_clear_actions */

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)clear_actions);
    value = of_instruction_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_disable_src_mac_check */

    of_instruction_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_src_mac_check);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_src_mac_check);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_actions */

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_actions);
    value = of_instruction_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_arp_offload */

    of_instruction_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_arp_offload);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_arp_offload);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_disable_split_horizon_check */

    of_instruction_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_split_horizon_check);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_disable_split_horizon_check);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_goto_table */

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)goto_table);
    value = of_instruction_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_meter */

    of_instruction_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)meter);
    value = of_instruction_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)meter);
    value = of_instruction_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_apply_actions */

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)apply_actions);
    value = of_instruction_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_deny */

    of_instruction_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_deny);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_deny);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_dhcp_offload */

    of_instruction_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_dhcp_offload);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_dhcp_offload);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_packet_of_death */

    of_instruction_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_packet_of_death);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_packet_of_death);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_write_metadata */

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)write_metadata);
    value = of_instruction_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_bsn_permit */

    of_instruction_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_permit);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_append_bind(list,
            (of_instruction_t *)bsn_permit);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_instruction generated by
 * of_list_instruction_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_OF_VERSION_1_3_check(
    of_list_instruction_t *list, int value)
{

    of_instruction_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_instruction_clear_actions_t *clear_actions;
    of_instruction_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_write_actions_t *write_actions;
    of_instruction_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_goto_table_t *goto_table;
    of_instruction_meter_t *meter;
    of_instruction_apply_actions_t *apply_actions;
    of_instruction_bsn_deny_t *bsn_deny;
    of_instruction_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_write_metadata_t *write_metadata;
    of_instruction_bsn_permit_t *bsn_permit;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    goto_table = &elt.goto_table;
    meter = &elt.meter;
    apply_actions = &elt.apply_actions;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    bsn_permit = &elt.bsn_permit;
    TEST_OK(of_list_instruction_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 8);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_CLEAR_ACTIONS);
    value = of_instruction_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_disable_src_mac_check */
    TEST_ASSERT(bsn_disable_src_mac_check->length == 16);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_disable_src_mac_check->length == 16);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_WRITE_ACTIONS);
    value = of_instruction_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_arp_offload */
    TEST_ASSERT(bsn_arp_offload->length == 16);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_BSN_ARP_OFFLOAD);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_arp_offload->length == 16);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_BSN_ARP_OFFLOAD);
    value = of_instruction_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_disable_split_horizon_check */
    TEST_ASSERT(bsn_disable_split_horizon_check->length == 16);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_disable_split_horizon_check->length == 16);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(goto_table->length == 8);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_GOTO_TABLE);
    value = of_instruction_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type meter */
    TEST_ASSERT(meter->length == 8);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_METER);
    value = of_instruction_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(meter->length == 8);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_METER);
    value = of_instruction_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_APPLY_ACTIONS);
    value = of_instruction_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_deny */
    TEST_ASSERT(bsn_deny->length == 16);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_BSN_DENY);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_deny->length == 16);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_BSN_DENY);
    value = of_instruction_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_dhcp_offload */
    TEST_ASSERT(bsn_dhcp_offload->length == 16);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_BSN_DHCP_OFFLOAD);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_dhcp_offload->length == 16);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_BSN_DHCP_OFFLOAD);
    value = of_instruction_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_packet_of_death */
    TEST_ASSERT(bsn_packet_of_death->length == 16);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_BSN_PACKET_OF_DEATH);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_packet_of_death->length == 16);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_BSN_PACKET_OF_DEATH);
    value = of_instruction_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 24);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 24);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_WRITE_METADATA);
    value = of_instruction_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    /* Check two instances of type bsn_permit */
    TEST_ASSERT(bsn_permit->length == 16);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_BSN_PERMIT);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_next(list, &elt));

    TEST_ASSERT(bsn_permit->length == 16);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_BSN_PERMIT);
    value = of_instruction_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_INSTRUCTION_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 26);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_instruction_t *dup;

        TEST_ASSERT((dup = of_list_instruction_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_instruction_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_instruction_id with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_id_OF_VERSION_1_3_populate(
    of_list_instruction_id_t *list, int value)
{

    of_instruction_id_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_instruction_id_clear_actions_t *clear_actions;
    of_instruction_id_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_id_write_actions_t *write_actions;
    of_instruction_id_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_id_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_id_apply_actions_t *apply_actions;
    of_instruction_id_meter_t *meter;
    of_instruction_id_bsn_permit_t *bsn_permit;
    of_instruction_id_bsn_deny_t *bsn_deny;
    of_instruction_id_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_id_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_id_write_metadata_t *write_metadata;
    of_instruction_id_goto_table_t *goto_table;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    apply_actions = &elt.apply_actions;
    meter = &elt.meter;
    bsn_permit = &elt.bsn_permit;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    goto_table = &elt.goto_table;

    /* Append two instances of type of_instruction_id_clear_actions */

    of_instruction_id_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)clear_actions);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_clear_actions_init(clear_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)clear_actions);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_populate(
        clear_actions, value);
    cur_len += clear_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_disable_src_mac_check */

    of_instruction_id_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_src_mac_check);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_disable_src_mac_check_init(bsn_disable_src_mac_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_src_mac_check);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_populate(
        bsn_disable_src_mac_check, value);
    cur_len += bsn_disable_src_mac_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_write_actions */

    of_instruction_id_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_actions);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_actions);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_arp_offload */

    of_instruction_id_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_arp_offload);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_arp_offload_init(bsn_arp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_arp_offload);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_populate(
        bsn_arp_offload, value);
    cur_len += bsn_arp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_disable_split_horizon_check */

    of_instruction_id_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_split_horizon_check);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_disable_split_horizon_check_init(bsn_disable_split_horizon_check, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_disable_split_horizon_check);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_populate(
        bsn_disable_split_horizon_check, value);
    cur_len += bsn_disable_split_horizon_check->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_apply_actions */

    of_instruction_id_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)apply_actions);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)apply_actions);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_meter */

    of_instruction_id_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)meter);
    value = of_instruction_id_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_meter_init(meter, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)meter);
    value = of_instruction_id_meter_OF_VERSION_1_3_populate(
        meter, value);
    cur_len += meter->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_permit */

    of_instruction_id_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_permit);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_permit_init(bsn_permit, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_permit);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_populate(
        bsn_permit, value);
    cur_len += bsn_permit->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_deny */

    of_instruction_id_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_deny);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_deny_init(bsn_deny, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_deny);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_populate(
        bsn_deny, value);
    cur_len += bsn_deny->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_dhcp_offload */

    of_instruction_id_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_dhcp_offload);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_dhcp_offload_init(bsn_dhcp_offload, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_dhcp_offload);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_populate(
        bsn_dhcp_offload, value);
    cur_len += bsn_dhcp_offload->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_bsn_packet_of_death */

    of_instruction_id_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_packet_of_death);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_bsn_packet_of_death_init(bsn_packet_of_death, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)bsn_packet_of_death);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_populate(
        bsn_packet_of_death, value);
    cur_len += bsn_packet_of_death->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_write_metadata */

    of_instruction_id_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_metadata);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_write_metadata_init(write_metadata, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)write_metadata);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_populate(
        write_metadata, value);
    cur_len += write_metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_instruction_id_goto_table */

    of_instruction_id_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)goto_table);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    of_instruction_id_goto_table_init(goto_table, OF_VERSION_1_3, -1, 1);
    of_list_instruction_id_append_bind(list,
            (of_instruction_id_t *)goto_table);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_populate(
        goto_table, value);
    cur_len += goto_table->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_instruction_id generated by
 * of_list_instruction_id_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_instruction_id_OF_VERSION_1_3_check(
    of_list_instruction_id_t *list, int value)
{

    of_instruction_id_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_instruction_id_clear_actions_t *clear_actions;
    of_instruction_id_bsn_disable_src_mac_check_t *bsn_disable_src_mac_check;
    of_instruction_id_write_actions_t *write_actions;
    of_instruction_id_bsn_arp_offload_t *bsn_arp_offload;
    of_instruction_id_bsn_disable_split_horizon_check_t *bsn_disable_split_horizon_check;
    of_instruction_id_apply_actions_t *apply_actions;
    of_instruction_id_meter_t *meter;
    of_instruction_id_bsn_permit_t *bsn_permit;
    of_instruction_id_bsn_deny_t *bsn_deny;
    of_instruction_id_bsn_dhcp_offload_t *bsn_dhcp_offload;
    of_instruction_id_bsn_packet_of_death_t *bsn_packet_of_death;
    of_instruction_id_write_metadata_t *write_metadata;
    of_instruction_id_goto_table_t *goto_table;

    /* Instantiate pointers for each subclass */
    clear_actions = &elt.clear_actions;
    bsn_disable_src_mac_check = &elt.bsn_disable_src_mac_check;
    write_actions = &elt.write_actions;
    bsn_arp_offload = &elt.bsn_arp_offload;
    bsn_disable_split_horizon_check = &elt.bsn_disable_split_horizon_check;
    apply_actions = &elt.apply_actions;
    meter = &elt.meter;
    bsn_permit = &elt.bsn_permit;
    bsn_deny = &elt.bsn_deny;
    bsn_dhcp_offload = &elt.bsn_dhcp_offload;
    bsn_packet_of_death = &elt.bsn_packet_of_death;
    write_metadata = &elt.write_metadata;
    goto_table = &elt.goto_table;
    TEST_OK(of_list_instruction_id_first(list, &elt));

    /* Check two instances of type clear_actions */
    TEST_ASSERT(clear_actions->length == 4);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_ID_CLEAR_ACTIONS);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(clear_actions->length == 4);
    if (clear_actions->wire_length_get != NULL) {
        int length;

        clear_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(clear_actions->object_id == OF_INSTRUCTION_ID_CLEAR_ACTIONS);
    value = of_instruction_id_clear_actions_OF_VERSION_1_3_check(
        clear_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_disable_src_mac_check */
    TEST_ASSERT(bsn_disable_src_mac_check->length == 12);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_disable_src_mac_check->length == 12);
    if (bsn_disable_src_mac_check->wire_length_get != NULL) {
        int length;

        bsn_disable_src_mac_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_src_mac_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SRC_MAC_CHECK);
    value = of_instruction_id_bsn_disable_src_mac_check_OF_VERSION_1_3_check(
        bsn_disable_src_mac_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->length == 4);
    if (write_actions->wire_length_get != NULL) {
        int length;

        write_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_ID_WRITE_ACTIONS);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(write_actions->length == 4);
    if (write_actions->wire_length_get != NULL) {
        int length;

        write_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_actions->object_id == OF_INSTRUCTION_ID_WRITE_ACTIONS);
    value = of_instruction_id_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_arp_offload */
    TEST_ASSERT(bsn_arp_offload->length == 12);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_ID_BSN_ARP_OFFLOAD);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_arp_offload->length == 12);
    if (bsn_arp_offload->wire_length_get != NULL) {
        int length;

        bsn_arp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_arp_offload->object_id == OF_INSTRUCTION_ID_BSN_ARP_OFFLOAD);
    value = of_instruction_id_bsn_arp_offload_OF_VERSION_1_3_check(
        bsn_arp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_disable_split_horizon_check */
    TEST_ASSERT(bsn_disable_split_horizon_check->length == 12);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_disable_split_horizon_check->length == 12);
    if (bsn_disable_split_horizon_check->wire_length_get != NULL) {
        int length;

        bsn_disable_split_horizon_check->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_disable_split_horizon_check->object_id == OF_INSTRUCTION_ID_BSN_DISABLE_SPLIT_HORIZON_CHECK);
    value = of_instruction_id_bsn_disable_split_horizon_check_OF_VERSION_1_3_check(
        bsn_disable_split_horizon_check, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->length == 4);
    if (apply_actions->wire_length_get != NULL) {
        int length;

        apply_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_ID_APPLY_ACTIONS);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(apply_actions->length == 4);
    if (apply_actions->wire_length_get != NULL) {
        int length;

        apply_actions->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(apply_actions->object_id == OF_INSTRUCTION_ID_APPLY_ACTIONS);
    value = of_instruction_id_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type meter */
    TEST_ASSERT(meter->length == 4);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_ID_METER);
    value = of_instruction_id_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(meter->length == 4);
    if (meter->wire_length_get != NULL) {
        int length;

        meter->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(meter->object_id == OF_INSTRUCTION_ID_METER);
    value = of_instruction_id_meter_OF_VERSION_1_3_check(
        meter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_permit */
    TEST_ASSERT(bsn_permit->length == 12);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_ID_BSN_PERMIT);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_permit->length == 12);
    if (bsn_permit->wire_length_get != NULL) {
        int length;

        bsn_permit->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_permit->object_id == OF_INSTRUCTION_ID_BSN_PERMIT);
    value = of_instruction_id_bsn_permit_OF_VERSION_1_3_check(
        bsn_permit, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_deny */
    TEST_ASSERT(bsn_deny->length == 12);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_ID_BSN_DENY);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_deny->length == 12);
    if (bsn_deny->wire_length_get != NULL) {
        int length;

        bsn_deny->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_deny->object_id == OF_INSTRUCTION_ID_BSN_DENY);
    value = of_instruction_id_bsn_deny_OF_VERSION_1_3_check(
        bsn_deny, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_dhcp_offload */
    TEST_ASSERT(bsn_dhcp_offload->length == 12);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_ID_BSN_DHCP_OFFLOAD);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_dhcp_offload->length == 12);
    if (bsn_dhcp_offload->wire_length_get != NULL) {
        int length;

        bsn_dhcp_offload->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_dhcp_offload->object_id == OF_INSTRUCTION_ID_BSN_DHCP_OFFLOAD);
    value = of_instruction_id_bsn_dhcp_offload_OF_VERSION_1_3_check(
        bsn_dhcp_offload, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type bsn_packet_of_death */
    TEST_ASSERT(bsn_packet_of_death->length == 12);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_ID_BSN_PACKET_OF_DEATH);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(bsn_packet_of_death->length == 12);
    if (bsn_packet_of_death->wire_length_get != NULL) {
        int length;

        bsn_packet_of_death->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_packet_of_death->object_id == OF_INSTRUCTION_ID_BSN_PACKET_OF_DEATH);
    value = of_instruction_id_bsn_packet_of_death_OF_VERSION_1_3_check(
        bsn_packet_of_death, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type write_metadata */
    TEST_ASSERT(write_metadata->length == 4);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_ID_WRITE_METADATA);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(write_metadata->length == 4);
    if (write_metadata->wire_length_get != NULL) {
        int length;

        write_metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(write_metadata->object_id == OF_INSTRUCTION_ID_WRITE_METADATA);
    value = of_instruction_id_write_metadata_OF_VERSION_1_3_check(
        write_metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    /* Check two instances of type goto_table */
    TEST_ASSERT(goto_table->length == 4);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_ID_GOTO_TABLE);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_instruction_id_next(list, &elt));

    TEST_ASSERT(goto_table->length == 4);
    if (goto_table->wire_length_get != NULL) {
        int length;

        goto_table->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 4);
    }

    TEST_ASSERT(goto_table->object_id == OF_INSTRUCTION_ID_GOTO_TABLE);
    value = of_instruction_id_goto_table_OF_VERSION_1_3_check(
        goto_table, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_instruction_id_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_INSTRUCTION_ID_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 26);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_instruction_id_t *dup;

        TEST_ASSERT((dup = of_list_instruction_id_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_instruction_id_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_meter_band with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_meter_band_OF_VERSION_1_3_populate(
    of_list_meter_band_t *list, int value)
{

    of_meter_band_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_meter_band_dscp_remark_t *dscp_remark;
    of_meter_band_drop_t *drop;
    of_meter_band_ofdpa_color_set_t *ofdpa_color_set;

    /* Instantiate pointers for each subclass */
    dscp_remark = &elt.dscp_remark;
    drop = &elt.drop;
    ofdpa_color_set = &elt.ofdpa_color_set;

    /* Append two instances of type of_meter_band_dscp_remark */

    of_meter_band_dscp_remark_init(dscp_remark, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)dscp_remark);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_populate(
        dscp_remark, value);
    cur_len += dscp_remark->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_dscp_remark_init(dscp_remark, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)dscp_remark);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_populate(
        dscp_remark, value);
    cur_len += dscp_remark->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_meter_band_drop */

    of_meter_band_drop_init(drop, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)drop);
    value = of_meter_band_drop_OF_VERSION_1_3_populate(
        drop, value);
    cur_len += drop->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_drop_init(drop, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)drop);
    value = of_meter_band_drop_OF_VERSION_1_3_populate(
        drop, value);
    cur_len += drop->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_meter_band_ofdpa_color_set */

    of_meter_band_ofdpa_color_set_init(ofdpa_color_set, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)ofdpa_color_set);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_populate(
        ofdpa_color_set, value);
    cur_len += ofdpa_color_set->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_ofdpa_color_set_init(ofdpa_color_set, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_append_bind(list,
            (of_meter_band_t *)ofdpa_color_set);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_populate(
        ofdpa_color_set, value);
    cur_len += ofdpa_color_set->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_meter_band generated by
 * of_list_meter_band_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_meter_band_OF_VERSION_1_3_check(
    of_list_meter_band_t *list, int value)
{

    of_meter_band_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_meter_band_dscp_remark_t *dscp_remark;
    of_meter_band_drop_t *drop;
    of_meter_band_ofdpa_color_set_t *ofdpa_color_set;

    /* Instantiate pointers for each subclass */
    dscp_remark = &elt.dscp_remark;
    drop = &elt.drop;
    ofdpa_color_set = &elt.ofdpa_color_set;
    TEST_OK(of_list_meter_band_first(list, &elt));

    /* Check two instances of type dscp_remark */
    TEST_ASSERT(dscp_remark->length == 16);
    if (dscp_remark->wire_length_get != NULL) {
        int length;

        dscp_remark->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(dscp_remark->object_id == OF_METER_BAND_DSCP_REMARK);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_check(
        dscp_remark, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    TEST_ASSERT(dscp_remark->length == 16);
    if (dscp_remark->wire_length_get != NULL) {
        int length;

        dscp_remark->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(dscp_remark->object_id == OF_METER_BAND_DSCP_REMARK);
    value = of_meter_band_dscp_remark_OF_VERSION_1_3_check(
        dscp_remark, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    /* Check two instances of type drop */
    TEST_ASSERT(drop->length == 16);
    if (drop->wire_length_get != NULL) {
        int length;

        drop->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(drop->object_id == OF_METER_BAND_DROP);
    value = of_meter_band_drop_OF_VERSION_1_3_check(
        drop, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    TEST_ASSERT(drop->length == 16);
    if (drop->wire_length_get != NULL) {
        int length;

        drop->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(drop->object_id == OF_METER_BAND_DROP);
    value = of_meter_band_drop_OF_VERSION_1_3_check(
        drop, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    /* Check two instances of type ofdpa_color_set */
    TEST_ASSERT(ofdpa_color_set->length == 42);
    if (ofdpa_color_set->wire_length_get != NULL) {
        int length;

        ofdpa_color_set->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 42);
    }

    TEST_ASSERT(ofdpa_color_set->object_id == OF_METER_BAND_OFDPA_COLOR_SET);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_check(
        ofdpa_color_set, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_next(list, &elt));

    TEST_ASSERT(ofdpa_color_set->length == 42);
    if (ofdpa_color_set->wire_length_get != NULL) {
        int length;

        ofdpa_color_set->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 42);
    }

    TEST_ASSERT(ofdpa_color_set->object_id == OF_METER_BAND_OFDPA_COLOR_SET);
    value = of_meter_band_ofdpa_color_set_OF_VERSION_1_3_check(
        ofdpa_color_set, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_meter_band_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_METER_BAND_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 6);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_meter_band_t *dup;

        TEST_ASSERT((dup = of_list_meter_band_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_meter_band_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_meter_band_stats with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_meter_band_stats_OF_VERSION_1_3_populate(
    of_list_meter_band_stats_t *list, int value)
{

    of_meter_band_stats_t elt;
    int cur_len = 0;
    /* No subclasses for of_meter_band_stats */
    of_meter_band_stats_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_meter_band_stats */

    of_meter_band_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_stats_append_bind(list,
            (of_meter_band_stats_t *)elt_p);
    value = of_meter_band_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_band_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_band_stats_append_bind(list,
            (of_meter_band_stats_t *)elt_p);
    value = of_meter_band_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_meter_band_stats generated by
 * of_list_meter_band_stats_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_meter_band_stats_OF_VERSION_1_3_check(
    of_list_meter_band_stats_t *list, int value)
{

    of_meter_band_stats_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_meter_band_stats */
    of_meter_band_stats_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_meter_band_stats_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_METER_BAND_STATS);
    value = of_meter_band_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_band_stats_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_METER_BAND_STATS);
    value = of_meter_band_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_meter_band_stats_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_METER_BAND_STATS_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_meter_band_stats_t *dup;

        TEST_ASSERT((dup = of_list_meter_band_stats_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_meter_band_stats_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_meter_stats with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_meter_stats_OF_VERSION_1_3_populate(
    of_list_meter_stats_t *list, int value)
{

    of_meter_stats_t elt;
    int cur_len = 0;
    /* No subclasses for of_meter_stats */
    of_meter_stats_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_meter_stats */

    of_meter_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_stats_append_bind(list,
            (of_meter_stats_t *)elt_p);
    value = of_meter_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_meter_stats_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_meter_stats_append_bind(list,
            (of_meter_stats_t *)elt_p);
    value = of_meter_stats_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_meter_stats generated by
 * of_list_meter_stats_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_meter_stats_OF_VERSION_1_3_check(
    of_list_meter_stats_t *list, int value)
{

    of_meter_stats_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_meter_stats */
    of_meter_stats_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_meter_stats_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_METER_STATS);
    value = of_meter_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_meter_stats_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_METER_STATS);
    value = of_meter_stats_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_meter_stats_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_METER_STATS_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_meter_stats_t *dup;

        TEST_ASSERT((dup = of_list_meter_stats_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_meter_stats_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_oxm with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_oxm_OF_VERSION_1_3_populate(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_oxm_ofdpa_mpls_ttl_t *ofdpa_mpls_ttl;
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_ofdpa_color_actions_index_t *ofdpa_color_actions_index;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_ofdpa_qos_index_t *ofdpa_qos_index;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_ofdpa_rxfcl_t *ofdpa_rxfcl;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_ofdpa_bfd_discriminator_masked_t *ofdpa_bfd_discriminator_masked;
    of_oxm_ofdpa_actset_output_t *ofdpa_actset_output;
    of_oxm_ofdpa_dei_t *ofdpa_dei;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_ofdpa_oam_y1731_mdl_t *ofdpa_oam_y1731_mdl;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_ofdpa_bfd_discriminator_t *ofdpa_bfd_discriminator;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ofdpa_mpls_data_first_nibble_t *ofdpa_mpls_data_first_nibble;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ofdpa_mpls_l2_port_masked_t *ofdpa_mpls_l2_port_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_ofdpa_protection_index_t *ofdpa_protection_index;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_ofdpa_color_t *ofdpa_color;
    of_oxm_ofdpa_vrf_t *ofdpa_vrf;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_ofdpa_lmep_id_t *ofdpa_lmep_id;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_ofdpa_mpls_next_label_is_gal_t *ofdpa_mpls_next_label_is_gal;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_ofdpa_rx_timestamp_t *ofdpa_rx_timestamp;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ofdpa_mpls_l2_port_t *ofdpa_mpls_l2_port;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_ofdpa_txfcl_t *ofdpa_txfcl;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_ofdpa_ovid_t *ofdpa_ovid;
    of_oxm_ofdpa_traffic_class_t *ofdpa_traffic_class;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_ofdpa_l3_in_port_t *ofdpa_l3_in_port;
    of_oxm_ofdpa_oam_y1731_opcode_t *ofdpa_oam_y1731_opcode;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_ofdpa_eth_subtype_t *ofdpa_eth_subtype;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_ofdpa_mpls_ach_channel_t *ofdpa_mpls_ach_channel;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    ofdpa_mpls_ttl = &elt.ofdpa_mpls_ttl;
    mpls_tc_masked = &elt.mpls_tc_masked;
    ofdpa_color_actions_index = &elt.ofdpa_color_actions_index;
    arp_sha = &elt.arp_sha;
    ofdpa_qos_index = &elt.ofdpa_qos_index;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    ofdpa_rxfcl = &elt.ofdpa_rxfcl;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    ofdpa_bfd_discriminator_masked = &elt.ofdpa_bfd_discriminator_masked;
    ofdpa_actset_output = &elt.ofdpa_actset_output;
    ofdpa_dei = &elt.ofdpa_dei;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    ofdpa_oam_y1731_mdl = &elt.ofdpa_oam_y1731_mdl;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    ofdpa_bfd_discriminator = &elt.ofdpa_bfd_discriminator;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ofdpa_mpls_data_first_nibble = &elt.ofdpa_mpls_data_first_nibble;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ofdpa_mpls_l2_port_masked = &elt.ofdpa_mpls_l2_port_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    ofdpa_protection_index = &elt.ofdpa_protection_index;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    ofdpa_color = &elt.ofdpa_color;
    ofdpa_vrf = &elt.ofdpa_vrf;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    ofdpa_lmep_id = &elt.ofdpa_lmep_id;
    in_phy_port_masked = &elt.in_phy_port_masked;
    ofdpa_mpls_next_label_is_gal = &elt.ofdpa_mpls_next_label_is_gal;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    ofdpa_rx_timestamp = &elt.ofdpa_rx_timestamp;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ofdpa_mpls_l2_port = &elt.ofdpa_mpls_l2_port;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    ofdpa_txfcl = &elt.ofdpa_txfcl;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    ofdpa_ovid = &elt.ofdpa_ovid;
    ofdpa_traffic_class = &elt.ofdpa_traffic_class;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    ofdpa_l3_in_port = &elt.ofdpa_l3_in_port;
    ofdpa_oam_y1731_opcode = &elt.ofdpa_oam_y1731_opcode;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    ofdpa_eth_subtype = &elt.ofdpa_eth_subtype;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    ofdpa_mpls_ach_channel = &elt.ofdpa_mpls_ach_channel;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;

    /* Append two instances of type of_oxm_ofdpa_mpls_ttl */

    of_oxm_ofdpa_mpls_ttl_init(ofdpa_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ttl);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_populate(
        ofdpa_mpls_ttl, value);
    cur_len += ofdpa_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_ttl_init(ofdpa_mpls_ttl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ttl);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_populate(
        ofdpa_mpls_ttl, value);
    cur_len += ofdpa_mpls_ttl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_tc_masked */

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_masked_init(mpls_tc_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc_masked);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_populate(
        mpls_tc_masked, value);
    cur_len += mpls_tc_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_color_actions_index */

    of_oxm_ofdpa_color_actions_index_init(ofdpa_color_actions_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color_actions_index);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_populate(
        ofdpa_color_actions_index, value);
    cur_len += ofdpa_color_actions_index->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_color_actions_index_init(ofdpa_color_actions_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color_actions_index);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_populate(
        ofdpa_color_actions_index, value);
    cur_len += ofdpa_color_actions_index->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha */

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_3_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_init(arp_sha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha);
    value = of_oxm_arp_sha_OF_VERSION_1_3_populate(
        arp_sha, value);
    cur_len += arp_sha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_qos_index */

    of_oxm_ofdpa_qos_index_init(ofdpa_qos_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_qos_index);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_populate(
        ofdpa_qos_index, value);
    cur_len += ofdpa_qos_index->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_qos_index_init(ofdpa_qos_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_qos_index);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_populate(
        ofdpa_qos_index, value);
    cur_len += ofdpa_qos_index->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src_masked */

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_masked_init(eth_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src_masked);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_populate(
        eth_src_masked, value);
    cur_len += eth_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst */

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_3_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_init(eth_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst);
    value = of_oxm_eth_dst_OF_VERSION_1_3_populate(
        eth_dst, value);
    cur_len += eth_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_rxfcl */

    of_oxm_ofdpa_rxfcl_init(ofdpa_rxfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rxfcl);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_populate(
        ofdpa_rxfcl, value);
    cur_len += ofdpa_rxfcl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_rxfcl_init(ofdpa_rxfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rxfcl);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_populate(
        ofdpa_rxfcl, value);
    cur_len += ofdpa_rxfcl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id */

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_init(bsn_lag_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_populate(
        bsn_lag_id, value);
    cur_len += bsn_lag_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id */

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_init(tunnel_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_populate(
        tunnel_id, value);
    cur_len += tunnel_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op */

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_3_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_init(arp_op, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op);
    value = of_oxm_arp_op_OF_VERSION_1_3_populate(
        arp_op, value);
    cur_len += arp_op->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type */

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_3_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_init(eth_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type);
    value = of_oxm_eth_type_OF_VERSION_1_3_populate(
        eth_type, value);
    cur_len += eth_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_bfd_discriminator_masked */

    of_oxm_ofdpa_bfd_discriminator_masked_init(ofdpa_bfd_discriminator_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator_masked);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator_masked, value);
    cur_len += ofdpa_bfd_discriminator_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_bfd_discriminator_masked_init(ofdpa_bfd_discriminator_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator_masked);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator_masked, value);
    cur_len += ofdpa_bfd_discriminator_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_actset_output */

    of_oxm_ofdpa_actset_output_init(ofdpa_actset_output, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_actset_output);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_populate(
        ofdpa_actset_output, value);
    cur_len += ofdpa_actset_output->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_actset_output_init(ofdpa_actset_output, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_actset_output);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_populate(
        ofdpa_actset_output, value);
    cur_len += ofdpa_actset_output->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_dei */

    of_oxm_ofdpa_dei_init(ofdpa_dei, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_dei);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_populate(
        ofdpa_dei, value);
    cur_len += ofdpa_dei->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_dei_init(ofdpa_dei, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_dei);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_populate(
        ofdpa_dei, value);
    cur_len += ofdpa_dei->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128_masked */

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_masked_init(bsn_in_ports_128_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128_masked);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_populate(
        bsn_in_ports_128_masked, value);
    cur_len += bsn_in_ports_128_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos */

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_init(mpls_bos, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_populate(
        mpls_bos, value);
    cur_len += mpls_bos->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_oam_y1731_mdl */

    of_oxm_ofdpa_oam_y1731_mdl_init(ofdpa_oam_y1731_mdl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_mdl);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_mdl, value);
    cur_len += ofdpa_oam_y1731_mdl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_oam_y1731_mdl_init(ofdpa_oam_y1731_mdl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_mdl);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_mdl, value);
    cur_len += ofdpa_oam_y1731_mdl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa */

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_3_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_init(arp_spa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa);
    value = of_oxm_arp_spa_OF_VERSION_1_3_populate(
        arp_spa, value);
    cur_len += arp_spa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type_masked */

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_masked_init(icmpv4_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type_masked);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_populate(
        icmpv4_type_masked, value);
    cur_len += icmpv4_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_sha_masked */

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_sha_masked_init(arp_sha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_sha_masked);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_populate(
        arp_sha_masked, value);
    cur_len += arp_sha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst */

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_3_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_init(udp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst);
    value = of_oxm_udp_dst_OF_VERSION_1_3_populate(
        udp_dst, value);
    cur_len += udp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf_masked */

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_masked_init(bsn_vrf_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf_masked);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_populate(
        bsn_vrf_masked, value);
    cur_len += bsn_vrf_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target */

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_init(ipv6_nd_target, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_populate(
        ipv6_nd_target, value);
    cur_len += ipv6_nd_target->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_type_masked */

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_type_masked_init(eth_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_type_masked);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_populate(
        eth_type_masked, value);
    cur_len += eth_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_bfd_discriminator */

    of_oxm_ofdpa_bfd_discriminator_init(ofdpa_bfd_discriminator, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator, value);
    cur_len += ofdpa_bfd_discriminator->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_bfd_discriminator_init(ofdpa_bfd_discriminator, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_bfd_discriminator);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_populate(
        ofdpa_bfd_discriminator, value);
    cur_len += ofdpa_bfd_discriminator->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed_masked */

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_masked_init(bsn_global_vrf_allowed_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed_masked);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed_masked, value);
    cur_len += bsn_global_vrf_allowed_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6_masked */

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_masked_init(bsn_udf6_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6_masked);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_populate(
        bsn_udf6_masked, value);
    cur_len += bsn_udf6_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_type */

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_type_init(icmpv4_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_type);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_populate(
        icmpv4_type, value);
    cur_len += icmpv4_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_dst_masked */

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_dst_masked_init(udp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_dst_masked);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_populate(
        udp_dst_masked, value);
    cur_len += udp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst */

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_init(tcp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_populate(
        tcp_dst, value);
    cur_len += tcp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto */

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_3_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_init(ip_proto, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto);
    value = of_oxm_ip_proto_OF_VERSION_1_3_populate(
        ip_proto, value);
    cur_len += ip_proto->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata */

    of_oxm_metadata_init(metadata, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_3_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_init(metadata, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata);
    value = of_oxm_metadata_OF_VERSION_1_3_populate(
        metadata, value);
    cur_len += metadata->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_spa_masked */

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_spa_masked_init(arp_spa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_spa_masked);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_populate(
        arp_spa_masked, value);
    cur_len += arp_spa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_data_first_nibble */

    of_oxm_ofdpa_mpls_data_first_nibble_init(ofdpa_mpls_data_first_nibble, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_data_first_nibble);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_populate(
        ofdpa_mpls_data_first_nibble, value);
    cur_len += ofdpa_mpls_data_first_nibble->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_data_first_nibble_init(ofdpa_mpls_data_first_nibble, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_data_first_nibble);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_populate(
        ofdpa_mpls_data_first_nibble, value);
    cur_len += ofdpa_mpls_data_first_nibble->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel */

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_init(ipv6_flabel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_populate(
        ipv6_flabel, value);
    cur_len += ipv6_flabel->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id_masked */

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_masked_init(bsn_egr_port_group_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id_masked);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id_masked, value);
    cur_len += bsn_egr_port_group_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll_masked */

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_masked_init(ipv6_nd_tll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll_masked);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_tll_masked, value);
    cur_len += ipv6_nd_tll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa */

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_init(arp_tpa, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_populate(
        arp_tpa, value);
    cur_len += arp_tpa->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_src */

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_3_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_src_init(eth_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_src);
    value = of_oxm_eth_src_OF_VERSION_1_3_populate(
        eth_src, value);
    cur_len += eth_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src */

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_init(ipv6_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_populate(
        ipv6_src, value);
    cur_len += ipv6_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_metadata_masked */

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_metadata_masked_init(metadata_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)metadata_masked);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_populate(
        metadata_masked, value);
    cur_len += metadata_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_l2_port_masked */

    of_oxm_ofdpa_mpls_l2_port_masked_init(ofdpa_mpls_l2_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port_masked);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port_masked, value);
    cur_len += ofdpa_mpls_l2_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_l2_port_masked_init(ofdpa_mpls_l2_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port_masked);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port_masked, value);
    cur_len += ofdpa_mpls_l2_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll */

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_init(ipv6_nd_sll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_populate(
        ipv6_nd_sll, value);
    cur_len += ipv6_nd_sll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tunnel_id_masked */

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tunnel_id_masked_init(tunnel_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tunnel_id_masked);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_populate(
        tunnel_id_masked, value);
    cur_len += tunnel_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_protection_index */

    of_oxm_ofdpa_protection_index_init(ofdpa_protection_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_protection_index);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_populate(
        ofdpa_protection_index, value);
    cur_len += ofdpa_protection_index->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_protection_index_init(ofdpa_protection_index, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_protection_index);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_populate(
        ofdpa_protection_index, value);
    cur_len += ofdpa_protection_index->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_egr_port_group_id */

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_egr_port_group_id_init(bsn_egr_port_group_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_egr_port_group_id);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_populate(
        bsn_egr_port_group_id, value);
    cur_len += bsn_egr_port_group_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid */

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_init(vlan_vid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_populate(
        vlan_vid, value);
    cur_len += vlan_vid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha */

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_3_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_init(arp_tha, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha);
    value = of_oxm_arp_tha_OF_VERSION_1_3_populate(
        arp_tha, value);
    cur_len += arp_tha->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp */

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_init(ip_dscp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_populate(
        ip_dscp, value);
    cur_len += ip_dscp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7_masked */

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_masked_init(bsn_udf7_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7_masked);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_populate(
        bsn_udf7_masked, value);
    cur_len += bsn_udf7_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_bos_masked */

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_bos_masked_init(mpls_bos_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_bos_masked);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_populate(
        mpls_bos_masked, value);
    cur_len += mpls_bos_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst */

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_init(sctp_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_populate(
        sctp_dst, value);
    cur_len += sctp_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id */

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_init(bsn_l3_src_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id, value);
    cur_len += bsn_l3_src_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_eth_dst_masked */

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_eth_dst_masked_init(eth_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)eth_dst_masked);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_populate(
        eth_dst_masked, value);
    cur_len += eth_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2_masked */

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_masked_init(bsn_udf2_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2_masked);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_populate(
        bsn_udf2_masked, value);
    cur_len += bsn_udf2_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port_masked */

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_masked_init(in_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port_masked);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_populate(
        in_port_masked, value);
    cur_len += in_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn */

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_init(ip_ecn, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_populate(
        ip_ecn, value);
    cur_len += ip_ecn->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code_masked */

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_masked_init(icmpv4_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code_masked);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_populate(
        icmpv4_code_masked, value);
    cur_len += icmpv4_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_dst_masked */

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_dst_masked_init(tcp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_dst_masked);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_populate(
        tcp_dst_masked, value);
    cur_len += tcp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src_masked */

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_masked_init(tcp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src_masked);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_populate(
        tcp_src_masked, value);
    cur_len += tcp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst */

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_init(ipv4_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_populate(
        ipv4_dst, value);
    cur_len += ipv4_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_dst_masked */

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_dst_masked_init(ipv4_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_dst_masked);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_populate(
        ipv4_dst_masked, value);
    cur_len += ipv4_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_color */

    of_oxm_ofdpa_color_init(ofdpa_color, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_populate(
        ofdpa_color, value);
    cur_len += ofdpa_color->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_color_init(ofdpa_color, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_color);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_populate(
        ofdpa_color, value);
    cur_len += ofdpa_color->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_vrf */

    of_oxm_ofdpa_vrf_init(ofdpa_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_vrf);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_populate(
        ofdpa_vrf, value);
    cur_len += ofdpa_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_vrf_init(ofdpa_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_vrf);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_populate(
        ofdpa_vrf, value);
    cur_len += ofdpa_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1 */

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_init(bsn_udf1, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_populate(
        bsn_udf1, value);
    cur_len += bsn_udf1->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0 */

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_init(bsn_udf0, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_populate(
        bsn_udf0, value);
    cur_len += bsn_udf0->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_tll */

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_tll_init(ipv6_nd_tll, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_tll);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_populate(
        ipv6_nd_tll, value);
    cur_len += ipv6_nd_tll->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf2 */

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf2_init(bsn_udf2, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_populate(
        bsn_udf2, value);
    cur_len += bsn_udf2->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5 */

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_init(bsn_udf5, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_populate(
        bsn_udf5, value);
    cur_len += bsn_udf5->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4 */

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_init(bsn_udf4, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_populate(
        bsn_udf4, value);
    cur_len += bsn_udf4->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf7 */

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf7_init(bsn_udf7, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_populate(
        bsn_udf7, value);
    cur_len += bsn_udf7->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_proto_masked */

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_proto_masked_init(ip_proto_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_proto_masked);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_populate(
        ip_proto_masked, value);
    cur_len += ip_proto_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_lmep_id */

    of_oxm_ofdpa_lmep_id_init(ofdpa_lmep_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_lmep_id);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_populate(
        ofdpa_lmep_id, value);
    cur_len += ofdpa_lmep_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_lmep_id_init(ofdpa_lmep_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_lmep_id);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_populate(
        ofdpa_lmep_id, value);
    cur_len += ofdpa_lmep_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port_masked */

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_masked_init(in_phy_port_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port_masked);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_populate(
        in_phy_port_masked, value);
    cur_len += in_phy_port_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_next_label_is_gal */

    of_oxm_ofdpa_mpls_next_label_is_gal_init(ofdpa_mpls_next_label_is_gal, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_next_label_is_gal);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_populate(
        ofdpa_mpls_next_label_is_gal, value);
    cur_len += ofdpa_mpls_next_label_is_gal->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_next_label_is_gal_init(ofdpa_mpls_next_label_is_gal, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_next_label_is_gal);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_populate(
        ofdpa_mpls_next_label_is_gal, value);
    cur_len += ofdpa_mpls_next_label_is_gal->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp_masked */

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_masked_init(vlan_pcp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp_masked);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_populate(
        vlan_pcp_masked, value);
    cur_len += vlan_pcp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst_masked */

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_masked_init(ipv6_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst_masked);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_populate(
        ipv6_dst_masked, value);
    cur_len += ipv6_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_pcp */

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_pcp_init(vlan_pcp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_pcp);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_populate(
        vlan_pcp, value);
    cur_len += vlan_pcp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src */

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_init(ipv4_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_populate(
        ipv4_src, value);
    cur_len += ipv4_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_dst */

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_dst_init(ipv6_dst, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_dst);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_populate(
        ipv6_dst, value);
    cur_len += ipv6_dst->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv4_src_masked */

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv4_src_masked_init(ipv4_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv4_src_masked);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_populate(
        ipv4_src_masked, value);
    cur_len += ipv4_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_op_masked */

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_op_masked_init(arp_op_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_op_masked);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_populate(
        arp_op_masked, value);
    cur_len += arp_op_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src */

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_3_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_init(sctp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src);
    value = of_oxm_sctp_src_OF_VERSION_1_3_populate(
        sctp_src, value);
    cur_len += sctp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code */

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_init(icmpv6_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_populate(
        icmpv6_code, value);
    cur_len += icmpv6_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf5_masked */

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf5_masked_init(bsn_udf5_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf5_masked);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_populate(
        bsn_udf5_masked, value);
    cur_len += bsn_udf5_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_code_masked */

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_code_masked_init(icmpv6_code_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_code_masked);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_populate(
        icmpv6_code_masked, value);
    cur_len += icmpv6_code_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src_masked */

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_masked_init(udp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src_masked);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_populate(
        udp_src_masked, value);
    cur_len += udp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_ecn_masked */

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_ecn_masked_init(ip_ecn_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_ecn_masked);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_populate(
        ip_ecn_masked, value);
    cur_len += ip_ecn_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_udp_src */

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_3_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_udp_src_init(udp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)udp_src);
    value = of_oxm_udp_src_OF_VERSION_1_3_populate(
        udp_src, value);
    cur_len += udp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_vlan_vid_masked */

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_vlan_vid_masked_init(vlan_vid_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)vlan_vid_masked);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_populate(
        vlan_vid_masked, value);
    cur_len += vlan_vid_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tha_masked */

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tha_masked_init(arp_tha_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tha_masked);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_populate(
        arp_tha_masked, value);
    cur_len += arp_tha_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_rx_timestamp */

    of_oxm_ofdpa_rx_timestamp_init(ofdpa_rx_timestamp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rx_timestamp);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_populate(
        ofdpa_rx_timestamp, value);
    cur_len += ofdpa_rx_timestamp->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_rx_timestamp_init(ofdpa_rx_timestamp, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_rx_timestamp);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_populate(
        ofdpa_rx_timestamp, value);
    cur_len += ofdpa_rx_timestamp->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_src_masked */

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_src_masked_init(sctp_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_src_masked);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_populate(
        sctp_src_masked, value);
    cur_len += sctp_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_port */

    of_oxm_in_port_init(in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_3_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_port_init(in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_port);
    value = of_oxm_in_port_OF_VERSION_1_3_populate(
        in_port, value);
    cur_len += in_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id */

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_init(bsn_l3_interface_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id, value);
    cur_len += bsn_l3_interface_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_l2_port */

    of_oxm_ofdpa_mpls_l2_port_init(ofdpa_mpls_l2_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port, value);
    cur_len += ofdpa_mpls_l2_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_l2_port_init(ofdpa_mpls_l2_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_l2_port);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_populate(
        ofdpa_mpls_l2_port, value);
    cur_len += ofdpa_mpls_l2_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_sll_masked */

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_sll_masked_init(ipv6_nd_sll_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_sll_masked);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_populate(
        ipv6_nd_sll_masked, value);
    cur_len += ipv6_nd_sll_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_tcp_src */

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_3_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_tcp_src_init(tcp_src, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)tcp_src);
    value = of_oxm_tcp_src_OF_VERSION_1_3_populate(
        tcp_src, value);
    cur_len += tcp_src->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_nd_target_masked */

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_nd_target_masked_init(ipv6_nd_target_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_nd_target_masked);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_populate(
        ipv6_nd_target_masked, value);
    cur_len += ipv6_nd_target_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type_masked */

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_masked_init(icmpv6_type_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type_masked);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_populate(
        icmpv6_type_masked, value);
    cur_len += icmpv6_type_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf0_masked */

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf0_masked_init(bsn_udf0_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf0_masked);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_populate(
        bsn_udf0_masked, value);
    cur_len += bsn_udf0_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_txfcl */

    of_oxm_ofdpa_txfcl_init(ofdpa_txfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_txfcl);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_populate(
        ofdpa_txfcl, value);
    cur_len += ofdpa_txfcl->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_txfcl_init(ofdpa_txfcl, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_txfcl);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_populate(
        ofdpa_txfcl, value);
    cur_len += ofdpa_txfcl->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3_masked */

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_masked_init(bsn_udf3_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3_masked);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_populate(
        bsn_udf3_masked, value);
    cur_len += bsn_udf3_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label */

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_3_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_init(mpls_label, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label);
    value = of_oxm_mpls_label_OF_VERSION_1_3_populate(
        mpls_label, value);
    cur_len += mpls_label->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_ovid */

    of_oxm_ofdpa_ovid_init(ofdpa_ovid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_ovid);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_populate(
        ofdpa_ovid, value);
    cur_len += ofdpa_ovid->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_ovid_init(ofdpa_ovid, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_ovid);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_populate(
        ofdpa_ovid, value);
    cur_len += ofdpa_ovid->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_traffic_class */

    of_oxm_ofdpa_traffic_class_init(ofdpa_traffic_class, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_traffic_class);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_populate(
        ofdpa_traffic_class, value);
    cur_len += ofdpa_traffic_class->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_traffic_class_init(ofdpa_traffic_class, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_traffic_class);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_populate(
        ofdpa_traffic_class, value);
    cur_len += ofdpa_traffic_class->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_in_ports_128 */

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_in_ports_128_init(bsn_in_ports_128, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_in_ports_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_populate(
        bsn_in_ports_128, value);
    cur_len += bsn_in_ports_128->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_src_class_id_masked */

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_src_class_id_masked_init(bsn_l3_src_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_src_class_id_masked);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_src_class_id_masked, value);
    cur_len += bsn_l3_src_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_flabel_masked */

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_flabel_masked_init(ipv6_flabel_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_flabel_masked);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_populate(
        ipv6_flabel_masked, value);
    cur_len += ipv6_flabel_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_lag_id_masked */

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_lag_id_masked_init(bsn_lag_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_lag_id_masked);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_populate(
        bsn_lag_id_masked, value);
    cur_len += bsn_lag_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ip_dscp_masked */

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ip_dscp_masked_init(ip_dscp_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ip_dscp_masked);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_populate(
        ip_dscp_masked, value);
    cur_len += ip_dscp_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv6_type */

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv6_type_init(icmpv6_type, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv6_type);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_populate(
        icmpv6_type, value);
    cur_len += icmpv6_type->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_interface_class_id_masked */

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_interface_class_id_masked_init(bsn_l3_interface_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_interface_class_id_masked);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_interface_class_id_masked, value);
    cur_len += bsn_l3_interface_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_l3_in_port */

    of_oxm_ofdpa_l3_in_port_init(ofdpa_l3_in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_l3_in_port);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_populate(
        ofdpa_l3_in_port, value);
    cur_len += ofdpa_l3_in_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_l3_in_port_init(ofdpa_l3_in_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_l3_in_port);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_populate(
        ofdpa_l3_in_port, value);
    cur_len += ofdpa_l3_in_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_oam_y1731_opcode */

    of_oxm_ofdpa_oam_y1731_opcode_init(ofdpa_oam_y1731_opcode, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_opcode);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_opcode, value);
    cur_len += ofdpa_oam_y1731_opcode->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_oam_y1731_opcode_init(ofdpa_oam_y1731_opcode, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_oam_y1731_opcode);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_populate(
        ofdpa_oam_y1731_opcode, value);
    cur_len += ofdpa_oam_y1731_opcode->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_tc */

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_tc_init(mpls_tc, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_tc);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_populate(
        mpls_tc, value);
    cur_len += mpls_tc->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf3 */

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf3_init(bsn_udf3, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_populate(
        bsn_udf3, value);
    cur_len += bsn_udf3->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id_masked */

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_masked_init(bsn_l3_dst_class_id_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id_masked);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id_masked, value);
    cur_len += bsn_l3_dst_class_id_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_eth_subtype */

    of_oxm_ofdpa_eth_subtype_init(ofdpa_eth_subtype, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_eth_subtype);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_populate(
        ofdpa_eth_subtype, value);
    cur_len += ofdpa_eth_subtype->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_eth_subtype_init(ofdpa_eth_subtype, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_eth_subtype);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_populate(
        ofdpa_eth_subtype, value);
    cur_len += ofdpa_eth_subtype->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_arp_tpa_masked */

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_arp_tpa_masked_init(arp_tpa_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)arp_tpa_masked);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_populate(
        arp_tpa_masked, value);
    cur_len += arp_tpa_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_sctp_dst_masked */

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_sctp_dst_masked_init(sctp_dst_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)sctp_dst_masked);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_populate(
        sctp_dst_masked, value);
    cur_len += sctp_dst_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf6 */

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf6_init(bsn_udf6, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_populate(
        bsn_udf6, value);
    cur_len += bsn_udf6->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_icmpv4_code */

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_icmpv4_code_init(icmpv4_code, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)icmpv4_code);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_populate(
        icmpv4_code, value);
    cur_len += icmpv4_code->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_vrf */

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_vrf_init(bsn_vrf, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_vrf);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_populate(
        bsn_vrf, value);
    cur_len += bsn_vrf->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf1_masked */

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf1_masked_init(bsn_udf1_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf1_masked);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_populate(
        bsn_udf1_masked, value);
    cur_len += bsn_udf1_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_udf4_masked */

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_udf4_masked_init(bsn_udf4_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_udf4_masked);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_populate(
        bsn_udf4_masked, value);
    cur_len += bsn_udf4_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ipv6_src_masked */

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ipv6_src_masked_init(ipv6_src_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ipv6_src_masked);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_populate(
        ipv6_src_masked, value);
    cur_len += ipv6_src_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_global_vrf_allowed */

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_global_vrf_allowed_init(bsn_global_vrf_allowed, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_global_vrf_allowed);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_populate(
        bsn_global_vrf_allowed, value);
    cur_len += bsn_global_vrf_allowed->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_in_phy_port */

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_in_phy_port_init(in_phy_port, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)in_phy_port);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_populate(
        in_phy_port, value);
    cur_len += in_phy_port->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_ofdpa_mpls_ach_channel */

    of_oxm_ofdpa_mpls_ach_channel_init(ofdpa_mpls_ach_channel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ach_channel);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_populate(
        ofdpa_mpls_ach_channel, value);
    cur_len += ofdpa_mpls_ach_channel->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_ofdpa_mpls_ach_channel_init(ofdpa_mpls_ach_channel, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)ofdpa_mpls_ach_channel);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_populate(
        ofdpa_mpls_ach_channel, value);
    cur_len += ofdpa_mpls_ach_channel->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_mpls_label_masked */

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_mpls_label_masked_init(mpls_label_masked, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)mpls_label_masked);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_populate(
        mpls_label_masked, value);
    cur_len += mpls_label_masked->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_oxm_bsn_l3_dst_class_id */

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    of_oxm_bsn_l3_dst_class_id_init(bsn_l3_dst_class_id, OF_VERSION_1_3, -1, 1);
    of_list_oxm_append_bind(list,
            (of_oxm_t *)bsn_l3_dst_class_id);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_populate(
        bsn_l3_dst_class_id, value);
    cur_len += bsn_l3_dst_class_id->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_oxm generated by
 * of_list_oxm_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_oxm_OF_VERSION_1_3_check(
    of_list_oxm_t *list, int value)
{

    of_oxm_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_oxm_ofdpa_mpls_ttl_t *ofdpa_mpls_ttl;
    of_oxm_mpls_tc_masked_t *mpls_tc_masked;
    of_oxm_ofdpa_color_actions_index_t *ofdpa_color_actions_index;
    of_oxm_arp_sha_t *arp_sha;
    of_oxm_ofdpa_qos_index_t *ofdpa_qos_index;
    of_oxm_eth_src_masked_t *eth_src_masked;
    of_oxm_eth_dst_t *eth_dst;
    of_oxm_ofdpa_rxfcl_t *ofdpa_rxfcl;
    of_oxm_bsn_lag_id_t *bsn_lag_id;
    of_oxm_tunnel_id_t *tunnel_id;
    of_oxm_arp_op_t *arp_op;
    of_oxm_eth_type_t *eth_type;
    of_oxm_ofdpa_bfd_discriminator_masked_t *ofdpa_bfd_discriminator_masked;
    of_oxm_ofdpa_actset_output_t *ofdpa_actset_output;
    of_oxm_ofdpa_dei_t *ofdpa_dei;
    of_oxm_bsn_in_ports_128_masked_t *bsn_in_ports_128_masked;
    of_oxm_mpls_bos_t *mpls_bos;
    of_oxm_ofdpa_oam_y1731_mdl_t *ofdpa_oam_y1731_mdl;
    of_oxm_arp_spa_t *arp_spa;
    of_oxm_icmpv4_type_masked_t *icmpv4_type_masked;
    of_oxm_arp_sha_masked_t *arp_sha_masked;
    of_oxm_udp_dst_t *udp_dst;
    of_oxm_bsn_vrf_masked_t *bsn_vrf_masked;
    of_oxm_ipv6_nd_target_t *ipv6_nd_target;
    of_oxm_eth_type_masked_t *eth_type_masked;
    of_oxm_ofdpa_bfd_discriminator_t *ofdpa_bfd_discriminator;
    of_oxm_bsn_global_vrf_allowed_masked_t *bsn_global_vrf_allowed_masked;
    of_oxm_bsn_udf6_masked_t *bsn_udf6_masked;
    of_oxm_icmpv4_type_t *icmpv4_type;
    of_oxm_udp_dst_masked_t *udp_dst_masked;
    of_oxm_tcp_dst_t *tcp_dst;
    of_oxm_ip_proto_t *ip_proto;
    of_oxm_metadata_t *metadata;
    of_oxm_arp_spa_masked_t *arp_spa_masked;
    of_oxm_ofdpa_mpls_data_first_nibble_t *ofdpa_mpls_data_first_nibble;
    of_oxm_ipv6_flabel_t *ipv6_flabel;
    of_oxm_bsn_egr_port_group_id_masked_t *bsn_egr_port_group_id_masked;
    of_oxm_ipv6_nd_tll_masked_t *ipv6_nd_tll_masked;
    of_oxm_arp_tpa_t *arp_tpa;
    of_oxm_eth_src_t *eth_src;
    of_oxm_ipv6_src_t *ipv6_src;
    of_oxm_metadata_masked_t *metadata_masked;
    of_oxm_ofdpa_mpls_l2_port_masked_t *ofdpa_mpls_l2_port_masked;
    of_oxm_ipv6_nd_sll_t *ipv6_nd_sll;
    of_oxm_tunnel_id_masked_t *tunnel_id_masked;
    of_oxm_ofdpa_protection_index_t *ofdpa_protection_index;
    of_oxm_bsn_egr_port_group_id_t *bsn_egr_port_group_id;
    of_oxm_vlan_vid_t *vlan_vid;
    of_oxm_arp_tha_t *arp_tha;
    of_oxm_ip_dscp_t *ip_dscp;
    of_oxm_bsn_udf7_masked_t *bsn_udf7_masked;
    of_oxm_mpls_bos_masked_t *mpls_bos_masked;
    of_oxm_sctp_dst_t *sctp_dst;
    of_oxm_bsn_l3_src_class_id_t *bsn_l3_src_class_id;
    of_oxm_eth_dst_masked_t *eth_dst_masked;
    of_oxm_bsn_udf2_masked_t *bsn_udf2_masked;
    of_oxm_in_port_masked_t *in_port_masked;
    of_oxm_ip_ecn_t *ip_ecn;
    of_oxm_icmpv4_code_masked_t *icmpv4_code_masked;
    of_oxm_tcp_dst_masked_t *tcp_dst_masked;
    of_oxm_tcp_src_masked_t *tcp_src_masked;
    of_oxm_ipv4_dst_t *ipv4_dst;
    of_oxm_ipv4_dst_masked_t *ipv4_dst_masked;
    of_oxm_ofdpa_color_t *ofdpa_color;
    of_oxm_ofdpa_vrf_t *ofdpa_vrf;
    of_oxm_bsn_udf1_t *bsn_udf1;
    of_oxm_bsn_udf0_t *bsn_udf0;
    of_oxm_ipv6_nd_tll_t *ipv6_nd_tll;
    of_oxm_bsn_udf2_t *bsn_udf2;
    of_oxm_bsn_udf5_t *bsn_udf5;
    of_oxm_bsn_udf4_t *bsn_udf4;
    of_oxm_bsn_udf7_t *bsn_udf7;
    of_oxm_ip_proto_masked_t *ip_proto_masked;
    of_oxm_ofdpa_lmep_id_t *ofdpa_lmep_id;
    of_oxm_in_phy_port_masked_t *in_phy_port_masked;
    of_oxm_ofdpa_mpls_next_label_is_gal_t *ofdpa_mpls_next_label_is_gal;
    of_oxm_vlan_pcp_masked_t *vlan_pcp_masked;
    of_oxm_ipv6_dst_masked_t *ipv6_dst_masked;
    of_oxm_vlan_pcp_t *vlan_pcp;
    of_oxm_ipv4_src_t *ipv4_src;
    of_oxm_ipv6_dst_t *ipv6_dst;
    of_oxm_ipv4_src_masked_t *ipv4_src_masked;
    of_oxm_arp_op_masked_t *arp_op_masked;
    of_oxm_sctp_src_t *sctp_src;
    of_oxm_icmpv6_code_t *icmpv6_code;
    of_oxm_bsn_udf5_masked_t *bsn_udf5_masked;
    of_oxm_icmpv6_code_masked_t *icmpv6_code_masked;
    of_oxm_udp_src_masked_t *udp_src_masked;
    of_oxm_ip_ecn_masked_t *ip_ecn_masked;
    of_oxm_udp_src_t *udp_src;
    of_oxm_vlan_vid_masked_t *vlan_vid_masked;
    of_oxm_arp_tha_masked_t *arp_tha_masked;
    of_oxm_ofdpa_rx_timestamp_t *ofdpa_rx_timestamp;
    of_oxm_sctp_src_masked_t *sctp_src_masked;
    of_oxm_in_port_t *in_port;
    of_oxm_bsn_l3_interface_class_id_t *bsn_l3_interface_class_id;
    of_oxm_ofdpa_mpls_l2_port_t *ofdpa_mpls_l2_port;
    of_oxm_ipv6_nd_sll_masked_t *ipv6_nd_sll_masked;
    of_oxm_tcp_src_t *tcp_src;
    of_oxm_ipv6_nd_target_masked_t *ipv6_nd_target_masked;
    of_oxm_icmpv6_type_masked_t *icmpv6_type_masked;
    of_oxm_bsn_udf0_masked_t *bsn_udf0_masked;
    of_oxm_ofdpa_txfcl_t *ofdpa_txfcl;
    of_oxm_bsn_udf3_masked_t *bsn_udf3_masked;
    of_oxm_mpls_label_t *mpls_label;
    of_oxm_ofdpa_ovid_t *ofdpa_ovid;
    of_oxm_ofdpa_traffic_class_t *ofdpa_traffic_class;
    of_oxm_bsn_in_ports_128_t *bsn_in_ports_128;
    of_oxm_bsn_l3_src_class_id_masked_t *bsn_l3_src_class_id_masked;
    of_oxm_ipv6_flabel_masked_t *ipv6_flabel_masked;
    of_oxm_bsn_lag_id_masked_t *bsn_lag_id_masked;
    of_oxm_ip_dscp_masked_t *ip_dscp_masked;
    of_oxm_icmpv6_type_t *icmpv6_type;
    of_oxm_bsn_l3_interface_class_id_masked_t *bsn_l3_interface_class_id_masked;
    of_oxm_ofdpa_l3_in_port_t *ofdpa_l3_in_port;
    of_oxm_ofdpa_oam_y1731_opcode_t *ofdpa_oam_y1731_opcode;
    of_oxm_mpls_tc_t *mpls_tc;
    of_oxm_bsn_udf3_t *bsn_udf3;
    of_oxm_bsn_l3_dst_class_id_masked_t *bsn_l3_dst_class_id_masked;
    of_oxm_ofdpa_eth_subtype_t *ofdpa_eth_subtype;
    of_oxm_arp_tpa_masked_t *arp_tpa_masked;
    of_oxm_sctp_dst_masked_t *sctp_dst_masked;
    of_oxm_bsn_udf6_t *bsn_udf6;
    of_oxm_icmpv4_code_t *icmpv4_code;
    of_oxm_bsn_vrf_t *bsn_vrf;
    of_oxm_bsn_udf1_masked_t *bsn_udf1_masked;
    of_oxm_bsn_udf4_masked_t *bsn_udf4_masked;
    of_oxm_ipv6_src_masked_t *ipv6_src_masked;
    of_oxm_bsn_global_vrf_allowed_t *bsn_global_vrf_allowed;
    of_oxm_in_phy_port_t *in_phy_port;
    of_oxm_ofdpa_mpls_ach_channel_t *ofdpa_mpls_ach_channel;
    of_oxm_mpls_label_masked_t *mpls_label_masked;
    of_oxm_bsn_l3_dst_class_id_t *bsn_l3_dst_class_id;

    /* Instantiate pointers for each subclass */
    ofdpa_mpls_ttl = &elt.ofdpa_mpls_ttl;
    mpls_tc_masked = &elt.mpls_tc_masked;
    ofdpa_color_actions_index = &elt.ofdpa_color_actions_index;
    arp_sha = &elt.arp_sha;
    ofdpa_qos_index = &elt.ofdpa_qos_index;
    eth_src_masked = &elt.eth_src_masked;
    eth_dst = &elt.eth_dst;
    ofdpa_rxfcl = &elt.ofdpa_rxfcl;
    bsn_lag_id = &elt.bsn_lag_id;
    tunnel_id = &elt.tunnel_id;
    arp_op = &elt.arp_op;
    eth_type = &elt.eth_type;
    ofdpa_bfd_discriminator_masked = &elt.ofdpa_bfd_discriminator_masked;
    ofdpa_actset_output = &elt.ofdpa_actset_output;
    ofdpa_dei = &elt.ofdpa_dei;
    bsn_in_ports_128_masked = &elt.bsn_in_ports_128_masked;
    mpls_bos = &elt.mpls_bos;
    ofdpa_oam_y1731_mdl = &elt.ofdpa_oam_y1731_mdl;
    arp_spa = &elt.arp_spa;
    icmpv4_type_masked = &elt.icmpv4_type_masked;
    arp_sha_masked = &elt.arp_sha_masked;
    udp_dst = &elt.udp_dst;
    bsn_vrf_masked = &elt.bsn_vrf_masked;
    ipv6_nd_target = &elt.ipv6_nd_target;
    eth_type_masked = &elt.eth_type_masked;
    ofdpa_bfd_discriminator = &elt.ofdpa_bfd_discriminator;
    bsn_global_vrf_allowed_masked = &elt.bsn_global_vrf_allowed_masked;
    bsn_udf6_masked = &elt.bsn_udf6_masked;
    icmpv4_type = &elt.icmpv4_type;
    udp_dst_masked = &elt.udp_dst_masked;
    tcp_dst = &elt.tcp_dst;
    ip_proto = &elt.ip_proto;
    metadata = &elt.metadata;
    arp_spa_masked = &elt.arp_spa_masked;
    ofdpa_mpls_data_first_nibble = &elt.ofdpa_mpls_data_first_nibble;
    ipv6_flabel = &elt.ipv6_flabel;
    bsn_egr_port_group_id_masked = &elt.bsn_egr_port_group_id_masked;
    ipv6_nd_tll_masked = &elt.ipv6_nd_tll_masked;
    arp_tpa = &elt.arp_tpa;
    eth_src = &elt.eth_src;
    ipv6_src = &elt.ipv6_src;
    metadata_masked = &elt.metadata_masked;
    ofdpa_mpls_l2_port_masked = &elt.ofdpa_mpls_l2_port_masked;
    ipv6_nd_sll = &elt.ipv6_nd_sll;
    tunnel_id_masked = &elt.tunnel_id_masked;
    ofdpa_protection_index = &elt.ofdpa_protection_index;
    bsn_egr_port_group_id = &elt.bsn_egr_port_group_id;
    vlan_vid = &elt.vlan_vid;
    arp_tha = &elt.arp_tha;
    ip_dscp = &elt.ip_dscp;
    bsn_udf7_masked = &elt.bsn_udf7_masked;
    mpls_bos_masked = &elt.mpls_bos_masked;
    sctp_dst = &elt.sctp_dst;
    bsn_l3_src_class_id = &elt.bsn_l3_src_class_id;
    eth_dst_masked = &elt.eth_dst_masked;
    bsn_udf2_masked = &elt.bsn_udf2_masked;
    in_port_masked = &elt.in_port_masked;
    ip_ecn = &elt.ip_ecn;
    icmpv4_code_masked = &elt.icmpv4_code_masked;
    tcp_dst_masked = &elt.tcp_dst_masked;
    tcp_src_masked = &elt.tcp_src_masked;
    ipv4_dst = &elt.ipv4_dst;
    ipv4_dst_masked = &elt.ipv4_dst_masked;
    ofdpa_color = &elt.ofdpa_color;
    ofdpa_vrf = &elt.ofdpa_vrf;
    bsn_udf1 = &elt.bsn_udf1;
    bsn_udf0 = &elt.bsn_udf0;
    ipv6_nd_tll = &elt.ipv6_nd_tll;
    bsn_udf2 = &elt.bsn_udf2;
    bsn_udf5 = &elt.bsn_udf5;
    bsn_udf4 = &elt.bsn_udf4;
    bsn_udf7 = &elt.bsn_udf7;
    ip_proto_masked = &elt.ip_proto_masked;
    ofdpa_lmep_id = &elt.ofdpa_lmep_id;
    in_phy_port_masked = &elt.in_phy_port_masked;
    ofdpa_mpls_next_label_is_gal = &elt.ofdpa_mpls_next_label_is_gal;
    vlan_pcp_masked = &elt.vlan_pcp_masked;
    ipv6_dst_masked = &elt.ipv6_dst_masked;
    vlan_pcp = &elt.vlan_pcp;
    ipv4_src = &elt.ipv4_src;
    ipv6_dst = &elt.ipv6_dst;
    ipv4_src_masked = &elt.ipv4_src_masked;
    arp_op_masked = &elt.arp_op_masked;
    sctp_src = &elt.sctp_src;
    icmpv6_code = &elt.icmpv6_code;
    bsn_udf5_masked = &elt.bsn_udf5_masked;
    icmpv6_code_masked = &elt.icmpv6_code_masked;
    udp_src_masked = &elt.udp_src_masked;
    ip_ecn_masked = &elt.ip_ecn_masked;
    udp_src = &elt.udp_src;
    vlan_vid_masked = &elt.vlan_vid_masked;
    arp_tha_masked = &elt.arp_tha_masked;
    ofdpa_rx_timestamp = &elt.ofdpa_rx_timestamp;
    sctp_src_masked = &elt.sctp_src_masked;
    in_port = &elt.in_port;
    bsn_l3_interface_class_id = &elt.bsn_l3_interface_class_id;
    ofdpa_mpls_l2_port = &elt.ofdpa_mpls_l2_port;
    ipv6_nd_sll_masked = &elt.ipv6_nd_sll_masked;
    tcp_src = &elt.tcp_src;
    ipv6_nd_target_masked = &elt.ipv6_nd_target_masked;
    icmpv6_type_masked = &elt.icmpv6_type_masked;
    bsn_udf0_masked = &elt.bsn_udf0_masked;
    ofdpa_txfcl = &elt.ofdpa_txfcl;
    bsn_udf3_masked = &elt.bsn_udf3_masked;
    mpls_label = &elt.mpls_label;
    ofdpa_ovid = &elt.ofdpa_ovid;
    ofdpa_traffic_class = &elt.ofdpa_traffic_class;
    bsn_in_ports_128 = &elt.bsn_in_ports_128;
    bsn_l3_src_class_id_masked = &elt.bsn_l3_src_class_id_masked;
    ipv6_flabel_masked = &elt.ipv6_flabel_masked;
    bsn_lag_id_masked = &elt.bsn_lag_id_masked;
    ip_dscp_masked = &elt.ip_dscp_masked;
    icmpv6_type = &elt.icmpv6_type;
    bsn_l3_interface_class_id_masked = &elt.bsn_l3_interface_class_id_masked;
    ofdpa_l3_in_port = &elt.ofdpa_l3_in_port;
    ofdpa_oam_y1731_opcode = &elt.ofdpa_oam_y1731_opcode;
    mpls_tc = &elt.mpls_tc;
    bsn_udf3 = &elt.bsn_udf3;
    bsn_l3_dst_class_id_masked = &elt.bsn_l3_dst_class_id_masked;
    ofdpa_eth_subtype = &elt.ofdpa_eth_subtype;
    arp_tpa_masked = &elt.arp_tpa_masked;
    sctp_dst_masked = &elt.sctp_dst_masked;
    bsn_udf6 = &elt.bsn_udf6;
    icmpv4_code = &elt.icmpv4_code;
    bsn_vrf = &elt.bsn_vrf;
    bsn_udf1_masked = &elt.bsn_udf1_masked;
    bsn_udf4_masked = &elt.bsn_udf4_masked;
    ipv6_src_masked = &elt.ipv6_src_masked;
    bsn_global_vrf_allowed = &elt.bsn_global_vrf_allowed;
    in_phy_port = &elt.in_phy_port;
    ofdpa_mpls_ach_channel = &elt.ofdpa_mpls_ach_channel;
    mpls_label_masked = &elt.mpls_label_masked;
    bsn_l3_dst_class_id = &elt.bsn_l3_dst_class_id;
    TEST_OK(of_list_oxm_first(list, &elt));

    /* Check two instances of type ofdpa_mpls_ttl */
    TEST_ASSERT(ofdpa_mpls_ttl->length == 11);
    if (ofdpa_mpls_ttl->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_ttl->object_id == OF_OXM_OFDPA_MPLS_TTL);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_check(
        ofdpa_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_ttl->length == 11);
    if (ofdpa_mpls_ttl->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ttl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_ttl->object_id == OF_OXM_OFDPA_MPLS_TTL);
    value = of_oxm_ofdpa_mpls_ttl_OF_VERSION_1_3_check(
        ofdpa_mpls_ttl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_tc_masked */
    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc_masked->length == 6);
    if (mpls_tc_masked->wire_length_get != NULL) {
        int length;

        mpls_tc_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_tc_masked->object_id == OF_OXM_MPLS_TC_MASKED);
    value = of_oxm_mpls_tc_masked_OF_VERSION_1_3_check(
        mpls_tc_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_color_actions_index */
    TEST_ASSERT(ofdpa_color_actions_index->length == 14);
    if (ofdpa_color_actions_index->wire_length_get != NULL) {
        int length;

        ofdpa_color_actions_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_color_actions_index->object_id == OF_OXM_OFDPA_COLOR_ACTIONS_INDEX);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_check(
        ofdpa_color_actions_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_color_actions_index->length == 14);
    if (ofdpa_color_actions_index->wire_length_get != NULL) {
        int length;

        ofdpa_color_actions_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_color_actions_index->object_id == OF_OXM_OFDPA_COLOR_ACTIONS_INDEX);
    value = of_oxm_ofdpa_color_actions_index_OF_VERSION_1_3_check(
        ofdpa_color_actions_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha */
    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_3_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha->length == 10);
    if (arp_sha->wire_length_get != NULL) {
        int length;

        arp_sha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_sha->object_id == OF_OXM_ARP_SHA);
    value = of_oxm_arp_sha_OF_VERSION_1_3_check(
        arp_sha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_qos_index */
    TEST_ASSERT(ofdpa_qos_index->length == 11);
    if (ofdpa_qos_index->wire_length_get != NULL) {
        int length;

        ofdpa_qos_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_qos_index->object_id == OF_OXM_OFDPA_QOS_INDEX);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_check(
        ofdpa_qos_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_qos_index->length == 11);
    if (ofdpa_qos_index->wire_length_get != NULL) {
        int length;

        ofdpa_qos_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_qos_index->object_id == OF_OXM_OFDPA_QOS_INDEX);
    value = of_oxm_ofdpa_qos_index_OF_VERSION_1_3_check(
        ofdpa_qos_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src_masked */
    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src_masked->length == 16);
    if (eth_src_masked->wire_length_get != NULL) {
        int length;

        eth_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_src_masked->object_id == OF_OXM_ETH_SRC_MASKED);
    value = of_oxm_eth_src_masked_OF_VERSION_1_3_check(
        eth_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst */
    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_3_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst->length == 10);
    if (eth_dst->wire_length_get != NULL) {
        int length;

        eth_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_dst->object_id == OF_OXM_ETH_DST);
    value = of_oxm_eth_dst_OF_VERSION_1_3_check(
        eth_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_rxfcl */
    TEST_ASSERT(ofdpa_rxfcl->length == 18);
    if (ofdpa_rxfcl->wire_length_get != NULL) {
        int length;

        ofdpa_rxfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rxfcl->object_id == OF_OXM_OFDPA_RXFCL);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_check(
        ofdpa_rxfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_rxfcl->length == 18);
    if (ofdpa_rxfcl->wire_length_get != NULL) {
        int length;

        ofdpa_rxfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rxfcl->object_id == OF_OXM_OFDPA_RXFCL);
    value = of_oxm_ofdpa_rxfcl_OF_VERSION_1_3_check(
        ofdpa_rxfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id */
    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id->length == 8);
    if (bsn_lag_id->wire_length_get != NULL) {
        int length;

        bsn_lag_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_lag_id->object_id == OF_OXM_BSN_LAG_ID);
    value = of_oxm_bsn_lag_id_OF_VERSION_1_3_check(
        bsn_lag_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id */
    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id->length == 12);
    if (tunnel_id->wire_length_get != NULL) {
        int length;

        tunnel_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(tunnel_id->object_id == OF_OXM_TUNNEL_ID);
    value = of_oxm_tunnel_id_OF_VERSION_1_3_check(
        tunnel_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op */
    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_3_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op->length == 6);
    if (arp_op->wire_length_get != NULL) {
        int length;

        arp_op->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(arp_op->object_id == OF_OXM_ARP_OP);
    value = of_oxm_arp_op_OF_VERSION_1_3_check(
        arp_op, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type */
    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_3_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type->length == 6);
    if (eth_type->wire_length_get != NULL) {
        int length;

        eth_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(eth_type->object_id == OF_OXM_ETH_TYPE);
    value = of_oxm_eth_type_OF_VERSION_1_3_check(
        eth_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_bfd_discriminator_masked */
    TEST_ASSERT(ofdpa_bfd_discriminator_masked->length == 18);
    if (ofdpa_bfd_discriminator_masked->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator_masked->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR_MASKED);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_bfd_discriminator_masked->length == 18);
    if (ofdpa_bfd_discriminator_masked->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator_masked->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR_MASKED);
    value = of_oxm_ofdpa_bfd_discriminator_masked_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_actset_output */
    TEST_ASSERT(ofdpa_actset_output->length == 14);
    if (ofdpa_actset_output->wire_length_get != NULL) {
        int length;

        ofdpa_actset_output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_actset_output->object_id == OF_OXM_OFDPA_ACTSET_OUTPUT);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_check(
        ofdpa_actset_output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_actset_output->length == 14);
    if (ofdpa_actset_output->wire_length_get != NULL) {
        int length;

        ofdpa_actset_output->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_actset_output->object_id == OF_OXM_OFDPA_ACTSET_OUTPUT);
    value = of_oxm_ofdpa_actset_output_OF_VERSION_1_3_check(
        ofdpa_actset_output, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_dei */
    TEST_ASSERT(ofdpa_dei->length == 11);
    if (ofdpa_dei->wire_length_get != NULL) {
        int length;

        ofdpa_dei->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_dei->object_id == OF_OXM_OFDPA_DEI);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_check(
        ofdpa_dei, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_dei->length == 11);
    if (ofdpa_dei->wire_length_get != NULL) {
        int length;

        ofdpa_dei->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_dei->object_id == OF_OXM_OFDPA_DEI);
    value = of_oxm_ofdpa_dei_OF_VERSION_1_3_check(
        ofdpa_dei, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128_masked */
    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128_masked->length == 36);
    if (bsn_in_ports_128_masked->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(bsn_in_ports_128_masked->object_id == OF_OXM_BSN_IN_PORTS_128_MASKED);
    value = of_oxm_bsn_in_ports_128_masked_OF_VERSION_1_3_check(
        bsn_in_ports_128_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos */
    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos->length == 5);
    if (mpls_bos->wire_length_get != NULL) {
        int length;

        mpls_bos->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_bos->object_id == OF_OXM_MPLS_BOS);
    value = of_oxm_mpls_bos_OF_VERSION_1_3_check(
        mpls_bos, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_oam_y1731_mdl */
    TEST_ASSERT(ofdpa_oam_y1731_mdl->length == 11);
    if (ofdpa_oam_y1731_mdl->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_mdl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_mdl->object_id == OF_OXM_OFDPA_OAM_Y1731_MDL);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_mdl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_y1731_mdl->length == 11);
    if (ofdpa_oam_y1731_mdl->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_mdl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_mdl->object_id == OF_OXM_OFDPA_OAM_Y1731_MDL);
    value = of_oxm_ofdpa_oam_y1731_mdl_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_mdl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa */
    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_3_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa->length == 8);
    if (arp_spa->wire_length_get != NULL) {
        int length;

        arp_spa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_spa->object_id == OF_OXM_ARP_SPA);
    value = of_oxm_arp_spa_OF_VERSION_1_3_check(
        arp_spa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type_masked */
    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type_masked->length == 6);
    if (icmpv4_type_masked->wire_length_get != NULL) {
        int length;

        icmpv4_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_type_masked->object_id == OF_OXM_ICMPV4_TYPE_MASKED);
    value = of_oxm_icmpv4_type_masked_OF_VERSION_1_3_check(
        icmpv4_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_sha_masked */
    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_sha_masked->length == 16);
    if (arp_sha_masked->wire_length_get != NULL) {
        int length;

        arp_sha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_sha_masked->object_id == OF_OXM_ARP_SHA_MASKED);
    value = of_oxm_arp_sha_masked_OF_VERSION_1_3_check(
        arp_sha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst */
    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_3_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst->length == 6);
    if (udp_dst->wire_length_get != NULL) {
        int length;

        udp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_dst->object_id == OF_OXM_UDP_DST);
    value = of_oxm_udp_dst_OF_VERSION_1_3_check(
        udp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf_masked */
    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf_masked->length == 12);
    if (bsn_vrf_masked->wire_length_get != NULL) {
        int length;

        bsn_vrf_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_vrf_masked->object_id == OF_OXM_BSN_VRF_MASKED);
    value = of_oxm_bsn_vrf_masked_OF_VERSION_1_3_check(
        bsn_vrf_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target */
    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target->length == 20);
    if (ipv6_nd_target->wire_length_get != NULL) {
        int length;

        ipv6_nd_target->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_nd_target->object_id == OF_OXM_IPV6_ND_TARGET);
    value = of_oxm_ipv6_nd_target_OF_VERSION_1_3_check(
        ipv6_nd_target, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_type_masked */
    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_type_masked->length == 8);
    if (eth_type_masked->wire_length_get != NULL) {
        int length;

        eth_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(eth_type_masked->object_id == OF_OXM_ETH_TYPE_MASKED);
    value = of_oxm_eth_type_masked_OF_VERSION_1_3_check(
        eth_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_bfd_discriminator */
    TEST_ASSERT(ofdpa_bfd_discriminator->length == 14);
    if (ofdpa_bfd_discriminator->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_bfd_discriminator->length == 14);
    if (ofdpa_bfd_discriminator->wire_length_get != NULL) {
        int length;

        ofdpa_bfd_discriminator->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_bfd_discriminator->object_id == OF_OXM_OFDPA_BFD_DISCRIMINATOR);
    value = of_oxm_ofdpa_bfd_discriminator_OF_VERSION_1_3_check(
        ofdpa_bfd_discriminator, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed_masked */
    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed_masked->length == 6);
    if (bsn_global_vrf_allowed_masked->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(bsn_global_vrf_allowed_masked->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED_MASKED);
    value = of_oxm_bsn_global_vrf_allowed_masked_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6_masked */
    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6_masked->length == 12);
    if (bsn_udf6_masked->wire_length_get != NULL) {
        int length;

        bsn_udf6_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf6_masked->object_id == OF_OXM_BSN_UDF6_MASKED);
    value = of_oxm_bsn_udf6_masked_OF_VERSION_1_3_check(
        bsn_udf6_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_type */
    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_type->length == 5);
    if (icmpv4_type->wire_length_get != NULL) {
        int length;

        icmpv4_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_type->object_id == OF_OXM_ICMPV4_TYPE);
    value = of_oxm_icmpv4_type_OF_VERSION_1_3_check(
        icmpv4_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_dst_masked */
    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_dst_masked->length == 8);
    if (udp_dst_masked->wire_length_get != NULL) {
        int length;

        udp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_dst_masked->object_id == OF_OXM_UDP_DST_MASKED);
    value = of_oxm_udp_dst_masked_OF_VERSION_1_3_check(
        udp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst */
    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst->length == 6);
    if (tcp_dst->wire_length_get != NULL) {
        int length;

        tcp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_dst->object_id == OF_OXM_TCP_DST);
    value = of_oxm_tcp_dst_OF_VERSION_1_3_check(
        tcp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto */
    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_3_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto->length == 5);
    if (ip_proto->wire_length_get != NULL) {
        int length;

        ip_proto->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_proto->object_id == OF_OXM_IP_PROTO);
    value = of_oxm_ip_proto_OF_VERSION_1_3_check(
        ip_proto, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata */
    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_3_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata->length == 12);
    if (metadata->wire_length_get != NULL) {
        int length;

        metadata->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(metadata->object_id == OF_OXM_METADATA);
    value = of_oxm_metadata_OF_VERSION_1_3_check(
        metadata, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_spa_masked */
    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_spa_masked->length == 12);
    if (arp_spa_masked->wire_length_get != NULL) {
        int length;

        arp_spa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_spa_masked->object_id == OF_OXM_ARP_SPA_MASKED);
    value = of_oxm_arp_spa_masked_OF_VERSION_1_3_check(
        arp_spa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_data_first_nibble */
    TEST_ASSERT(ofdpa_mpls_data_first_nibble->length == 11);
    if (ofdpa_mpls_data_first_nibble->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_data_first_nibble->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_data_first_nibble->object_id == OF_OXM_OFDPA_MPLS_DATA_FIRST_NIBBLE);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_check(
        ofdpa_mpls_data_first_nibble, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_data_first_nibble->length == 11);
    if (ofdpa_mpls_data_first_nibble->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_data_first_nibble->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_data_first_nibble->object_id == OF_OXM_OFDPA_MPLS_DATA_FIRST_NIBBLE);
    value = of_oxm_ofdpa_mpls_data_first_nibble_OF_VERSION_1_3_check(
        ofdpa_mpls_data_first_nibble, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel */
    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel->length == 8);
    if (ipv6_flabel->wire_length_get != NULL) {
        int length;

        ipv6_flabel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv6_flabel->object_id == OF_OXM_IPV6_FLABEL);
    value = of_oxm_ipv6_flabel_OF_VERSION_1_3_check(
        ipv6_flabel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id_masked */
    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id_masked->length == 12);
    if (bsn_egr_port_group_id_masked->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_egr_port_group_id_masked->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID_MASKED);
    value = of_oxm_bsn_egr_port_group_id_masked_OF_VERSION_1_3_check(
        bsn_egr_port_group_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll_masked */
    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll_masked->length == 16);
    if (ipv6_nd_tll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_tll_masked->object_id == OF_OXM_IPV6_ND_TLL_MASKED);
    value = of_oxm_ipv6_nd_tll_masked_OF_VERSION_1_3_check(
        ipv6_nd_tll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa */
    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa->length == 8);
    if (arp_tpa->wire_length_get != NULL) {
        int length;

        arp_tpa->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_tpa->object_id == OF_OXM_ARP_TPA);
    value = of_oxm_arp_tpa_OF_VERSION_1_3_check(
        arp_tpa, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_src */
    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_3_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_src->length == 10);
    if (eth_src->wire_length_get != NULL) {
        int length;

        eth_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(eth_src->object_id == OF_OXM_ETH_SRC);
    value = of_oxm_eth_src_OF_VERSION_1_3_check(
        eth_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src */
    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src->length == 20);
    if (ipv6_src->wire_length_get != NULL) {
        int length;

        ipv6_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_src->object_id == OF_OXM_IPV6_SRC);
    value = of_oxm_ipv6_src_OF_VERSION_1_3_check(
        ipv6_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type metadata_masked */
    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(metadata_masked->length == 20);
    if (metadata_masked->wire_length_get != NULL) {
        int length;

        metadata_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(metadata_masked->object_id == OF_OXM_METADATA_MASKED);
    value = of_oxm_metadata_masked_OF_VERSION_1_3_check(
        metadata_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_l2_port_masked */
    TEST_ASSERT(ofdpa_mpls_l2_port_masked->length == 18);
    if (ofdpa_mpls_l2_port_masked->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port_masked->object_id == OF_OXM_OFDPA_MPLS_L2_PORT_MASKED);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_l2_port_masked->length == 18);
    if (ofdpa_mpls_l2_port_masked->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port_masked->object_id == OF_OXM_OFDPA_MPLS_L2_PORT_MASKED);
    value = of_oxm_ofdpa_mpls_l2_port_masked_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll */
    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll->length == 10);
    if (ipv6_nd_sll->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_sll->object_id == OF_OXM_IPV6_ND_SLL);
    value = of_oxm_ipv6_nd_sll_OF_VERSION_1_3_check(
        ipv6_nd_sll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tunnel_id_masked */
    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tunnel_id_masked->length == 20);
    if (tunnel_id_masked->wire_length_get != NULL) {
        int length;

        tunnel_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(tunnel_id_masked->object_id == OF_OXM_TUNNEL_ID_MASKED);
    value = of_oxm_tunnel_id_masked_OF_VERSION_1_3_check(
        tunnel_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_protection_index */
    TEST_ASSERT(ofdpa_protection_index->length == 11);
    if (ofdpa_protection_index->wire_length_get != NULL) {
        int length;

        ofdpa_protection_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_protection_index->object_id == OF_OXM_OFDPA_PROTECTION_INDEX);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_check(
        ofdpa_protection_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_protection_index->length == 11);
    if (ofdpa_protection_index->wire_length_get != NULL) {
        int length;

        ofdpa_protection_index->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_protection_index->object_id == OF_OXM_OFDPA_PROTECTION_INDEX);
    value = of_oxm_ofdpa_protection_index_OF_VERSION_1_3_check(
        ofdpa_protection_index, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_egr_port_group_id */
    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_egr_port_group_id->length == 8);
    if (bsn_egr_port_group_id->wire_length_get != NULL) {
        int length;

        bsn_egr_port_group_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_egr_port_group_id->object_id == OF_OXM_BSN_EGR_PORT_GROUP_ID);
    value = of_oxm_bsn_egr_port_group_id_OF_VERSION_1_3_check(
        bsn_egr_port_group_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid */
    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid->length == 6);
    if (vlan_vid->wire_length_get != NULL) {
        int length;

        vlan_vid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_vid->object_id == OF_OXM_VLAN_VID);
    value = of_oxm_vlan_vid_OF_VERSION_1_3_check(
        vlan_vid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha */
    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_3_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha->length == 10);
    if (arp_tha->wire_length_get != NULL) {
        int length;

        arp_tha->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(arp_tha->object_id == OF_OXM_ARP_THA);
    value = of_oxm_arp_tha_OF_VERSION_1_3_check(
        arp_tha, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp */
    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp->length == 5);
    if (ip_dscp->wire_length_get != NULL) {
        int length;

        ip_dscp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_dscp->object_id == OF_OXM_IP_DSCP);
    value = of_oxm_ip_dscp_OF_VERSION_1_3_check(
        ip_dscp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7_masked */
    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7_masked->length == 12);
    if (bsn_udf7_masked->wire_length_get != NULL) {
        int length;

        bsn_udf7_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf7_masked->object_id == OF_OXM_BSN_UDF7_MASKED);
    value = of_oxm_bsn_udf7_masked_OF_VERSION_1_3_check(
        bsn_udf7_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_bos_masked */
    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_bos_masked->length == 6);
    if (mpls_bos_masked->wire_length_get != NULL) {
        int length;

        mpls_bos_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(mpls_bos_masked->object_id == OF_OXM_MPLS_BOS_MASKED);
    value = of_oxm_mpls_bos_masked_OF_VERSION_1_3_check(
        mpls_bos_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst */
    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst->length == 6);
    if (sctp_dst->wire_length_get != NULL) {
        int length;

        sctp_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_dst->object_id == OF_OXM_SCTP_DST);
    value = of_oxm_sctp_dst_OF_VERSION_1_3_check(
        sctp_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id */
    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id->length == 8);
    if (bsn_l3_src_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_src_class_id->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID);
    value = of_oxm_bsn_l3_src_class_id_OF_VERSION_1_3_check(
        bsn_l3_src_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type eth_dst_masked */
    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(eth_dst_masked->length == 16);
    if (eth_dst_masked->wire_length_get != NULL) {
        int length;

        eth_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(eth_dst_masked->object_id == OF_OXM_ETH_DST_MASKED);
    value = of_oxm_eth_dst_masked_OF_VERSION_1_3_check(
        eth_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2_masked */
    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2_masked->length == 12);
    if (bsn_udf2_masked->wire_length_get != NULL) {
        int length;

        bsn_udf2_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf2_masked->object_id == OF_OXM_BSN_UDF2_MASKED);
    value = of_oxm_bsn_udf2_masked_OF_VERSION_1_3_check(
        bsn_udf2_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port_masked */
    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port_masked->length == 12);
    if (in_port_masked->wire_length_get != NULL) {
        int length;

        in_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_port_masked->object_id == OF_OXM_IN_PORT_MASKED);
    value = of_oxm_in_port_masked_OF_VERSION_1_3_check(
        in_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn */
    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn->length == 5);
    if (ip_ecn->wire_length_get != NULL) {
        int length;

        ip_ecn->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(ip_ecn->object_id == OF_OXM_IP_ECN);
    value = of_oxm_ip_ecn_OF_VERSION_1_3_check(
        ip_ecn, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code_masked */
    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code_masked->length == 6);
    if (icmpv4_code_masked->wire_length_get != NULL) {
        int length;

        icmpv4_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv4_code_masked->object_id == OF_OXM_ICMPV4_CODE_MASKED);
    value = of_oxm_icmpv4_code_masked_OF_VERSION_1_3_check(
        icmpv4_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_dst_masked */
    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_dst_masked->length == 8);
    if (tcp_dst_masked->wire_length_get != NULL) {
        int length;

        tcp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_dst_masked->object_id == OF_OXM_TCP_DST_MASKED);
    value = of_oxm_tcp_dst_masked_OF_VERSION_1_3_check(
        tcp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src_masked */
    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src_masked->length == 8);
    if (tcp_src_masked->wire_length_get != NULL) {
        int length;

        tcp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(tcp_src_masked->object_id == OF_OXM_TCP_SRC_MASKED);
    value = of_oxm_tcp_src_masked_OF_VERSION_1_3_check(
        tcp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst */
    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst->length == 8);
    if (ipv4_dst->wire_length_get != NULL) {
        int length;

        ipv4_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_dst->object_id == OF_OXM_IPV4_DST);
    value = of_oxm_ipv4_dst_OF_VERSION_1_3_check(
        ipv4_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_dst_masked */
    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_dst_masked->length == 12);
    if (ipv4_dst_masked->wire_length_get != NULL) {
        int length;

        ipv4_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_dst_masked->object_id == OF_OXM_IPV4_DST_MASKED);
    value = of_oxm_ipv4_dst_masked_OF_VERSION_1_3_check(
        ipv4_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_color */
    TEST_ASSERT(ofdpa_color->length == 11);
    if (ofdpa_color->wire_length_get != NULL) {
        int length;

        ofdpa_color->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_color->object_id == OF_OXM_OFDPA_COLOR);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_check(
        ofdpa_color, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_color->length == 11);
    if (ofdpa_color->wire_length_get != NULL) {
        int length;

        ofdpa_color->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_color->object_id == OF_OXM_OFDPA_COLOR);
    value = of_oxm_ofdpa_color_OF_VERSION_1_3_check(
        ofdpa_color, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_vrf */
    TEST_ASSERT(ofdpa_vrf->length == 12);
    if (ofdpa_vrf->wire_length_get != NULL) {
        int length;

        ofdpa_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_vrf->object_id == OF_OXM_OFDPA_VRF);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_check(
        ofdpa_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_vrf->length == 12);
    if (ofdpa_vrf->wire_length_get != NULL) {
        int length;

        ofdpa_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_vrf->object_id == OF_OXM_OFDPA_VRF);
    value = of_oxm_ofdpa_vrf_OF_VERSION_1_3_check(
        ofdpa_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1 */
    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1->length == 8);
    if (bsn_udf1->wire_length_get != NULL) {
        int length;

        bsn_udf1->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf1->object_id == OF_OXM_BSN_UDF1);
    value = of_oxm_bsn_udf1_OF_VERSION_1_3_check(
        bsn_udf1, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0 */
    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0->length == 8);
    if (bsn_udf0->wire_length_get != NULL) {
        int length;

        bsn_udf0->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf0->object_id == OF_OXM_BSN_UDF0);
    value = of_oxm_bsn_udf0_OF_VERSION_1_3_check(
        bsn_udf0, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_tll */
    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_tll->length == 10);
    if (ipv6_nd_tll->wire_length_get != NULL) {
        int length;

        ipv6_nd_tll->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 10);
    }

    TEST_ASSERT(ipv6_nd_tll->object_id == OF_OXM_IPV6_ND_TLL);
    value = of_oxm_ipv6_nd_tll_OF_VERSION_1_3_check(
        ipv6_nd_tll, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf2 */
    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf2->length == 8);
    if (bsn_udf2->wire_length_get != NULL) {
        int length;

        bsn_udf2->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf2->object_id == OF_OXM_BSN_UDF2);
    value = of_oxm_bsn_udf2_OF_VERSION_1_3_check(
        bsn_udf2, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5 */
    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5->length == 8);
    if (bsn_udf5->wire_length_get != NULL) {
        int length;

        bsn_udf5->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf5->object_id == OF_OXM_BSN_UDF5);
    value = of_oxm_bsn_udf5_OF_VERSION_1_3_check(
        bsn_udf5, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4 */
    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4->length == 8);
    if (bsn_udf4->wire_length_get != NULL) {
        int length;

        bsn_udf4->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf4->object_id == OF_OXM_BSN_UDF4);
    value = of_oxm_bsn_udf4_OF_VERSION_1_3_check(
        bsn_udf4, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf7 */
    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf7->length == 8);
    if (bsn_udf7->wire_length_get != NULL) {
        int length;

        bsn_udf7->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf7->object_id == OF_OXM_BSN_UDF7);
    value = of_oxm_bsn_udf7_OF_VERSION_1_3_check(
        bsn_udf7, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_proto_masked */
    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_proto_masked->length == 6);
    if (ip_proto_masked->wire_length_get != NULL) {
        int length;

        ip_proto_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_proto_masked->object_id == OF_OXM_IP_PROTO_MASKED);
    value = of_oxm_ip_proto_masked_OF_VERSION_1_3_check(
        ip_proto_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_lmep_id */
    TEST_ASSERT(ofdpa_lmep_id->length == 14);
    if (ofdpa_lmep_id->wire_length_get != NULL) {
        int length;

        ofdpa_lmep_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_lmep_id->object_id == OF_OXM_OFDPA_LMEP_ID);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_check(
        ofdpa_lmep_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_lmep_id->length == 14);
    if (ofdpa_lmep_id->wire_length_get != NULL) {
        int length;

        ofdpa_lmep_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_lmep_id->object_id == OF_OXM_OFDPA_LMEP_ID);
    value = of_oxm_ofdpa_lmep_id_OF_VERSION_1_3_check(
        ofdpa_lmep_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port_masked */
    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port_masked->length == 12);
    if (in_phy_port_masked->wire_length_get != NULL) {
        int length;

        in_phy_port_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(in_phy_port_masked->object_id == OF_OXM_IN_PHY_PORT_MASKED);
    value = of_oxm_in_phy_port_masked_OF_VERSION_1_3_check(
        in_phy_port_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_next_label_is_gal */
    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->length == 11);
    if (ofdpa_mpls_next_label_is_gal->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_next_label_is_gal->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->object_id == OF_OXM_OFDPA_MPLS_NEXT_LABEL_IS_GAL);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_check(
        ofdpa_mpls_next_label_is_gal, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->length == 11);
    if (ofdpa_mpls_next_label_is_gal->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_next_label_is_gal->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_mpls_next_label_is_gal->object_id == OF_OXM_OFDPA_MPLS_NEXT_LABEL_IS_GAL);
    value = of_oxm_ofdpa_mpls_next_label_is_gal_OF_VERSION_1_3_check(
        ofdpa_mpls_next_label_is_gal, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp_masked */
    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp_masked->length == 6);
    if (vlan_pcp_masked->wire_length_get != NULL) {
        int length;

        vlan_pcp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(vlan_pcp_masked->object_id == OF_OXM_VLAN_PCP_MASKED);
    value = of_oxm_vlan_pcp_masked_OF_VERSION_1_3_check(
        vlan_pcp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst_masked */
    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst_masked->length == 36);
    if (ipv6_dst_masked->wire_length_get != NULL) {
        int length;

        ipv6_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_dst_masked->object_id == OF_OXM_IPV6_DST_MASKED);
    value = of_oxm_ipv6_dst_masked_OF_VERSION_1_3_check(
        ipv6_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_pcp */
    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_pcp->length == 5);
    if (vlan_pcp->wire_length_get != NULL) {
        int length;

        vlan_pcp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(vlan_pcp->object_id == OF_OXM_VLAN_PCP);
    value = of_oxm_vlan_pcp_OF_VERSION_1_3_check(
        vlan_pcp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src */
    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src->length == 8);
    if (ipv4_src->wire_length_get != NULL) {
        int length;

        ipv4_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(ipv4_src->object_id == OF_OXM_IPV4_SRC);
    value = of_oxm_ipv4_src_OF_VERSION_1_3_check(
        ipv4_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_dst */
    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_dst->length == 20);
    if (ipv6_dst->wire_length_get != NULL) {
        int length;

        ipv6_dst->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(ipv6_dst->object_id == OF_OXM_IPV6_DST);
    value = of_oxm_ipv6_dst_OF_VERSION_1_3_check(
        ipv6_dst, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv4_src_masked */
    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv4_src_masked->length == 12);
    if (ipv4_src_masked->wire_length_get != NULL) {
        int length;

        ipv4_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv4_src_masked->object_id == OF_OXM_IPV4_SRC_MASKED);
    value = of_oxm_ipv4_src_masked_OF_VERSION_1_3_check(
        ipv4_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_op_masked */
    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_op_masked->length == 8);
    if (arp_op_masked->wire_length_get != NULL) {
        int length;

        arp_op_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(arp_op_masked->object_id == OF_OXM_ARP_OP_MASKED);
    value = of_oxm_arp_op_masked_OF_VERSION_1_3_check(
        arp_op_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src */
    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_3_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src->length == 6);
    if (sctp_src->wire_length_get != NULL) {
        int length;

        sctp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(sctp_src->object_id == OF_OXM_SCTP_SRC);
    value = of_oxm_sctp_src_OF_VERSION_1_3_check(
        sctp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code */
    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code->length == 5);
    if (icmpv6_code->wire_length_get != NULL) {
        int length;

        icmpv6_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_code->object_id == OF_OXM_ICMPV6_CODE);
    value = of_oxm_icmpv6_code_OF_VERSION_1_3_check(
        icmpv6_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf5_masked */
    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf5_masked->length == 12);
    if (bsn_udf5_masked->wire_length_get != NULL) {
        int length;

        bsn_udf5_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf5_masked->object_id == OF_OXM_BSN_UDF5_MASKED);
    value = of_oxm_bsn_udf5_masked_OF_VERSION_1_3_check(
        bsn_udf5_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_code_masked */
    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_code_masked->length == 6);
    if (icmpv6_code_masked->wire_length_get != NULL) {
        int length;

        icmpv6_code_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_code_masked->object_id == OF_OXM_ICMPV6_CODE_MASKED);
    value = of_oxm_icmpv6_code_masked_OF_VERSION_1_3_check(
        icmpv6_code_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src_masked */
    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src_masked->length == 8);
    if (udp_src_masked->wire_length_get != NULL) {
        int length;

        udp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(udp_src_masked->object_id == OF_OXM_UDP_SRC_MASKED);
    value = of_oxm_udp_src_masked_OF_VERSION_1_3_check(
        udp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_ecn_masked */
    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_ecn_masked->length == 6);
    if (ip_ecn_masked->wire_length_get != NULL) {
        int length;

        ip_ecn_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_ecn_masked->object_id == OF_OXM_IP_ECN_MASKED);
    value = of_oxm_ip_ecn_masked_OF_VERSION_1_3_check(
        ip_ecn_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type udp_src */
    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_3_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(udp_src->length == 6);
    if (udp_src->wire_length_get != NULL) {
        int length;

        udp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(udp_src->object_id == OF_OXM_UDP_SRC);
    value = of_oxm_udp_src_OF_VERSION_1_3_check(
        udp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type vlan_vid_masked */
    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(vlan_vid_masked->length == 8);
    if (vlan_vid_masked->wire_length_get != NULL) {
        int length;

        vlan_vid_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(vlan_vid_masked->object_id == OF_OXM_VLAN_VID_MASKED);
    value = of_oxm_vlan_vid_masked_OF_VERSION_1_3_check(
        vlan_vid_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tha_masked */
    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tha_masked->length == 16);
    if (arp_tha_masked->wire_length_get != NULL) {
        int length;

        arp_tha_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(arp_tha_masked->object_id == OF_OXM_ARP_THA_MASKED);
    value = of_oxm_arp_tha_masked_OF_VERSION_1_3_check(
        arp_tha_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_rx_timestamp */
    TEST_ASSERT(ofdpa_rx_timestamp->length == 18);
    if (ofdpa_rx_timestamp->wire_length_get != NULL) {
        int length;

        ofdpa_rx_timestamp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rx_timestamp->object_id == OF_OXM_OFDPA_RX_TIMESTAMP);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_check(
        ofdpa_rx_timestamp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_rx_timestamp->length == 18);
    if (ofdpa_rx_timestamp->wire_length_get != NULL) {
        int length;

        ofdpa_rx_timestamp->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_rx_timestamp->object_id == OF_OXM_OFDPA_RX_TIMESTAMP);
    value = of_oxm_ofdpa_rx_timestamp_OF_VERSION_1_3_check(
        ofdpa_rx_timestamp, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_src_masked */
    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_src_masked->length == 8);
    if (sctp_src_masked->wire_length_get != NULL) {
        int length;

        sctp_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_src_masked->object_id == OF_OXM_SCTP_SRC_MASKED);
    value = of_oxm_sctp_src_masked_OF_VERSION_1_3_check(
        sctp_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_port */
    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_3_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_port->length == 8);
    if (in_port->wire_length_get != NULL) {
        int length;

        in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_port->object_id == OF_OXM_IN_PORT);
    value = of_oxm_in_port_OF_VERSION_1_3_check(
        in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id */
    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id->length == 8);
    if (bsn_l3_interface_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_interface_class_id->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID);
    value = of_oxm_bsn_l3_interface_class_id_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_l2_port */
    TEST_ASSERT(ofdpa_mpls_l2_port->length == 14);
    if (ofdpa_mpls_l2_port->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port->object_id == OF_OXM_OFDPA_MPLS_L2_PORT);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_l2_port->length == 14);
    if (ofdpa_mpls_l2_port->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_l2_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_mpls_l2_port->object_id == OF_OXM_OFDPA_MPLS_L2_PORT);
    value = of_oxm_ofdpa_mpls_l2_port_OF_VERSION_1_3_check(
        ofdpa_mpls_l2_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_sll_masked */
    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_sll_masked->length == 16);
    if (ipv6_nd_sll_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_sll_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(ipv6_nd_sll_masked->object_id == OF_OXM_IPV6_ND_SLL_MASKED);
    value = of_oxm_ipv6_nd_sll_masked_OF_VERSION_1_3_check(
        ipv6_nd_sll_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type tcp_src */
    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_3_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(tcp_src->length == 6);
    if (tcp_src->wire_length_get != NULL) {
        int length;

        tcp_src->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(tcp_src->object_id == OF_OXM_TCP_SRC);
    value = of_oxm_tcp_src_OF_VERSION_1_3_check(
        tcp_src, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_nd_target_masked */
    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_nd_target_masked->length == 36);
    if (ipv6_nd_target_masked->wire_length_get != NULL) {
        int length;

        ipv6_nd_target_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_nd_target_masked->object_id == OF_OXM_IPV6_ND_TARGET_MASKED);
    value = of_oxm_ipv6_nd_target_masked_OF_VERSION_1_3_check(
        ipv6_nd_target_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type_masked */
    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type_masked->length == 6);
    if (icmpv6_type_masked->wire_length_get != NULL) {
        int length;

        icmpv6_type_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(icmpv6_type_masked->object_id == OF_OXM_ICMPV6_TYPE_MASKED);
    value = of_oxm_icmpv6_type_masked_OF_VERSION_1_3_check(
        icmpv6_type_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf0_masked */
    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf0_masked->length == 12);
    if (bsn_udf0_masked->wire_length_get != NULL) {
        int length;

        bsn_udf0_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf0_masked->object_id == OF_OXM_BSN_UDF0_MASKED);
    value = of_oxm_bsn_udf0_masked_OF_VERSION_1_3_check(
        bsn_udf0_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_txfcl */
    TEST_ASSERT(ofdpa_txfcl->length == 18);
    if (ofdpa_txfcl->wire_length_get != NULL) {
        int length;

        ofdpa_txfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_txfcl->object_id == OF_OXM_OFDPA_TXFCL);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_check(
        ofdpa_txfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_txfcl->length == 18);
    if (ofdpa_txfcl->wire_length_get != NULL) {
        int length;

        ofdpa_txfcl->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 18);
    }

    TEST_ASSERT(ofdpa_txfcl->object_id == OF_OXM_OFDPA_TXFCL);
    value = of_oxm_ofdpa_txfcl_OF_VERSION_1_3_check(
        ofdpa_txfcl, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3_masked */
    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3_masked->length == 12);
    if (bsn_udf3_masked->wire_length_get != NULL) {
        int length;

        bsn_udf3_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf3_masked->object_id == OF_OXM_BSN_UDF3_MASKED);
    value = of_oxm_bsn_udf3_masked_OF_VERSION_1_3_check(
        bsn_udf3_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label */
    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_3_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label->length == 8);
    if (mpls_label->wire_length_get != NULL) {
        int length;

        mpls_label->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(mpls_label->object_id == OF_OXM_MPLS_LABEL);
    value = of_oxm_mpls_label_OF_VERSION_1_3_check(
        mpls_label, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_ovid */
    TEST_ASSERT(ofdpa_ovid->length == 12);
    if (ofdpa_ovid->wire_length_get != NULL) {
        int length;

        ofdpa_ovid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_ovid->object_id == OF_OXM_OFDPA_OVID);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_check(
        ofdpa_ovid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_ovid->length == 12);
    if (ofdpa_ovid->wire_length_get != NULL) {
        int length;

        ofdpa_ovid->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_ovid->object_id == OF_OXM_OFDPA_OVID);
    value = of_oxm_ofdpa_ovid_OF_VERSION_1_3_check(
        ofdpa_ovid, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_traffic_class */
    TEST_ASSERT(ofdpa_traffic_class->length == 11);
    if (ofdpa_traffic_class->wire_length_get != NULL) {
        int length;

        ofdpa_traffic_class->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_traffic_class->object_id == OF_OXM_OFDPA_TRAFFIC_CLASS);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_check(
        ofdpa_traffic_class, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_traffic_class->length == 11);
    if (ofdpa_traffic_class->wire_length_get != NULL) {
        int length;

        ofdpa_traffic_class->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_traffic_class->object_id == OF_OXM_OFDPA_TRAFFIC_CLASS);
    value = of_oxm_ofdpa_traffic_class_OF_VERSION_1_3_check(
        ofdpa_traffic_class, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_in_ports_128 */
    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_in_ports_128->length == 20);
    if (bsn_in_ports_128->wire_length_get != NULL) {
        int length;

        bsn_in_ports_128->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 20);
    }

    TEST_ASSERT(bsn_in_ports_128->object_id == OF_OXM_BSN_IN_PORTS_128);
    value = of_oxm_bsn_in_ports_128_OF_VERSION_1_3_check(
        bsn_in_ports_128, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_src_class_id_masked */
    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_src_class_id_masked->length == 12);
    if (bsn_l3_src_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_src_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_src_class_id_masked->object_id == OF_OXM_BSN_L3_SRC_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_src_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_src_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_flabel_masked */
    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_flabel_masked->length == 12);
    if (ipv6_flabel_masked->wire_length_get != NULL) {
        int length;

        ipv6_flabel_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ipv6_flabel_masked->object_id == OF_OXM_IPV6_FLABEL_MASKED);
    value = of_oxm_ipv6_flabel_masked_OF_VERSION_1_3_check(
        ipv6_flabel_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_lag_id_masked */
    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_lag_id_masked->length == 12);
    if (bsn_lag_id_masked->wire_length_get != NULL) {
        int length;

        bsn_lag_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_lag_id_masked->object_id == OF_OXM_BSN_LAG_ID_MASKED);
    value = of_oxm_bsn_lag_id_masked_OF_VERSION_1_3_check(
        bsn_lag_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ip_dscp_masked */
    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ip_dscp_masked->length == 6);
    if (ip_dscp_masked->wire_length_get != NULL) {
        int length;

        ip_dscp_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 6);
    }

    TEST_ASSERT(ip_dscp_masked->object_id == OF_OXM_IP_DSCP_MASKED);
    value = of_oxm_ip_dscp_masked_OF_VERSION_1_3_check(
        ip_dscp_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv6_type */
    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv6_type->length == 5);
    if (icmpv6_type->wire_length_get != NULL) {
        int length;

        icmpv6_type->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv6_type->object_id == OF_OXM_ICMPV6_TYPE);
    value = of_oxm_icmpv6_type_OF_VERSION_1_3_check(
        icmpv6_type, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_interface_class_id_masked */
    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_interface_class_id_masked->length == 12);
    if (bsn_l3_interface_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_interface_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_interface_class_id_masked->object_id == OF_OXM_BSN_L3_INTERFACE_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_interface_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_interface_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_l3_in_port */
    TEST_ASSERT(ofdpa_l3_in_port->length == 14);
    if (ofdpa_l3_in_port->wire_length_get != NULL) {
        int length;

        ofdpa_l3_in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_l3_in_port->object_id == OF_OXM_OFDPA_L3_IN_PORT);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_check(
        ofdpa_l3_in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_l3_in_port->length == 14);
    if (ofdpa_l3_in_port->wire_length_get != NULL) {
        int length;

        ofdpa_l3_in_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 14);
    }

    TEST_ASSERT(ofdpa_l3_in_port->object_id == OF_OXM_OFDPA_L3_IN_PORT);
    value = of_oxm_ofdpa_l3_in_port_OF_VERSION_1_3_check(
        ofdpa_l3_in_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_oam_y1731_opcode */
    TEST_ASSERT(ofdpa_oam_y1731_opcode->length == 11);
    if (ofdpa_oam_y1731_opcode->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_opcode->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_opcode->object_id == OF_OXM_OFDPA_OAM_Y1731_OPCODE);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_opcode, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_oam_y1731_opcode->length == 11);
    if (ofdpa_oam_y1731_opcode->wire_length_get != NULL) {
        int length;

        ofdpa_oam_y1731_opcode->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_oam_y1731_opcode->object_id == OF_OXM_OFDPA_OAM_Y1731_OPCODE);
    value = of_oxm_ofdpa_oam_y1731_opcode_OF_VERSION_1_3_check(
        ofdpa_oam_y1731_opcode, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_tc */
    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_tc->length == 5);
    if (mpls_tc->wire_length_get != NULL) {
        int length;

        mpls_tc->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(mpls_tc->object_id == OF_OXM_MPLS_TC);
    value = of_oxm_mpls_tc_OF_VERSION_1_3_check(
        mpls_tc, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf3 */
    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf3->length == 8);
    if (bsn_udf3->wire_length_get != NULL) {
        int length;

        bsn_udf3->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf3->object_id == OF_OXM_BSN_UDF3);
    value = of_oxm_bsn_udf3_OF_VERSION_1_3_check(
        bsn_udf3, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id_masked */
    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id_masked->length == 12);
    if (bsn_l3_dst_class_id_masked->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_l3_dst_class_id_masked->object_id == OF_OXM_BSN_L3_DST_CLASS_ID_MASKED);
    value = of_oxm_bsn_l3_dst_class_id_masked_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_eth_subtype */
    TEST_ASSERT(ofdpa_eth_subtype->length == 11);
    if (ofdpa_eth_subtype->wire_length_get != NULL) {
        int length;

        ofdpa_eth_subtype->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_eth_subtype->object_id == OF_OXM_OFDPA_ETH_SUBTYPE);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_check(
        ofdpa_eth_subtype, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_eth_subtype->length == 11);
    if (ofdpa_eth_subtype->wire_length_get != NULL) {
        int length;

        ofdpa_eth_subtype->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 11);
    }

    TEST_ASSERT(ofdpa_eth_subtype->object_id == OF_OXM_OFDPA_ETH_SUBTYPE);
    value = of_oxm_ofdpa_eth_subtype_OF_VERSION_1_3_check(
        ofdpa_eth_subtype, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type arp_tpa_masked */
    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(arp_tpa_masked->length == 12);
    if (arp_tpa_masked->wire_length_get != NULL) {
        int length;

        arp_tpa_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(arp_tpa_masked->object_id == OF_OXM_ARP_TPA_MASKED);
    value = of_oxm_arp_tpa_masked_OF_VERSION_1_3_check(
        arp_tpa_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type sctp_dst_masked */
    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(sctp_dst_masked->length == 8);
    if (sctp_dst_masked->wire_length_get != NULL) {
        int length;

        sctp_dst_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(sctp_dst_masked->object_id == OF_OXM_SCTP_DST_MASKED);
    value = of_oxm_sctp_dst_masked_OF_VERSION_1_3_check(
        sctp_dst_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf6 */
    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf6->length == 8);
    if (bsn_udf6->wire_length_get != NULL) {
        int length;

        bsn_udf6->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_udf6->object_id == OF_OXM_BSN_UDF6);
    value = of_oxm_bsn_udf6_OF_VERSION_1_3_check(
        bsn_udf6, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type icmpv4_code */
    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(icmpv4_code->length == 5);
    if (icmpv4_code->wire_length_get != NULL) {
        int length;

        icmpv4_code->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(icmpv4_code->object_id == OF_OXM_ICMPV4_CODE);
    value = of_oxm_icmpv4_code_OF_VERSION_1_3_check(
        icmpv4_code, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_vrf */
    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_vrf->length == 8);
    if (bsn_vrf->wire_length_get != NULL) {
        int length;

        bsn_vrf->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_vrf->object_id == OF_OXM_BSN_VRF);
    value = of_oxm_bsn_vrf_OF_VERSION_1_3_check(
        bsn_vrf, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf1_masked */
    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf1_masked->length == 12);
    if (bsn_udf1_masked->wire_length_get != NULL) {
        int length;

        bsn_udf1_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf1_masked->object_id == OF_OXM_BSN_UDF1_MASKED);
    value = of_oxm_bsn_udf1_masked_OF_VERSION_1_3_check(
        bsn_udf1_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_udf4_masked */
    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_udf4_masked->length == 12);
    if (bsn_udf4_masked->wire_length_get != NULL) {
        int length;

        bsn_udf4_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(bsn_udf4_masked->object_id == OF_OXM_BSN_UDF4_MASKED);
    value = of_oxm_bsn_udf4_masked_OF_VERSION_1_3_check(
        bsn_udf4_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ipv6_src_masked */
    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ipv6_src_masked->length == 36);
    if (ipv6_src_masked->wire_length_get != NULL) {
        int length;

        ipv6_src_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 36);
    }

    TEST_ASSERT(ipv6_src_masked->object_id == OF_OXM_IPV6_SRC_MASKED);
    value = of_oxm_ipv6_src_masked_OF_VERSION_1_3_check(
        ipv6_src_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_global_vrf_allowed */
    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_global_vrf_allowed->length == 5);
    if (bsn_global_vrf_allowed->wire_length_get != NULL) {
        int length;

        bsn_global_vrf_allowed->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 5);
    }

    TEST_ASSERT(bsn_global_vrf_allowed->object_id == OF_OXM_BSN_GLOBAL_VRF_ALLOWED);
    value = of_oxm_bsn_global_vrf_allowed_OF_VERSION_1_3_check(
        bsn_global_vrf_allowed, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type in_phy_port */
    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(in_phy_port->length == 8);
    if (in_phy_port->wire_length_get != NULL) {
        int length;

        in_phy_port->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(in_phy_port->object_id == OF_OXM_IN_PHY_PORT);
    value = of_oxm_in_phy_port_OF_VERSION_1_3_check(
        in_phy_port, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type ofdpa_mpls_ach_channel */
    TEST_ASSERT(ofdpa_mpls_ach_channel->length == 12);
    if (ofdpa_mpls_ach_channel->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ach_channel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_mpls_ach_channel->object_id == OF_OXM_OFDPA_MPLS_ACH_CHANNEL);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_check(
        ofdpa_mpls_ach_channel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(ofdpa_mpls_ach_channel->length == 12);
    if (ofdpa_mpls_ach_channel->wire_length_get != NULL) {
        int length;

        ofdpa_mpls_ach_channel->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(ofdpa_mpls_ach_channel->object_id == OF_OXM_OFDPA_MPLS_ACH_CHANNEL);
    value = of_oxm_ofdpa_mpls_ach_channel_OF_VERSION_1_3_check(
        ofdpa_mpls_ach_channel, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type mpls_label_masked */
    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(mpls_label_masked->length == 12);
    if (mpls_label_masked->wire_length_get != NULL) {
        int length;

        mpls_label_masked->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 12);
    }

    TEST_ASSERT(mpls_label_masked->object_id == OF_OXM_MPLS_LABEL_MASKED);
    value = of_oxm_mpls_label_masked_OF_VERSION_1_3_check(
        mpls_label_masked, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    /* Check two instances of type bsn_l3_dst_class_id */
    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_oxm_next(list, &elt));

    TEST_ASSERT(bsn_l3_dst_class_id->length == 8);
    if (bsn_l3_dst_class_id->wire_length_get != NULL) {
        int length;

        bsn_l3_dst_class_id->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 8);
    }

    TEST_ASSERT(bsn_l3_dst_class_id->object_id == OF_OXM_BSN_L3_DST_CLASS_ID);
    value = of_oxm_bsn_l3_dst_class_id_OF_VERSION_1_3_check(
        bsn_l3_dst_class_id, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_oxm_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_OXM_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 266);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_oxm_t *dup;

        TEST_ASSERT((dup = of_list_oxm_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_oxm_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_packet_queue with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_3_populate(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int cur_len = 0;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_packet_queue */

    of_packet_queue_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_packet_queue_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_packet_queue_append_bind(list,
            (of_packet_queue_t *)elt_p);
    value = of_packet_queue_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_packet_queue generated by
 * of_list_packet_queue_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_packet_queue_OF_VERSION_1_3_check(
    of_list_packet_queue_t *list, int value)
{

    of_packet_queue_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_packet_queue */
    of_packet_queue_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_packet_queue_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_packet_queue_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PACKET_QUEUE);
    value = of_packet_queue_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_packet_queue_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PACKET_QUEUE_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_packet_queue_t *dup;

        TEST_ASSERT((dup = of_list_packet_queue_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_packet_queue_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_desc with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_3_populate(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_desc */

    of_port_desc_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_desc_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_desc_append_bind(list,
            (of_port_desc_t *)elt_p);
    value = of_port_desc_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_desc generated by
 * of_list_port_desc_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_desc_OF_VERSION_1_3_check(
    of_list_port_desc_t *list, int value)
{

    of_port_desc_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_desc */
    of_port_desc_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_desc_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_desc_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_DESC);
    value = of_port_desc_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_desc_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_DESC_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_desc_t *dup;

        TEST_ASSERT((dup = of_list_port_desc_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_desc_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_port_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_3_populate(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_port_stats_entry */

    of_port_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_port_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_port_stats_entry_append_bind(list,
            (of_port_stats_entry_t *)elt_p);
    value = of_port_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_port_stats_entry generated by
 * of_list_port_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_port_stats_entry_OF_VERSION_1_3_check(
    of_list_port_stats_entry_t *list, int value)
{

    of_port_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_port_stats_entry */
    of_port_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_port_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_port_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_PORT_STATS_ENTRY);
    value = of_port_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_port_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_PORT_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_port_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_port_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_port_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_prop with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_3_populate(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;

    /* Append two instances of type of_queue_prop_min_rate */

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_min_rate_init(min_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)min_rate);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_populate(
        min_rate, value);
    cur_len += min_rate->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_queue_prop_max_rate */

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_prop_max_rate_init(max_rate, OF_VERSION_1_3, -1, 1);
    of_list_queue_prop_append_bind(list,
            (of_queue_prop_t *)max_rate);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_populate(
        max_rate, value);
    cur_len += max_rate->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_prop generated by
 * of_list_queue_prop_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_prop_OF_VERSION_1_3_check(
    of_list_queue_prop_t *list, int value)
{

    of_queue_prop_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_queue_prop_min_rate_t *min_rate;
    of_queue_prop_max_rate_t *max_rate;

    /* Instantiate pointers for each subclass */
    min_rate = &elt.min_rate;
    max_rate = &elt.max_rate;
    TEST_OK(of_list_queue_prop_first(list, &elt));

    /* Check two instances of type min_rate */
    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(min_rate->length == 16);
    if (min_rate->wire_length_get != NULL) {
        int length;

        min_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(min_rate->object_id == OF_QUEUE_PROP_MIN_RATE);
    value = of_queue_prop_min_rate_OF_VERSION_1_3_check(
        min_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    /* Check two instances of type max_rate */
    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_prop_next(list, &elt));

    TEST_ASSERT(max_rate->length == 16);
    if (max_rate->wire_length_get != NULL) {
        int length;

        max_rate->wire_length_get(
            (of_object_t *)&elt, &length);
        TEST_ASSERT(length == 16);
    }

    TEST_ASSERT(max_rate->object_id == OF_QUEUE_PROP_MAX_RATE);
    value = of_queue_prop_max_rate_OF_VERSION_1_3_check(
        max_rate, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_prop_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_PROP_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 4);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_prop_t *dup;

        TEST_ASSERT((dup = of_list_queue_prop_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_prop_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_queue_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_3_populate(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_queue_stats_entry */

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_queue_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_queue_stats_entry_append_bind(list,
            (of_queue_stats_entry_t *)elt_p);
    value = of_queue_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_queue_stats_entry generated by
 * of_list_queue_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_queue_stats_entry_OF_VERSION_1_3_check(
    of_list_queue_stats_entry_t *list, int value)
{

    of_queue_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_queue_stats_entry */
    of_queue_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_queue_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_queue_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_QUEUE_STATS_ENTRY);
    value = of_queue_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_queue_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_QUEUE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_queue_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_queue_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_queue_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_table_feature_prop with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_table_feature_prop_OF_VERSION_1_3_populate(
    of_list_table_feature_prop_t *list, int value)
{

    of_table_feature_prop_t elt;
    int cur_len = 0;
    /* Declare pointers for each subclass */
    of_table_feature_prop_next_tables_miss_t *next_tables_miss;
    of_table_feature_prop_wildcards_t *wildcards;
    of_table_feature_prop_experimenter_miss_t *experimenter_miss;
    of_table_feature_prop_write_actions_t *write_actions;
    of_table_feature_prop_write_actions_miss_t *write_actions_miss;
    of_table_feature_prop_next_tables_t *next_tables;
    of_table_feature_prop_apply_actions_miss_t *apply_actions_miss;
    of_table_feature_prop_apply_actions_t *apply_actions;
    of_table_feature_prop_apply_setfield_miss_t *apply_setfield_miss;
    of_table_feature_prop_experimenter_t *experimenter;
    of_table_feature_prop_write_setfield_miss_t *write_setfield_miss;
    of_table_feature_prop_write_setfield_t *write_setfield;
    of_table_feature_prop_instructions_miss_t *instructions_miss;
    of_table_feature_prop_apply_setfield_t *apply_setfield;
    of_table_feature_prop_match_t *match;
    of_table_feature_prop_instructions_t *instructions;

    /* Instantiate pointers for each subclass */
    next_tables_miss = &elt.next_tables_miss;
    wildcards = &elt.wildcards;
    experimenter_miss = &elt.experimenter_miss;
    write_actions = &elt.write_actions;
    write_actions_miss = &elt.write_actions_miss;
    next_tables = &elt.next_tables;
    apply_actions_miss = &elt.apply_actions_miss;
    apply_actions = &elt.apply_actions;
    apply_setfield_miss = &elt.apply_setfield_miss;
    experimenter = &elt.experimenter;
    write_setfield_miss = &elt.write_setfield_miss;
    write_setfield = &elt.write_setfield;
    instructions_miss = &elt.instructions_miss;
    apply_setfield = &elt.apply_setfield;
    match = &elt.match;
    instructions = &elt.instructions;

    /* Append two instances of type of_table_feature_prop_next_tables_miss */

    of_table_feature_prop_next_tables_miss_init(next_tables_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables_miss);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_populate(
        next_tables_miss, value);
    cur_len += next_tables_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_next_tables_miss_init(next_tables_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables_miss);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_populate(
        next_tables_miss, value);
    cur_len += next_tables_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_wildcards */

    of_table_feature_prop_wildcards_init(wildcards, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)wildcards);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_populate(
        wildcards, value);
    cur_len += wildcards->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_wildcards_init(wildcards, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)wildcards);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_populate(
        wildcards, value);
    cur_len += wildcards->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_experimenter_miss */

    of_table_feature_prop_experimenter_miss_init(experimenter_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter_miss);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_populate(
        experimenter_miss, value);
    cur_len += experimenter_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_experimenter_miss_init(experimenter_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter_miss);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_populate(
        experimenter_miss, value);
    cur_len += experimenter_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_actions */

    of_table_feature_prop_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_actions_init(write_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_populate(
        write_actions, value);
    cur_len += write_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_actions_miss */

    of_table_feature_prop_write_actions_miss_init(write_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions_miss);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_populate(
        write_actions_miss, value);
    cur_len += write_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_actions_miss_init(write_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_actions_miss);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_populate(
        write_actions_miss, value);
    cur_len += write_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_next_tables */

    of_table_feature_prop_next_tables_init(next_tables, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_populate(
        next_tables, value);
    cur_len += next_tables->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_next_tables_init(next_tables, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)next_tables);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_populate(
        next_tables, value);
    cur_len += next_tables->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_actions_miss */

    of_table_feature_prop_apply_actions_miss_init(apply_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions_miss);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_populate(
        apply_actions_miss, value);
    cur_len += apply_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_actions_miss_init(apply_actions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions_miss);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_populate(
        apply_actions_miss, value);
    cur_len += apply_actions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_actions */

    of_table_feature_prop_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_actions_init(apply_actions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_actions);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_populate(
        apply_actions, value);
    cur_len += apply_actions->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_setfield_miss */

    of_table_feature_prop_apply_setfield_miss_init(apply_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield_miss);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_populate(
        apply_setfield_miss, value);
    cur_len += apply_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_setfield_miss_init(apply_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield_miss);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_populate(
        apply_setfield_miss, value);
    cur_len += apply_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_experimenter */

    of_table_feature_prop_experimenter_init(experimenter, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_populate(
        experimenter, value);
    cur_len += experimenter->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_experimenter_init(experimenter, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)experimenter);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_populate(
        experimenter, value);
    cur_len += experimenter->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_setfield_miss */

    of_table_feature_prop_write_setfield_miss_init(write_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield_miss);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_populate(
        write_setfield_miss, value);
    cur_len += write_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_setfield_miss_init(write_setfield_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield_miss);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_populate(
        write_setfield_miss, value);
    cur_len += write_setfield_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_write_setfield */

    of_table_feature_prop_write_setfield_init(write_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_populate(
        write_setfield, value);
    cur_len += write_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_write_setfield_init(write_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)write_setfield);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_populate(
        write_setfield, value);
    cur_len += write_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_instructions_miss */

    of_table_feature_prop_instructions_miss_init(instructions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions_miss);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_populate(
        instructions_miss, value);
    cur_len += instructions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_instructions_miss_init(instructions_miss, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions_miss);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_populate(
        instructions_miss, value);
    cur_len += instructions_miss->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_apply_setfield */

    of_table_feature_prop_apply_setfield_init(apply_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_populate(
        apply_setfield, value);
    cur_len += apply_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_apply_setfield_init(apply_setfield, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)apply_setfield);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_populate(
        apply_setfield, value);
    cur_len += apply_setfield->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_match */

    of_table_feature_prop_match_init(match, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)match);
    value = of_table_feature_prop_match_OF_VERSION_1_3_populate(
        match, value);
    cur_len += match->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_match_init(match, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)match);
    value = of_table_feature_prop_match_OF_VERSION_1_3_populate(
        match, value);
    cur_len += match->length;
    TEST_ASSERT(list->length == cur_len);

    /* Append two instances of type of_table_feature_prop_instructions */

    of_table_feature_prop_instructions_init(instructions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_populate(
        instructions, value);
    cur_len += instructions->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_feature_prop_instructions_init(instructions, OF_VERSION_1_3, -1, 1);
    of_list_table_feature_prop_append_bind(list,
            (of_table_feature_prop_t *)instructions);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_populate(
        instructions, value);
    cur_len += instructions->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_table_feature_prop generated by
 * of_list_table_feature_prop_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_table_feature_prop_OF_VERSION_1_3_check(
    of_list_table_feature_prop_t *list, int value)
{

    of_table_feature_prop_t elt;
    int count = 0;
    int rv;
    /* Declare pointers for each subclass */
    of_table_feature_prop_next_tables_miss_t *next_tables_miss;
    of_table_feature_prop_wildcards_t *wildcards;
    of_table_feature_prop_experimenter_miss_t *experimenter_miss;
    of_table_feature_prop_write_actions_t *write_actions;
    of_table_feature_prop_write_actions_miss_t *write_actions_miss;
    of_table_feature_prop_next_tables_t *next_tables;
    of_table_feature_prop_apply_actions_miss_t *apply_actions_miss;
    of_table_feature_prop_apply_actions_t *apply_actions;
    of_table_feature_prop_apply_setfield_miss_t *apply_setfield_miss;
    of_table_feature_prop_experimenter_t *experimenter;
    of_table_feature_prop_write_setfield_miss_t *write_setfield_miss;
    of_table_feature_prop_write_setfield_t *write_setfield;
    of_table_feature_prop_instructions_miss_t *instructions_miss;
    of_table_feature_prop_apply_setfield_t *apply_setfield;
    of_table_feature_prop_match_t *match;
    of_table_feature_prop_instructions_t *instructions;

    /* Instantiate pointers for each subclass */
    next_tables_miss = &elt.next_tables_miss;
    wildcards = &elt.wildcards;
    experimenter_miss = &elt.experimenter_miss;
    write_actions = &elt.write_actions;
    write_actions_miss = &elt.write_actions_miss;
    next_tables = &elt.next_tables;
    apply_actions_miss = &elt.apply_actions_miss;
    apply_actions = &elt.apply_actions;
    apply_setfield_miss = &elt.apply_setfield_miss;
    experimenter = &elt.experimenter;
    write_setfield_miss = &elt.write_setfield_miss;
    write_setfield = &elt.write_setfield;
    instructions_miss = &elt.instructions_miss;
    apply_setfield = &elt.apply_setfield;
    match = &elt.match;
    instructions = &elt.instructions;
    TEST_OK(of_list_table_feature_prop_first(list, &elt));

    /* Check two instances of type next_tables_miss */
    TEST_ASSERT(next_tables_miss->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES_MISS);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_check(
        next_tables_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(next_tables_miss->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES_MISS);
    value = of_table_feature_prop_next_tables_miss_OF_VERSION_1_3_check(
        next_tables_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type wildcards */
    TEST_ASSERT(wildcards->object_id == OF_TABLE_FEATURE_PROP_WILDCARDS);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_check(
        wildcards, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(wildcards->object_id == OF_TABLE_FEATURE_PROP_WILDCARDS);
    value = of_table_feature_prop_wildcards_OF_VERSION_1_3_check(
        wildcards, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type experimenter_miss */
    TEST_ASSERT(experimenter_miss->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER_MISS);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_check(
        experimenter_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(experimenter_miss->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER_MISS);
    value = of_table_feature_prop_experimenter_miss_OF_VERSION_1_3_check(
        experimenter_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_actions */
    TEST_ASSERT(write_actions->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_actions->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS);
    value = of_table_feature_prop_write_actions_OF_VERSION_1_3_check(
        write_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_actions_miss */
    TEST_ASSERT(write_actions_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS_MISS);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_check(
        write_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_actions_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_ACTIONS_MISS);
    value = of_table_feature_prop_write_actions_miss_OF_VERSION_1_3_check(
        write_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type next_tables */
    TEST_ASSERT(next_tables->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_check(
        next_tables, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(next_tables->object_id == OF_TABLE_FEATURE_PROP_NEXT_TABLES);
    value = of_table_feature_prop_next_tables_OF_VERSION_1_3_check(
        next_tables, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_actions_miss */
    TEST_ASSERT(apply_actions_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS_MISS);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_check(
        apply_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_actions_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS_MISS);
    value = of_table_feature_prop_apply_actions_miss_OF_VERSION_1_3_check(
        apply_actions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_actions */
    TEST_ASSERT(apply_actions->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_actions->object_id == OF_TABLE_FEATURE_PROP_APPLY_ACTIONS);
    value = of_table_feature_prop_apply_actions_OF_VERSION_1_3_check(
        apply_actions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_setfield_miss */
    TEST_ASSERT(apply_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD_MISS);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_check(
        apply_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD_MISS);
    value = of_table_feature_prop_apply_setfield_miss_OF_VERSION_1_3_check(
        apply_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type experimenter */
    TEST_ASSERT(experimenter->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_check(
        experimenter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(experimenter->object_id == OF_TABLE_FEATURE_PROP_EXPERIMENTER);
    value = of_table_feature_prop_experimenter_OF_VERSION_1_3_check(
        experimenter, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_setfield_miss */
    TEST_ASSERT(write_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD_MISS);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_check(
        write_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_setfield_miss->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD_MISS);
    value = of_table_feature_prop_write_setfield_miss_OF_VERSION_1_3_check(
        write_setfield_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type write_setfield */
    TEST_ASSERT(write_setfield->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_check(
        write_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(write_setfield->object_id == OF_TABLE_FEATURE_PROP_WRITE_SETFIELD);
    value = of_table_feature_prop_write_setfield_OF_VERSION_1_3_check(
        write_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type instructions_miss */
    TEST_ASSERT(instructions_miss->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS_MISS);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_check(
        instructions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(instructions_miss->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS_MISS);
    value = of_table_feature_prop_instructions_miss_OF_VERSION_1_3_check(
        instructions_miss, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type apply_setfield */
    TEST_ASSERT(apply_setfield->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_check(
        apply_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(apply_setfield->object_id == OF_TABLE_FEATURE_PROP_APPLY_SETFIELD);
    value = of_table_feature_prop_apply_setfield_OF_VERSION_1_3_check(
        apply_setfield, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type match */
    TEST_ASSERT(match->object_id == OF_TABLE_FEATURE_PROP_MATCH);
    value = of_table_feature_prop_match_OF_VERSION_1_3_check(
        match, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(match->object_id == OF_TABLE_FEATURE_PROP_MATCH);
    value = of_table_feature_prop_match_OF_VERSION_1_3_check(
        match, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    /* Check two instances of type instructions */
    TEST_ASSERT(instructions->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_check(
        instructions, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_feature_prop_next(list, &elt));

    TEST_ASSERT(instructions->object_id == OF_TABLE_FEATURE_PROP_INSTRUCTIONS);
    value = of_table_feature_prop_instructions_OF_VERSION_1_3_check(
        instructions, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_feature_prop_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_TABLE_FEATURE_PROP_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 32);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_table_feature_prop_t *dup;

        TEST_ASSERT((dup = of_list_table_feature_prop_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_table_feature_prop_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_table_features with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_table_features_OF_VERSION_1_3_populate(
    of_list_table_features_t *list, int value)
{

    of_table_features_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_features */
    of_table_features_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_features */

    of_table_features_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_features_append_bind(list,
            (of_table_features_t *)elt_p);
    value = of_table_features_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_features_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_features_append_bind(list,
            (of_table_features_t *)elt_p);
    value = of_table_features_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_table_features generated by
 * of_list_table_features_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_table_features_OF_VERSION_1_3_check(
    of_list_table_features_t *list, int value)
{

    of_table_features_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_table_features */
    of_table_features_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_features_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_FEATURES);
    value = of_table_features_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_features_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_FEATURES);
    value = of_table_features_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_features_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_TABLE_FEATURES_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_table_features_t *dup;

        TEST_ASSERT((dup = of_list_table_features_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_table_features_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_table_stats_entry with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_3_populate(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int cur_len = 0;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_table_stats_entry */

    of_table_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_table_stats_entry_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_table_stats_entry_append_bind(list,
            (of_table_stats_entry_t *)elt_p);
    value = of_table_stats_entry_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_table_stats_entry generated by
 * of_list_table_stats_entry_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_table_stats_entry_OF_VERSION_1_3_check(
    of_list_table_stats_entry_t *list, int value)
{

    of_table_stats_entry_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_table_stats_entry */
    of_table_stats_entry_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_table_stats_entry_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_table_stats_entry_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_TABLE_STATS_ENTRY);
    value = of_table_stats_entry_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_table_stats_entry_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_TABLE_STATS_ENTRY_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_table_stats_entry_t *dup;

        TEST_ASSERT((dup = of_list_table_stats_entry_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_table_stats_entry_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_uint32 with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_uint32_OF_VERSION_1_3_populate(
    of_list_uint32_t *list, int value)
{

    of_uint32_t elt;
    int cur_len = 0;
    /* No subclasses for of_uint32 */
    of_uint32_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_uint32 */

    of_uint32_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint32_append_bind(list,
            (of_uint32_t *)elt_p);
    value = of_uint32_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_uint32_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint32_append_bind(list,
            (of_uint32_t *)elt_p);
    value = of_uint32_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_uint32 generated by
 * of_list_uint32_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_uint32_OF_VERSION_1_3_check(
    of_list_uint32_t *list, int value)
{

    of_uint32_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_uint32 */
    of_uint32_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_uint32_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_UINT32);
    value = of_uint32_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_uint32_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_UINT32);
    value = of_uint32_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_uint32_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_UINT32_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_uint32_t *dup;

        TEST_ASSERT((dup = of_list_uint32_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_uint32_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_uint64 with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_uint64_OF_VERSION_1_3_populate(
    of_list_uint64_t *list, int value)
{

    of_uint64_t elt;
    int cur_len = 0;
    /* No subclasses for of_uint64 */
    of_uint64_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_uint64 */

    of_uint64_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint64_append_bind(list,
            (of_uint64_t *)elt_p);
    value = of_uint64_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_uint64_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint64_append_bind(list,
            (of_uint64_t *)elt_p);
    value = of_uint64_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_uint64 generated by
 * of_list_uint64_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_uint64_OF_VERSION_1_3_check(
    of_list_uint64_t *list, int value)
{

    of_uint64_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_uint64 */
    of_uint64_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_uint64_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_UINT64);
    value = of_uint64_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_uint64_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_UINT64);
    value = of_uint64_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_uint64_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_UINT64_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_uint64_t *dup;

        TEST_ASSERT((dup = of_list_uint64_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_uint64_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Populate a list of type of_list_uint8 with two of each type of subclass
 * @param list Pointer to the list to be populated
 * @param value The seed value to use in populating the list
 * @returns The value after increments for this object's values
 */
int
of_list_uint8_OF_VERSION_1_3_populate(
    of_list_uint8_t *list, int value)
{

    of_uint8_t elt;
    int cur_len = 0;
    /* No subclasses for of_uint8 */
    of_uint8_t *elt_p;

    elt_p = &elt;

    /* Append two instances of type of_uint8 */

    of_uint8_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint8_append_bind(list,
            (of_uint8_t *)elt_p);
    value = of_uint8_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    of_uint8_init(elt_p, OF_VERSION_1_3, -1, 1);
    of_list_uint8_append_bind(list,
            (of_uint8_t *)elt_p);
    value = of_uint8_OF_VERSION_1_3_populate(
        elt_p, value);
    cur_len += elt_p->length;
    TEST_ASSERT(list->length == cur_len);

    return value;
}

/**
 * Check a list of type of_list_uint8 generated by
 * of_list_uint8_OF_VERSION_1_3_populate
 * @param list Pointer to the list that was populated
 * @param value Starting value for checking
 * @returns The value after increments for this object's values
 */
int
of_list_uint8_OF_VERSION_1_3_check(
    of_list_uint8_t *list, int value)
{

    of_uint8_t elt;
    int count = 0;
    int rv;
    /* No subclasses for of_uint8 */
    of_uint8_t *elt_p;

    elt_p = &elt;
    TEST_OK(of_list_uint8_first(list, &elt));

    /* Check two instances of type elt_p */
    TEST_ASSERT(elt_p->object_id == OF_UINT8);
    value = of_uint8_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_OK(of_list_uint8_next(list, &elt));

    TEST_ASSERT(elt_p->object_id == OF_UINT8);
    value = of_uint8_OF_VERSION_1_3_check(
        elt_p, value);
    TEST_ASSERT(value != 0);
    TEST_ASSERT(of_list_uint8_next(list, &elt) == OF_ERROR_RANGE);


    /* Do an iterate to test the iterator */
    OF_LIST_UINT8_ITER(list, &elt, rv) {
        count += 1;
    }

    TEST_ASSERT(rv == OF_ERROR_RANGE);
    TEST_ASSERT(count == 2);

    /* We shoehorn a test of the dup functions here */
    {
        of_list_uint8_t *dup;

        TEST_ASSERT((dup = of_list_uint8_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);

        /* And now for the generic dup function */
        TEST_ASSERT((dup = (of_list_uint8_t *)
            of_object_dup(list)) != NULL);
        TEST_ASSERT(dup->length == list->length);
        TEST_ASSERT(dup->object_id == list->object_id);
        TEST_ASSERT(dup->version == list->version);
        TEST_ASSERT(MEMCMP(OF_OBJECT_BUFFER_INDEX(dup, 0),
            OF_OBJECT_BUFFER_INDEX(list, 0), list->length) == 0);
        of_object_delete((of_object_t *)dup);
    }

    return value;
}

/**
 * Test cases for all flag accessor macros
 * These only test self consistency (and that they compile)
 */
int
test_ident_macros(void)
{
    int value __attribute__((unused));
    uint32_t flags;

    value = OF_GROUP_MOD_FAILED_WATCH_UNSUPPORTED;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_WATCH_UNSUPPORTED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_WATCH_UNSUPPORTED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_WATCH_UNSUPPORTED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_WATCH_UNSUPPORTED_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_100MB_FD;
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_FD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_FD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_FD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_FD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_100MB_FD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_100MB_FD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_FD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_FEATURE_FLAG_100MB_FD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_100MB_FD_TEST(flags, OF_VERSION_1_0));
    value = OF_QUEUE_MAX_RATE_UNCFG;
    TEST_ASSERT(!OF_QUEUE_MAX_RATE_UNCFG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_QUEUE_MAX_RATE_UNCFG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_MAX_RATE_UNCFG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_MAX_RATE_UNCFG_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_BAD_TABLE_ID;
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_DEST_NONE;
    TEST_ASSERT(OF_PORT_DEST_NONE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_DEST_NONE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_DEST_NONE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_DEST_NONE_SUPPORTED(OF_VERSION_1_3));
    value = OF_BSN_VPORT_STATUS_OK;
    TEST_ASSERT(OF_BSN_VPORT_STATUS_OK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_BSN_VPORT_STATUS_OK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_BSN_VPORT_STATUS_OK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_BSN_VPORT_STATUS_OK_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_GET_ASYNC_REPLY;
    TEST_ASSERT(!OF_OBJ_TYPE_GET_ASYNC_REPLY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OBJ_TYPE_GET_ASYNC_REPLY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_OBJ_TYPE_GET_ASYNC_REPLY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_GET_ASYNC_REPLY_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_DL_SRC;
    TEST_ASSERT(OF_FLOW_WC_V1_DL_SRC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_SRC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_SRC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_SRC_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_SET_ARGUMENT;
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_SET_ARGUMENT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_SET_ARGUMENT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_SET_ARGUMENT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_SET_ARGUMENT_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_STP_BLOCK;
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_BLOCK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_BLOCK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_BLOCK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_BLOCK_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_STATE_FLAG_STP_BLOCK_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_STATE_FLAG_STP_BLOCK_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_BLOCK_TEST(flags, OF_VERSION_1_0));
    OF_PORT_STATE_FLAG_STP_BLOCK_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_BLOCK_TEST(flags, OF_VERSION_1_0));
    value = LACP_OUT_OF_SYNC;
    TEST_ASSERT(!LACP_OUT_OF_SYNC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!LACP_OUT_OF_SYNC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!LACP_OUT_OF_SYNC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(LACP_OUT_OF_SYNC_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_TP_DST;
    TEST_ASSERT(OF_FLOW_WC_V1_TP_DST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_TP_DST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_TP_DST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_TP_DST_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_TABLE_FEATURES;
    TEST_ASSERT(!OF_STATS_TYPE_TABLE_FEATURES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_STATS_TYPE_TABLE_FEATURES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_STATS_TYPE_TABLE_FEATURES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_TABLE_FEATURES_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_LEN;
    TEST_ASSERT(OF_ACTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_OP_FAILED_BAD_QUEUE;
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_RX_DROPPED;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_DROPPED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_DROPPED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_DROPPED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_RX_DROPPED_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_BAD_TIMEOUT;
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_BAD_TIMEOUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_TIMEOUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_TIMEOUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_TIMEOUT_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_REMOVED_REASON_HARD_TIMEOUT;
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_HARD_TIMEOUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_HARD_TIMEOUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_HARD_TIMEOUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_HARD_TIMEOUT_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_10MB_HD;
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_HD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_HD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_HD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_HD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_10MB_HD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_10MB_HD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_HD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_FEATURE_FLAG_10MB_HD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_10MB_HD_TEST(flags, OF_VERSION_1_0));
    value = OF_ACTION_FAILED_UNSUPPORTED_ORDER;
    TEST_ASSERT(!OF_ACTION_FAILED_UNSUPPORTED_ORDER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_UNSUPPORTED_ORDER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_UNSUPPORTED_ORDER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_UNSUPPORTED_ORDER_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_ALL;
    TEST_ASSERT(!OF_TABLE_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_COMMAND_DELETE;
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_NO_PACKET_IN;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_PACKET_IN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_PACKET_IN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_PACKET_IN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_PACKET_IN_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_NO_PACKET_IN_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_NO_PACKET_IN_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_PACKET_IN_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_NO_PACKET_IN_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_PACKET_IN_TEST(flags, OF_VERSION_1_0));
    value = OF_ERROR_TYPE_TABLE_FEATURES_FAILED;
    TEST_ASSERT(!OF_ERROR_TYPE_TABLE_FEATURES_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ERROR_TYPE_TABLE_FEATURES_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ERROR_TYPE_TABLE_FEATURES_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_TABLE_FEATURES_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OF_SWITCH_CONFIG_FAILED_BAD_LEN;
    TEST_ASSERT(!OF_SWITCH_CONFIG_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_PUSH_MPLS;
    TEST_ASSERT(!OF_ACTION_TYPE_PUSH_MPLS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_PUSH_MPLS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_PUSH_MPLS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_PUSH_MPLS_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_TX_BYTES;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_UNSUP_METADATA_MASK;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_UNSUP_METADATA_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_METADATA_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_METADATA_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_METADATA_MASK_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_MOD_FAILED_BAD_PORT;
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_NONE;
    TEST_ASSERT(!OFDPA_SNM_NONE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_NONE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_NONE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_NONE_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_AUTH;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_AUTH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_AUTH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_AUTH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_AUTH_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_AUTH_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_AUTH_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_AUTH_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_AUTH_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_AUTH_TEST(flags, OF_VERSION_1_3));
    value = OF_METER_MAX;
    TEST_ASSERT(!OF_METER_MAX_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MAX_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MAX_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MAX_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_CONTROLLER_ROLE_REASON_EXPERIMENTER;
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_CONTROLLER_ROLE_REASON_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_OXM_CLASS_NXM_0;
    TEST_ASSERT(!OF_OXM_CLASS_NXM_0_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OXM_CLASS_NXM_0_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OXM_CLASS_NXM_0_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OXM_CLASS_NXM_0_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_NO_FWD;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_FWD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_FWD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_FWD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_FWD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_NO_FWD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_NO_FWD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_FWD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_NO_FWD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_FWD_TEST(flags, OF_VERSION_1_0));
    value = OFDPA_PSM_WRR;
    TEST_ASSERT(!OFDPA_PSM_WRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_WRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_WRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_WRR_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_DROP_STATUS_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_DROP_STATUS_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_DROP_STATUS_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_DROP_STATUS_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_DROP_STATUS_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_ANY;
    TEST_ASSERT(!OF_GROUP_ANY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_ANY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_ANY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_ANY_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_CONTROLLER;
    TEST_ASSERT(!OF_METER_CONTROLLER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_CONTROLLER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_CONTROLLER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_CONTROLLER_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_PUSH_CW;
    TEST_ASSERT(!OFDPA_ACT_PUSH_CW_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_PUSH_CW_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_PUSH_CW_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_PUSH_CW_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_SET_DSCP_FROM_TABLE;
    TEST_ASSERT(!OFDPA_ACT_SET_DSCP_FROM_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_SET_DSCP_FROM_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_SET_DSCP_FROM_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_SET_DSCP_FROM_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_TAG;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_INSTRUCTIONS;
    TEST_ASSERT(!OF_TABLE_FEATURE_INSTRUCTIONS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_INSTRUCTIONS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_INSTRUCTIONS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_INSTRUCTIONS_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_TABLE_ID;
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_3));
    value = OF_OXM_CLASS_OPENFLOW_BASIC;
    TEST_ASSERT(!OF_OXM_CLASS_OPENFLOW_BASIC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OXM_CLASS_OPENFLOW_BASIC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OXM_CLASS_OPENFLOW_BASIC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OXM_CLASS_OPENFLOW_BASIC_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_POP_MPLS;
    TEST_ASSERT(!OF_ACTION_TYPE_POP_MPLS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_POP_MPLS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_POP_MPLS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_POP_MPLS_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_EXPERIMENTER_TYPE;
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_GROUP;
    TEST_ASSERT(!OF_STATS_TYPE_GROUP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_FLOW_MOD;
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_MOD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_MOD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_MOD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_MOD_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_QUEUE;
    TEST_ASSERT(!OF_ACTION_TYPE_SET_QUEUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_QUEUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_SET_QUEUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_SET_QUEUE_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_DEST;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_DEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_DEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_DEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_DEST_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_DEST_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_DEST_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_DEST_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_DEST_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_DEST_TEST(flags, OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_NW_DST_ALL;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_DST_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_DEST_CONTROLLER;
    TEST_ASSERT(OF_PORT_DEST_CONTROLLER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_CONTROLLER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_CONTROLLER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_CONTROLLER_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_STAT;
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_STAT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_STAT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_STAT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_STAT_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_PSM_SP_DWRR;
    TEST_ASSERT(!OFDPA_PSM_SP_DWRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_SP_DWRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_SP_DWRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_SP_DWRR_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_UNKNOWN_INST;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_UNKNOWN_INST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNKNOWN_INST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNKNOWN_INST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNKNOWN_INST_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_INVALID_METER;
    TEST_ASSERT(!OF_METER_MOD_FAILED_INVALID_METER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_INVALID_METER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_INVALID_METER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_INVALID_METER_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BUFFER_EMPTY;
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_EMPTY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_EMPTY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_EMPTY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_EMPTY_SUPPORTED(OF_VERSION_1_3));
    value = OF_NICIRA_CONTROLLER_ROLE_MASTER;
    TEST_ASSERT(OF_NICIRA_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_3));
    value = LACP_SUCCESS;
    TEST_ASSERT(!LACP_SUCCESS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!LACP_SUCCESS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!LACP_SUCCESS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(LACP_SUCCESS_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_TOO_MANY;
    TEST_ASSERT(OF_ACTION_FAILED_TOO_MANY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_TOO_MANY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_TOO_MANY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_TOO_MANY_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_SWITCH_CONFIG_FAILED;
    TEST_ASSERT(!OF_ERROR_TYPE_SWITCH_CONFIG_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_SWITCH_CONFIG_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_SWITCH_CONFIG_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_SWITCH_CONFIG_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OF_ROLE_REQUEST_FAILED_STALE;
    TEST_ASSERT(!OF_ROLE_REQUEST_FAILED_STALE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ROLE_REQUEST_FAILED_STALE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ROLE_REQUEST_FAILED_STALE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ROLE_REQUEST_FAILED_STALE_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_40GB_FD;
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_40GB_FD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_40GB_FD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_40GB_FD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_40GB_FD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_40GB_FD_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_40GB_FD_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_40GB_FD_TEST(flags, OF_VERSION_1_1));
    OF_PORT_FEATURE_FLAG_40GB_FD_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_40GB_FD_TEST(flags, OF_VERSION_1_1));
    value = OFP_BSN_PORT_COUNTER_TX_ERRORS;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_ERRORS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_ERRORS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_ERRORS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_TX_ERRORS_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_COMMAND_ADD;
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_3));
    value = OF_MAX_PORT_NAME_LEN;
    TEST_ASSERT(OF_MAX_PORT_NAME_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MAX_PORT_NAME_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MAX_PORT_NAME_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MAX_PORT_NAME_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONFIG_FRAG_NORMAL;
    TEST_ASSERT(OF_CONFIG_FRAG_NORMAL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_FRAG_NORMAL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONFIG_FRAG_NORMAL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONFIG_FRAG_NORMAL_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_NEW_HOST;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_NEW_HOST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_NEW_HOST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_NEW_HOST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_NEW_HOST_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_STP;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_STP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_STP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_STP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_STP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_STP_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_STP_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_STP_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_STP_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_STP_TEST(flags, OF_VERSION_1_0));
    value = OF_ERROR_TYPE_EXPERIMENTER;
    TEST_ASSERT(!OF_ERROR_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ERROR_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_SWITCH_CONFIG_FAILED_BAD_FLAGS;
    TEST_ASSERT(!OF_SWITCH_CONFIG_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_EXPERIMENTER;
    TEST_ASSERT(OF_ACTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_DHCP;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DHCP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DHCP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DHCP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_DHCP_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_TYPE_METER;
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_METER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_METER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_METER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_METER_SUPPORTED(OF_VERSION_1_3));
    value = OFPPMPT_OPTICAL;
    TEST_ASSERT(!OFPPMPT_OPTICAL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFPPMPT_OPTICAL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFPPMPT_OPTICAL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFPPMPT_OPTICAL_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_MOD_FAILED_BAD_TABLE;
    TEST_ASSERT(!OF_TABLE_MOD_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_EXPERIMENTER;
    TEST_ASSERT(!OF_TABLE_FEATURE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_CONTROLLER_ROLE_REASON_CONFIG;
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_CONFIG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_CONFIG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_CONFIG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_CONTROLLER_ROLE_REASON_CONFIG_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_SLOWPATH;
    TEST_ASSERT(!OF_METER_SLOWPATH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_SLOWPATH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_SLOWPATH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_SLOWPATH_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_APPLY_SETFIELD_MISS;
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_APPLY_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_CAPABILITIES_CHAINING_CHECKS;
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_CHAINING_CHECKS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_CHAINING_CHECKS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_CHAINING_CHECKS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_CHAINING_CHECKS_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_ALL_TABLES_FULL;
    TEST_ASSERT(OF_FLOW_MOD_FAILED_ALL_TABLES_FULL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_ALL_TABLES_FULL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_ALL_TABLES_FULL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_ALL_TABLES_FULL_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_TYPE_INDIRECT;
    TEST_ASSERT(!OF_GROUP_TYPE_INDIRECT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_TYPE_INDIRECT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_TYPE_INDIRECT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_TYPE_INDIRECT_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_VERSION;
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_VERSION_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_VERSION_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_VERSION_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_VERSION_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_ARP_MATCH_IP;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_ARP_MATCH_IP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_ARP_MATCH_IP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_ARP_MATCH_IP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_ARP_MATCH_IP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_ARP_MATCH_IP_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_ARP_MATCH_IP_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_ARP_MATCH_IP_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_ARP_MATCH_IP_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_ARP_MATCH_IP_TEST(flags, OF_VERSION_1_0));
    value = OF_PACKET_IN_REASON_INVALID_TTL;
    TEST_ASSERT(!OF_PACKET_IN_REASON_INVALID_TTL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_INVALID_TTL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PACKET_IN_REASON_INVALID_TTL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_INVALID_TTL_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_PUSH_PBB;
    TEST_ASSERT(!OF_ACTION_TYPE_PUSH_PBB_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_TYPE_PUSH_PBB_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_PUSH_PBB_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_PUSH_PBB_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_NO_STP;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_STP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_STP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_STP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_STP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_NO_STP_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_NO_STP_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_STP_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_NO_STP_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_STP_TEST(flags, OF_VERSION_1_0));
    value = OF_METER_FLAG_STATS;
    TEST_ASSERT(!OF_METER_FLAG_STATS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_FLAG_STATS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_FLAG_STATS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_FLAG_STATS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_METER_FLAG_STATS_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_METER_FLAG_STATS_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_METER_FLAG_STATS_TEST(flags, OF_VERSION_1_3));
    OF_METER_FLAG_STATS_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_METER_FLAG_STATS_TEST(flags, OF_VERSION_1_3));
    value = OF_PORT_DEST_FLOOD;
    TEST_ASSERT(OF_PORT_DEST_FLOOD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_FLOOD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_FLOOD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_FLOOD_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_PROPERTY_NONE;
    TEST_ASSERT(OF_QUEUE_PROPERTY_NONE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_QUEUE_PROPERTY_NONE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_QUEUE_PROPERTY_NONE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_QUEUE_PROPERTY_NONE_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_NW_DST_SHIFT;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_DST_SHIFT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_SHIFT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_SHIFT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_SHIFT_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_COMMAND_DELETE;
    TEST_ASSERT(!OF_GROUP_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_METER_FEATURES;
    TEST_ASSERT(!OF_STATS_TYPE_METER_FEATURES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_STATS_TYPE_METER_FEATURES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_STATS_TYPE_METER_FEATURES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_METER_FEATURES_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_NO_RECV;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_RECV_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_RECV_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_RECV_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_RECV_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_NO_RECV_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_NO_RECV_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_RECV_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_NO_RECV_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_RECV_TEST(flags, OF_VERSION_1_0));
    value = OF_BSN_VPORT_Q_IN_Q_UNTAGGED;
    TEST_ASSERT(OF_BSN_VPORT_Q_IN_Q_UNTAGGED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_BSN_VPORT_Q_IN_Q_UNTAGGED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_BSN_VPORT_Q_IN_Q_UNTAGGED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_BSN_VPORT_Q_IN_Q_UNTAGGED_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_PROPERTY_MAX_RATE;
    TEST_ASSERT(!OF_QUEUE_PROPERTY_MAX_RATE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_QUEUE_PROPERTY_MAX_RATE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_PROPERTY_MAX_RATE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_PROPERTY_MAX_RATE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_SP_WRR;
    TEST_ASSERT(!OFDPA_SNM_SP_WRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_SP_WRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_SP_WRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_SP_WRR_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_COMMAND_MODIFY;
    TEST_ASSERT(!OF_METER_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_DL_VLAN;
    TEST_ASSERT(OF_FLOW_WC_V1_DL_VLAN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_DL_VLAN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_VLAN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_VLAN_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_EXPERIMENTER_TYPE;
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE;
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_SET_MPLS_PCPDEI_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_UNKNOWN_METER;
    TEST_ASSERT(!OF_METER_MOD_FAILED_UNKNOWN_METER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_UNKNOWN_METER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_UNKNOWN_METER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_UNKNOWN_METER_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_TYPE_ALL;
    TEST_ASSERT(!OF_GROUP_TYPE_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_TYPE_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_TYPE_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_TYPE_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_TYPE;
    TEST_ASSERT(OF_ACTION_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FLAG_NO_BYT_COUNTS;
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_TEST(flags, OF_VERSION_1_3));
    OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_BYT_COUNTS_TEST(flags, OF_VERSION_1_3));
    value = OFP_BSN_VLAN_COUNTER_RX_PACKETS;
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_RX_PACKETS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_RX_PACKETS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_RX_PACKETS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_VLAN_COUNTER_RX_PACKETS_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_SP;
    TEST_ASSERT(!OFDPA_SNM_SP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_SP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_SP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_SP_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_METER_EXISTS;
    TEST_ASSERT(!OF_METER_MOD_FAILED_METER_EXISTS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_METER_EXISTS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_METER_EXISTS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_METER_EXISTS_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_BAD_FLAGS;
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_MIN_RATE_UNCFG;
    TEST_ASSERT(OF_QUEUE_MIN_RATE_UNCFG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_QUEUE_MIN_RATE_UNCFG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_MIN_RATE_UNCFG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_MIN_RATE_UNCFG_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_OUT_OF_GROUPS;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_OUT_OF_GROUPS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_OUT_OF_GROUPS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_OUT_OF_GROUPS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_OUT_OF_GROUPS_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_SET_ASYNC;
    TEST_ASSERT(!OF_OBJ_TYPE_SET_ASYNC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OBJ_TYPE_SET_ASYNC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_OBJ_TYPE_SET_ASYNC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_SET_ASYNC_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_MULTIPART_BUFFER_OVERFLOW;
    TEST_ASSERT(!OF_REQUEST_FAILED_MULTIPART_BUFFER_OVERFLOW_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_REQUEST_FAILED_MULTIPART_BUFFER_OVERFLOW_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_REQUEST_FAILED_MULTIPART_BUFFER_OVERFLOW_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_MULTIPART_BUFFER_OVERFLOW_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_PSM_DWRR;
    TEST_ASSERT(!OFDPA_PSM_DWRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_DWRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_DWRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_DWRR_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_FLOW_REMOVED;
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_REMOVED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_REMOVED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_REMOVED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_FLOW_REMOVED_SUPPORTED(OF_VERSION_1_3));
    value = OF_HELLO_ELEM_TYPE__VERSIONBITMAP;
    TEST_ASSERT(!OF_HELLO_ELEM_TYPE__VERSIONBITMAP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_HELLO_ELEM_TYPE__VERSIONBITMAP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_HELLO_ELEM_TYPE__VERSIONBITMAP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_HELLO_ELEM_TYPE__VERSIONBITMAP_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_NO_FLOOD;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_FLOOD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_FLOOD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_FLOOD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_FLOOD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_NO_FLOOD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_NO_FLOOD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_FLOOD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_NO_FLOOD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_FLOOD_TEST(flags, OF_VERSION_1_0));
    value = OF_CAPABILITIES_FLAG_PORT_STATS;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_STATS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_STATS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_STATS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_STATS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_PORT_STATS_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_PORT_STATS_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_STATS_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_PORT_STATS_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_PORT_STATS_TEST(flags, OF_VERSION_1_0));
    value = OF_STATS_TYPE_QUEUE;
    TEST_ASSERT(OF_STATS_TYPE_QUEUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_QUEUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_QUEUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_QUEUE_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_BAD_ACTION;
    TEST_ASSERT(OF_ERROR_TYPE_BAD_ACTION_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_ACTION_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_ACTION_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_ACTION_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_SP_DWRR;
    TEST_ASSERT(!OFDPA_SNM_SP_DWRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_SP_DWRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_SP_DWRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_SP_DWRR_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_SCHEDULER_NODE_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_SCHEDULER_NODE_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_SCHEDULER_NODE_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_SCHEDULER_NODE_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_SCHEDULER_NODE_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_FIELD;
    TEST_ASSERT(!OF_ACTION_TYPE_SET_FIELD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_FIELD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_SET_FIELD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_SET_FIELD_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_EPERM;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_IN_PORT;
    TEST_ASSERT(OF_FLOW_WC_V1_IN_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_IN_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_IN_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_IN_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_MOD_FAILED_EPERM;
    TEST_ASSERT(!OF_TABLE_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_COPY_TTL_IN;
    TEST_ASSERT(!OF_ACTION_TYPE_COPY_TTL_IN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_COPY_TTL_IN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_COPY_TTL_IN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_COPY_TTL_IN_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_SCHEDULER_NODE_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_SCHEDULER_NODE_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_SCHEDULER_NODE_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_SCHEDULER_NODE_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_SCHEDULER_NODE_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_OAM_LM_TX_COUNT;
    TEST_ASSERT(!OFDPA_ACT_OAM_LM_TX_COUNT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_OAM_LM_TX_COUNT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_OAM_LM_TX_COUNT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_OAM_LM_TX_COUNT_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_MASK;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_MASK_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_WRITE_SETFIELD_MISS;
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_WRITE_SETFIELD_MISS_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_FRAG;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_FRAG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_FRAG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_FRAG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_FRAG_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_FRAG_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_FRAG_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_FRAG_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_FRAG_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_FRAG_TEST(flags, OF_VERSION_1_3));
    value = OF_STATS_TYPE_PORT_DESC;
    TEST_ASSERT(!OF_STATS_TYPE_PORT_DESC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_STATS_TYPE_PORT_DESC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_STATS_TYPE_PORT_DESC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_PORT_DESC_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_TABLE_MOD;
    TEST_ASSERT(!OF_OBJ_TYPE_TABLE_MOD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_TABLE_MOD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_TABLE_MOD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_TABLE_MOD_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_EPERM;
    TEST_ASSERT(!OF_MATCH_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_MATCH_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_COMMAND_MODIFY;
    TEST_ASSERT(!OF_GROUP_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_3));
    value = OF_SWITCH_CONFIG_FAILED_EPERM;
    TEST_ASSERT(!OF_SWITCH_CONFIG_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_SWITCH_CONFIG_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_SWITCH_CONFIG_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_APPLY_ACTIONS_MISS;
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_APPLY_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_NO_ROUTE;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_NO_ROUTE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_NO_ROUTE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_NO_ROUTE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_NO_ROUTE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_PSM_SP_WFQ;
    TEST_ASSERT(!OFDPA_PSM_SP_WFQ_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_SP_WFQ_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_SP_WFQ_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_SP_WFQ_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CHANGE_REASON_DELETE;
    TEST_ASSERT(OF_PORT_CHANGE_REASON_DELETE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_DELETE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_DELETE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_DELETE_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_SUBTYPE;
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_SUBTYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_SUBTYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_SUBTYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_SUBTYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_ICMP_CODE;
    TEST_ASSERT(OF_FLOW_WC_V1_ICMP_CODE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_ICMP_CODE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_ICMP_CODE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_ICMP_CODE_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_WRITE_ACTIONS;
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_EXPERIMENTER;
    TEST_ASSERT(OF_OBJ_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_BUFFER_ID_NO_BUFFER;
    TEST_ASSERT(!OF_BUFFER_ID_NO_BUFFER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_BUFFER_ID_NO_BUFFER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_BUFFER_ID_NO_BUFFER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_BUFFER_ID_NO_BUFFER_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_TYPE_OXM;
    TEST_ASSERT(!OF_MATCH_TYPE_OXM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_MATCH_TYPE_OXM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_TYPE_OXM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_TYPE_OXM_SUPPORTED(OF_VERSION_1_3));
    value = OF_TCP_PORT;
    TEST_ASSERT(OF_TCP_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_TCP_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TCP_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TCP_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_BAD_RATE;
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_RATE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_RATE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_RATE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_BAD_RATE_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_GROUP_MOD;
    TEST_ASSERT(!OF_OBJ_TYPE_GROUP_MOD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_GROUP_MOD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_GROUP_MOD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_GROUP_MOD_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_COMMAND_MODIFY_STRICT;
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_STRICT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_STRICT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_STRICT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_STRICT_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_NW_DST_BITS;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_DST_BITS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_BITS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_BITS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_BITS_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_SET_MPLS_PCPDEI_FROM_VPN_TABLE;
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_PCPDEI_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_PCPDEI_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_PCPDEI_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_SET_MPLS_PCPDEI_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_SET_MPLS_TC_FROM_VPN_TABLE;
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_TC_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_TC_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_TC_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_SET_MPLS_TC_FROM_VPN_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_WRITE_SETFIELD;
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_SETFIELD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_SETFIELD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_SETFIELD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_WRITE_SETFIELD_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_MPLS_TC;
    TEST_ASSERT(!OF_ACTION_TYPE_SET_MPLS_TC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_MPLS_TC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_MPLS_TC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_MPLS_TC_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_IP_REASM;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_IP_REASM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_IP_REASM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_IP_REASM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_IP_REASM_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_IP_REASM_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_IP_REASM_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_IP_REASM_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_IP_REASM_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_IP_REASM_TEST(flags, OF_VERSION_1_0));
    value = OF_GROUP_MOD_FAILED_GROUP_EXISTS;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_GROUP_EXISTS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_GROUP_EXISTS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_GROUP_EXISTS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_GROUP_EXISTS_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONFIG_FRAG_REASM;
    TEST_ASSERT(OF_CONFIG_FRAG_REASM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_FRAG_REASM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONFIG_FRAG_REASM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONFIG_FRAG_REASM_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CONFIG_FRAG_REASM_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CONFIG_FRAG_REASM_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_FRAG_REASM_TEST(flags, OF_VERSION_1_0));
    OF_CONFIG_FRAG_REASM_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CONFIG_FRAG_REASM_TEST(flags, OF_VERSION_1_0));
    value = OF_REQUEST_FAILED_BAD_EXPERIMENTER;
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_PROPERTY_EXPERIMENTER;
    TEST_ASSERT(!OF_QUEUE_PROPERTY_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_QUEUE_PROPERTY_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_PROPERTY_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_PROPERTY_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_BAD_COMMAND;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_STP_MASK;
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_MASK_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_STATE_FLAG_STP_MASK_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_STATE_FLAG_STP_MASK_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_MASK_TEST(flags, OF_VERSION_1_0));
    OF_PORT_STATE_FLAG_STP_MASK_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_MASK_TEST(flags, OF_VERSION_1_0));
    value = OF_ERROR_TYPE_ROLE_REQUEST_FAILED;
    TEST_ASSERT(!OF_ERROR_TYPE_ROLE_REQUEST_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ERROR_TYPE_ROLE_REQUEST_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_ROLE_REQUEST_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_ROLE_REQUEST_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_DEC_TTL_MTU;
    TEST_ASSERT(!OFDPA_ACT_DEC_TTL_MTU_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_DEC_TTL_MTU_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_DEC_TTL_MTU_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_DEC_TTL_MTU_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONTROLLER_PKT_NO_BUFFER;
    TEST_ASSERT(!OF_CONTROLLER_PKT_NO_BUFFER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CONTROLLER_PKT_NO_BUFFER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONTROLLER_PKT_NO_BUFFER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONTROLLER_PKT_NO_BUFFER_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_ROUTER;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ROUTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ROUTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ROUTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_ROUTER_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_ROUTER_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_ROUTER_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_ROUTER_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_ROUTER_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ROUTER_TEST(flags, OF_VERSION_1_3));
    value = OFDPA_ACT_SET_MPLS_TC_FROM_TUNNEL_TABLE;
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_TC_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_TC_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_SET_MPLS_TC_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_SET_MPLS_TC_FROM_TUNNEL_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_VLAN_COUNTER_TX_PACKETS;
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_TX_PACKETS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_TX_PACKETS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_TX_PACKETS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_VLAN_COUNTER_TX_PACKETS_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_REQUEST_FAILED_BAD_TABLE;
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_REQUEST_FAILED_BAD_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_REQUEST_FAILED_BAD_METADATA;
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_METADATA_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_METADATA_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_METADATA_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_REQUEST_FAILED_BAD_METADATA_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_WFQ;
    TEST_ASSERT(!OFDPA_SNM_WFQ_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_WFQ_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_WFQ_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_WFQ_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FLAG_EMERG;
    TEST_ASSERT(OF_FLOW_MOD_FLAG_EMERG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_EMERG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_EMERG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_EMERG_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_FLOW_MOD_FLAG_EMERG_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_FLOW_MOD_FLAG_EMERG_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_EMERG_TEST(flags, OF_VERSION_1_0));
    OF_FLOW_MOD_FLAG_EMERG_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_EMERG_TEST(flags, OF_VERSION_1_0));
    value = OF_SERIAL_NUM_LEN;
    TEST_ASSERT(OF_SERIAL_NUM_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_SERIAL_NUM_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_SERIAL_NUM_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_SERIAL_NUM_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_HELLO_FAILED_EPERM;
    TEST_ASSERT(OF_HELLO_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_HELLO_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_HELLO_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_HELLO_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_COPY_TTL_OUT;
    TEST_ASSERT(!OF_ACTION_TYPE_COPY_TTL_OUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_COPY_TTL_OUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_COPY_TTL_OUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_COPY_TTL_OUT_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_BAD_FLAGS;
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_BAD_FLAGS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_METER_MOD_FAILED_BAD_FLAGS_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_METER_MOD_FAILED_BAD_FLAGS_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_METER_MOD_FAILED_BAD_FLAGS_TEST(flags, OF_VERSION_1_3));
    OF_METER_MOD_FAILED_BAD_FLAGS_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_FLAGS_TEST(flags, OF_VERSION_1_3));
    value = OF_OBJ_TYPE_PORT_STATUS;
    TEST_ASSERT(OF_OBJ_TYPE_PORT_STATUS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_PORT_STATUS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_PORT_STATUS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_PORT_STATUS_SUPPORTED(OF_VERSION_1_3));
    value = OF_DL_TYPE_ETH2_CUTOFF;
    TEST_ASSERT(OF_DL_TYPE_ETH2_CUTOFF_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_DL_TYPE_ETH2_CUTOFF_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_DL_TYPE_ETH2_CUTOFF_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_DL_TYPE_ETH2_CUTOFF_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_CHECK_DROP_STATUS;
    TEST_ASSERT(!OFDPA_ACT_CHECK_DROP_STATUS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_CHECK_DROP_STATUS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_CHECK_DROP_STATUS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_CHECK_DROP_STATUS_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_BAD_LEN;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_L2_INTERFACE_REMARK_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_L2_INTERFACE_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_L2_INTERFACE_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_L2_INTERFACE_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_L2_INTERFACE_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_UNSUP_INST;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_UNSUP_INST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_INST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_INST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_INST_SUPPORTED(OF_VERSION_1_3));
    value = OFPQDPT_MAX_RATE;
    TEST_ASSERT(!OFPQDPT_MAX_RATE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFPQDPT_MAX_RATE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFPQDPT_MAX_RATE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFPQDPT_MAX_RATE_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_100MB_HD;
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_HD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_HD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_HD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_HD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_100MB_HD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_100MB_HD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100MB_HD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_FEATURE_FLAG_100MB_HD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_100MB_HD_TEST(flags, OF_VERSION_1_0));
    value = OF_ACTION_FAILED_BAD_SET_LEN;
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_SET_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_SET_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_SET_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_SET_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_TYPE_SELECT;
    TEST_ASSERT(!OF_GROUP_TYPE_SELECT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_TYPE_SELECT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_TYPE_SELECT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_TYPE_SELECT_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_CAPABILITIES_CHAINING;
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_CHAINING_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_CHAINING_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_CHAINING_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_CHAINING_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_IS_SLAVE;
    TEST_ASSERT(!OF_REQUEST_FAILED_IS_SLAVE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_REQUEST_FAILED_IS_SLAVE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_IS_SLAVE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_IS_SLAVE_SUPPORTED(OF_VERSION_1_3));
    value = OFPPMPT_ETERNET;
    TEST_ASSERT(!OFPPMPT_ETERNET_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFPPMPT_ETERNET_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFPPMPT_ETERNET_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFPPMPT_ETERNET_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_BAD_BAND_VALUE;
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BAND_VALUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BAND_VALUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BAND_VALUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_BAD_BAND_VALUE_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_EPERM;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_FEATURES_REPLY;
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REPLY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REPLY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REPLY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REPLY_SUPPORTED(OF_VERSION_1_3));
    value = OF_BSN_VPORT_STATUS_FAILED;
    TEST_ASSERT(OF_BSN_VPORT_STATUS_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_BSN_VPORT_STATUS_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_BSN_VPORT_STATUS_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_BSN_VPORT_STATUS_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_CLASS_BASED_TX_COUNT;
    TEST_ASSERT(!OFDPA_ACT_CLASS_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_CLASS_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_CLASS_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_CLASS_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_NW_ADDR_MASK;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_NW_ADDR_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_NW_ADDR_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_NW_ADDR_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_NW_ADDR_MASK_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_MPLS_SET_QOS_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_MPLS_SET_QOS_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_MPLS_SET_QOS_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_MPLS_SET_QOS_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_MPLS_SET_QOS_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_UNSUPPORTED;
    TEST_ASSERT(OF_FLOW_MOD_FAILED_UNSUPPORTED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_UNSUPPORTED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_UNSUPPORTED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_UNSUPPORTED_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_PUSH_L2_Header;
    TEST_ASSERT(!OFDPA_ACT_PUSH_L2_Header_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_PUSH_L2_Header_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_PUSH_L2_Header_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_PUSH_L2_Header_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_LEN;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_NEXT_TABLES_MISS;
    TEST_ASSERT(!OF_TABLE_FEATURE_NEXT_TABLES_MISS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_NEXT_TABLES_MISS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_NEXT_TABLES_MISS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_NEXT_TABLES_MISS_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_NW_SRC_ALL;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_SRC_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_OUT_PORT;
    TEST_ASSERT(OF_ACTION_FAILED_BAD_OUT_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_OUT_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_OUT_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_OUT_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_MOD_FAILED_BAD_HW_ADDR;
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_HW_ADDR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_HW_ADDR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_HW_ADDR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_HW_ADDR_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_TYPE_WRITE_ACTIONS;
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_WRITE_ACTIONS_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_100GB_FD;
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_100GB_FD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100GB_FD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100GB_FD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100GB_FD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_100GB_FD_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_100GB_FD_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_100GB_FD_TEST(flags, OF_VERSION_1_1));
    OF_PORT_FEATURE_FLAG_100GB_FD_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_100GB_FD_TEST(flags, OF_VERSION_1_1));
    value = OF_PACKET_IN_REASON_BSN_DEBUG;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEBUG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEBUG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEBUG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_DEBUG_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_PREREQ;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_PREREQ_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_PREREQ_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_PREREQ_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_PREREQ_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_OUT_OF_BUCKETS;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_OUT_OF_BUCKETS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_OUT_OF_BUCKETS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_OUT_OF_BUCKETS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_OUT_OF_BUCKETS_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_FRAGMENTATION_REQUIRED;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_FRAGMENTATION_REQUIRED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_FRAGMENTATION_REQUIRED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_FRAGMENTATION_REQUIRED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_FRAGMENTATION_REQUIRED_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_BAD_BURST;
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BURST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BURST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BURST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_BAD_BURST_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_STRIP_VLAN;
    TEST_ASSERT(OF_ACTION_TYPE_STRIP_VLAN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_TYPE_STRIP_VLAN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_STRIP_VLAN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_STRIP_VLAN_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_DEST_WILDCARD;
    TEST_ASSERT(!OF_PORT_DEST_WILDCARD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_WILDCARD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_WILDCARD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_WILDCARD_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_PORT_PROP_SCHEDULER;
    TEST_ASSERT(!OFDPA_EXP_PORT_PROP_SCHEDULER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_PORT_PROP_SCHEDULER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_PORT_PROP_SCHEDULER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_PORT_PROP_SCHEDULER_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_SET_CONFIG;
    TEST_ASSERT(OF_OBJ_TYPE_SET_CONFIG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_SET_CONFIG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_SET_CONFIG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_SET_CONFIG_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_CHAINING_UNSUPPORTED;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_CHAINING_UNSUPPORTED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_CHAINING_UNSUPPORTED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_CHAINING_UNSUPPORTED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_CHAINING_UNSUPPORTED_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_AGGREGATE;
    TEST_ASSERT(OF_STATS_TYPE_AGGREGATE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_AGGREGATE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_AGGREGATE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_AGGREGATE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_ICMP_TYPE;
    TEST_ASSERT(OF_FLOW_WC_V1_ICMP_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_ICMP_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_ICMP_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_ICMP_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_GET_CONFIG_REQUEST;
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_REPLY_FLAG_REPLY_MORE;
    TEST_ASSERT(OF_STATS_REPLY_FLAG_REPLY_MORE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_REPLY_FLAG_REPLY_MORE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_REPLY_FLAG_REPLY_MORE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_REPLY_FLAG_REPLY_MORE_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_STATS_REPLY_FLAG_REPLY_MORE_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_STATS_REPLY_FLAG_REPLY_MORE_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_REPLY_FLAG_REPLY_MORE_TEST(flags, OF_VERSION_1_0));
    OF_STATS_REPLY_FLAG_REPLY_MORE_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_STATS_REPLY_FLAG_REPLY_MORE_TEST(flags, OF_VERSION_1_0));
    value = OF_TABLE_REQUEST_FAILED_EPERM;
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_DESC_STR_LEN;
    TEST_ASSERT(OF_DESC_STR_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_DESC_STR_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_DESC_STR_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_DESC_STR_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_DEC_MPLS_TTL;
    TEST_ASSERT(!OF_ACTION_TYPE_DEC_MPLS_TTL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_DEC_MPLS_TTL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_DEC_MPLS_TTL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_DEC_MPLS_TTL_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_MPLS_TC;
    TEST_ASSERT(!OF_FLOW_WC_V1_MPLS_TC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_MPLS_TC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_MPLS_TC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_MPLS_TC_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_UNSUP_METADATA;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_UNSUP_METADATA_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_METADATA_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_METADATA_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_METADATA_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_OTHER;
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_OTHER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_OTHER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_OTHER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_OTHER_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_OTHER_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_OTHER_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_OTHER_TEST(flags, OF_VERSION_1_1));
    OF_PORT_FEATURE_FLAG_OTHER_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_OTHER_TEST(flags, OF_VERSION_1_1));
    value = OF_ACTION_TYPE_SET_VLAN_VID;
    TEST_ASSERT(OF_ACTION_TYPE_SET_VLAN_VID_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_VLAN_VID_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_VLAN_VID_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_VLAN_VID_SUPPORTED(OF_VERSION_1_3));
    value = OF_ROLE_REQUEST_FAILED_BAD_ROLE;
    TEST_ASSERT(!OF_ROLE_REQUEST_FAILED_BAD_ROLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ROLE_REQUEST_FAILED_BAD_ROLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ROLE_REQUEST_FAILED_BAD_ROLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ROLE_REQUEST_FAILED_BAD_ROLE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_QUEUE_DESC_PROP_PARENT;
    TEST_ASSERT(!OFDPA_EXP_QUEUE_DESC_PROP_PARENT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_QUEUE_DESC_PROP_PARENT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_QUEUE_DESC_PROP_PARENT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_QUEUE_DESC_PROP_PARENT_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_MAX;
    TEST_ASSERT(!OF_TABLE_MAX_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_MAX_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_MAX_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_MAX_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_TYPE;
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_BAND_TYPE_DROP;
    TEST_ASSERT(!OF_METER_BAND_TYPE_DROP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_BAND_TYPE_DROP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_BAND_TYPE_DROP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_BAND_TYPE_DROP_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_ROLE_REQUEST;
    TEST_ASSERT(!OF_OBJ_TYPE_ROLE_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OBJ_TYPE_ROLE_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_ROLE_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_ROLE_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_DEST_IN_PORT;
    TEST_ASSERT(OF_PORT_DEST_IN_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_IN_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_IN_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_IN_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_NW_DST_MASK;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_DST_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_DST_MASK_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_METER_CONFIG;
    TEST_ASSERT(!OF_STATS_TYPE_METER_CONFIG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_STATS_TYPE_METER_CONFIG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_STATS_TYPE_METER_CONFIG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_METER_CONFIG_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_HELLO_FAILED;
    TEST_ASSERT(OF_ERROR_TYPE_HELLO_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_HELLO_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_HELLO_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_HELLO_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_COLOR_SET_BAND_MODE_TRTCM;
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_TRTCM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_TRTCM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_TRTCM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_COLOR_SET_BAND_MODE_TRTCM_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_NW_ECN;
    TEST_ASSERT(!OF_ACTION_TYPE_SET_NW_ECN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_NW_ECN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_NW_ECN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_NW_ECN_SUPPORTED(OF_VERSION_1_3));
    value = OF_HELLO_FAILED_INCOMPATIBLE;
    TEST_ASSERT(OF_HELLO_FAILED_INCOMPATIBLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_HELLO_FAILED_INCOMPATIBLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_HELLO_FAILED_INCOMPATIBLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_HELLO_FAILED_INCOMPATIBLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_NUMBER_MAX;
    TEST_ASSERT(OF_PORT_NUMBER_MAX_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_NUMBER_MAX_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_NUMBER_MAX_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_NUMBER_MAX_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_MSG_MOD_ADD;
    TEST_ASSERT(!OFDPA_MSG_MOD_ADD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_MSG_MOD_ADD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_MSG_MOD_ADD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_MSG_MOD_ADD_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_1TB_FD;
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_1TB_FD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1TB_FD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1TB_FD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1TB_FD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_1TB_FD_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_1TB_FD_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1TB_FD_TEST(flags, OF_VERSION_1_1));
    OF_PORT_FEATURE_FLAG_1TB_FD_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_1TB_FD_TEST(flags, OF_VERSION_1_1));
    value = OF_VLAN_TAG_PRESENT;
    TEST_ASSERT(!OF_VLAN_TAG_PRESENT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_VLAN_TAG_PRESENT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_VLAN_TAG_PRESENT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_VLAN_TAG_PRESENT_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_TYPE;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_OXM_CLASS_NXM_1;
    TEST_ASSERT(!OF_OXM_CLASS_NXM_1_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OXM_CLASS_NXM_1_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OXM_CLASS_NXM_1_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OXM_CLASS_NXM_1_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_POP_VLAN;
    TEST_ASSERT(!OF_ACTION_TYPE_POP_VLAN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_POP_VLAN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_POP_VLAN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_POP_VLAN_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_INSTRUCTIONS_MISS;
    TEST_ASSERT(!OF_TABLE_FEATURE_INSTRUCTIONS_MISS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_INSTRUCTIONS_MISS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_INSTRUCTIONS_MISS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_INSTRUCTIONS_MISS_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONTROLLER_ROLE_SLAVE;
    TEST_ASSERT(!OF_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_TABLE_MOD_FAILED;
    TEST_ASSERT(!OF_ERROR_TYPE_TABLE_MOD_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_TABLE_MOD_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_TABLE_MOD_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_TABLE_MOD_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_QDC_TAIL_DROP;
    TEST_ASSERT(!OFDPA_QDC_TAIL_DROP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_QDC_TAIL_DROP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_QDC_TAIL_DROP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_QDC_TAIL_DROP_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_POP_L2_Header;
    TEST_ASSERT(!OFDPA_ACT_POP_L2_Header_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_POP_L2_Header_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_POP_L2_Header_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_POP_L2_Header_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_QUEUE_STATS;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_QUEUE_STATS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_QUEUE_STATS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_QUEUE_STATS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_QUEUE_STATS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_QUEUE_STATS_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_QUEUE_STATS_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_QUEUE_STATS_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_QUEUE_STATS_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_QUEUE_STATS_TEST(flags, OF_VERSION_1_0));
    value = OFPPMPT_EXPERIMENTER;
    TEST_ASSERT(!OFPPMPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFPPMPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFPPMPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFPPMPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_BAD_VLAN;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_BAD_VLAN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_BAD_VLAN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_BAD_VLAN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_BAD_VLAN_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_APPLY_SETFIELD;
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_SETFIELD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_SETFIELD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_SETFIELD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_APPLY_SETFIELD_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_APPLY_ACTIONS;
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_MPLS_LABEL;
    TEST_ASSERT(!OF_ACTION_TYPE_SET_MPLS_LABEL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_MPLS_LABEL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_MPLS_LABEL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_MPLS_LABEL_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_UNKNOWN;
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_DEST_USE_TABLE;
    TEST_ASSERT(OF_PORT_DEST_USE_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_USE_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_USE_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_USE_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_LINK_DOWN;
    TEST_ASSERT(OF_PORT_STATE_FLAG_LINK_DOWN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LINK_DOWN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LINK_DOWN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LINK_DOWN_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_STATE_FLAG_LINK_DOWN_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_STATE_FLAG_LINK_DOWN_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LINK_DOWN_TEST(flags, OF_VERSION_1_0));
    OF_PORT_STATE_FLAG_LINK_DOWN_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_STATE_FLAG_LINK_DOWN_TEST(flags, OF_VERSION_1_0));
    value = OFDPA_ACT_COPY_TC_OUT;
    TEST_ASSERT(!OFDPA_ACT_COPY_TC_OUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_COPY_TC_OUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_COPY_TC_OUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_COPY_TC_OUT_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_MOD_FAILED_BAD_CONFIG;
    TEST_ASSERT(!OF_PORT_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_CONTROLLER_CONNECTION_STATE_DISCONNECTED;
    TEST_ASSERT(!OFP_BSN_CONTROLLER_CONNECTION_STATE_DISCONNECTED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_CONNECTION_STATE_DISCONNECTED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_CONNECTION_STATE_DISCONNECTED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_CONTROLLER_CONNECTION_STATE_DISCONNECTED_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_COLOR_BASED_RX_COUNT;
    TEST_ASSERT(!OFDPA_ACT_COLOR_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_COLOR_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_COLOR_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_COLOR_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_DEST_ALL;
    TEST_ASSERT(OF_PORT_DEST_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_FIELD;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_FIELD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_FIELD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_FIELD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_FIELD_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_DEST_LOCAL;
    TEST_ASSERT(OF_PORT_DEST_LOCAL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_LOCAL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_LOCAL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_LOCAL_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_WILDCARDS;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_WILDCARDS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_WILDCARDS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_WILDCARDS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_WILDCARDS_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_PORT;
    TEST_ASSERT(OF_STATS_TYPE_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_RX_PACKETS_UNICAST;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_RX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_SET_PCPDEI_FROM_TABLE;
    TEST_ASSERT(!OFDPA_ACT_SET_PCPDEI_FROM_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_SET_PCPDEI_FROM_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_SET_PCPDEI_FROM_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_SET_PCPDEI_FROM_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_TYPE_FF;
    TEST_ASSERT(!OF_GROUP_TYPE_FF_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_TYPE_FF_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_TYPE_FF_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_TYPE_FF_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_BAD_BAND;
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BAND_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BAND_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_BAND_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_BAD_BAND_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_FLAG_KBPS;
    TEST_ASSERT(!OF_METER_FLAG_KBPS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_FLAG_KBPS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_FLAG_KBPS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_FLAG_KBPS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_METER_FLAG_KBPS_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_METER_FLAG_KBPS_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_METER_FLAG_KBPS_TEST(flags, OF_VERSION_1_3));
    OF_METER_FLAG_KBPS_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_METER_FLAG_KBPS_TEST(flags, OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_1GB_FD;
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_FD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_FD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_FD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_FD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_1GB_FD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_1GB_FD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_FD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_FEATURE_FLAG_1GB_FD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_1GB_FD_TEST(flags, OF_VERSION_1_0));
    value = OF_FLOW_WC_V1_NW_SRC_SHIFT;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_SRC_SHIFT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_SHIFT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_SHIFT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_SHIFT_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_RR;
    TEST_ASSERT(!OFDPA_SNM_RR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_RR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_RR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_RR_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_MOD_FAILED_BAD_CONFIG;
    TEST_ASSERT(!OF_TABLE_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_MOD_FAILED_BAD_CONFIG_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_BAND_TYPE_DSCP_REMARK;
    TEST_ASSERT(!OF_METER_BAND_TYPE_DSCP_REMARK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_BAND_TYPE_DSCP_REMARK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_BAND_TYPE_DSCP_REMARK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_BAND_TYPE_DSCP_REMARK_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_MSG_MOD_MODIFY;
    TEST_ASSERT(!OFDPA_MSG_MOD_MODIFY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_MSG_MOD_MODIFY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_MSG_MOD_MODIFY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_MSG_MOD_MODIFY_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_VLAN_COUNTER_TX_BYTES;
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_VLAN_COUNTER_TX_BYTES_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_STP_FORWARD;
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_FORWARD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_FORWARD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_FORWARD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_FORWARD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_STATE_FLAG_STP_FORWARD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_STATE_FLAG_STP_FORWARD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_FORWARD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_STATE_FLAG_STP_FORWARD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_FORWARD_TEST(flags, OF_VERSION_1_0));
    value = OF_OBJ_TYPE_ECHO_REPLY;
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REPLY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REPLY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REPLY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REPLY_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_OFPMBT_COLOR_SET;
    TEST_ASSERT(!OFDPA_OFPMBT_COLOR_SET_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_OFPMBT_COLOR_SET_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_OFPMBT_COLOR_SET_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_OFPMBT_COLOR_SET_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_GET_CONFIG_REPLY;
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REPLY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REPLY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REPLY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_GET_CONFIG_REPLY_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_REQUEST_FAILED_BAD_TYPE;
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_REQUEST_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_BAD_REQUEST;
    TEST_ASSERT(OF_ERROR_TYPE_BAD_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_EPERM;
    TEST_ASSERT(OF_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_DEFAULT_PRIORITY;
    TEST_ASSERT(OF_DEFAULT_PRIORITY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_DEFAULT_PRIORITY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_DEFAULT_PRIORITY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_DEFAULT_PRIORITY_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_UNKNOWN_GROUP;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_UNKNOWN_GROUP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_UNKNOWN_GROUP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_UNKNOWN_GROUP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_UNKNOWN_GROUP_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_TX_PACKETS_MULTICAST;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_TX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_MOD_FAILED_EPERM;
    TEST_ASSERT(!OF_PORT_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_MOD_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_CAPABILITIES_SELECT_LIVENESS;
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_SELECT_LIVENESS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_SELECT_LIVENESS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_SELECT_LIVENESS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_SELECT_LIVENESS_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_QUEUE_DESC_PROP_CONGESTION;
    TEST_ASSERT(!OFDPA_EXP_QUEUE_DESC_PROP_CONGESTION_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_QUEUE_DESC_PROP_CONGESTION_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_QUEUE_DESC_PROP_CONGESTION_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_QUEUE_DESC_PROP_CONGESTION_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_GROUP;
    TEST_ASSERT(!OF_ACTION_TYPE_GROUP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_GROUP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_GROUP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_GROUP_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_TX_PACKETS_BROADCAST;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_TX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_3));
    value = OF_NICIRA_CONTROLLER_ROLE_OTHER;
    TEST_ASSERT(OF_NICIRA_CONTROLLER_ROLE_OTHER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_OTHER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_OTHER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_OTHER_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_UDF_ANCHOR_PACKET_START;
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_PACKET_START_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_PACKET_START_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_PACKET_START_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_UDF_ANCHOR_PACKET_START_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_BAD_WATCH;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_WATCH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_WATCH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_WATCH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_WATCH_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_ALL;
    TEST_ASSERT(!OF_GROUP_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_OUT_OF_METERS;
    TEST_ASSERT(!OF_METER_MOD_FAILED_OUT_OF_METERS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_OUT_OF_METERS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_OUT_OF_METERS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_OUT_OF_METERS_SUPPORTED(OF_VERSION_1_3));
    value = OF_ROLE_REQUEST_FAILED_UNSUP;
    TEST_ASSERT(!OF_ROLE_REQUEST_FAILED_UNSUP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ROLE_REQUEST_FAILED_UNSUP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ROLE_REQUEST_FAILED_UNSUP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ROLE_REQUEST_FAILED_UNSUP_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_TABLE_STATS;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_TABLE_STATS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_TABLE_STATS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_TABLE_STATS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_TABLE_STATS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_TABLE_STATS_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_TABLE_STATS_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_TABLE_STATS_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_TABLE_STATS_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_TABLE_STATS_TEST(flags, OF_VERSION_1_0));
    value = OF_OBJ_TYPE_PACKET_OUT;
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_OUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_OUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_OUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_OUT_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_OAM_LM_RX_COUNT;
    TEST_ASSERT(!OFDPA_ACT_OAM_LM_RX_COUNT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_OAM_LM_RX_COUNT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_OAM_LM_RX_COUNT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_OAM_LM_RX_COUNT_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_SET_TYPE;
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_SET_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_SET_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_SET_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_SET_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_ICMP_ECHO_REQUEST;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_ICMP_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_ICMP_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_ICMP_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_ICMP_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_UNSEQ;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNSEQ_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNSEQ_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNSEQ_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_UNSEQ_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_UNSEQ_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_UNSEQ_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_UNSEQ_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_UNSEQ_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNSEQ_TEST(flags, OF_VERSION_1_3));
    value = OF_QUEUE_OP_FAILED_EPERM;
    TEST_ASSERT(OF_QUEUE_OP_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_CONFIG_DEPRECATED_MASK;
    TEST_ASSERT(!OF_TABLE_CONFIG_DEPRECATED_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_CONFIG_DEPRECATED_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_CONFIG_DEPRECATED_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_CONFIG_DEPRECATED_MASK_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_TABLE_CONFIG_DEPRECATED_MASK_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_TABLE_CONFIG_DEPRECATED_MASK_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_TABLE_CONFIG_DEPRECATED_MASK_TEST(flags, OF_VERSION_1_3));
    OF_TABLE_CONFIG_DEPRECATED_MASK_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_TABLE_CONFIG_DEPRECATED_MASK_TEST(flags, OF_VERSION_1_3));
    value = OFPQDPT_MIN_RATE;
    TEST_ASSERT(!OFPQDPT_MIN_RATE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFPQDPT_MIN_RATE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFPQDPT_MIN_RATE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFPQDPT_MIN_RATE_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_POP_PBB;
    TEST_ASSERT(!OF_ACTION_TYPE_POP_PBB_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_TYPE_POP_PBB_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_POP_PBB_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_POP_PBB_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_MATCH;
    TEST_ASSERT(!OF_TABLE_FEATURE_MATCH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_MATCH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_MATCH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_MATCH_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_GROUP_MOD_FAILED;
    TEST_ASSERT(!OF_ERROR_TYPE_GROUP_MOD_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_GROUP_MOD_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_GROUP_MOD_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_GROUP_MOD_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_10MB_FD;
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_FD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_FD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_FD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_FD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_10MB_FD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_10MB_FD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10MB_FD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_FEATURE_FLAG_10MB_FD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_10MB_FD_TEST(flags, OF_VERSION_1_0));
    value = OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_MPLS_VPN_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_PUSH_VLAN;
    TEST_ASSERT(!OF_ACTION_TYPE_PUSH_VLAN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_PUSH_VLAN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_PUSH_VLAN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_PUSH_VLAN_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_TP_DST;
    TEST_ASSERT(OF_ACTION_TYPE_SET_TP_DST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_TP_DST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_TP_DST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_TP_DST_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_EXPERIMENTER;
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_FLOW;
    TEST_ASSERT(OF_STATS_TYPE_FLOW_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_FLOW_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_FLOW_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_FLOW_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_FLAG_PKTPS;
    TEST_ASSERT(!OF_METER_FLAG_PKTPS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_FLAG_PKTPS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_FLAG_PKTPS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_FLAG_PKTPS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_METER_FLAG_PKTPS_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_METER_FLAG_PKTPS_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_METER_FLAG_PKTPS_TEST(flags, OF_VERSION_1_3));
    OF_METER_FLAG_PKTPS_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_METER_FLAG_PKTPS_TEST(flags, OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_TP_SRC;
    TEST_ASSERT(OF_ACTION_TYPE_SET_TP_SRC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_TP_SRC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_TP_SRC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_TP_SRC_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_PSM_RR;
    TEST_ASSERT(!OFDPA_PSM_RR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_RR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_RR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_RR_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_10GB_FD;
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10GB_FD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10GB_FD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10GB_FD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10GB_FD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_10GB_FD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_10GB_FD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_10GB_FD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_FEATURE_FLAG_10GB_FD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_10GB_FD_TEST(flags, OF_VERSION_1_0));
    value = OF_GROUP_MOD_FAILED_CHAINED_GROUP;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_CHAINED_GROUP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_CHAINED_GROUP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_CHAINED_GROUP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_CHAINED_GROUP_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_BSN_MIRROR_DEST_TEST(flags, OF_VERSION_1_0));
    value = OF_OXM_CLASS_EXPERIMENTER;
    TEST_ASSERT(!OF_OXM_CLASS_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OXM_CLASS_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OXM_CLASS_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OXM_CLASS_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_EXPERIMENTER_MISS;
    TEST_ASSERT(!OF_TABLE_FEATURE_EXPERIMENTER_MISS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_EXPERIMENTER_MISS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_EXPERIMENTER_MISS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_EXPERIMENTER_MISS_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_EXPERIMENTER;
    TEST_ASSERT(OF_STATS_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_REMOVED_REASON_IDLE_TIMEOUT;
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_IDLE_TIMEOUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_IDLE_TIMEOUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_IDLE_TIMEOUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_IDLE_TIMEOUT_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_DROP_STATUS_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_DROP_STATUS_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_DROP_STATUS_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_DROP_STATUS_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_DROP_STATUS_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_NICIRA_CONTROLLER_ROLE_SLAVE;
    TEST_ASSERT(OF_NICIRA_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_NICIRA_CONTROLLER_ROLE_SLAVE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_POP_CW_OR_ACH;
    TEST_ASSERT(!OFDPA_ACT_POP_CW_OR_ACH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_POP_CW_OR_ACH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_POP_CW_OR_ACH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_POP_CW_OR_ACH_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_QUEUE;
    TEST_ASSERT(OF_ACTION_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_QUEUE_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_PERMANENT;
    TEST_ASSERT(OF_FLOW_PERMANENT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_PERMANENT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_PERMANENT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_PERMANENT_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_PROPERTY_MIN_RATE;
    TEST_ASSERT(OF_QUEUE_PROPERTY_MIN_RATE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_QUEUE_PROPERTY_MIN_RATE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_PROPERTY_MIN_RATE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_PROPERTY_MIN_RATE_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_CONFIG_TABLE_MISS_MASK;
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_MASK_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_TABLE_CONFIG_TABLE_MISS_MASK_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_TABLE_CONFIG_TABLE_MISS_MASK_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_MASK_TEST(flags, OF_VERSION_1_1));
    OF_TABLE_CONFIG_TABLE_MISS_MASK_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_MASK_TEST(flags, OF_VERSION_1_1));
    value = OF_CONFIG_FRAG_MASK;
    TEST_ASSERT(OF_CONFIG_FRAG_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_FRAG_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONFIG_FRAG_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONFIG_FRAG_MASK_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CONFIG_FRAG_MASK_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CONFIG_FRAG_MASK_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_FRAG_MASK_TEST(flags, OF_VERSION_1_0));
    OF_CONFIG_FRAG_MASK_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CONFIG_FRAG_MASK_TEST(flags, OF_VERSION_1_0));
    value = OFDPA_EXP_COLOR_BASED_CTR_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_COLOR_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_COLOR_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_COLOR_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_COLOR_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_TYPE;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_ERROR;
    TEST_ASSERT(OF_OBJ_TYPE_ERROR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_ERROR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_ERROR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_ERROR_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_PACKET_OF_DEATH;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_PACKET_OF_DEATH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_PACKET_OF_DEATH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_PACKET_OF_DEATH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_PACKET_OF_DEATH_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_DEST_HOST_UNREACHABLE;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_HOST_UNREACHABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_HOST_UNREACHABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_HOST_UNREACHABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_DEST_HOST_UNREACHABLE_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_RX_ERRORS;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_ERRORS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_ERRORS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_ERRORS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_RX_ERRORS_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_ALL;
    TEST_ASSERT(!OF_METER_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_GROUP_DESC;
    TEST_ASSERT(!OF_STATS_TYPE_GROUP_DESC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_DESC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_DESC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_DESC_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_CLASS_BASED_RX_COUNT;
    TEST_ASSERT(!OFDPA_ACT_CLASS_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_CLASS_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_CLASS_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_CLASS_BASED_RX_COUNT_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_UNKNOWN;
    TEST_ASSERT(!OF_METER_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_UNKNOWN_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_BAD_EMERG_TIMEOUT;
    TEST_ASSERT(OF_FLOW_MOD_FAILED_BAD_EMERG_TIMEOUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_BAD_EMERG_TIMEOUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_BAD_EMERG_TIMEOUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_BAD_EMERG_TIMEOUT_SUPPORTED(OF_VERSION_1_3));
    value = OF_SSL_PORT;
    TEST_ASSERT(OF_SSL_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_SSL_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_SSL_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_SSL_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_BAND_TYPE_EXPERIMENTER;
    TEST_ASSERT(!OF_METER_BAND_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_BAND_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_BAND_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_BAND_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = LACP_TIMEDOUT;
    TEST_ASSERT(!LACP_TIMEDOUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!LACP_TIMEDOUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!LACP_TIMEDOUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(LACP_TIMEDOUT_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_RX_PACKETS_BROADCAST;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_RX_PACKETS_BROADCAST_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_PSM_SP_WRR;
    TEST_ASSERT(!OFDPA_PSM_SP_WRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_SP_WRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_SP_WRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_SP_WRR_SUPPORTED(OF_VERSION_1_3));
    value = OF_VLAN_TAG_ANY;
    TEST_ASSERT(!OF_VLAN_TAG_ANY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_VLAN_TAG_ANY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_VLAN_TAG_ANY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_VLAN_TAG_ANY_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_ESP;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ESP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ESP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ESP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_ESP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_ESP_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_ESP_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_ESP_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_ESP_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_ESP_TEST(flags, OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_BAD_TABLE_ID;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_TABLE_ID_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_FEATURES_REQUEST;
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_FEATURES_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_CLASS_BASED_CTR_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_CLASS_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_CLASS_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_CLASS_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_CLASS_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_REPLY_FLAG_REQ_MORE;
    TEST_ASSERT(!OF_STATS_REPLY_FLAG_REQ_MORE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_STATS_REPLY_FLAG_REQ_MORE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_STATS_REPLY_FLAG_REQ_MORE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_REPLY_FLAG_REQ_MORE_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_STATS_REPLY_FLAG_REQ_MORE_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_STATS_REPLY_FLAG_REQ_MORE_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_STATS_REPLY_FLAG_REQ_MORE_TEST(flags, OF_VERSION_1_3));
    OF_STATS_REPLY_FLAG_REQ_MORE_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_STATS_REPLY_FLAG_REQ_MORE_TEST(flags, OF_VERSION_1_3));
    value = OF_MATCH_FAILED_BAD_VALUE;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_VALUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_VALUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_VALUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_VALUE_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_LEN;
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_BAD_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_MPLS_TTL;
    TEST_ASSERT(!OF_ACTION_TYPE_SET_MPLS_TTL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_MPLS_TTL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_SET_MPLS_TTL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_SET_MPLS_TTL_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_PORT;
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_CAPABILITIES_SELECT_WEIGHT;
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_SELECT_WEIGHT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_CAPABILITIES_SELECT_WEIGHT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_SELECT_WEIGHT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_CAPABILITIES_SELECT_WEIGHT_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_VLAN_COUNTER_RX_BYTES;
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_VLAN_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_VLAN_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_TYPE_CLEAR_ACTIONS;
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_CLEAR_ACTIONS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_CLEAR_ACTIONS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_CLEAR_ACTIONS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_CLEAR_ACTIONS_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_INVALID_GROUP;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_INVALID_GROUP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_INVALID_GROUP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_INVALID_GROUP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_INVALID_GROUP_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_ECHO_REQUEST;
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_ECHO_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_DWRR;
    TEST_ASSERT(!OFDPA_SNM_DWRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_DWRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_DWRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_DWRR_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_MPLS_SET_QOS_FROM_TABLE;
    TEST_ASSERT(!OFDPA_ACT_MPLS_SET_QOS_FROM_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_MPLS_SET_QOS_FROM_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_MPLS_SET_QOS_FROM_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_MPLS_SET_QOS_FROM_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_COMMAND_ADD;
    TEST_ASSERT(!OF_GROUP_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_ARGUMENT;
    TEST_ASSERT(OF_ACTION_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_PACKET_IN;
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_IN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_IN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_IN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_PACKET_IN_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_NONEXT;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_NONEXT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_NONEXT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_NONEXT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_NONEXT_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_NONEXT_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_NONEXT_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_NONEXT_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_NONEXT_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_NONEXT_TEST(flags, OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_TAG;
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_TAG_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONTROLLER_ROLE_NOCHANGE;
    TEST_ASSERT(!OF_CONTROLLER_ROLE_NOCHANGE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CONTROLLER_ROLE_NOCHANGE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONTROLLER_ROLE_NOCHANGE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONTROLLER_ROLE_NOCHANGE_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_TYPE_WRITE_METADATA;
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_WRITE_METADATA_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_WRITE_METADATA_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_WRITE_METADATA_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_WRITE_METADATA_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_WILDCARDS;
    TEST_ASSERT(!OF_TABLE_FEATURE_WILDCARDS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_WILDCARDS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_WILDCARDS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_WILDCARDS_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_COLOR_BASED_TX_COUNT;
    TEST_ASSERT(!OFDPA_ACT_COLOR_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_COLOR_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_COLOR_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_COLOR_BASED_TX_COUNT_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_BAD_TYPE;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_TX_DROPPED;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_DROPPED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_DROPPED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_DROPPED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_TX_DROPPED_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_NO_MATCH;
    TEST_ASSERT(OF_PACKET_IN_REASON_NO_MATCH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PACKET_IN_REASON_NO_MATCH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PACKET_IN_REASON_NO_MATCH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_NO_MATCH_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_PSM_WFQ;
    TEST_ASSERT(!OFDPA_PSM_WFQ_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_WFQ_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_WFQ_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_WFQ_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FLAG_RESET_COUNTS;
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_RESET_COUNTS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_RESET_COUNTS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_RESET_COUNTS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_RESET_COUNTS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_FLOW_MOD_FLAG_RESET_COUNTS_SET(flags, OF_VERSION_1_2);
    TEST_ASSERT(flags == OF_FLOW_MOD_FLAG_RESET_COUNTS_BY_VERSION(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_RESET_COUNTS_TEST(flags, OF_VERSION_1_2));
    OF_FLOW_MOD_FLAG_RESET_COUNTS_CLEAR(flags, OF_VERSION_1_2);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_RESET_COUNTS_TEST(flags, OF_VERSION_1_2));
    value = OF_OBJ_TYPE_ROLE_REPLY;
    TEST_ASSERT(!OF_OBJ_TYPE_ROLE_REPLY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OBJ_TYPE_ROLE_REPLY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_ROLE_REPLY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_ROLE_REPLY_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_TYPE_STANDARD;
    TEST_ASSERT(!OF_MATCH_TYPE_STANDARD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_TYPE_STANDARD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_TYPE_STANDARD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_TYPE_STANDARD_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_STP_LISTEN;
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_LISTEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_LISTEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_LISTEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_LISTEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_NO_RECV_STP;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_RECV_STP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_RECV_STP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_RECV_STP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_RECV_STP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_NO_RECV_STP_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_NO_RECV_STP_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_NO_RECV_STP_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_NO_RECV_STP_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_NO_RECV_STP_TEST(flags, OF_VERSION_1_0));
    value = OF_STATS_TYPE_GROUP_FEATURES;
    TEST_ASSERT(!OF_STATS_TYPE_GROUP_FEATURES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_STATS_TYPE_GROUP_FEATURES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_FEATURES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_GROUP_FEATURES_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_MPLS_SET_QOS_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_MPLS_SET_QOS_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_MPLS_SET_QOS_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_MPLS_SET_QOS_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_MPLS_SET_QOS_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_RESERVED;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_RESERVED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_RESERVED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_RESERVED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_RESERVED_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_RESERVED_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_RESERVED_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_RESERVED_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_RESERVED_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_RESERVED_TEST(flags, OF_VERSION_1_0));
    value = OFDPA_EXP_CLASS_BASED_CTR_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_CLASS_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_CLASS_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_CLASS_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_CLASS_BASED_CTR_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONTROLLER_ROLE_EQUAL;
    TEST_ASSERT(!OF_CONTROLLER_ROLE_EQUAL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CONTROLLER_ROLE_EQUAL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONTROLLER_ROLE_EQUAL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONTROLLER_ROLE_EQUAL_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_RX_PACKETS_MULTICAST;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_RX_PACKETS_MULTICAST_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_OUTPUT;
    TEST_ASSERT(OF_ACTION_TYPE_OUTPUT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_OUTPUT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_OUTPUT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_OUTPUT_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_OAM_SET_COUNTER_FIELDS;
    TEST_ASSERT(!OFDPA_ACT_OAM_SET_COUNTER_FIELDS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_OAM_SET_COUNTER_FIELDS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_OAM_SET_COUNTER_FIELDS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_OAM_SET_COUNTER_FIELDS_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_NW_SRC_MASK;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_SRC_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_MASK_SUPPORTED(OF_VERSION_1_3));
    value = OF_DEFAULT_MISS_SEND_LEN;
    TEST_ASSERT(OF_DEFAULT_MISS_SEND_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_DEFAULT_MISS_SEND_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_DEFAULT_MISS_SEND_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_DEFAULT_MISS_SEND_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_REMOVED_REASON_GROUP_DELETE;
    TEST_ASSERT(!OF_FLOW_REMOVED_REASON_GROUP_DELETE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_GROUP_DELETE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_GROUP_DELETE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_GROUP_DELETE_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CHANGE_REASON_ADD;
    TEST_ASSERT(OF_PORT_CHANGE_REASON_ADD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_ADD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_ADD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_ADD_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_BLOCKED;
    TEST_ASSERT(!OF_PORT_STATE_FLAG_BLOCKED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_BLOCKED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_STATE_FLAG_BLOCKED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_STATE_FLAG_BLOCKED_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_STATE_FLAG_BLOCKED_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_PORT_STATE_FLAG_BLOCKED_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_STATE_FLAG_BLOCKED_TEST(flags, OF_VERSION_1_1));
    OF_PORT_STATE_FLAG_BLOCKED_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_STATE_FLAG_BLOCKED_TEST(flags, OF_VERSION_1_1));
    value = OF_MATCH_FAILED_BAD_DL_ADDR_MASK;
    TEST_ASSERT(!OF_MATCH_FAILED_BAD_DL_ADDR_MASK_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_DL_ADDR_MASK_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_DL_ADDR_MASK_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_BAD_DL_ADDR_MASK_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FLAG_BSN_SEND_IDLE;
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_TEST(flags, OF_VERSION_1_3));
    OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_BSN_SEND_IDLE_TEST(flags, OF_VERSION_1_3));
    value = OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_OAM_DATA_PLANE_COUNTER_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_DESTINATION_LOOKUP_FAILURE;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DESTINATION_LOOKUP_FAILURE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DESTINATION_LOOKUP_FAILURE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DESTINATION_LOOKUP_FAILURE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_DESTINATION_LOOKUP_FAILURE_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FAILED_TABLE_FULL;
    TEST_ASSERT(!OF_FLOW_MOD_FAILED_TABLE_FULL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_TABLE_FULL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_TABLE_FULL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FAILED_TABLE_FULL_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_TYPE_GOTO_TABLE;
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_GOTO_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_GOTO_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_GOTO_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_GOTO_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FLAG_CHECK_OVERLAP;
    TEST_ASSERT(OF_FLOW_MOD_FLAG_CHECK_OVERLAP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_CHECK_OVERLAP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_CHECK_OVERLAP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_CHECK_OVERLAP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_FLOW_MOD_FLAG_CHECK_OVERLAP_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_FLOW_MOD_FLAG_CHECK_OVERLAP_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_CHECK_OVERLAP_TEST(flags, OF_VERSION_1_0));
    OF_FLOW_MOD_FLAG_CHECK_OVERLAP_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_CHECK_OVERLAP_TEST(flags, OF_VERSION_1_0));
    value = OF_CONTROLLER_ROLE_MASTER;
    TEST_ASSERT(!OF_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONTROLLER_ROLE_MASTER_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_PORT_BLOCKED;
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_PORT_BLOCKED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_PORT_BLOCKED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_BLOCKED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_BLOCKED_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_PORT_BLOCKED_SET(flags, OF_VERSION_1_2);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_PORT_BLOCKED_BY_VERSION(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_PORT_BLOCKED_TEST(flags, OF_VERSION_1_2));
    OF_CAPABILITIES_FLAG_PORT_BLOCKED_CLEAR(flags, OF_VERSION_1_2);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_PORT_BLOCKED_TEST(flags, OF_VERSION_1_2));
    value = OF_ETH_ALEN;
    TEST_ASSERT(OF_ETH_ALEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ETH_ALEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ETH_ALEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ETH_ALEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_ACTION;
    TEST_ASSERT(OF_PACKET_IN_REASON_ACTION_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PACKET_IN_REASON_ACTION_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PACKET_IN_REASON_ACTION_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_ACTION_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_COLOR_BASED_CTR_MULTIPART;
    TEST_ASSERT(!OFDPA_EXP_COLOR_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_COLOR_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_COLOR_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_COLOR_BASED_CTR_MULTIPART_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_CONFIG_TABLE_MISS_CONTROLLER;
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_CONTROLLER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_CONTROLLER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_CONTROLLER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_CONTROLLER_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_CONFIG_TABLE_MISS_DROP;
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_DROP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_DROP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_DROP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_DROP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_TABLE_CONFIG_TABLE_MISS_DROP_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_TABLE_CONFIG_TABLE_MISS_DROP_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_DROP_TEST(flags, OF_VERSION_1_1));
    OF_TABLE_CONFIG_TABLE_MISS_DROP_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_DROP_TEST(flags, OF_VERSION_1_1));
    value = OF_METER_MOD_COMMAND_DELETE;
    TEST_ASSERT(!OF_METER_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_COMMAND_DELETE_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_L2_INTERFACE_REMARK_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_L2_INTERFACE_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_L2_INTERFACE_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_L2_INTERFACE_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_L2_INTERFACE_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_STATION_MOVE;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_STATION_MOVE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_STATION_MOVE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_STATION_MOVE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_STATION_MOVE_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_DEC_NW_TTL;
    TEST_ASSERT(!OF_ACTION_TYPE_DEC_NW_TTL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_DEC_NW_TTL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_DEC_NW_TTL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_DEC_NW_TTL_SUPPORTED(OF_VERSION_1_3));
    value = OF_IPV6_EXT_HDR_FLAG_HOP;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_HOP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_HOP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_HOP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_HOP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_HOP_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_HOP_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_HOP_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_HOP_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_HOP_TEST(flags, OF_VERSION_1_3));
    value = OF_INSTRUCTION_TYPE_EXPERIMENTER;
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_PSM_SP;
    TEST_ASSERT(!OFDPA_PSM_SP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_PSM_SP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_PSM_SP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_PSM_SP_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_MATCH_INCONSISTENT;
    TEST_ASSERT(!OF_ACTION_FAILED_MATCH_INCONSISTENT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_MATCH_INCONSISTENT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_MATCH_INCONSISTENT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_MATCH_INCONSISTENT_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_DESC;
    TEST_ASSERT(OF_STATS_TYPE_DESC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_DESC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_DESC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_DESC_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_ACT_COPY_TC_IN;
    TEST_ASSERT(!OFDPA_ACT_COPY_TC_IN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_ACT_COPY_TC_IN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_ACT_COPY_TC_IN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_ACT_COPY_TC_IN_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_NW_TTL;
    TEST_ASSERT(!OF_ACTION_TYPE_SET_NW_TTL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_NW_TTL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_TYPE_SET_NW_TTL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_TYPE_SET_NW_TTL_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BUFFER_UNKNOWN;
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_UNKNOWN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_UNKNOWN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_UNKNOWN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BUFFER_UNKNOWN_SUPPORTED(OF_VERSION_1_3));
    value = OF_REQUEST_FAILED_BAD_PACKET;
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_PACKET_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_REQUEST_FAILED_BAD_PACKET_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_PACKET_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_REQUEST_FAILED_BAD_PACKET_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_MSG_MOD_DELETE;
    TEST_ASSERT(!OFDPA_MSG_MOD_DELETE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_MSG_MOD_DELETE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_MSG_MOD_DELETE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_MSG_MOD_DELETE_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FLAG_SEND_FLOW_REM;
    TEST_ASSERT(OF_FLOW_MOD_FLAG_SEND_FLOW_REM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_SEND_FLOW_REM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_SEND_FLOW_REM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_SEND_FLOW_REM_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_FLOW_MOD_FLAG_SEND_FLOW_REM_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_FLOW_MOD_FLAG_SEND_FLOW_REM_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_SEND_FLOW_REM_TEST(flags, OF_VERSION_1_0));
    OF_FLOW_MOD_FLAG_SEND_FLOW_REM_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_SEND_FLOW_REM_TEST(flags, OF_VERSION_1_0));
    value = OF_IPV6_EXT_HDR_FLAG_UNREP;
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNREP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNREP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNREP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_UNREP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_IPV6_EXT_HDR_FLAG_UNREP_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_IPV6_EXT_HDR_FLAG_UNREP_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_IPV6_EXT_HDR_FLAG_UNREP_TEST(flags, OF_VERSION_1_3));
    OF_IPV6_EXT_HDR_FLAG_UNREP_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_IPV6_EXT_HDR_FLAG_UNREP_TEST(flags, OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_TX_PACKETS_UNICAST;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_TX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_TX_PACKETS_UNICAST_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_QDC_WRED;
    TEST_ASSERT(!OFDPA_QDC_WRED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_QDC_WRED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_QDC_WRED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_QDC_WRED_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_BAD_COMMAND;
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_BAD_COMMAND_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_FAILED_DUP_FIELD;
    TEST_ASSERT(!OF_MATCH_FAILED_DUP_FIELD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_MATCH_FAILED_DUP_FIELD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MATCH_FAILED_DUP_FIELD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MATCH_FAILED_DUP_FIELD_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_STP_LEARN;
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_LEARN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_LEARN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_LEARN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_LEARN_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_STATE_FLAG_STP_LEARN_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_STATE_FLAG_STP_LEARN_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_STP_LEARN_TEST(flags, OF_VERSION_1_0));
    OF_PORT_STATE_FLAG_STP_LEARN_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_STATE_FLAG_STP_LEARN_TEST(flags, OF_VERSION_1_0));
    value = OF_MAX_TABLE_NAME_LEN;
    TEST_ASSERT(OF_MAX_TABLE_NAME_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MAX_TABLE_NAME_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_MAX_TABLE_NAME_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_MAX_TABLE_NAME_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_COLOR_SET_BAND_MODE_SRTCM;
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_SRTCM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_SRTCM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_SRTCM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_COLOR_SET_BAND_MODE_SRTCM_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_FAILED_BAD_OUT_GROUP;
    TEST_ASSERT(!OF_ACTION_FAILED_BAD_OUT_GROUP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_OUT_GROUP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_OUT_GROUP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_BAD_OUT_GROUP_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_TYPE_APPLY_ACTIONS;
    TEST_ASSERT(!OF_INSTRUCTION_TYPE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_INSTRUCTION_TYPE_APPLY_ACTIONS_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_FLOW_STATS;
    TEST_ASSERT(OF_CAPABILITIES_FLAG_FLOW_STATS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_FLOW_STATS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_FLOW_STATS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_FLOW_STATS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_FLOW_STATS_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_FLOW_STATS_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_FLOW_STATS_TEST(flags, OF_VERSION_1_0));
    OF_CAPABILITIES_FLAG_FLOW_STATS_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_FLOW_STATS_TEST(flags, OF_VERSION_1_0));
    value = OF_OBJ_TYPE_HELLO;
    TEST_ASSERT(OF_OBJ_TYPE_HELLO_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_OBJ_TYPE_HELLO_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_OBJ_TYPE_HELLO_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_HELLO_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_MPLS_TUNNEL_LABEL_REMARK_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_SP_WFQ;
    TEST_ASSERT(!OFDPA_SNM_SP_WFQ_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_SP_WFQ_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_SP_WFQ_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_SP_WFQ_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_FAILED_OUT_OF_BANDS;
    TEST_ASSERT(!OF_METER_MOD_FAILED_OUT_OF_BANDS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_FAILED_OUT_OF_BANDS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_FAILED_OUT_OF_BANDS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_FAILED_OUT_OF_BANDS_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_CONFIG_FLAG_PORT_DOWN;
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_PORT_DOWN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_PORT_DOWN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_PORT_DOWN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_PORT_DOWN_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_CONFIG_FLAG_PORT_DOWN_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_CONFIG_FLAG_PORT_DOWN_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CONFIG_FLAG_PORT_DOWN_TEST(flags, OF_VERSION_1_0));
    OF_PORT_CONFIG_FLAG_PORT_DOWN_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_CONFIG_FLAG_PORT_DOWN_TEST(flags, OF_VERSION_1_0));
    value = OF_PORT_DEST_NORMAL;
    TEST_ASSERT(OF_PORT_DEST_NORMAL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_DEST_NORMAL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_DEST_NORMAL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_DEST_NORMAL_SUPPORTED(OF_VERSION_1_3));
    value = OF_CAPABILITIES_FLAG_GROUP_STATS;
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_GROUP_STATS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_GROUP_STATS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_GROUP_STATS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_GROUP_STATS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CAPABILITIES_FLAG_GROUP_STATS_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_CAPABILITIES_FLAG_GROUP_STATS_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_CAPABILITIES_FLAG_GROUP_STATS_TEST(flags, OF_VERSION_1_1));
    OF_CAPABILITIES_FLAG_GROUP_STATS_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CAPABILITIES_FLAG_GROUP_STATS_TEST(flags, OF_VERSION_1_1));
    value = OF_FLOW_WC_V1_NW_SRC_BITS;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_SRC_BITS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_BITS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_BITS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_SRC_BITS_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_CONTROLLER_CONNECTION_STATE_CONNECTED;
    TEST_ASSERT(!OFP_BSN_CONTROLLER_CONNECTION_STATE_CONNECTED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_CONNECTION_STATE_CONNECTED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_CONNECTION_STATE_CONNECTED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_CONTROLLER_CONNECTION_STATE_CONNECTED_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_COLOR_SET_BAND_MODE_RFC4115;
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_RFC4115_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_RFC4115_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_COLOR_SET_BAND_MODE_RFC4115_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_COLOR_SET_BAND_MODE_RFC4115_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_OP_FAILED_BAD_PORT;
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_OP_FAILED_BAD_PORT_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONFIG_INVALID_TTL_TO_CONTROLLER;
    TEST_ASSERT(!OF_CONFIG_INVALID_TTL_TO_CONTROLLER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_INVALID_TTL_TO_CONTROLLER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONFIG_INVALID_TTL_TO_CONTROLLER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_CONFIG_INVALID_TTL_TO_CONTROLLER_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CONFIG_INVALID_TTL_TO_CONTROLLER_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_CONFIG_INVALID_TTL_TO_CONTROLLER_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONFIG_INVALID_TTL_TO_CONTROLLER_TEST(flags, OF_VERSION_1_1));
    OF_CONFIG_INVALID_TTL_TO_CONTROLLER_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CONFIG_INVALID_TTL_TO_CONTROLLER_TEST(flags, OF_VERSION_1_1));
    value = OF_FLOW_WC_V1_TP_SRC;
    TEST_ASSERT(OF_FLOW_WC_V1_TP_SRC_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_TP_SRC_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_TP_SRC_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_TP_SRC_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_METER_MOD;
    TEST_ASSERT(!OF_OBJ_TYPE_METER_MOD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OBJ_TYPE_METER_MOD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_OBJ_TYPE_METER_MOD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_METER_MOD_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_CONFIG_TABLE_MISS_CONTINUE;
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_TEST(flags, OF_VERSION_1_1));
    OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_TABLE_CONFIG_TABLE_MISS_CONTINUE_TEST(flags, OF_VERSION_1_1));
    value = OF_FLOW_MOD_COMMAND_DELETE_STRICT;
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_STRICT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_STRICT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_STRICT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_DELETE_STRICT_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_SNM_WRR;
    TEST_ASSERT(!OFDPA_SNM_WRR_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_SNM_WRR_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_SNM_WRR_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_SNM_WRR_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_MPLS_LABEL;
    TEST_ASSERT(!OF_FLOW_WC_V1_MPLS_LABEL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_MPLS_LABEL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_MPLS_LABEL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_MPLS_LABEL_SUPPORTED(OF_VERSION_1_3));
    value = OF_OBJ_TYPE_GET_ASYNC_REQUEST;
    TEST_ASSERT(!OF_OBJ_TYPE_GET_ASYNC_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_OBJ_TYPE_GET_ASYNC_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_OBJ_TYPE_GET_ASYNC_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_OBJ_TYPE_GET_ASYNC_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_BAD_MATCH;
    TEST_ASSERT(!OF_ERROR_TYPE_BAD_MATCH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_MATCH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_MATCH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_MATCH_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_MOD_COMMAND_ADD;
    TEST_ASSERT(!OF_METER_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_MOD_COMMAND_ADD_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MAX;
    TEST_ASSERT(!OF_GROUP_MAX_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MAX_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MAX_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MAX_SUPPORTED(OF_VERSION_1_3));
    value = OF_METER_FLAG_BURST;
    TEST_ASSERT(!OF_METER_FLAG_BURST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_METER_FLAG_BURST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_METER_FLAG_BURST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_METER_FLAG_BURST_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_METER_FLAG_BURST_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_METER_FLAG_BURST_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_METER_FLAG_BURST_TEST(flags, OF_VERSION_1_3));
    OF_METER_FLAG_BURST_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_METER_FLAG_BURST_TEST(flags, OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_NEXT_TABLES;
    TEST_ASSERT(!OF_TABLE_FEATURE_NEXT_TABLES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_NEXT_TABLES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_NEXT_TABLES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_NEXT_TABLES_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_UDF_ANCHOR_L4_HEADER_START;
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_L4_HEADER_START_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_L4_HEADER_START_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_L4_HEADER_START_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_UDF_ANCHOR_L4_HEADER_START_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONFIG_FRAG_DROP;
    TEST_ASSERT(OF_CONFIG_FRAG_DROP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_FRAG_DROP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONFIG_FRAG_DROP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONFIG_FRAG_DROP_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_CONFIG_FRAG_DROP_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_CONFIG_FRAG_DROP_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_CONFIG_FRAG_DROP_TEST(flags, OF_VERSION_1_0));
    OF_CONFIG_FRAG_DROP_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_CONFIG_FRAG_DROP_TEST(flags, OF_VERSION_1_0));
    value = OF_PORT_CHANGE_REASON_MODIFY;
    TEST_ASSERT(OF_PORT_CHANGE_REASON_MODIFY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_MODIFY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_MODIFY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_CHANGE_REASON_MODIFY_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_METER_MOD_FAILED;
    TEST_ASSERT(!OF_ERROR_TYPE_METER_MOD_FAILED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ERROR_TYPE_METER_MOD_FAILED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ERROR_TYPE_METER_MOD_FAILED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_METER_MOD_FAILED_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_REQUEST_FAILED_BAD_LEN;
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_REQUEST_FAILED_BAD_LEN_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_DL_DST;
    TEST_ASSERT(OF_FLOW_WC_V1_DL_DST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_DST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_DST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_DL_DST_SUPPORTED(OF_VERSION_1_3));
    value = OF_MATCH_TYPE_STANDARD_LENGTH;
    TEST_ASSERT(!OF_MATCH_TYPE_STANDARD_LENGTH_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_MATCH_TYPE_STANDARD_LENGTH_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_MATCH_TYPE_STANDARD_LENGTH_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_MATCH_TYPE_STANDARD_LENGTH_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_COMMAND_MODIFY;
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_COMMAND_MODIFY_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_EXP_QUEUE_MOD_MSG;
    TEST_ASSERT(!OFDPA_EXP_QUEUE_MOD_MSG_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_EXP_QUEUE_MOD_MSG_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_EXP_QUEUE_MOD_MSG_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_EXP_QUEUE_MOD_MSG_SUPPORTED(OF_VERSION_1_3));
    value = OF_INSTRUCTION_FAILED_UNSUP_EXP_INST;
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_UNSUP_EXP_INST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_INSTRUCTION_FAILED_UNSUP_EXP_INST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_UNSUP_EXP_INST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_INSTRUCTION_FAILED_UNSUP_EXP_INST_SUPPORTED(OF_VERSION_1_3));
    value = OFDPA_QDC_NONE;
    TEST_ASSERT(!OFDPA_QDC_NONE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFDPA_QDC_NONE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFDPA_QDC_NONE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFDPA_QDC_NONE_SUPPORTED(OF_VERSION_1_3));
    value = OF_QUEUE_ALL;
    TEST_ASSERT(OF_QUEUE_ALL_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_QUEUE_ALL_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_QUEUE_ALL_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_QUEUE_ALL_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_ARP;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_ARP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_ARP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_ARP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_ARP_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_WEIGHT_UNSUPPORTED;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_WEIGHT_UNSUPPORTED_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_WEIGHT_UNSUPPORTED_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_WEIGHT_UNSUPPORTED_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_WEIGHT_UNSUPPORTED_SUPPORTED(OF_VERSION_1_3));
    value = OF_DL_TYPE_NOT_ETH_TYPE;
    TEST_ASSERT(OF_DL_TYPE_NOT_ETH_TYPE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_DL_TYPE_NOT_ETH_TYPE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_DL_TYPE_NOT_ETH_TYPE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_DL_TYPE_NOT_ETH_TYPE_SUPPORTED(OF_VERSION_1_3));
    value = OF_CONTROLLER_PKT_MAX;
    TEST_ASSERT(!OF_CONTROLLER_PKT_MAX_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_CONTROLLER_PKT_MAX_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_CONTROLLER_PKT_MAX_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_CONTROLLER_PKT_MAX_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_BAD_BUCKET;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_BUCKET_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_BAD_BUCKET_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_BUCKET_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_BAD_BUCKET_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_PORT_COUNTER_RX_BYTES;
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_PORT_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_PORT_COUNTER_RX_BYTES_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_ENQUEUE;
    TEST_ASSERT(OF_ACTION_TYPE_ENQUEUE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_ACTION_TYPE_ENQUEUE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_ENQUEUE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_ENQUEUE_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_REQUEST_FAILED_BAD_ARGUMENT;
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_REQUEST_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_REQUEST_FAILED_BAD_ARGUMENT_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_WC_V1_NW_PROTO;
    TEST_ASSERT(OF_FLOW_WC_V1_NW_PROTO_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_WC_V1_NW_PROTO_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_PROTO_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_FLOW_WC_V1_NW_PROTO_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_FEATURE_FLAG_1GB_HD;
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_HD_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_HD_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_HD_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_HD_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_FEATURE_FLAG_1GB_HD_SET(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == OF_PORT_FEATURE_FLAG_1GB_HD_BY_VERSION(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_FEATURE_FLAG_1GB_HD_TEST(flags, OF_VERSION_1_0));
    OF_PORT_FEATURE_FLAG_1GB_HD_CLEAR(flags, OF_VERSION_1_0);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_FEATURE_FLAG_1GB_HD_TEST(flags, OF_VERSION_1_0));
    value = OF_STATS_TYPE_METER;
    TEST_ASSERT(!OF_STATS_TYPE_METER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_STATS_TYPE_METER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_STATS_TYPE_METER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_METER_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_STATE_FLAG_LIVE;
    TEST_ASSERT(!OF_PORT_STATE_FLAG_LIVE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LIVE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LIVE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LIVE_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_PORT_STATE_FLAG_LIVE_SET(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == OF_PORT_STATE_FLAG_LIVE_BY_VERSION(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_STATE_FLAG_LIVE_TEST(flags, OF_VERSION_1_1));
    OF_PORT_STATE_FLAG_LIVE_CLEAR(flags, OF_VERSION_1_1);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_PORT_STATE_FLAG_LIVE_TEST(flags, OF_VERSION_1_1));
    value = OF_FLOW_REMOVED_REASON_DELETE;
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_DELETE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_DELETE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_DELETE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_REMOVED_REASON_DELETE_SUPPORTED(OF_VERSION_1_3));
    value = OF_STATS_TYPE_TABLE;
    TEST_ASSERT(OF_STATS_TYPE_TABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_STATS_TYPE_TABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_STATS_TYPE_TABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_STATS_TYPE_TABLE_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_CONTROLLER_ROLE_REASON_MASTER_REQUEST;
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_MASTER_REQUEST_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_MASTER_REQUEST_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_CONTROLLER_ROLE_REASON_MASTER_REQUEST_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_CONTROLLER_ROLE_REASON_MASTER_REQUEST_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_DEST_PORT_UNREACHABLE;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_PORT_UNREACHABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_PORT_UNREACHABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_PORT_UNREACHABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_DEST_PORT_UNREACHABLE_SUPPORTED(OF_VERSION_1_3));
    value = OF_ACTION_TYPE_SET_VLAN_PCP;
    TEST_ASSERT(OF_ACTION_TYPE_SET_VLAN_PCP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_TYPE_SET_VLAN_PCP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_VLAN_PCP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(!OF_ACTION_TYPE_SET_VLAN_PCP_SUPPORTED(OF_VERSION_1_3));
    value = OFP_BSN_UDF_ANCHOR_L3_HEADER_START;
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_L3_HEADER_START_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_L3_HEADER_START_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFP_BSN_UDF_ANCHOR_L3_HEADER_START_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFP_BSN_UDF_ANCHOR_L3_HEADER_START_SUPPORTED(OF_VERSION_1_3));
    value = OF_TABLE_FEATURE_WRITE_ACTIONS_MISS;
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_TABLE_FEATURE_WRITE_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_TABLE_FEATURE_WRITE_ACTIONS_MISS_SUPPORTED(OF_VERSION_1_3));
    value = OF_FLOW_MOD_FLAG_NO_PKT_COUNTS;
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_SUPPORTED(OF_VERSION_1_3));

    flags = 0;
    OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_SET(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_BY_VERSION(OF_VERSION_1_3));
    TEST_ASSERT(OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_TEST(flags, OF_VERSION_1_3));
    OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_CLEAR(flags, OF_VERSION_1_3);
    TEST_ASSERT(flags == 0);
    TEST_ASSERT(!OF_FLOW_MOD_FLAG_NO_PKT_COUNTS_TEST(flags, OF_VERSION_1_3));
    value = OF_ACTION_FAILED_EPERM;
    TEST_ASSERT(OF_ACTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ACTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ACTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ACTION_FAILED_EPERM_SUPPORTED(OF_VERSION_1_3));
    value = OF_GROUP_MOD_FAILED_LOOP;
    TEST_ASSERT(!OF_GROUP_MOD_FAILED_LOOP_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_LOOP_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_LOOP_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_GROUP_MOD_FAILED_LOOP_SUPPORTED(OF_VERSION_1_3));
    value = BSN_PDU_SLOT_NUM_ANY;
    TEST_ASSERT(BSN_PDU_SLOT_NUM_ANY_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(BSN_PDU_SLOT_NUM_ANY_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(BSN_PDU_SLOT_NUM_ANY_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(BSN_PDU_SLOT_NUM_ANY_SUPPORTED(OF_VERSION_1_3));
    value = OF_PORT_MOD_FAILED_BAD_ADVERTISE;
    TEST_ASSERT(!OF_PORT_MOD_FAILED_BAD_ADVERTISE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_ADVERTISE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_ADVERTISE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PORT_MOD_FAILED_BAD_ADVERTISE_SUPPORTED(OF_VERSION_1_3));
    value = OF_ERROR_TYPE_BAD_INSTRUCTION;
    TEST_ASSERT(!OF_ERROR_TYPE_BAD_INSTRUCTION_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_INSTRUCTION_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_INSTRUCTION_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_ERROR_TYPE_BAD_INSTRUCTION_SUPPORTED(OF_VERSION_1_3));
    value = OF_PACKET_IN_REASON_BSN_DEST_NETWORK_UNREACHABLE;
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_NETWORK_UNREACHABLE_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_NETWORK_UNREACHABLE_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OF_PACKET_IN_REASON_BSN_DEST_NETWORK_UNREACHABLE_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OF_PACKET_IN_REASON_BSN_DEST_NETWORK_UNREACHABLE_SUPPORTED(OF_VERSION_1_3));
    value = OFPQDPT_EXPERIMENTER;
    TEST_ASSERT(!OFPQDPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_0));
    TEST_ASSERT(!OFPQDPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_1));
    TEST_ASSERT(!OFPQDPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_2));
    TEST_ASSERT(OFPQDPT_EXPERIMENTER_SUPPORTED(OF_VERSION_1_3));

    return TEST_PASS;
}


/**
 * Test object dump functions
 */

int
test_dump_objs(void)
{
    of_object_t *obj;

    FILE *out = fopen("/dev/null", "w");

    /* Call each obj dump function */

    obj = (of_object_t *)of_aggregate_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_aggregate_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_action_error_msg_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_request_error_msg_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_ip_mask_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_ip_mask_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_l2_table_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_l2_table_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_header_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_hybrid_get_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_hybrid_get_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_timeout_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_ip_mask_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_l2_table_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_l2_table_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_mirroring_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_shell_command_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_shell_output_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_shell_status_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_error_msg_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_add_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_strict_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_failed_error_msg_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_strict_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_removed_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_failed_error_msg_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_nicira_controller_role_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_nicira_controller_role_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_nicira_header_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_in_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_out_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_failed_error_msg_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_status_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_op_failed_error_msg_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_set_config_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_mod_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_reply_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_request_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_mirror_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_set_tunnel_dst_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_enqueue_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_experimenter_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_header_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_dec_ttl_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_output_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_dl_dst_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_dl_src_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_dst_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_src_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_tos_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_tp_dst_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_tp_src_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_vlan_pcp_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_vlan_vid_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_strip_vlan_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_interface_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_header_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_q_in_q_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_header_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_match_v1_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_queue_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_desc_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_header_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_min_rate_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_action_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_interface_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_flow_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_packet_queue_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_desc_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_prop_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_table_stats_entry_new(OF_VERSION_1_0);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_aggregate_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_aggregate_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_action_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_instruction_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_match_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_request_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_header_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_timeout_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_mirroring_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_add_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_strict_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_failed_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_strict_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_removed_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_add_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_delete_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_mod_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_mod_failed_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_modify_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_failed_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_nicira_header_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_in_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_out_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_failed_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_status_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_op_failed_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_set_config_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_switch_config_failed_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_mod_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_mod_failed_error_msg_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_reply_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_request_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_mirror_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_set_tunnel_dst_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_copy_ttl_in_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_copy_ttl_out_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_dec_mpls_ttl_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_dec_nw_ttl_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_experimenter_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_group_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_header_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_dec_ttl_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_output_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_pop_mpls_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_pop_vlan_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_push_mpls_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_push_vlan_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_dl_dst_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_dl_src_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_mpls_label_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_mpls_tc_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_mpls_ttl_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_dst_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_ecn_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_src_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_tos_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_ttl_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_queue_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_tp_dst_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_tp_src_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_vlan_pcp_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_vlan_vid_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_interface_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_header_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_q_in_q_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bucket_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bucket_counter_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_header_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_apply_actions_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_clear_actions_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_experimenter_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_goto_table_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_header_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_write_actions_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_write_metadata_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_match_v2_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_queue_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_desc_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_header_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_min_rate_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_action_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_interface_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bucket_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bucket_counter_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_flow_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_group_desc_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_group_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_instruction_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_packet_queue_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_desc_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_prop_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_table_stats_entry_new(OF_VERSION_1_1);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_aggregate_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_aggregate_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_action_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_instruction_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_match_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_request_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_timeout_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_mirroring_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_add_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_strict_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_strict_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_removed_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_add_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_delete_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_features_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_features_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_mod_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_mod_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_modify_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_nicira_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_in_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_out_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_status_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_op_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_role_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_role_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_role_request_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_set_config_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_switch_config_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_mod_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_mod_failed_error_msg_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_reply_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_request_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_mirror_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_set_tunnel_dst_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_copy_ttl_in_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_copy_ttl_out_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_dec_mpls_ttl_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_dec_nw_ttl_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_experimenter_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_group_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_dec_ttl_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_output_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_pop_mpls_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_pop_vlan_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_push_mpls_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_push_vlan_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_field_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_mpls_ttl_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_ttl_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_queue_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_interface_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_q_in_q_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bucket_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bucket_counter_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_apply_actions_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_clear_actions_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_experimenter_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_goto_table_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_write_actions_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_write_metadata_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_match_v3_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_op_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_op_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_sha_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_sha_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_spa_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_spa_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tha_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tha_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tpa_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tpa_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_egr_port_group_id_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_egr_port_group_id_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_global_vrf_allowed_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_global_vrf_allowed_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_in_ports_128_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_in_ports_128_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_dst_class_id_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_dst_class_id_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_interface_class_id_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_interface_class_id_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_src_class_id_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_src_class_id_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_lag_id_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_lag_id_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf0_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf0_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf1_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf1_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf2_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf2_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf3_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf3_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf4_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf4_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf5_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf5_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf6_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf6_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf7_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf7_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_vrf_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_vrf_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_dst_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_dst_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_src_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_src_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_type_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_type_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_code_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_code_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_type_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_type_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_code_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_code_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_type_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_type_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_phy_port_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_phy_port_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_port_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_port_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_dscp_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_dscp_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_ecn_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_ecn_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_proto_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_proto_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_dst_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_dst_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_src_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_src_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_dst_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_dst_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_flabel_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_flabel_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_sll_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_sll_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_target_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_target_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_tll_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_tll_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_src_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_src_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_metadata_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_metadata_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_bos_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_bos_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_label_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_label_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_tc_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_tc_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_dst_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_dst_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_src_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_src_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_dst_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_dst_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_src_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_src_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tunnel_id_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tunnel_id_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_dst_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_dst_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_src_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_src_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_pcp_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_pcp_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_vid_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_vid_masked_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_queue_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_desc_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_experimenter_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_header_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_max_rate_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_min_rate_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_action_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_interface_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bucket_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bucket_counter_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_flow_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_group_desc_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_group_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_instruction_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_oxm_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_packet_queue_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_desc_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_prop_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_table_stats_entry_new(OF_VERSION_1_2);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_aggregate_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_aggregate_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_async_get_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_async_get_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_async_set_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_action_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_instruction_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_match_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bad_request_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_barrier_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_arp_idle_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_clear_data_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_get_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_bw_enable_set_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_controller_connections_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_controller_connections_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_debug_counter_desc_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_debug_counter_desc_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_debug_counter_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_debug_counter_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_checksum_bucket_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_checksum_bucket_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_idle_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_idle_enable_get_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_idle_enable_get_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_idle_enable_set_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_idle_enable_set_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_bucket_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_bucket_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_clear_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_clear_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_desc_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_desc_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_add_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_delete_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_desc_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_desc_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_set_buckets_size_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_interfaces_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_mirroring_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_switch_pipeline_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_get_switch_pipeline_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_lacp_convergence_notif_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_lacp_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_lacp_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_rx_timeout_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_pdu_tx_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_port_counter_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_port_counter_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_role_status_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_aux_cxns_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_aux_cxns_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_lacp_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_lacp_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_mirroring_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_pktin_suppression_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_switch_pipeline_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_set_switch_pipeline_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_switch_pipeline_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_switch_pipeline_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_table_checksum_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_table_checksum_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_table_set_buckets_size_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_time_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_time_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_create_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_virtual_port_remove_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vlan_counter_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vlan_counter_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_desc_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_echo_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_multipart_reply_ofdpa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_multipart_request_ofdpa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_ofdpa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_experimenter_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_features_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_add_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_delete_strict_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_mod_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_modify_strict_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_removed_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_get_config_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_add_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_delete_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_features_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_features_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_mod_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_mod_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_modify_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_add_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_band_ofdpa_color_set_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_config_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_config_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_delete_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_features_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_features_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_mod_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_mod_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_modify_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_nicira_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_in_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_out_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_desc_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_desc_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_mod_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_status_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_get_config_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_op_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_role_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_role_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_role_request_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_set_config_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_switch_config_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_features_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_features_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_features_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_mod_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_mod_failed_error_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_class_based_ctr_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_class_based_ctr_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_class_based_ctr_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_color_based_ctr_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_color_based_ctr_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_color_based_ctr_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_l2_interface_remark_action_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_l2_interface_remark_action_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_l2_interface_remark_action_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_set_qos_action_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_set_qos_action_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_set_qos_action_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_tunnel_label_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_tunnel_label_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_tunnel_label_remark_action_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_vpn_label_remark_action_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_vpn_label_remark_action_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_mpls_vpn_label_remark_action_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_oam_data_plane_ctr_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_oam_data_plane_ctr_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_oam_data_plane_ctr_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_oam_drop_status_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_oam_drop_status_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_oam_drop_status_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_port_experimenter_prop_scheduling_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_queue_desc_prop_congestion_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_queue_desc_prop_congestion_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_queue_desc_prop_parent_multipart_reply_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_queue_desc_prop_parent_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_queue_mod_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_scheduler_node_mod_msg_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_scheduler_node_multipart_request_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofdpa_scheduler_node_multipart_response_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_mirror_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_bsn_set_tunnel_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_copy_ttl_in_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_copy_ttl_out_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_dec_mpls_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_dec_nw_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_group_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_bsn_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_bsn_mirror_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_bsn_set_tunnel_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_copy_ttl_in_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_copy_ttl_out_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_dec_mpls_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_dec_nw_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_group_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_nicira_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_nicira_dec_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_check_drop_status_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_class_based_rx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_class_based_tx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_color_based_rx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_color_based_tx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_copy_tc_in_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_copy_tc_out_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_dec_ttl_mtu_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_oam_lm_rx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_oam_lm_tx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_pop_cw_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_pop_l2hdr_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_push_cw_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_push_l2hdr_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_counter_fields_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_dscp_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_mpls_pcpdei_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_mpls_pcpdei_from_tunnel_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_mpls_qos_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_mpls_tc_from_tunnel_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_mpls_tc_from_vpn_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_ofdpa_set_pcpdei_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_output_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_pop_mpls_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_pop_pbb_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_pop_vlan_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_push_mpls_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_push_pbb_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_push_vlan_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_set_field_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_set_mpls_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_set_nw_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_id_set_queue_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_nicira_dec_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_check_drop_status_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_class_based_rx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_class_based_tx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_color_based_rx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_color_based_tx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_copy_tc_in_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_copy_tc_out_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_dec_ttl_mtu_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_oam_lm_rx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_oam_lm_tx_count_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_pop_cw_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_pop_l2hdr_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_push_cw_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_push_l2hdr_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_counter_fields_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_dscp_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_mpls_pcpdei_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_mpls_pcpdei_from_tunnel_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_mpls_qos_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_mpls_tc_from_tunnel_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_mpls_tc_from_vpn_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_ofdpa_set_pcpdei_from_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_output_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_pop_mpls_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_pop_pbb_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_pop_vlan_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_push_mpls_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_push_pbb_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_push_vlan_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_field_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_mpls_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_nw_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_action_set_queue_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_controller_connection_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_debug_counter_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_debug_counter_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_flow_checksum_bucket_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_bucket_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_entry_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_gentable_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_interface_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_lacp_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_port_counter_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_switch_pipeline_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_table_checksum_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_broadcast_query_timeout_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_circuit_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_idle_notification_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_idle_time_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_idle_timeout_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_ipv4_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_mac_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_miss_packets_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_port_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_reply_packets_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_request_packets_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_rx_packets_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_tx_packets_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_udf_anchor_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_udf_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_udf_length_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_udf_offset_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_unicast_query_timeout_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_tlv_vlan_vid_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vlan_counter_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bsn_vport_q_in_q_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bucket_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_bucket_counter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_flow_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_group_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_elem_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_hello_elem_versionbitmap_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_apply_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_arp_offload_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_deny_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_dhcp_offload_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_disable_split_horizon_check_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_disable_src_mac_check_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_packet_of_death_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_bsn_permit_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_clear_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_goto_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_apply_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_arp_offload_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_deny_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_dhcp_offload_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_disable_split_horizon_check_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_disable_src_mac_check_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_packet_of_death_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_bsn_permit_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_clear_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_goto_table_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_meter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_write_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_id_write_metadata_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_meter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_write_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_instruction_write_metadata_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_match_v3_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_band_drop_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_band_dscp_remark_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_band_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_band_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_band_stats_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_config_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_features_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_meter_stats_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_op_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_op_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_sha_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_sha_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_spa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_spa_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tha_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tha_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tpa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_arp_tpa_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_egr_port_group_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_egr_port_group_id_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_global_vrf_allowed_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_global_vrf_allowed_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_in_ports_128_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_in_ports_128_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_dst_class_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_dst_class_id_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_interface_class_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_interface_class_id_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_src_class_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_l3_src_class_id_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_lag_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_lag_id_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf0_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf0_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf1_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf1_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf2_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf2_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf3_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf3_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf4_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf4_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf5_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf5_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf6_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf6_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf7_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_udf7_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_vrf_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_bsn_vrf_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_dst_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_src_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_src_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_type_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_eth_type_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_code_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_code_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_type_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv4_type_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_code_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_code_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_type_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_icmpv6_type_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_phy_port_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_phy_port_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_port_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_in_port_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_dscp_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_dscp_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_ecn_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_ecn_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_proto_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ip_proto_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_dst_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_src_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv4_src_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_dst_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_flabel_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_flabel_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_sll_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_sll_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_target_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_target_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_tll_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_nd_tll_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_src_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ipv6_src_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_metadata_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_metadata_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_bos_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_bos_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_label_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_label_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_tc_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_mpls_tc_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_actset_output_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_bfd_discriminator_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_bfd_discriminator_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_color_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_color_actions_index_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_dei_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_eth_subtype_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_l3_in_port_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_lmep_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_mpls_ach_channel_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_mpls_data_first_nibble_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_mpls_l2_port_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_mpls_l2_port_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_mpls_next_label_is_gal_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_mpls_ttl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_oam_y1731_mdl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_oam_y1731_opcode_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_ovid_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_protection_index_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_qos_index_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_rx_timestamp_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_rxfcl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_traffic_class_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_txfcl_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_ofdpa_vrf_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_dst_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_src_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_sctp_src_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_dst_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_src_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tcp_src_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tunnel_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_tunnel_id_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_dst_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_dst_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_src_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_udp_src_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_pcp_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_pcp_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_vid_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_oxm_vlan_vid_masked_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_packet_queue_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_desc_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_port_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_max_rate_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_prop_min_rate_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_queue_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_apply_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_apply_actions_miss_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_apply_setfield_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_apply_setfield_miss_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_experimenter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_experimenter_miss_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_header_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_instructions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_instructions_miss_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_match_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_next_tables_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_next_tables_miss_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_wildcards_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_write_actions_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_write_actions_miss_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_write_setfield_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_feature_prop_write_setfield_miss_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_features_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_table_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_uint32_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_uint64_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_uint8_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)ofp_queue_desc_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_action_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_action_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_controller_connection_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_debug_counter_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_debug_counter_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_flow_checksum_bucket_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_gentable_bucket_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_gentable_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_gentable_entry_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_gentable_entry_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_gentable_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_interface_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_lacp_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_port_counter_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_switch_pipeline_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_table_checksum_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_tlv_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bsn_vlan_counter_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bucket_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_bucket_counter_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_flow_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_group_desc_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_group_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_hello_elem_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_instruction_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_instruction_id_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_meter_band_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_meter_band_stats_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_meter_stats_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_oxm_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_packet_queue_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_desc_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_port_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_prop_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_queue_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_table_feature_prop_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_table_features_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_table_stats_entry_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_uint32_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_uint64_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    obj = (of_object_t *)of_list_uint8_new(OF_VERSION_1_3);
    of_object_dump((loci_writer_f)fprintf, out, obj);
    of_object_delete(obj);

    fclose(out);
    return TEST_PASS;
}
